package org.eso.ohs.phase2.apps.masktracker;

// $Id: MaskBrowser.java,v 1.26 2007/10/11 11:42:56 pnunes Exp $

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.eso.ohs.core.dbb.client.DbbArrowButton;
import org.eso.ohs.core.dbb.client.DbbCheckBox;
import org.eso.ohs.core.dbb.client.DbbIntegerTextField;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbSelectionList;
import org.eso.ohs.core.dbb.client.DbbSpinnerInt;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.client.DbbWidget;
import org.eso.ohs.core.dbb.rmi.DbbLocalEngine;
import org.eso.ohs.core.dbb.server.DbbSession;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.server.DbbSqlEngineImpl;
import org.eso.ohs.core.dbb.sql.DbbQueryStarted;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.eso.ohs.core.utilities.AssocList;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.Mask;
import org.eso.ohs.dfs.MaskStatusType;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.actions.DumpMaskAdpAdmData;

/**
 * This class us used to browser the masks table from within the OT
 * It uses the standars set of Database Browser classes from the 
 * Dbb package to achieve this
 *
 */

public class MaskBrowser extends JFrame implements ObjectContainer {

    //private DbbCheckBox ignoredCheckBox_ = null;
    
    
    private static final String OB_ID = "OB ID";
    
    private static final String OB_TYPE = "Type";
    
    private static final String MASK_ID = "Mask ID";


    
    private static MaskBrowser maskBrowser_;

    private DbbTextField ignoredCheckBox_ = null;


    /**
     * Phase 1 databasee name
     */

    protected static String dbNamePhase1 =
        Config.getCfg().getDbName(ObservingRun.getSuffix());

    /**
     * Phase 2 database name
     */

    protected static String dbNamePhase2 =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());

    /**
     * Owner of the dbase
     */

    protected static String dbOwner = "dbo";

    /** Masks table*/

    protected DbbSqlTable masksTable;

    /** obs table*/

    protected DbbSqlTable obsTable;

    /** Users table*/
    protected DbbSqlTable usersTable;

    /** Sched table */

    protected DbbSqlTable schedTable;

    protected DbbSqlTable obsMasksTable;

    protected DbbSqlTable odTable;

    /** Dbb Selection Table*/

    protected DbbPanel selPanel = new DbbPanel("Selection Criteria", 3);

    /** RMI server url used to connect to the remote server */
    public static String RMI_SERVER_URL = "";

    /**
     * Associated sql engine reference (server). It must be an
     * interface and it has to be set creating a DbbLocalEngine that
     * connects (via RMI) the remote server. 
     */
    protected DbbSqlEngine engine_;

    /** Period Selector widget*/

    DbbSpinnerInt periodWidget_ = null;

    /** Aggregated view (client) */
    protected DbbView view_;

    private JButton deleteButton_;

    private JButton updateMaskTable_;

    /** Goes direc to database not appserver*/

    protected static boolean REMOTE = false;

    private DbbSelectionList statusList_;

    protected JButton cabinetButton = new JButton("Instrument Cabinet");

    protected JButton storageButton = new JButton("Storage Cabinet");

    private DbbArrowButton slotArrowButton_;

    private DbbArrowButton maskIdArrowButton_;

    //----------------------------------------
    // Constructor
    //----------------------------------------

    /**
     * Create an object setting all its internal variable. The
     * constructor, invoking the private method buildSqlEngine,
     * creates the associated sql engine (DbbLocalEngine implements
     * the DbbSqlEngine interface) connecting to the RMI server.
     */
    public MaskBrowser() {

        buildTables();
        buildSqlEngine();
        view_ = new DbbView(engine_);
        buildColumns();
        buildRows();
        buildSort();
        MaskDetailView detail_view = new MaskDetailView("Mask", new MaskView());
        view_.setDetailView(detail_view, "South");
        view_.addListSelectionListener(new ListListener(this));
        view_.addSummaryButton(cabinetButton);
        view_.addSummaryButton(storageButton);
        cabinetButton.addActionListener(new InstCabinetListener());
        storageButton.addActionListener(new StorageCabinetListener());
        getContentPane().add(view_);
        setTitle("Mask Browser");
        
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent ev) {
                setVisible(false);
            }
        });

        createFileMenu();

        
        pack();
        setVisible(true);
    }

    //----------------------------------------
    // Private Methods
    //----------------------------------------

    /**
     * Build the SQL engine setting up a RMI connection to the server
     * and add the dafault sql chunks and join conditions.
     */
    protected void buildSqlEngine() {
        try {

            if (REMOTE) {
                System.out.println(
                    "DEBUG: Connecting the server via RMI: <"
                        + RMI_SERVER_URL
                        + ">");

                /* NOTE: it must be a local engine created passing the RMI
                 * server url because it has been delegated to set up the
                 * connection to the RMI server 
                 */
                engine_ = new DbbLocalEngine(RMI_SERVER_URL);

                System.out.println(
                    "DEBUG: Server connected: <" + RMI_SERVER_URL + ">");
            } else {

                System.out.println("DEBUG: Direct connection to the database");

                // Build a DBB session and connecto to the database
                DbbSession session =
                    new DbbSession(
                        Config.getCfg().getUrl(ObservationBlock.getSuffix()),
                        dbNamePhase2,
                        Config.getCfg().getDbUserName(),
                        Config.getCfg().getDbPassword());

                // Build an SQL engine for our session
                engine_ = new DbbSqlEngineImpl(session, false);
            }
            setSqlDefaults();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Build the neccessary tables for the Borwser
     */

    protected void buildTables() {

        masksTable = new DbbSqlTable(dbNamePhase2, dbOwner, "masks");

        obsMasksTable = new DbbSqlTable(dbNamePhase2, dbOwner, "obsblk_masks");

        obsTable = new DbbSqlTable(dbNamePhase2, dbOwner, "obs_blocks");

        usersTable = new DbbSqlTable(dbNamePhase1, dbOwner, "eso_users");

        schedTable = new DbbSqlTable(dbNamePhase1, dbOwner, "sched_rep");
        odTable = new DbbSqlTable(dbNamePhase2, "dbo", "obs_descriptions");
    }

    /**
     * Build the default joins 
     */

    protected void setSqlDefaults() throws Exception {

        /* Default chunks */
        DbbSqlChunk def = null;
        
        
        DbbSqlTable table = obsTable;
        String column = "ob_id";
        String label = "OB ID";

        def = new DbbSqlChunk(table, column, label);
        
        engine_.addDefaultColumnsChunk(def);

        // New selection widget
        table = obsTable;
        column = "item_name";
        label = "OB Name";

        def = new DbbSqlChunk(table, column, label);               
        
        table = obsTable;
        column = "type";
        label = "Type";

        def = new DbbSqlChunk(table, column, label);

        
        engine_.addDefaultColumnsChunk(def);


        def = new DbbSqlChunk(masksTable, "mask_id","Mask ID");
        engine_.addDefaultColumnsChunk(def);

        def = new DbbSqlChunk(masksTable, "mask_naid", "NAID");
        engine_.addDefaultColumnsChunk(def);

        def = new DbbSqlChunk(masksTable, "mask_slot_num", "Slot Num");
        engine_.addDefaultColumnsChunk(def);

        def = new DbbSqlChunk(masksTable, "bar_code", "Barcode");
        engine_.addDefaultColumnsChunk(def);

        //  MaskStatusType.class
        def =
            new DbbSqlChunk(
                masksTable,
                "mask_status",
                "Mask Status",
                MaskStatusType.class);
        engine_.addDefaultColumnsChunk(def);

        def = new DbbSqlChunk(masksTable, "mask_name", "Mask Name");
        engine_.addDefaultColumnsChunk(def);

        def = new DbbSqlChunk(masksTable, "exec_count", "Execution Count");
        engine_.addDefaultColumnsChunk(def);

        /* Add the default join conditions */
        ((DbbSqlEngineImpl) engine_).addJoinCondition(
            masksTable,
            "mask_id",
            obsMasksTable,
            "mask_id",
            masksTable,
            obsMasksTable);

        ((DbbSqlEngineImpl) engine_).addJoinCondition(
            obsTable,
            "user_id",
            usersTable,
            "id",
            obsTable,
            usersTable);

        ((DbbSqlEngineImpl) engine_).addJoinCondition(
            obsTable,
            "run_id",
            schedTable,
            "run_id",
            obsTable,
            schedTable);

        ((DbbSqlEngineImpl) engine_).addJoinCondition(
            obsTable,
            "ob_id",
            obsMasksTable,
            "ob_id",
            obsTable,
            obsMasksTable);

    }

    /**
     * Build the column selection panel. 
     */
    protected void buildColumns() {

        DbbSqlChunk c;
        DbbCheckBox w;
        DbbSqlTable table;

        String label = "";
        String column = "";

        // Panel has five rows of widgets
        DbbPanel panel = new DbbPanel("Selected Columns", 5);

        // Define a default operator
        DbbSqlOperator op = null;

        try {
            op = new DbbSqlOperator("=");
        } catch (Exception e) {
            e.printStackTrace();
        }


        // New selection widget
        table = usersTable;
        column = "surname_dl";
        label = "PI";

        c = new DbbSqlChunk(table, column, label);

        w = new DbbCheckBox(label, c);
        panel.addWidget(w);

        // New selection widget
        table = schedTable;
        column = "progid";
        label = "Programme";

        c = new DbbSqlChunk(table, column, label, String.class);

        w = new DbbCheckBox(label, c);
        panel.addWidget(w);


        /* Add this panel to the view as the column selection panel */
        view_.setColumnsPanel(panel, "North");
    }

    /**
     * Build the rows selection panel.
     *
     * We allow the user to search by (exact) name and by country code
     */
    protected void buildRows() {

        DbbWidget w;
        DbbSqlChunk c;

        DbbSqlOperator op_like = null;
        DbbSqlOperator op_eq = null;
        DbbSqlOperator op_neq = null;
        DbbSqlOperator op_in = null;
        DbbSqlOperator op_le = null;
        DbbSqlOperator op_ge = null;

        DbbSqlTable table;
        String label = "";
        String column = "";

        // Panel has five rows of widgets
        //DbbPanel panel = new DbbPanel("Selection Criteria", 3);

        // Build the operators
        try {
            op_eq = new DbbSqlOperator("=");
            op_neq = new DbbSqlOperator("!=");
            op_like = new DbbSqlOperator("LIKE");
            op_in = new DbbSqlOperator("in");
            op_le = new DbbSqlOperator("<=");
            op_ge = new DbbSqlOperator(">=");
        } catch (Exception e) {
            e.printStackTrace();
        }

        table = schedTable;
        column = "progid";
        label = "Programme";

        c = new DbbSqlChunk(table, column, label, op_like, String.class);

        w = new DbbTextField(label, c);

        selPanel.addWidget(w);

        // Naid
        table = masksTable;
        column = "mask_naid";
        label = "NAID";

        c = new DbbSqlChunk(table, column, label, op_like, String.class);

        w = new DbbTextField(label, c);

        selPanel.addWidget(w);

        // Mask id
        table = masksTable;
        column = "bar_code";
        label = "Barcode";

        c = new DbbSqlChunk(table, column, label, op_eq, String.class);

        w = new DbbIntegerTextField(label, c);

        selPanel.addWidget(w);

        // Slot number
        table = masksTable;
        column = "mask_slot_num";
        label = "Slot Num";

        c = new DbbSqlChunk(table, column, label, op_eq, Integer.class);

        w = new DbbIntegerTextField(label, c);

        selPanel.addWidget(w);

        // OB name
        table = obsTable;
        column = "item_name";
        label = "OB Name";

        c = new DbbSqlChunk(table, column, label, op_like, String.class);

        w = new DbbTextField(label, c);

        selPanel.addWidget(w);

        // OB Status
        table = obsTable;
        column = "status";
        label = "OB Status";

        c = new DbbSqlChunk(table, column, label, op_in, String.class);
        w =
            new DbbSelectionList(
                label,
                c,
                OBStatusType.getDatabaseValues(),
                OBStatusType.getDisplayValues());
        selPanel.addWidget(w);

        // Mask status Status
        table = masksTable;
        column = "mask_status";
        label = "Mask Status";

        c = new DbbSqlChunk(table, column, label, op_in, String.class);
        AssocList assList = Mask.getStatusValues();
        statusList_ =
            new DbbSelectionList(
                label,
                c,
                assList.valuesAsStrings(),
                assList.keys());
        selPanel.addWidget(statusList_);

        selPanel.addWidget(w);

        // PI Name
        table = usersTable;
        column = "surname_dl";
        label = "PI";

        c = new DbbSqlChunk(table, column, label, op_like);

        w = new DbbTextField(label, c);
        selPanel.addWidget(w);


        /* Add this panel to the view as the rows selection panel; that
         * is, the panel to define the where clause 
         */
        view_.setWherePanel(selPanel, "North");
    }

    /**
     * Build the row sorting selection panel.
     *
     * We allow the user to sort by ID and name
     */
    protected void buildSort() {

        DbbSqlChunk c;
        DbbWidget w;
        DbbSqlTable table;

        String label = "";
        String column = "";

        // Panel has one rows of widgets
        DbbPanel panel = new DbbPanel("Sorting Criteria", 1);

        // OB ID
        table = obsTable;
        column = "ob_id";
        label = "OB ID";

        c = new DbbSqlChunk(table, column, label);
        w = new DbbArrowButton(label, c);
        panel.addWidget(w);

        table = obsTable;
        column = "item_name";
        label = "OB Name";

        c = new DbbSqlChunk(table, column, label);
        w = new DbbArrowButton(label, c);
        panel.addWidget(w);

        // OB Status
        table = masksTable;
        column = "mask_status";
        label = "Mask Status";

        c = new DbbSqlChunk(table, column, label);
        w = new DbbArrowButton(label, c);
        panel.addWidget(w);

        // NAID
        table = masksTable;
        column = "mask_naid";
        label = "NAID";

        c = new DbbSqlChunk(table, column, label);
        w = new DbbArrowButton(label, c);
        panel.addWidget(w);

        // Mask ID
        table = masksTable;
        column = "bar_code";
        label = "Barcode";

        c = new DbbSqlChunk(table, column, label);
        maskIdArrowButton_ = new DbbArrowButton(label, c);
        panel.addWidget(maskIdArrowButton_);

        // Slot number
        table = masksTable;
        column = "mask_slot_num";
        label = "Slot Num";

        c = new DbbSqlChunk(table, column, label);
        slotArrowButton_ = new DbbArrowButton(label, c);
        panel.addWidget(slotArrowButton_);

        // Programme ID
        table = schedTable;
        column = "progid";
        label = "Programme";

        c = new DbbSqlChunk(table, column, label);
        w = new DbbArrowButton(label, c);
        panel.addWidget(w);

        // PI
        table = usersTable;
        column = "surname_dl";
        label = "PI";

        c = new DbbSqlChunk(table, column, label);
        w = new DbbArrowButton(label, c);
        panel.addWidget(w);

        /* Add this panel to the view as the rows sorting panel; that
         * is, the panel to define the where clause 
         */
        view_.setSortPanel(panel, "West");
    }
    
    
    /**
     * Create and populate the "File" menu in the database browser view.
     * @exception ObjectIOException object IO Exception
     */
    private void createFileMenu() {
        final String menuName = "File";
        JMenu menu = new JMenu(menuName);
        JMenuBar mb = new JMenuBar();        
        JMenuItem close = new JMenuItem("Close");
        close.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_W, InputEvent.CTRL_MASK, false));
        close.addActionListener(new ActionListener() {       
             public void actionPerformed(ActionEvent e) {
                  setVisible(false);
             }
        });        
        menu.add(new DumpMaskAdpAdmData(this,this));                    
        menu.add(close); 
        mb.add(menu);
        setJMenuBar(mb);
    }



    //----------------------------------------
    // Public Methods
    //----------------------------------------

    public void displayDetailView() {

        int row = view_.getSelectedRow();
        int maskIdCol = view_.getTable().getColIdFromName(MASK_ID);

        long db_id = ((Integer) view_.getValueAt(row,maskIdCol)).intValue();
        Mask mask_object = null;
        try {
            ObjectManager objMgr = ObjectManager.getObjectManager();

            long id = Config.getCfg().tableToUniqueId(db_id, Mask.class);
            if (objMgr.isInRegistry(id)) {
                objMgr.refreshBusObj(Media.PHASE2_DB, id, Mask.class);
            }
            mask_object =
                (Mask) (objMgr.getBusObj(Media.PHASE2_DB, id, Mask.class));

            MaskDetailView detail_view =
                (MaskDetailView) (view_.getDetailView());

            detail_view.setObject(mask_object);
        } catch (ObjectNotFoundException e) {
            e.printStackTrace();
        } catch (ObjectIOException e) {
            e.printStackTrace();
        }

    }

    /** Main method, to run directly the example. */
    public static void main(String[] args) {

        int len = args.length;
        String msg = "Usage: MaskBrowser [rmi_server_url]";

        msg = msg + "\n" + " - Example: MaskBrowser [ //usg3.hq.eso.org/DBB ]";

        if (len > 1) {
            System.out.println(msg);
            System.exit(1);
        }

        if (len == 1) {
            REMOTE = true;
            RMI_SERVER_URL = args[0];
        } else
            REMOTE = false;

        MaskBrowser dbb = new MaskBrowser();
        dbb.setTitle("Mask Browser");

        WindowListener l = new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.out.println("Exiting...");
                System.exit(0);
            }
        };

        dbb.addWindowListener(l);
        dbb.pack();
        dbb.setVisible(true);
    }

    public static MaskBrowser getDbaseWindow() throws ObjectIOException {
        if (maskBrowser_ == null) {
            synchronized (MaskBrowser.class) {
                if (maskBrowser_ == null) {
                    maskBrowser_ = new MaskBrowser();
                }
            }
        }
        return maskBrowser_;
    }

    ///////
    protected class ListListener implements ListSelectionListener {

        MaskBrowser browser_;

        public ListListener(MaskBrowser browser) {
            browser_ = browser;
        }

        public void valueChanged(ListSelectionEvent ev) {

            if (ev.getValueIsAdjusting()) {
                return;
            }
            ListSelectionModel lsm = (ListSelectionModel) ev.getSource();
            //int selectedRow = lsm.getMinSelectionIndex();
            if (!lsm.isSelectionEmpty())
                browser_.displayDetailView();
        }
    }

    protected class InstCabinetListener implements ActionListener {

        public InstCabinetListener() {
            //browser_ = browser;
        }

        public void actionPerformed(ActionEvent ev) {
            statusList_.set("(\'" + Mask.MASK_INSERTED + "\')");
            slotArrowButton_.set("1");
            slotArrowButton_.actionPerformed(null);
            maskIdArrowButton_.set("1");
            maskIdArrowButton_.actionPerformed(null);
            view_.sqlStartAction(new DbbQueryStarted(this));
        }
    }

    protected class StorageCabinetListener implements ActionListener {

        public StorageCabinetListener() {
        }

        public void actionPerformed(ActionEvent ev) {
            statusList_.set("(\'" + Mask.MASK_MANUFACTURED + "\')");
            maskIdArrowButton_.set("1");
            maskIdArrowButton_.actionPerformed(null);
            view_.sqlStartAction(new DbbQueryStarted(this));
        }
    }
    
    public Long[] getAllSelected() {
        int[] selectedRows = view_.getSelectedRows();
        Long[] selectedOBsId = new Long[selectedRows.length];
        
        JTable table = view_.getTable().getTable();            
        int obIdCol = view_.getTable().getColIdFromName(OB_ID);
        int statusCol = view_.getTable().getColIdFromName(OB_TYPE);                      

        
        for (int i = 0; i < selectedRows.length; i++) {
            Integer value =
                new Integer(table.getValueAt(selectedRows[i],obIdCol).toString());
            String type = (String)table.getValueAt(selectedRows[i],statusCol);
            if (type.equals("O")) {
                selectedOBsId[i] =
                    new Long(
                        Config.getCfg().tableToUniqueId(
                            value.longValue(),
                            ObservationBlock.class));
            } else {
                selectedOBsId[i] =
                    new Long(
                        Config.getCfg().tableToUniqueId(
                            value.longValue(),
                            CalibrationBlock.class));
            }
        }
        return selectedOBsId;
    }

    public Long getSelected() {
        Class objTypeSelected;
        int row = view_.getSelectedRow();
        if (row == -1)
            return new Long(row);
        JTable table = view_.getTable().getTable();            
        int obIdCol = view_.getTable().getColIdFromName(OB_ID);
        int statusCol = view_.getTable().getColIdFromName(OB_TYPE);                      
        Integer id = new Integer(table.getValueAt(row,obIdCol).toString());
        String type = (String) table.getValueAt(row,statusCol);
        if (type.equalsIgnoreCase(CalibrationBlock.OBSBLK_TYPE))
            objTypeSelected = ObservationBlock.class;
        else
            objTypeSelected = CalibrationBlock.class;

        Long dbid =
            new Long(
                Config.getCfg().tableToUniqueId(
                    id.longValue(),
                    objTypeSelected));

        return dbid;
    }

    public Media getMedia() {
        return Media.DBASE;
    }

    public Long[] getAll() {
        int selectedRows = view_.getRows();
        Long[] selectedOBsId = new Long[selectedRows];
        JTable table = view_.getTable().getTable();
        int obIdCol = view_.getTable().getColIdFromName(OB_ID);
        int statusCol = view_.getTable().getColIdFromName(OB_TYPE);
        for (int i = 0; i < selectedRows; i++) {
            Integer value = new Integer(table.getValueAt(i,obIdCol).toString());
            String type = (String)table.getValueAt(i,statusCol);
            if (type.equals("O")) {
                selectedOBsId[i] =
                    new Long(
                        Config.getCfg().tableToUniqueId(
                            value.longValue(),
                            ObservationBlock.class));
            } else {
                selectedOBsId[i] =
                    new Long(
                        Config.getCfg().tableToUniqueId(
                            value.longValue(),
                            CalibrationBlock.class));
            }
        }
        return selectedOBsId;
    }
    /**
     * @see org.eso.ohs.persistence.ObjectContainer#getTopLevelAncestor()
     */
    public Container getTopLevelAncestor() {
        return view_.getTable().getTable().getTopLevelAncestor();
    }
    /**
     * @see org.eso.ohs.persistence.ObjectContainer#selectAllObjects()
     */
    public void selectAllObjects() {
    }
    
    

    
}
