package org.eso.ohs.phase2.apps.masktracker;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JMenu;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbSelectionList;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.dfs.Mask;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.phase2.apps.masktracker.actions.DiscardingReportAction;
import org.eso.ohs.phase2.apps.ot.dbb.BrowserConfig;

/**
 * The QueueItemsBView class is a DB search engine used for 
 * displaying queues of OBs according to a user-definable set of 
 * properties
 * @author G Passera
 * @author Tim Canavan 
 * @author cmavros Add call to unsubscribe execution action.
 * @author cmavros 1709/OT Call ExecutionSequenceAction 
 *               class to implement progress bar
 *
 * @version $Id: MaskDiscardExecSequence.java,v 1.20 2007/10/11 11:43:12 pnunes Exp $
 *
 * @author cmavros Add call to unsubscribe execution action.
 * @author cmavros 1709/OT Call ExecutionSequenceAction 
 *               class to implement progress bar
 *
 * @version $Id: MaskDiscardExecSequence.java,v 1.20 2007/10/11 11:43:12 pnunes Exp $
 *
 */

public class MaskDiscardExecSequence extends MaskExecSequence {

     private static Logger stdlog_ =
	Logger.getLogger(MaskDiscardExecSequence.class );


    private SelectedDeleteListener selectedDeleteListener_ = null;
   
  
    private JCheckBox selected_;

  /**
   * Default constructor
   *
   */
    public MaskDiscardExecSequence(MaskQueue newQueue,MaskQueueView mkView) {
        super(newQueue,null,mkView);
        selectedDeleteListener_ = new SelectedDeleteListener();
        getView().addListSelectionListener(selectedDeleteListener_);
        appendRows();
        createPopupMenu();
        appendToolBar();
    }

    protected void removeListeners() {
        getView().removeListSelectionListener(selectedDeleteListener_);
    }
    
    
    public void appendRows() {    
      DbbSqlOperator op_in = null;
      DbbSqlOperator op_equals = null;
      try {
          op_in = new DbbSqlOperator("in");
          op_equals = new DbbSqlOperator("=");
      }
      catch( Exception e ) {
          stdlog_.error( e ); 
      }
      DbbPanel rowPanel  = getRowPanel();          
      String label_name = "Mask status";
      String column_name = "mask_status";
      DbbSqlChunk c = new DbbSqlChunk(BrowserConfig.masks_table, column_name, label_name, op_in, String.class);
      Object[] statusValues = {Mask.MASK_MANUFACTURED,Mask.PENDING_DISCARDED};
      String[] statusDispValues = {Mask.getDisplayStatusValue(Mask.MASK_MANUFACTURED),
                                   Mask.getDisplayStatusValue(Mask.PENDING_DISCARDED)};  
      DbbSelectionList status = new DbbSelectionList(label_name,c,statusValues, statusDispValues);
      rowPanel.addWidget(status);
      status.set("(" + "'" + Mask.MASK_MANUFACTURED + "'" + ",'" + Mask.PENDING_DISCARDED + "'" + ")");
      status.fireDbbWidgetAction();       
      getView().setWherePanel(rowPanel, "North");          
    }  


  public JPopupMenu createPopupMenu() {
        JPopupMenu popupMenu = new JPopupMenu();
        JMenu disMenu  = new JMenu("Mask Discarding Order");
        JTable table = getOtDbbView().getJTable();
        disMenu.add(new DiscardingReportAction(getOtDbbView(),table,"Mask Discarding Order",true));
        disMenu.add(new DiscardingReportAction(getOtDbbView(),table,"Mask Discarding Statistics",false));
        popupMenu.add(disMenu);
        MaskDiscarding.getInstance().setSelectedMaskList(getSelectedMaskIds());
        return popupMenu; 
    }


    public void appendToolBar() {
         selected_ = new JCheckBox();
         getToolBar().add(selected_);
         selected_.setText("Selected");
         selected_.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent evt) {
                     MaskDiscarding.getInstance().setSingleSelect(selected_.isSelected());
                     MaskDiscarding.getInstance().setSelectedMaskList(getSelectedMaskIds());
                     
                 } 
         });        
         selected_.addActionListener(new ActionListener() {
                 public void actionPerformed(ActionEvent evt) {
                     MaskDiscarding.getInstance().setSingleSelect(selected_.isSelected());
                 } 
         });
    } 

    private class SelectedDeleteListener implements ListSelectionListener {
        
        public SelectedDeleteListener() { }
        
        public void valueChanged(ListSelectionEvent ev) {    
            if (ev.getValueIsAdjusting()) {
                return;
            }
            ListSelectionModel lsm = (ListSelectionModel)ev.getSource();
            int currentRow = lsm.getMinSelectionIndex();
            if(!lsm.isSelectionEmpty() && currentRow >= 0) {
                  MaskDiscarding.getInstance().setSelectedMaskList(getSelectedMaskIds());
            }
                     
        }
    } 
      
}
