package org.eso.ohs.phase2.apps.masktracker;

import java.io.IOException;
import java.io.StringReader;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.widgets.OutOfMemoryReport;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.PafData;
import org.eso.ohs.core.utilities.PafFile;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.Mask;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.MaskInstrument;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.MaskEvent;
import org.eso.ohs.persistence.dbase.phase2.MaskDBIO;
import org.eso.ohs.persistence.dbase.phase2.MaskQueueManager;
import org.eso.ohs.persistence.dbase.phase2.MaskDBIO.IdInfo;
import org.eso.ohs.phase2.apps.ot.gui.QueueView;
import org.eso.ohs.phase2.icdVcs.IcdVcsCfg;

/**
 * Listener for script events from the BOB interface This class only get
 * instanciated one. It is a Singleton reference
 * <p>
 * $Id: MaskDiscarding.java,v 1.23 2007/10/11 11:43:12 pnunes Exp $<br>
 * 
 * @author Bob Kemp
 */
public class MaskDiscarding extends MaskActionGenerator {

    private static String obrepDbName_ = Config.getCfg().getDbName(
            ObservationBlock.getSuffix());

    private static TextDisplayWidget log_ = new TextDisplayWidget("Report");

    /**
     * Mask Discarding singleton reference
     */

    private static MaskDiscarding instance_ = null;

    private Vector<String> maskIdVec_ = new Vector<String>();

    private Vector<IdInfo> cannotDisc_ = new Vector<IdInfo>();

    /**
     * Default constructor can only be called from outside this class to make
     * sure that only one reference to it exists
     */


    private MaskDiscarding() {
    }

    private String[] selectedMaskList_;

    public void setSelectedMaskList(String[] selectedMaskList) {
        selectedMaskList_ = selectedMaskList;
    }

    public boolean inSelectedMaskList(String mask) {
        for (int i = 0; i < selectedMaskList_.length; i++) {
            if (selectedMaskList_[i].equals(mask))
                return true;
        }
        return false;
    }

    /**
     * Get an instance of the Mask Discarding Singleton refernce
     * 
     * @return MaskDiscarding instance
     */

    public static MaskDiscarding getInstance() {
        if (instance_ == null) {
            instance_ = new MaskDiscarding();
        }

        return instance_;
    }

    /**
     * Generaate a mask discarding order
     * 
     * @param maxNumObs
     *            Max number of OB's to be requested
     */

    public String generateOrder(int maxNumOBs) {
        try {
            IcdVcsCfg cfg = IcdVcsCfg.getCfg();
            long[] obIdList = cfg.getSelections(maxNumOBs);
            QueueView queueView = MTViewManager.getInstance()
                    .getSelectedQueueView();
            if (queueView == null)
                return "";
            String question = "Are you sure you to send a discarding order for "
                    + queueView.getQueue().getName() + " ?";

            SelectionDialog d = new SelectionDialog(queueView, question,
                    "Mask Discarding", true, getDiscardingReport(obIdList),
                    "Discarding Report Statistics");
            d.show();
            int value = d.getValue();
            if (value == JOptionPane.OK_OPTION
                    || value == JOptionPane.CANCEL_OPTION) {
                d.setVisible(false);
                d.dispose();
            }
            if (value == JOptionPane.CANCEL_OPTION)
                return "";
            String obd = getDiscardingText(obIdList);
            Instrument inst = InstrumentList.getInstance().getInstrument(
                    "VIMOS");
            MaskQueueManager.setMaskRowStatus(maskIdVec_,
                    Mask.PENDING_DISCARDED, ((MaskInstrument) inst)
                            .getDummyNaid());
            saveFile(TextUtils.currentISODateTime() + ".mdo", obd.toString());
            return obd.toString();
        } catch (OutOfMemoryError e) {
            e.printStackTrace();
            OutOfMemoryReport.showMessage();
            throw e;
        } catch (Exception e) {
            e.printStackTrace();
        }

        return "";
    }

    public String reportString() {
        StringBuffer naidReport = new StringBuffer();
        if (maskIdVec_.size() <= 0)
            naidReport.append("No masks will be discarded for these ob's");
        else {
            for (int i = 0; i < maskIdVec_.size(); i++)
                naidReport.append(maskIdVec_.elementAt(i) + "\n");
            if (maskIdVec_.size() > 0)
                naidReport.append("A total of " + maskIdVec_.size()
                        + " will be discarded for these ob's\n");
        }
        if (cannotDisc_.size() > 0) {
            naidReport.append("The following Masks could not be discarded\n");
            naidReport
                    .append("Mask Status must be MaskStored and have a valid mask id \n");
            for (int i = 0; i < cannotDisc_.size(); i++) {
                naidReport.append(cannotDisc_.elementAt(i).toString());
                naidReport.append("\n");
            }
        }
        return naidReport.toString();
    }

    public String getDiscardingText(long[] idList) throws SQLException {
        if ((idList != null) && (idList.length > 0)) {
            //Instrument inst =
            // InstrumentList.getInstrument(cbList[0].getOd().getInstrument());
            StringBuffer paf = new StringBuffer();
            SimpleDateFormat isoTimeFormat_;
            paf.append(ReportHeaders.generateDiscardingOrderHeader());
            //Vector maskIdVec = new Vector();
            paf.append(getMaskIdsString(idList));
            paf.append("\n");
            paf.append("TPL.FILE.DIRNAME   \"\"");
            paf.append("\n");
            StringBuffer obd = new StringBuffer();
            obd.append(ReportHeaders.generateObdHeader());
            String body = ReportHeaders.getObdBody("Mask Discarding OB");
            obd.append(body);
            obd.append(ReportHeaders
                    .generateMaskTemplate("VIMOS_mmu_Discarding_order"));
            String pafStr = paf.toString();
            obd.append("SEQ.MDO" + "\t\t\"" + TextUtils.escapedString(pafStr)
                    + "\"");
            return obd.toString();
        }
        return "";
        //System.out.println("String = " + obd.toString());
    }

    public String getDiscardingReport(long[] idList) throws SQLException {

        StringBuffer paf = new StringBuffer();
        Instrument inst = InstrumentList.getInstance().getInstrument("VIMOS");
        int totMasks = 0;
        Vector<IdInfo> vecMaskIds = new Vector<IdInfo>();
        for (int i = 0; i < idList.length; i++) {
            IdInfo[] tmpMaskIds = MaskDBIO.getMaskIdsForOB(idList[i]);

            for (int j = 0; j < tmpMaskIds.length; j++) {
                if (tmpMaskIds[j].getMaskStatus()
                        .equals(Mask.MASK_MANUFACTURED)
                        && !tmpMaskIds[j].getMaskId().equals("-99")) {
                    if (isSingleSelect()) {
                        if (inSelectedMaskList(tmpMaskIds[j].getMaskId())) {
                            if (!vecMaskIds.contains(tmpMaskIds[j])) {
                                vecMaskIds.addElement(tmpMaskIds[j]);
                            }
                        }
                    } else {
                        if (!vecMaskIds.contains(tmpMaskIds[j])) {
                            vecMaskIds.addElement(tmpMaskIds[j]);
                        }
                    }
                } else {
                    cannotDisc_.addElement(tmpMaskIds[j]);
                }
            }

        }
        if (vecMaskIds.size() == 0) {
            paf.append("No Masks will be discarded\n");
            paf
                    .append("Masks need to have a mask status of MaskStored\nand have a valid barcode to be discarded\n");
        } else {
            IdInfo[] maskIds = new IdInfo[vecMaskIds.size()];
            vecMaskIds.copyInto(maskIds);
            for (int j = 0; j < maskIds.length; j++) {
                String status = TextUtils.padString(Mask
                        .getDisplayStatusValue(maskIds[j].getMaskStatus()), 12);
                paf.append(status);
                paf.append(" ");
                paf.append("INS.MASK" + maskIds[j].getAdpIndex() + ".ID");
                paf.append("\t");
                paf.append(maskIds[j].getMaskId());
                paf.append("\t");
                paf.append(maskIds[j].getMaskName());
                paf.append("\n");
            }
            paf.append("A total of " + vecMaskIds.size()
                    + " masks will be discarded\n");
        }
        return paf.toString();
    }

    public String getMaskIdsString(long[] idList) throws SQLException {
        maskIdVec_.removeAllElements();
        cannotDisc_.removeAllElements();
        StringBuffer paf = new StringBuffer();
        Instrument inst = InstrumentList.getInstance().getInstrument("VIMOS");
        int maxMaskCount = 0;
        int maxIndex = 0;
        int maskCount = MaskDBIO.getNumMasksForStatus(Mask.MASK_INSERTED)
                + MaskDBIO.getNumMasksForStatus(Mask.PENDING_INSERTION);

        for (int i = 0; i < idList.length; i++) {
            maxMaskCount = ((MaskInstrument) inst).getMaxMaskCount();
            IdInfo[] tmpMaskIds = MaskDBIO.getMaskIdsForOB(idList[i]);
            Vector<IdInfo> vecMaskIds = new Vector<IdInfo>();
            for (int j = 0; j < tmpMaskIds.length; j++) {
                if (tmpMaskIds[j].getMaskStatus()
                        .equals(Mask.MASK_MANUFACTURED)
                        && !tmpMaskIds[j].getMaskId().equals("-99")) {
                    if (isSingleSelect()) {
                        if (inSelectedMaskList(tmpMaskIds[j].getMaskId())) {
                            vecMaskIds.addElement(tmpMaskIds[j]);
                        }
                    } else
                        vecMaskIds.addElement(tmpMaskIds[j]);
                } else {
                    cannotDisc_.addElement(tmpMaskIds[j]);
                }
            }
            IdInfo[] maskIds = new IdInfo[vecMaskIds.size()];
            vecMaskIds.copyInto(maskIds);
            for (int j = 0; j < maskIds.length; j++) {
                if (maskIdVec_.contains(maskIds[j].getMaskId()))
                    continue;
                else
                    maskIdVec_.addElement(maskIds[j].getMaskId());
                paf.append("INS.MASK" + maskIds[j].getAdpIndex() + ".ID");
                paf.append("\t\t\"");
                paf.append(maskIds[j].getMaskId());
                paf.append("\"");
                paf.append("\n");
            }
        }
        return paf.toString();
    }

    // COPY OF METHOD IN MASK MANUFACTURING CLASS
    // XXX This should go into a separate interface class in
    //     order to keep mask code distinct from the actual SQL.

    /**
     * Receive a mask discarding report from BOB. This accepts a string from BOB
     * and parses it to work out what mask have to be deleted from the database
     * table
     * 
     * @param report
     *            Complete report from BOB
     */

    public void receiveReport(String report) {
        System.out.println("$$$$$$$$$ Mask Discarding Report sent " + report);
        //Updater.propagateMaskInfo();
        try {
            MaskInstrument inst = (MaskInstrument) InstrumentList.getInstance()
                    .getInstrument("VIMOS");
            PafFile paf = new PafFile(new StringReader(report));
            PafData[] pd = paf.getIdentifierList();
            StringBuffer sb = new StringBuffer();
            for (int j = 0; j < pd.length; j++) {
                String name = pd[j].getIdentifier();
                System.out.println("Name = " + name);
                if (name.startsWith("INS.MASK") && name.endsWith(".ID")) {

                    sb.append("'" + pd[j].getValue() + "'");
                    if (j < pd.length - 1)
                        sb.append(',');
                }

            }

            if (sb.length() == 0)
                return;

            String maskIdList = sb.toString();
            String sql = "UPDATE " + obrepDbName_ + "..masks set "
                    + obrepDbName_ + "..masks.mask_status = "
                    + SqlUtils.sqlEscapedString(Mask.MASK_DISCARDED)
                    + " WHERE " + obrepDbName_ + "..masks.bar_code in ("
                    + maskIdList + ")" + " AND " + obrepDbName_
                    + "..masks.mask_slot_num = "
                    + Mask.DEFAULT_MASK_SLOT_NUMBER;

            executeUpdate(sql);

        } catch (SQLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        MaskQueueManager.fireMaskPropertyChange(new MaskEvent("MaskStatus",
                Mask.MASK_DISCARDED));

    }

    // XXX This should go into a separate interface class in
    //     order to keep mask code distinct from the actual SQL.

    /**
     * Execute the report to delete the masks from the masks table.
     * 
     * @param sql
     *            SQL generated from the parsing of the deletion report
     */

    private void executeUpdate(String sql) throws SQLException {
        ConnectionPool pool = ConnectionPool.getPool();
        Connection conn = null;
        Statement stmt = null;

        try {
            conn = pool.acquireConnection(Config.getCfg().getUrl(
                    ObservationBlock.getSuffix()), Config.getCfg()
                    .getDbUserName(), Config.getCfg().getDbPassword());

            stmt = conn.createStatement();

            stmt.executeUpdate(sql);
        } finally {
            if (conn != null) {
                try {
                    stmt.close();
                } catch (SQLException e) {
                    // ignore
                }
                pool.releaseConnection(conn);
            }
        }
    }

}

