package org.eso.ohs.phase2.apps.masktracker;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableColumnModel;

import org.eso.ohs.core.dbb.client.DbaseDec;
import org.eso.ohs.core.dbb.client.DbaseRA;
import org.eso.ohs.core.dbb.client.DbbArrowButton;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.dfs.MaskStatusType;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.MaskEvent;
import org.eso.ohs.persistence.MaskStatusListener;
import org.eso.ohs.persistence.OBEvent;
import org.eso.ohs.persistence.OBListener;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.dbase.phase2.MaskQueueManager;
import org.eso.ohs.persistence.dbase.phase2.QueueManager;
import org.eso.ohs.phase2.apps.ot.dbb.BrowserConfig;
import org.eso.ohs.phase2.apps.ot.dbb.OTDbbView;
import org.eso.ohs.phase2.apps.ot.gui.OBTreeView;
import org.eso.ohs.phase2.apps.ot.gui.TableListSelectionListener;
import org.eso.ohs.phase2.apps.ot.gui.TableMenuEnable;

/**
 * The QueueItemsBView class is a DB search engine used for 
 * displaying queues of OBs according to a user-definable set of 
 * properties 
 *
 * @author Giorgio Passera DMD/ESO <gpassera@eso.org>
 */
public abstract class MaskExecSequence
    
    implements MaskStatusListener,OBListener,TableMenuEnable {

    public static String MASK_NAID = "NAID";

    public static String MASK_ID = "Barcode";

    private DbbPanel rowPanel;
    
    private DbbArrowButton seqWidget;
    
    
    private OTDbbView otDbbView;
    
    
    private DbbView view_;
    
    
    private Queue queue_;
    
    
    
    
    private JButton removeButton;
    
    
    private JButton removeAllButton;


    private ListSelectionListener listListener_;
    
    
    private JToolBar toolBar_;

    /**
     * Status window which displays reports on imports.     
     */

    private static TextDisplayWidget log_ =
        new TextDisplayWidget("Execution Sequence Report");

    private final static String HEADER =
        ("EXECUTION MESSAGE LOG "
            + TextUtils.currentISODateTime()
            + "(UT)\n"
            + "==================================================\n\n");

    protected JCheckBox selected_ = new JCheckBox("Selected");

    private MaskQueueView mkView_;

    private RightClickStatusPopupListener mouseListener_ =
        new RightClickStatusPopupListener();

	private DbbTextField queueIDTF;
	
	
	private JPanel outerPanel;
	
	

    /**
      /**
       * Default constructor
     * @throws SQLException 
       *
       */
    public MaskExecSequence(
        MaskQueue newQueue,
        OBTreeView obTreeView,
        MaskQueueView mkView)  {

        try {
        	otDbbView = new OTDbbView();
        	view_ = otDbbView.getDbbView();
            queue_ = newQueue;    	
            mkView_ = mkView;
            buildToolBar();
            listListener_ = new TableListSelectionListener(this);
            view_.addListSelectionListener(listListener_);
			buildColumns();
			buildRows();
			buildSort();
			buildStructure();
	        MaskQueueManager.addMaskListener(this);
	        QueueManager.addOBListener(this);
	        view_.getTable().getTable().addMouseListener(mouseListener_);
		} catch (MalformedURLException e) {
            MsgManager.errmsg(e.getMessage());			
		} catch (RemoteException e) {
            MsgManager.errmsg(e.getMessage());
		} catch (NotBoundException e) {
            MsgManager.errmsg(e.getMessage());
		} catch (SQLException e) {
            MsgManager.errmsg(e.getMessage());
		}
    }
    
    
    /**
     * void
     */
    private void  buildStructure() {
        outerPanel = new JPanel(new BorderLayout());
        outerPanel.add(toolBar_, BorderLayout.NORTH);
        outerPanel.add(view_, BorderLayout.CENTER);        
        setEnabled(false);            
    }

    

    public int getMaskNaidColIndex() {
        JTable table = view_.getTable().getTable();
        TableColumnModel tblMdl = table.getColumnModel();
        return tblMdl.getColumnIndex(MASK_NAID);

    }

    public int getMaskIdColIndex() {
        JTable table = view_.getTable().getTable();
        TableColumnModel tblMdl = table.getColumnModel();
        return tblMdl.getColumnIndex(MASK_ID);

    }

    /**Returns  the ID of an OB of a specific row*/
    public String[] getSelectedMaskNaids() {
        int[] selectedRows = view_.getSelectedRows();
        Vector<String> vec = new Vector<String>();
        for (int i = 0; i < selectedRows.length; i++) {
            String value =
                new String(
                    (String) view_.getValueAt(
                        selectedRows[i],
                        getMaskNaidColIndex()));
            if (!vec.contains(value))
                vec.addElement(value);
        }
        String[] naids = new String[vec.size()];
        vec.copyInto(naids);
        return naids;

    }

    /**Returns  the ID of an OB of a specific row*/
    public String[] getSelectedMaskIds() {
        int[] selectedRows = view_.getSelectedRows();
        Vector<String> vec = new Vector<String>();
        for (int i = 0; i < selectedRows.length; i++) {
            String value =
                new String(
                    (String) view_.getValueAt(
                        selectedRows[i],
                        getMaskIdColIndex()));
            if (!vec.contains(value))
                vec.addElement(value);
        }
        String[] maskIds = new String[vec.size()];
        vec.copyInto(maskIds);
        return maskIds;

    }
    
    /**
     * Set the new Queue that will be displayed into the view
     */
    public void updateView(Queue newQueue) throws SQLException {
        queueIDTF.set(new Long(Config.getCfg().uniqueToTableId(newQueue.getId())));
        if (view_.getSortPanel() != null) {
            view_.getSortPanel().reset();
            seqWidget.actionPerformed(new ActionEvent(this, 0, null));
        }
        view_.sqlStartAction(null);
    }

    public void obPropertyChange(OBEvent ev) {
        if (ev.getValue().equals("Delete") ||
            ev.getValue().equals(CalibrationBlock.DEFINED) ||
            ev.getValue().equals(CalibrationBlock.DEFINED)) {
            long[] obIds = ev.getOBsId();
            if (obIds != null && obIds.length > 0) {
                removeOBs(obIds);
            }
        } else {
            long[] objMgrObIds = ev.getOBsId();
            for (int i = 0; i < objMgrObIds.length; i++) {
                long obId = Config.getCfg().uniqueToTableId(objMgrObIds[i]);
                refreshViewsStatus(Config.getCfg().uniqueToTableId(objMgrObIds[i]), ev.getValue());                
            }
        	
        }
    }

    public void removeOBs(long[] selectedOBIDs) {
        try {
            QueueManager.deleteOBsFromTable(
                selectedOBIDs,
                queue_.getId(),
                QueueManager.exec_tmp);
            ((MaskQueue) queue_).removeExecQueueItems(selectedOBIDs);
            mkView_.reQueryExecSequence();
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(view_.getTable().getTable(), e);
        }

    }

    
    public void startQuery() {
    	view_.sqlStartAction(null);
    }
    
    
    public void maskPropertyChange(MaskEvent ev) {
    	otDbbView.sqlStartAction();
    }
    
    
    private void removeOBs() {
        removeOBs(otDbbView.getAllSelectedOBIds());	
    }
    
    private void removeAllOBs() {
        removeOBs(otDbbView.getAllOBIds());	
    }

    private void buildToolBar() {
        toolBar_ = new JToolBar();
        removeButton = new JButton("Remove");
        removeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                removeOBs();
            }

        });
        removeAllButton = new JButton("Remove All");
        removeAllButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                removeAllOBs();
            }

        });

        
        toolBar_.add(removeButton);
        toolBar_.add(removeAllButton);
        setEnabled(false);
    }

    private class RightClickStatusPopupListener extends MouseAdapter {
        public void mousePressed(MouseEvent e) {
            if (e.getID() == MouseEvent.MOUSE_PRESSED
                && (e.getModifiers() & MouseEvent.BUTTON3_MASK) != 0) {
                JPopupMenu popup = createPopupMenu();
                popup.show(e.getComponent(), e.getX(), e.getY());
            }
        }
    }

    public abstract JPopupMenu createPopupMenu();

    public void setEnabled(boolean enabled) {
        removeAllButton.setEnabled(enabled);
        removeButton.setEnabled(enabled);
    }

    /**Return a set of OBsID that belong to the execution sequence
     *@param maxNumOBs specifies the number of ObsId requested
     *@return a set of OBsID that belong to the execution sequence 
     *        or null
     */
    public long[] getOBsIdFromExecutionSequence(int maxNumOBs) {
        try {

            if (maxNumOBs > view_.getRows()) {
                maxNumOBs = view_.getRows();
            }
            long[] returnList = new long[maxNumOBs];
            for (int i = 0; i < maxNumOBs; i++) {
                returnList[i] = otDbbView.getOBID(i);
            }
            return returnList;
        } catch (NumberFormatException e) {
            e.printStackTrace();
            return null;
        }
    }

    public boolean isOBSelected() {
        int[] selectedRows = view_.getSelectedRows();
        return (selectedRows.length != 0);
    }

    /**
      * Build DB browser view: SELECT panel
      *
      * @exception SQLException JDBC exception occurs
      * @exception RemoteException RMI exception occurs
      * @exception MalformedURLException malformed url i.e. servername//port
      * @exception NotBoundException client could not bind to server
      */
    public void buildColumns()
        throws
            NotBoundException,
            MalformedURLException,
            SQLException,
            RemoteException {
        // collect the default SQLChunks from super.
        //        super.buildColumns();

        String label_name;
        String column_name;

        //label_name = "QueueID";
        //column_name = "sch_id";
        //engine_.addDefaultColumnsChunk(
        //    new DbbSqlChunk(si_table,column_name, label_name));
        
        DbbSqlEngine engine = otDbbView.getDbbView().getSqlEngine();

        column_name = "ob_id";
        label_name = "OB ID";
        engine.addDefaultColumnsChunk(
            new DbbSqlChunk(BrowserConfig.es_tmp, column_name, label_name));

        DbbPanel panel = new DbbPanel("Selected Columns", 7);
        view_.setColumnsPanel(panel, "North", false);
        
        
        column_name = "type";
        label_name = "Type";
        engine.addDefaultColumnsChunk(
            new DbbSqlChunk(BrowserConfig.ob, column_name, label_name));

        // Status 
        column_name = "status";
        label_name = "Status";
        engine.addDefaultColumnsChunk(
            new DbbSqlChunk(BrowserConfig.ob, column_name, label_name));

        // OB Name 
        label_name = "OB Name";
        column_name = "item_name";
        engine.addDefaultColumnsChunk(
            new DbbSqlChunk(BrowserConfig.ob, column_name, label_name));


        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.obs_masks, "adp_index", "Channel"));

        // ProgID
        label_name = "ProgID";
        column_name = "progid";
        engine.addDefaultColumnsChunk(
            new DbbSqlChunk(BrowserConfig.sr, column_name, label_name, String.class));

        column_name = "mask_status";
        label_name = "Mask Status";
        engine.addDefaultColumnsChunk(
            new DbbSqlChunk(
                BrowserConfig.masks,
                column_name,
                label_name,
                MaskStatusType.class));

        column_name = "mask_name";
        label_name = "Mask Name";
        engine.addDefaultColumnsChunk(
            new DbbSqlChunk(BrowserConfig.masks, column_name, label_name));

        DbbSqlChunk def = new DbbSqlChunk(BrowserConfig.masks, "mask_naid", "NAID");
        engine.addDefaultColumnsChunk(def);

        def = new DbbSqlChunk(BrowserConfig.masks, "bar_code", "Barcode");
        engine.addDefaultColumnsChunk(def);

        def = new DbbSqlChunk(BrowserConfig.masks, "mask_slot_num", "Slot Num");
        engine.addDefaultColumnsChunk(def);

        // PI (surname) 
        label_name = "LastName";
        column_name = "last_name";
        engine.addDefaultColumnsChunk(
            new DbbSqlChunk(BrowserConfig.profiles, column_name, label_name));

        // RA 
        label_name = "RA";
        column_name = "ra";
        engine.addDefaultColumnsChunk(
            new DbbSqlChunk(BrowserConfig.tp, column_name, label_name, DbaseRA.class));

        // Dec 
        label_name = "Dec";
        column_name = "dec";
        engine.addDefaultColumnsChunk(
            new DbbSqlChunk(BrowserConfig.tp, column_name, label_name, DbaseDec.class));

        

    }

    public void makeComponentsAavailableForGC() {
        view_.removeGlassPaneMouseListener();
        MaskQueueManager.removeMaskStatusListener(this);
        QueueManager.removeOBListener(this);
        view_.getTable().getTable().removeMouseListener(mouseListener_);
        
        
        //super.removeListeners();
    }

    public DbbPanel getRowPanel() {
        return rowPanel;
    }

    /**Build DB browser view: rows selection panel*/
    public void buildRows() {

        DbbSqlChunk c;

        String label_name;
        String column_name;

        DbbSqlOperator op_equal = null;
        DbbSqlOperator op_min = null;
        DbbSqlOperator op_max = null;
        DbbSqlOperator op_like = null;
        DbbSqlOperator op_in = null;

        try {
            op_equal = new DbbSqlOperator("=");
            op_min = new DbbSqlOperator("<=");
            op_max = new DbbSqlOperator(">=");
            op_like = new DbbSqlOperator("like");
            op_in = new DbbSqlOperator("in");
        } catch (Exception e) {
            e.printStackTrace();
        }

        rowPanel = new DbbPanel("Selection Criteria", 3);

        // Queue ID
        label_name = "Queue ID";
        column_name = "sch_id";

        c =
            new DbbSqlChunk(
                BrowserConfig.es_tmp,
                column_name,
                label_name,
                op_equal,
                Integer.class);
        queueIDTF = new DbbTextField(label_name, c);
        rowPanel.addWidget(queueIDTF);

        queueIDTF.set(new Long(Config.getCfg().uniqueToTableId(queue_.getId())));
        queueIDTF.setEnabled(false);
        //   queueIDTF.setVisible(false);

        view_.setWherePanel(rowPanel, "North", false);
    }

    /**
     * Method to append OBs that pass verification test to the 
     * Execution Sequence View.
     * Called by endOfAction method in ot.actions.ExecutionSequenceAction
     *
     * @param setOBIDs list of OBs to be added to the  Execution Sequence View
     */
    public boolean appendOBs(long[] appendIds) {
        JTable table = view_.getTable().getTable();
        try {
            ((MaskQueue) queue_).appendToExecSequence(appendIds,ObjectManager.getObjectManager().getTypes(appendIds));
            int length = ((MaskQueue) queue_).getExecutionSequence().length;
            QueueManager.appendOBsToMaskExecSequence(
                appendIds,
                Config.getCfg().uniqueToTableId(queue_.getId()),
                length);
            mkView_.reQueryExecSequence();
            //startQuery();
        } catch (ObjectIOException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    /** Build DB browser view: sort selection panel*/
    public void buildSort() {

        DbbPanel panel = new DbbPanel("Sorting Criteria", 0);
        DbbSqlChunk c;
        DbbArrowButton w;
        String column_name;
        String label_name;

        //sequence
        label_name = "sequence";
        column_name = "sequence";

        c = new DbbSqlChunk(BrowserConfig.es_tmp, column_name, label_name);
        seqWidget = new DbbArrowButton(label_name, c, 1);

        panel.addWidget(seqWidget);
        seqWidget.actionPerformed(new ActionEvent(this, 0, null));

        view_.setSortPanel(panel, "West", false);
    }

    public void refreshViewsStatus(long obId, String status) {
        otDbbView.refreshViewsStatus(obId,status);
    }


	public JPanel getOuterPanel() {
		return outerPanel;
	}


	public void setOuterPanel(JPanel outerPanel) {
		this.outerPanel = outerPanel;
	}


	public DbbView getView() {
		return view_;
	}


	public void setView(DbbView view) {
		this.view_ = view;
	}


	public OTDbbView getOtDbbView() {
		return otDbbView;
	}


	public void setOtDbbView(OTDbbView otDbbView) {
		this.otDbbView = otDbbView;
	}


	public JToolBar getToolBar() {
		return toolBar_;
	}


	public void setToolBar(JToolBar toolBar) {
		this.toolBar_ = toolBar;
	}


	public Queue getQueue() {
		return queue_;
	}


	public void setQueue(Queue queue) {
		this.queue_ = queue;
	}



}
