package org.eso.ohs.phase2.apps.masktracker;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.sql.SQLException;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.basic.BasicTabbedPaneUI;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.phase2.apps.ot.gui.OBTreeView;
import org.eso.ohs.phase2.apps.ot.gui.QueuePropertiesView;
import org.eso.ohs.phase2.apps.ot.gui.QueueView;
import org.eso.ohs.phase2.apps.ot.gui.QueueViewMenuItems;

/**
 * This view displays information about MaskQueue objects
 * 
 * @author G.Passera, DMD/ESO, <gpassera@eso.org>
 * @author A M Chavan, 13-Jun-2001: removed 'rename queue' menu item
 * @author cmavros, 07/02/02, 1510/OT and add log4j
 * @author cmavros, 12-02-02 1641/OT
 * @author tcanavan, 25/02/02 Cleaned up indentation and put in some javadoc
 * @version $Revision: 1.31 $
 */

public class MaskQueueView extends QueueView {

	public static final int TAB_MAN = 0;

	public static final int TAB_INSERTED = 1;

	public static final int TAB_DISCARDED = 2;

	private static TextDisplayWidget log_ = new TextDisplayWidget(
			"Append Queue Items Report");

	private static final String CONFIG_KEYWORD = "MTQUEUEVIEW.CONFIG.CONFIGMENU";

	private MaskQueue maskQueue_;

	private JTabbedPane tabPane_;

	private OBTreeView obTreeView_;

	private MaskManQueueBrowser manQueueBrowser_;

	private MaskInsertedQueueBrowser insertQueueBrowser_;

	private MaskDiscardedQueueBrowser discardQueueBrowser_;

	private MaskExecSequence maskManSequence_;

	private MaskExecSequence maskInsertSequence_;

	private MaskExecSequence maskDiscardSequence_;


	public MaskQueueView(MaskQueue newQueue) {
		super();
		initMaskQueueView(newQueue);
	}
	
	private void initMaskQueueView(Queue newQueue) {
        newQueue(newQueue);
        setQueuePropertiesView(new QueuePropertiesView(newQueue));
        initLocalVars();
        createView();
        menuItems = new QueueViewMenuItems(this,getQueueItemsView().getView(),getQueueItemsView().getView().getJTable(),getQueueItemsView());
        menuItems.buildCompleteMainMenu();
        menuItems.buildMenu(queueItemsView_.getView().getJTable());
        reQueryView();
        //queueItemsView_.getDbbView().sqlStartAction(null);
        setTitle("Queue view: " + queue_.getName() + " ("
                + queue_.getTelescope() + ")");
        ActionSuperclass actionMenuDefault = getActionMenuDefault(); 
        if (actionMenuDefault != null) {
            actionMenuDefault.actionPerformedImpl(new ActionEvent(
                    actionMenuDefault, 0, ""));
        }                        
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        WindowAdapter wl = new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                closeQueue();
            }
        };
        addWindowListener(wl);

    }


	private void initLocalVars() {
		tabPane_ = new JTabbedPane();
		tabPane_.setUI(new MyTabbedPaneUI(Color.MAGENTA));
		maskManSequence_ = new MaskManExecSequence((MaskQueue) queue_, this);
		maskInsertSequence_ = new MaskInsertExecSequence((MaskQueue) queue_,
				this);
		maskDiscardSequence_ = new MaskDiscardExecSequence((MaskQueue) queue_,
				this);
		manQueueBrowser_ = new MaskManQueueBrowser(queue_,
				maskManSequence_, this);
		insertQueueBrowser_ = new MaskInsertedQueueBrowser(queue_,
				maskInsertSequence_, this);
		discardQueueBrowser_ = new MaskDiscardedQueueBrowser(queue_,
				obTreeView_, maskDiscardSequence_, this);

	}

	public void disposeNoSave() {
		maskManSequence_.makeComponentsAavailableForGC();
		manQueueBrowser_.removeMaskListener();
		manQueueBrowser_.makeComponentsAavailableForGC();
		maskInsertSequence_.makeComponentsAavailableForGC();
		insertQueueBrowser_.removeMaskListener();
		insertQueueBrowser_.makeComponentsAavailableForGC();
		maskDiscardSequence_.makeComponentsAavailableForGC();
		discardQueueBrowser_.removeMaskListener();
		discardQueueBrowser_.makeComponentsAavailableForGC();
		super.dispose();
	}

	public void finalize() throws Throwable {
		System.out.println("Finalising MaskQueueView");
		super.finalize();
	}

	protected String getOptionsKeyword() {
		return CONFIG_KEYWORD;
	}

	/** Initialyzes the view */
	protected void createView() {
		manQueueBrowser_.getOuterPanel().setSize(250, 250);
		JSplitPane splitPane1 = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				manQueueBrowser_.getOuterPanel(), maskManSequence_.getOuterPanel());
		tabPane_.add("Mask Manufacturing", splitPane1);
		queuePropertiesView_ = new QueuePropertiesView(queue_);
		JSplitPane splitPane2 = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				insertQueueBrowser_.getOuterPanel(), maskInsertSequence_.getOuterPanel());
		tabPane_.add("Mask Insertion", splitPane2);
		JSplitPane splitPane3 = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				discardQueueBrowser_.getOuterPanel(), maskDiscardSequence_.getOuterPanel());
		tabPane_.add("Mask Discarding", splitPane3);
		setQueueItemsView(manQueueBrowser_);
		JPanel content = new JPanel();
		content.setLayout(new BorderLayout());
		content.add(queuePropertiesView_, BorderLayout.NORTH);
		content.add(tabPane_, BorderLayout.CENTER);
		getContentPane().add(content, BorderLayout.CENTER);
		pack();
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();

		int frame_width = (screen.width / 5) * 4;
		int frame_height = (screen.height / 10) * 9;
		Dimension window = new Dimension(frame_width, frame_height);
		splitPane1.setDividerLocation((frame_height / 4) * 2);
		splitPane2.setDividerLocation((frame_height / 4) * 2);
		splitPane3.setDividerLocation((frame_height / 4) * 2);

		setSize(frame_width, frame_height);

		int x, y;
		x = (5 * screen.width / 7) - (window.width / 2);
		if (x + window.width > screen.width) {
			x = screen.width - window.width;
		}
		if (x < BORDER_WIDTH_) {
			x = BORDER_WIDTH_;
		}
		y = screen.height / 3 - (window.height / 2);
		if (y + window.height > screen.height) {
			y = screen.height - window.height;
		}

		if (y < BORDER_HEIGHT_) {
			y = BORDER_HEIGHT_;
		}
		tabPane_.addChangeListener(new TabChangeListener());
		setLocation(x, y);
		//buildMenu();
		setQueueItemsView(manQueueBrowser_);
		setVisible(true);
		//setMaskListenerType();
	}

	/**
	 * Updates the view
	 * 
	 * @param newQueue
	 *            specifies the new queue to be displayed
	 */
	public void setQueue(Queue newQueue) throws SQLException {
		super.updateView(newQueue);
		maskManSequence_.setQueue(queue_);
		maskInsertSequence_.setQueue(queue_);
		maskDiscardSequence_.setQueue(queue_);
		maskManSequence_.updateView(queue_);
		maskInsertSequence_.updateView(queue_);
		maskDiscardSequence_.updateView(queue_);
		manQueueBrowser_.updateView(queue_);
		discardQueueBrowser_.updateView(queue_);
		insertQueueBrowser_.updateView(queue_);

		setTitle(
		"Queue view: "
		+ queue_.getName()
		+ " ("
		 + queue_.getTelescope()
		 + ")");
	}

	/**
	 * Appends OBs to the view
	 * 
	 * @param obIds
	 *            specifies the id of obs to be added to the view
	 * 
	 * NOTE: should never add OBs to a queue that have a status of "-" or "D",
	 * this is however difficult to check here without loading the OB from the
	 * database (SLOW!) It is assumed that that check has already been made (at
	 * the Repository Browser level) and that all ids listed here can be safely
	 * added to the queue.
	 */
	public void checkForDuplicates(long[] obIds) {
		Vector setOBIDs = new Vector();
		int option = -1;
		StringBuffer messageBuffer = new StringBuffer();
		int numFailed = 0;
		for (int j = 0; j < obIds.length; j++) {
			if (queue_.inQueueItems(obIds[j])) {
				numFailed++;
				messageBuffer.append("\n");
				messageBuffer.append(Config.getCfg().uniqueToTableId(obIds[j]));
			}
		}

		if (numFailed == 1) {
			String message = "OB " + messageBuffer.toString()
					+ " already inserted into the Queue";
			JOptionPane.showMessageDialog(null, message, "Warning",
					JOptionPane.WARNING_MESSAGE);
		}
		if (numFailed > 1) {
			log_.clearTextArea();
			String message = "The following OBS are already inserted into the queue\n"
					+ messageBuffer.toString();
			log_.setTextArea(message);
			log_.setVisible(true);
		}
	}

	public void appendOB(long[] obIds) {
		 manQueueBrowser_.appendOBsWithoutMessge(obIds);
		 manQueueBrowser_.startQuery();
		 discardQueueBrowser_.appendOBsWithoutMessge(obIds);
		 discardQueueBrowser_.startQuery();
		 insertQueueBrowser_.appendOBsWithoutMessge(obIds);
		 insertQueueBrowser_.startQuery();
	}

	public void reQueryExecSequence() {
		 maskManSequence_.startQuery();
		 maskInsertSequence_.startQuery();
		 maskDiscardSequence_.startQuery();
	}

	public void reQueryView() {
		 manQueueBrowser_.startQuery();
		 discardQueueBrowser_.startQuery();
		 insertQueueBrowser_.startQuery();
		 maskManSequence_.startQuery();
		 maskInsertSequence_.startQuery();
		 maskDiscardSequence_.startQuery();
	}

	public int getTabPaneIndex() {
		return tabPane_.getSelectedIndex();
	}

	/**
	 * Listener for new TabbedPane selections. Fires an ObjectSelectionEvent
	 * when a different tabPane is selected.
	 * 
	 * @author cmavros 25-10-01 [1506] Add Reports to Execution Sequence
	 */
	public class TabChangeListener implements ChangeListener {

		/**
		 * Handler for TabbedPane selection events. Fires an
		 * ObjectSelectionEvent when a different tabPane is selected.
		 */
		public void stateChanged(ChangeEvent ev) {
			MaskListener mskLnr = MaskListener.getInstance();
			if (tabPane_.getSelectedIndex() == TAB_MAN) {
				setQueueItemsView(manQueueBrowser_);
			}
			if (tabPane_.getSelectedIndex() == TAB_INSERTED) {
				setQueueItemsView(insertQueueBrowser_);
			}
			if (tabPane_.getSelectedIndex() == TAB_DISCARDED) {
				setQueueItemsView(discardQueueBrowser_);
			}
			menuItems.setQueueItemsView(getQueueItemsView());
			menuItems.setContainer(getQueueItemsView().getView());
			menuItems.reinitOBsMenu();
			// TODO Reintroduce for Masktracker.
			menuItems.getOtMenu().reinitReportsMenu(getQueueItemsView().getView(),getQueueItemsView().getView().getJTable());
			menuItems.getOtMenu().reinitFCMenu(getQueueItemsView().getView(),getQueueItemsView().getView().getJTable());
			setMaskListenerType();
		}

	}

	/**
	 * @param mskLnr
	 */
	public void setMaskListenerType() {
		MaskListener mskLnr = MaskListener.getInstance();
		if (!MTViewManager.getInstance().isQueueSelected(queue_))
			return;
		if (tabPane_.getSelectedIndex() == TAB_MAN) {
			mskLnr.setType(MaskManufacturing.getInstance());

		}
		if (tabPane_.getSelectedIndex() == TAB_INSERTED) {
			mskLnr.setType(MaskInsertion.getInstance());
		}
		if (tabPane_.getSelectedIndex() == TAB_DISCARDED) {
			mskLnr.setType(MaskDiscarding.getInstance());
		}
	}

	public long[] getSelectedExecSequenceOBIds() {
		long[] obIds = {};
		if (tabPane_.getSelectedIndex() == TAB_MAN) {
			 obIds = maskManSequence_.getOtDbbView().getAllOBIds();
			 MaskManufacturing.getInstance().setSelectedNaidList(
			 maskManSequence_.getSelectedMaskNaids());
		}
		if (tabPane_.getSelectedIndex() == TAB_INSERTED) {
			 obIds = maskInsertSequence_.getOtDbbView().getAllOBIds();;

		}
		if (tabPane_.getSelectedIndex() == TAB_DISCARDED) {
			 obIds = maskDiscardSequence_.getOtDbbView().getAllOBIds();
			 MaskDiscarding.getInstance().setSelectedMaskList(
			 maskDiscardSequence_.getSelectedMaskIds());
		}
		return obIds;
	}



	/**
	 * Builds the file menu
	 */
	public void buildFileMenu() {
//		 reloadQueueMenuItem =
//		 fileMenu.add(new ReloadQueueAction(this, "Revert"));
//		 reloadQueueMenuItem.setAccelerator(
//		 KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
//		 closeQueueMenuItem = fileMenu.add(new CloseQueueAction(this,
//		 "Close"));
//		 fileMenu.addSeparator();
//		 saveQueueMenuItem = fileMenu.add(new SaveQueueAction(this, "Save"));
//		 saveAsQueueMenuItem =
//		 fileMenu.add(
//		 new ExportBOAction(
//		 Media.DBASE,
//		 queueItemsView_.getView(),
//		 "Export...",
//		 true,
//		 true));
//		 fileMenu.addSeparator();
//		 maskQueueViewMenuItem = fileMenu.add(new MaskQueueBrowserAction());
//		 fileMenu.add(maskQueueViewMenuItem);
//		 closeQueueMenuItem.setAccelerator(
//		 KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK));
//		 closeQueueMenuItem.setMnemonic(KeyEvent.VK_W + KeyEvent.CTRL_MASK);
//		
//		 saveQueueMenuItem.setAccelerator(
//		 KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
//		 saveQueueMenuItem.setMnemonic(KeyEvent.VK_S + KeyEvent.CTRL_MASK);
	}

	private class MyTabbedPaneUI extends BasicTabbedPaneUI {
		// default color is decided by the Default UI.
		Color selectedTabBgColor = UIManager
				.getColor("TabbedPane.lightHighlight");

		Insets contentBorderInsets = new Insets(1, 0, 0, 0);

		public MyTabbedPaneUI(Color color) {
			selectedTabBgColor = color;
		}

		public void setSelectedTabBgColor(Color color) {
			selectedTabBgColor = color;
		}

		// Overriding method. defines the offset from the top of the
		// tabbed pane to the component it contains.
		protected Insets getContentBorderInsets(int tabPlacement) {
			return contentBorderInsets;
		}

		// Overriding method. paints the background of the tabs.
		protected void paintTabBackground(Graphics g, int tabPlacement,
				int tabIndex, int x, int y, int w, int h, boolean isSelected) {
			g.setColor(tabPane.getBackgroundAt(tabIndex));
			// these two lines are all that makes this method
			// different from the paintTabBackground() in
			// BasicTabbedPaneUI. we want to force the background
			// color of the tab to be selectedTabBgColor, not
			// the default one from the tabbed pane. (light gray)
			if (isSelected)
				g.setColor(selectedTabBgColor);

			switch (tabPlacement) {
			case LEFT:
				g.fillRect(x + 1, y + 1, w - 2, h - 3);
				break;
			case RIGHT:
				g.fillRect(x, y + 1, w - 2, h - 3);
				break;
			case BOTTOM:
				g.fillRect(x + 1, y, w - 3, h - 1);
				break;
			case TOP:
			default:
				g.fillRect(x + 1, y + 1, w - 3, h - 1);
			}
		}
	}

}
