package org.eso.ohs.phase2.apps.masktracker;

import java.awt.GridLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.eso.ohs.core.docview.gui.DocView;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.Mask;


/**
 * This is a specialised JPanel with subcomponents linked to the
 * given Mask object's fields.  When this component is
 * displayed, changes to it will be reflected in the Mask
 * object.  Likewise changes to the Mask object will be
 * reflected in the displayed fields.  Some fields of an Mask
 * may not be visible.  Some fields may not be modifiable.
 *
 * <p>
 * $Id: MaskView.java,v 1.15 2007/10/11 11:43:12 pnunes Exp $
 */

public class MaskView extends JPanel implements PropertyChangeListener {


    /** Currently displayed Mask, or null */
    private Mask currentMask_;

   
    /* GUI component for the MaskNAID field */
    private DocView fieldMaskNAID_;

    /* GUI component for the MaskId field */
    private DocView fieldMaskId_;


  /* GUI component for the MaskId field */
    private DocView fieldMaskName_;


    /* GUI component for the SlotNum field */
    private DocView fieldSlotNum_;


    private DocView fieldMaskStatus_;

    /* Button to allow the acquisition template to be edited */
    private JButton createButton_;

    /** Button to allow the user to update the object on external media */
    private JButton saveButton_;


    /**
     * Construct the JPanel and add the subcomponents and listeners, etc.
     * Some fields of an Mask may not be visible.
     * Some fields may not be modifiable.
     *
     * <p> Initially, no OB is displayed.
     */
    public MaskView() {
        JTextField tf;
        DocView dView;
        JLabel lab;

        setLayout(new GridLayout(1,0));

        JPanel col1Left  = new JPanel(new GridLayout(0,1));
        JPanel col1Right = new JPanel(new GridLayout(0,1));

        JPanel col2Left  = new JPanel(new GridLayout(0,1));
        JPanel col2Right = new JPanel(new GridLayout(0,1));

	    add(col1Left);
        add(col1Right);

        add(col2Left);
        add(col2Right);

        lab = new JLabel("                        Mask NAID : ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        col1Left.add(lab);

        tf = new JTextField();
        tf.setOpaque(false);
        //tf.setEditable(false);
        fieldMaskNAID_ = new DocView(tf);
        col1Right.add(fieldMaskNAID_);

        lab = new JLabel("                        Mask Name : ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        col1Left.add(lab);

        tf = new JTextField();
        tf.setOpaque(false);
        //tf.setEditable(false);
        fieldMaskName_ = new DocView(tf);
        col1Right.add(fieldMaskName_);



        lab = new JLabel("Mask Status: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        col1Left.add(lab);

        JComboBox cb = new JComboBox(Mask.getStatusValues().keys());
        cb.setOpaque(false);
        fieldMaskStatus_ = new DocView(cb);
        col1Right.add(fieldMaskStatus_);



        lab = new JLabel("Mask Id: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        col2Left.add(lab);

        tf = new JTextField();
        tf.setOpaque(false);
        fieldMaskId_ = new DocView(tf);
        col2Right.add(fieldMaskId_);


        lab = new JLabel("Slot number: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        col2Left.add(lab);

        tf = new JTextField();
        tf.setOpaque(false);
        fieldSlotNum_ = new DocView(tf);
        col2Right.add(fieldSlotNum_);

        setObject((Mask)null);
    }


    /**
     * Construct the JPanel and add the subcomponents and listeners, etc.
     * Some fields of an Mask may not be visible.
     * Some fields may not be modifiable.
     *
     * @param ob the Mask object to link to.
     */
    public MaskView(Mask ob) {
	this();
	setObject(ob);
    }

    /**
     * Returns a reference to the currently displayed OB.
     *
     * @return reference to an Mask.
     */
    public BusinessObject getObject() {
	return currentMask_;
    }

    /**
     * Sets a new value of the current OB.
     * This method is included to satisfy the ObjectView
     * interface, which in turn allows for generalisation of
     * other parts of the system.
     *
     * @param newValue reference to a BusinessObject.
     */
    public void setObject(BusinessObject newValue) {
	setObject((Mask)newValue);
    }


    /**
     * Enable or disable the fields of this panel and any
     * nested panels.  The "locked" icon is displayed
     * next to the Name field if the fields are locked.
     * 
     * @param enable	True if the fields are to be enabled,
     *			false otherwise.
     */
    public void setEnabled(boolean enable) {
	super.setEnabled(enable);

	fieldMaskNAID_.setEnabled(enable);
	fieldMaskId_.setEnabled(enable);
        fieldSlotNum_.setEnabled(enable);

    }


    /**
     * Handle a change in the property value for the associated
     * BusinessObject.  Enable or disable changes to the fields,
     * according to whether they are allowed to be modified.
     * Update nested ObjectViews.
     * 
     * @param ev    Property Change Event
     */
    public void propertyChange(PropertyChangeEvent ev) {
		    // XXX Should never happen but...
	if (currentMask_ == null) {
	    System.out.println("@@ ??? Property change for 'null' OB??: "
			    + ev.getSource());
	    return;
	}

	// XXX FINISH

    }


    /**
     * Updates the DocView components to refer to the correct OB.
     * 
     * @param newMsk	    Mask object to be displayed.
     * Creates a component with subcomponents linked to the
     * Mask object's fields.  When the returned component is
     * displayed, changes to it will be reflected in the Mask
     * object.  Som6e fields of an Mask may not be visible.
     * Some fields may not be modifiable.
     *
     * @param ob the Mask object to link to.
     *
     */
    public void setObject(Mask newMsk) {
	if (newMsk == currentMask_) {
	        return;
	}

	if (currentMask_ != null) {
	    currentMask_.removePropertyChangeListener(this);
	}      
	currentMask_ = newMsk;
    if(currentMask_ != null) {
        fieldMaskNAID_.setObjectProperty(
                                         currentMask_, "MaskNAID"
                                         );

        fieldMaskId_.setObjectProperty(
                                       currentMask_, "MaskId"
                                       );

        fieldSlotNum_.setObjectProperty(
                                        currentMask_, "SlotNum"
                                        );
        fieldMaskStatus_.setObjectProperty(
                                           currentMask_, "Status"
                                           );

        fieldMaskName_.setObjectProperty(currentMask_, "Name"
                                           );
    }
    }

    public static void main(String[] args) {
	JFrame frm = new JFrame("Mask Test");

	MaskView mv = new MaskView();
	frm.setContentPane(mv);

	mv.setObject(new Mask());

	frm.setBounds(300, 400, 150, 600);
	frm.setVisible(true);
    }

}


