package org.eso.ohs.phase2.apps.masktracker;

import java.text.SimpleDateFormat;
import java.util.Hashtable;

import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.persistence.OHSApplication;


/**
 *
 * This class generates the Report Headers for thr
 * Mask Insertion,Deletion,Discarding Reports
 */

public class ReportHeaders {

    public static Hashtable<Integer,String> orderList = new Hashtable<Integer,String>();


    /** Mask manufacturing order index */


    private static final int MASK_MANUFACTURING_ORDER = 0;

    /** Mask Insertion order index*/
    private static final int MASK_INSERTION_ORDER = 1;
    
    /** Mask Deletion order index*/
    private static final int MASK_DELETION_ORDER = 2; 


    /** Hashtable of Strings and indexes */

    static {
        orderList.put(new Integer(MASK_MANUFACTURING_ORDER),".mmr");
        orderList.put(new Integer(MASK_INSERTION_ORDER),".mio");
        orderList.put(new Integer(MASK_DELETION_ORDER),".mdo");
    }

   /** Generates Discarding order header*/   

   public static String generateDiscardingOrderHeader() {
         return generateOrderHeader(MASK_DELETION_ORDER);        
   }


   /** Generates Insetrtion order header*/

   public static String generateInsertionOrderHeader(){
         return generateOrderHeader(MASK_INSERTION_ORDER);
   }

   /** Generates Manufacturing order header*/

   public static String generateManufacturingOrderHeader() {
         return generateOrderHeader(MASK_MANUFACTURING_ORDER);
   }
    

    /** 
     * Generate order header giving a specific index 0
     * for Manufacturing
     */

   private static  String generateOrderHeader(int order) {
        String ext = (String)orderList.get(new Integer(order));
        SimpleDateFormat isoTimeFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
        String name = "BOB"; 
        // String name = isoTimeFormat.format(new java.util.Date()) + ext;
        StringBuffer paf = new StringBuffer();
	paf.append("PAF.HDR.START;\n");
        paf.append("PAF.TYPE		\"OB Description\";\n");        
	paf.append("PAF.ID		\"");
	paf.append(name);
	paf.append("\";\n");
	paf.append("PAF.NAME		\"");
	paf.append(name);
	paf.append("\";\n");
        paf.append("PAF.DESC		\"\";\n");
	paf.append("PAF.CRTE.NAME	\"\";\n");
	paf.append("PAF.CRTE.DAYTIM	\"\";\n");
	paf.append("PAF.LCHG.NAME	\"\";\n");
	paf.append("PAF.LCHG.DAYTIM	\"\";\n");
	paf.append("PAF.CHCK.NAME	\"\";\n");
        paf.append("PAF.CHCK.DAYTIM	\"\";\n");
	paf.append("PAF.CHCK.CHECKSUM	\"\";\n");
	paf.append("PAF.HDR.END		\"\";\n");
	paf.append("\n");
        return paf.toString();
    } 

    /**
     * Generate OBD Header
     */

    public static String generateObdHeader() {
	String text;
        String dateTime = TextUtils.currentISODateTime();

	text = "PAF.HDR.START;                             # Marks start of header"
	+ "\nPAF.TYPE           \"OB Description\"; # Type of param file"
	+ "\nPAF.ID             \"\""
	+ "\nPAF.NAME           \"\""
	+ "\nPAF.DESC           \"\""
	+ "\nPAF.CRTE.NAME      \"" + 
	    OHSApplication.getVersion() + 
	    "\";  # Creator application"
	+ "\nPAF.CRTE.DAYTIM    \"" + dateTime + "\";  	# Creation date"
	+ "\nPAF.LCHG.NAME      \"\";                   # Who did last change"
	+ "\nPAF.LCHG.DAYTIM    \"" + dateTime + "\";  	# Time of last change"
	+ "\nPAF.CHCK.NAME      \"\";                   # Applic checking file"
	+ "\nPAF.CHCK.DAYTIM    \"\";                   # Date of last check"
	+ "\nPAF.CHCK.CHECKSUM  \"\";                   # Param file checksum"
	+ "\nPAF.HDR.END"
	+ "\n" 
	+ "\n";
	return text;
    }



   /**
     * Returns the body of an OBD format representation of this object.
     * 
     * @return an OBD body
     */
    public static String getObdBody(String obName) {

      //String obName = "Mask Insertion OB";       
	String text =  "# Observation Block description follows\n";

	text +=  "\n";

	long id = 0;

	text += "\nOBS.ID\t\"" + id + "\"\t# Observation block Id";

	text += "\nOBS.NAME\t\"" + obName + "\"\t# Observation block Name";	
	text += "\nOBS.GRP	      \"0\"		# Unused";
	text += "\nOBS.PROG.ID    \"" + "UNKOWN" + "\"";
	text += "\nOBS.PI-COI.ID  \"" + "UNKOWN" + "\"";
	text += "\nOBS.PI-COI.NAME  \"" + "UNKNOWN" + "\"";

	text += "\nOBS.OBSERVER  \"" + "UNKNOWN" + "\"";
        text += "\n";
	return text;
    }

    /**
     * Generate dummy mask template
     * @param templateName Name of template to generate header for
     */

    public static String generateMaskTemplate(String templateName) {
        StringBuffer sb = new StringBuffer();
     
	sb.append("# " + templateName);
	sb.append("\n");
	sb.append("TPL.ID      \"" + templateName + "\"");
	sb.append("\n");
	sb.append("TPL.NAME    \"" + templateName + "\"");
	sb.append("\n");
	sb.append("TPL.MODE    \"spectroscopy\"");
	sb.append("\n");
        sb.append("\n");      
        return sb.toString();
    }

}
