package org.eso.ohs.phase2.apps.masktracker.actions;

import java.awt.event.ActionEvent;
import java.sql.SQLException;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.phase2.apps.masktracker.MaskDiscarding;

/**
 * Action class to display the selected ObservationBlock in PAF form.
 *
 * <p>
 * $Id: DiscardingReportAction.java,v 1.13 2006/12/19 00:51:49 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * @author cmavros 1501/OT Change to allow user to select more than
 *    one OB.
 */

public class DiscardingReportAction extends ActionSuperclass {

    private ObjectContainer objectContainer_;
    private boolean  report_;

    
    private static TextDisplayWidget log_ = new TextDisplayWidget();
    
    
    private JComponent component;
  
    public DiscardingReportAction(ObjectContainer objectContainer, 
             JComponent component,String name,boolean report) {
        super(component, name);
        objectContainer_ = objectContainer;
        this.component = component;
        report_ = report;
    }

    /**
     * Event handler for the ShowPAFDescrAction action.
     * 
     * @param ev       user-initiated ActionEvent to quit.
     */
    public void actionPerformedImpl(ActionEvent evt) {
        Long [] idLongs = null;
        idLongs = objectContainer_.getAllSelected();
        idLongs = (Long[])ObjUtils.uniqueOccurrences(idLongs);
        if(idLongs == null || idLongs.length <= 0) {
            JOptionPane.showMessageDialog(
                                          objectContainer_.getTopLevelAncestor(),
                                          new Object[] {
                                              "There are no selected business objects",
                                              " ",
                                              "You must first select business objects to report on."
                                          },
                                          "Nothing selected",
                                          JOptionPane.ERROR_MESSAGE
                                          );
            return;
        }
        generateReports(idLongs);
    }
    
    
    private void generateReports(Long [] idLongs) {
        log_.setVisible(false);
        log_.clearTextArea();
        try {
            String reportStr = null;
            long [] ids = new long [idLongs.length];
            for(int i=0;i<ids.length;i++) {
                ids[i] = idLongs[i].longValue();
            }
            if(report_)
                 reportStr = MaskDiscarding.getInstance().getDiscardingText(ids);
            else
                 reportStr = MaskDiscarding.getInstance().getDiscardingReport(ids);              
            log_.setTextArea(reportStr);
            log_.setVisible(true);
        }
        catch(SQLException e) {
        	ErrorMessages.sqlError(e,component);
        }

    }
}



 
