package org.eso.ohs.phase2.apps.masktracker.actions;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.masktracker.MaskManufacturing;

/**
 * Action class for Manufacturing Report
 *
 * <p>
 * $Id: ManufacturingReportAction.java,v 1.16 2007/10/11 11:43:19 pnunes Exp $
 * <br>
 * @author Bob Kemp
 */

public class ManufacturingReportAction extends ActionSuperclass {

    private ObjectContainer objectContainer_;

    protected static TextDisplayWidget log_ = new TextDisplayWidget(); 
  
    /**
     * Constructs a ShowPAFDescrAction action.
     */
    public ManufacturingReportAction(ObjectContainer objectContainer,JComponent table 
             ,String name) {
        super(table, name);
        objectContainer_ = objectContainer;
    }

    /**
     * Event handler for the ShowPAFDescrAction action.
     * 
     * @param ev       user-initiated ActionEvent to quit.
     */
    public void actionPerformedImpl(ActionEvent evt) {
        Long [] idLongs = null;
        idLongs = objectContainer_.getAll();
        idLongs = (Long[])ObjUtils.uniqueOccurrences(idLongs);

        if(idLongs == null || idLongs.length <= 0) {
            JOptionPane.showMessageDialog(
                                          objectContainer_.getTopLevelAncestor(),
                                          new Object[] {
                                              "There are no selected business objects",
                                              " ",
                                              "You must first select business objects to report on."
                                          },
                                          "Nothing selected",
                                          JOptionPane.ERROR_MESSAGE
                                          );
            return;
        }
        new IncrementalActionAutomaton(
                                       new IncrementalReport( idLongs),
                                       (JFrame)objectContainer_.getTopLevelAncestor(),"Reports...");
    }

    public class IncrementalReport implements IncrementalAction {

        /** Array of ids of objects to be duplicated */
        private Long[] idLongs_;
        StringBuffer buf = new StringBuffer();

        Vector<CalibrationBlock> cbList = new Vector<CalibrationBlock>();

        /**
         * Constructor.
       	 *
         * @param objType   type of object to be duplicated
       	 */
        public IncrementalReport(Long[] idLongs) {
            idLongs_  = idLongs;
        }

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(idLongs_,counter);
        }

        public void startOfAction() {
            log_.setVisible(false);
            log_.clearTextArea();
        }

        public void endOfAction() {
            try {
                StringBuffer buf = new StringBuffer();
                CalibrationBlock [] cbs = new CalibrationBlock[cbList.size()];
                cbList.copyInto(cbs);
                String report = MaskManufacturing.getInstance().getNaidsForMasks(cbs);
                if(report.length() == 0)
                    buf.append("No masks will be manufactured for the obs selected");
                else {
                    buf.append(report);
                }
                log_.setTextArea(buf.toString());
                log_.setVisible(true);
            }
            catch(IOException e) {
            	ErrorMessages.announceIOError(objectContainer_.getTopLevelAncestor(),e);
            }
        }


        /**
         * Method to duplicate the n'th element of obs, where n = counter.
       	 *
         * @param counter counter to show which element of obs to check in.
         * @return true if the duplicating should continue
         *   and false otherwise.
       	 */
        public void performStep(int counter) {

            CalibrationBlock cb = null;
            if (idLongs_[counter] != null) {
                try {
                    ObjectManager objMgr = ObjectManager.getObjectManager();
                    Class objType = Config.getCfg().getClassFromId(idLongs_[counter].longValue());
                    cb = (CalibrationBlock)objMgr.getBusObj(Media.DBASE, idLongs_[counter].longValue(), objType);
                }
                catch(ObjectIOException e) {
                	ErrorMessages.announceIOError(objectContainer_.getTopLevelAncestor(),e);
                }
                catch(ObjectNotFoundException e) {
                	ErrorMessages.announceNoObject(objectContainer_.getTopLevelAncestor(),e);
                }

                cbList.addElement(cb);
            }
         }
    }

 
   
}



 
