package org.eso.ohs.phase2.apps.ot.actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.OverwriteOptionPane;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.ot.gui.OTViewManager;
import org.eso.ohs.phase2.apps.ot.gui.QueueContainer;
import org.eso.ohs.phase2.apps.ot.gui.QueueItemsBView;
import org.eso.ohs.phase2.apps.ot.gui.QueueView;

/**
 * Action class to create a subdirectory of the selected directory.
 *
 * <p>
 * $Id: DumpQueueAction.java,v 1.20 2006/01/31 11:53:38 ddorigo Exp $
 * <br>
 * Author: Bob Kemp
 */
public class DumpQueueAction extends ActionSuperclass {

  
  //private JTable queueTable_;

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( DumpQueueAction.class );

    private QueueContainer otMainView_;




    private static SwingFileChooser fileChooser_ = new SwingFileChooser();
    static {         
         File f1 = new File(AppConfig.getAppConfig().getDumpDir());
         fileChooser_.setCurrentDirectory(new File(f1.getParent()));
         fileChooser_.setSelectedFile(f1);
    }

    public DumpQueueAction(QueueContainer otMainView, String name) {
	super((JFrame)otMainView,name);
	// stdlog_.debug("Creating Dump Action");
        otMainView_ = otMainView;
        
    }
  
   

    public void actionPerformedImpl(ActionEvent ev) {
	stdlog_.debug("!!!!!!!!!!!!!!! Dump Action Event " + ev);
        ObjectManager objMgr = ObjectManager.getObjectManager(); 
        String fileName = "";
        long [] queueIds = otMainView_.getSelectedOpenQueueIds();       
        fileChooser_.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY); 
        int returnVal = fileChooser_.showOpenDialog(null); 
        int returnValueOpt = -1;
	stdlog_.debug("Ids " + Config.getCfg().tableToUniqueId(queueIds[0],Queue.class));
        QueueView manager = OTViewManager.getOTViewMgr().getQueueView(queueIds[0]);
        QueueItemsBView view = manager.getQueueItemsView();
        long [] obIds = view.getView().getAllOBIds();
        

        
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            File f = fileChooser_.getSelectedFile();    
            if(!f.canWrite()) {
                 JOptionPane.showMessageDialog(
	         null,
                 new Object[] {
	         "No write permissions to dump queue to this directory"			       
	         },		           
                 "No write permisssions",		        
	         JOptionPane.ERROR_MESSAGE		       
	         );
                 return;                  
            }
	    String path = fileChooser_.getSelectedFile().getAbsolutePath() +
	              File.separator;
            int result = JOptionPane.YES_OPTION;
	    switch (result) {
	        case JOptionPane.YES_OPTION:	   
	            new IncrementalActionAutomaton(
                        new IncrementalDumpQueue(obIds,path),
		        otMainView_.getParentFrame(),
                       "Dumping...."
		    );
	        default:return;
	   }
	}                             
    }




 /**
     * Utility class to duplicate a sequence of OBs, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalDumpQueue implements IncrementalAction {


	/** array of ids of objects to be duplicated */
	private long [] obIds_;
  
        private int objCount_;

        private String path_;

        int returnValueOpt_ = -1;

        private ObjectManager objMgr_;


	/**
	 * Constructor.
	 * 
	 * @param frm	    window on which to base any dialog boxes.
	 * @param device    media containing the objects to be duplicated
	 * @param deepCopy  should we duplicate attached objects?
	 * @param dir	    directory containing the ObsBlocks
	 * @param objType   type of object to be duplicated
	 * @param idLongs   array of ids of objects to be duplicated
	 */
	public IncrementalDumpQueue(long [] obIds,String path) {	   
	    obIds_  = obIds;    
	    objCount_ = obIds_.length;
            path_ = path;
            objMgr_ = ObjectManager.getObjectManager();
	    	       
	}

       /**
        * Number of steps in the action that are still remaining to be done.
        * 
        * @return number of steps remaining (eg objects to acted upon)
        */
        public int getNumStepsRemaining(int counter) {
            Long [] obIds = new Long[obIds_.length];
            for(int i=0;i<obIds_.length;i++)
		obIds[i] = new Long(obIds_[i]);
            return ObjUtils.howManyNotNull(obIds, counter);          
        }

        /**
         * Method to perform one incremental step of a larger action.
         * 
         * @param stepNo	Counter to indicate the n'th step of a larger action
         */
        public void performStep(int counter) {
            try {           
	         doDump(obIds_[counter]); 
	    }
            catch(ObjectNotFoundException e1) {
	    }
            catch(ObjectIOException e2) {
	    }
            catch(IOException e3) {
	    }                 
        }

        /**
         * Hook for initialisation of action object.
         */
        public void startOfAction() {
        }

        /**
         * Hook for cleanup of action object.
         */
        public void endOfAction() {
        }


	/**
	 * Utility method to duplicate a given business object
	 * 
	 * @param id	    id of business object to duplicate
	 */
	public void doDump(long obId) throws ObjectIOException,ObjectNotFoundException,IOException {
            stdlog_.debug("@@@@@@@@@@@@@@ OB ID " + obId);
            Class c = Config.getCfg().getClassFromId(obId); 
            stdlog_.debug("Class = " + c);
	    CalibrationBlock cb= 
	    (CalibrationBlock)objMgr_.getBusObj(Media.DBASE,
					         obId,
					         c);	    	 
	    String fileName =  obId+cb.getName().replace(' ','-') + ".obd" ;
            File file = new File(path_ + fileName);
            if(file.exists() && returnValueOpt_ != OverwriteOptionPane.YES_TO_ALL_OPTION) {
                     OverwriteOptionPane pane = new OverwriteOptionPane();
                     returnValueOpt_ = OverwriteOptionPane.showOptionDialog(null,"Do you wish to overwrite previously dumped file "
                                            + fileName + " ?",
                                            "Overwrite files");
	    }
            if(!file.exists() || returnValueOpt_ == OverwriteOptionPane.YES_OPTION 
                     || returnValueOpt_ == OverwriteOptionPane.YES_TO_ALL_OPTION) {
	        stdlog_.debug("file " + path_ + fileName );
	        FileWriter fw = new FileWriter(path_+fileName);
	        fw.write(cb.toOBD(true,objMgr_.getDbaseId(cb),OHSApplication.getShortDesc()));
	        fw.close();
	    }                             	   
	}
    }
}

