package org.eso.ohs.phase2.apps.ot.actions;

import java.awt.Dimension;
import java.awt.event.ActionEvent;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.phase2.apps.ot.wizard.OrangEngine;
import org.eso.ohs.phase2.orang.OBScheduleInfo;

/**
 * Display the explanation string for the ranking score in a top level dialog.
 * 
 * @version $Revision: 1.2 $
 * @author amchavan, Mar 1, 2006
 */

// $Id: JustifyRankingAction.java,v 1.2 2006/03/13 11:48:07 tcanavan Exp $

public class JustifyRankingAction extends ActionSuperclass {

    /** Where we get our OBs from */
    private ObjectContainer container;

    /** Where we get our OB information from */
    private OrangEngine engine;

    /** Flag: If <code>true</code>, consider all OBs,
     *        otherwise only the selected OBs */
    protected boolean all;

    /** Where we display the justification text */
    private static TextDisplayWidget displayWidget = null;

    /**
     * @param container  The reference container: used to place error
     *                   and info messages.
     * @param comp       Component in front of which dialog boxes will appear
     * @param all        If <code>true</code>, consider all OBs,
     *                   otherwise only the selected OBs
     * @param name       Name of action, for menu item
     * @param engine     Where we get our OB info from
     */
    public JustifyRankingAction( ObjectContainer folderView, 
                                 JComponent comp,
                                 boolean folder, 
                                 String name, 
                                 OrangEngine engine ) {
        super( comp, name );
        this.container = folderView;
        this.all = folder;
        this.engine = engine;
    }

    /**
     * Event handler: does all the work
     * 
     * @param evt  Ignored
     */
    public void actionPerformedImpl( ActionEvent evt ) {
        Long[] idLongs = null;
        
        if( all ) {
            idLongs = container.getAll();
        }
        else {
            idLongs = container.getAllSelected();
        }

        idLongs = (Long[]) ObjUtils.uniqueOccurrences( idLongs );

        if( idLongs == null || idLongs.length <= 0 ) {
            JOptionPane.showMessageDialog(
                    container.getTopLevelAncestor(),
                    new Object[] { "There are no selected OBs",
                                   "",
                                   "You must first select business objects to report on." },
                                   "Nothing selected", 
                                   JOptionPane.ERROR_MESSAGE );
            return;
        }
        
        // Everything is OK, now let's do it
        perform( idLongs );
    }

    /** Build and display the justification report */
    private void perform( Long[] obids ) {
        ObjectManager objMgr = ObjectManager.getObjectManager();

        if( displayWidget == null ) { 
            displayWidget = new TextDisplayWidget();
        }

        
        //displayWidget.setVisible( false );
        displayWidget.clearTextArea();

        StringBuffer report = new StringBuffer();
        OBScheduleInfo[] infobs = engine.getObRanked();

        for( int i = 0; i < obids.length; i++ ) {
            long id = obids[i].longValue() / 100;
            
            // Look for an OBScheduleInfo instance matching our ID
            for( int j = 0; j < infobs.length; j++ ) {
                OBScheduleInfo infob = infobs[j];
                long infoid = infob.getID();
                if( infob.getID() != id )
                    continue;
                
                String justification = infob.getRankingDesc();
                report.append( id )
                      .append( ":\n" )
                      .append( justification )
                      .append( "--------------\n\n" );
            }
        }
        
        if( (report.toString()).length() > 0 ) {
            displayWidget.setTextArea( report.toString() );
            displayWidget.setSize( new Dimension( 600, 400 ) );
            displayWidget.setVisible( true );
        }
        else {
            JOptionPane.showMessageDialog( container.getTopLevelAncestor(),
                            new Object[] { "Internal error, empty report", "", "" }, 
                            "Ranking justification",
                            JOptionPane.ERROR_MESSAGE );
        }
    }

	public OrangEngine getEngine() {
		return engine;
	}

	public void setEngine(OrangEngine engine) {
		this.engine = engine;
	}
}
