package org.eso.ohs.phase2.apps.ot.dbb;

import java.sql.SQLException;

import org.eso.ohs.core.dbb.server.DbbSession;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.server.DbbSqlEngineImpl;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.dbase.phase2.DbaseHandlerQueue;

/**
 * This class provides only static methods to build up a 
 * DB Repository browser.
 * @author Giorgio Passera <ESO\DMD> <gpassera@eso.org>
 */
public class BrowserConfig {

  // Tables
  public static DbbSqlTable ob;
  public static DbbSqlTable od;
  public static DbbSqlTable ts;
  public static DbbSqlTable tp;
  public static DbbSqlTable pa;
  public static DbbSqlTable ti;
  public static DbbSqlTable cs;
  public static DbbSqlTable sr;
  public static DbbSqlTable pm;
  public static DbbSqlTable sc;
  public static DbbSqlTable si; //schedule_items
  public static DbbSqlTable es; //execution_sequence;
  public static DbbSqlTable oes; //online_execution_sequence;

  public static DbbSqlTable si_tmp;
  public static DbbSqlTable es_tmp;
  public static DbbSqlTable cr_tmp; // storage of temporary config report.
  public static DbbSqlTable masks;
  public static DbbSqlTable or;
  public static DbbSqlTable accounts;
  public static DbbSqlTable profiles;
  
  public static DbbSqlTable obs_masks;
  public static DbbSqlTable finds_view;
  public static DbbSqlTable masks_view;
  public static DbbSqlTable standard_time_intervals_view;
  public static DbbSqlTable sidereal_time_intervals_view;
  public static DbbSqlTable baselines;
  public static DbbSqlTable readme;
  public static DbbSqlTable readme_status_history;
  public static DbbSqlTable ephemeris_file;


  

  
    /*
     * Table where the state of an Observing Run is kept.
     * @author amchavan, 24-Oct-2001 
     * @author amchavan, 21-Nov-2001: commented out again, see 0704
     
    public static DbbSqlTable run_states_; 
     */

    /*
     * List of the possible states of an Observing Run, as
     * specified by 
     * <A HREF="http://www.eso.org/~usg/STATUS/README.RUNSTATES">the 
     * OR lifecycle document</A> (which will possibily change).
     * @author amchavan, 24-Oct-2001
     * @author amchavan, 21-Nov-2001: commented out again, see 0704

    public static final String[] run_state_values_ = {
    "SUBMITTED",
    "FAULT",
    "READY",
    "HOLD",
    "STARTED",
    "NOT_STARTED",
    "COMPLETED/VERIFY",
    "COMPLETED",
    "COMPLETED/PRE",
    "TERMINATED/EA",
    "TERMINATED/UR",
    "TERMINATED/EP",
    "TERMINATED/UA",
    "TERMINATED/PO",
    "TOO",
    "OPEN"
    };
     */

  protected static String obrepDbName_ = Config.getCfg().getDbName(
                                       ObservationBlock.getSuffix());


  private static  String opcDbName_ = Config.getCfg().getDbName(ObservingRun.getSuffix());
public static DbbSqlTable masks_table;
public static DbbSqlTable obsblk_masks_table;

    /*
     * See <A HREF="http://www.eso.org/~usg/OPS/OPSDOCS/Database_phase2.txt">Database_phase2.txt</A> for a description of this database.
     *
     * @author amchavan, 24-Oct-2001: added 
     * @author amchavan, 21-Nov-2001: commented out again, see 0704
    private static  String phase2DbName_ = "phase2";
    */

 

  /**
   * Build the sql engine and return its reference
   * @return refence to the SqlEngine for building query
   * @exception SQLException JDBC exception on opening a session and 
   * creating the engine 
   */
  public static DbbSqlEngine getEngine()
    throws SQLException {
     
      DbbSession session = 
      new DbbSession(Config.getCfg().getUrl(ObservationBlock.getSuffix()),
                      obrepDbName_,
                      Config.getCfg().getDbUserName(),
                      Config.getCfg().getDbPassword());


      /* Create the server engine without building the key hash table
       * from the database metadata. We assume that the information
       * provided by this end application is sufficient to build the sql
       * join clauses.  
       * C.Boarotto 16-Jan-2000
       */
      DbbSqlEngineImpl engine = new DbbSqlEngineImpl(session,
                             false);

      generateTables();     
      buildDefaultJoins(engine);
        
      return engine;
  }


  /**
   * Build the application table objects.
   */
  public static void generateTables() {
   
    ob = new DbbSqlTable(obrepDbName_, "dbo", "obs_blocks");
    od = new DbbSqlTable(obrepDbName_, "dbo", "obs_descriptions");
    ts = new DbbSqlTable(obrepDbName_, "dbo", "template_signatures");
    tp = new DbbSqlTable(obrepDbName_, "dbo", "target_packages");
    pa = new DbbSqlTable(obrepDbName_, "dbo", "parameters");
    ti = new DbbSqlTable(obrepDbName_, "dbo", "time_intervals");
    cs = new DbbSqlTable(obrepDbName_, "dbo", "constraint_sets");
    pm = new DbbSqlTable(obrepDbName_, "dbo", "parameters");
    readme = new DbbSqlTable(obrepDbName_, "dbo", "readme");
    ephemeris_file = new DbbSqlTable(obrepDbName_, "dbo", "ephemeris_file");

    sc = new DbbSqlTable(obrepDbName_, "dbo", "schedules");
    oes = new DbbSqlTable(obrepDbName_, "dbo", "online_execution_sequence");

    si = new DbbSqlTable(obrepDbName_, "dbo", "schedule_items");
    es = new DbbSqlTable(obrepDbName_, "dbo", "exec_sequences");
    or = new DbbSqlTable(opcDbName_, "dbo", "obs_runs");
    accounts = new DbbSqlTable(opcDbName_, "dbo", "accounts");
    profiles = new DbbSqlTable(opcDbName_,"dbo","profiles");

    // @author amchavan, 24-Oct-2001: added
    // @author amchavan, 21-Nov-2001: commented out again, see 0704
    // run_states_ = new DbbSqlTable( phase2DbName_, "", "run_states" );
  
    //to be checked 
    si_tmp = new DbbSqlTable("tempdb","",DbaseHandlerQueue.TMP_SCHEDULE_ITEMS);
    es_tmp = new DbbSqlTable("tempdb","","#tmp_exec_sequences");  

    cr_tmp = new DbbSqlTable("tempdb","","#tmp_config_report");  
   
    sr = new DbbSqlTable(opcDbName_,"dbo","sched_rep");
    finds_view = new DbbSqlTable(obrepDbName_, "dbo", "finds_view");
    
    standard_time_intervals_view = new DbbSqlTable(obrepDbName_, "dbo", "standard_time_intervals");
    sidereal_time_intervals_view = new DbbSqlTable(obrepDbName_, "dbo", "sidereal_time_intervals");

    finds_view = new DbbSqlTable(obrepDbName_, "dbo", "finds_view");
    baselines = new DbbSqlTable(obrepDbName_, "dbo", "baselines");

    createMaskTables();
  }

private static void createMaskTables() {   
    masks = new DbbSqlTable(obrepDbName_, "dbo", "masks");
    obs_masks = new DbbSqlTable(obrepDbName_, "dbo", "obsblk_masks");
    masks_view = new DbbSqlTable(obrepDbName_, "dbo", "masks_view");
  
if(!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
      masks_table = BrowserConfig.masks_view;
      obsblk_masks_table = BrowserConfig.masks_view;
   } else {
      masks_table = BrowserConfig.masks;
      obsblk_masks_table = BrowserConfig.obs_masks;
   }   
}
  //----------------------------------------
  // Private Methods
  //----------------------------------------

  /**
   * Set up the end application information necessary to build the sql
   * join clauses within the sql engine.
   */
  protected static void buildDefaultJoins(DbbSqlEngineImpl engine) {

 
    engine.addJoinCondition(ob,"run_id",sr,"run_id",ob,sr);
    
    // add join condition between ob and readme table
    
    String [] runCols = {"run_id"};
    String [] readmeCols = {"run_id"};
    
    engine.addCompositeJoinCondition(ob,runCols,readme,readmeCols,true);
    
    String [] ephCols = {"ob_id"};
    
    engine.addCompositeJoinCondition(ob,ephCols,ephemeris_file,ephCols,true);

    // obs_blocks - phase I tables
    engine.addJoinCondition(ob, "run_id", or, "id", ob, or);
    //engine.addJoinCondition(ob, "user_id", eu, "id", ob, eu);

    // obs_blocks - obs_descriptions
    engine.addJoinCondition(ob, "ob_id", od, "od_id", ob, od);

    // obs_blocks - target_packages
    engine.addJoinCondition(ob, "tp_id", tp, "tp_id", ob, tp);

    // obs_blocks - constraint_sets
    engine.addJoinCondition(ob, "cs_id", cs, "cs_id", ob, cs);
    // constriants - baselines
    engine.addJoinCondition(cs, "baseline_id",baselines, "baseline_id",cs,baselines);


    // obs_blocks - time_intervals
    // @author amchavan, 09-Oct-2001: fixed
    engine.addJoinCondition(ob, "ob_id",standard_time_intervals_view, "ti_id", ob, standard_time_intervals_view);
        
    String [] fromCols = {"ob_id"};
    String [] toCols = {"ti_id"};
    engine.addCompositeJoinCondition(ob,fromCols,sidereal_time_intervals_view,toCols,true);

    // obs_descriptions - template_signatures
    engine.addJoinCondition(ts, "od_id", od, "od_id", ts, od);

    // obs_descriptions - parameters
    engine.addJoinCondition(pa, "od_id", od, "od_id", pa, od);

     //Giorgio 
    engine.addJoinCondition(si, "ob_id", ob, "ob_id", si, ob);
    engine.addJoinCondition(sc, "sch_id",si, "sch_id",sc, si);
    
    //engine.addJoinCondition(es, "ob_id", ob, "ob_id", es, ob);
    engine.addJoinCondition(sc, "sch_id",es, "sch_id",sc, es);
    
    engine.addJoinCondition(ob, "user_id", accounts, "account_id", ob, accounts);
    
    engine.addJoinCondition(accounts, "account_id", profiles, "account_id",accounts,profiles);
    
    //tmp tables
    engine.addJoinCondition(si_tmp, "ob_id", ob, "ob_id", si_tmp, ob);
    engine.addJoinCondition(sc, "sch_id",si_tmp, "sch_id",sc, si_tmp);    
  
    engine.addJoinCondition(oes, "ob_id",ob, "ob_id",oes, ob);    

    //engine.addJoinCondition(masks,"ob_id",ob,"ob_id",ob,masks);
   

    
    
                                    
                                   

    // @author amchavan, 24-Oct-2001: added
    // @author amchavan, 21-Nov-2001: commented out again, see 0704
    // engine.addJoinCondition( run_states_, "run_id", or, "id", run_states_, or );

     engine.addJoinCondition(es_tmp, "ob_id", ob, "ob_id", es_tmp, ob);
     
     String [] cols = {"ob_id"};
     engine.addCompositeJoinCondition(ob,cols,finds_view,cols,true);
     if(!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
       engine.addCompositeJoinCondition(ob,cols,
                                    masks_view,cols,true);
     } else {
       engine.addJoinCondition(obs_masks,"mask_id",masks,"mask_id",obs_masks,masks);
       engine.addJoinCondition(ob, "ob_id",
                                    obs_masks, "ob_id",ob,obs_masks);
     }

     
     
  }
}
