package org.eso.ohs.phase2.apps.ot.gui;

/**
 * The ConfigReportBrowser class is a DB search engine used for 
 * displaying a configuration report. The report is already generated
 * and placed into a temporary table. This browser is simply used as a
 * convenient display tool as well as allowing the user to append selected
 * OBs to queues in the OT.
 *
 * @version $Id: ConfigReportBrowser.java,v 1.14 2006/04/25 16:36:31 pnunes Exp $
 * @author Nick Kornweibel for ESO.
 * <br>ChangeLog: cmavros 22MAY02 [1723/OT] Add setEnabled() method.
 */

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbArrowButton;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.phase2.apps.ot.actions.DeselectAllAction;
import org.eso.ohs.phase2.apps.ot.actions.ExecutionSequenceAction;
import org.eso.ohs.phase2.apps.ot.dbb.BrowserConfig;
import org.eso.ohs.phase2.apps.ot.dbb.OTDbbView;

public class ConfigReportBrowser extends JPanel {

    protected int currentRow_;

    protected JToolBar toolBar_;

    protected String report_;

    protected Action deselectAllAction;

    JPanel panel_;

    JButton executeButton_;

    protected DbbSqlEngine engine_;

    private DbbView view_;

    DbbSqlTable cr_table;

    protected String title_;

    protected static TextDisplayWidget reportLog_ = null;

    protected TableListSelectionListener listListener_;

    protected static boolean configReportTempTableSetup = false;

    /**
     * Standard log channel
     */
    private static Logger stdlog_ = Logger.getLogger(ConfigReportBrowser.class);

    /**
     * Constructor
     */
    public ConfigReportBrowser(String title) {
        createConfigReportTable();
        try {
            boolean clearButton = false;
            boolean queryButton = true;
            boolean sortButton = false;

            engine_ = BrowserConfig.getEngine();
            title_ = title;

            buildDefaults();

            OTDbbView otdbbView = new OTDbbView();
            view_ = otdbbView.getDbbView();
            view_.hideSortButton();
            view_.hideResetButton();
            listListener_ = new TableListSelectionListener();
            view_.addListSelectionListener(listListener_);
            view_.getTable().removeSortListener();
            addDefaultColumns();
            buildSort();
            buildToolBar();

            executeButton_ = new JButton("Add Selection to Execution Sequence");

            view_.addSummaryButton(executeButton_);
            executeButton_.addActionListener(new ExecutionSequenceAction(otdbbView,otdbbView.getJTable(),
                    "Execution"));

            JButton textReportViewer = new JButton("View Text Report");
            textReportViewer.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    displayReportViewer();
                }
            });

            view_.addSummaryButton(textReportViewer);

            layoutBrowser();

        } catch (RemoteException e) {
            String message = "Error: " + e.getMessage();
            JOptionPane.showMessageDialog(null, message, "ERROR",
                    JOptionPane.ERROR_MESSAGE);
            stdlog_.error(e);
        } catch (MalformedURLException e) {
            String message = "Error: " + e.getMessage();
            JOptionPane.showMessageDialog(null, message, "ERROR",
                    JOptionPane.ERROR_MESSAGE);
            stdlog_.error(e);
        } catch (NotBoundException e) {
            String message = "Error: " + e.getMessage();
            JOptionPane.showMessageDialog(null, message, "ERROR",
                    JOptionPane.ERROR_MESSAGE);
            stdlog_.error(e);
        } catch (SQLException e) {
            String message = "Error: " + e.getMessage();
            JOptionPane.showMessageDialog(null, message, "ERROR",
                    JOptionPane.ERROR_MESSAGE);
            stdlog_.error(e);
        }
    }

    /**
     * Enable or disable the control buttons depending on the value of the
     * boolean parameter enable.
     * 
     * @param enable
     *            Boolean value to enable or disable the control buttons.
     */
    public void setEnabled(boolean enabled) {
        executeButton_.setEnabled(enabled);
    }

    /**
     * initialize and report the reportLog GUI display widget.
     */
    protected void displayReportViewer() {
        if (reportLog_ == null) {
            reportLog_ = new TextDisplayWidget("Configuration Report");
        }
        reportLog_.setVisible(false);
        reportLog_.clearTextArea();
        reportLog_.setTextArea(getReport());
        reportLog_.setSize(new Dimension(750, 900));
        reportLog_.setVisible(true);
    }

    /**
     * Set the report that is to be displayed when the report viewer button is
     * pressed.
     */
    public void setReport(String report) {
        report_ = report;
    }

    /**
     * @return the report that is displayed when the report viewer button is
     *         selected.
     */
    public String getReport() {
        return report_ == null ? "" : report_;
    }

    protected void removeListeners() {
        view_.removeListSelectionListener(listListener_);
        // QueueManager.removeOBListener(this);
    }

    public void buildDefaults() {
        cr_table = BrowserConfig.cr_tmp;
    }

    /**
     * Layout the browser - the Program Name is set to be right justified here -
     * I am not sure if this is the correct place to do this, but I do not know
     * where the correct place is.........
     */
    public void layoutBrowser() {
        JPanel tmpPanel = new JPanel(new BorderLayout());
        tmpPanel.add(toolBar_, BorderLayout.NORTH);
        tmpPanel.add(view_, BorderLayout.CENTER);
        setLayout(new BorderLayout());
        add(new JLabel(title_), BorderLayout.NORTH);
        add(tmpPanel, BorderLayout.CENTER);
    }

    public Action getDeselectAction() {
        return deselectAllAction;
    }

    public void addNotify() {
        super.addNotify();
        startQuery();
    }

    public DbbView getView() {
        return view_;
    }

    /**
     * Builds the toolBar
     */
    protected void buildToolBar() {
        toolBar_ = new JToolBar();
        toolBar_.setFloatable(false);

        deselectAllAction = new DeselectAllAction(view_);
        // deselectAllButton.addActionListener(deselectAllAction);
    }

    public void setCurrentSelection() {
        int rowCount = view_.getTable().getRowCount();
        if (rowCount == 0) {
            setEnabled(false);
            currentRow_ = -1;
            return;
        }
        if (currentRow_ < view_.getTable().getRowCount()) {
            view_.setSelectedRow(currentRow_);
        } else if (currentRow_ == rowCount) {
            view_.setSelectedRow((currentRow_) - 1);
        }
        JTable table = view_.getTable().getTable();
        Rectangle r = table.getCellRect(currentRow_, 1, true);
        table.scrollRectToVisible(r);
    }

    public void selectAll() {
        view_.selectAllRows();
    }

    public void clearSelection() {
        view_.clearSelection();
    }

    /**
     * Test if the View contains a specific obId
     * 
     * @param obId
     *            specifies the ID of the OB to be checked
     * @return true if a specific obId is contained into the view
     */
    public boolean containsOB(long obId) {
        for (int i = 0; i < view_.getRows(); i++) {
            if (obId == getOBID(i))
                return true;
        }
        return false;
    }

    /**
     * return the row id for the given OBId, or -1 if not found.
     */
    public int getRowID(long OBId) {
        for (int i = 0; i < view_.getRows(); i++) {
            if (OBId == getOBID(i))
                return i;
        }
        return -1;
    }

    public int[] getRowsForOBID(long OBId) {
        Vector<Integer> newVec = new Vector<Integer>();
        for (int i = 0; i < view_.getRows(); i++) {
            if (OBId == getOBID(i)) {
                newVec.addElement(new Integer(i));
            }
        }
        int[] rows = new int[newVec.size()];
        for (int j = 0; j < rows.length; j++) {
            rows[j] = ( newVec.elementAt(j)).intValue();
        }
        return rows;
    }

    public int[] getRowIDs(long[] OBIds) {
        Vector<Integer> resultV = new Vector<Integer>();
        for (int i = 0; i < OBIds.length; i++) {
            int row = getRowID(OBIds[i]);
            if (row >= 0)
                resultV.addElement(new Integer(row));
        }
        int[] returnSet = new int[resultV.size()];
        for (int j = 0; j < resultV.size(); j++) {
            Integer tmp =resultV.elementAt(j);
            returnSet[j] = tmp.intValue();
        }
        return returnSet;
    }

    /**
     * Returns the selected rows
     * 
     * @return an array with the index of selected rows an array of size 0 is
     *         returned if no rows are selected
     */
    public int[] getSelectedRows() {
        return view_.getSelectedRows();
    }

    /**
     * Return the index of all rows that contain the same OBID of a specific row
     * 
     * @param row
     *            specified the row
     */
    public int[] getDuplicatedRows(int row) {
        Vector<Integer> v = new Vector<Integer>();
        long[] OBIDs = getAllOBsId(true);
        for (int i = 0; i < OBIDs.length; i++) {
            if (getOBID(row) == OBIDs[i]) {
                v.addElement(new Integer(i));
            }
        }
        int[] returnRows = new int[v.size()];
        for (int i = 0; i < v.size(); i++) {
            returnRows[i] = ( v.elementAt(i)).intValue();
        }
        return returnRows;
    }

    /**
     * Returns all OBIDs displayed into the view
     */
    public long[] getAllOBsId(boolean allowDuplicate) {
        Vector<Long> v = new Vector<Long>();
        for (int i = 0; i < view_.getRows(); i++) {
            Long value = new Long(getOBID(i));
            if ((!v.contains(value)) || (allowDuplicate)) {
                v.addElement(value);
            }
        }
        long[] returnOBIDs = new long[v.size()];
        for (int i = 0; i < v.size(); i++) {
            returnOBIDs[i] = ( v.elementAt(i)).longValue();
        }
        return returnOBIDs;
    }

    /**
     * Return the ids of all selected OBs An array of size 0 is returned if no
     * OBs are selected
     * 
     * @return the id of all selected OB
     */
    public long[] getSelectedOBIDs(boolean allowDuplicate) {
        int[] selectedRows = view_.getSelectedRows();
        return getOBIDs(selectedRows, allowDuplicate);
    }

    /**
     * Returns the ID of an OB of a specific row
     */
    public long getOBID(int row) {
        Integer value = (Integer) view_.getValueAt(row, OTDbbView.OB_ID);
        String type = (String) view_.getValueAt(row, OTDbbView.TYPE);
        if (type.equals("O")) {
            return Config.getCfg().tableToUniqueId(value.longValue(),
                    ObservationBlock.class);
        }

        return Config.getCfg().tableToUniqueId(value.longValue(),
                CalibrationBlock.class);
    }

    /**
     * Retruns a set of IDs of OBs of specific rows
     */
    public long[] getOBIDs(int[] rows, boolean allowDuplicate) {
        Vector<Long> v = new Vector<Long>();
        for (int i = 0; i < rows.length; i++) {
            Long value = new Long(getOBID(rows[i]));
            if ((allowDuplicate) || (!v.contains(value))) {
                v.addElement(value);
            }
        }
        long[] returnOBIDs = new long[v.size()];
        for (int i = 0; i < v.size(); i++) {
            returnOBIDs[i] = ( v.elementAt(i)).longValue();
        }
        return returnOBIDs;
    }

    /**
     * start the sql engine query mechanism.
     */
    public void startQuery() {
        view_.sqlStartAction(null);
    }

    public void addDefaultColumns()
            throws NotBoundException, MalformedURLException, SQLException,
            RemoteException {

        String label_name;
        String column_name;

        // ID
        column_name = "ob_id";
        label_name = "OB ID";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(cr_table, column_name,
                label_name));

        // Type
        column_name = "ob_type";
        label_name = "Type";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(cr_table, column_name,
                label_name));

        // Status
        column_name = "ob_status";
        label_name = "Status";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(cr_table, column_name,
                label_name));

        // ProgID
        column_name = "progid";
        label_name = "ProgID";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(cr_table, column_name,
                label_name, String.class));

        // OB Name
        column_name = "item_name";
        label_name = "OB Name";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(cr_table, column_name,
                label_name));

        // RA Value
        column_name = "ra";
        label_name = "RA";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(cr_table, column_name,
                label_name));

        // Configuration number
        column_name = "config_idx";
        label_name = "Configuration";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(cr_table, column_name,
                label_name));
    }

    /**
     * Build DB browser view: sort selection panel. This panel is required to
     * sort the rows, but it is never displayed.
     */
    public void buildSort() {
        DbbPanel panel = new DbbPanel("Sorting Criteria", 1);
        DbbSqlChunk c;
        DbbArrowButton w;
        String column_name;
        String label_name;

        // config idx
        label_name = "Configuration Idx";
        column_name = "config_idx";

        c = new DbbSqlChunk(cr_table, column_name, label_name);
        w = new DbbArrowButton(label_name, c, 1);

        panel.addWidget(w);
        w.actionPerformed(new ActionEvent(this, 0, null));

        // ob id
        label_name = "OB ID";
        column_name = "ob_id";

        c = new DbbSqlChunk(cr_table, column_name, label_name);
        w = new DbbArrowButton(label_name, c, 1);

        panel.addWidget(w);
        w.actionPerformed(new ActionEvent(this, 0, null));

        // ra
        label_name = "RA";
        column_name = "ra";

        c = new DbbSqlChunk(cr_table, column_name, label_name);
        w = new DbbArrowButton(label_name, c, 1);

        panel.addWidget(w);
        w.actionPerformed(new ActionEvent(this, 0, null));

        view_.setSortPanel(panel, "West", true);
    }

    private class TableListSelectionListener implements ListSelectionListener {
        public TableListSelectionListener() {
        }

        public void valueChanged(ListSelectionEvent ev) {
            if (ev.getValueIsAdjusting()) {
                return;
            }
            ListSelectionModel lsm = (ListSelectionModel) ev.getSource();
            currentRow_ = lsm.getMinSelectionIndex();
            if (!lsm.isSelectionEmpty() && currentRow_ >= 0) {
                setEnabled(true);
            } else {
                setEnabled(false);
            }
        }
    }

    public static boolean createConfigReportTable() {
        boolean rc;
        if (configReportTempTableSetup) {
            return emptyConfigReport();
        }
        stdlog_.debug("Creating config report");
        Connection con = null;
        try {
            String sqlText = "create table #tmp_config_report " + " ( "
                    + " ob_id       int not null, "
                    + " ob_type     char(1) not null, "
                    + " ob_status   char(1) not null, "
                    + " progid      varchar(32) not null, "
                    + " item_name   varchar(32) not null, "
                    + " ra          varchar(32), "
                    + " config_idx  tinyint not null )";

            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(), cfg.getDbPassword());
            Statement stmt = con.createStatement();
            stdlog_.info(sqlText);
            stmt.executeUpdate(sqlText);
            stmt.close();
            configReportTempTableSetup = true;
            rc = true;
        } catch (SQLException e) {
            stdlog_.error("Failed to create config report table. "
                    + e.toString());
            configReportTempTableSetup = false;
            rc = false;
        } finally {
            if (con != null) {
                ConnectionPool.getPool().releaseConnection(con);
            }
        }
        return rc;
    }

    public static boolean emptyConfigReport() {
        if (!configReportTempTableSetup) {
            return false;
        }
        boolean rc;
        stdlog_.debug("Adding to config report");
        Connection con = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(), cfg.getDbPassword());
            Statement stmt = con.createStatement();
            stmt.executeUpdate("delete from #tmp_config_report");
            stmt.close();
            rc = true;
        } catch (SQLException e) {
            rc = false;
            stdlog_.error("Failed to empty config report: " + e.toString());
        } finally {
            if (con != null) {
                ConnectionPool.getPool().releaseConnection(con);
            }
        }
        return rc;
    }

    /**
     * Add a row to the table. The RA value maye be null, all other values are
     * required.
     * 
     * @return true if insert is sucessful, else false.
     */
    public static boolean addToOBToConfigReportTable(int obid, String obtype,
            String obstatus, String progId, String obName, String ra,
            int configIdx) {
        if (!configReportTempTableSetup) {
            return false;
        }
        if (ra == null) {
            ra = "";
        }
        boolean rc;
        stdlog_.debug("Adding to config report");
        Connection con = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(), cfg.getDbPassword());
            Statement stmt = con.createStatement();
            stmt
                    .executeUpdate("insert #tmp_config_report "
                            + " ( ob_id, ob_type, ob_status, progid, item_name, ra, config_idx ) "
                            + " values ( " + " " + obid + ", '" + obtype
                            + "', '" + obstatus + "', '" + progId + "', '"
                            + obName + "', '" + ra + "', " + configIdx + ")");
            stmt.close();
            rc = true;
        } catch (SQLException e) {
            rc = false;
            stdlog_.error("Failed to insert into config report: "
                    + e.toString());
        } finally {
            if (con != null) {
                ConnectionPool.getPool().releaseConnection(con);
            }
        }
        return rc;
    }
}
