package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbaseDec;
import org.eso.ohs.core.dbb.client.DbaseRA;
import org.eso.ohs.core.dbb.client.DbbExecutionTime;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.client.MoonDistance;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.phase2.actions.FindingChartViewer;
import org.eso.ohs.phase2.actions.ShowFindingChartsAction;
import org.eso.ohs.phase2.apps.ot.dbb.BrowserConfig;
import org.eso.ohs.phase2.apps.ot.dbb.OTDbbView;

/**
 * This is the main class for the finding chart viewer. 
 * It is a simple dbb which 
 * 
 * @author tcanavan
 *
 */
public class FindingChartBrowser extends JFrame {

	private OTDbbView view_;
    
    private DbbView dbbView_;



	/** Standard log channel */
	private static Logger stdlog_ =
		Logger.getLogger(FindingChartBrowser.class);

	private ShowFindingChartsAction findingChartAction_;

	/**
	 * Constructor for FindingChartBrowser.
	 */
	public FindingChartBrowser(String title) {
		super(title);
		try {
			BrowserConfig.getEngine();
			view_ = new OTDbbView();
			dbbView_ = view_.getDbbView();
            dbbView_.hideSortButton();
            dbbView_.hideResetButton();
			buildRows();
			buildColumns();
			findingChartAction_ =
				new ShowFindingChartsAction(Media.DBASE,view_,view_.getJTable(),false);
			JButton fcButton = new JButton();
			fcButton.setAction(findingChartAction_);
			dbbView_.addSummaryButton(fcButton);
			getContentPane().add(dbbView_);
			//if(Config.getCfg().isInOperationsMode()) {
			view_.addObjectSelectionListener(FindingChartViewer.getInstance());
            view_.createDefaultListListener();        

			//} 
			setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
			addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent ev) {
					int result = askAndQuit();
					if (result != JOptionPane.YES_OPTION) {
						return;
					} else {
						System.exit(0);
					}

				}
			});
			String menuName = "File";
			JMenu menu = new JMenu(menuName);
			JMenuItem close = new JMenuItem("Close");
			close.setAccelerator(
				KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_MASK, false));
			close.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					int result = askAndQuit();
					if (result != JOptionPane.YES_OPTION) {
						return;
					} else {
						System.exit(0);
					}
				}
			});
			menu.add(close);
			JMenuBar mb = new JMenuBar();
			mb.add(menu);
			setJMenuBar(mb);			
			pack();
			setVisible(true);
		} catch (RemoteException e) {
			String message = "Error: " + e.getMessage();
			JOptionPane.showMessageDialog(
				null,
				message,
				"ERROR",
				JOptionPane.ERROR_MESSAGE);
			stdlog_.error(e);
		} catch (MalformedURLException e) {
			String message = "Error: " + e.getMessage();
			JOptionPane.showMessageDialog(
				null,
				message,
				"ERROR",
				JOptionPane.ERROR_MESSAGE);
			stdlog_.error(e);
		} catch (NotBoundException e) {
			String message = "Error: " + e.getMessage();
			JOptionPane.showMessageDialog(
				null,
				message,
				"ERROR",
				JOptionPane.ERROR_MESSAGE);
			stdlog_.error(e);
		} catch (SQLException e) {
			String message = "Error: " + e.getMessage();
			JOptionPane.showMessageDialog(
				null,
				message,
				"ERROR",
				JOptionPane.ERROR_MESSAGE);
			stdlog_.error(e);
		}

	}
	
	private int askAndQuit() {
		boolean quit = true;
		int result =
			JOptionPane.showConfirmDialog(
				dbbView_.getTable().getTable(),
				"Would you like to quit now ?",
				"Quit Now?",
				JOptionPane.YES_NO_OPTION);
		return result;
	}

	/**
	  * Build DB browser view: SELECT panel
	  *
	  * <br>ChangeLog: Gorgio Passera
	  * @exception SQLException JDBC exception occurs
	  * @exception RemoteException RMI exception occurs
	  * @exception MalformedURLException malformed url i.e. servername//port
	  * @exception NotBoundException client could not bind to server
	  */
	private void buildColumns()
		throws
			NotBoundException,
			MalformedURLException,
			SQLException,
			RemoteException {
		// collect the default SQLChunks from super.
		//        super.buildColumns();

		String label_name;
		String column_name;
		column_name = "ob_id";
		label_name = "OB ID";
		DbbSqlEngine engine = dbbView_.getSqlEngine();
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(BrowserConfig.ob, column_name, label_name));

		// Type
		column_name = "type";
		label_name = "Type";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(BrowserConfig.ob, column_name, label_name));

		// Status
		column_name = "status";
		label_name = "Status";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(BrowserConfig.ob, column_name, label_name));

		// OB Name
		label_name = "OB Name";
		column_name = "item_name";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(BrowserConfig.ob, column_name, label_name));
            
        label_name = "Finding Charts";
        column_name = "fc_name";
        engine.addDefaultColumnsChunk(
                new DbbSqlChunk(BrowserConfig.finds_view, column_name, label_name));
            

		label_name = "Instrument";
		column_name = "instrument";

		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(BrowserConfig.od, column_name, label_name));

		// ProgID
		label_name = "ProgID";
		column_name = "progid";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(
				BrowserConfig.sr,
				column_name,
				label_name,
				String.class));

		// PI (surname)
		label_name = "LastName";
		column_name = "last_name";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(BrowserConfig.profiles, column_name, label_name));

		// RA
		label_name = "RA";
		column_name = "ra";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(
				BrowserConfig.tp,
				column_name,
				label_name,
				DbaseRA.class));

		// Dec
		label_name = "Dec";
		column_name = "dec";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(
				BrowserConfig.tp,
				column_name,
				label_name,
				DbaseDec.class));

		// Exec Time
		label_name = "ExecTime";
		column_name = "execution_time";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(
				BrowserConfig.ob,
				column_name,
				label_name,
				DbbExecutionTime.class));

		// Rank Class
		label_name = "RankClass";
		column_name = "rank_class";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(
				BrowserConfig.sr,
				column_name,
				label_name,
				String.class));

		// UsrP
		label_name = "UsrP";
		column_name = "userPriority";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(BrowserConfig.ob, column_name, label_name));

		// Seeing
		label_name = "Seeing";
		column_name = "seeing";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(BrowserConfig.cs, column_name, label_name));

		// Airmass
		label_name = "Airmass";
		column_name = "airmass";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(BrowserConfig.cs, column_name, label_name));

		// Sky Tran
		label_name = "SkyTran";
		column_name = "sky_transparency";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(BrowserConfig.cs, column_name, label_name));

		// FLI
		label_name = "FLI";
		column_name = "fli";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(BrowserConfig.cs, column_name, label_name));

		// MoonDis
		label_name = "MoonDis";
		column_name = "moon_distance";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(
				BrowserConfig.cs,
				column_name,
				label_name,
				MoonDistance.class));

		// Strehl
		label_name = "Strehl";
		column_name = "strehl_ratio";
		engine.addDefaultColumnsChunk(
			new DbbSqlChunk(BrowserConfig.cs, column_name, label_name));


		//label_name = "hostname";
		//column_name = "hostname";
		//engine_.addDefaultColumnsChunk(
		//	new DbbSqlChunk(BrowserConfig.oes, column_name, label_name));

		DbbPanel panel = new DbbPanel("Selected Columns", 9);
		dbbView_.setColumnsPanel(panel, "North",false);
	}

	/**Build DB browser view: rows selection panel*/
	private void buildRows() {

		DbbPanel panel = new DbbPanel("Selection Criteria", 5);
		DbbSqlChunk c;

		String label_name;
		String column_name;

		DbbSqlOperator op_equal = null;
		DbbSqlOperator op_min = null;
		DbbSqlOperator op_max = null;
		DbbSqlOperator op_like = null;
		DbbSqlOperator op_in = null;

		try {
			op_equal = new DbbSqlOperator("=");
			op_min = new DbbSqlOperator("<=");
			op_max = new DbbSqlOperator(">=");
			op_like = new DbbSqlOperator("like");
			op_in = new DbbSqlOperator("in");
		} catch (Exception e) {
			stdlog_.error(e);
		}

		// Queue ID
		/*label_name = "Queue ID";
		column_name = "sch_id";
		
		c =
		    new DbbSqlChunk(
		        es_table,
		        column_name,
		        label_name,
		        op_equal,
		        Integer.class);
		queueIDTF = new DbbTextField(label_name, c);
		panel.addWidget(queueIDTF);
		
		queueIDTF.set(new Long(Config.getCfg().uniqueToTableId(queue.getId())));
		queueIDTF.setEnabled(false);*/

		label_name = "hostname";
		column_name = "hostname";
		c =
			new DbbSqlChunk(
				BrowserConfig.oes,
				column_name,
				label_name,
				op_equal,
				String.class);
		DbbTextField hostTextField = new DbbTextField(label_name, c,false);
		panel.addWidget(hostTextField);
        dbbView_.setWherePanel(panel, "North", false);
        
		try {
			hostTextField.set(ExecutionSequenceBView.getHostNameNODot());
		} catch (java.net.UnknownHostException e) {
            JTable table1 = dbbView_.getTable().getTable();
            String[] text = new String[1];
            text[0] = "Cannot resolve IP host name.";
            ErrorMessages.announceError(table1, text, e);
		}
        hostTextField.setEnabled(false);
	}

	/** 
	 * start the sql engine query mechanism. 
	 */
	public void startQuery() {
		view_.sqlStartAction();
	}

	public void addNotify() {
		super.addNotify();
		startQuery();
		stdlog_.debug("!!!!!!!!!!!!!!!!! Before start query");
	}

	public static void main(String[] args) {
		FindingChartBrowser fcBrowser =
			new FindingChartBrowser("Finding Chart Browser");
		fcBrowser.pack();
		fcBrowser.setVisible(true);

	}

}
