package org.eso.ohs.phase2.apps.ot.gui;

import java.util.Vector;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.QueueItem;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectSelector;
import org.eso.ohs.phase2.apps.ot.wizard.OrangEngine;
import org.eso.ohs.phase2.orang.OBScheduleInfo;
import org.eso.ohs.phase2.orang.OrangModel;
import org.eso.ohs.phase2.visiplot.VisiPlotOptions;

/**
 * This class manages interaction between different views
 *
 * @author Gorgio Passera <DMD-ESO> <gpassera@eso.org>
 */

public class OTViewManager {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(OTViewManager.class);

    //a set of view references
    private OTMainView otMainView = null;

    //a vector of QueueVievManager objects
    //This vector contains all QueueViewManager that are currently managed

    private Vector<ObjectSelector> queueViewMgrVector = new Vector<ObjectSelector>();

    //a reference to a singleton OTViewManager instance 
    private static OTViewManager otViewMgr = new OTViewManager();

    /**Default constructor*/
    protected OTViewManager() {
    }

    /**
     *Returns the one and only instace of a the OTViewManager 
     *@return the singleton OTViewManager instance
     */
    public static OTViewManager getOTViewMgr() {
        return otViewMgr;
    }

    /**
     *Returns a set of QueueViewManager(s) managed
     *@return a set of QueueVievManager(s) managed
     */
    public Vector<ObjectSelector> getQueueViews() {
        return queueViewMgrVector;
    }

    public static void setInstance(OTViewManager instance) {
        otViewMgr = instance;
    }

    /**
     *Returns the reference to the OTMainView
     *@return the reference to the OTMainView
     */
    public OTMainView getOTMainView() {
        return otMainView;
    }

    public void setOTMainView(OTMainView otMainView) {
        this.otMainView = otMainView;
    }

    /**
     *Retruns the QueueViewManager that manage a specific queue
     *@param queueId the id of the Queue
     *@return the QueueViewManager of a specific queue
     *        null is retruned if the queue is not managed by 
     *        any QueueViewManager 
     */
    public QueueView getQueueView(long queueId) {

        for (int i = 0; i < queueViewMgrVector.size(); i++) {
            QueueView mgr = (QueueView) queueViewMgrVector.elementAt(i);
            //	    stdlog_.debug(mgr.getQueue().getId());
            if (mgr.getQueue().getId() == queueId) {
                return mgr;
            }
        }
        return null;
    }
    /**Return the current selected view into the OTMainPanel*/
    public QueueView getSelectedQueueView() {
        if (!isQueueSelected()) {
            return null;
        }
        int[] selectedRows = otMainView.getSelectedRows();
        QueueView qvm =
            (QueueView) queueViewMgrVector.elementAt(selectedRows[0]);
        return qvm;
    }

    public void addQueueView(ObjectSelector queueView) {
        queueViewMgrVector.insertElementAt(queueView, 0);
    }

    public void showObject(BusinessObject busObj, Class c) {
      
        if (c.equals(Queue.class) || c.equals(MaskQueue.class)) {
            QueueView queueView = new QueueView((Queue) busObj);
            addQueueView(queueView);
            ((Queue) busObj).setUpdate(false);
        }
        ((Queue) busObj).setUpdate(false);
    }

    public void showOrangObservableQueue(Queue queue, VisiPlotOptions settings,
            OrangModel model, OBScheduleInfo[] obs, QueueItem[] queueItems, OrangEngine engine) {
        QueueOrangView queueView = new QueueOrangView(obs,model,engine); 
     
            queueView.initRankedQueueView(queue,settings);           
            addQueueView(queueView);
            queue.setUpdate(false); 
           
            queueView.appendQueueItems(queueItems); 
             queueView.getQueueItemsView().getView().sqlStartAction();
    }
    
    public void showOrangNotObservableQueue(Queue queue,
            VisiPlotOptions settings,  OrangModel model, 
            OBScheduleInfo [] obs, QueueItem[] queueItems, OrangEngine engine) {
        QueueOrangView queueView = new QueueOrangView(obs,model,engine); 
            queueView.initQueueView(queue,settings);           
            addQueueView(queueView);
            queue.setUpdate(false); 
            long[] queueItemsLong = new long[queueItems.length];
            for (int i = 0; i < queueItems.length; i++) {
                queueItemsLong[i] = queueItems[i].getObId();
            }
            queueView.appendOB(queueItemsLong); 
            queueView.getQueueItemsView().getView().sqlStartAction();
    }
    /** 
     * A utility method to remove a queue from the managed vector 
     * @param queue Queue to be remnoved.
     */

   
    /**Action performed for each OTMainView  menu item*/
    public void askAndQuit() {
        boolean quit = true;
        int result =
            JOptionPane.showConfirmDialog(
                otMainView,
                "Would you like to quit now ?",
                "Quit Now?",
                JOptionPane.YES_NO_OPTION);

        if (result != JOptionPane.YES_OPTION) {
            return;
        }

        for (int i = (queueViewMgrVector.size()) - 1; i >= 0; i--) {
            QueueView qvm = (QueueView) queueViewMgrVector.elementAt(i);
            Queue queue = qvm.getQueue();
            if (queue.isUpdate())
                quit = qvm.closeQueue();

            if (!quit) {
                break;
            }
        }
        for (int i = (queueViewMgrVector.size()) - 1; i >= 0; i--) {
            QueueView qvm = (QueueView) queueViewMgrVector.elementAt(i);
            quit = qvm.closeQueue();
            if (!quit) {
                break;
            }
        }
        if (!quit) {
            return;
        }
        System.exit(0);
    }

    public boolean isQueueSelected() {
        int[] selectedRows = otMainView.getSelectedRows();
        return (selectedRows.length != 0);
    }
    
    

    public boolean isQueueOpen(long queueId) {
        for (int i = 0; i < queueViewMgrVector.size(); i++) {
            QueueView mgr = (QueueView) queueViewMgrVector.elementAt(i);
            if (mgr.getQueue().getId() == queueId) {
                return true;
            }
        }
        return false;
    }

    /**Return the current selected view into the OTMainPanel*/
    public Queue getSelectedQueue() {
        if (!isQueueSelected()) {
            return null;
        }

        int[] selectedRows = otMainView.getSelectedRows();
        QueueView qvm =
            (QueueView) queueViewMgrVector.elementAt(selectedRows[0]);

        return qvm.getQueue();
    }
    
    
    /**
     * Close a queue and drop from the object manager.
     * If the queue is null or closed, does nothing.
     * @param queue the queue to close
     */
    public void forceQueueClose (Queue queue) {
        
        if (queue!= null && isQueueOpen(queue.getId())) {
            QueueView queueView = getQueueView(queue.getId());
            queueView.disposeNoSave();
            ObjectManager.getObjectManager().forgetBusObj(Media.DBASE,queue.getId(),Queue.class);
        }
    }
    
    /**
     * Return the selected qeue 
     * @return boolean
     */
    
    public boolean isQueueSelected(Queue queue) {
        if(isQueueSelected()) {
            if(getSelectedQueue().getId() == queue.getId())
                return true;
        }
        return false;
    }
    
}
