package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.Vector;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableColumnModel;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbaseDec;
import org.eso.ohs.core.dbb.client.DbaseRA;
import org.eso.ohs.core.dbb.client.DbbArrowButton;
import org.eso.ohs.core.dbb.client.DbbCheckBox;
import org.eso.ohs.core.dbb.client.DbbExecutionTime;
import org.eso.ohs.core.dbb.client.DbbHHMMDateTime;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbSortTerminated;
import org.eso.ohs.core.dbb.client.DbbSortTerminatedListener;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.client.MoonDistance;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.CommentType;
import org.eso.ohs.dfs.MaskStatusType;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.QueueItem;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OBEvent;
import org.eso.ohs.persistence.OBListener;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.phase2.DbaseHandlerQueue;
import org.eso.ohs.persistence.dbase.phase2.QueueManager;
import org.eso.ohs.phase2.actions.DumpMaskAdpAdmData;
import org.eso.ohs.phase2.actions.ExportBOAction;
import org.eso.ohs.phase2.actions.ShowFindingChartsAction;
import org.eso.ohs.phase2.apps.ot.actions.CopyToClipboardAction;
import org.eso.ohs.phase2.apps.ot.actions.DeselectAllAction;
import org.eso.ohs.phase2.apps.ot.actions.DisplayOBAction;
import org.eso.ohs.phase2.apps.ot.actions.ExecutionSequenceAction;
import org.eso.ohs.phase2.apps.ot.actions.JustifyRankingAction;
import org.eso.ohs.phase2.apps.ot.actions.MarkReadmeStatusAction;
import org.eso.ohs.phase2.apps.ot.actions.MarkStatusAction;
import org.eso.ohs.phase2.apps.ot.actions.MoveDownRowsAction;
import org.eso.ohs.phase2.apps.ot.actions.MoveUpRowsAction;
import org.eso.ohs.phase2.apps.ot.actions.OBHistoryAction;
import org.eso.ohs.phase2.apps.ot.actions.OpenCommentsAction;
import org.eso.ohs.phase2.apps.ot.actions.PasteToClipboardAction;
import org.eso.ohs.phase2.apps.ot.actions.ReloadQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.ReloadQueueItemsAction;
import org.eso.ohs.phase2.apps.ot.actions.RemoveQueueItemsAction;
import org.eso.ohs.phase2.apps.ot.dbb.BrowserConfig;
import org.eso.ohs.phase2.apps.ot.dbb.OTDbbView;
import org.eso.ohs.phase2.apps.ot.wizard.OrangEngine;

/**
 * The QueueItemsBView class is a DB search engine used for displaying queues of
 * OBs according to a user-definable set of properties
 * 
 * @author G Passera <br>
 *         ChangeLog: Tim Canavan <br>
 *         ChangeLog: cmavros Add call to unsubscribe execution action. <br>
 *         ChangeLog: cmavros 1709/OT Call ExecutionSequenceAction class to
 *         implement progress bar <br>
 *         ChangeLog: cmavros 23May02 [1643/OT] Add selection items so user can
 *         customize. <br>
 *         ChangeLog: cmavros 25JUL02 [?/OT] Change "Execution Time" label to
 *         "ExecTime" becasue the sort does an comparison with the label.
 * 
 * @version $Id: QueueItemsBView.java,v 1.82.2.3 2007/10/23 08:09:12 pnunes Exp $
 * 
 */

public class QueueItemsBView implements QueueContainer, OBListener,
        TableMenuEnable, TreeViewUpdate {

    private static final String README_STATUS_COLUMN = "Readme Status";

    static Logger stdlog_ = Logger.getLogger(QueueItemsBView.class);

    private Action reloadQueueItemsAction_;

    private ExecutionSequenceAction executionAction_;

    private Action removeAction_;

    private Action markAction_;

    private Action markReadmeAction_;

    private Action openCommentsAction_;

    private Action exportAction_;

    private Action dumpAdpAdmDataAction_;
    


    private static TextDisplayWidget log_ = new TextDisplayWidget(
            "Append Queue Items Report");

    // private DbbSqlTable BrowserConfig.si_tmp = BrowserConfig.si_tmp;

    private PasteToClipboardAction pasteActionListener;

    private CopyToClipboardAction copyActionListener_;

    protected DbbArrowButton seqWidget;

    private JButton reload;

    protected static DbbPanel colPanel_;

    protected static DbbPanel rowPanel_;

    protected static DbbPanel sortPanel_;

    protected static DbbPanel colObsPanel_;

    protected static DbbPanel rowObsPanel_;

    protected static DbbPanel sortObsPanel_;

    private  JPanel outerPanel;

    private DbbCheckBox maskStatusBox;

    private DbbCheckBox maskSlotNumBox;

    private DbbCheckBox maskChannelBox;

    private DbbSqlChunk obIDChunk;

    private Queue queue;
 
    private OBTreeView tree_;
    
    private OTDbbView view;

    private DbbView dbbView_;

    protected JToolBar toolBar_;

    private TableListSelectionListener listListener_;

    private ListSelectionListener treeViewClearer_;

    private DeselectAllAction deselectAllAction;

    private MoveUpRowsAction moveUpAction;

    private MoveDownRowsAction moveDownAction;

    private DisplayOBAction displayAction;

    private OBHistoryAction obHistoryAction;

    private ShowFindingChartsAction findingChartAction;

    private JButton markButton;

    private JButton markReadmeButton;

    private DbbTextField queueIDTF;
    
    
    
    
    public static QueueItemsBView createNormalQueue(Queue newQueue, OBTreeView tree) {
        QueueItemsBView queueDbbView =new QueueItemsBView(newQueue,tree);
        queueDbbView.createNormalDbbPanel();
        queueDbbView.buildQueueToolBar();
        queueDbbView.buildStructure();
        return queueDbbView;
    }

	private  void createNormalDbbPanel() {
		try {
            buildColumns();            
            buildRows();
            buildSort();                
        } catch (Exception e) {
            MsgManager.errDialog(e);
        }
	}
    
    public static QueueItemsBView createObservableQueue(Queue newQueue, OBTreeView tree) {
        QueueItemsBView queueDbbView = new QueueItemsBView(newQueue,tree);
        queueDbbView.createOrangObservableDbbPanels();
        queueDbbView.buildStandardOrangeQueueToolBar();
        queueDbbView.buildStructureStandard();
        queueDbbView.hideRevertButton();
        return queueDbbView;
    }

	private void createOrangObservableDbbPanels() {
		try {
			buildOrangColumns();
			buildRows();
			buildOrangSort();
			dbbView_.hideSortButton();
		} catch (MalformedURLException e) {
			MsgManager.errDialog(e);
		} catch (RemoteException e) {
			MsgManager.errDialog(e);
		} catch (NotBoundException e) {
			MsgManager.errDialog(e);

		} catch (SQLException e) {
			MsgManager.errDialog(e);
		}
	}
	
	public void addJustifyAction(OrangEngine engine) {
		Action justify = new JustifyRankingAction(view,view.getJTable(),false,"Justify ranking",engine);
		JButton justifyBtn = new JButton("Justify ranking");
		justifyBtn.setAction(justify);
		view.getDbbView().addSummaryButton(justifyBtn);
	}
	
	private void createOrangNonObservableDbbPanels() {
		try {
			buildColumns();
			buildRows();
			dbbView_.hideSortButton();
			
		} catch (MalformedURLException e) {
			MsgManager.errDialog(e);
		} catch (RemoteException e) {
			MsgManager.errDialog(e);
		} catch (NotBoundException e) {
			MsgManager.errDialog(e);

		} catch (SQLException e) {
			MsgManager.errDialog(e);
		}
	}

    
    public static QueueItemsBView createNotObservableQueue(Queue newQueue, OBTreeView tree) {
        QueueItemsBView queueDbbView = new QueueItemsBView(newQueue,tree);
        queueDbbView.createOrangNonObservableDbbPanels();
        queueDbbView.buildStandardOrangeQueueToolBar();
        queueDbbView.buildStructureStandard();
        queueDbbView.hideRevertButton();
        return queueDbbView;
    }
    
    
    // To be refactoed later. 
    
    public void hideRevertButton() {
    	reload.setVisible(false);
    }
    
    
    
    public static QueueItemsBView createObservableWizardQueue(Queue newQueue, OBTreeView tree) {
        QueueItemsBView queueDbbView = new QueueItemsBView(newQueue,tree);
        queueDbbView.createOrangObservableDbbPanels();
        queueDbbView.buildQueueToolBar();
        queueDbbView.buildStructureStandard();
        return queueDbbView;
    }
    
    public static QueueItemsBView createNotObservableWizardQueue(Queue newQueue, OBTreeView tree) {
        QueueItemsBView queueDbbView = new QueueItemsBView(newQueue,tree);
        queueDbbView.createOrangObservableDbbPanels();
        queueDbbView.buildQueueToolBar();
        queueDbbView.buildStructureStandard();
        return queueDbbView;
    }

       

        
    public QueueItemsBView(Queue newQueue, OBTreeView tree) {
        assert newQueue != null && tree != null : MsgManager.errmsg("queue must be not null"); 
        try {
            initView(newQueue,tree);
            createActions();

        } catch (Exception e) {
            MsgManager.errDialog(e);
        }
    }


    /**
     * @param newQueue
     * @param title
     * @param clearButton
     * @param queryButton
     * @param sortButton
     * @param tree
     * @param OB_ID_COLUMN
     * @param OB_TYPE_COLUMN
     * @param OB_STATUS_COLUMN
     * @throws SQLException
     *             void
     */
    private void initView(Queue newQueue, OBTreeView tree)
            throws SQLException {
        queue = newQueue;
        tree_ = tree;
        view = new OTDbbView();
        dbbView_ = view.getDbbView();
        
        listListener_ = new TableListSelectionListener(this);
        getDbbView().addListSelectionListener(listListener_);
        treeViewClearer_ = new TreeViewClearer(this);
        getDbbView().addListSelectionListener(treeViewClearer_);
        getDbbView().getTable().removeSortListener();
        QueueManager.addOBListener(this);
    }

 
   


    

    public void addDisplayTextAction(JFrame frame) {
        getDbbView().addDisplayTextAction(frame, "Text View", queue.getName(),
                queue.getName());
    }
    
    public void addDisplayTextAction(JComponent comp) {
        getDbbView().addDisplayTextAction(comp,"Text View", queue.getName(),
                queue.getName());
    }
    
    
    //private void buildOrangLayout()

    /**
     * void
     */
    private void  buildStructure() {
        outerPanel = new JPanel(new BorderLayout());
        outerPanel.add(toolBar_, BorderLayout.NORTH);
        outerPanel.add(getDbbView(), BorderLayout.CENTER);        
        getDbbView().addSortTerminatedListener(new SortTerminatedListener());
        addDisplayTextAction(view.getJTable());
        setEnabled(false);    
        
//        outerPanel = new JPanel();
//        outerPanel.add(toolBar_);
//        outerPanel.add(getDbbView());        
//        getDbbView().addSortTerminatedListener(new SortTerminatedListener());
//        addDisplayTextAction(view.getJTable());
//        addReloadButtons();
//        setEnabled(false);    
    }
    
    
    private void  buildStructureStandard() {        
        outerPanel = new JPanel(new BorderLayout());
        outerPanel.add(toolBar_,BorderLayout.NORTH);
        outerPanel.add(getDbbView(),BorderLayout.CENTER);        
        getDbbView().addSortTerminatedListener(new SortTerminatedListener());
        addDisplayTextAction(view.getJTable());
        setEnabled(false);    
    }

    
    /**
     * Delete queue Items and save them. 
     * To be made optimal later. 
     * @param queue
     */

     public void saveQueue(Queue queue) {
    	 deleteQueueItems();	
 		 appendQueueItems(queue.getQueueList()); 
     }



    private void createActions() {

        deselectAllAction = new DeselectAllAction(getDbbView());
        // deselectAllButton.addActionListener(deselectAllAction);

        moveUpAction = new MoveUpRowsAction(view,queue);
        // moveUpButton.addActionListener(moveUpAction);

        moveDownAction = new MoveDownRowsAction(view,queue);
        
        displayAction = new DisplayOBAction(view, tree_);

        obHistoryAction = new OBHistoryAction(view);
        findingChartAction = new ShowFindingChartsAction(Media.DBASE, view,
                view.getJTable(), false);
        
        pasteActionListener = new PasteToClipboardAction(this);
        
        executionAction_ = new ExecutionSequenceAction(view,view.getJTable(),
                        "Execution Sequence");
        removeAction_ = new RemoveQueueItemsAction(this);
        dumpAdpAdmDataAction_ = new DumpMaskAdpAdmData(view,view.getJTable());
        openCommentsAction_ = new OpenCommentsAction(this);
        exportAction_ = new ExportBOAction(Media.DBASE, view,view.getJTable(), "Export...",true);
        copyActionListener_ = new CopyToClipboardAction(view);
        markAction_ = new MarkStatusAction();
        markReadmeAction_ = new MarkReadmeStatusAction();
        reload = new JButton("Revert");
        getDbbView().addSummaryButton(reload);
        reload.addActionListener(new ReloadActionListener(this));
        getDbbView().getSummaryButtons().removeStartQueryListener();
        reload.setEnabled(false);
        reloadQueueItemsAction_ = new ReloadQueueItemsAction(this);
        JButton queryButton = getDbbView().getSummaryButtons().getQueryButton();
        queryButton.addActionListener(reloadQueueItemsAction_);
    
        
    }
    
    public void removeOBs(long[] selectedOBIDs) {
        try {
            
            //  Updatw Temporary Table
            QueueManager.deleteOBsFromQueueTable(
                selectedOBIDs,
                queue.getId(),
                DbaseHandlerQueue.TMP_SCHEDULE_ITEMS);
            // Remove From view 
            
            view.removeSelectedOBs(selectedOBIDs);
            
            // remove from business object
            
            
            queue.removeQueueItems(selectedOBIDs);
            
            
            // If any obs are curerently displayed in the tree are removed
            // do not show anything in the tree control
            
            if (tree_ != null && tree_.getOB() != null) {
                for (int i = 0; i < selectedOBIDs.length; i++) {
                    if (tree_.getOB().getId() == selectedOBIDs[i]) {
                        tree_.updateView(null);
                        break;
                    }
                }
            }
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(view.getJTable(), e);
        }
    }
    

    public void refreshReadmeStatus(long runId, String status) {
        int colIndex = getReadmeStatusColIndex();
        if (colIndex == -1)
            return;
        int[] rows = getDbbView().getRowsFromColumnName(new Long(runId), OTDbbView.RUN_ID);
        for (int i = 0; i < rows.length; i++) {
            // increment colindex because one
            // default column was removed.
            // This should be fixed properly
            view.setValueAt(status, rows[i], README_STATUS_COLUMN);
        }
    }

    public int getReadmeStatusColIndex() {
        JTable table = getDbbView().getTable().getTable();
        TableColumnModel tblMdl = table.getColumnModel();
        try {
            int colIndex = tblMdl.getColumnIndex(README_STATUS_COLUMN);
            colIndex++;
            return colIndex;
        } catch (IllegalArgumentException e) {
            return -1;
        }
    }

    private void removeToolBarButtons() {
        while (toolBar_.getComponentAtIndex(0) != null) {
            JButton c = (JButton) toolBar_.getComponentAtIndex(0);
            if (c != null)
                toolBar_.remove(c);

        }
    }

    public void finalize() throws Throwable {
        // This cannot use stdlog_.debug() as it gives the closed appender
        // error message from log4j
        System.out.println("FINALIZE - - ot.gui.QueueItemsBView - - FINALIZE");        
        super.finalize();
    }
    private void removeListeners() {
        getDbbView().removeListSelectionListener(listListener_);
        getDbbView().removeListSelectionListener(treeViewClearer_);
        QueueManager.removeOBListener(this);
    }

    public void makeComponentsAavailableForGC() {
        getDbbView().removeGlassPaneMouseListener();
        removeToolBarButtons();
        removeListeners();        
        executionAction_.unsubscribe();
    }

    public JFrame getParentFrame() {
        Container c = getDbbView().getTopLevelAncestor();
        stdlog_.debug("ot.gui.QueueItemsBView: " + c);
        return (JFrame) c;
    }

    public long[] getSelectedOpenQueueIds() {
        long[] ids = new long[1];
        ids[0] = queue.getId();
        return ids;
    }

    /** Builds the toolBar */
    protected void buildQueueToolBar() {
        toolBar_ = new JToolBar();
        toolBar_.add(displayAction);
        markButton = toolBar_.add(markAction_);
        MouseListener popupListener = new PopupListener();
        markButton.addMouseListener(popupListener);
        if (AppConfig.getAppConfig().getBoolean(OHSKeys.USGEnabledKey)) {
            markReadmeButton = toolBar_.add(markReadmeAction_);
            popupListener = new PopupReadmeStatusListener();
            markReadmeButton.addMouseListener(popupListener);
        }
        toolBar_.add(removeAction_);
        toolBar_.add(openCommentsAction_);
        toolBar_.add(moveUpAction);
        toolBar_.add(moveDownAction);
        toolBar_.add(executionAction_);
        toolBar_.add(copyActionListener_);
        toolBar_.add(pasteActionListener);
        toolBar_.add(exportAction_);
        toolBar_.add(obHistoryAction);
        toolBar_.add(findingChartAction);
    }

    /** Builds the toolBar */
    private void buildStandardOrangeQueueToolBar() {
    	toolBar_ = new JToolBar();
        toolBar_.add(executionAction_);
        toolBar_.add(copyActionListener_);
        toolBar_.add(exportAction_);
        toolBar_.add(obHistoryAction);
        toolBar_.add(findingChartAction);    	
    }


    /**
     * void
     */
    private void addFirstBunchButtons() {
    }

    /**
     * void
     */
    private void addMoveButtons() {
        toolBar_.add(moveUpAction);
        toolBar_.add(moveDownAction);
    }

    public void setEnabled(boolean enabled) {
        displayAction.setEnabled(enabled); // Sepetate action
        //removeAction_.setEnabled(enabled); // inline action
        if (AppConfig.getAppConfig().getBoolean(OHSKeys.USGEnabledKey)) {
            markAction_.setEnabled(enabled); // Seperate action
        }
        if (AppConfig.getAppConfig().getBoolean(OHSKeys.USGEnabledKey)) {
            markReadmeAction_.setEnabled(enabled); // Seperate action
        }
        if (!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
            openCommentsAction_.setEnabled(enabled); // Seperate Action
        }       
        deselectAllAction.setEnabled(enabled); // inline
        executionAction_.setEnabled(enabled);
        // must have reference to execSequence
        moveUpAction.setEnabled(enabled); // inline
        moveDownAction.setEnabled(enabled);
        copyActionListener_.setEnabled(enabled); // inline
        pasteActionListener.setEnabled(true);
        // should always be able to paste
        reload.setEnabled(enabled);
        exportAction_.setEnabled(enabled);
        obHistoryAction.setEnabled(enabled);
        findingChartAction.setEnabled(enabled);

    }

    public Action getReloadQueueItemsAction() {
        return reloadQueueItemsAction_;
    }

    public Action getExecutionAction() {
        return executionAction_;
    }

    public Action getRemoveAction() {
        return removeAction_;
    }

    public Action getMarkAction() {
        return markAction_;
    }

    public Action getOpenCommentsAction() {
        return openCommentsAction_;
    }

    public Action getExportAction() {
        return exportAction_;
    }

    /**
     * Return the action for use when buildig menus
     * 
     * @return Action
     */

    public Action getDumpAdpAdmDataAction() {
        return dumpAdpAdmDataAction_;
    }

    /**
     * Set the new Queue that will be displayed into the view
     */
    public void updateView(Queue newQueue) throws SQLException {
        queue = newQueue;
        queueIDTF.set(new Long(Config.getCfg().uniqueToTableId(queue.getId())));
        if (getDbbView().getSortPanel() != null) {
            getDbbView().getSortPanel().reset();
            seqWidget.actionPerformed(new ActionEvent(this, 0, null));
        }
        view.sqlStartAction();
    }

    /** Appends selected QueueItems to execution sequence */
    public void appendOBtoES() {
        ExecutionSequenceBView execView = ExecutionSequenceBView.getInstance();
        execView.appendOBs(view.getAllSelectedOBIds());
    }



    /**
     * Appends OBs to the view
     * 
     * @param obIds
     *            specifies the id of obs to be added to the view
     * 
     * NOTE: should never add OBs to a queue that have a status of "-" or "D",
     * this is however difficult to check here without loading the OB from the
     * database (SLOW!) It is assumed that that check has already been made (at
     * the Repository Browser level) and that all ids listed here can be safely
     * added to the queue.
     */
    public boolean appendOBs(long[] obIds) {
        Vector<Long> setOBIDs = new Vector<Long>();
        int option = -1;
        StringBuffer messageBuffer = new StringBuffer();
        int numFailed = 0;
        for (int j = 0; j < obIds.length; j++) {
        	stdlog_.debug("OB IDs " + obIds[j]);
            if (queue.inQueueItems(obIds[j])) {
                if (numFailed >= 1) {
                    messageBuffer.append("\n");
                }
                messageBuffer.append(Config.getCfg().uniqueToTableId(obIds[j]));
                numFailed++;
            } else if (setOBIDs.contains(new Long(obIds[j])) == false) {
                setOBIDs.addElement(new Long(obIds[j]));
                /** 03/12/01 cmavros Flag a change has happened to the Queue */
                queue.setUpdate(true);
            }
        }
        // if an OB must be appended
        if (setOBIDs.size() > 0) {
            try {
                long[] uniqueobIds = new long[setOBIDs.size()];
                for (int i = 0; i < uniqueobIds.length; i++) {
                    Long id = setOBIDs.elementAt(i);
                    uniqueobIds[i] = id.longValue();
                }
                QueueManager.appendObsToQueue(queue, queue.getAllUniqueOBIds(),
                        uniqueobIds, getDbbView().getRows());
                queue.appendQueueItems(uniqueobIds, ObjectManager.getObjectManager().getTypes(uniqueobIds));
                if (getDbbView().getSortPanel() != null) {
                    getDbbView().getSortPanel().reset();
                    seqWidget.actionPerformed(new ActionEvent(this, 0, null));
                }
                view.sqlStartAction();
            } catch (ObjectIOException e) {
                stdlog_.error(e);
            }
        }

        if (numFailed == 1) {
            String message = "OB " + messageBuffer.toString()
                    + " already inserted into the Queue";
            JOptionPane.showMessageDialog(null, message, "Warning",
                    JOptionPane.WARNING_MESSAGE);
        }
        if (numFailed > 1) {
            log_.clearTextArea();
            String message = "The following OBS are already inserted into the queue\n"
                    + messageBuffer.toString();
            log_.setTextArea(message);
            log_.setVisible(true);
        }
        if (setOBIDs.size() > 0) {
            return true;
        }
        return false;

    }
    
    
    /**
     * Delete all the qeuue Items from the  queue.
     */
    
    public void deleteQueueItems() {
    	//queue.setQueueList(new QueueItem[0]);
        try {
            QueueManager.deleteOBsFromQueueTable(
                    view.getAllOBIds(),
                    queue.getId(),
                    DbaseHandlerQueue.TMP_SCHEDULE_ITEMS);
        	//queue.setQueueList(new QueueItem[0]);

        } catch (ObjectIOException e) {
            stdlog_.error(e);
        }
    	
    }
    

    /**
     * Appends OBs to the view
     * 
     * @param obIds
     *            specifies the id of obs to be added to the view
     * 
     * NOTE: should never add OBs to a queue that have a status of "-" or "D",
     * this is however difficult to check here without loading the OB from the
     * database (SLOW!) It is assumed that that check has already been made (at
     * the Repository Browser level) and that all ids listed here can be safely
     * added to the queue.
     */
    public boolean appendQueueItems(QueueItem[] queueItems) {
        queue.setQueueList(queueItems);
        try {
            QueueManager.appendQueueItemsToQueue(queue);
        } catch (ObjectIOException e) {
            stdlog_.error(e);
        }
        if(getDbbView().getSortPanel() != null) {
            seqWidget.actionPerformed(new ActionEvent(this, 0, null));
        }
        view.sqlStartAction();        
        return true;
    }
    
    /**
     * Appends OBs to the view
     * 
     * @param obIds
     *            specifies the id of obs to be added to the view
     * 
     * NOTE: should never add OBs to a queue that have a status of "-" or "D",
     * this is however difficult to check here without loading the OB from the
     * database (SLOW!) It is assumed that that check has already been made (at
     * the Repository Browser level) and that all ids listed here can be safely
     * added to the queue.
     */
    public boolean deleteAndAppendQueueItems(QueueItem[] queueItems) {
        queue.setQueueList(queueItems);
        try {
            QueueManager.deleteAndInsertObs(queue);
        } catch (ObjectIOException e) {
            stdlog_.error(e);
        }
        if(getDbbView().getSortPanel() != null) {
            seqWidget.actionPerformed(new ActionEvent(this, 0, null));
        }
        view.sqlStartAction();        
        return true;
    }


   

    // ----------------------------------------
    // Private Methods
    // ----------------------------------------

    /**
     * Build DB browser view: SELECT panel
     * 
     * <br>
     * ChangeLog: Gorgio Passera
     * @param dbbView_2 
     * 
     * @exception SQLException
     *                JDBC exception occurs
     * @exception RemoteException
     *                RMI exception occurs
     * @exception MalformedURLException
     *                malformed url i.e. servername//port
     * @exception NotBoundException
     *                client could not bind to server
     */
    private void buildColumns() 
            throws NotBoundException, MalformedURLException, SQLException,
            RemoteException {
       
        // collect the default SQLChunks from super.
        String label_name;
        String column_name;

        label_name = "OB ID";
        column_name = "ob_id";
        obIDChunk = new DbbSqlChunk(BrowserConfig.ob, column_name, label_name);
        
        DbbSqlEngine engine = getDbbView().getSqlEngine(); 

        engine.addDefaultColumnsChunk(obIDChunk);

        // Status
        column_name = "status";
        label_name = OTDbbView.STATUS;
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.ob, column_name,
                label_name));

        column_name = "run_id";
        label_name = OTDbbView.RUN_ID;
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.sr, column_name,
                label_name));
        
        colPanel_ = new DbbPanel("Selected Columns", 7);
        DbbSqlChunk c;
        DbbCheckBox w;

        // OB Name
        label_name = "OB Name";
        column_name = "item_name";
        colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.ob, column_name, label_name)));
        
        
        column_name = "userComments";
        label_name = "OB Comment";
        c = new DbbSqlChunk(BrowserConfig.ob, column_name, label_name,CommentType.class);
        w = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(w);
        
        column_name = "inst_comments";
        label_name = "Inst Comment";
        c = new DbbSqlChunk(BrowserConfig.ob, column_name, label_name,CommentType.class);
        w = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(w);

        // ProgID
        label_name = "ProgID";
        column_name = "progid";
        colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.sr, column_name, label_name, String.class)));

        // PI (surname)
        label_name = "LastName";
        column_name = "last_name";
        c = new DbbSqlChunk(BrowserConfig.profiles, column_name, label_name,"rtrim(@column@)");
        w = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(w);

        // Target
        label_name = "Target";
        column_name = "item_name";
        c = new DbbSqlChunk(BrowserConfig.tp, column_name, label_name);
        w = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(w);

        // RA
        label_name = "RA";
        column_name = "ra";
        colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.tp, column_name, label_name, DbaseRA.class)));

        // Dec
        label_name = "Dec";
        column_name = "dec";
        colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.tp, column_name, label_name, DbaseDec.class)));

        // Instrument widget
        label_name = "Instrument";
        column_name = "instrument";
        c = new DbbSqlChunk(BrowserConfig.od, column_name, label_name);
        w = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(w);

        // User's priority
        label_name = "UsrP";
        column_name = "userPriority";
        c = new DbbSqlChunk(BrowserConfig.ob, column_name, label_name);
        w = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(w);

        // Template widget
        label_name = "Templ";
        column_name = "item_name";
        c = new DbbSqlChunk(BrowserConfig.ts, column_name, label_name);
        w = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(w);

        // Seeing
        label_name = "Seeing";
        column_name = "seeing";
        colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.cs, column_name, label_name)));

        // Sky Tran
        label_name = "SkyTran";
        column_name = "sky_transparency";
        colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.cs, column_name, label_name)));

        // Airmass
        label_name = "Airmass";
        column_name = "airmass";
        colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.cs, column_name, label_name)));

        // FLI
        label_name = "FLI";
        column_name = "fli";
        colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.cs, column_name, label_name)));

        // MoonDis
        label_name = "MoonDis";
        column_name = "moon_distance";
        colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.cs, column_name, label_name, MoonDistance.class)));

        // StrehlRatio
        label_name = "Strehl";
        column_name = "strehl_ratio";
        colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.cs, column_name, label_name)));

        label_name = "ExecTime";
        column_name = "execution_time";
        colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.ob, column_name, label_name, DbbExecutionTime.class)));

        // Filter
        // Optical elements widget
        label_name = "OptElem";
        column_name = "value";
        c = new DbbSqlChunk(BrowserConfig.pa, column_name, label_name);
        w = new DbbCheckBox(label_name, c);

        DbbSqlOperator op = null;
        try {
            op = new DbbSqlOperator(">");
        } catch (Exception e) {
            stdlog_.error(e);
        }
        column_name = "subtype";
        c = new DbbSqlChunk(BrowserConfig.pa, column_name, op, Integer.class);
        c.setValue("0");
        w.addSqlChunk(c);

        colPanel_.addWidget(w);

        // Rank class
        label_name = "RankClass";
        column_name = "rank_class";
        c = new DbbSqlChunk(BrowserConfig.sr, column_name, label_name);
        w = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(w);

        // QCGrade
        label_name = "QC Grade";
        column_name = "obs_grade";
        colPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.ob, column_name, label_name)));

        label_name = "Finding Charts";
        column_name = "fc_name";

        if (AppConfig.getAppConfig().getBoolean(OHSKeys.USGEnabledKey)) {
            label_name = "Readme Version";
            column_name = "version";
            c = new DbbSqlChunk(BrowserConfig.readme, column_name, label_name);
            w = new DbbCheckBox(label_name, c);
            colPanel_.addWidget(w);
            column_name = "status";
            c = new DbbSqlChunk(BrowserConfig.readme, column_name, README_STATUS_COLUMN);
            w = new DbbCheckBox(README_STATUS_COLUMN, c);
            colPanel_.addWidget(w);
        }

        label_name = "Sidereal Time Start";
        column_name = "start_interval";
        c = new DbbSqlChunk(BrowserConfig.sidereal_time_intervals_view, column_name, label_name,
                DbbHHMMDateTime.class);
        w = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(w);

        label_name = "Sidereal Time End";
        column_name = "end_interval";
        c = new DbbSqlChunk(BrowserConfig.sidereal_time_intervals_view, column_name, label_name,
                DbbHHMMDateTime.class);
        w = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(w);

        label_name = "Baseline";
        column_name = "baseline_name";
        c = new DbbSqlChunk(BrowserConfig.baselines, column_name, label_name);
        w = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(w);

        label_name = "Ephemeris File";
        column_name = "file_name";
        c = new DbbSqlChunk(BrowserConfig.ephemeris_file, column_name, label_name);
        w = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(w);

        // Mask Status
        /**
         * cmavros 06/Nov/01 Only display option if Masktracker is Enabled.
         */
        label_name = "Mask Status";
        column_name = "mask_status";
        maskStatusBox = new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.masks_table, column_name, label_name, MaskStatusType.class));
        colPanel_.addWidget(maskStatusBox);

        label_name = "Mask Slot Num";
        column_name = "mask_slot_num";
        maskSlotNumBox = new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.masks_table, column_name, label_name));
        colPanel_.addWidget(maskSlotNumBox);

        label_name = "Channel";
        column_name = "adp_index";
        maskChannelBox = new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.obsblk_masks_table, column_name, label_name));
        colPanel_.addWidget(maskChannelBox);

        // Type
        column_name = "type";
        label_name = OTDbbView.TYPE;
        DbbCheckBox typeCheckBox = new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.ob, column_name, label_name));
        colPanel_.addWidget(typeCheckBox);

        label_name = OTDbbView.QUEUE_ID;
        column_name = "sch_id";

        c = new DbbSqlChunk(BrowserConfig.si_tmp, column_name, label_name);
        DbbCheckBox queueIdWidget = new DbbCheckBox(label_name, c);
        colPanel_.addWidget(queueIdWidget);
        typeCheckBox.set("1");
        queueIdWidget.set("1");
        typeCheckBox.fireDbbWidgetAction();
        queueIdWidget.fireDbbWidgetAction();
        queueIdWidget.setVisible(false);
        typeCheckBox.setVisible(false);
        view.getDbbView().setColumnsPanel(colPanel_, "North");
        getDbbView().hideColumn(OTDbbView.QUEUE_ID);
        getDbbView().hideColumn(OTDbbView.RUN_ID);

        // dbbView_.hideColumn(dbbView_.TYPE);

    }

    /** Build DB browser view: rows selection panel */
    private void buildRows() {
        rowPanel_ = new DbbPanel("Selection Criteria", 5);
        DbbSqlChunk c;

        String label_name;
        String column_name;

        DbbSqlOperator op_equal = null;
        DbbSqlOperator op_min = null;
        DbbSqlOperator op_max = null;
        DbbSqlOperator op_like = null;
        DbbSqlOperator op_in = null;

        try {
            op_equal = new DbbSqlOperator("=");
            op_min = new DbbSqlOperator("<=");
            op_max = new DbbSqlOperator(">=");
            op_like = new DbbSqlOperator("like");
            op_in = new DbbSqlOperator("in");
        } catch (Exception e) {
            stdlog_.error(e);
        }

        // Queue ID
        label_name = "Queue ID";
        column_name = "sch_id";

        c = new DbbSqlChunk(BrowserConfig.si_tmp, column_name, label_name, op_equal,
                Integer.class);
        queueIDTF = new DbbTextField(label_name, c);
        rowPanel_.addWidget(queueIDTF);
        queueIDTF.set(new Long(Config.getCfg().uniqueToTableId(queue.getId())));
        queueIDTF.setEnabled(false);
        getDbbView().setWherePanel(rowPanel_, "North", false);

    }

    /**
     * Build DB browser view: SELECT panel
     * 
     * <br>
     * ChangeLog: Gorgio Passera
     * 
     * @exception SQLException
     *                JDBC exception occurs
     * @exception RemoteException
     *                RMI exception occurs
     * @exception MalformedURLException
     *                malformed url i.e. servername//port
     * @exception NotBoundException
     *                client could not bind to server
     */
    public void buildOrangColumns()
            throws NotBoundException, MalformedURLException, SQLException,
            RemoteException {

        // collect the default SQLChunks from super.
        String label_name;
        String column_name;

        label_name = "OB ID";
        column_name = "ob_id";
        obIDChunk = new DbbSqlChunk(BrowserConfig.ob, column_name, label_name);
        
        DbbSqlEngine engine = getDbbView().getSqlEngine(); 


        engine.addDefaultColumnsChunk(obIDChunk);
        // engine.addDefaultColumnsChunk(obIDMaskChunk);
        // obIDMaskChunk.setActive(false);
        // engine.addDefaultColumnsChunk(adpMaskChunk);
        // adpMaskChunk.setActive(false);

        // Status
        column_name = "status";
        label_name = "Status";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.ob, column_name,
                label_name));

        column_name = "run_id";
        label_name = OTDbbView.RUN_ID;
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.sr, column_name,
                label_name));

        colObsPanel_ = new DbbPanel("Selected Columns", 7);
        DbbSqlChunk c;
        DbbCheckBox w;

        // OB Name
        label_name = "OB Name";
        column_name = "item_name";
        colObsPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.ob, column_name, label_name)));

        column_name = "userComments";
        label_name = "OB Comment";
        c = new DbbSqlChunk(BrowserConfig.ob, column_name, label_name,CommentType.class);
        w = new DbbCheckBox(label_name, c);
        colObsPanel_.addWidget(w);
        
        column_name = "inst_comments";
        label_name = "Inst Comment";
        c = new DbbSqlChunk(BrowserConfig.ob, column_name, label_name,CommentType.class);
        w = new DbbCheckBox(label_name, c);
        colObsPanel_.addWidget(w);

        
        
        // ProgID
        label_name = "ProgID";
        column_name = "progid";
        colObsPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.sr, column_name, label_name, String.class)));

        // PI (surname)
        label_name = "LastName";
        column_name = "last_name";
        colObsPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.profiles, column_name, label_name)));

        // Target
        label_name = "Target";
        column_name = "item_name";
        c = new DbbSqlChunk(BrowserConfig.tp, column_name, label_name);
        w = new DbbCheckBox(label_name, c);
        colObsPanel_.addWidget(w);

        // RA
        label_name = "RA";
        column_name = "ra";
        colObsPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.tp, column_name, label_name, DbaseRA.class)));

        // Dec
        label_name = "Dec";
        column_name = "dec";
        colObsPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.tp, column_name, label_name, DbaseDec.class)));

        // Instrument widget
        label_name = "Instrument";
        column_name = "instrument";
        c = new DbbSqlChunk(BrowserConfig.od, column_name, label_name);
        w = new DbbCheckBox(label_name, c);
        colObsPanel_.addWidget(w);

        // User's priority
        label_name = "UsrP";
        column_name = "userPriority";
        c = new DbbSqlChunk(BrowserConfig.ob, column_name, label_name);
        w = new DbbCheckBox(label_name, c);
        colObsPanel_.addWidget(w);

        // Template widget
        label_name = "Templ";
        column_name = "item_name";
        c = new DbbSqlChunk(BrowserConfig.ts, column_name, label_name);
        w = new DbbCheckBox(label_name, c);
        colObsPanel_.addWidget(w);

        // Seeing
        label_name = "Seeing";
        column_name = "seeing";
        colObsPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.cs, column_name, label_name)));

        // Sky Tran
        label_name = "SkyTran";
        column_name = "sky_transparency";
        colObsPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.cs, column_name, label_name)));

        // Airmass
        label_name = "Airmass";
        column_name = "airmass";
        colObsPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.cs, column_name, label_name)));

        // FLI
        label_name = "FLI";
        column_name = "fli";
        colObsPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.cs, column_name, label_name)));

        // MoonDis
        label_name = "MoonDis";
        column_name = "moon_distance";
        colObsPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.cs, column_name, label_name, MoonDistance.class)));

        // StrehlRatio
        label_name = "Strehl";
        column_name = "strehl_ratio";
        colObsPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.cs, column_name, label_name)));

        label_name = "ExecTime";
        column_name = "execution_time";
        colObsPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.ob, column_name, label_name, DbbExecutionTime.class)));

        // Filter
        // Optical elements widget
        label_name = "OptElem";
        column_name = "value";
        c = new DbbSqlChunk(BrowserConfig.pa, column_name, label_name);
        w = new DbbCheckBox(label_name, c);

        DbbSqlOperator op = null;
        try {
            op = new DbbSqlOperator(">");
        } catch (Exception e) {
            stdlog_.error(e);
        }
        column_name = "subtype";
        c = new DbbSqlChunk(BrowserConfig.pa, column_name, op, Integer.class);
        c.setValue("0");
        w.addSqlChunk(c);

        colObsPanel_.addWidget(w);

        // Rank class
        label_name = "RankClass";
        column_name = "rank_class";
        c = new DbbSqlChunk(BrowserConfig.sr, column_name, label_name);
        w = new DbbCheckBox(label_name, c);
        colObsPanel_.addWidget(w);

        // QCGrade
        label_name = "QC Grade";
        column_name = "obs_grade";
        colObsPanel_.addWidget(new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.ob, column_name, label_name)));

        label_name = "Finding Charts";
        column_name = "fc_name";

        if (AppConfig.getAppConfig().getBoolean(OHSKeys.USGEnabledKey)) {
            label_name = "Readme Version";
            column_name = "version";
            c = new DbbSqlChunk(BrowserConfig.readme, column_name, label_name);
            w = new DbbCheckBox(label_name, c);
            colObsPanel_.addWidget(w);
            column_name = "status";
            c = new DbbSqlChunk(BrowserConfig.readme, column_name, README_STATUS_COLUMN);
            w = new DbbCheckBox(README_STATUS_COLUMN, c);
            colObsPanel_.addWidget(w);
        }

        label_name = "Sidereal Time Start";
        column_name = "start_interval";
        c = new DbbSqlChunk(BrowserConfig.sidereal_time_intervals_view, column_name, label_name,
                DbbHHMMDateTime.class);
        w = new DbbCheckBox(label_name, c);
        colObsPanel_.addWidget(w);

        label_name = "Sidereal Time End";
        column_name = "end_interval";
        c = new DbbSqlChunk(BrowserConfig.sidereal_time_intervals_view, column_name, label_name,
                DbbHHMMDateTime.class);
        w = new DbbCheckBox(label_name, c);
        colObsPanel_.addWidget(w);

        label_name = "Baseline";
        column_name = "baseline_name";
        c = new DbbSqlChunk(BrowserConfig.baselines, column_name, label_name);
        w = new DbbCheckBox(label_name, c);
        colObsPanel_.addWidget(w);

        label_name = "Ephemeris File";
        column_name = "file_name";
        c = new DbbSqlChunk(BrowserConfig.ephemeris_file, column_name, label_name);
        w = new DbbCheckBox(label_name, c);
        colObsPanel_.addWidget(w);

        // Mask Status
        /**
         * cmavros 06/Nov/01 Only display option if Masktracker is Enabled.
         */
        label_name = "Mask Status";
        column_name = "mask_status";
        maskStatusBox = new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.masks_table, column_name, label_name, MaskStatusType.class));
        colObsPanel_.addWidget(maskStatusBox);

        label_name = "Mask Slot Num";
        column_name = "mask_slot_num";
        maskSlotNumBox = new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.masks_table, column_name, label_name));
        colObsPanel_.addWidget(maskSlotNumBox);

        label_name = "Channel";
        column_name = "adp_index";
        maskChannelBox = new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.obsblk_masks_table, column_name, label_name));
        colObsPanel_.addWidget(maskChannelBox);

        // sequence
        label_name = OTDbbView.ORANG_RANK;
        column_name = "rank";

        c = new DbbSqlChunk(BrowserConfig.si_tmp, column_name, label_name);
        DbbCheckBox rankWidget = new DbbCheckBox(label_name, c);
        colObsPanel_.addWidget(rankWidget);

        // sequence
        label_name = OTDbbView.ORANG_RANK_HIDE;
        column_name = "rank";

        c = new DbbSqlChunk(BrowserConfig.si_tmp, column_name, label_name);
        DbbCheckBox rankWidgetHide = new DbbCheckBox(label_name, c);
        colObsPanel_.addWidget(rankWidgetHide);

        label_name = OTDbbView.QUEUE_ID;
        column_name = "sch_id";

        c = new DbbSqlChunk(BrowserConfig.si_tmp, column_name, label_name);
        DbbCheckBox queueIdWidget = new DbbCheckBox(label_name, c);
        colObsPanel_.addWidget(queueIdWidget);

        // Type
        column_name = "type";
        label_name = OTDbbView.TYPE;
        DbbCheckBox typeCheckBox = new DbbCheckBox(label_name, new DbbSqlChunk(
                BrowserConfig.ob, column_name, label_name));
        colObsPanel_.addWidget(typeCheckBox);

        getDbbView().setColumnsPanel(colObsPanel_, "North");
        setHideCol(typeCheckBox);
        setHideCol(queueIdWidget);
        setHideCol(rankWidgetHide);
        // dbbView_.hideColumn(OTDbbView.TYPE);
        getDbbView().hideColumn(OTDbbView.QUEUE_ID);
        getDbbView().hideColumn(OTDbbView.RUN_ID);
        getDbbView().hideColumn(OTDbbView.ORANG_RANK_HIDE);
    }

    /**
     * @param checkBox
     *            void
     */
    private void setHideCol(DbbCheckBox checkBox) {
        checkBox.set("1");
        checkBox.fireDbbWidgetAction();
        checkBox.setVisible(false);
    }

    /** Build DB browser view: sort selection panel */
    private void buildSort() {
        sortPanel_ = new DbbPanel("Sorting Criteria", 1);
        DbbSqlChunk c;
        DbbArrowButton w;
        String column_name;
        String label_name;

        // sequence
        label_name = "sequence";
        column_name = "sequence";

        c = new DbbSqlChunk(BrowserConfig.si_tmp, column_name, label_name);
        seqWidget = new DbbArrowButton(label_name, c, 1);
        sortPanel_.addWidget(seqWidget);
        seqWidget.actionPerformed(new ActionEvent(this, 0, null));

        // OB ID
        label_name = "OB ID";
        column_name = "ob_id";

        c = new DbbSqlChunk(BrowserConfig.ob, column_name, label_name);
        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        // OB NAME
        label_name = "OB Name";
        column_name = "item_name";
        c = new DbbSqlChunk(BrowserConfig.ob, column_name, label_name);
        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        // ////////////////////////
        label_name = "ProgID";
        column_name = "progid";

        c = new DbbSqlChunk(BrowserConfig.sr, column_name, label_name);

        // /////// It was a bug /////////
        // label_name = "ProgID";
        // column_name = "proposalID";

        // c = new DbbSqlChunk(op,
        // column_name,
        // label_name);

        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        // RA
        label_name = "RA";
        column_name = "ra";

        c = new DbbSqlChunk(BrowserConfig.tp, column_name, label_name);
        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        // DEC
        label_name = "Dec";
        column_name = "dec";

        c = new DbbSqlChunk(BrowserConfig.tp, column_name, label_name);
        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        // seeing
        label_name = "Seeing";
        column_name = "seeing";

        c = new DbbSqlChunk(BrowserConfig.cs, column_name, label_name);
        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        // Airmass
        label_name = "Airmass";
        column_name = "airmass";

        c = new DbbSqlChunk(BrowserConfig.cs, column_name, label_name);
        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        // FLI
        label_name = "FLI";
        column_name = "fli";

        c = new DbbSqlChunk(BrowserConfig.cs, column_name, label_name);
        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        // Moon Distance
        label_name = "MoonDis";
        column_name = "moon_distance";

        c = new DbbSqlChunk(BrowserConfig.cs, column_name, label_name);
        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        // Sky Trans
        label_name = "SkyTran";
        column_name = "sky_transparency";

        c = new DbbSqlChunk(BrowserConfig.cs, column_name, label_name);
        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        // RankClass
        label_name = "RankClass";
        column_name = "rank_class";
        c = new DbbSqlChunk(BrowserConfig.sr, column_name, label_name);
        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        // Execution Time
        // cmavros added 10/Sept/2001
        label_name = "ExecTime";
        column_name = "execution_time";
        c = new DbbSqlChunk(BrowserConfig.ob, column_name, label_name);
        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        label_name = "Finding Charts";
        column_name = "fc_name";
        c = new DbbSqlChunk(BrowserConfig.finds_view, column_name, label_name);
        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        label_name = "Mask Status";
        column_name = "mask_status";
        w = new DbbArrowButton(label_name, new DbbSqlChunk(BrowserConfig.masks_table,
                column_name, label_name));
        sortPanel_.addWidget(w);

        label_name = "Mask Slot Num";
        column_name = "mask_slot_num";
        w = new DbbArrowButton(label_name, new DbbSqlChunk(BrowserConfig.masks_table,
                column_name, label_name));
        sortPanel_.addWidget(w);

        label_name = "Channel";
        column_name = "adp_index";
        w = new DbbArrowButton(label_name, new DbbSqlChunk(BrowserConfig.obsblk_masks_table,
                column_name, label_name));
        sortPanel_.addWidget(w);

        label_name = "Sidereal Time Start";
        column_name = "start_interval";
        c = new DbbSqlChunk(BrowserConfig.sidereal_time_intervals_view, column_name, label_name);
        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        label_name = "Sidereal Time End";
        column_name = "end_interval";
        c = new DbbSqlChunk(BrowserConfig.sidereal_time_intervals_view, column_name, label_name);

        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        label_name = "Baseline";
        column_name = "baseline_name";
        c = new DbbSqlChunk(BrowserConfig.baselines, column_name, label_name);
        w = new DbbArrowButton(label_name, c);
        sortPanel_.addWidget(w);

        getDbbView().setSortPanel(sortPanel_, "West");
        getDbbView().setSortPanel(sortPanel_, "West", true);
    }

    /** Build DB browser view: sort selection panel */
    public void buildOrangSort() {
        sortObsPanel_ = new DbbPanel("Sorting Criteria", 1);
        DbbSqlChunk c;
        DbbArrowButton w;
        String column_name;
        String label_name;

        // sequence
        label_name = OTDbbView.ORANG_RANK;
        column_name = "rank";

        c = new DbbSqlChunk(BrowserConfig.si_tmp, column_name, label_name);
        seqWidget = new DbbArrowButton(label_name, c, 1);
        sortObsPanel_.addWidget(seqWidget);
        seqWidget.fireDbbWidgetAction();
        getDbbView().setSortPanel(sortObsPanel_, "West");
        getDbbView().setSortPanel(sortObsPanel_, "West", true);
        sortObsPanel_.setVisible(false);
    }

    /**
     * This class is uesd to bring up a popup menu PopupMenu should be displayed
     * when the rigth mouse button is pressed. In this case we determine the
     * coordinates of the click to see if its occurs or not on a tree node or
     * leaf
     */
    public class PopupListener extends MouseAdapter {
        JPopupMenu popup;

        public void mousePressed(MouseEvent e) {
            // if (e.isPopupTrigger())
            if (e.getID() == MouseEvent.MOUSE_PRESSED) {
                createPopupMenu();
                popup.show(e.getComponent(), e.getX(), e.getY());
            }
        }

        private void createPopupMenu() {
            popup = new JPopupMenu();
            for (int i = 0; i < CalibrationBlock.markLabels.length; i++) {
                final int j = i;
                JMenuItem crrItem = new JMenuItem(
                        CalibrationBlock.markLabels[i]);
                popup.add(crrItem);
                crrItem.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        markStatus(j);
                        // queue.setUpdate(true);
                    }
                });
            }
        }
    }

    /**
     * This class is used to bring up a popup menu PopupMenu should be displayed
     * when the rigth mouse button is pressed. In this case we determine the
     * coordinates of the click to see if its occurs or not on a tree node or
     * leaf
     */
    public class PopupReadmeStatusListener extends MouseAdapter {
        JPopupMenu popup;

        public void mousePressed(MouseEvent e) {
            // if (e.isPopupTrigger())
            if (e.getID() == MouseEvent.MOUSE_PRESSED) {
                createPopupMenu();
                popup.show(e.getComponent(), e.getX(), e.getY());
            }
        }

        private void createPopupMenu() {
            popup = new JPopupMenu();
            String[] items = Readme.statusValues_.keys();
            for (int i = 0; i < items.length; i++) {
                final int j = i;
                JMenuItem crrItem = new JMenuItem(items[i]);
                popup.add(crrItem);
                crrItem.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        ObjectManager objMgr = ObjectManager.getObjectManager();
                        long[] obIDs = view.getAllSelectedOBIds();
                        CalibrationBlock cb;
                        try {
                            cb = (CalibrationBlock) objMgr.getBusObj(
                                    Media.DBASE, obIDs[0], Config
                                            .getCfg()
                                            .getClassFromId(obIDs[0]));
                            ObservingRun obsRun = cb.getObsRun();
                            QueueView.markReadmeStatus(obsRun, j);
                            // setValueAt
                        } catch (ObjectNotFoundException e1) {
                            ErrorMessages.announceNoObject(view.getTopLevelAncestor(), e1);
                        } catch (ObjectIOException e1) {
                            ErrorMessages.announceIOError(view.getTopLevelAncestor(), e1);
                        } catch (IOException e1) {
                            ErrorMessages.announceIOError(view.getTopLevelAncestor(), e1);
                        }
                    }
                });
            }
        }
    }
    
    
    
    public QueueItem[] getQueueItems() {
        Vector<QueueItem> queueItemVec = new Vector<QueueItem>();
        long[] obIds = view.getAllOBIds();
        for (int i = 0; i < obIds.length; i++) {
        	Class c = Config.getCfg().getClassFromId(obIds[i]);
        	String type = CalibrationBlock.CALBLK_TYPE;
        	if(c.equals(ObservationBlock.class)) {
        		type = CalibrationBlock.OBSBLK_TYPE;
        	}        	    
            QueueItem queueItem =
                new QueueItem(
                        obIds[i],
                    queue.getId(),
                    i,type);
            queueItemVec.addElement(queueItem);
        }
        QueueItem[] queueItems = new QueueItem[queueItemVec.size()];
        queueItemVec.copyInto(queueItems);
        return queueItems;
    }



    protected class SortTerminatedListener implements DbbSortTerminatedListener {
    	public SortTerminatedListener() {
    		
    	}
        public void sortTerminatedAction(DbbSortTerminated ev) {

            getDbbView().getTable().saveSortCriteriaToModel();
            stdlog_
                    .debug("ot.gui.QueueItemsBView: !!!!!!! Setting sorted list");
            QueueItem[] qList = getQueueItems();
            long[] obIdList = new long[qList.length];
            for (int i = 0; i < qList.length; i++) {
                obIdList[i] = qList[i].getObId();
            }
            // queue.setQueueList(qList,destIndex);
            queue.sortQueueItems(obIdList);

            getDbbView().getTable().saveSortCriteriaToModel();
        }
    }

    private class ReloadActionListener implements ActionListener {
        QueueItemsBView qbView_;

        protected ReloadActionListener(QueueItemsBView qbView) {
            qbView_ = qbView;
        }

        public void actionPerformed(ActionEvent e) {
            ReloadQueueAction reloadAction = new ReloadQueueAction(qbView_,
                    getParentFrame(),"Revert");
            reloadAction.actionPerformed(e);
        }
    }

    /**
     * Marks the status of an Observation Block
     */
    private void markStatus(int i) {
        long[] OBIDs = view.getAllSelectedOBIds();
        ObjectManager objMgr = ObjectManager.getObjectManager();
        boolean obDequeueRequired = QueueView.isNotQueueableStatus(i);
        // If this status to be set requirs dequeuing the selected OB,
        // inform the user that this status requires all selected OBs to
        // be removed from Queues prior to setting the status - continue y/n?
        if (!obDequeueRequired
                || JOptionPane.showConfirmDialog(getParentFrame(),
                        "Setting OBs to status '"
                                + CalibrationBlock.markLabels[i]
                                + "'\nwill remove them from all queues."
                                + "\nDo you wish to continue?",
                        "Setting Status", JOptionPane.OK_CANCEL_OPTION,
                        JOptionPane.QUESTION_MESSAGE, null) == JOptionPane.YES_OPTION) {
            try {
                QueueView.markStatus(OBIDs, i);
            } catch (ObjectIOException e) {
                String message = "Error: " + e.getMessage();
                JOptionPane.showMessageDialog(getParentFrame(), message,
                        "ERROR", JOptionPane.ERROR_MESSAGE);
                stdlog_.error(e);
            } catch (ObjectNotFoundException e) {
                String message = "Error: " + e.getMessage();
                JOptionPane.showMessageDialog(getParentFrame(), message,
                        "ERROR", JOptionPane.ERROR_MESSAGE);
                stdlog_.error(e);
            }
        }
    }

    
    /**
     *Test if the View contains a specific obId
     *@param obgMgrObId specifies the ID of the OB to be checked 
     *@return true if a specific obId is contained into the view
     */

     public boolean containsOB(long obId) {
         boolean found = false;
         for (int i = 0; i < getDbbView().getRows(); i++) {
             Integer value = (Integer) getDbbView().getValueAt(i, OTDbbView.OB_ID);
             if (obId == value.longValue()){
                 found = true;
                 break;
             }
         }
         return found;
     }

    public void obPropertyChange(OBEvent event) {
        try {
            long[] objMgrObIds = event.getOBsId();
            for (int i = 0; i < objMgrObIds.length; i++) {
                long obId = Config.getCfg().uniqueToTableId(objMgrObIds[i]);
                if (containsOB(obId)) {
                    if (queue.isUpdate()) {
                        QueueManager.updateOBsSequences(view.getAllOBIds(),
                                queue.getId(),
                                DbaseHandlerQueue.TMP_SCHEDULE_ITEMS);
                    }
                    // fix for 2.7.2
                    if (!event.getValue().equals(DELETE_EVENT))
                        view.refreshViewsStatus(Config.getCfg().uniqueToTableId(objMgrObIds[i]), event.getValue());
                }
            }

        } catch (ObjectIOException e) {
            String message = "Error: " + e.getMessage();
            JOptionPane.showMessageDialog(null, message, "ERROR",
                    JOptionPane.ERROR_MESSAGE);
            stdlog_.error(e);
        }
    }


    public void updateView(CalibrationBlock object) {
        if (tree_ != null) {
            tree_.updateView(object);
        }
    }

    public OTDbbView getView() {
        return view;
    }

    public Queue getQueue() {
        return queue;
    }
    
    public void setQueue(Queue queue) {
        this.queue=queue;
    }
    
    public void setDbbQueueId() {
        queueIDTF.set(new Long(Config.getCfg().uniqueToTableId(queue.getId())));
    }



    public OBHistoryAction getObHistoryAction() {
        return obHistoryAction;
    }

    public void setObHistoryAction(OBHistoryAction obHistoryAction) {
        this.obHistoryAction = obHistoryAction;
    }

    public DisplayOBAction getDisplayAction() {
        return displayAction;
    }

    public void setDisplayAction(DisplayOBAction displayAction) {
        this.displayAction = displayAction;
    }

    public MoveUpRowsAction getMoveUpAction() {
        return moveUpAction;
    }

    public void setMoveUpAction(MoveUpRowsAction moveUpAction) {
        this.moveUpAction = moveUpAction;
    }

    public MoveDownRowsAction getMoveDownAction() {
        return moveDownAction;
    }

    public void setMoveDownAction(MoveDownRowsAction moveDownAction) {
        this.moveDownAction = moveDownAction;
    }

   
    /**
     * @return Returns the dbbView_.
     */
    public DbbView getDbbView() {
        return dbbView_;
    }

    public JPanel getOuterPanel() {
        return outerPanel;
    }

    public void setOuterPanel(JPanel outerPanel) {
        this.outerPanel = outerPanel;
    }

	public DeselectAllAction getDeselectAllAction() {
		return deselectAllAction;
	}

	public void setDeselectAllAction(DeselectAllAction deselectAllAction) {
		this.deselectAllAction = deselectAllAction;
	}

	public Action getDumpAdpAdmDataAction_() {
		return dumpAdpAdmDataAction_;
	}

	public void setDumpAdpAdmDataAction(Action dumpAdpAdmDataAction) {
		this.dumpAdpAdmDataAction_ = dumpAdpAdmDataAction;
	}

	public ExecutionSequenceAction getExecutionAction_() {
		return executionAction_;
	}

	public void setExecutionAction_(ExecutionSequenceAction executionAction_) {
		this.executionAction_ = executionAction_;
	}

	
	public void setExportAction_(Action exportAction) {
		this.exportAction_ = exportAction;
	}

	public ShowFindingChartsAction getFindingChartAction() {
		return findingChartAction;
	}

	public void setFindingChartAction(ShowFindingChartsAction findingChartAction) {
		this.findingChartAction = findingChartAction;
	}

	public Action getMarkAction_() {
		return markAction_;
	}

	public void setMarkAction(Action markAction) {
		this.markAction_ = markAction;
	}

	public JButton getMarkButton() {
		return markButton;
	}

	public void setMarkButton(JButton markButton) {
		this.markButton = markButton;
	}

	public Action getMarkReadmeAction_() {
		return markReadmeAction_;
	}

	public void setMarkReadmeAction(Action markReadmeAction) {
		this.markReadmeAction_ = markReadmeAction;
	}

	public JButton getMarkReadmeButton() {
		return markReadmeButton;
	}

	public void setMarkReadmeButton(JButton markReadmeButton) {
		this.markReadmeButton = markReadmeButton;
	}

	public CopyToClipboardAction getCopyActionListener() {
		return copyActionListener_;
	}

	public void setCopyActionListener(CopyToClipboardAction copyActionListener) {
		this.copyActionListener_ = copyActionListener;
	}

	public PasteToClipboardAction getPasteActionListener() {
		return pasteActionListener;
	}

	public void setPasteActionListener(PasteToClipboardAction pasteActionListener) {
		this.pasteActionListener = pasteActionListener;
	}

	public TableListSelectionListener getListListener() {
		return listListener_;
	}

	public void setListListener(TableListSelectionListener listListener) {
		this.listListener_ = listListener;
	}



    
    
}
