package org.eso.ohs.phase2.apps.ot.gui;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.ot.ObjectTableListener;

/**
 * P2PP-specific table model: data corresponds to a subset of the
 * summary info in a Folder.
 * <br>
 * $Id: QueueSummaryModel.java,v 1.12 2006/04/25 16:39:50 pnunes Exp $
 * <br>
 * @version $Revision: 1.12 $
 */
public class QueueSummaryModel
    extends AbstractTableModel
    implements ObjectTableListener, PropertyChangeListener {

    public static final String cvsID_ =
        "$Id: QueueSummaryModel.java,v 1.12 2006/04/25 16:39:50 pnunes Exp $";

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(QueueSummaryModel.class);

    /** Contents of the model: a vector of Summary objects */
    private Vector<Summary> contents_ = new Vector<Summary>();

    /** Property names for the columns stored of this model */
    private String[] colProps_;

    /** Display names for the columns stored of this model */
    private String[] colNames_;


    /**
     * Default constructor
     */
    public QueueSummaryModel() {}


    /**
     * Constructor
     *
     * @param objType   type of object to be stored in the table.
     * @param isReadOnly        should all entries in the table be readonly?
     */
    public QueueSummaryModel(Summary[] contents) {
        colProps_ = AppConfig.getAppConfig().folderColumnProperties(Queue.class);
        colNames_ = AppConfig.getAppConfig().folderColumnNames(Queue.class);
        for (int i = 0; i < contents.length; i++) {
            contents_.addElement(contents[i]);
        }
    }


    /**
      * Reorder the summary objects stored in this model.
      *
      * @param objType  type of object to be stored in the table.
      * @param isReadOnly       should all entries in the table be readonly?
     */
    public void reorderContents(int indexes[]) {
        Vector<Summary> contents = new Vector<Summary>();
        for (int i = 0; i < indexes.length; i++) {
            Summary summ = contents_.elementAt(indexes[i]);
            contents_.addElement(summ);
        }
        contents_.removeAllElements();
        contents_ = contents;
        fireTableDataChanged();
    }


    /**
     * Returns the value of the property displayed in the given row
     * and column.  The row number determines the summary object and
     * the column determines the property.
     * NB: model columns may not be the same as table columns.
     *
     * @param   row     row of the model
     * @param   column  column of the model
     *
     * @return  current value of the object's property
     */
    public Object getValueAt(int row, int column) {
        Summary queueSumm = contents_.elementAt(row);
        if (column == 0) {
            Long id = (Long)queueSumm.getPropertyValue(colProps_[column]);
            Long dbId = new Long(Config.getCfg().uniqueToTableId(id.longValue()));
            return dbId;
        }
        return queueSumm.getPropertyValue(colProps_[column]);
    }


    public long getQueueId(int row) {
        Long id = (Long)getValueAt(row, 0);
        //stdlog_.debug("@@@@@@@@@@@ We are here");
        return Config.getCfg().tableToUniqueId(id.longValue(), getObjectClass(row));
        //return new Long(id.intValue()).longValue();
    }


    public long getDbaseId(int row) {
        Long id = (Long)getValueAt(row, 0);
        //stdlog_.debug("@@@@@@@@@@@ We are here");
        return id.longValue();
        //return new Long(id.intValue()).longValue();
    }


    public String getQueueName(long queueId) {
        int row = getRowFromQueueId(queueId);
        if (row != -1)
            return (String)getValueAt(row, 1);
        return "";
    }


    public Class getQueueClass(int row) {
        String type = (String)getValueAt(row, 4);
        if (type.equals("M"))
            return MaskQueue.class;
        return Queue.class;
    }


    public int getRowFromQueueId(long queueId) {
        for (int i = 0; i < contents_.size(); i++) {
            Summary queueSumm = contents_.elementAt(i);
            if (((Long)queueSumm.getPropertyValue("QueueId")).longValue()
                == queueId) {
                return i;
            }
        }
        return -1;
    }


    public void propertyChange(PropertyChangeEvent ev) {
        if (ev.getPropertyName() == null) {
            return;
        }

        Queue eventQueue = (Queue)ev.getSource();
        int row = getRowFromQueueId(eventQueue.getId());
        if (row == -1)
            return;
        if (ev.getPropertyName() != null
            && ev.getPropertyName().equals("name")
            || ev.getPropertyName().equals("QIListSize")
            || ev.getPropertyName().equals("Size")
            || ev.getPropertyName().equals("telescope")) {
            Summary queueSumm = contents_.elementAt(row);
            Integer size = (Integer)queueSumm.getPropertyValue("Size");
            if (!queueSumm.getPropertyValue("Name").equals(eventQueue.getName())
                || size.intValue() != eventQueue.getSize()
                || !eventQueue.getTelescope().equals(
                    queueSumm.getPropertyValue("Telescope"))) {
                queueSumm.setPropertyValue("Name", eventQueue.getName());
                queueSumm.setPropertyValue("Size", new Integer(eventQueue.getSize()));
                queueSumm.setPropertyValue("Telescope", eventQueue.getTelescope());
                fireTableDataChanged();
            }
        }
    }


    /**
     * Returns the summary object displayed in the given row.
     *
     * @param   row     row of the model
     * @return  current value of the selected summary object
     */
    public Summary getObjectAt(int row) {
        return contents_.elementAt(row);
    }


    public Class getObjectClass(int row) {
        return getObjectAt(row).getObjectClass();
    }


    /**
     * Returns the summary object displayed in the given row.
     *
     * @param   row     row of the model
     * @return  current value of the selected summary object
     */
    public String getColumnName(int col) {
        return colNames_[col];
    }


    /**
     * Returns the number of columns displayed in the model,
     * including non-displayed columns.
     *
     * @return  number of columns in the table
     */
    public int getColumnCount() {
        String[] colProps =
            AppConfig.getAppConfig().folderColumnProperties(Queue.class);
        return colProps.length;
    }


    /**
     * Returns the number of rows displayed in the model, ie
     * the number of summary objects in the model.
     *
     * @return  number of summary objects in the table
     */
    public int getRowCount() {
        return contents_.size();
    }


    /**
     * Editable if table is not read-only and col is editable
     * and BO is not checked in.
     *
     * @return  true if cell is editable, false otherwise
     */
    public boolean isCellEditable(int row, int column) {
        return false;
    }


    public void removeElementAt(int row) {
        contents_.removeElementAt(row);
        fireTableStructureChanged();
    }


    public void insertElementAt(Summary summ, int row) {
        contents_.insertElementAt(summ, row);
    }


    /**
     * A new business object was added to the folder summary information.
     * Event handler implementing ObjectTableListener.
     *
     * @param ev        description of affected object
     */
    public void tableObjectAdded(SummaryTableEvent ev) {
        Summary summ = ev.getQueueSummary();
        Long lid = (Long)summ.getPropertyValue("QueueId");
        //stdlog_.debug("Id = " + lid);
        //long id = ((Long)summ.getPropertyValue("QueueId")).longValue();
        int row = getRowFromQueueId(lid.longValue());
        long id = lid.longValue();
        if (row == -1) {
            //org.eso.ohs.utilities.DebugUtils.stackTrace();
            insertElementAt(summ, 0);
            fireTableStructureChanged();
        }
        ObjectManager objMgr_ = ObjectManager.getObjectManager();
        try {
            Class c = Config.getCfg().getClassFromId(id);
            Queue queue = (Queue)objMgr_.getBusObj(Media.DBASE, id, c);
            //stdlog_.debug("!!!! @@@@@@@@ Summ Size " + summ.getSize());
            //queue.setSize(summ.getSize());
            queue.addPropertyChangeListener(this);
        } catch (ObjectNotFoundException e) {
            stdlog_.error(e);
        } catch (ObjectIOException e1) {
            e1.printStackTrace();
        }
    }


    /**
     * A business object in the folder summary information was changed.
     * Event handler implementing ObjectTableListener.
     *
     * @param ev        description of affected object
     */
    public void tableObjectChanged(SummaryTableEvent ev) {
        Summary summ = ev.getQueueSummary();
        Long lid = (Long)summ.getPropertyValue("QueueId");
        int row = getRowFromQueueId(lid.longValue());
        if (row != -1) {
            contents_.setElementAt(summ, row);
            fireTableDataChanged();
        }
    }


    /**
     * A business object was removed from the folder summary information.
     * Event handler implementing ObjectTableListener.
     *
     * @param ev        description of affected object
     */
    public void tableObjectRemoved(SummaryTableEvent ev) {
        Summary summ = ev.getQueueSummary();
        //stdlog_.debug("Summary " + summ);

        Long lid = (Long)summ.getPropertyValue("QueueId");
        int row = getRowFromQueueId(lid.longValue());
        if (row != -1) {
            removeElementAt(row);

        }
    }
}

