package org.eso.ohs.phase2.apps.ot.icdVcs;

import java.awt.Component;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.OutOfMemoryReport;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.icdVcs.IcdVcsCfg;

/**
 * Auxiliary class to check that an OB/OD/TG unit is valid,
 * eg there is a Target (TG).  The run method for this class
 * runs in the EventQueue thread, so that it can perform Swing
 * operations, eg display dialog boxes for errors.
 * 
 * <p>
 * $Id: OTOBChecker.java,v 1.2 2006/04/25 16:36:00 pnunes Exp $
 * <br>
 * @author Giorgio Passera
 * @author Bob Kemp
 * @author Tim Canavan
 */
public class OTOBChecker implements Runnable {

    private Queue queue; /*current Queue managed by bob*/

    /** null or a reference to the last OB approved for transfer to BOB */
    private CalibrationBlock approvedBO_ = null;
    private Vector<CalibrationBlock>           approvedBOs = null; //a vector of approved OBs for transfer to BOB
    private int           num; // Max number of OBs to be transfered to BOB

    private boolean offline_ = false;

    private String status_;

    private Vector obIdErrorList_ = new Vector();

    /**
     * Default constructor.
     * 
     * @param fv	FolderGridView containing the selected OB
     */
    public OTOBChecker(int maxNumOBs) {
        num = maxNumOBs;
    }


    public OTOBChecker(int maxNumObs,boolean offline,String status) {
        offline_  = offline;
        num = maxNumObs;
        status_ = status;
    }



    public long [] getOBIDErrorList() {
        long [] obIdList  = new long[obIdErrorList_.size()];
        for(int i=0;i<obIdErrorList_.size();i++) {
            obIdList[i] = (((Long)obIdErrorList_.elementAt(i))).longValue();
        }
        return obIdList;
    }   


    public CalibrationBlock getOB() {
	return approvedBO_;
    }



    /**Retrun a 0 size length*/
    public CalibrationBlock[] getOBs()  {
        CalibrationBlock[] returnList = new CalibrationBlock[approvedBOs.size()];
        for (int i=0; i<approvedBOs.size(); i++) {
            returnList[i] = approvedBOs.elementAt(i);
        }
        return returnList;
    }

    public Vector getErrorList() {
        return obIdErrorList_;        
    }  



    /**
     * This method must only be run from the System Event Queue Thread !!
     * It manipulates the GUI and calls ObjectManager, neither of which
     * are Thread-safe.
     */
    
   public void run() {
       try {
           ObjectManager objMgr = ObjectManager.getObjectManager();
           IcdVcsCfg cfg = IcdVcsCfg.getCfg();      
           Component comp = cfg.getContainer();
           approvedBOs    = new Vector<CalibrationBlock>();
           long[] obIdList        = null;
           CalibrationBlock  cb   = null;
           DirectoryNode     node = null;
           char [] maskStatuses = {};
           try {
               try {      
                   obIdList   = cfg.getSelections(num);   	
               }
               catch(NullPointerException e) {
                   showErrorMsg(e.getMessage());
                   return;
               }
 
               if (obIdList == null || obIdList.length == 0) {
                   String message = "The Execution Sequence is empty";
                   showWarningMsg1(message);
                   return;	    // There is no OB: into the execution qeue !
               }

               for (int i=0; i<obIdList.length;i++) {
                   long obId = obIdList[i];
                   Class objType =
                   Config.getCfg().getClassFromId(obId);
                   if(offline_)
                        objMgr.refreshBusObjRecursive(Media.DBASE,obId,objType);
                   cb = (CalibrationBlock)objMgr.getBusObj(Media.DBASE,obId,objType);
                   String errs = cb.verify();
                   if (errs.length() != 0) {
                       int result = showErrorMsg2(errs);
                       if (result == JOptionPane.NO_OPTION
                           || result == JOptionPane.CLOSED_OPTION) {
                           continue;
                       }
                   }          
                   if(offline_) {
                       /*String dbStatus = null;
                       try {
                           dbStatus  = DBUtil.getMaskStatus(Config.getCfg().uniqueToTableId(obId));
                       }
                       catch(SQLException e1) {
                           e1.printStackTrace();
                       }            
                       int returnValue = JOptionPane.YES_OPTION;
                       if(status_.equals(Mask.PENDING_MANUFACTURED)  && !dbStatus.equals(Mask.NO_MASK)) {
                             obIdErrorList_.addElement(new Long(obIdList[i]));
                       }
                       if(status_.equals(Mask.PENDING_INSERTION)  && !dbStatus.equals(Mask.MASK_MANUFACTURED)){
                             obIdErrorList_.addElement(new Long(obIdList[i]));
                       }
                       if(status_.equals(Mask.PENDING_DISCARDED)  && !dbStatus.equals(Mask.MASK_MANUFACTURED)) {
                             obIdErrorList_.addElement(new Long(obIdList[i]));
                       }
                       if (returnValue != JOptionPane.YES_OPTION) {
                           continue;
                           }*/
                 
                   }
                   cb.setStatus(CalibrationBlock.INITIATED);
                   objMgr.queueObject(cb);
                   approvedBOs.addElement(cb);
               }
          }
	  catch (ObjectNotFoundException e) {
              e.printStackTrace();
              ErrorMessages.announceNoObject(comp, e);
              return;	// Failure: return to caller
          }	    
          catch (ObjectIOException e) {
              e.printStackTrace();
              ErrorMessages.announceIOError(comp, e);
              return;	// Failure: return to caller
          }     
       }
       catch (OutOfMemoryError e) {
           e.printStackTrace();
           OutOfMemoryReport.showMessage();
           throw e;
       }
   }

   private void showErrorMsg1(String message) {
       JOptionPane.showMessageDialog(null,message,"Error",JOptionPane.ERROR_MESSAGE);

   }


   private void showWarningMsg1(String message) {
       JOptionPane.showMessageDialog(null,message,"Warning",JOptionPane.WARNING_MESSAGE);

   }
  

   private void showErrorMsg(String message) {
       IcdVcsCfg cfg = IcdVcsCfg.getCfg();
       Component comp = cfg.getContainer();
       JOptionPane.showMessageDialog(comp,new Object[] {
			  "There is no queue selected " +
			  " You must first select a Queue from the Open Queues Tab Pane"
			},
			"Nothing selected",
			JOptionPane.ERROR_MESSAGE);

   }


    private int showErrorMsg2(String errs) {
        IcdVcsCfg cfg = IcdVcsCfg.getCfg(); 
        Component comp = cfg.getContainer();
  
        return JOptionPane.showOptionDialog(comp,new Object[] {"One or more errors were found:",errs," ",
				      "Should the object"
				       + " be sent to the telescope software","despite the errors?"},
				      "Error",JOptionPane.YES_NO_OPTION,JOptionPane.ERROR_MESSAGE,
				      null,null,"No"); 
    }


    private int showStatusMsg(String msg) {
        IcdVcsCfg cfg = IcdVcsCfg.getCfg();    
        Component comp = cfg.getContainer();
  
        return JOptionPane.showOptionDialog(comp,new Object[] {msg},
				      "Error",JOptionPane.YES_NO_OPTION,JOptionPane.ERROR_MESSAGE,
				      null,null,"No"); 
   }

}


