## -*- ksh -*-
## $Id: $

## Site configuration file for use at Paranal and La Silla Observatories.
##

## Enable CCS for communication with BOB

CCS.CONNECTION.ENABLED "true"

##------------------------------------------------------------
## The following configuration items are 
## meaningful for:
##    OT            : All keywords
##    P2PP AppServer: All keywords
##    P2PP          : SINGLEIPMODE, IPCACHE.FOLDER keywords.
##------------------------------------------------------------

## OT, P2PP(Operations mode) and AppServer Configuration Combinations:
##
##  SINGLE |  AUTO    |  IP CACHE |
## IP MODE | DOWNLOAD |   FOLDER  |
##-----------------------------------------------------------
##         |          |           |   Load OBs with the IP version of the OB
##    n    |    n     | optional  |   If the IP does not exist in the local
##         |          |           |   IP cache, do not load the OB.
##         |          |           |
##         |          |           |   Load OBs with the IP version of the OB
##    n    |    y     | required  |   If the IP does not exist, attempt to
##         |          |           |   download the IP from the database.
##         |          |           |
##         |          |           |   Load OBs with the latest IP version
##    y    |    n     | optional  |   for that instrument (ignores IP version
##         |          |           |   of the OB.
##         |          |           |
##    y    |    y     | required  |   INVALID.
##-------------------------------------------------------------
##
## P2PP(non-Operations Mode) Configuration Combination:
##
##  SINGLE |  IP CACHE |  
## IP MODE |   FOLDER  |
##------------------------------------------------
##         |           |   Load OBs with the IP version of the Obs Run
##    n    | required  |   If the IP does not exist in the local
##         |           |   IP cache when Obs Run is downloaded, IP is
##         |           |   downloaded with ObsRun.
##         |           |
##         |           |   Load OBs with the latest IP version
##    y    | optional  |   for that instrument (ignores IP version
##         |           |   of the OB or ObsRun. No downloading
##         |           |   IPs.
##         |           |
##-------------------------------------------------------------
##

## This keyword sets OHS applications into a single IP version mode, where
## the latest IP versions for each instrument are used.
## This mode should be set to "y" for OHS applications running at "the mountain".
##
## If enabled, INSTRUMENTS.IPVERSIONING.IPCACHE.AUTODOWNLOAD must be disabled.
##
## Warning: this setting determines the mode of this application and
## should be correctly set at installation time. Do not modify this setting.
##
INSTRUMENTS.IPVERSIONING.SINGLEIPMODE   "y"

## This keyword enables the automatic download of IPs as required, from the
## IP database.
## This is typically enabled for OT and AppServer when running at Garching.
##
## If enabled, the INSTRUMENTS.IPVERSIONING.IPCACHE.FOLDER must be configured.
## If enabled, INSTRUMENTS.IPVERSIONING.SINGLEIPMODE must be disabled.
##
## Warning: this setting determines the mode of this application and
## should be correctly set at installation time. Do not modify this setting.
##
INSTRUMENTS.IPVERSIONING.IPCACHE.AUTODOWNLOAD "n"


## FOLDER config setting determining where IPs are installed when
## downloaded from the database by an application.
##
## If Automatic IP download is enabled, this keyword must be set.
##
## Warning: this setting determines the mode of this application and
## should be correctly set at installation time. Do not modify this setting.
##
## INSTRUMENTS.IPVERSIONING.IPCACHE.FOLDER "cache/instruments"


## INSTRUMENTS.FOLDER
## Pathname of the directory where the Instrument Packages are
## installed; it defaults to instruments.
##
INSTRUMENTS.FOLDER "/home/instmgr/instruments"

## Part of the Multiple Instruments Package project:
## This property, when enabled ('y') will only permit checking-in of
## OBs if the version of the IP used in the OB matches the required IP version
## in the Observing Run. This config item has relevance at the database level,
## and so will affect the following applications:
## - P2PP (in direct mode ONLY).
## - AppServer.
##
# IPVERSIONING.CHECKIN.IPVERSIONCHECK "y"


## FONT.SIZE.OFFSET
## Can be used to increase or decrease the font size used on
## the GUIs.
## Range: Integer values between -4 and 4 
#FONT.SIZE.OFFSET   "2"

## SCRIPTING.DRIVER.n
## Script interpreters to be loaded. All entries must be valid and at
## least one is required to initialize the scripting extensions.
## This keyword should *never* be changed!
SCRIPTING.DRIVER.1 "org.eso.ohs.scripting.JaclScriptInterpreter"

## SCRIPTING.SETUP.n
## Location of specific API setup scripts; they are not required for
## the scripting extensions themselves.
## This keyword should *never* be changed!
##
SCRIPTING.SETUP.1  "lib/scriptAPIsetup.tcl"

## Python scripting is not available
## SCRIPTING.DRIVER.2 "org.eso.ohs.scripting.JPythonScriptInterpreter"
## SCRIPTING.SETUP.2  "lib/scriptAPIsetup.tcl"

############################
## ORANG settings

## ORANG scripting setup file
ORANG.SETUP "lib/orangAPIsetup.tcl"

## ORANG scripting setup dir
ORANG.SCRIPTDIR "/home/instmgr/config"

## the range in minutes  from one visibility computation to another
ORANG.STEP.RANGE "10 20 30 60"

## The kind of panel to start at startup. the possible options are:
## "Wizard" "Standard" "Small"
ORANG.PANEL.TYPE "Small"

## Start orang at startup or not: "Y" or "N"
ORANG.PANEL.STARTUP "Y"

## the total orang visibility time computation in hour. Must be an integer. There is an "All Night" by default 
ORANG.DURATION.RANGE "1 2 4"

## Observable Queue name
ORANG.OBSERVABLE.NAME "Orang observable"

# Non observable Queue name 
ORANG.NOTOBSERVABLE.NAME "Orang not observable"

## pattern for retreving the today queues
ORANG.TODAY.INST.PATTERN "SMTS\\.(\\w+)\\.(\\w+)\\.TODAY"

## group index of the instrument element
ORANG.TODAY.INST.INDEX "1"

## default end date in inst_tel table in obrep2
ORANG.TODAY.ENDDATE "1-1-2025"


## telescope elevation constraint in degrees (around zenith)
TELESCOPE.ELEVATION.CONSTRAINT 5

##Minimum angle between the apparent target position (azimuth)
##  and wind direction (defaults to 90 degrees)
ORANG.WIND.CONSTRAINT.ANGLE 90


## delay in the start time for visibility computation (minutes)
ORANG.STARTTIME.DELAY 5


##------------------------------------------------------------
## The following configuration items are only 
## meaningful for P2PP
##------------------------------------------------------------

##
## P2PP Setting
## If set to "Y" will present the user with an "ok" dialog
## when an OB (or CB) has been converted from one IP version to another. If not set
## or set to "N", no dialog is presented. For converting a large number of OBs, it
## is suggested that this setting be commented out or set to "N".
P2PP.IPVERSIONING.SHOWCONVERSION "Y"


## List of instruments that are known to have no IP.
## No attempt will be made (by P2PP) to download these IPs
## and install them.
## The list is a case sensitive list of instrument names separated with ";" characters.
INSTRUMENTS.WITHNOIP "1.3/3.0Rec;1.3Bolo;2.0/3.0Rec;ADONISSH;AdonisCO;AdonisSH;CASPECL;CAT;CassBCSpec;DFOSC;Special"

## CACHE.FOLDER
## Pathname of the directory/folder where cache files and
## directories can be created; it defaults to cache; 
CACHE.FOLDER       "cache"

## IMPEX.FOLDER
## Pathname of the directory where import/export files can be created
## (note that this pathname can be altered interactively at run time);
## it defaults to impex. 
IMPEX.FOLDER       "impex"

## PARAMFILE.FOLDER
## Pathname of the directory where files in PAF (PArameter File)
## format are stored. When assigning a value to a parameter
## of type paramfile, this directory will be initially searched. 
## It defaults to the user's home directory. 
# PARAMFILE.FOLDER ""

## APPSERVER.HTTPPROXYHOST and APPSERVER.HTTPPROXYPORT
## If these keywords are defined, P2PP will try to connect to its
## Application Server via an HTTP proxy server as specified by those
## keywords.. 
## It is normally not necessary to uncomment the following lines.
## Only sites with strict firewall policies will need to use
## an HTTP proxy server; other sites may rely on the default RMI
## connection.
#APPSERVER.HTTPPROXYHOST "phase2.hq.eso.org"
#APPSERVER.HTTPPROXYPORT "80"


## APPSERVER.CHECKIN.PERIODS:
## Set of periods for which the app server will check-in OBs/CBs.
## OBs/CBs from other periods will not be checked-in - error message
## is returned to the user.
## If empty or commented out, OBs/CBs from all periods may be checked-in.
## Example, for restricting appserver OB/CB check-in to period 70 and 71:
## APPSERVER.CHECKIN.PERIODS "70, 71"
# APPSERVER.CHECKIN.PERIODS ""

##
## P2PP:
## The set of attributes of the Target and ConstraintSet objects that are
## read from param files attached to templates. The keywords list the
## parameter names that are read from any param file attached.
## 
## The Parameter names may be numbered, to allow multiple param files to be
## handled, for example, in the following definition, RA may be read from a
## param file if it matches either of the parameters specified:
## 
## TARGET.RA.PARAM.1 "TEL.TARG.ALPHA"
## TARGET.RA.PARAM.2 "TEL.TARG.RA"
## 
## In the above example, if TEL.TARG.ALPHA will be taken from the param
## file (if present), before TEL.TARG.RA, due to the numbering of the
## parameters: 1 and 2.
## 
## An example that does not used numbering: to read the SIM.PERF.AM parameter
## from a param file and set it to the airmass of a ConstraintSet:
## CONSTRAINTSET.AIRMASS.PARAM     "SIM.PERF.AM"
##
##
## Constraint Set properties read from param files:
##

CONSTRAINTSET.AIRMASS.PARAM.1   "SIM.ATM.AIRMASS"
CONSTRAINTSET.SEEING.PARAM.1    "SIM.ATM.ZSEEING"
## CONSTRAINTSET.FLI.PARAM ""
## CONSTRAINTSET.MOON.PARAM ""
## CONSTRAINTSET.SKY.PARAM ""
## CONSTRAINTSET.TELTEMP.PARAM ""
## CONSTRAINTSET.VAPOUR.PARAM ""
##
## The parameter name extracted from Param files for populating
## the Strehl Ratio constraint in P2PP.
## This parameter could also have a ".1" suffix (CONSTRAINTSET.STREHLRATIO.PARAM.1)
## but is left in this form for backwards compatibility (both work).
##
CONSTRAINTSET.STREHLRATIO.PARAM "SIM.PERF.ALTSTREHL"

##
## Target properties read from param files:
##

TARGET.RA.PARAM.1        "TEL.TARG.ALPHA"
TARGET.RA.PARAM.2        "TEL.TARG.RA"
	 
TARGET.DEC.PARAM.1       "TEL.TARG.DELTA"
TARGET.DEC.PARAM.2       "TEL.TARG.DEC"

TARGET.EQUINOX.PARAM.1   "TEL.TARG.EQUINOX"
TARGET.EPOCH.PARAM.1     "TEL.TARG.EPOCH"
TARGET.PROPRA.PARAM.1    "TEL.TARG.PROPRA"
TARGET.PROPRA.PARAM.2    "TEL.TARG.PMA"
TARGET.PROPDEC.PARAM.1   "TEL.TARG.PRODEC"
TARGET.PROPDEC.PARAM.2   "TEL.TARG.PMD"
TARGET.COLOR.PARAM.1     "TEL.TARG.COLOR"
TARGET.MAG.PARAM.1       "TEL.TARG.MAG"
TARGET.NAME.PARAM.1      "TEL.TARG.NAME"


##
## List of instruments for which VM OBs may be checked in 
## to the repository.
##
## This key is already set and should not be modified.
##
## The set is read numerically and in order and the first key which is
## missing or empty terminates the sequence. That is, in the following:
## INSTRUMENTS.VMCHECKINOK.1 "VIMOS"
## INSTRUMENTS.VMCHECKINOK.2 "FORS1"
## INSTRUMENTS.VMCHECKINOK.4 "ISAAC"
##
## Only 2 values: "VIMOS" and "FORS1" would be valid.
## 
INSTRUMENTS.VMCHECKINOK.1 "VIMOS"

#The instruments are allowed sidereal time
INSTRUMENTS.SIDEREALTIME.1 "MIDI"
INSTRUMENTS.SIDEREALTIME.2 "VINCI"
INSTRUMENTS.SIDEREALTIME.3 "AMBER"

##------------------------------------------------------------
## The following configuration items are only 
## meaningful for P2PP Application Server
## installations
##------------------------------------------------------------

APPSERVER.LOG      "logs/appserver.log"
APPSERVER.NAME     "P2PP-7"
APPSERVER.PORT     "1222"
APPSERVER.DATAPORT "1223"



##------------------------------------------------------------
## The following configuration items are only 
## meaningful for OT
##------------------------------------------------------------

## PAF.DIRECTORY
## Pathname of the directory where the MaskTracker module will store
## the reports it receives; it defaults to pafdata.
PAF.DIRECTORY	   "data"

## DUMP.DIRECTORY
## Pathname of the directory where OBD files will be saved.
## It defaults to the user's home directory. 
# DUMP.DIRECTORY ""

## CONFIGREPORT.<inst>.<key>
## For more information about these keywords, please see
## http://www.eso.org/~uss/ohs/docs/specs/configreports1.html
CONFIGREPORT.FORS2.KEY1		"TPL.INSTRUM"
CONFIGREPORT.FORS2.KEY2		"TPL.MODE"
CONFIGREPORT.FORS2.KEY3		"DET.READ.CLKIND"
CONFIGREPORT.FORS2.KEY4		"DET.WIN1.STRX"
CONFIGREPORT.FORS2.KEY5		"DET.WIN1.STRY"
CONFIGREPORT.FORS2.KEY6		"DET.WIN1.NX"
CONFIGREPORT.FORS2.KEY7		"DET.WIN1.NY"
CONFIGREPORT.FORS2.KEY8		"INS.FILT1.NAME"
CONFIGREPORT.FORS2.KEY9		"INS.FILT2.NAME"
CONFIGREPORT.FORS2.KEY12	"INS.SLIT.NAID"
CONFIGREPORT.FORS2.KEY11	"INS.GRIS.NAME"
CONFIGREPORT.FORS2.KEY10	"INS.COLL.NAID"


# for VISIR


CONFIGREPORT.VISIR.KEY1 "TPL.INSTRUM"
CONFIGREPORT.VISIR.KEY3 "INS.MODE"
CONFIGREPORT.VISIR.KEY4 "INS.PFOV"
CONFIGREPORT.VISIR.KEY5 "INS.FILT1.NAME"
CONFIGREPORT.VISIR.KEY6 "INS.FILT2.NAME"
CONFIGREPORT.VISIR.KEY7 "INS.RESOL"
CONFIGREPORT.VISIR.KEY8 "INS.GRAT1.WLEN"
CONFIGREPORT.VISIR.KEY9 "INS.SLIT1.WIDTH" 

## OPTIONS.FOLDER
## Location of personal configuration and option selections.
## Note: The OPTIONS.FOLDER keyword and one of the following 
##       CONFIGMENU keywords must be designated to activate 
##       the Option menu in the toolbar. 
#OPTIONS.FOLDER "config"

## REPOSITORYBROWSER.CONFIG.CONFIGMENU
## File containing personal option selections for Repository Browser.
REPOSITORYBROWSER.CONFIG.CONFIGMENU "ot-helper.cf"

## QUEUEVIEW.CONFIG.CONFIGMENU
## File containing personal option selections for Queue View.
QUEUEVIEW.CONFIG.CONFIGMENU "qv-helper.cf"

## QUEUEVIEW.CONFIG.MENUDEFAULT
## Default option selection name for Queue View.
QUEUEVIEW.CONFIG.MENUDEFAULT.defaultMenuName "default"

## OT.REFRESHQUEUE.ENABLED
## It can be set to true or false (default). If the value is
## true, the Queues->Refresh menu item (main OT panel)
## is enabled.
#OT.REFRESHQUEUE.ENABLED "true"

##------------------------------------------------------------
## The following configuration items are only 
## meaningful for PanelTool
##------------------------------------------------------------

## DEFAULT.PRINTER
## Text files will be sent to this printer.
DEFAULT.PRINTER  "ppbla3"


##------------------------------------------------------------
## The following configuration items are only 
## meaningful for IPacMan
##------------------------------------------------------------

## IPACMAN.TEMPDIR.FOLDER
## Pathname of the directory where temporary files and directories are created
IPACMAN.TEMPDIR.FOLDER  "/tmp"


## IPACMAN.MISSINGIPS.IGNORELIST
## The set of instruments that are to be "ignored" in the Missing IPs report of
## IPACMAN.
IPACMAN.MISSINGIPS.IGNORELIST "1.3/3.0Rec;1.3Bolo;2.0/3.0Rec;ADONISSH;AdonisCO;AdonisSH;CASPECL;CAT;CassBCSpec;DFOSC;Special"


## 
## The maximum number of finding charts that can 
## be attached to an OB
MAX.FINDING.CHARTS "4"


## 
## The total size of the finding chart in bytes
## that is allowed to be attached to an OB.
MAX.FINDING.CHARTSSIZE "1000000"


# Notes thart Paranal want to add to the finding
# charts 

FC.NOTES.ENABLED "true"


# Whether the image should be scaled or not
# by default.

FC.SCALED.ENABLED "false"


# Directory to save jpegs to.
# 

#FC.FOLDER "findingcharts"

#
# LaSilla Observatory longitude in radians. Default is Paranal


OBSERVATORY.LONG "1.234471"

#
# La Silla Observatory latitude in radians. Default is paranal 

OBSERVATORY.LAT  "-0.510625"

# Engineering role name in the user portal database
ENGINEERING.ROLE.NAME "engineering" 


# Operations role name in the user portal database
OPERATIONS.ROLE.NAME "operations"

