package org.eso.ohs.phase2.apps.p2pp;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.KeyStroke;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbaseDec;
import org.eso.ohs.core.dbb.client.DbaseRA;
import org.eso.ohs.core.dbb.client.DbbAirmassTextField;
import org.eso.ohs.core.dbb.client.DbbArrowButton;
import org.eso.ohs.core.dbb.client.DbbCheckBox;
import org.eso.ohs.core.dbb.client.DbbComboBox;
import org.eso.ohs.core.dbb.client.DbbCoordinateTextField;
import org.eso.ohs.core.dbb.client.DbbExecTimeTextField;
import org.eso.ohs.core.dbb.client.DbbExecutionTime;
import org.eso.ohs.core.dbb.client.DbbFliTextField;
import org.eso.ohs.core.dbb.client.DbbHHMMDateTime;
import org.eso.ohs.core.dbb.client.DbbISODateTime;
import org.eso.ohs.core.dbb.client.DbbIntegerTextField;
import org.eso.ohs.core.dbb.client.DbbMoonAngularTextField;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbSeeingTextField;
import org.eso.ohs.core.dbb.client.DbbSelectionList;
import org.eso.ohs.core.dbb.client.DbbSiderealTime;
import org.eso.ohs.core.dbb.client.DbbStrehlRatioTextField;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.DbbTextFieldPattern;
import org.eso.ohs.core.dbb.client.DbbWidget;
import org.eso.ohs.core.dbb.client.DismissAction;
import org.eso.ohs.core.dbb.client.MoonDistance;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.BetweenChunkGroup;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlChunkGroupMember;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.Period;
import org.eso.ohs.core.utilities.SystemStatusEvent;
import org.eso.ohs.core.utilities.SystemStatusListener;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectSelectionEvent;
import org.eso.ohs.persistence.ObjectSelectionListener;
import org.eso.ohs.persistence.dbase.phase2.ProcessExtraConstraints;
import org.eso.ohs.phase2.actions.FindingChartViewer;
import org.eso.ohs.phase2.actions.PrintFindingCharts;
import org.eso.ohs.phase2.actions.SaveFCAction;
import org.eso.ohs.phase2.actions.ShowFindingChartsAction;
import org.eso.ohs.phase2.apps.p2pp.actions.CheckOutAction;
import org.eso.ohs.phase2.apps.p2pp.actions.EditDbDetailsAction;
import org.eso.ohs.phase2.apps.p2pp.actions.QuitAction;
import org.eso.ohs.phase2.apps.p2pp.dbb.Declination;
import org.eso.ohs.phase2.apps.p2pp.dbb.P2PPView;
import org.eso.ohs.phase2.apps.p2pp.dbb.RightAscension;
import org.eso.ohs.phase2.apps.p2ppAppServer.BrowserConfig;
import org.eso.ohs.phase2.apps.p2ppAppServer.DbbLocalEngineP2PP;

/**
 * Application database view window
 * <p>
 * @version $Revision: 1.48.4.5 $
 * @author Bob Kemp
 * <br>
 * <br>ChangeLog: Tim Canavan
 * <br>ChangeLog: cmavros 28/Aug/2001 Add Execution Time display option
 * <br>ChangeLog: cmavros 05/OCT/2001 Change Sky Transparency and Instruments
 *   to multiple select JLists; Rearranged Selection Criteria
 *   to reflect the Repositoy Browser.
 * <br>ChangeLog: cmavros 06/02/2002
 *   1656/P2PP Change User Id to DbbIntegerTextField.
 *   Added log4j
 * <br>ChangeLog: cmavros 26/06/2002 ]Change method call from
 *     getCurrentPeriod() to getPeriod() correcting the default period
 */
public class DbaseWindow extends MenuGroups {

    public static final String cvsID_ =
        "$Id: DbaseWindow.java,v 1.48.4.5 2007/10/24 11:31:18 pnunes Exp $";

    private static Logger stdlog_ = Logger.getLogger(DbaseWindow.class);

    private static DbaseWindow dbWdw_;


    private JMenu printFCMenu;


    private JMenu saveFCMenu;

    DbbSqlEngine engine_;
    P2PPView view_;

    JPanel panel_;
    Media device_;

    // Phase I Tables
    DbbSqlTable op;
    DbbSqlTable or;
    DbbSqlTable accounts;
    DbbSqlTable profiles;
    DbbSqlTable pr;
    DbbSqlTable inst;

    // Phase II Tables
    DbbSqlTable ob;
    DbbSqlTable od;
    DbbSqlTable ts;
    DbbSqlTable tp;
    DbbSqlTable pa;
    DbbSqlTable ti;
    DbbSqlTable cs;
    DbbSqlTable sr;
    DbbSqlTable pm;
    DbbSqlTable fcs;
    DbbSqlTable baselines;
    DbbSqlTable siderealTimeInterval;
    DbbSqlTable standardTimeInterval;
    DbbSqlTable ephemerisfile;



    private EditDbDetailsAction editDetailsAction_;
    private ShowFindingChartsAction showFindingChartsAction_;

    //private ShowDetailsAction showDetailsAction_;
    private final int SELECT_COLUMNS = 7;
    private final int SELECT_CRITERIA = 5;
    private JMenuItem checkOutMenuItem_;

    private JButton findingChart_ = new JButton("Finding Chart(s)");

    //----------------------------------------
    // Constructor
    //----------------------------------------
    /**
     * <br>ChangeLog: A M Chavan, 17-Jan-00, added title to titlebar
     * @exception ObjectIOException Object I/O Exception
     */
    public DbaseWindow() throws ObjectIOException {

        try {
            P2PP.addSystemStatusListener(new SystemStatusListener() {
                public void systemStatusChange(SystemStatusEvent ev) {
                    if (ev.getType() == SystemStatusEvent.RELOGIN) {
                        if (dbWdw_ != null) {
                            dbWdw_.setVisible(false);
                            dbWdw_ = null;
                        }
                    }
                }
            });
            if (AppConfig.getAppConfig().shouldUseAppServer()) {
                engine_ =
                    new DbbLocalEngineP2PP(
                        AppConfig.getAppConfig().getAppServerUrl(),
                        true);
                BrowserConfig.generateTables();
            } else {
                engine_ = BrowserConfig.getEngine();
            }

            view_ = new P2PPView(Media.DBASE, engine_);
            view_.addObjectSelectionListener(MainWindow.getView());
            if(Config.getCfg().isInOperationsMode()) {
                view_.addObjectSelectionListener(FindingChartViewer.getInstance());
            }
            view_.addObjectSelectionListener(new ReportsMenuUpdater());
            view_.createDefaultListListener();
            if (Config.getCfg().isInOperationsMode()) {
                /* Create the Edit button for the operation mode */
                JButton edit_button = new JButton("Edit");
                if (editDetailsAction_ == null) {
                    editDetailsAction_ = new EditDbDetailsAction(view_);
                }
                edit_button.addActionListener(editDetailsAction_);
                view_.addSummaryButton(edit_button);
            } else {
                /* Create the View button for the non operation modes */
                JButton view_button = new JButton("View");
                if (editDetailsAction_ == null) {
                    editDetailsAction_ = new EditDbDetailsAction(view_);
                }
                view_button.addActionListener(editDetailsAction_);
                view_.addSummaryButton(view_button);
            }

            // allways allow viewing finding charts

           // if(Config.getCfg().isInOperationsMode()) {
                view_.addSummaryButton(findingChart_);
                showFindingChartsAction_ =
                    new ShowFindingChartsAction(Media.DBASE, view_,this,false);
                findingChart_.addActionListener(showFindingChartsAction_);
           // }

            // XXX HACK -- BADLY NEEDS TIDYING UP !!!
            op = BrowserConfig.op;
            or = BrowserConfig.or;
            accounts = BrowserConfig.accounts;
            profiles = BrowserConfig.profiles;
            pr = BrowserConfig.pr;
            sr = BrowserConfig.sr;

            ob = BrowserConfig.ob;
            od = BrowserConfig.od;
            ts = BrowserConfig.ts;
            tp = BrowserConfig.tp;
            pa = BrowserConfig.pa;
            ti = BrowserConfig.ti;
            cs = BrowserConfig.cs;
            pm = BrowserConfig.pm;
            baselines=BrowserConfig.baselines;
            ephemerisfile=BrowserConfig.ephemeris_file;

            fcs = BrowserConfig.finds_view;
            siderealTimeInterval = BrowserConfig.sidereal_time_intervals_view;
            standardTimeInterval = BrowserConfig.standard_time_intervals_view;

            buildColumns();
            stdlog_.debug("p2pp.DbaseWindow Building Columns");

            buildRows();
            stdlog_.debug("p2pp.DbaseWindow Building rows");
            buildSort();
            createMenus();
            stdlog_.debug("p2pp.DbaseWindow Building sort");

        } catch (RemoteException e) {
            throw new ObjectIOException(e);
        } catch (MalformedURLException e) {
            throw new ObjectIOException(e);
        } catch (NotBoundException e) {
            throw new ObjectIOException(e);
        } catch (SQLException e) {
            throw new ObjectIOException(e);
        }

        getContentPane().add(view_);
        setTitle("ESO Database Browser");
        pack();

        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent ev) {
                if (Config.getCfg().isInOperationsMode()) {
                    P2PP.askAndQuit();
                } else {
                    dbWdw_.setVisible(false);
                }
            }
        });
    }



    public void createFCPrintMenu(ObjectContainer container) {
    	JTable table = view_.getTable().getTable();

        String menuName = "Finding Charts";
        if (printFCMenu == null)
            printFCMenu = new JMenu("Print");
        printFCMenu.add(
            new PrintFindingCharts(Media.DBASE,container,
            		table,"All",true));
        printFCMenu.add(
            new PrintFindingCharts(Media.DBASE,container,table,"Selected", false));
        addSubMenu(menuName, printFCMenu);

        addSubMenu(menuName, printFCMenu);
        String saveMenu = "Save As jpg";
        if (saveFCMenu == null)
            saveFCMenu = new JMenu(saveMenu);
        saveFCMenu.add(new SaveFCAction(Media.DBASE, container,table, "All",true));
        saveFCMenu.add(new SaveFCAction(Media.DBASE, container,table,"Selected", false));
        addSubMenu(menuName,saveFCMenu);


    }

    //----------------------------------------
    // Private Methods
    //----------------------------------------

    /**
      * Build DB browser view: SELECT panel
      *
      * <br>ChangeLog: A M Chavan, 17-Jan-2000: [0597,0584] Fixed selection of grisms.
      * @exception SQLException JDBC exception occurs
      * @exception RemoteException RMI exception occurs
      * @exception MalformedURLException malformed url i.e. servername//port
      * @exception NotBoundException client could not bind to server
      */
    private void buildColumns()
        throws
            NotBoundException,
            MalformedURLException,
            SQLException,
            RemoteException {
        DbbPanel panel = new DbbPanel("Selected Columns", SELECT_COLUMNS);

        DbbSqlChunk c;
        DbbCheckBox dbbCBox;
        String label_name;
        String column_name;

        // OB id (default chunk)
        label_name = "OB ID";
        column_name = "ob_id";
        engine_.addDefaultColumnsChunk(
            new DbbSqlChunk(ob, column_name, label_name));

        column_name = "type";
        engine_.addDefaultColumnsChunk(new DbbSqlChunk(ob, column_name));

        // OB name widget
        label_name = "OB Name";
        column_name = "item_name";
        c = new DbbSqlChunk(ob, column_name, label_name);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        // Programme ID widget
        label_name = "ProgID";
        column_name = "progid";
        c = new DbbSqlChunk(sr, column_name, label_name, String.class);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        // User ID widget
        // amchavan, 13-Jun-2002: commented out
        // label_name = "UserID";
        // column_name = "user_id";
        // c = new DbbSqlChunk(ob, column_name, label_name);

        // User name widget
        label_name = "LastName";
        column_name = "last_name";
        c = new DbbSqlChunk(profiles, column_name, label_name,"rtrim(@column@)");        
        // c.setExpression( "substring( surname_dl, 1, 20 )" );
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        // Target
        label_name = "Target";
        column_name = "item_name";
        c = new DbbSqlChunk(tp, column_name, label_name);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        // RA widget
        label_name = "RA";
        column_name = "ra";
        c = new DbbSqlChunk(tp, column_name, label_name, DbaseRA.class);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        // Dec widget
        label_name = "Dec";
        column_name = "dec";
        c = new DbbSqlChunk(tp, column_name, label_name, DbaseDec.class);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        /*
            OB type widget
        label_name = "Type";
        column_name = "type";
        c = new DbbSqlChunk(ob, column_name, label_name);
        dbbCBox = new DbbCheckBox(label_name,c);
        panel.addWidget(dbbCBox);
        */

        // Optical elements widget
        label_name = "OptElem";
        column_name = "value";
        c = new DbbSqlChunk(pa, column_name, label_name);
        dbbCBox = new DbbCheckBox(label_name, c);

        /////////// test ///////////
        DbbSqlOperator op = null;
        try {
            op = new DbbSqlOperator(">");
        } catch (Exception e) {
            stdlog_.error(e);
        }
        column_name = "subtype";
        c = new DbbSqlChunk(pa, column_name, op, Integer.class);
        c.setValue("0");
        dbbCBox.addSqlChunk(c);
        panel.addWidget(dbbCBox);
        /////////// test ///////////

        // Instrument widget
        label_name = "Instrument";
        column_name = "instrument";
        c = new DbbSqlChunk(od, column_name, label_name);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        // IP Version column only visible in operations mode.
        if (Config.getCfg().isInOperationsMode()) {
            // ipversion widget
            label_name = "IP Version";
            column_name = "ipversion";
            c = new DbbSqlChunk(od, column_name, label_name);
            dbbCBox = new DbbCheckBox(label_name, c);
            panel.addWidget(dbbCBox);
        }

        // User's priority
        label_name = "UsrP";
        column_name = "userPriority";
        c = new DbbSqlChunk(ob, column_name, label_name);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        // Template widget
        label_name = "Template";
        column_name = "item_name";
        c = new DbbSqlChunk(ts, column_name, label_name);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        // OB Status widget
        label_name = "Status";
        column_name = "status";
        c = new DbbSqlChunk(ob, column_name, label_name);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        // CS Seeing widget
        label_name = "Seeing";
        column_name = "seeing";
        c = new DbbSqlChunk(cs, column_name, label_name);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        // CS SkyTransparency widget
        label_name = "SkyTran";
        column_name = "sky_transparency";
        c = new DbbSqlChunk(cs, column_name, label_name);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        // CS Airmass widget
        label_name = "Airmass";
        column_name = "airmass";
        c = new DbbSqlChunk(cs, column_name, label_name);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        // CS FLI widget
        label_name = "FLI";
        column_name = "fli";
        c = new DbbSqlChunk(cs, column_name, label_name);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        // CS MoonDis widget
        label_name = "MoonDis";
        column_name = "moon_distance";
        c = new DbbSqlChunk(cs, column_name, label_name, MoonDistance.class);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);


        // Finding Chart
        label_name = "Finding Charts";
        column_name = "fc_name";
        c = new DbbSqlChunk(fcs, column_name, label_name,String.class);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);


        // CS Strehl Ratio widget
        label_name = "Strehl";
        column_name = "strehl_ratio";
        c = new DbbSqlChunk(cs, column_name, label_name);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        // CS Execution Time widget
        // cmavros added 28/Aug/2001
        label_name = "ExecTime";
        column_name = "execution_time";
        c =
            new DbbSqlChunk(
                ob,
                column_name,
                label_name,
                DbbExecutionTime.class);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);


        label_name = "Sidereal Time Start";
        column_name = "start_interval";
        c =
            new DbbSqlChunk(
                siderealTimeInterval,
                column_name,
                label_name,
                DbbHHMMDateTime.class);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);


        label_name = "Sidereal Time End";
        column_name = "end_interval";
        c =
            new DbbSqlChunk(
                siderealTimeInterval,
                column_name,
                label_name,
                DbbHHMMDateTime.class);
        dbbCBox=new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        label_name = "Baseline";
        column_name = "baseline_name";
        c =
            new DbbSqlChunk(
                baselines,
                column_name,
                label_name
                );
        dbbCBox=new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        label_name = "Ephemeris File";
        column_name = "file_name";
        c = new DbbSqlChunk(ephemerisfile, column_name, label_name);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        if (Config.getCfg().isInOperationsMode()) {
            label_name = "LastMod";
            column_name = "last_mod_date";
            c =
                new DbbSqlChunk(
                    ob,
                    column_name,
                    label_name,
            DbbISODateTime.class);
            dbbCBox = new DbbCheckBox(label_name, c);
            panel.addWidget(dbbCBox);




        }

        view_.setColumnsPanel(panel, "North");
        //view_.setColumnsPanel(panel, "West");
    }

    // Build DB browser view: reows selection panel
    private void buildRows() {

        DbbPanel panel = new DbbPanel("Selection Criteria", SELECT_CRITERIA);
        DbbSqlChunk c;
        DbbTextField dbbText;
        String label_name;
        String column_name;

        DbbSqlOperator op_equal = null;
        DbbSqlOperator op_min = null;
        DbbSqlOperator op_max = null;
        DbbSqlOperator op_like = null;
        DbbSqlOperator op_in = null;
        DbbSqlOperator op_between = null;
        try {
            op_equal = new DbbSqlOperator("=");
            op_min = new DbbSqlOperator("<=");
            op_max = new DbbSqlOperator(">=");
            op_like = new DbbSqlOperator("like");
            op_in = new DbbSqlOperator("in");
            op_between = new DbbSqlOperator("between");
        } catch (Exception e) {
            stdlog_.error(e);
        }

        // Period
        label_name = "Period";
        column_name = "period";
        c = new DbbSqlChunk(sr, column_name, label_name, op_in, Integer.class);
        final int lastPeriod_ = Period.getCurrentPeriod() + Period.ADDITIONAL;
        int numPeriods = lastPeriod_ - Period.STARTPERIOD + 1;
        Integer[] periodArr = new Integer[numPeriods];
        for (int i = 0; i < numPeriods; i++) {
            periodArr[i] = new Integer(i + Period.STARTPERIOD);
        }
        DbbSelectionList widget =
            new DbbSelectionList(label_name, c, periodArr);
        panel.addWidget(widget);

        // ProposalID
        label_name = "Program ID";
        column_name = "progid";
        c = new DbbSqlChunk(sr, column_name, label_name, op_like, String.class);
        dbbText = new DbbTextFieldPattern(label_name, c);
        panel.addWidget(dbbText);

        // Username
        label_name = "UserName";
        column_name = "username";
        c =
            new DbbSqlChunk(
                accounts,
                column_name,
                label_name,
                op_like,
                String.class);
        dbbText = new DbbTextFieldPattern(label_name, c);
        panel.addWidget(dbbText);
        if (!Config.getCfg().isInOperationsMode()) {
            stdlog_.debug("p2pp.DbaseWindow ............. Before set");
            dbbText.set(" " + Config.getCfg().getUsername()+ " "); //HACK
//            dbbText.set(Config.getCfg().getUsername());
            stdlog_.debug("p2pp.DbaseWindow ............. After set");
            dbbText.setEnabled(false);
        }

        //Status
        label_name = "Status";
        column_name = "status";
        c = new DbbSqlChunk(ob, column_name, label_name, op_in, String.class);
        /*
          Object [] statusDispValues = { "Partially Defined", "Defined", "Accepted",
          "Rejected", "Queued", "Executed", "Aborted", "Must Repeat", "Verified", "Alarm"};
        */

        Object[] statusValues = CalibrationBlock.getStatusValues().valuesAsObjects();
        String[] statusDispValues = CalibrationBlock.getStatusValues().keys();

        /*
         DbbComboBox status = new DbbComboBox(label_name,c,
           statusDispValues, statusValues, true);
        */

        DbbSelectionList status =
            new DbbSelectionList(label_name, c, statusValues, statusDispValues);
        panel.addWidget(status);

        /** Instruments
         *  cmavros 05/10/01 Change to a multi-select list
         */
        label_name = "Instrument";
        column_name = "instrument";
        c = new DbbSqlChunk(od, column_name, label_name, op_in, String.class);
        DbbSelectionList w1 =
            new DbbSelectionList(
                label_name,
                c,
                InstrumentList.getInstance().getInstrumentNames());
        panel.addWidget(w1, true);

        // Templates
        label_name = "Template";
        column_name = "item_name";
        c = new DbbSqlChunk(ts, column_name, label_name, op_like, String.class);
        dbbText = new DbbTextFieldPattern(label_name, c);
        panel.addWidget(dbbText);

        // Opt Element
        label_name = "OptElem";
        column_name = "subtype";
        // Object[] fltDisp = { "Filter", "Grism" };
        // Object[] fltDb = { new Integer(1), new Integer(2)};
        c = new DbbSqlChunk(pm, column_name, label_name, op_in, Integer.class);
        // Parameter.filters.printKeys();
        // Parameter.filters.printValues();

        DbbSelectionList filter =
            new DbbSelectionList(
                label_name,
                c,
                Parameter.filters.valuesAsObjects(),
                Parameter.filters.keys());
        panel.addWidget(filter);

        // OB Name
        label_name = "OB Name";
        column_name = "item_name";
        c = new DbbSqlChunk(ob, column_name, label_name, op_like, String.class);
        dbbText = new DbbTextFieldPattern(label_name, c);
        panel.addWidget(dbbText);



        // OB Id min
        label_name = "OB ID Min";
        column_name = "ob_id";
        DbbSqlChunkGroupMember minOBId =
            new DbbSqlChunkGroupMember(
                ob,
                column_name,
                label_name,
                op_max,
                Integer.class);
        DbbSqlChunkGroupMember maxOBId =
            new DbbSqlChunkGroupMember(
                ob,
                column_name,
                label_name,
                op_min,
                Integer.class);

        // group min and max together in a SqlChunkGroup
        // DbbSqlChunkGroup raGroup = new CyclingMaxMinChunk(min, max);



        panel.addWidget(
            new DbbIntegerTextField(label_name, minOBId),
            true);

        // OB ID  max
        label_name = "OB ID Max";
        column_name = "ob_id";
        panel.addWidget(new DbbIntegerTextField(label_name, maxOBId));





















        // RA min
        label_name = "RA min";
        column_name = "ra";
        DbbSqlChunkGroupMember min =
            new DbbSqlChunkGroupMember(
                tp,
                column_name,
                label_name,
                op_max,
                RightAscension.class);
        DbbSqlChunkGroupMember max =
            new DbbSqlChunkGroupMember(
                tp,
                column_name,
                label_name,
                op_min,
                RightAscension.class);

        // group min and max together in a SqlChunkGroup
        // DbbSqlChunkGroup raGroup = new CyclingMaxMinChunk(min, max);

        panel.addWidget(
            new DbbCoordinateTextField(label_name, min, true),
            true);

        // RA max
        label_name = "RA max";
        column_name = "ra";
        panel.addWidget(new DbbCoordinateTextField(label_name, max, true));

        // Dec min
        label_name = "Dec min";
        column_name = "dec";
        c =
            new DbbSqlChunk(
                tp,
                column_name,
                label_name,
                op_max,
                Declination.class);

        panel.addWidget(new DbbCoordinateTextField(label_name, c, false));

        // Dec max
        label_name = "Dec max";
        column_name = "dec";
        c =
            new DbbSqlChunk(
                tp,
                column_name,
                label_name,
                op_min,
                Declination.class);

        panel.addWidget(new DbbCoordinateTextField(label_name, c, false));

        // Airmass min
        label_name = "Airmass min";
        column_name = "airmass";
        c = new DbbSqlChunk(cs, column_name, label_name, op_max, Double.class);
        DbbAirmassTextField airmassMin = new DbbAirmassTextField(label_name, c);
        airmassMin.setToolTipText("  Enter value between range : 1.0..5.0");
        panel.addWidget(airmassMin, true);

        // Airmass max
        label_name = "Airmass max";
        column_name = "airmass";
        c = new DbbSqlChunk(cs, column_name, label_name, op_min, Double.class);
        DbbAirmassTextField airmassMax = new DbbAirmassTextField(label_name, c);
        airmassMax.setToolTipText("  Enter value between range : 1.0..5.0");
        panel.addWidget(airmassMax);

        // Seeing min
        label_name = "Seeing min";
        column_name = "seeing";
        c = new DbbSqlChunk(cs, column_name, label_name, op_max, Double.class);
        DbbSeeingTextField seeingMin = new DbbSeeingTextField(label_name, c);
        seeingMin.setToolTipText(" Enter value between range : 0.2..2.0");
        panel.addWidget(seeingMin);

        // Seeing max
        label_name = "Seeing max";
        column_name = "seeing";
        c = new DbbSqlChunk(cs, column_name, label_name, op_min, Double.class);
        DbbSeeingTextField seeingMax = new DbbSeeingTextField(label_name, c);
        seeingMax.setToolTipText(" Enter value between range : 0.2..2.0");
        panel.addWidget(seeingMax);

        // MoonDis min
        label_name = "MoonDis min";
        column_name = "moon_distance";
        c = new DbbSqlChunk(cs, column_name, label_name, op_max, Double.class);
        DbbMoonAngularTextField moonMin =
            new DbbMoonAngularTextField(label_name, c);
        moonMin.setToolTipText(" Enter value between range : 0..180");
        panel.addWidget(moonMin, true);

        // MoonDis max
        label_name = "MoonDis max";
        column_name = "moon_distance";
        c = new DbbSqlChunk(cs, column_name, label_name, op_min, Double.class);
        DbbMoonAngularTextField moonMax =
            new DbbMoonAngularTextField(label_name, c);
        moonMax.setToolTipText(" Enter value between range : 0..180");
        panel.addWidget(moonMax);

        // FLI min
        label_name = "FLI min";
        column_name = "fli";
        c = new DbbSqlChunk(cs, column_name, label_name, op_max, Double.class);
        DbbFliTextField fliMin = new DbbFliTextField(label_name, c);
        fliMin.setToolTipText(" Enter value between range : 0.0..1.0");
        panel.addWidget(fliMin);

        // FLI max
        label_name = "FLI max";
        column_name = "fli";
        c = new DbbSqlChunk(cs, column_name, label_name, op_min, Double.class);
        DbbFliTextField fliMax = new DbbFliTextField(label_name, c);
        fliMax.setToolTipText(" Enter value between range : 0.0..1.0");
        panel.addWidget(fliMax);

        // Strehl Ratio min
        label_name = "Strehl min";
        column_name = "strehl_ratio";
        c = new DbbSqlChunk(cs, column_name, label_name, op_max, Double.class);
        DbbStrehlRatioTextField srMin =
            new DbbStrehlRatioTextField(label_name, c);
        srMin.setToolTipText(" Enter value between range : 0.0..99.99");
        panel.addWidget(srMin, true);

        // Strehl Ratio max
        label_name = "Strehl max";
        column_name = "strehl_ratio";
        c = new DbbSqlChunk(cs, column_name, label_name, op_min, Double.class);
        DbbStrehlRatioTextField srMax =
            new DbbStrehlRatioTextField(label_name, c);
        srMax.setToolTipText(" Enter value between range : 0.0..99.99");
        panel.addWidget(srMax);


        if(Config.getCfg().isInOperationsMode()) {
            label_name = "Sidereal Min";
            column_name = "start_interval";
            DbbSqlChunkGroupMember minSTTime =
                new DbbSqlChunkGroupMember(
                    siderealTimeInterval,
                    column_name,
                    label_name,
                    op_between,
                    DbbSiderealTime.class);
            label_name = "Sidereal Max";
            column_name = "end_interval";
            DbbSqlChunkGroupMember maxSTTime =
                new DbbSqlChunkGroupMember(
                    siderealTimeInterval,
                    column_name,
                    label_name,
                    op_between,
                    DbbSiderealTime.class);

            BetweenChunkGroup clorChunk = new BetweenChunkGroup(minSTTime,maxSTTime);

            minSTTime.setSqlChunkGroup(clorChunk);
            maxSTTime.setSqlChunkGroup(clorChunk);
            //rowPanel_.addWidget(clorChunk);
            label_name = "Sidereal Min";
            DbbTextField dbField = new DbbTextField(label_name, minSTTime);
            panel.addWidget(
                dbField,
                true);
            dbField.setToolTipText(
                " Enter value between range : 00:00...23:59");

            label_name = "Sidereal Max";
            dbField = new DbbTextField(label_name, maxSTTime);
            dbField.setToolTipText(
                " Enter value between range : 00:00..23:59");
            panel.addWidget(dbField);

        }




        // Exec Time Min
        // cmavros added 28/Aug/2001
        label_name = " ExecTime min";
        column_name = "execution_time";
        c =
            new DbbSqlChunk(
                ob,
                column_name,
                label_name,
                op_max,
                DbbExecutionTime.class);
        DbbExecTimeTextField execTimeMin =
            new DbbExecTimeTextField(label_name, c);
        execTimeMin.setToolTipText(
            " Enter value between range : 00:00:00...23:59:59");
        panel.addWidget(execTimeMin);

        // Exec Time Max
        // cmavros added 28/Aug/2001
        label_name = " ExecTime max";
        c =
            new DbbSqlChunk(
                ob,
                column_name,
                label_name,
                op_min,
                DbbExecutionTime.class);
        DbbExecTimeTextField execTimeMax =
            new DbbExecTimeTextField(label_name, c);
        execTimeMax.setToolTipText(
            " Enter value between range : 00:00:00...23:59:59");
        panel.addWidget(execTimeMax);

        // OB Type
        label_name = "OB Type";
        column_name = "type";
        c =
            new DbbSqlChunk(
                ob,
                column_name,
                label_name,
                op_equal,
                String.class);
        Object[] dbValues = { "O", "C" };
        Object[] dispValues = { "Observation", "Calibration" };
        DbbComboBox type =
            new DbbComboBox(label_name, c, dispValues, dbValues, true);
        panel.addWidget(type, true);

        /** Sky tansparency
         * cmavros 05/10/01 Change to a multi-select list
         */
        label_name = "SkyTran";
        column_name = "sky_transparency";
        c = new DbbSqlChunk(cs, column_name, label_name, op_in, String.class);
        DbbSelectionList sky_tra =
            new DbbSelectionList(
                label_name,
                c,
                ConstraintSet.getDbSkyTransparency(),
                ConstraintSet.getValidSkyTransparency());
        panel.addWidget(sky_tra);

        if(Config.getCfg().isInOperationsMode()) {
            label_name = "Baseline";
            column_name = "baseline_name";
            c = new DbbSqlChunk(baselines, column_name, label_name, op_in, String.class);
            try {
                w1 =
                    new DbbSelectionList(
                        label_name,
                        c,
                ProcessExtraConstraints.getBaseLines());
            } catch (SQLException e1) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
            }
            panel.addWidget(w1);

        }



        /*label_name = "Finding Charts";
        column_name = "fc_name";
        c = new DbbSqlChunk(fcs, column_name, label_name, op_like, String.class);
        dbbText = new OHSDbbTextField(label_name, c);
        panel.addWidget(dbbText);*/


        /*
          label_name = "OB min";
          column_name = "ob_id";
          c = new DbbSqlChunk(ob, column_name, label_name, op_max, Integer.class);
          panel.addWidget(new DbbSpinnerInt(label_name, c, -1, 1, 0, 1000000, true));
          // OB max
          label_name = "OB max";
          column_name = "ob_id";
          c = new DbbSqlChunk(ob, column_name, label_name, op_min, Integer.class);
          panel.addWidget(new DbbSpinnerInt(label_name, c, -1, 1, 0, 1000000, true));
         */

        view_.setWherePanel(panel, "North");
    }

    // Build DB browser view: reows selection panel
    private void buildSort() {

        //DbbPanel panel = new DbbPanel("Sorting Criteria", 4);
        DbbPanel panel = new DbbPanel("Sorting Criteria", 1);
        DbbSqlChunk c;
        DbbWidget dbbWidget;
        String column_name;
        String label_name;

        // OB ID
        label_name = "OB ID";
        column_name = "ob_id";
        c = new DbbSqlChunk(ob, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        // OB Name
        label_name = "OB Name";
        column_name = "item_name";
        c = new DbbSqlChunk(ob, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        // Prog ID
        label_name = "ProgID";
        column_name = "progid";
        c = new DbbSqlChunk(sr, column_name, label_name);

        ///////// It was a bug /////////
        // label_name = "ProgID";
        // column_name = "proposalID";
        // c = new DbbSqlChunk(op, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        // RA
        label_name = "RA";
        column_name = "ra";
        c = new DbbSqlChunk(tp, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        // Dec
        label_name = "Dec";
        column_name = "dec";
        c = new DbbSqlChunk(tp, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        //Seeing
        label_name = "Seeing";
        column_name = "seeing";
        c = new DbbSqlChunk(cs, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        //Airmass
        label_name = "Airmass";
        column_name = "airmass";
        c = new DbbSqlChunk(cs, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        //FLI
        label_name = "FLI";
        column_name = "fli";
        c = new DbbSqlChunk(cs, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        // Moon Distance
        label_name = "MoonDis";
        column_name = "moon_distance";
        c = new DbbSqlChunk(cs, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        // SkyTran
        label_name = "SkyTran";
        column_name = "sky_transparency";
        c = new DbbSqlChunk(cs, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        //Strehl Ratio
        label_name = "Strehl";
        column_name = "strehl_ratio";
        c = new DbbSqlChunk(cs, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        label_name = "Finding Charts";
        column_name = "fc_name";
        c = new DbbSqlChunk(fcs, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);


        // Execution Time
        // cmavros added 28-Aug-2001
        label_name = "ExecTime";
        column_name = "execution_time";
        c = new DbbSqlChunk(ob, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        //view_.setSortPanel(panel, "North");
        view_.setSortPanel(panel, "West");

    }

    private void createMenus() {

        createFileMenu();
        createReportsMenu(view_,view_.getTable().getTable(),Media.DBASE);
        // allways allow viewing finding charts

        //if(Config.getCfg().isInOperationsMode()) {
            createFCPrintMenu(view_);
        //}
    }

    //----------------------------------------
    // Public Methods
    //----------------------------------------

    /**
     * Create and populate the "File" menu in the database browser view.
     * @exception ObjectIOException object IO Exception
     */
    private void createFileMenu() {

        final String menuName = MenuGroups.FILEMENU_NAME;
        JMenuItem mnItem;

        if (Config.getCfg().isInOperationsMode()) {
            mnItem = addMenuItem(menuName, new QuitAction(this));
            mnItem.setAccelerator(
                KeyStroke.getKeyStroke(
                    KeyEvent.VK_Q,
                    InputEvent.CTRL_MASK,
                    false));
        } else {
            checkOutMenuItem_ =
                addMenuItem(menuName, new CheckOutAction(view_));
            checkOutMenuItem_.setAccelerator(
                KeyStroke.getKeyStroke(
                    KeyEvent.VK_T,
                    InputEvent.CTRL_MASK,
                    false));
            checkOutMenuItem_.setEnabled(false);
            mnItem = addMenuItem(menuName, new DismissAction(this));
            mnItem.setAccelerator(
                KeyStroke.getKeyStroke(
                    KeyEvent.VK_W,
                    InputEvent.CTRL_MASK,
                    false));
        }
    }


    public ObjectContainer getFolderView() {
        return view_;
    }

    private class ReportsMenuUpdater implements ObjectSelectionListener {

        //private ObjectSelector objSelector_;

        public ReportsMenuUpdater() {
            //objSelector_ = objSelector;
        }

        public void objectSelected(ObjectSelectionEvent ev) {
            if (ev.getObjectId() == ObjectSelectionEvent.NO_OBJECT) {
                if (!Config.getCfg().isInOperationsMode()) {
                    checkOutMenuItem_.setEnabled(false);
                }
                if (ev.getRowCount() <= 0) {
                    setEnabledReports(false);
                } else {
                    setEnabledSelectedReports(false);
                }
            } else {
                if (!Config.getCfg().isInOperationsMode()) {
                    checkOutMenuItem_.setEnabled(true);
                }
                setEnabledReports(true);
                setEnabledSelectedReports(true);
            }
        }
    }

    public static DbaseWindow getDbaseWindow(Media device)
        throws ObjectIOException {
        if (dbWdw_ == null) {
            synchronized (DbaseWindow.class) {
                if (dbWdw_ == null) {
                    P2PP.askForAuthentication(Media.DBASE);
                    dbWdw_ = new DbaseWindow();
                }
            }
        }
        return dbWdw_;
    }
}
