package org.eso.ohs.phase2.apps.p2pp;

import java.lang.reflect.InvocationTargetException;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.instruments.InstrumentException;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;

public class Phase1Information {
    
    /**
     * Log4J class.
     */
    private static Logger stdlog_ =
       Logger.getLogger(Phase1Information.class);

	public Phase1Information() {
	}

	public static Summary[] obsRunsForPeriod(int chosenPeriod)
		throws ObjectIOException {
		Persistence objMgr = Persistence.getInstance();
		DirectoryNode dbRoot = objMgr.getRoot(Media.PHASE1_DB);
		Summary[] summ = objMgr.listObjects(Media.PHASE1_DB, dbRoot);
		Vector<Summary> selected = new Vector<Summary>();

		for (int j = 0; j < summ.length; j++) {
			Class objType = summ[j].getObjectClass();

			if (!ObservingRun.class.equals(objType)) {
				continue;
			}

			Number period = (Number) summ[j].getPropertyValue("period");

			if ((period != null && period.intValue() == chosenPeriod)
				|| chosenPeriod == HierarchyView.ALL_PERIODS) {
				selected.addElement(summ[j]);
			}
		}

		Summary[] result = new Summary[selected.size()];
		selected.copyInto(result);

		return result;
	}

	/**
	 * Download the observing runs from the database.
	 * MIPS Project:
	 * If IPVersioning is configured, the available IP versions are 
	 * checked with what is specified in the observing run, and the 
	 * IP is downloaded if required.
	 */

	public static boolean refreshObsRuns(Summary[] ids)
		throws ObjectNotFoundException, ObjectIOException {
		return refreshObsRuns(ids, null);
	}

	public static boolean refreshObsRuns(
		Summary[] ids,
		Phase1Information.MessageListener listener)
		throws ObjectNotFoundException, ObjectIOException {
       
		Persistence objMgr = Persistence.getInstance();
		DirectoryNode locRoot = objMgr.getRoot(Media.LOCAL);
		boolean downloadedAnObservingRun = false;
		ObservingRun locOR;
		ObservingRun dbOR;
		Folder folder;
		DirectoryNode orNode;
		boolean refreshInstrumentList = false;
		boolean ipVersioningEnabled =
			AppConfig.getAppConfig().isIPVersioningEnabled();
		boolean singleIPMode =
			AppConfig.getAppConfig().isIPVersioningSingleIPMode();
		// cache of all ips downloaded this session.
		Hashtable ipsDownloaded = new Hashtable();
		for (int j = 0; j < ids.length; j++) {
			Class objType = ids[j].getObjectClass();

            if (listener != null) {
                listener.infoMessage("Downloading Observing Run "+(j+1)+" of "+ids.length+"...");
            }
			if (!ObservingRun.class.equals(objType)) {
				continue;
			}

			// WARNING: This bypasses the Registry to avoid caching.
			// That means that dbOR is not registered and
			//  normal ObjMgr methods may fail on it!
			dbOR = objMgr.getObservingRun(ids[j].getId());

			orNode = subdirForObsRun(dbOR.getId());

			if (orNode == null) {
				long subdirId = objMgr.createDirectory(Media.LOCAL, locRoot);

				if (subdirId == ObjectManager.NO_OBJECT) {
					throw new ObjectIOException(
						"Cannot create local directory for observing run: "
							+ dbOR.getId());
				}

				orNode = locRoot.getDirectory(subdirId);
			}

			try {
				locOR =
					(ObservingRun) objMgr.getBusObj(
						Media.LOCAL,
						ids[j].getId(),
						ObservingRun.class);

				// Update local copy of ObsRun from dbase copy
				String[] properties = locOR.simpleProperties();
				dbOR.copyProperties(properties, locOR);
			} catch (ObjectNotFoundException e) {
				locOR =
					(ObservingRun) objMgr.deepCopy(
						Media.LOCAL,
						orNode,
						dbOR,
						dbOR.getId());
			} catch (IllegalAccessException e) {
				throw new ObjectIOException(e);
			} catch (InvocationTargetException e) {
				throw new ObjectIOException(e.getTargetException());
			} catch (NoSuchMethodException e) {
				throw new ObjectIOException(e);
			}

			folder = FolderFactory.getFolder(Media.LOCAL, orNode);

			folder.setObsRun(locOR);
            
            if (ipVersioningEnabled && !singleIPMode && !InstrumentList.getInstance().instrumentExists(
                                    locOR.getInstCode(),
                                    locOR.getIPVersion())
                && !Persistence.getInstance().isInstrumentWithNoIP(locOR.getInstCode())) {
                    String errMsg = "Error downloading IP for Observing Run: "
                        +locOR.getName()+" IP: "+locOR.getInstCode()+" v"+locOR.getIPVersion();
                    if (listener == null) {
                        stdlog_.error(errMsg);
                    } else {
                        listener.errorMessage(errMsg);
                    }
            }
			//if (ipVersioningEnabled && !singleIPMode) {
				/*
				 * MIPS Project - ensure we have the IP for the observing run if we are
				 * in Multiple IP mode.
				 * Check the IP versions and instruments required are here or download the required
				 * IPs to the directory specified in the config.
				 * Note that the name and version of all downloaded IPs is also cached locally
				 * to avoid downloading the same IP twice (when two Obs Runs use the same IP
				 * for example).
				 */
/*
				String inst = locOR.getInstCode();
				float ipVer = locOR.getIPVersion();
				String ipid = inst + "_" + ipVer;
				Object exists = ipsDownloaded.get(ipid);

				if (exists == null
					&& !InstrumentList.getInstance().instrumentExists(
						locOR.getInstCode(),
						locOR.getIPVersion())) {

					try {
						Persistence.getInstance().downloadInstrumentPackage(
							inst,
							ipVer);

						ipsDownloaded.put(ipid, ipid);
					} catch (ObjectIOException e) {
						// we want to catch this exception and report it directly to the user
						// rather than throwing it and interrupting the download, as interrupting
						// the download results in a corrupted cache.
                        if (listener == null) {
                            stdlog_.error("Error downloading Observing Run: "+e.getMessage());
                        } else {
                            listener.errorMessage(e.getMessage());
                        }
					}
					refreshInstrumentList = true;

				}
			}
*/
			folder.setName(locOR.getName());
			downloadedAnObservingRun = true;
		}
		if (refreshInstrumentList) {
            if (listener != null) {
                listener.infoMessage("Reading Instrument data...");
            }
			// New IPs were downloaded, we need to update the InstrumentList.
			try {
                String instDir = AppConfig.getAppConfig().getInstrumDir();
                String ipCacheDir = AppConfig.getAppConfig().getIPVersionIPCache();
				InstrumentList.reloadInstrumentList(singleIPMode,instDir,ipCacheDir);
			} catch (InstrumentException ie) {
				throw new ObjectIOException(ie.getMessage());
			}
		}

		return downloadedAnObservingRun;
	}

	/**
	 * Returns the id of the folder containing the ObservingRun
	 * with the given id.
	 * 
	 * @param   id      id of ObservingRun to find
	 * 
	 * @return      id of the containing folder for the OR,
	 *          or null if not found
	 */
	private static DirectoryNode subdirForObsRun(long id) {
		Persistence objMgr = Persistence.getInstance();

		if (objMgr.isInRegistry(id)) {
			return objMgr.getDirectoryFor(id);
		} else {
			return null;
		}
	}
    
    /**
     * Interface implemented by calling objects to provide
     * a hook into error messages reported during the download
     * of Phase1Information (obs runs, IPs, etc).
     */
    public static interface MessageListener {
        public void errorMessage(String message);
        public void infoMessage(String message);
    }

}

