package org.eso.ohs.phase2.apps.p2pp;

import java.awt.datatransfer.Clipboard;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.event.EventListenerList;

public class PropertyClipboard extends Clipboard {

    /** Support for handling PropertyChangeEvent */
    private transient EventListenerList listenerList;

    /** PropertyChangeEvent to pass to listeners */
    private transient PropertyChangeEvent propertyChangeEvent;

    PropertyClipboard(String name) {
        super(name);
        initialise();
    }

    /**
     * Initialise property change event support.
     */
    private void initialise() {
        listenerList = new EventListenerList();
        propertyChangeEvent =
            new PropertyChangeEvent(this, "Contents", null, null);
    }

    public void propertyChange(PropertyChangeEvent ev) {
        // System.out.println("Property Change Event..." + ev);
    }
    /**
       * Adds a new listener to be notified of changes.
       * 
       * @param l the listener
       */
    public void addPropertyChangeListener(PropertyChangeListener l) {
        listenerList.add(PropertyChangeListener.class, l);
    }

    /**
     * Removes a new listener, which will no longer be notified of changes.
     * 
     * @param l the listener
     */
    public void removePropertyChangeListener(PropertyChangeListener l) {
        listenerList.remove(PropertyChangeListener.class, l);
    }

    /**
     * Informs listeners of a property change.
     * No details of which property of the old/new value are given.
     */
    public void firePropertyChangeEvent() {
        Object[] listeners = listenerList.getListenerList();

        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i] == PropertyChangeListener.class) {
                ((PropertyChangeListener) listeners[i + 1]).propertyChange(
                    propertyChangeEvent);
            }
        }
    }
}
