package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ExtensionFileFilter;
import org.eso.ohs.core.gui.widgets.FilePreviewDialog;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.EphemerisFile;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.ObjectSelector;
import org.eso.ohs.phase2.apps.p2pp.FolderGridView;

/**
 * @author tcanavan
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class AttachEphFile extends ActionSuperclass {

    private static Logger stdlog_ = Logger.getLogger(AttachEphFile.class);


    private static FilePreviewDialog filePreviewDialog_ = null;

    private static SwingFileChooser chooser_ = null;


    private static ExtensionFileFilter ephFilter = new ExtensionFileFilter("eph","Ephemeris Files");
  



    private FolderGridView folderGridView_;

    private ObjectSelector objSelector_;

    private JFrame frm_;

    /**
     * Status window which displays reports on imports.     
     */

    /**
        * Constructs a create BusinessObject action.
        * 
        * @param device      medium containing the parent directory.
        * @param folderView  GUI view of (summary information in) the folder.
        */

    public AttachEphFile(FolderGridView folderGridView) {
        super(folderGridView, "Attach Ephemeris File");
        folderGridView_ = folderGridView;
    }

    /**
     * @see org.eso.ohs.core.gui.baseaction.ActionSuperclass#actionPerformedImpl(java.awt.event.ActionEvent)
     */
    public void actionPerformedImpl(ActionEvent ev) {
        if(filePreviewDialog_ == null)
            filePreviewDialog_ = new FilePreviewDialog((JFrame)folderGridView_.getTopLevelAncestor(),"Select Report Files",true);
        chooser_ = filePreviewDialog_.getFileChooser();
        filePreviewDialog_.addChoosableFileFilter(ephFilter);

        chooser_.cancelSelection();
        int response = -1;
        ObservationBlock ob = null;
        response =
        chooser_.showDialog(
                folderGridView_.getTopLevelAncestor(),
                "Attach Ephemeris files");
        if (response == JOptionPane.CLOSED_OPTION
            || response == JOptionPane.CANCEL_OPTION)
            return;

        File file = chooser_.getSelectedFile();
        if (response == JFileChooser.APPROVE_OPTION) {
            Long[] idLongs = folderGridView_.getAllSelected();
            stdlog_.debug("Before Attach");
            try {
                new IncrementalActionAutomaton(
                    new IncrementalAttach(file, idLongs),
                    (JFrame) folderGridView_.getTopLevelAncestor(),
                    "Attaching...");
            } catch (IOException e) {
                e.printStackTrace();
                announceIOError(e);
            } catch (ObjectIOException e) {
                e.printStackTrace();
                announceIOError(e);
            } catch (ObjectNotFoundException e) {
                e.printStackTrace();
                announceNoObject(e);
            }

        }

    }
    /**
     * Utility class to duplicate a sequence of OBs, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */

    public class IncrementalAttach implements IncrementalAction {

        /** cached reference to MediaMgr */
        private ObjectManager mMgr_ = ObjectManager.getObjectManager();

        /** array of ids of objects for the file to be attached to */
        private Long[] idLongs_;

        private File file_;

        private boolean cancelPressed = false;

        private StringBuffer alreadyCheckedIn = new StringBuffer();

        /**
         * Constructor.
         *
         * @param frm       window on which to base any dialog boxes.
         * @param device    media containing the objects to be duplicated
         * @param deepCopy  should we duplicate attached objects?
         * @param dir       directory containing the ObsBlocks
         * @param objType   type of object to be duplicated
         * @param idLongs   array of ids of objects to be duplicated
         */
        public IncrementalAttach(File file, Long[] idLongs)
            throws IOException, ObjectNotFoundException, ObjectIOException {

            idLongs_ = idLongs;
            file_ = file;
        }

        /**
         * Number of steps in the action that are still remaining to be done.
         *
         * @return number of steps remaining (eg objects to acted upon)
         */
        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(idLongs_, counter);
        }

        /**
         * Method to perform one incremental step of a larger action.
         *
         * @param stepNo        Counter to indicate the n'th step of a larger action
         */
        public void performStep(int counter) {
            if (idLongs_[counter] != null && !cancelPressed) {
                doAttach(idLongs_[counter].longValue());
            }
        }

        /**
         * Hook for initialisation of action object.
         */
        public void startOfAction() {
        }

        /**
         * Hook for cleanup of action object.
         */
        public void endOfAction() {

        }

        public void doAttach(long id) {
            String fileName = file_.getName();
            try {
                String fileData =
                    FileIOUtils.fileToString(file_.getAbsolutePath());
                EphemerisFile eph = new EphemerisFile();
                eph.setFilename(fileName);
                eph.setFileString(fileData);
                ObservationBlock ob = null;
                stdlog_.debug(">>>>>>>>>>>> ID " + id);
                ob =
                    (ObservationBlock) mMgr_.getBusObj(
                        Media.LOCAL,
                        id,
                        ObservationBlock.class);

                if (ob.isCheckedIn()) {
                    alreadyCheckedIn.append(ob.getName());
                    alreadyCheckedIn.append("\n");
                    return;
                }
                ob.setEphemerisFile(eph);
            } catch (ObjectNotFoundException e) {
                e.printStackTrace();
                announceNoObject(e);
                return;
            } catch (ObjectIOException e) {
                e.printStackTrace();
                announceIOError(e);
                return;
            } catch (IOException e) {
                e.printStackTrace();
                announceIOError(e);
                return;                
            }

        }
    }
}
