package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.p2pp.MainWindow;
import org.eso.ohs.phase2.apps.p2pp.MenuGroups;
import org.eso.ohs.phase2.apps.p2pp.Persistence;
import org.eso.ohs.phase2.apps.p2pp.dbb.P2PPView;
/**
 * Action class to check-out the selected OB.
 *
 * <p>
 * $Id: CheckOutAction.java,v 1.15.4.1 2007/07/20 11:58:33 pnunes Exp $
 * <br>
 * @author Bob Kemp
 */
public class CheckOutAction extends ActionSuperclass {

    /** ObjectContainer for the folder containing the selected OB */
    private ObjectContainer folderView_;

    /**
     * Constructs a check-out action.
     */
    public CheckOutAction(ObjectContainer folderView) {
        super((JComponent) folderView, "Check-out");

        folderView_ = folderView;
    }

    /**
     * Event handler that is called when the menu item, toolbar item,
     * etc is selected.
     *
     * @param ev    event giving more information
     * <br>ChangeLog: A M Chavan, 17-Jan-00: changed wording of message
     */
    public void actionPerformedImpl(ActionEvent ev) {
        Persistence objMgr = Persistence.getInstance();
        Long[] idLongs = null;
        CalibrationBlock[] cbs = null;
        CalibrationBlock cb = null;
        // What kind of object are we checking in?
        idLongs = folderView_.getAllSelected();
        cbs = new CalibrationBlock[idLongs.length];

        int option =
            JOptionPane.showConfirmDialog(
                folderView_.getTopLevelAncestor(),
                new Object[] { "Are you sure you want to check-out the selected objects?" },
                "Continue with Check-Out?",
                JOptionPane.OK_CANCEL_OPTION);
        switch (option) {
            case JOptionPane.CLOSED_OPTION :
                return;
            case JOptionPane.CANCEL_OPTION :
                return;
        }
        MainWindow.getView().hideViews();
        Checker chk = new Checker(folderView_);
        Vector<Long> idVec = new Vector<Long>();

        for (int i = 0; i < idLongs.length; i++) {
            boolean duplicate = false;
            for (int j = i; j < idLongs.length; j++) {
                if (idLongs[j].equals(idLongs[i]) && j > i) {
                    duplicate = true;
                }
            }
            if (!duplicate)
                idVec.addElement(idLongs[i]);
        }
        Long[] noDup = new Long[idVec.size()];
        idVec.copyInto(noDup);

        for (int j = 0; j < noDup.length; j++) {
            try {
                long id = noDup[j].longValue();
                cb =
                    (CalibrationBlock) objMgr.getBusObj(
                        Media.DBASE,
                        id,
                        ((P2PPView) folderView_).getClassForId(
                            Config.getCfg().uniqueToTableId(id)));


                cbs[j] = cb;
            } catch (ObjectNotFoundException e) {
                announceNoObject(e);
                return;
            } catch (ObjectIOException e) {
                announceIOError(e);
                return;
            }
        }

        chk.reportErrors();

        int numErrs = chk.getErrorCount();

        if (cbs.length > numErrs) {
        	//pnunes : not needed anymore
            //Folder folder = ((P2PPView) folderView_).getFolder();

            if (!OHSApplication.ensureAuthenticated()) {
                return;
            }

            // before checking out, check if the observing run folders for all these OBs are there
            for (CalibrationBlock calBlock : cbs) {
            	ObservingRun obsRun = calBlock.getObsRun();



					DirectoryNode dir = objMgr.getDirectoryFor(obsRun);
					Folder folder = (Folder) dir.getObject();
					ObservingRun folderRun = folder.getObsRun();

					if (folderRun == null) {

						JOptionPane
								.showMessageDialog(
										folderView_.getTopLevelAncestor(),
										new Object[] {
												"There is no Observing Run folder in your local cache for the Observation Block with database id = "
														+ calBlock.getDbaseId(),
												"Please download/refresh your observing runs" },
										"Invalid local cache state",
										JOptionPane.ERROR_MESSAGE);

						return;

					}


			}


            new P2PPIncrementalActionAutomaton(
                new IncrementalCheckOut(folderView_, cbs),
                (MenuGroups) folderView_.getTopLevelAncestor(),
                "Checking out...");

        } else { // There are no selected OBs now!
            JOptionPane
                .showMessageDialog(
                    folderView_.getTopLevelAncestor(),
                    new Object[] {
                // A M Chavan, 17-Jan-00: changed wording below
                "There are no selected Observation Blocks",
                    "after invalid objects have been excluded.",
                    " ",
                    "None of the Observation Blocks could be checked-out." },
                    "Nothing selected",
                    JOptionPane.ERROR_MESSAGE);
            return;
        }

    }

    private static class Checker {

        private ObjectContainer folderView_;

        private int notCheckOutStatus_ = 0;

        public Checker(ObjectContainer folderView) {
            folderView_ = folderView;
        }

        /**
        * <br>ChangeLog: A M Chavan, 17-Jan-00: changed wording of message
        */
        public void reportErrors() {
            if (notCheckOutStatus_ > 0) {
                String msg2;
                String msg1;

                if (notCheckOutStatus_ > 1) {
                    msg1 = notCheckOutStatus_ + " Observation Blocks have";
                    msg2 = "They will be ignored.";
                } else {
                    msg1 = "One Observation Block has";
                    msg2 = "It will be ignored.";
                }

                JOptionPane
                    .showMessageDialog(
                        folderView_.getTopLevelAncestor(),
                        new Object[] {
                            msg1 + " can no longer be checked out ",
                            msg2,
                            " ",
                            "An Observation Block can only be checked out",
                    // A M Chavan, 17-Jan-00: changed wording below
                    "if its status is Defined (D) or Rejected (-)" },
                        "Error",
                        JOptionPane.ERROR_MESSAGE);
            }
        }

        public int getErrorCount() {
            return notCheckOutStatus_;
        }
    }

    /**
     * Utility class to check out a sequence of OBs, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public static class IncrementalCheckOut implements IncrementalAction {

        /** ObjectContainer for the folder containing the selected OB */
        private ObjectContainer folderView_;

        /** cached reference to Persistence */
        private Persistence objMgr = Persistence.getInstance();

        /** array of ids for OBs to be checked-out */
        private CalibrationBlock[] cbs_;

        private Class objType_;

        /**
         * Constructor.
         *
         * @param frm	    window on which to base any dialog boxes.
         * @param idLongs   ids of OBs that are to be checked-out.
         */
        public IncrementalCheckOut(
            ObjectContainer folderView,
            CalibrationBlock[] cbs) {
            folderView_ = folderView;
            cbs_ = cbs;
        }

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(cbs_, counter);
        }

        public void startOfAction() {
        }

        public void endOfAction() {
            if (folderView_ instanceof P2PPView) {
                ((P2PPView) folderView_).fireDbbQueryStartedAction();
            }
        }

        /**
         * Method to check-out the n'th element of idLongs, where n = counter.
         *
         * @param counter   counter of which element of idLongs to checkOut
         * @return	    true if the checking-in should continue
         *		    and false otherwise.
         */
        public void performStep(int counter) {
            if (cbs_[counter] != null) {
                Persistence objMgr = Persistence.getInstance();

                doCheckOut(cbs_[counter]);
            }
        }

        /**
         * Method to check-out a given ObservationBlock.
         *
         * @param id	    id of ObservationBlock to check-out
         */
        public void doCheckOut(CalibrationBlock dbCB) {
            try {
                objMgr.checkOut(dbCB);
            } catch (ObjectIOException e) {
                ErrorMessages.announceIOError(null, e);
            } catch (ObjectNotFoundException e) {
                ErrorMessages.announceNoObject(null, e);
            }
        }

    }

}
