package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.phase2.apps.p2pp.HierarchyView;
import org.eso.ohs.phase2.apps.p2pp.Persistence;

/**
 * Action class to delete the selected directory.
 * 
 * <p>
 * $Id: DeleteDirectoryAction.java,v 1.9 2006/04/27 13:25:00 pnunes Exp $
 */

public class DeleteDirectoryAction extends ActionSuperclass {

    private static Logger stdlog_ =
        Logger.getLogger(DeleteDirectoryAction.class);

    /** medium on which the directory exists */
    Media device_;

    /** window on which any dialog boxes should be based, eg for errors */
    HierarchyView hView_;

    private JFrame parentFrame_;

    /**
     * Constructs a delete directory action.
     * 
     * @param device   medium containing the directory.
     * @param path     path in HierarchyView tree to the selected directory.
     */
    public DeleteDirectoryAction(
        JFrame parentFrame,
        Media device,
        HierarchyView hView) {
        super(hView, "Delete");

        device_ = device;
        hView_ = hView;
        parentFrame_ = parentFrame;
    }

    /**
     * Event handler for the delete directory action, eg from a menu item.
     * 
     * @param ev       user-initiated ActionEvent to delete a directory.
     */
    public void actionPerformedImpl(ActionEvent ev) {
        Persistence objMgr = Persistence.getInstance();
        Folder folder;

        try {
            folder = hView_.getSelectedFolder();

            if (folder == null) {
                JOptionPane.showMessageDialog(
                    parentFrame_,
                    new Object[] {
                        "There is no selected folder.",
                        "Which folder should be deleted?",
                        },
                    "Nothing selected",
                    JOptionPane.ERROR_MESSAGE);
                return;
            }

            DirectoryNode parent = folder.getNode().getParent();

            // Cannot delete root node or it's immediate children
            // (It's children are the Observing Run folders)
            if (parent == null || parent.getParent() == null) {
                JOptionPane.showMessageDialog(
                    hView_.getTopLevelAncestor(),
                    new Object[] { "Top-level folders cannot be deleted.", },
                    "Cannot delete",
                    JOptionPane.ERROR_MESSAGE);
                return;
            }

            Summary[] summ = objMgr.listObjects(device_, folder.getNode());

            if (summ.length > 1
                || (summ.length == 1
                    && summ[0].getObjectClass() != Folder.class)) {
                JOptionPane.showMessageDialog(
                    parentFrame_,
                    "Cannot delete folder because it still"
                        + " contains data.\n"
                        + "To delete the folder, delete its contents"
                        + " first and\n"
                        + "then delete the folder.",
                    "Cannot delete folder",
                    JOptionPane.WARNING_MESSAGE);
                return;
            }
            JFrame frm = new JFrame("Deletion Dialog");
            int option =
                JOptionPane.showConfirmDialog(
                    frm,
                    "Are you sure you want to delete folder "
                        + folder.getName()
                        + " ?",
                    "Delete",
                    JOptionPane.OK_CANCEL_OPTION,
                    JOptionPane.QUESTION_MESSAGE,
                    null);

            switch (option) {
                case JOptionPane.CLOSED_OPTION :
                    return;
                case JOptionPane.CANCEL_OPTION :
                    return;
            }
            boolean deleted = objMgr.deleteDirectory(device_, folder.getNode());
            if (!deleted) {
                JOptionPane.showMessageDialog(
                    parentFrame_,
                    "Cannot delete folder",
                    "Cannot delete folder",
                    JOptionPane.WARNING_MESSAGE);
                return;
            }
        } catch (ObjectIOException e) {
            announceIOError(e);
        }
    }

}
