package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.instruments.ParamFileParameter;
import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.p2pp.MainWindow;
import org.eso.ohs.phase2.apps.p2pp.Persistence;

/**
 * Action class to re-read the target information
 * from the Parm file.
 *
 * @author cmavros 20-11-01
 */

public class RefreshTargetDataAction extends ActionSuperclass {

    private ObjectContainer folderView_;

    private Media device_;

    protected StringBuffer msgLog_ = null;

    /**
     * Constructs a RefreshTargetDataAction action.
     */
    public RefreshTargetDataAction(Media device, ObjectContainer folderView){

	super((JComponent)folderView, "Refresh Object Target Data");
        device_ = device;
	folderView_ = folderView;
            
    }


    /**
     * Event handler for the RefreshTargetDataAction action.
     * 
     * @param ev user-initiated ActionEvent to quit.
     */
    public void actionPerformedImpl(ActionEvent evt) {     
        Long [] idLongs = folderView_.getAllSelected();
	if (idLongs.length == 0) {
	    JOptionPane.showMessageDialog(
	      folderView_.getTopLevelAncestor(),
	      new Object[] {
	          "There is no selected object to display.",
	          "Which object should be displayed?",
	      },
	      "Nothing selected",
	      JOptionPane.ERROR_MESSAGE);
	    return;
	}

        new IncrementalActionAutomaton(
            new IncrementalRefresh(idLongs),
            (JFrame)folderView_.getTopLevelAncestor(), "Refreshing...");
    }

    public class IncrementalRefresh implements IncrementalAction  {

	private ObjectManager objMgr = ObjectManager.getObjectManager();
	private Long[] idLongs_;
	private Class objType_;

	public IncrementalRefresh(Long[] ids) {
		idLongs_ = ids;
	}

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(idLongs_, counter);
	}

	public void startOfAction() { 
            msgLog_ = new StringBuffer();
            msgLog_.append("\nTarget Data Refresh\n");
            msgLog_.append("------------------- \n");
        }

	public void endOfAction() { 
            JOptionPane.showMessageDialog( MainWindow.getPrincipalWindow(),
                msgLog_.toString(), "Refresh Complete", JOptionPane.PLAIN_MESSAGE ); 
        }

	public void performStep(int counter) {
	    if (idLongs_[counter] != null) {
		doRefresh(idLongs_[counter]);
            }
	}

    /**
     * Utility method to refresh a target data file
     *
     * @param id business object id
     */
    public void doRefresh(Long id) {
        try {
	    Persistence objMgr = Persistence.getInstance();
            Class objType = Config.getCfg().getClassFromId(id.longValue());
	    BusinessObject bo = (BusinessObject)objMgr.getBusObj(device_, id.longValue(), objType);

            if(objType.equals(ObservationBlock.class)){ 
	        ObservationBlock obsBlock = (ObservationBlock)bo;
                String obsName = "";
                try{
                    obsName = ((ObservationBlock)bo).getObsRun().getName();
                }
                catch(NullPointerException e) {
                    ErrorMessages.announceNoObject(
                      folderView_.getTopLevelAncestor(), e);
                    return;
                }
                String obsId = bo.getName();
                TemplateSignature tsig = null;
                /** Check if ob is check-in */
                if(!obsBlock.isCheckedIn()){
                    /** Check if the template signature */
                    if((tsig = obsBlock.getTemplateSignature()) != null){
                        if(tsig.getTemplateType().equals("acquisition")){
                            Parameter plist[] = tsig.getParamList();
                            for(int i = 0; i < plist.length; i++){
                                if(plist[i].getType().equals("paramfile")) {
                                    ParamFileParameter pfp = (ParamFileParameter) plist[i];
                                    try {
                                        obsBlock.getTarget().
                                        updateFromParamFileParameter( pfp );
                                    }
                                    catch( Exception e ) {
                               	        String name = pfp.getParameterName();
                                        String msg = "Error while refreshing paramfile\n" + name 
                                          + ":\n" + e.getMessage() 
                                          + "\nTarget info was not updated.";
                                      
                                        JOptionPane.showMessageDialog( MainWindow.getPrincipalWindow(),
                                          msg, "Error in paramfile", JOptionPane.ERROR_MESSAGE ); 
                                    }
                                    try {
                                    	ConstraintSet cs = obsBlock.getConstraintSet();
                                    	if (cs != null) {
                                            cs.updateFromParamFileParameter( pfp );
                                        }
                                    }
                                    catch( Exception e ) {
                               	        String name = pfp.getParameterName();
                                        String msg = "Error while refreshing paramfile\n" + name 
                                          + ":\n" + e.getMessage() 
                                          + "\nConstraint Set info was not updated.";
                                       
                                        JOptionPane.showMessageDialog( MainWindow.getPrincipalWindow(),
                                          msg, "Error in paramfile", JOptionPane.ERROR_MESSAGE ); 
                                    }		
                                }
                            }
                            /** Display a message that the refresh completed. */
                            msgLog_.append("\n" + obsName + " / " + obsId + " - Refresh successfully\n"); 
                       }
                    }
                    else{
                       msgLog_.append("\n" + obsName + " / " + obsId + " -  No Target Data found\n");
                    }
                }
                else {
                    msgLog_.append("\n" + obsName + " / " + obsId + 
                          " - Cannot refresh a checked-in Obs Block\n");
                }
            }
        }
       	catch(ObjectNotFoundException e) {
	        ErrorMessages.announceNoObject(folderView_.getTopLevelAncestor(), e);
        }
	catch(ObjectIOException e) {
	    ErrorMessages.announceIOError(
	      folderView_.getTopLevelAncestor(), e);
        }

    }

    }

}
