package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.Component;
import java.awt.event.ActionEvent;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.SimpleInputDialog;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.phase2.apps.p2pp.HierarchyView;
import org.eso.ohs.phase2.apps.p2pp.Persistence;

/**
 * Action class to rename the selected folder
 * 
 * <p>
 * $Id: RenameDirectoryAction.java,v 1.9 2006/04/27 13:25:00 pnunes Exp $
 */

public class RenameDirectoryAction extends ActionSuperclass {
    /** folder hierarchy for selecting the folder to rename */
    HierarchyView hView_;

    private JFrame parentFrame_;
    
    /**
     * Constructs a "copy BusinessObject" action.
     * 
     * @param folderView  GUI view of (summary information in) the folder.
     */
    public RenameDirectoryAction(JFrame parentFrame,HierarchyView hView) {
	super(hView, "Rename");

	hView_ = hView;
	parentFrame_ = parentFrame;
    }

    /**
     * Event handler for the create BusinessObject action.
     * 
     * @param ev       user-initiated ActionEvent to create an OD.
     */
    public void actionPerformedImpl(ActionEvent ev) {
	try {
	    Folder folder = hView_.getSelectedFolder();

	    if (folder == null) {
		JOptionPane.showMessageDialog(
		    parentFrame_,
		    new Object[] {
		      "There is no selected folder.",
		      "Which folder should be renamed?",
		    },
		    "Nothing selected",
		    JOptionPane.ERROR_MESSAGE
		);
		return;		// nothing selected?
	    }

	    DirectoryNode parent = folder.getNode().getParent();

		// Cannot rename root node or it's immediate children
		// (It's children are the Observing Run folders)
	    if (parent == null || parent.getParent() == null) {
		JOptionPane.showMessageDialog(
		    parentFrame_,
		    new Object[] {
		      "Top-level folders cannot be renamed.",
		    },
		    "Cannot rename",
		    JOptionPane.ERROR_MESSAGE
		);
		return;
	    }

	    renameDirectoryViaDialog(parentFrame_,hView_, folder);
	}              
	catch (ObjectIOException e) {
	    announceIOError(e);
	}
    }

    public boolean renameDirectoryViaDialog(JFrame parentFrame,JComponent comp,
                                                Folder folder) {
        Persistence objMgr = Persistence.getInstance();
        String oldName = folder.getNode().getDescription();
        JTextField newNameTF = new JTextField();

        newNameTF.setText(oldName);

        SimpleInputDialog dlg = new SimpleInputDialog(
                            parentFrame_,
                            "Enter new name",
                            "Enter the new name for the folder: " + oldName,
                            new String[] { "New name:" },
                            new Component[] { newNameTF }
        );

        dlg.setVisible(true);

        if (dlg.getUserChoice() == JOptionPane.OK_OPTION) {
            objMgr.renameDirectory(folder.getMedia(), folder.getNode(),
                                    newNameTF.getText());
            return true;
        }
        else {
            return false;               // Cancelled or closed window?
        }
    }

}

