package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.Dimension;
import java.awt.event.ActionEvent;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.ReadmeAttribute;
import org.eso.ohs.instruments.ReadmeAttributesFactory;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.phase2.apps.p2pp.HierarchyView;

/**
 * 
 * Class VerifyReadmeFileAction;
 * Verify that a readme is valid  based on the data in the .xsd files
 * in the readme directory of the instrument packages. 
 *
 * TODO Javadoc
 *
 * @version $Revision: 1.10 $
 * @author tcanavan, Nov 15, 2004
 */

// $Id: VerifyReadmeFileAction.java,v 1.10 2006/04/27 13:25:00 pnunes Exp $



public class VerifyReadmeFileAction extends ActionSuperclass {
    
    
    private Logger stdlog_ = (Logger) Logger.getInstance(VerifyReadmeFileAction.class);

    private HierarchyView hView_; 
    
    
    
    /**
     * Status window which displays reports on imports.     
     */

    private static TextDisplayWidget log_ =
        new TextDisplayWidget("Readme Verification Report");

    
    
    /**
     * Verify the readme file
     * @param comp
     */
    public VerifyReadmeFileAction(HierarchyView hView) {
        super(hView,"Verify Readme");
        hView_ = hView;
    }

    
  

    /**
     * Event handler for verifying a readme file. .
     * 
     * @param ev       user-initiated ActionEvent to verify a readme file
     */
    public void actionPerformedImpl(ActionEvent ev) {
        try {
            Folder folder = hView_.getSelectedFolder();
            if (folder != null) {
                ObservingRun obsRun = folder.findObsRun();
                stdlog_.debug(obsRun.getObsMode());
                if (!obsRun
                    .getObsMode()
                    .equalsIgnoreCase(ObservingRun.OBSMODE_STANDARD)) {
                    JOptionPane.showMessageDialog(
                        hView_,
                        new Object[] {
                            "The readme file option only applies to Service mode Observing Runs",
                            },
                        "Readme Available",
                        JOptionPane.ERROR_MESSAGE);
                    return;
                }
                Instrument inst = InstrumentList.getInstance().getInstrument(obsRun.getInstCode(),obsRun.getIPVersion());
                
                
                if(!inst.hasReadmeData()) {
                        JOptionPane.showMessageDialog(
                            hView_,
                            new Object[] {
                                "There is no readme configuration available for this observing run.",
                                "Try download refresh observing runs",
                                "If this still does not work then contact usd-help@eso.org",
                                },
                            "Readme Available Error",
                            JOptionPane.ERROR_MESSAGE);
                        return;
                }

                
                if(obsRun.getReadme() != null) {                                        
                      Readme readme = obsRun.getReadme();
                      stdlog_.debug("OLD IP VERSION " + obsRun.getIPVersion());
                      stdlog_.debug("NEW IP VERSION " + obsRun.getReadme().getIpVersion());
                      if(obsRun.getIPVersion()!=obsRun.getReadme().getIpVersion()) {
                          boolean isEquals = ReadmeAttributesFactory.isEqualsMeta(inst,readme.getRedmeAttributes());
                          ReadmeAttribute [] newReadmeAttributes = ReadmeAttributesFactory.refreshReadmeAttributes(inst,readme.getRedmeAttributes());
                          readme.setIpVersion(obsRun.getIPVersion());
                          readme.setRedmeAttributes(newReadmeAttributes);
                          if(!isEquals) {
                              JOptionPane.showMessageDialog(
                                  hView_,
                                  new Object[] {
                                      "This Readme has been updated to use",
                                      "the Instrument Package specified in the Observing Run." },
                                  "Update to new Instrument Package",
                                  JOptionPane.INFORMATION_MESSAGE);

                          }
                          // fire event to save new ip version
                          obsRun.setReadme(readme);
                      }
                      log_.setSize(new Dimension(700, 300));
                      log_.clearTextArea();
                      log_.setVisible(true);
                      String report = readme.verifyReport();
                      if(report.length() > 0) {                          
                          log_.setTextArea(report.toString());
                      } else {
                          log_.setTextArea("Readme Verified Succesfully");
                      }
                } else {
                    JOptionPane.showMessageDialog(
                        hView_,
                        new Object[] {
                            "Readme file not created yet for this observing run"
                            },
                        "Readme Not Created",
                        JOptionPane.ERROR_MESSAGE);

                }
            }
        } catch (ObjectIOException e) {
            announceIOError(e);
        }
    }
}
