package org.eso.ohs.phase2.apps.p2pp.dbb;

import java.awt.Container;

import javax.swing.CellEditor;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.ObjectSelectionEvent;
import org.eso.ohs.persistence.ObjectSelectionListener;
import org.eso.ohs.persistence.ObjectSelector;
import org.eso.ohs.phase2.apps.p2pp.Persistence;

public class P2PPView
    extends DbbView
    implements ObjectContainer, ObjectSelector {

    private Class objType_;

    private EventListenerList listenerList_;

    private Media device_;

    public P2PPView(Media device, DbbSqlEngine sqlEngine) {
        super(sqlEngine);

        device_ = device;

        listenerList_ = new EventListenerList();

    }

    public void createDefaultListListener() {
        addListSelectionListener(new ListListener(summaryTable_.getTable()));
    }

    public Long[] getAll() {
        StorableObject sobj;
        Class objTypeSelected;
        Config cfg = Config.getCfg();
        int rowCount = summaryTable_.getRowCount();
        Long[] ids = new Long[rowCount];
        for (int i = 0; i < rowCount; i++) {
            Integer id = (Integer) summaryTable_.getValueAt(i, 0);
            String type = (String) getValueAt(i, 1);
            if (type.equalsIgnoreCase("O"))
                objTypeSelected = ObservationBlock.class;
            else
                objTypeSelected = CalibrationBlock.class;
            ids[i] =
                new Long(cfg.tableToUniqueId(id.longValue(), objTypeSelected));
        }
        return ids;
    }

    public void selectAllObjects() {
        ListSelectionModel lsm = summaryTable_.getTable().getSelectionModel();
        CellEditor ed = summaryTable_.getTable().getCellEditor();
        if (ed != null) {
            ed.stopCellEditing();
        }
        lsm.addSelectionInterval(0, summaryTable_.getModel().getRowCount() - 1);
    }

    public Long[] getAllSelected() {
        int rows[] = getSelectedRows();
        Long[] ids = new Long[rows.length];
        Class objTypeSelected;
        Config cfg = Config.getCfg();

        for (int i = 0; i < rows.length; i++) {
            Integer id = (Integer) getValueAt(rows[i], 0);
            String type = (String) getValueAt(rows[i], 1);

            if (type.equalsIgnoreCase("O"))
                objTypeSelected = ObservationBlock.class;
            else
                objTypeSelected = CalibrationBlock.class;

            ids[i] =
                new Long(cfg.tableToUniqueId(id.longValue(), objTypeSelected));
        }

        return ids;
    }

    public Class getClassForId(long id) throws NullPointerException {
        int rowCount = summaryTable_.getRowCount();

        for (int i = 0; i < rowCount; i++) {
            Integer rowid = (Integer) getValueAt(i, 0);
            String type = (String) getValueAt(i, 1);
            if (rowid.longValue() == id) {
                if (type.equalsIgnoreCase("O"))
                    return ObservationBlock.class;
                if (type.equalsIgnoreCase("C"))
                    return CalibrationBlock.class;
            }
        }
        return null;
    }

    public Class getDisplayedObjectClass() {
        int row = getSelectedRow();
        // Java1.4 port if (row >= 0 && row < summaryTable_.getRows()) {
        if (row >= 0 && row < getRows()) {
            String type = (String) getValueAt(row, 1);
            if (type.equalsIgnoreCase("O"))
                return ObservationBlock.class;
            else
                return CalibrationBlock.class;
        }
        return null;
    }

    public Folder getFolder() {
        return null;
    }

    public Media getMedia() {
        return device_;
    }

    public Long getSelected() {
        Long dbid;
        Class objTypeSelected;
        /** cmavros Return null if nothing is selected */
        try {
            int row = getSelectedRow();
            Integer id = (Integer) getValueAt(row, 0);
            String type = (String) getValueAt(row, 1);

            if (type.equalsIgnoreCase(CalibrationBlock.OBSBLK_TYPE))
                objTypeSelected = ObservationBlock.class;
            else
                objTypeSelected = CalibrationBlock.class;

            dbid =
                new Long(
                    Config.getCfg().tableToUniqueId(
                        id.longValue(),
                        objTypeSelected));
        } catch (Exception e) {
            return null;
        }

        return dbid;
    }

    public BusinessObject getDisplayedObject() {
        BusinessObject bo = null;
        Long idLong = getSelected();
        Persistence objMgr = Persistence.getInstance();

        try {
            if (idLong != null) {
                bo =
                    (BusinessObject) objMgr.getBusObj(
                        getMedia(),
                        idLong.longValue(),
                        Config.getCfg().getClassFromId(idLong.longValue()));
            }
        } catch (ObjectNotFoundException e) {
            ErrorMessages.announceIOError(this.getTable().getTable(), e);
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(this.getTable().getTable(), e);
        }

        return bo;
    }

    public Container getTopLevelAncestor() {
        return summaryTable_.getTable().getTopLevelAncestor();
    }

    public void setDisplayedObjectClass(Class objType) {
        objType_ = objType;
    }

    protected void fireObjectSelectionEvent(
        Media device,
        long id,
        Class objType,
        boolean forceShow,
        boolean forceToFront,
        int rowCount) {

        Object[] listeners = listenerList_.getListenerList();
        ObjectSelectionListener lnr = null;
        Class visibleObjType = getDisplayedObjectClass();
        ObjectSelectionEvent selnEvt =
            new ObjectSelectionEvent(
                this,
                device,
                id,
                visibleObjType,
                forceShow,
                forceToFront,
                rowCount);

        for (int i = 0; i < listeners.length; i += 2) {
            lnr = (ObjectSelectionListener) listeners[i + 1];
            lnr.objectSelected(selnEvt);
        }
    }

    /**
     * Adds a new listener to be notified of selection changes.
     * 
     * @param lnr the listener
     */
    public void addObjectSelectionListener(ObjectSelectionListener lnr) {
        listenerList_.add(ObjectSelectionListener.class, lnr);
    }

    /**
     * Removes a new listener, which will no longer be notified of
     * selection changes.
     * 
     * @param lnr the listener
     */
    public void removeObjectSelectionListener(ObjectSelectionListener lnr) {
        listenerList_.remove(ObjectSelectionListener.class, lnr);
    }

    public class ListListener implements ListSelectionListener {

        JTable summaryTable_;

        public ListListener() {
            summaryTable_ = null;
        }

        public ListListener(JTable summaryTable) {
            summaryTable_ = summaryTable;
        }

        public void valueChanged(ListSelectionEvent ev) {
            if (ev.getValueIsAdjusting()) {
                return;
            }
            ListSelectionModel lsm = (ListSelectionModel) ev.getSource();
            int selectedRow = lsm.getMinSelectionIndex();
            Long idObj = getSelected();

            long id = ObjectSelectionEvent.NO_OBJECT;

            if (!lsm.isSelectionEmpty()) {
                if (selectedRow >= 0
                    && selectedRow < summaryTable_.getModel().getRowCount()) {
                    id = idObj.longValue();
                }
                fireObjectSelectionEvent(
                    Media.DBASE,
                    id,
                    Config.getCfg().getClassFromId(id),
                    false,
                    true,
                    summaryTable_.getModel().getRowCount());
            }
            if (id != ObjectSelectionEvent.NO_OBJECT)
                fireObjectSelectionEvent(
                    Media.DBASE,
                    id,
                    Config.getCfg().getClassFromId(id),
                    false,
                    true,
                    summaryTable_.getModel().getRowCount());
            else
                fireObjectSelectionEvent(
                    Media.DBASE,
                    id,
                    ObservationBlock.class,
                    false,
                    true,
                    summaryTable_.getModel().getRowCount());
        }
    }

}
