// $Id: ODTemplateTable.java,v 1.12 2006/04/27 13:25:00 pnunes Exp $

package org.eso.ohs.phase2.apps.p2pp.views;

import java.awt.Container;
import java.awt.KeyboardFocusManager;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Enumeration;
import java.util.EventObject;

import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JToolTip;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.TemplateSignature;

/**
 * This JTable class delegates to the ODGridModel and the
 * AcquisitionModel. he ODGridModel gets displayed in the OD view and the
 * AcquisitonModel gets displayed in the OBView.
 * 
 * The AcquisitionModel can contain an acquisition template in 
 * column 0. The acquiition model handles this situation and deleegates
 * any neccessary calls to the currently active ODGridModel.
 * This JTable class handles both models
 *
 * $Id: ODTemplateTable.java,v 1.12 2006/04/27 13:25:00 pnunes Exp $
 * @author Tim Canavan
 *
 */

public class ODTemplateTable extends JTable {

    /**
     * This is the number of columns that are allowed before the horizontal
     * scroll bar is enabled to allow scrolling through the columns. This
     * only applies if inScrollPane_ is true, indicating that the table is
     * inside a scroll pane.
     */
    protected static int maxColumnCountBeforeScrolling = 6;

    private EventObject eventObject_;

    /**
     * Header for the JTable
     */

    private JTableHeader tableHeader_;

    /**
     * Reference to the factory that generates 
     * the neccessary editor controls for the grid
     *
     */

    private EditorFactory editFact_;

    /**
     * Referennce to the renderer factory that generates
     * the controls when a particular cell is rendered
     */

    private RendererFactory rendFact_;

    /**
     * Whether the grid is made read only or not. 
     * This is currently not used but maybe used in the future
     */

    private boolean readOnly_;

    /**
     * Listener to mouse clicks on the column header
     */

    private ColumnListener mouseListener_;

    /**
     * Listener to mouse motion events
     */

    private ColumnListener mouseMotionListener_;

    /**
     * Ancestor control for the JTable
     */

    private Ancestor ancestor_;

    /**   
     * MultiLineToolTip to display help & range text.
     */

    private JToolTip multiLineToolTip_ = new JToolTip();

    /**
     * Used to indicate if this table is in a scroll pane or not. If so,
     * when the table exceeds maxColumnCountBeforeScrolling in columns, 
     * the horizontal scroll bar will be enabled to allow scrolling
     * through the template columns.
     */
    private boolean inScrollPane_ = false;

    /**
     * Create a reference to a JTable
     * @param model initialise this table with this model
     */

    ODTemplateTable(TableModel model) {
        super(model);
        init();
    }

    /**
     * Create a table using the default model
     */

    ODTemplateTable() {
        super();
        init();
    }

    /**
     * Override JComponents createToolTip so the grid 
     * can display multi line tootip text.
     */

    public JToolTip createToolTip() {

        return multiLineToolTip_;
    }

    /**
     * Initialise the mouse listeners and the 
     * data model for the grid
     */

    public void init() {
        setAutoCreateColumnsFromModel(true);
        //getTableHeader().setUpdateTableInRealTime(true);
        rendFact_ = new RendererFactory();
        setDefaultRenderer(Object.class, rendFact_);
        editFact_ = new EditorFactory(new JTextField());
        setDefaultEditor(Object.class, editFact_);
        if (getModel() != null)
            addMouseListeners();
        setCellSelectionEnabled(true);
        //setAutoResizeMode(AUTO_RESIZE_OFF);
        setSurrendersFocusOnKeystroke(true);
    }

    /**
     * Used to indicate to this table that is is nested in
     * a scroll pane.
     */
    public void setIsInScrollPane(boolean inScrollPane) {
        inScrollPane_ = inScrollPane;
    }

    /**
     * @return true if this table is nested in a scroll pane
     */
    public boolean isInScrollPane() {
        return inScrollPane_;
    }

    /**
     * Set the selection for a single cell in the grid
     * @param row row
     * @param column column
     */

    public void setRowColumn(int row, int column) {
        if (column > 0) {
            setColumnSelectionInterval(column, column);
        }
        if (row > 0) {
            setRowSelectionInterval(row, row);
        }
    }

    /**
     * Set the data model for the table
     * param data model
     */

    public void setModel(TableModel model) {
        if (getCellEditor() != null) {
            getCellEditor().stopCellEditing();
        }
        if (dataModel instanceof AcquisitionModel) {
            ((AcquisitionModel) dataModel).saveChanges();
        } else if (dataModel instanceof ODGridModel) {
            ((ODGridModel) dataModel).saveChanges();
        }
        super.setModel(model);
        addMouseListeners();

    }

    /**
     * Process the key event for the JTable
     */

    public void processKeyEvent(KeyEvent e) {
        super.processKeyEvent(e);
    }

    /**
     * Overrides JTable's table changed event so 
     * @param e TableModelEvent
     */

    public void tableChanged(TableModelEvent e) {
        super.tableChanged(e);
        if (e == null || e.getFirstRow() == TableModelEvent.HEADER_ROW) {
            sizeColumnsToFit(-1);
            createDefaultSelectionModel();
            if (getModel().getColumnCount() > 0) {
                int currColumn = ((TemplateModel) getModel()).getCurrColumn();
                // Java1.4 port setRowSelectionInterval(0,0);
                clearSelection();
                setColumnSelectionInterval(currColumn, currColumn);
                TableColumnModel cm = getColumnModel();
                //                 System.out.println("TABLE CHANGED: "+getModel().getColumnCount()+" "+isInScrollPane());
                if (getModel().getColumnCount()
                    <= maxColumnCountBeforeScrolling
                    || !isInScrollPane()) {
                    /**
                     * Either "too few" columns to enable the scroll bar, or this table
                     * is not in a scroll pane.
                     */
                    setAutoResizeMode(AUTO_RESIZE_ALL_COLUMNS);
                } else {
                    /**
                     * The table is in a scroll pane and there are "enough" columns
                     * to enable the scroll bar.
                     */
                    setAutoResizeMode(AUTO_RESIZE_OFF);
                    for (Enumeration enum0 = cm.getColumns();
                        enum0.hasMoreElements();
                        ) {
                        TableColumn tc = (TableColumn) enum0.nextElement();
                        tc.setPreferredWidth(200);
                    }
                }
            } else {
                //Java1.4 port  setColumnSelectionInterval(-1,-1);           
                //Java1.4 port setRowSelectionInterval(-1,-1);
                clearSelection();
            }
        }
    }

    /**
     * Add mouse listerers to the table header so 
     * columns can be moved around
     * Also an ancestor listener which will make sure
     * the table will stop edting when it looses focus  
     *
     */

    public void addMouseListeners() {
        tableHeader_ = getTableHeader();
        if (tableHeader_ != null) {
            if (mouseListener_ == null) {
                mouseListener_ = new ColumnListener();
                tableHeader_.addMouseListener(mouseListener_);
                tableHeader_.addMouseMotionListener(mouseListener_);
            }
        }
        if (ancestor_ == null) {
            ancestor_ = new Ancestor();
            addAncestorListener(ancestor_);
        }
    }

    /**
     * Remove mouse listeners
     */

    private void removeMouseListeners() {
        if (mouseListener_ != null) {
            tableHeader_.removeMouseListener(mouseListener_);
            tableHeader_.removeMouseMotionListener(mouseListener_);
        }
    }

    /**
     * This had to be overridden because characters were not appearing
     * in the text field if we put a constraint on data entry like 
     * only entering numeric values. This is a known problem
     * in Swing lets hope they fix it one day
     */

    //public boolean editCellAt(int row, int column, EventObject e) {
    //    eventObject_ = e;
    //   boolean edit = super.editCellAt(row,column,e);
    //   if(edit) {         
    //       editorComp.repaint();
    //    }
    //    return edit;      
    // }

    private boolean didIt = false;
    private String badGuy = "javax.swing.JTable$CellEditorRemover";
    
    /**
     * This call sets the eventObject_ memeber to drive the selection
     * of the editor object. The eventObject_ call is set to null
     * at the end of the call.
     * 
     * @see javax.swing.JTable#editCellAt(int, int, EventObject)
     */
    public boolean editCellAt(int row, int column, EventObject e) {
        eventObject_ = e;
        if (!super.editCellAt(row, column, e)) {
            eventObject_ = null;
            return false;
        }
        if (didIt) {
            eventObject_ = null;
            return true;
        }
        didIt = true;
        KeyboardFocusManager fm =
            KeyboardFocusManager.getCurrentKeyboardFocusManager();
        java.beans.PropertyChangeListener[] pcls =
            fm.getPropertyChangeListeners("focusOwner");
        for (int i = 0; i < pcls.length; i++) {
            if (pcls[i].getClass().getName().equals(badGuy)) {
                fm.removePropertyChangeListener("focusOwner", pcls[i]);
                break;
            }
        }
        eventObject_ = null;
        return true;
    }
    
    /**
     * @see javax.swing.JTable#editCellAt(int, int)
     */
    public boolean editCellAt(int row, int column) {
        return editCellAt(row, column, null);
    }

    public void setColumnSelectionInterval(int column, int column1) {
        super.setColumnSelectionInterval(column, column1);
    }

    /**
     * Stop cell editing 
     */

    public void stopCellEditing() {
        //System.out.println("Stop Cell Editing " + getSelectedColumn());
        if (isEditing()) {
            TableCellEditor ed;
            ed = getCellEditor();
            System.out.println(ed);
            ed.stopCellEditing();
        }
        //System.out.println("Stop Cell Editing " + getSelectedColumn());
    }

    /**
     * The eventObject_ member used here was set in the editCellAt()
     * method. The properties of the event are used to determine which
     * editor should be used.
     * This works on the assumption that getCellEditor() is called
     * within the editCellAt() call.
     * @see javax.swing.JTable#getCellEditor(int, int)
     */
    public TableCellEditor getCellEditor(int row, int column) {
        boolean shiftPressed = false;
        //System.out.println("Event Object " + eventObject_);
        TableCellEditor ed = super.getCellEditor(row, column);
        if (eventObject_ instanceof MouseEvent) {
            if (((MouseEvent) eventObject_).isShiftDown()
                || ((MouseEvent) eventObject_).isMetaDown()
                || ((MouseEvent) eventObject_).isAltDown()) {
                shiftPressed = true;
            }
        } else if (eventObject_ instanceof KeyEvent) {
            if (((KeyEvent) eventObject_).isMetaDown())
                shiftPressed = true;
        }
        editFact_.setShiftPressed(shiftPressed);
        return ed;
    }

    /**    
     * This class is added as a listener to the table so that columns can 
     * be dragged and dropped from within the table;
     */

    private class ColumnListener
        implements MouseListener, MouseMotionListener {

        /**
         * The index of the column when the user puhes on the 
         * column header
         */
        private int pressedIndex_;

        /**
         * Initialised when a resize event happens so this class does not
         * override the default behaviour of the resizing of columns
         */

        private TableColumn resize;

        /** 
         * Captures the mouse releaded event and call the moveColumn method 
         * in the tableModel and then redraw the table.
         * @param e mouse released event
         */

        public void mouseReleased(MouseEvent e) {
            if (resize != null) {
                return;
            }
            TemplateModel model = (TemplateModel) getModel();
            if (((TemplateModel) getModel()).isLabel(pressedIndex_)) {
                JOptionPane.showMessageDialog(
                    null,
                    "Cannot move template description columns\n"
                        + "Only columns with a number heading can be moved",
                    "Invalid Move",
                    JOptionPane.WARNING_MESSAGE);
                ((AbstractTableModel) model).fireTableStructureChanged();
                return;
            }
            Point p = e.getPoint();
            int releasedIndex = tableHeader_.columnAtPoint(p);
            if (releasedIndex == -1)
                releasedIndex =
                    ((AbstractTableModel) model).getColumnCount() - 1;
            TableColumnModel columnModel = getColumnModel();
            if (pressedIndex_ != -1
                && releasedIndex != -1
                && pressedIndex_ != releasedIndex) {
                model.moveColumn(
                    pressedIndex_,
                    releasedIndex);
            }

            if (getModel().getColumnCount() > 0) {
                setColumnSelectionInterval(
                    ((TemplateModel) getModel()).getCurrColumn(),
                    ((TemplateModel) getModel()).getCurrColumn());
                setRowSelectionInterval(0, 0);
                ObservationDescription od =
                    ((TemplateModel) getModel()).getOD();
                TemplateSignature[] tsfs =
                    ((TemplateModel) getModel()).getTemplateSignatures();
                od.setSignatures(tsfs);
            }
        }

        /**
         * Captures the mouse pressed event so the index in the gird where
         * the mouse was pressed is initialised.
         * @param e Mouse Pressed Event
         */

        public void mousePressed(MouseEvent e) {
            resize = null;
            Point p = e.getPoint();
            TableColumnModel columnModel = tableHeader_.getColumnModel();
            pressedIndex_ = columnModel.getColumnIndexAtX(p.x);
            stopCellEditing();
        }

        /**  
         * mouseExited
         * @param e mouse exited event
         */

        public void mouseExited(MouseEvent e) {
            stopCellEditing();
        }

        /**
         * mouseClicked
         * @param e mouse clicked event
         */

        public void mouseClicked(MouseEvent e) {
        }

        /**
         * mouseEntered
         * @param e mouse entered event
         */

        public void mouseEntered(MouseEvent e) {
        }

        /**
         * mouseDragged
         * @param e Mouse Dragged Event
         */

        public void mouseDragged(MouseEvent e) {
            if (resize == null) {
                resize = tableHeader_.getResizingColumn();
            }
        }

        /**
         * mouseMoved
         * @param e mouse moved event
         */

        public void mouseMoved(MouseEvent e) {
            TemplateModel model = (TemplateModel) getModel();
            Point p = e.getPoint();
            TableColumnModel columnModel = tableHeader_.getColumnModel();
            int columnIndex = columnModel.getColumnIndexAtX(p.x);
            TableColumn column = null;
            if (columnIndex >= 0 && columnIndex < columnModel.getColumnCount()) {
                column = columnModel.getColumn(columnIndex);
            }
            TableCellRenderer oldHeaderRenderer = null;
            if (column != null) {
                oldHeaderRenderer =  column.getHeaderRenderer();
            }
            if (oldHeaderRenderer == null) {
                // Java1.4 port: getHeaderRenderer() may return null, in which
                // case we must get the renderer from 
                // JTableHeader.getDefaultRenderer()
                oldHeaderRenderer = getTableHeader().getDefaultRenderer();
            }
            while (!model.isLabel(columnIndex) && columnIndex > 0) {
                columnIndex--;
            }

            if (columnIndex >= 0) {
                column = columnModel.getColumn(columnIndex);
                TableCellRenderer headerRenderer = column.getHeaderRenderer();
                if (headerRenderer == null) {
                    // Java1.4 port: getHeaderRenderer() may return null,
                    // in which case we must get the renderer from 
                    // JTableHeader.getDefaultRenderer()
                    headerRenderer = getTableHeader().getDefaultRenderer();
                }

                String columnName = (String) column.getHeaderValue();
                String tooltp = null;
                Instrument inst = model.getInstrument();
                String desc = inst.getTemplateDesc(columnName);
                if (desc == null || desc.length() <= 0)
                    tooltp = "No Template Description : " + columnName;
                else
                    tooltp = columnName + " : " + desc;
                    
                ((DefaultTableCellRenderer) oldHeaderRenderer).setToolTipText(
                    "<html>" + tooltp + "</html>");
            }
        }
    }

    private class Ancestor implements AncestorListener {

        /**  
         * Table mouse listener
         */

        TableMouseListener listener;

        /**
         * Container for the JTable
         */

        Container c;

        /**
         * If the ancestor is added
         * @param ancestor addedevent
         */

        public void ancestorAdded(AncestorEvent event) {
            c = event.getAncestor();

            for (; c != null; c = c.getParent()) {
                if (c instanceof Window) {
                    //                    if(listener != null)
                    listener = new TableMouseListener();
                    c.addMouseListener(listener);
                }
            }
        }

        /**
         * Ancestor moved
         * @param event Ancestor moved event
         */

        public void ancestorMoved(AncestorEvent event) {

        }

        /**
         * Ancestor removed
         * @param event Ancestor removed event
         */

        public void ancestorRemoved(AncestorEvent event) {

            if (c != null) {
                c.removeMouseListener(listener);
            }
            if (getModel().getColumnCount() > 0) {
                stopCellEditing();
                ((TemplateModel) getModel()).saveChanges();
            }
        }

    }

    /**
     * Table Mouse Listener to make sure data is saved
     * to the business objects    
     *
     */

    private class TableMouseListener implements MouseListener {

        private void saveChanges() {
            TableModel model = getModel();
            if (model != null) {
                if (model.getColumnCount() > 0)
                    stopCellEditing();
                if (model instanceof TemplateModel)
                     ((TemplateModel) model).saveChanges();
            }
        }

        /**
         * Mouse clicked event 
         * @param e mouse clicked event
         */

        public void mouseClicked(MouseEvent e) {
        }

        /** 
         * Mouse pressed event
         * @param e mouse pressed event
         */

        public void mousePressed(MouseEvent e) {
        }

        /**
         * Mouse released event
         * @param e mouse released event
         */
        public void mouseReleased(MouseEvent e) {
            saveChanges();
        }

        /**  
         * Mouse entered event
         * @param e mouse entered event
         */

        public void mouseEntered(MouseEvent e) {
        }

        /**
         * Mouse exited event         * @param e mouse exited event
         */

        public void mouseExited(MouseEvent e) {
            saveChanges();
        }
    }
}
