package org.eso.ohs.phase2.apps.p2pp.views;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Vector;

import org.eso.ohs.core.docview.gui.GuiAdaptor;


public class TimeIntGuiAdaptor implements GuiAdaptor, PropertyChangeListener {
	
	private TIView.TimeIntervalsGUI tig_;
	
	private boolean modified_;
	
	
	public TimeIntGuiAdaptor(TIView.TimeIntervalsGUI tig) {
		tig_ = tig;
		tig_.addPropertyChangeListener(this);
	}
	
	/**
	 * Set the background colour of the underlying GUI component
	 * 
	 * @param bg	new value of background colour
	 */
	public void setForeground(Color bg) {
		tig_.setForeground(bg);
	}
	
	public void setEnabled(boolean enabled) {
		tig_.setEnabled(enabled);
	}
	
	public boolean isEnabled() {
		return tig_.isEnabled();
	}
	
	public void setValue(String newValue) {
		Vector<long[]> intervalVec = new Vector<long[]>();
		boolean loopAgain;
		int index = 0;
		int lastIndex = 0;
		
		for ( ; ; ) {
			index = newValue.indexOf(';', lastIndex);
			
			if (index < 0) {
				break;
			}
			
			String coordPair = newValue.substring(lastIndex, index);
			int sepIndex = coordPair.indexOf(':');
			
			if (sepIndex < 0) {
				throw new IllegalArgumentException("Missing ':' separator in TimeInterval");
			}
			
			intervalVec.addElement(
					new long[] {
							new Long(coordPair.substring(0, sepIndex)).longValue(),
							new Long(coordPair.substring(sepIndex+1)).longValue()
					}
			);
			lastIndex = index+1;
		}
		
		long[][] valueArray = new long[intervalVec.size()][2];
		intervalVec.copyInto(valueArray);
		
		tig_.setValue( valueArray);
	}
	
	public String getValue() {
		long[][] values = tig_.getValue();
		StringBuffer sb = new StringBuffer();
		
		for (int j=0; j < values.length; j++) {
			sb.append(values[j][0]);
			sb.append(":");
			sb.append(values[j][1]);
			sb.append(";");
		}
		
		return sb.toString();
	}
	
	public void setModified(boolean modified) {
		modified_ = modified;
	}
	
	public boolean isModified() {
		return modified_;
	}
	
	
	public void propertyChange(PropertyChangeEvent e) {
		setModified(true);
	}   
	
}

