#***************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: htSelectFile.tcl,v 1.1 2004/09/06 14:47:13 cguirao Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# fsogni    20/01/98  Create modifing the MAY97 version
#
#
#************************************************************************
#   NAME
#
#   panSelectFile  -  Calls a fileSelectionBox to select a file
#
#   SYNOPSIS
#
#   panSelectFile <dir> <filter> ?options?
#
#   DESCRIPTION
#
#   This procedure invokes a fileSelection Box to browse the file system.
#    
#      dir      Full path name of the directory where to start.
#      filter   Defines a pattern to select a file from in the <dir> directory.
#      options  Is a list of possible options to control the behaviour 
#               of the file selction box :
#               '-cd'    Forces to take always into account the directory 
#                        specified by <dir>
#               '-quit'  Force the fileSelectionBox to quit
#  
#   The fileSelectionBox provides the following interactive commands :
#   
#      OK      Select the file and close the dialog.
#      FILTER  Update the list of files according to the current filter
#      CANCEL  Quit the dialog          
#      HELP    Not used
#
#   After the user selection the dialog is iconified.
#   
#   FILES
#
#   ENVIRONMENT
#
#   RETURN VALUES
#
#   The return value depends on which button the user clicked :
#
#     OK     The full path name of the selected file.
#     CANCEL An empty string
#
#   CAUTIONS
#
#   EXAMPLES
#
#   global env
#   
#   set myDir   $env(HOME)/CCS/panel/src
#   set filter  "*.tcl"
#   
#   if { [set filePath [panSelectFile $myDir $filter]] == "" } {
#      No files selected
#      return
#   }
#
#
#   SEE ALSO
#
#   UifFileSelectionBox
#
#   BUGS     
#
#------------------------------------------------------------------------
#

proc htSelectFile { dir filter args } {
    global par fsbReturn 
    
    set c   ".panSelFile"; # Root name for config. panel
    set fsb $c.fsb

    if {[winfo exists $c] == 0} {
	set home [exec pwd]   ;# get current Directory
	toplevel $c
	set fsb  $c.fsb
	
	UifFileSelBox  $fsb -topLevel $c -dirGeometry 30x10 \
	    -fileGeometry 25x10 -iconify 1
	pack $fsb -fill both -expand 1 
	
	wm group     $c .
	wm minsize   $c 400 395
	
	$fsb CD $dir ; $fsb filter $filter
	
    } else {
	
      if {[lsearch $args "-cd"] != -1 } { $fsb CD $dir }
      
      $fsb filter $filter
      wm deiconify $c

  }
    
    raise $c

    uplevel #0 vwait ${fsb}_fsbReturn
#    vwait $fsb_fsbReturn  ;# Wait until the user do something ...
    
    if {[lsearch $args "-quit"] != -1 } { destroy $c }
    
    set fsbReturn [uplevel #0 set ${fsb}_fsbReturn]

   if {[string length $fsbReturn] < 1} { return "" }
    return $fsbReturn
}

#
# ___oOo___
