#Make a select with all the checks.
proc Sybsql {select} {
    global sybmsg gvar sybHandle
    
    set pid [string trimleft [clock clicks] "-"]
    
    if {[catch {set sybHandle($pid) [sybconnect $gvar(dbuser) $gvar(dbpassword) $gvar(dbserver) ]} handle]} {
	Log "Cannot connect to \"$gvar(dbserver)\": $sybmsg(dberrstr)" Error
	return ""
    }
    
    if {[catch {sybsql $sybHandle($pid) $select async} handle]} {
	Log "Cannot query/update database, reason: $sybmsg(msgtext); dberrstr = $sybmsg(dberrstr); $sybmsg(oserrstr)" Error
	set resultVar  "" 
    } else {
	set resultVar [getResult $sybHandle($pid)]
    }
    
    Sybclose "sybHandle($pid)"
    return $resultVar
}

proc Sybcancel {} {
    global sybHandle gvar
    
    catch {sybcancel $sybHandle}
    catch {sybclose $sybHandle}
    set gvar(queryCancelled) 1
    set gvar(sybaseBusy) 0
}

proc Sybclose {sybVar} {
    global sybHandle
    
    if {![info exists [set sybVar]]} { return }
    
    set handle [set $sybVar]
    catch {sybclose $handle}
    unset $sybVar
}

#procedure to get results from Sybase
proc getResult {sybHandle} {
    global sybmsg
    
    set timer 0
    if {[catch {set available [sybpoll $sybHandle 1000]}]} {
	return
    }
    
    set inc 0
    while {[string length $available] == 0} {
	if {$inc == 60} {
	    return
	}
	
	if {[catch {set available [sybpoll $sybHandle 1000]}]} {
	    return
	}
	incr inc
	update
	update idletasks
    }
    
    set buffer ""
    if {[catch { set row [sybnext $sybHandle]}]} {
	return
    }
    
    if {$sybmsg(nextrow) == "NO_MORE_RESULTS"} {
	return ""
    }
    
    while {$sybmsg(nextrow) != "NO_MORE_ROWS" && $sybmsg(nextrow) != "NO_MORE_RESULTS"} {
	lappend buffer $row
	if {[catch {set row [sybnext $sybHandle]}]} {
	    Log "Error retrieving data from the database." Error
	    return
	}
	update
	update idletasks
    }
    
    return $buffer
}

#Calculate the free space on disk
proc isThereDiskSpace {dir} {
    global gvar
    
    if {$gvar(os) == "HP-UX"} {
	set fd [open "|/usr/bin/bdf $dir" ]
    } elseif {$gvar(os) == "SunOS" || $gvar(os) == "Linux"} {
	set fd [open "|/bin/df -k $dir"]
    } else {
	Log "Unix System not recognized. Cannot use df"  Error
	return 1
    }
    
    set space -1
    gets $fd line
    gets $fd line
    scan $line "%s %d %d %d %d %s %s" d cap f space free d disk
    
    if {$space == -1} {
	scan $line "%d %d %d %d %s %s" cap f space free d disk
    }
    close $fd
    
    if {$space  < 6000000} {
	Log "There is not enough disk space to keep file $gvar(file)" Error
	return 0
    }
    
    return 1
}

proc incrementCounter {file} {
    if {[catch {set fid [open $file r]} msg]} {
	set val 0
    } else {
	gets $fid val
	close $fid
    }
    if {[catch {set fid [open $file w]} msg]} {
	Log $msg Error
	return 1
    }
    set val [expr $val + 1]
    puts $fid $val
    close $fid
    return 0
}

proc getDbrcInfo {myDbAlias} {
    set home [cd; pwd]
    set dbrc "$home/.dbrc"
    if {[catch {set fid [open $dbrc]} msg]} {
	Log $msg Error
	exit 1
    }
    set found 0
    while {![eof $fid]} {
	set line [gets $fid]
	if {[lindex $line 0] == "#"} {
	    continue
	}
	set ll [split $line]
	set dbserver [lindex $ll 0]
	set dbname [lindex $ll 1]
	set dbalias [lindex $ll 4]
	if {[string equal $dbalias $myDbAlias]} {
	    set found 1
	    break
	}
    }
    close $fid
    if {$found} {
	if {[catch {set ret [exec dbrcGet $dbserver $dbname]} msg]} {
	    Log $msg Error
	    exit 1
	}
	return "$dbserver $dbname $ret"
    } else {
	Log "Could not find $myDbAlias in $dbrc" Error
	exit 1
    }
    return ""
}

proc Log {text priority} {
    global gvar
    
    if {$text == ""} { set text "-" }
    if {[catch { eval exec "dfsmsg -host $gvar(HOST) -component postArrival_$gvar(version)\
-priority $priority -msg \{$text\}"} msg]} {
	puts "Error logging message with dfsLog: $msg"
    }
}


#
# ___oOo___
