# /bin/sh 


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# COPYRIGHT (c) 2003 European Southern Observatory
# LICENSE
#
# NAME:      checkFlatCalibDB.sh
# TYPE:      Bourne Shell script
# PROJECT:   VLT Data Flow System
# AUTHOR:    KH - ESO/SDD/SED/SEG
# DESCRIPTION:
# USAGE:     checkFlatCalibDB.sh
# EXAMPLES:
#
# $Id
#
# who            when            what
#------------------------------------------------------------------------------
# K. Haggouchi    04/02/08        created
#------------------------------------------------------------------------------
 
# for every instrument under /cal/, count how FITS or TFITS entries exist:
#   - on disk and under a flat calibDB structure: under /cal/<inst>/cal/
#   - in the MSQL calibDB and under a flat calibDB structure: under ins/<inst>/cal/
#   - on disk and under a NON flat calibDB structure: under /cal/<inst>/<mode>/cal/
#   - in the MSQL calibDB and under a NON flat calibDB structure: under ins/<inst>/<mode>/cal/


for i in `(ls -A /cal/)`
do
   echo "==============================================================================================="
   I=`echo $i | awk -F"/" '{print $1}' | tr [a-z] [A-Z]`
   echo "=== $I"

   ###
   flatDiskNb=`ls -lasRt /cal/$i/cal | grep fits | wc -l | awk '{print $1}'`
#   echo "Nb of master calibration frames present on disk following a FLAT directory structure: ${flatDiskNb}"

   ###
   flatDBNb=`echo "select * from component where comp_id like '$I%'\g" | msql calibDB | grep "ins/${i}/cal/" | grep fits | wc -l`
#   echo "Nb of master calibration frames declared in the MSQL database following a FLAT directory structure: ${flatDBNb}"

   if [ "${flatDiskNb}" != "${flatDBNb}" ]
   then
      echo "*** ERROR *** Inconsistency between the nb of master calibration frames present on disk (${flatDiskNb}) and declared in the MSQl database (${flatDBNb}), with a FLAT calibDB !"
   else
      if [ "${flatDiskNb}" = 0 -a "${flatDBNb}" = 0 ]
      then
	 echo "--- Warning: No master calibration frames present on disk (${flatDiskNb}), and no one declared in the MSQl database (${flatDBNb}), with a FLAT calibDB"
      else
         echo "Good ! same nb of master calibration frames present on disk (${flatDiskNb}) and declared in the MSQl database (${flatDBNb}), with a FLAT calibDB."
      fi
   fi

   ###
   nonFlatDiskNb=`find /cal/$i/ -name "*.*fits" | grep -v "/cal/$i/cal" | wc -l | awk '{print $1}'`
#   echo "Nb of master calibration frames present on disk NOT following a FLAT directory structure: ${nonFlatDiskNb}"

   if [ "${nonFlatDiskNb}" != 0 ]
   then
      echo "*** ERROR *** ${nonFlatDiskNb} master calibration frames present on disk following a NON flat directory structure (must not be !)"

   fi

   ###
   nonFlatDBNb=`echo "select * from component where comp_id like '$I%'\g" | msql calibDB | grep -v "ins/${i}/cal/" | grep fits | wc -l | awk '{print $1}'`
#   echo "Nb of master calibration frames declared in the MSQL database NOT following a FLAT directory structure: ${nonFlatDBNb}"

   if [ "${nonFlatDBNb}" != 0 ]
   then
      echo "*** ERROR *** ${nonFlatDBNb} master calibration frames declared in the MSQL calibDB following a NON flat directory structure (must not be !)"
   fi

   if [ "${nonFlatDiskNb}" = 0 -a "${nonFlatDBNb}" = 0 ]
   then
         echo "Good ! no master calibration frames present on disk (${nonFlatDiskNb}) and no one declared in the MSQl database (${nonFlatDBNb}), with a NON FLAT calibDB."
   fi
done

