# /bin/sh 
#
# $Id
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# COPYRIGHT (c) 2003 European Southern Observatory
# LICENSE
#
# NAME:      testFitsTranslate.sh
# TYPE:      Bourne Shell script
# PROJECT:   VLT Data Flow System
# AUTHOR:    CG - ESO/DMD/SEG
# DESCRIPTION:
# USAGE:     dfscheck.sh
# EXAMPLES:
#
# who            when            what
#------------------------------------------------------------------------------
# C. Guirao      12/10/06        created
#------------------------------------------------------------------------------
 

#
# Checking HTPP- Catalog Server installation
# 
check_http_installation ()
{
  check_group_exist websys
  check_group_exist webproj
  check_user_exist web
  check_user_exist http
  check_file_contains_regexpr /etc/group "^webproj::30000:web"
  check_file_contains_regexpr /etc/group "^websys::30010:http,web"
  check_file_contains_regexpr /etc/passwd "^web.*/home/web"
  check_file_contains_regexpr /etc/passwd "^http.*/home/http"
  if check_directory_exist  /var/logs/weblogs ; then
     check_owner_group /var/logs/weblogs web websys
     check_protections /var/logs/weblogs "drwxrwsr-x"
  fi
  if check_directory_exist  /catsrv ; then
     check_owner_group /catsrv web webproj
     check_protections /catsrv "drwxr-xr-x"
     check_is_a_link /catsrv/catalogs
     check_is_a_link /catsrv/dss
  fi
  if check_directory_exist /home/web/admin ; then
    check_soft_link /home/web/admin/logs /var/logs/weblogs 
    HOST_FULL=`cat /etc/hosts | grep $HOSTNAME | awk '{print $3}'`
    check_directory_exist /home/web/admin/logs/${HOST_FULL}.8080
    check_directory_exist /home/web/admin/logs/${HOST_FULL}.8123
    check_owner_group /home/web/admin/logs/${HOST_FULL}.8080 web websys
    check_owner_group /home/web/admin/logs/${HOST_FULL}.8123 web websys
    check_protections /home/web/admin/logs/${HOST_FULL}.8080 "drwxrwsr-x"
    check_protections /home/web/admin/logs/${HOST_FULL}.8123 "drwxrwsr-x"
    check_soft_link /home/web/admin/logs/http.catserver.main ${HOST_FULL}.8080
    check_soft_link /home/web/admin/logs/http.catserver.dev ${HOST_FULL}.8123
  fi
  /usr/sbin/swlist -l product 2>/dev/null | grep netpbm >/dev/null 
  if [ $? = 0 ]; then
    echo "Product netpbm is installed (OK)"
    check_directory_exist /opt/netpbm 
    check_soft_link /usr/local/bin/X11 /opt/netpbm/bin/X11
  else
    echo "Product netpbm is not installed (ERROR)"
  fi
  if check_file_exist /sbin/init.d/http.catserver.main  ; then
    check_owner_group /sbin/init.d/http.catserver.main bin bin
    check_protections /sbin/init.d/http.catserver.main -rwxr-xr-x
    check_soft_link /sbin/rc2.d/S921http.catserver.main /sbin/init.d/http.catserver.main
  fi
  check_process_run "httpd"
}

#
# Checking DFS installation
# 
check_dfs_installation()
{
  if [ `whoami` = "root" ]; then
    DFS_RELEASE=dfs
  fi

  if [ -z "$DFS_RELEASE" ]; then
    echo "Environment DFS_RELEASE is not defined (ERROR)"
    exit 1
  fi
  echo "Environment DFS_RELEASE is defined (OK)"

  if check_is_a_link ~flowmgr/dfs ; then
    DFS_LINK=`ls -l ~flowmgr/dfs | awk -F\> '{print $2}'`
    DFS_LINK=`basename $DFS_LINK`
  fi

  if [ "$DFS_RELEASE" = "dfs" ]; then
    echo "Environment DFS_RELEASE is defined as $DFS_RELEASE. (OK)"
  elif [ "$DFS_RELEASE" = "$DFS_LINK" ]; then 
    echo "Environment DFS_RELEASE=$DFS_RELEASE is identical to dfs link (OK)"
  else
    echo "Environment DFS_RELEASE=$DFS_RELEASE is neither dfs nor its link=$DFS_LINK (ERROR)"
  fi

  if [ "$DFS_RELEASE" = "dfs" ]; then
    if [ -n "$DFS_LINK" ]; then
      DFS_RELEASE=$DFS_LINK 
    else
      echo "Environment DFS_RELEASE=dfs but dfs is not a link (ERROR)"
    fi
  fi

  if ! check_directory_exist ~flowmgr/$DFS_RELEASE ; then
    exit 1
  fi

  #
  # Check if the $DFS_RELEASE belongs to the main MAYOR_DFS_RELEASE 
  # 
  MAYOR_DFS_RELEASE=`echo $DFS_RELEASE | sed 's/^\(dfs-._.\).*$/\1/'`
  if  [ "$MAYOR_DFS_RELEASE" != "$EXPECTED_DFS_RELEASE" ]; then
    echo "Environment DFS_RELEASE=$DFS_RELEASE does not belong to expected $EXPECTED_DFS_RELEASE (beta or patch) (ERROR)"
  else
    echo "Environment DFS_RELEASE=$DFS_RELEASE belongs to expected $EXPECTED_DFS_RELEASE (beta or patch) (OK)"
  fi
}

#
# Checking VLT/PECS Installations on role machines
#
check_vlt_installation()
{
  if ! check_directory_exist /vlt ; then exit 1;  fi
  if ! check_directory_exist /vlt/System ; then exit 1; fi
  #
  # VLTSW_RELEASE is either defined by PECS or it can be obtained from
  # the /vlt/System/vltsw_bootstrapping_release 
  #
#  if [ `whoami` = "root" ]; then
    if ! find_vlt_boot_release ; then exit 1; fi
#  fi

  if [ -z "$VLTSW_RELEASE" ]; then
    echo "Environment VLTSW_RELEASE not defined (ERROR)"
    exit 1
  fi

  if [ "$VLTSW_RELEASE" != "$EXPECTED_VLTSW_RELEASE" ]; then
    echo "Environment VLTSW_RELEASE not defined as $EXPECTED_VLTSW_RELEASE (ERROR)"
    exit 1
  else
    echo "Environment VLTSW_RELEASE is defined as $EXPECTED_VLTSW_RELEASE (OK)"
  fi

  check_directory_exist /vlt/$VLTSW_RELEASE
  check_directory_exist /vlt/$VLTSW_RELEASE/System
  check_directory_exist /vlt/$VLTSW_RELEASE/gnu
  check_directory_exist /vlt/$VLTSW_RELEASE/jdk1.6.0_06
  check_directory_exist /vlt/$VLTSW_RELEASE/tcltk 
  check_directory_exist /vlt/$VLTSW_RELEASE/NOCCS
#  check_soft_link /vlt/current $VLTSW_RELEASE
  if check_directory_exist /vlt/$VLTSW_RELEASE/CCSLite ; then
    if check_file_exist /vlt/$VLTSW_RELEASE/CCSLite/bin/msgServer1 ; then
      check_protections /vlt/$VLTSW_RELEASE/CCSLite/bin/msgServer1 "-rwsr-xr-x"
      check_owner_group /vlt/$VLTSW_RELEASE/CCSLite/bin/msgServer1 root vlt
    fi
    if check_file_exist /vlt/$VLTSW_RELEASE/CCSLite/bin/ccsScheduler1 ; then
      check_protections /vlt/$VLTSW_RELEASE/CCSLite/bin/ccsScheduler1 "-rwsr-xr-x"
      check_owner_group /vlt/$VLTSW_RELEASE/CCSLite/bin/ccsScheduler1 root vlt
    fi
  fi
  #
  # Check machine is in UTC time
  #
  check_string_contains_regexpr "`date`" "UTC"
}

check_ccslite_environment()
{
  # 
  # RTAPENV is either defined in dataflowUser.sh or it can be infered
  # from hostname
  #
  if [ -z "$RTAPENV" ]; then
    if ! find_ccslite_environment ; then exit 1; fi
  fi
  check_file_contains_regexpr /etc/services "^$RTAPENV[ 	]*..../tcp"
  if check_directory_exist /vltdata ; then
    if check_directory_exist /vltdata/msql ; then
      check_protections /vltdata/msql drwxrwxrwx
      check_owner_group /vltdata/msql vltmgr vlt
    fi
    if check_directory_exist /vltdata/tmp ; then
      check_protections /vltdata/tmp drwxrwxrwx
      check_owner_group /vltdata/tmp vltmgr vlt
    fi
    if check_directory_exist /vltdata/ENVIRONMENTS ; then
      check_protections /vltdata/ENVIRONMENTS drwxrwxrwx
      check_owner_group /vltdata/ENVIRONMENTS vltmgr vlt
      if check_directory_exist /vltdata/ENVIRONMENTS/$RTAPENV ; then
        check_protections /vltdata/ENVIRONMENTS/$RTAPENV drwxrwxrwx
        check_owner_group /vltdata/ENVIRONMENTS/$RTAPENV vlt vlt
        if check_file_exist /vltdata/ENVIRONMENTS/$RTAPENV/CcsEnvTable ; then
          check_file_contains_regexpr /vltdata/ENVIRONMENTS/$RTAPENV/CcsEnvTable "0  0  N  N  N  Y  N    1  128   schedule"
          check_file_contains_regexpr /vltdata/ENVIRONMENTS/$RTAPENV/CcsEnvTable "0  0  N  N  N  Y  N    1  128   p2pp"
          check_file_contains_regexpr /vltdata/ENVIRONMENTS/$RTAPENV/CcsEnvTable "0  0  N  N  N  Y  N    1  128   ot"
          check_file_contains_regexpr /vltdata/ENVIRONMENTS/$RTAPENV/CcsEnvTable "0  0  N  N  N  Y  N    1  128   masktracker"
        fi
      fi
    fi
    if check_directory_exist /vltdata/config ; then
      if check_file_exist /vltdata/config/CcsEnvList ; then
        check_file_contains_regexpr /vltdata/config/CcsEnvList "^$RTAPENV[	 ]*.*$HOSTNAME"
      fi
    fi
  fi
  check_file_contains_regexpr /etc/syslog.conf "^local1.warning	*/vltdata/tmp/logFile"
  check_file_contains_regexpr /etc/syslog.conf "^local2.warning	*/vltdata/tmp/logAuto"
  check_process_owned_by "ccsScheduler -e $RTAPENV" vlt
  check_process_owned_by "ccsSHManager" vlt
  check_process_owned_by "qsemu" vlt
  if check_directory_exist /vltdata/tmp ; then
        check_protections /vltdata/tmp/logFile -rw-rw-rw-
        check_protections /vltdata/tmp/logAuto -rw-rw-rw-
        check_owner_group /vltdata/tmp/logFile vlt vlt
        check_owner_group /vltdata/tmp/logAuto vlt vlt
  fi
}

check_pecs_installation()
{
check_directory_exist /etc/pecs
check_directory_exist /etc/pecs/releases/000/etc/locality
#check_file_exist /etc/pecs/releases/000/etc/locality/apps-$HOSTNAME.env
check_file_exist /etc/pecs/releases/000/etc/locality/pecsvltdfs-apps-all.env
#check_file_does_not_exist /etc/pecs/releases/000/etc/locality/pecsvltdfs-apps-$HOSTNAME.env
}

value_of_ohs_file()
{
  if egrep  "$2" "$1" >/dev/null ; then
    value=`egrep  "$2" "$1" | awk -F\" '{print $2}'`
    return 0
  else  
    echo "File $1 does not contains $2 (ERROR)"
    return 1
  fi
}


check_common_ohs_account() 
{
  if check_file_exist $1; then
    if value_of_ohs_file $1 "^IMPEX.FOLDER" ; then 
      check_directory_exist "$value"
    fi
    if value_of_ohs_file $1 "^CACHE.FOLDER" ; then 
      check_directory_exist "$value"
    fi
    return 0
  else
    return 1
  fi
}

check_msql_installation ()
{
  check_process_owned_by msql2d quality
  check_soft_link /MSQL2 ~quality/MSQL2 
  check_directory_exist ~quality/MSQL2
  if check_directory_exist /data/calibDB ; then 
    check_owner_group  /data/calibDB quality vlt
    check_protections /data/calibDB drwxr-x---
    check_soft_link /cal $DFS_DATA_ROOT/calibDB/certif/ins 
    check_directory_exist $DFS_DATA_ROOT/calibDB/certif/ins
    check_soft_link /tcal $DFS_DATA_ROOT/calibDB/check/ins 
    check_directory_exist $DFS_DATA_ROOT/calibDB/check/ins
  fi
}

check_midas_installation ()
{
  check_soft_link /midas ~flowmgr/midas 
  check_directory_exist  ~flowmgr/midas
  check_file_contains_regexpr /etc/services "^midxcon[ 	]*4101/tcp$"
  check_file_contains_regexpr /etc/services "^midserv[ 	]*4102/tcp$"
}

check_midas_environment ()
{
  if [ -z "$MIDASHOME" ]; then
    myecho "Environment MIDASHOME is not defined (ERROR)"
    return 1
  fi
  if [ "$MIDASHOME" = "/midas" ]; then
    myecho "Environment MIDASHOME is defined as /midas (OK)"
  else
    myecho "Environment MIDASHOME is not defined as /midas (ERROR)"
  fi
  if check_directory_exist $MIDASHOME ; then
    if [ -z "$MIDVERS" ]; then
      myecho "Environment MIDVERS is not defined (ERROR)"
      return 1
    else
      myecho "Environment MIDVERS is defined (OK)"
    fi
    check_directory_exist $MIDASHOME/$MIDVERS
    check_directory_exist $MIDASHOME/$MIDVERS/pipeline
  fi
  check_owner_group $MIDASHOME/$MIDVERS flowmgr vlt
}
   
check_pipe_exist()
{
  necho "File $1 "
  if [ -p $1 ]; then
    myecho "exists and is a pipe (OK)"
    return 0
  else
    myecho "does not exist or is not a pipe. (ERROR)"
    return 1
  fi
}

check_dfslog_server()
{
  check_file_exist $BOOT_DIR/$DFSLOGD
  if check_pipe_exist /tmp/dfslogPipe ; then 
     check_protections /tmp/dfslogPipe "prw-r--r--"
     check_owner_group /tmp/dfslogPipe root root
  fi

  check_file_contains_regexpr /etc/syslog.conf "^local3.debug	*/tmp/dfslogPipe"
  check_file_contains_regexpr /etc/syslog.conf "^local3.debug	*/data/msg/dfslog.log"
  check_file_exist /data/msg/dfslog.log
  check_protections /data/msg/dfslog.log -rw-r--r--
  check_process_run "/home/flowmgr/dfs/bin/dfslogd"
} 

check_dfslog_client()
{
  check_file_does_not_exist $BOOT_DIR/$DFSLOGD 
  check_file_contains_regexpr /etc/syslog.conf "^local3.debug	*@.*dhs.*$"
  check_file_contains_regexpr /etc/syslog.conf "^local3.debug	*/data/msg/dfslog.log"
  check_file_exist /data/msg/dfslog.log
  check_protections /data/msg/dfslog.log -rw-r--r--
} 

check_OhsWS_installation()
{
  check_vlt_installation
#  check_pecs_installation
#  if check_file_exist $BOOT_DIR/ccsLite ; then
#    check_ccslite_environment
#  fi
  if check_file_exist $BOOT_DIR/rtap ; then
    check_ccslite_environment
  fi
  check_group_exist vlt
  check_group_exist users
  check_user_exist vltmgr
  check_user_exist flowmgr
  check_user_exist pecsmgr
  check_user_exist visitor
  check_user_exist service
  check_user_exist instmgr
  check_user_exist vlt
  check_user_exist rtap
  check_dfs_installation
  check_protections ~flowmgr drwxr-xr-x
 
  #
  # Check external software
  #
  check_directory_exist $SYBASE_DIR

  #
  # Do not check for $ (end of line) as site.cf could be MS-DOS file 
  # with CTRL-M.
  if check_file_exist ~flowmgr/dfs/OHS/p2pp/config/site.cf ; then
    check_file_contains_regexpr ~flowmgr/dfs/OHS/p2pp/config/site.cf \
      "^CCS.CONNECTION.ENABLED.*\"true\""
    check_file_contains_regexpr ~flowmgr/dfs/OHS/p2pp/config/site.cf \
      "^INSTRUMENTS.IPVERSIONING.SINGLEIPMODE.*\"y\""
    check_file_contains_regexpr ~flowmgr/dfs/OHS/p2pp/config/site.cf \
      "^INSTRUMENTS.FOLDER.*\"/home/instmgr/instruments\""
  fi

  #
  # Check for scripts p2pp, ot, masktracker and fcviewer to be Java 1.5
  #
  if check_file_exist ~flowmgr/dfs/OHS/p2pp/bin/p2pp ; then
    check_file_contains_regexpr ~flowmgr/dfs/OHS/p2pp/bin/p2pp \
      "\$JRE -version 2>&1 | grep 1.5"
    check_file_contains_regexpr ~flowmgr/dfs/OHS/ot/bin/ot \
      "\$JRE -version 2>&1 | grep 1.5"
    check_file_contains_regexpr ~flowmgr/dfs/OHS/masktracker/bin/masktracker \
      "\$JRE -version 2>&1 | grep 1.5"
    check_file_contains_regexpr ~flowmgr/dfs/OHS/ot/bin/fcviewer \
      "\$JRE -version 2>&1 | grep 1.5"
  fi
}

check_OlasWS_installation()
{
  check_vlt_installation
#  check_pecs_installation
  check_group_exist vlt
  check_user_exist vltmgr
  check_user_exist flowmgr
  check_user_exist pecsmgr
  check_user_exist archeso
  check_protections ~flowmgr drwxr-xr-x
  check_dfs_installation
  if check_file_exist $BOOT_DIR/$DFSLOGD ; then
    check_dfslog_server
  fi
  check_file_exist $BOOT_DIR/olasDfs
  check_process_owned_by dhs archeso
  check_process_owned_by frameIngest archeso
  #
  # Check external software
  #
  check_directory_exist $SYBASE_DIR
}

check_media_structure()
{
  if check_directory_exist $3 ; then
     check_protections $3 drwxr-xr-x
     check_owner_group $3 $1 $2
     for dir  in D user stage romanic 
     do 
       if check_directory_exist $3/$dir ; then 
          check_protections $3/$dir drwxr-xr-x
          check_owner_group $3/$dir $1 $2
       fi
     done

     if [ "$HOSTNAME" != "wvgarc" ]  ; then
       dir_list="4.USR 7.USRDAT 8.USRDVD \
                 volume0 volume1 volume2 volume3 volume4 volume5"
     else
       dir_list="14.USR 17.USRDAT 18.USRDVD \
                 vlti-volume0 vlti-volume1 vlti-volume2 \
                 vlti-volume3 vlti-volume4 vlti-volume5"
     fi
     for dir in $dir_list
     do 
       if check_directory_exist $3/user/$dir ; then 
          check_protections $3/user/$dir drwxr-xr-x
          check_owner_group $3/user/$dir $1 $2
       fi
     done

     if [ "$HOSTNAME" != "wvgarc" ]  ; then
       dir_list="volume0 volume1 volume2 volume3 volume4 volume5"
     else
       dir_list="vlti-volume0 vlti-volume1 vlti-volume2 \
                 vlti-volume3 vlti-volume4 vlti-volume5"
     fi
     for dir in $dir_list
     do 
       if check_directory_exist $3/D/$dir ; then 
          check_protections $3/D/$dir drwxr-xr-x
          check_owner_group $3/D/$dir $1 $2
       fi
     done

     if [ "$HOSTNAME" != "wvgarc" ]  ; then
       dir_list="volume0 volume1 volume2 volume3 volume4 volume5 \
		 3.LOG 4.USR 5.PAF 6.OTH 7.USRDAT 8.USRDVD 9.FITS"
     else
       dir_list="vlti-volume0 vlti-volume1 vlti-volume2 \
                 vlti-volume3 vlti-volume4 vlti-volume5 \
		 13.LOG 14.USR 15.PAF 16.OTH 17.USRDAT 18.USRDVD 19.FITS"
     fi
     for dir  in $dir_list
     do 
       if check_directory_exist $3/stage/$dir ; then 
          check_is_not_a_link $3/stage/$dir
          check_protections $3/stage/$dir drwxr-xr-x
          check_owner_group $3/stage/$dir $1 $2
       fi
     done
     return 0
  else
     return 1
  fi
}

check_data_structure_astro()
{
  if check_directory_exist $3 ; then
     check_protections $3 drwxr-x---
     check_owner_group $3 $1 $2
     if check_directory_exist $3/msg ; then 
        check_protections $3/msg drwxrwx--T
        check_owner_group $3/msg $1 $2
     fi
     if check_directory_exist $3/raw ; then 
        check_protections $3/raw drwxr-x---
        check_owner_group $3/raw $1 $2
     fi
     if check_directory_exist $3/reduced ; then 
        check_protections $3/reduced drwxr-x---
        check_owner_group $3/reduced $1 $2
     fi
     if check_directory_exist $3/bad ; then 
        check_protections $3/bad drwxr-x---
        check_owner_group $3/bad $1 $2
     fi
     if check_directory_exist $3/backlog ; then 
        check_protections $3/backlog drwxr-x---
        check_owner_group $3/backlog $1 $2
     fi
     return 0
  else
     return 1
  fi
}

check_data_structure()
{
  if check_directory_exist $3 ; then
     check_protections $3 drwxr-x---
     check_owner_group $3 $1 $2
     if check_directory_exist $3/msg ; then 
        check_protections $3/msg drwxr-x--T
        check_owner_group $3/msg $1 $2
     fi
     if check_directory_exist $3/raw ; then 
        check_protections $3/raw drwxr-x---
        check_owner_group $3/raw $1 $2
     fi
     if check_directory_exist $3/reduced ; then 
        check_protections $3/reduced drwxr-x---
        check_owner_group $3/reduced $1 $2
     fi
     if check_directory_exist $3/bad ; then 
        check_protections $3/bad drwxr-x---
        check_owner_group $3/bad $1 $2
     fi
     if check_directory_exist $3/backlog ; then 
        check_protections $3/backlog drwxr-x---
        check_owner_group $3/backlog $1 $2
     fi
     return 0
  else
     return 1
  fi
}

check_PipelineWS_installation()
{
  check_vlt_installation
#  check_pecs_installation
  check_group_exist vlt
  check_user_exist vltmgr
  check_user_exist flowmgr
  check_user_exist pecsmgr
  check_user_exist pipeline
  check_user_exist quality
  check_protections ~flowmgr drwxr-xr-x
  check_protections ~quality drwxr-xr-x
  check_dfs_installation
  #
  # Check external software
  #
  check_dfslog_client
  check_directory_exist $SYBASE_DIR
#  check_directory_exist $MSQL_DIR
#  check_directory_exist $PYTHON_DIR
#  check_file_exist $BOOT_DIR/msql2d
  check_file_exist $BOOT_DIR/pipelineDfs
  check_process_owned_by dhs pipeline
  check_process_owned_by vcsolac pipeline
  check_process_owned_by dhsSubscribe pipeline
}

check_UserWS_installation() 
{
  check_vlt_installation
#  check_pecs_installation
  check_group_exist vlt
  check_group_exist ut1
  check_group_exist ut2
  check_group_exist ut3
  check_group_exist ut4
  check_group_exist vlti
  check_user_exist vltmgr
  check_user_exist flowmgr
  check_user_exist pecsmgr
  check_user_exist astro
  check_user_exist astro2
  check_user_exist astro3
  check_user_exist astro4
  check_user_exist astrov
  check_user_belongs_to_group archeso ut1
  check_user_belongs_to_group archeso ut2
  check_user_belongs_to_group archeso ut3
  check_user_belongs_to_group archeso ut4
  check_user_belongs_to_group archeso vlti
  check_user_belongs_to_group astro1 vlt
  check_user_belongs_to_group astro2 vlt
  check_user_belongs_to_group astro3 vlt
  check_user_belongs_to_group astro4 vlt
  check_user_belongs_to_group astrov vlt
  #
  check_protections ~flowmgr drwxr-xr-x
  check_dfs_installation
  #
  # Check external software
  #
  check_dfslog_client
  check_directory_exist $SYBASE_DIR
  check_directory_exist $PYTHON_DIR
  check_file_exist /usr/local/bin/remakeUsers
  check_file_exist /usr/local/bin/user_template.tar
}

check_AstoWS_installation()
{
  check_vlt_installation
#  check_pecs_installation
  check_group_exist vlt
  check_user_exist vltmgr
  check_user_exist flowmgr
  check_user_exist pecsmgr
  check_user_exist archeso
  check_protections ~flowmgr drwxr-xr-x
  check_dfs_installation
  check_directory_exist $SYBASE_DIR
  check_dfslog_client
  if [ -z "$DFS_DATA_ROOT" ]; then
    echo "Environment DFS_DATA_ROOT is not defined (ERROR)"
  else
    check_data_structure archeso vlt $DFS_DATA_ROOT
  fi

  if [ -z "$ASTO_STAGE_ROOT" ]; then
    echo "Environment ASTO_STAGE_ROOT is not defined (ERROR)"
  else
    check_media_structure archeso vlt /data/media
  fi

}

#
# MAIN
# Main. Starting here
# 

#
# Definitions for this release
#
EXPECTED_DFS_RELEASE=dfs-6_0	# Only the main  release without patch number
EXPECTED_VLTSW_RELEASE=VLT2010


#
# Some global definitions:
#
SYBASE_DIR=/opt/sybase
PYTHON_DIR=/opsw/util/python
#NGAMS_DIR=/opsw/NGAS
#MSQL_DIR=/opt/msql2
VERBOSE=0
BOOT_DIR=/etc/rc.d/init.d
DFSLOGD=dfslogd
RSH=rsh

#
# Load dfschecklib.sh function library
#
dirname=`dirname $0`
library=$dirname/dfschecklib.sh
. $library
set_echoes

#
# ECHO=1 : functions necho() and myecho() enable (to print to terminal)
# ECHO=0 : functions necho() and myecho() disable (do not print anything)
#
ECHO=1
HOSTNAME=`uname -n`
cmd=`basename $0`
roles="[UserWS OlasWS PipelineWS AstoWS OhsWS]"
usage1="Usage (as any user): $cmd"
usage2="Usage (as root): $cmd $roles"
#release="release: $Id"
release="release: 0.1"
current_dfs=`dirname $0`
current_dfs=`dirname $current_dfs`
current_dfs=`(cd $current_dfs ; /bin/pwd)`
current_dfs=`basename $current_dfs`

# 
# Check for arguments -r (release), -d (debug) or -h (help)
#
while [ -n "$1" ]
do
  case $1 in
  -r) 
     echo "$cmd $release"
     echo "Executable $cmd from DFS release $current_dfs"
     exit 0
     ;;
  -h)
     echo $usage1; echo $usage2
     exit 0
     ;;
  -d)
     shift
     VERBOSE=1
     ;;
   *)
     break
     ;;
   esac
done

if [ "$VERBOSE" = 0 ]; then
 exec $0 -d $* | egrep 'ERROR|WARNING'
 exit 0
fi
#
# Print date, release and DFS release 
#
echo "$cmd $release"
echo "Executable $cmd from DFS release $current_dfs"
date

#
# Check if this is 'root', if so it requires an argument 
#
if [ `whoami` = "root" ]; then
  if [ -z "$1" ]; then
      echo "ERROR: As root, this script requires an argument"
      echo $usage1; echo $usage2
      exit 1
  fi
  case $1 in
    UserWS|OlasWS|PipelineWS|AstoWS|OhsWS)
      role=$1
      ;;
    *)
      echo "ERROR: Wrong argument"
      echo $usage1; echo $usage2
      exit 1
      ;;
  esac;
#
# If I'm not root then I need to be in a $DFS_HOST and as a $DFS_USER
#
else
  if [ -z "$DFS_USER" ]; then echo "Environment DFS_USER not defined";exit 1;fi
fi

if [ ${HOSTNAME} = "$OhsWS" -o "$role" = "OhsWS" ]; then
  check_OhsWS_installation
  case ${DFS_USER} in
  visitor)
    ECHO=0
    if check_directory_exist /data; then
      ECHO=1
      check_protections /data/ drwxr-x---
      check_owner_group /data/ archeso vlt
    fi
    ECHO=1
#    check_vlt_ccstype ccslite
    check_env_contains_regexpr $PATH 'CCSLite' PATH
    if check_file_exist ~/.p2pp.cf; then
      check_file_contains_regexpr ~/.p2pp.cf "^CACHE.FOLDER.*cache\"$"
      if value_of_ohs_file ~/.p2pp.cf "^CACHE.FOLDER" ; then 
        check_directory_exist "$value"
      fi
      check_file_contains_regexpr ~/.p2pp.cf "^CCS.OHS.SERVER.NAME.*\"p2pp\"$"
      ECHO=0
      if check_file_contains_regexpr ~/.p2pp.cf "^INSTRUMENTS.FOLDER" ; then
        echo "File ~/.p2pp.cf contains an INSTRUMENTS.FOLDER entry (ERROR)"
      else
        echo "File ~/.p2pp.cf does not contains an INSTRUMENTS.FOLDER entry (OK)"
      fi
      ECHO=1
    fi
    check_file_exist ~/.bash_profile
#    check_directory_exist ~/.pecs
    ;;
  service)
    ECHO=0
    if check_directory_exist /data; then
      ECHO=1
      check_protections /data/ drwxr-x---
      check_owner_group /data/ archeso vlt
    fi
    ECHO=1
#    check_vlt_ccstype ccslite
    check_env_contains_regexpr $PATH 'CCSLite' PATH
    if check_file_exist ~/.ot.cf; then
      check_file_contains_regexpr ~/.ot.cf "^CCS.OHS.SERVER.NAME.*\"ot\"$"
      ECHO=0
      if check_file_contains_regexpr ~/.ot.cf "^INSTRUMENTS.FOLDER" ; then
        echo "File ~/.ot.cf contains an INSTRUMENTS.FOLDER entry (ERROR)"
      else
        echo "File ~/.ot.cf does not contains an INSTRUMENTS.FOLDER entry (OK)"
      fi
      ECHO=1
    fi
    check_file_exist ~/.bash_profile
#    check_directory_exist ~/.pecs
    if [ ${HOSTNAME} = "wu3dhs" ]; then 
      if check_file_exist ~/.mt.cf ; then
        check_file_contains_regexpr ~/.mt.cf "^CCS.OHS.SERVER.NAME.*\"masktracker\"$"
        check_file_contains_regexpr ~/.mt.cf "^CCS.MASKTRACKER.ENABLED.*\"true\"$"
      fi
    fi
    ;;
  instmgr)
    ECHO=0
    if check_directory_exist /data; then
      ECHO=1
      check_protections /data/ drwxr-x---
      check_owner_group /data/ archeso vlt
    fi
    ECHO=1
#    check_vlt_ccstype ccslite
    check_env_contains_regexpr $PATH 'CCSLite' PATH
    if check_file_exist ~/.ot.cf;  then
      ECHO=0
      if check_file_contains_regexpr ~/.ot.cf "^INSTRUMENTS.FOLDER" ; then
        echo "File ~/.ot.cf contains an INSTRUMENTS.FOLDER entry (ERROR)"
      else
        echo "File ~/.ot.cf does not contains an INSTRUMENTS.FOLDER entry (OK)"
      fi
      ECHO=1
    fi
    if check_file_exist ~/.p2pp.cf; then
      check_file_contains_regexpr ~/.p2pp.cf "^CACHE.FOLDER.*cache\"$"
      if value_of_ohs_file ~/.p2pp.cf "^CACHE.FOLDER" ; then 
        check_directory_exist "$value"
      fi
      ECHO=0
      if check_file_contains_regexpr ~/.p2pp.cf "^INSTRUMENTS.FOLDER" ; then
        echo "File ~/.p2pp.cf contains an INSTRUMENTS.FOLDER entry (ERROR)"
      else
        echo "File ~/.p2pp.cf does not contains an INSTRUMENTS.FOLDER entry (OK)"
      ECHO=1
      fi
    fi
    check_file_exist ~/.bash_profile
#    check_directory_exist ~/.pecs
    ;;
  esac
fi

if [ ${HOSTNAME} = "$OlasWS" -o "$role" = "OlasWS" ]; then
  case ${DFS_USER} in
  archeso)
    check_OlasWS_installation 
    # CG 30.03.04. DFS_DATA_ROOT not yet implemented for archeso@OlasWS
    # Here DFS_DATA_ROOT is forced to /data
    export DFS_DATA_ROOT=/data

    if [ -z "$DFS_DATA_ROOT" ]; then
      echo "Environment DFS_DATA_ROOT is not defined (ERROR)"
    else
      check_data_structure archeso vlt $DFS_DATA_ROOT
    fi

#    check_vlt_ccstype noccs
    check_env_contains_regexpr $LD_LIBRARY_PATH 'NOCCS' LD_LIBRARY_PATH
    check_env_contains_regexpr $PATH 'NOCCS' PATH
    check_file_exist ~/.bash_profile
#    check_directory_exist ~/.pecs
    check_is_not_a_link ~/.bashrc
    check_file_contains_regexpr ~/.bashrc 'if \[ "\$TERM" = "dumb" \]'

    if [ -f "$DFS_TOO_LIST" ]; then
      if [ -z "$DFS_TOO_RAW" ]; then
        echo "Environment DFS_TOO_RAW is not defined (ERROR)"
      else
        echo "Environment DFS_TOO_RAW is defined (OK)"
        if check_directory_exist $DFS_TOO_RAW ; then
          check_protections $DFS_TOO_RAW drwxr-x---
          check_owner_group $DFS_TOO_RAW archeso vlt
        fi
      fi
      if [ -z "$DFS_TOO_MSG" ]; then
        echo "Environment DFS_TOO_MSG is not defined (ERROR)"
      else
        echo "Environment DFS_TOO_MSG is defined (OK)"
        if check_directory_exist $DFS_TOO_MSG ; then
          check_owner_group $DFS_TOO_MSG archeso vlt
        fi
      fi
      if [ -z "$DFS_TOO_BACKLOG" ]; then
        echo "Environment DFS_TOO_BACKLOG is not defined (ERROR)"
      else
        echo "Environment DFS_TOO_BACKLOG is defined (OK)"
        if check_directory_exist $DFS_TOO_BACKLOG ; then
          check_protections $DFS_TOO_BACKLOG drwxr-x---
          check_owner_group $DFS_TOO_BACKLOG archeso vlt
        fi
      fi
      if [ -z "$DFS_TOO_STAGE" ]; then
        echo "Environment DFS_TOO_STAGE is not defined (ERROR)"
      else
        echo "Environment DFS_TOO_STAGE is defined (OK)"
        if check_directory_exist $DFS_TOO_STAGE ; then
          check_protections $DFS_TOO_STAGE drwxr-x---
          check_owner_group $DFS_TOO_STAGE archeso vlt
        fi
      fi
    fi 

    if [ "$DFS_NGAMS_ARCHIVE" = "YES" ]; then
      check_process_owned_by "ngamsIngest -dhsdata /data/raw-ngams-archive"  archeso
      if [ -z "$DFS_DATA_NGAMS_ARCHIVE" ]; then
        echo "Environment DFS_DATA_NGAMS_ARCHIVE is not defined (ERROR)"
      else
        echo "Environment DFS_DATA_NGAMS_ARCHIVE is defined (OK)"
        if check_directory_exist $DFS_DATA_NGAMS_ARCHIVE ; then
          check_protections $DFS_DATA_NGAMS_ARCHIVE drwxr-x---
          check_owner_group $DFS_DATA_NGAMS_ARCHIVE archeso vlt
        fi
      fi
      if [ -z "$DFS_BACKLOG_NGAMS_ARCHIVE" ]; then
        echo "Environment DFS_BACKLOG_NGAMS_ARCHIVE is not defined (ERROR)"
      else
        echo "Environment DFS_BACKLOG_NGAMS_ARCHIVE is defined (OK)"
        if check_directory_exist $DFS_BACKLOG_NGAMS_ARCHIVE ; then
          check_protections $DFS_BACKLOG_NGAMS_ARCHIVE drwxr-x---
          check_owner_group $DFS_BACKLOG_NGAMS_ARCHIVE archeso vlt
        fi
      fi
    fi

    if [ "$DFS_NGAMS_PREIMAGE" = "YES" ]; then
      check_process_owned_by "ngamsIngest -dhsdata /data/raw-ngams-preimage" archeso
      if [ -z "$DFS_DATA_NGAMS_PREIMAGE" ]; then
        echo "Environment DFS_DATA_NGAMS_PREIMAGE is not defined (ERROR)"
      else
        echo "Environment DFS_DATA_NGAMS_PREIMAGE is defined (OK)"
        if check_directory_exist $DFS_DATA_NGAMS_PREIMAGE ; then
          check_protections $DFS_DATA_NGAMS_PREIMAGE drwxr-x---
          check_owner_group $DFS_DATA_NGAMS_PREIMAGE archeso vlt
        fi
      fi
      if [ -z "$DFS_BACKLOG_NGAMS_PREIMAGE" ]; then
        echo "Environment DFS_BACKLOG_NGAMS_PREIMAGE is not defined (ERROR)"
      else
        echo "Environment DFS_BACKLOG_NGAMS_PREIMAGE is defined (OK)"
        if check_directory_exist $DFS_BACKLOG_NGAMS_PREIMAGE ; then
          check_protections $DFS_BACKLOG_NGAMS_PREIMAGE drwxr-x---
          check_owner_group $DFS_BACKLOG_NGAMS_PREIMAGE archeso vlt
        fi
      fi
    fi

    if check_file_exist ~/.dbrc ; then
       check_protections ~/.dbrc -rw-r--r--
       check_file_contains_regexpr ~/.dbrc "^$DSQUERY observations frameingest.*DPREP$"
       check_file_contains_regexpr ~/.dbrc "^$DSQUERY ambient frameingest.*AMBIREP$"
       check_file_contains_regexpr ~/.dbrc "^$DSQUERY asto asto.*ASTOREP$"
    fi

    check_directory_exist ~archeso/INS_ROOT
    if [ -z "$INS_ROOT" ] ; then
       echo "Environment INS_ROOT is not defined (ERROR)"
    else
       echo "Environment INS_ROOT is defined (OK)"
       if [ "$INS_ROOT" = ~archeso/INS_ROOT ] ; then
         echo "Environment INS_ROOT is set to ~quality/INS_ROOT (OK)"
         check_directory_exist ~archeso/INS_ROOT/SYSTEM/Dictionary 
       else
         echo "Environment INS_ROOT is not set to ~archeso/INS_ROOT (ERROR)"
       fi
    fi
    ;;
  esac
fi

if [ ${HOSTNAME} = "$PipelineWS" -o "$role" = "PipelineWS" ]; then
  check_PipelineWS_installation
  case ${DFS_USER} in
  pipeline)
#    check_vlt_ccstype noccs
    check_env_contains_regexpr $LD_LIBRARY_PATH 'NOCCS' LD_LIBRARY_PATH
    check_env_contains_regexpr $PATH 'NOCCS' PATH
    check_protections $HOME drwxr-xr-x
    check_file_exist ~/.bash_profile
#    check_directory_exist ~/.pecs
    check_is_not_a_link ~/.bashrc
    check_file_contains_regexpr ~/.bashrc 'if \[ "\$TERM" = "dumb" \]'
#    check_midas_environment
    if [ -z "$DFS_REDBLOCK" ] ; then
      echo "Environment DFS_REDBLOCK is not defined (ERROR)"
    else
      echo "Environment DFS_REDBLOCK is defined (OK)"
      if check_directory_exist "$DFS_REDBLOCK" ; then
        check_protections $DFS_REDBLOCK drwxr-x---
      fi
    fi
    if check_file_exist ~/.rhosts ; then
      check_protections ~/.rhosts -rw-------
    fi
    if [ -z "$DHS_HOST" ] ; then
       echo "Environment DHS_HOST is not defined (ERROR)"
    else
       echo "Environment DHS_HOST is defined (OK)"
       check_file_contains_regexpr ~/.rhosts "^$DHS_HOST[ 	]*archeso$"
       check_access_host_account $DHS_HOST archeso
    fi
    if [ -z "$QcWS" ] ; then
       echo "Environment QcWS is not defined (ERROR)"
    else
       echo "Environment QcWS is defined (OK)"
       if [ $QcWS = $HOSTNAME ]; then
         echo "Environment QcWS is equal to HOSTNAME (OK)"
         check_file_contains_regexpr ~/.rhosts "^$QcWS[ 	]*pipeline$"
         check_access_host_account $QcWS pipeline
       elif [ $QcWS != BLANK ] ; then
         echo "Environment QcWS is not HOSTNAME nor BLANK (ERROR)"
       fi
    fi
    if check_data_structure $USER vlt $DFS_DATA_ROOT ; then
      if check_directory_exist /data/lists ; then 
        check_protections /data/lists drwxr-x---
        check_owner_group /data/lists pipeline vlt
        if check_directory_exist /data/lists/raw_do ; then 
          check_owner_group /data/lists pipeline vlt
        fi
        if check_directory_exist /data/lists/raw_qcm ; then 
          check_owner_group /data/lists/raw_qcm pipeline vlt
        fi
        if check_directory_exist /data/lists/reduced_olas ; then 
          check_owner_group /data/lists/reduced_olas pipeline vlt
        fi
       fi
    fi
    check_file_exist ~/pipeline.config 
    # Implement fix to DFS04700
    if check_directory_exist ~/gasgano/gas-scripts ; then 
      check_soft_link ~/gasgano/gas-scripts/DO_submit.sh ~flowmgr/dfs/gasgano/bin/DO_submit.sh
    fi

    ;;
  quality)
#    check_msql_installation
#    check_midas_installation
#    check_vlt_ccstype noccs
    check_env_contains_regexpr $LD_LIBRARY_PATH 'NOCCS' LD_LIBRARY_PATH
    check_env_contains_regexpr $PATH 'NOCCS' PATH
    check_protections $HOME drwxr-xr-x
    check_file_exist ~/.bash_profile
#    check_directory_exist ~/.pecs
#    check_midas_environment
    check_directory_exist ~quality/qc1/Dictionary
#    check_directory_exist ~quality/INS_ROOT
#    if [ -z "$INS_ROOT" ] ; then
#       echo "Environment INS_ROOT is not defined (ERROR)"
#    else
#       echo "Environment INS_ROOT is defined (OK)"
#       if [ "$INS_ROOT" = ~quality/INS_ROOT ] ; then
#         echo "Environment INS_ROOT is set to ~quality/INS_ROOT (OK)"
#         check_directory_exist ~quality/INS_ROOT/SYSTEM/Dictionary 
#       else
#         echo "Environment INS_ROOT is not set to ~quality/INS_ROOT (ERROR)"
#       fi
#    fi
#    check_directory_exist ~quality/INS_ROOT_DO
#    if [ -z "$INS_ROOT_DO" ] ; then
#       echo "Environment INS_ROOT_DO is not defined (ERROR)"
#    else
#       echo "Environment INS_ROOT_DO is defined (OK)"
#       if [ "$INS_ROOT_DO" = ~quality/INS_ROOT_DO ] ; then
#         echo "Environment INS_ROOT_DO is set to ~quality/INS_ROOT (OK)"
#         check_directory_exist ~quality/INS_ROOT_DO/SYSTEM/Dictionary 
#       else
#         echo "Environment INS_ROOT is not set to ~quality/INS_ROOT_DO (ERROR)"
#       fi
#    fi
  esac
fi

if [ ${HOSTNAME} = "$UserWS" -o "$role" = "UserWS" ]; then
  check_UserWS_installation
  case ${DFS_USER} in
  astuser?)
#    check_vlt_ccstype noccs
    check_env_contains_regexpr $LD_LIBRARY_PATH 'NOCCS' LD_LIBRARY_PATH
    check_env_contains_regexpr $PATH 'NOCCS' PATH
    check_protections ~ drwxr-x---
    check_file_exist ~/.bash_profile
    check_soft_link ~/.bashrc /etc/pecs/etc/bashrc
    check_directory_exist ~/iraf
#    if check_directory_exist ~/.pecs ; then
#      check_file_contains_regexpr ~/.pecs/misc-all.env 'scisoft/bin/Setup.bash'
#      check_file_contains_regexpr ~/.pecs/misc-all.env 'LM_LICENSE_FILE'
#      check_file_contains_regexpr ~/.pecs/misc-all.ali 'alias cl='
#    fi
    if [ -z "$LM_LICENSE_FILE" ]; then
       echo "Environment LM_LICENSE_FILE is not defined (ERROR)"
    else
       echo "Environment LM_LICENSE_FILE is defined (OK)"
    fi
    check_env_contains_regexpr $LD_LIBRARY_PATH '/opt/sybase/lib' LD_LIBRARY_PATH
    ;;

  astro)
#    check_vlt_ccstype noccs
    check_env_contains_regexpr $LD_LIBRARY_PATH 'NOCCS' LD_LIBRARY_PATH
    check_env_contains_regexpr $PATH 'NOCCS' PATH
    check_protections ~ drwxr-x---
    # 
    # Try to sudo as archeso the command "env" with a wrong password
    # If successes sudo is properly setup, if not there is something wrong
    # with the /etc/sudoers files
    #
    if echo "WrongPassword" | sudo -u archeso -S env >/dev/null 2>&1 ; then
       echo "Trying to sudo cmd \"env\" as archeso succeeded (OK)"
    else
       echo "Trying to sudo command \"env\" as archeso failed. Check /etc/sudoers (ERROR)"
    fi

    check_file_exist ~/.bash_profile
    check_soft_link ~/.bashrc /etc/pecs/etc/bashrc
    check_directory_exist ~/iraf
#    if check_directory_exist ~/.pecs ; then
#      check_file_contains_regexpr ~/.pecs/misc-all.env 'scisoft/bin/Setup.bash'
#      check_file_contains_regexpr ~/.pecs/misc-all.env 'LM_LICENSE_FILE'
#      check_file_contains_regexpr ~/.pecs/misc-all.ali 'alias cl='
#      check_file_contains_regexpr ~/.pecs/misc-all.ali "alias remakeUsers='sudo /usr/local/bin/remakeUsers'"
#    fi
    if [ -z "$LM_LICENSE_FILE" ]; then
       echo "Environment LM_LICENSE_FILE is not defined (ERROR)"
    else
       echo "Environment LM_LICENSE_FILE is defined (OK)"
    fi
    gid=`id -gn`
    check_data_structure_astro archeso $gid $DFS_DATA_ROOT

    if [ -z "$DHS_HOST" ] ; then
       echo "Environment DHS_HOST is not defined (ERROR)"
    else
       echo "Environment DHS_HOST is defined (OK)"
       check_file_contains_regexpr ~archeso/.rhosts "^$DHS_HOST[ 	]*archeso$"
    fi
    if [ -z "$QcWS" ] ; then
       echo "Environment QcWS is not defined (ERROR)"
    else
       echo "Environment QcWS is defined (OK)"
       check_file_contains_regexpr ~archeso/.rhosts "^$QcWS[ 	]*pipeline$"
    fi
    if check_file_exist ~/.dataSubscriberConfig ; then
       check_file_contains_regexpr ~/.dataSubscriberConfig "^DHS_HOST-RAW.*$OlasWS"
       check_file_contains_regexpr ~/.dataSubscriberConfig "^DHS_CONFIG-RAW.*$DHS_CONFIG"
       check_file_contains_regexpr ~/.dataSubscriberConfig "^DHS_HOST-RED.*$QcWS"
       check_file_contains_regexpr ~/.dataSubscriberConfig "^DHS_CONFIG-RED.*pipeline@$QcWS"
    fi
    if check_file_exist ~/.dbrc ; then
       check_protections ~/.dbrc -rw-r--r--
       check_file_contains_regexpr ~/.dbrc "^$DSQUERY observations frameingest.*DPREP$"
       check_file_contains_regexpr ~/.dbrc "^$DSQUERY asto asto.*ASTOREP$"
       check_file_contains_regexpr ~/.dbrc "^$DSQUERY ambient archeso.*AMBIREP$"
    fi
    ;;

  archeso)
#    check_vlt_ccstype noccs
    check_env_contains_regexpr $LD_LIBRARY_PATH 'NOCCS' LD_LIBRARY_PATH
    check_env_contains_regexpr $PATH 'NOCCS' PATH
    check_protections ~ drwxr-xr-x
    check_file_exist ~/.bash_profile
#    check_directory_exist ~/.pecs
    check_is_not_a_link ~/.bashrc
    check_file_contains_regexpr ~/.bashrc 'if \[ "\$TERM" = "dumb" \]'
    if check_file_exist ~archeso/.rhosts ; then
      check_protections ~archeso/.rhosts -rw-r-----
    fi
#    Not here, the ~astroN:.dbrc is used instead
#    if check_file_exist ~/.dbrc ; then
#       check_protections ~/.dbrc -rw-r--r--
#       check_file_contains_regexpr ~/.dbrc "^$DSQUERY observations frameingest.*DPREP$"
#       check_file_contains_regexpr ~/.dbrc "^$DSQUERY asto asto.*ASTOREP$"
#       check_file_contains_regexpr ~/.dbrc "^$DSQUERY ambient archeso.*AMBIREP$"
#    fi
    ;;
  esac
fi

if [ ${HOSTNAME} = "$AstoWS" -o "$role" = "AstoWS" ]; then
  case ${DFS_USER} in
  archeso)
#    check_vlt_ccstype noccs
    check_env_contains_regexpr $LD_LIBRARY_PATH 'NOCCS' LD_LIBRARY_PATH
    check_env_contains_regexpr $PATH 'NOCCS' PATH
    check_AstoWS_installation
    check_file_exist ~/.bash_profile
#    check_directory_exist ~/.pecs
    check_is_not_a_link ~/.bashrc
    check_file_contains_regexpr ~/.bashrc 'if \[ "\$TERM" = "dumb" \]'
    if check_file_exist ~/.dbrc ; then
      check_protections ~/.dbrc -rw-------
      check_file_contains_regexpr ~/.dbrc "^$DSQUERY asto2 asto.*ASTOREP"
      check_file_contains_regexpr ~/.dbrc "^$DSQUERY opc70 asto.*OPCREP$"
    fi

    if check_file_exist ~/.netrc ; then
      check_protections ~/.netrc -rw-------
    fi

    if check_directory_exist ~/astoconf ; then
      check_file_exist ~/astoconf/astoControl.config
      check_file_exist ~/astoconf/dataSubscriberAsto.config
      check_file_exist ~/astoconf/libAd.config
      check_file_exist ~/astoconf/libAm.config
      check_file_exist ~/astoconf/writeControl.config
      check_file_exist ~/astoconf/mkisofs.config
    fi
    ;;
  web|http)
    check_http_installation
    ;;
  esac
fi
