/// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.2 $
// $Log: AmbientService.h,v $
// Revision 1.2  2007/10/25 09:08:23  cmoins
// adding of a constructor whose parameters all the database connection
// parameters + the observation site
//
// Revision 1.1  2007/09/21 15:31:51  cmoins
// ambient database service layer implementation
//
// 
//------------------------------------------------------------------------------

#ifndef AMBIENT_SERVICE_H
#define AMBIENT_SERVICE_H

#include <string>
#include "ServiceConnectionSyb.h"
#include "Seeing.h"
#include "SeeingDAOSyb.h"

/// Data Access Service layer to the Ambient database
class AmbientService
{
public:
 
  enum ObservationSite {PARANAL,LA_SILLA} ;

  /// \brief Constructor
  /// \param connAlias       data repository connection keyword
  /// \param observationSite site of observation to choose the right table
  AmbientService(std::string connAlias,
		 ObservationSite observationSite);

  /// \brief Constructor
  /**
   * \param dbServer DB server name
   * \param dbUser   DB user name
   * \param dbPasswd DB password
   * \param dbName   DB name
   */
  AmbientService(std::string dbServer,
		 std::string dbUser,
		 std::string dbPasswd,
		 std::string dbName,
		 ObservationSite observationSite);

  /// \brief Destructor
  ~AmbientService(){};

  /// \brief computes the mean seeing value on a given time range
  /**
   * \param mjdStart  time of exposure start expressed in mjd
   * \param expTime   total exposure time expressed in seconds
   * \return 0 if successful, -1 otherwise
   */
  int findMeanSeeingByDate(double mjdStart,
			   double expTime,
			   double& rmeanSeeing) const;

private :

  /// seeing objects DAO class
  SeeingDAOSyb seeingDAO_; 
  /// Database connection handler
  mutable ServiceConnectionSyb dsConnection_;

};
#endif // AMBIENT_SERVICE_H
