// -*-c++-*-
#ifndef _ArcQcrRep_h_
#define _ArcQcrRep_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcQcrRep.h,v 1.3 2005/10/17 13:36:39 nferguso Exp $
 *
 * ArcQcrRep.h - class definitions for class ArcQcrRep, a class 
 * representing an application that read/write from/to the
 * QC Report repository database
 *
 * who                  when         what
 * -------------------  -----------  ----------------------------------------
 * Elisabetta Angeloni  26 Jan 1998  Created
 * Neil Ferguson        23 Nov 2004  Converted to DfsStdLib/SybConn
 * Neil Ferguson        21 Sep 2005  Removed obsolete methods
 */

#include <ArcDbGen.h>
#include <ArcQCReport.h>
#include <ArcQCParam.h>

/**
 * This class and its methods are meant to be used for reading from
 * QC Reports repository database.
 * An instance consists of references to the database and the tables
 * where the observation blocks data resides.
 * This class includes a set of member functions for retrieving data.
 */

class ArcQcrRep : public ArcDbGen {

private:

	// QC Tables
    DfsCString  _qc_measurements ;
    DfsCString  _qc_param ;
    DfsCString  _qc_templ_sign_values ;
    DfsCString  _qc_ob_raw_mapping ;
    DfsCString  _qc_param_key_mapping ;
    DfsCString  _qc_observation_context ;
    // QC DB Report Table
    DfsCString qc_reportsTab_;
    // QC DB Report Parameter Table
    DfsCString qcr_paramTab_;
    
protected:
  void getTables();

public:

    
/**@name Constructors
*/
//@{
    // Default constructor: it supplies a connection to the QC report repository.
    // The default values are read from a configuration file
    ArcQcrRep(DfsCString dbAlias="QCREP");

    /// Constructor that requires username, password, server and database name (a default is supplied).
    // In case an empty server name is given, the value of $DSQUERY variable is taken
    ArcQcrRep(DfsCString dbName, DfsCString dbPasswd, 
              DfsCString dbServer="", DfsCString dbName="quc");

  int  getQcContext(DfsCString& context,
                    DfsCString& instrument,
                    DfsCString& mode,
                    DfsCString& tech);
		    
  int  getKeyfromParam (DfsCString &keyName,
                        DfsCString &parName,
                        DfsCString &context,
                        DfsCString &show,
                        DfsCString &min,
                        DfsCString &max);
  //@}

/**@name  Methods for data retrieval
 * These methods  retrieves data from the tables obs_blocks and
 * constraint_sets of OB Rep database using an outer join
 * All of them execute the following SELECT statement:
    SELECT qc_reports.dp_id
           qc_reports.ob_id 
           qc_reports.created 
           qc_reports.updated 
           qc_reports.qcr_st 
           qc_reports.qcr_com 
           qc_reports.do_cat 
           qc_reports.dp_st 
           qc_reports.dp_com 
           qc_reports.tp_name 
           qc_reports.tp_st 
           qc_reports.tp_com 
           qc_param.qcr_dp_id 
           qc_param.seeing 
           qc_param.seeing_tgt 
           qc_param.seeing_min 
           qc_param.seeing_max 
           qc_param.seeing_st 
           qc_param.seeing_com 
           qc_param.sky_transparency 
           qc_param.sky_transparency_tgt 
           qc_param.sky_transparency_min 
           qc_param.sky_transparency_max 
           qc_param.sky_transparency_st 
           qc_param.sky_transparency_com 
           qc_param.air_mass 
           qc_param.air_mass_tgt 
           qc_param.air_mass_min 
           qc_param.air_mass_max 
           qc_param.air_mass_st 
           qc_param.air_mass_com 
           qc_param.fli 
           qc_param.fli_tgt 
           qc_param.fli_min 
           qc_param.fli_max 
           qc_param.fli_st 
           qc_param.fli_com 
           qc_param.moon_distance 
           qc_param.moon_distance_tgt 
           qc_param.moon_distance_min 
           qc_param.moon_distance_max 
           qc_param.moon_distance_st 
           qc_param.moon_distance_com ;
	   FROM qc_reports, qcr_param
	   WHERE qc_reports.dp_id =* qcr_param.qcr_dp_id

	   * In addition, each function executes a specific where clause
	   */
  //@{

int  printTableContext();		  
int  printTableParamKeyMapping ();

  //@}


/*
 * These methods get the column's name of the tables qc_reports and
 * qcr_param of QC report Repository database.
 * These methods must be used for building a generic where clause
 */

 // Observation Block ID (column "qc_reports.ob_id")
  inline const DfsCString obsIdColumn() { return (qc_reportsTab_ + ".ob_id"); }
 
  // Data product ID (column "qc_reports.dp_id")
  inline const DfsCString dpIdColumn() { return (qc_reportsTab_ + ".dp_id"); }

  // Creation date (column "qc_reports.created")
  inline const DfsCString creationDateColumn() { return (qc_reportsTab_ + ".created"); }

  // Last modification date (column "qc_reports.updated")
  inline const DfsCString updateDateColumn() { return (qc_reportsTab_ + ".updated"); }

  // Information related to data product (column "qc_reports.qcr_com")
  inline const DfsCString qcrCommentColumn() { return (qc_reportsTab_ + ".qcr_com"); }

  // Status of the QC report (column "qc_reports.qcr_st")
  inline const DfsCString qcrStatusColumn() { return (qc_reportsTab_ + ".qcr_st"); }

  // DO category (column "qc_reports.do_cat")
  inline const DfsCString DOCatColumn() { return (qc_reportsTab_ + ".do_cat"); }

  // Information related to data product (column "qc_reports.dp_com")
  inline const DfsCString dpCommentColumn() { return (qc_reportsTab_ + ".dp_com"); }

  // Status of the data product (column "qc_reports.dp_st")
  inline const DfsCString dpStatusColumn() { return (qc_reportsTab_ + ".dp_st"); }

  // Template signature name (column "qc_reports.tp_name")
  inline const DfsCString tpNameColumn() { return (qc_reportsTab_ + ".tp_name"); }

  // Information related to the template signature (column "qc_reports.tp_com")
  inline const DfsCString tpCommentColumn() { return (qc_reportsTab_ + ".tp_com"); }

  // Status of the template signature (column "qc_reports.tp_st")
  inline const DfsCString tpStatusColumn() { return (qc_reportsTab_ + ".tp_st"); }

  // qc parameter dp_id (column "qcr_param.qcr_dp_id")
  inline const DfsCString qcrParamDpIdColumn() { return (qcr_paramTab_ + ".qcr_dp_id"); }
 
  // SEEING 
  // qc parameter seeing (column "qcr_param.seeing")
  inline const DfsCString seeingColumn() { return (qcr_paramTab_ + ".seeing"); }

  // qc parameter seeing target (column "qcr_param.seeing_tgt")
  inline const DfsCString seeingTargetColumn() { return (qcr_paramTab_ + ".seeing_tgt"); }

  // qc parameter seeing min (column "qcr_param.seeing_min")
  inline const DfsCString seeingMinColumn() { return (qcr_paramTab_ + ".seeing_min"); }

  // qc parameter seeing max (column "qcr_param.seeing_max")
  inline const DfsCString seeingMaxColumn() { return (qcr_paramTab_ + ".seeing_max"); }

  // qc parameter seeing status (column "qcr_param.seeing_st")
  inline const DfsCString seeingStatusColumn() { return (qcr_paramTab_ + ".seeing_st"); }

  // qc parameter seeing comment (column "qcr_param.seeing_com")
  inline const DfsCString seeingCommentColumn() { return (qcr_paramTab_ + ".seeing_com"); }
  
  // SKY TRANSPARENCY
  // qc parameter skyTransparency (column "qcr_param.sky_transparency")
  inline const DfsCString skyTransparencyColumn() { return (qcr_paramTab_ + ".sky_transparency"); }

  // qc parameter skyTransparency target (column "qcr_param.sky_transparency_tgt")
  inline const DfsCString skyTransparencyTargetColumn() { return (qcr_paramTab_ + ".sky_transparency_tgt"); }

  // qc parameter skyTransparency min (column "qcr_param.sky_transparency_min")
  inline const DfsCString skyTransparencyMinColumn() { return (qcr_paramTab_ + ".sky_transparency_min"); }

  // qc parameter skyTransparency max (column "qcr_param.sky_transparency_max")
  inline const DfsCString skyTransparencyMaxColumn() { return (qcr_paramTab_ + ".sky_transparency_max"); }

  // qc parameter skyTransparency status (column "qcr_param.sky_transparency_st")
  inline const DfsCString skyTransparencyStatusColumn() { return (qcr_paramTab_ + ".sky_transparency_st"); }

  // qc parameter skyTransparency comment (column "qcr_param.sky_transparency_com")
  inline const DfsCString skyTransparencyCommentColumn() { return (qcr_paramTab_ + ".sky_transparency_com"); }
           
  // AIR MASS
  // qc parameter airMass (column "qcr_param.air_mass")
  inline const DfsCString airMassColumn() { return (qcr_paramTab_ + ".air_mass"); }

  // qc parameter airMass target (column "qcr_param.air_mass_tgt")
  inline const DfsCString airMassTargetColumn() { return (qcr_paramTab_ + ".air_mass_tgt"); }

  // qc parameter airMass min (column "qcr_param.air_mass_min")
  inline const DfsCString airMassMinColumn() { return (qcr_paramTab_ + ".air_mass_min"); }

  // qc parameter airMass max (column "qcr_param.air_mass_max")
  inline const DfsCString airMassMaxColumn() { return (qcr_paramTab_ + ".air_mass_max"); }

  // qc parameter airMass status (column "qcr_param.air_mass_st")
  inline const DfsCString airMassStatusColumn() { return (qcr_paramTab_ + ".air_mass_st"); }

  // qc parameter airMass comment (column "qcr_param.air_mass_com")
  inline const DfsCString airMassCommentColumn() { return (qcr_paramTab_ + ".air_mass_com"); }

  // FLI 
  // qc parameter fli (column "qcr_param.fli")
  inline const DfsCString fliColumn() { return (qcr_paramTab_ + ".fli"); }

  // qc parameter fli target (column "qcr_param.fli_tgt")
  inline const DfsCString fliTargetColumn() { return (qcr_paramTab_ + ".fli_tgt"); }

  // qc parameter fli min (column "qcr_param.fli_min")
  inline const DfsCString fliMinColumn() { return (qcr_paramTab_ + ".fli_min"); }

  // qc parameter fli max (column "qcr_param.fli_max")
  inline const DfsCString fliMaxColumn() { return (qcr_paramTab_ + ".fli_max"); }

  // qc parameter fli status (column "qcr_param.fli_st")
  inline const DfsCString fliStatusColumn() { return (qcr_paramTab_ + ".fli_st"); }

  // qc parameter fli comment (column "qcr_param.fli_com")
  inline const DfsCString fliCommentColumn() { return (qcr_paramTab_ + ".fli_com"); }

  // MOON DISTANCE 
  // qc parameter moonDistance (column "qcr_param.moon_distance")
  inline const DfsCString moonDistanceColumn() { return (qcr_paramTab_ + ".moon_distance"); }

  // qc parameter moonDistance target (column "qcr_param.moon_distance_tgt")
  inline const DfsCString moonDistanceTargetColumn() { return (qcr_paramTab_ + ".moon_distance_tgt"); }

  // qc parameter moonDistance min (column "qcr_param.moon_distance_min")
  inline const DfsCString moonDistanceMinColumn() { return (qcr_paramTab_ + ".moon_distance_min"); }

  // qc parameter moonDistance max (column "qcr_param.moon_distance_max")
  inline const DfsCString moonDistanceMaxColumn() { return (qcr_paramTab_ + ".moon_distance_max"); }

  // qc parameter moonDistance status (column "qcr_param.moon_distance_st")
  inline const DfsCString moonDistanceStatusColumn() { return (qcr_paramTab_ + ".moon_distance_st"); }

  // qc parameter moonDistance comment (column "qcr_param.moon_distance_com")
  inline const DfsCString moonDistanceCommentColumn() { return (qcr_paramTab_ + ".moon_distance_com"); }
     
}; // end of definition of class ArcQcrRep

#endif /* _ArcQcrRep_h_ */

