// -*-c++-*-
#ifndef _ArcTemplate_h_
#define _ArcTemplate_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcTemplate.h,v 1.3 2005/08/18 17:03:27 nferguso Exp $
 *
 * ArcTemplate.h - class definitions for class ArcTemplate, a class 
 * representing an application that read/write from/to the
 * table template_signatures of OB repository database
 *
 * who                  when         what
 * -------------------  -----------  ----------------------------------------
 * Elisabetta Angeloni  26 Jan 1998  Created
 * Neil Ferguson        01 Nov 2004  Converted to DfsStdLib/SybConn
 */

#include "DfsCString.h"
#include "DfsTValSlist.h"
#include <sybconnection.h>
#include <iostream>

/**
 * This class is used by the ArcObRep class for storing the data retrieved or
 * to be recorded in the database table template_signatures; 
 * this class contains only methods
 * for accessing the class's members, none of them access the database fields. 
 */
class ArcTemplate {

private:

    // Template parameters (table "template_signatures")
    
    // Observation Block ID (column "od_id")
    int templateId_;
    
    // Template name (column "item_name")
    DfsCString templateName_;
    
public:
    
/**@name Constructors
 */
//@{
    /// Default constructor
    ArcTemplate();
    
//@}
    
    //
    DfsBoolean operator== (const ArcTemplate& t) const;
    
/**@name Members access (Read mode)
 */
//@{
    
    /// Get the value of the column template_signatures.id
    // Get the value of private member templateId_ 
    inline int getId() {return templateId_;}
    
    /// Get the value of the column template_signatures.item_name
    // Get the value of private member templateName_
    inline const DfsCString& getName() {return templateName_;}
    
//@}
    
/**@name Members access (Write mode)
 */
//@{

    /// Set the value of the column template_signatures.id
    // Set the value of private member templateId_
    inline void id(int id) {templateId_ = id;}
    
    /// Set the value of the column template_signatures.item_name
    // Set the value of private member templateName_
    inline void name(DfsCString name) {templateName_ = name;}
    
    /// @brief Set private member values based on the current row of the 
    /// given SybaseResultSet. 
    inline void storeSQLQueryResult(SybaseResultSet& results); 

//@}

/**
****************************************************************************
* This method prints out the content of the ArcTemplate class

@memo
Print out the content of the ArcTemplate class
@param
output stream (default is standard output)
*/
    
    std::ostream& print(std::ostream& o) const;
    
}; // end of definition of class ArcTemplate

inline ArcTemplate::ArcTemplate() 
    : templateId_(0), 
      templateName_("")
{ ; }

inline DfsBoolean ArcTemplate::operator== (const ArcTemplate& t) const
{
    return templateName_ == t.templateName_;
}

// Set private member values based on the current row of the given 
// SybaseResultSet. 
inline void ArcTemplate::storeSQLQueryResult(SybaseResultSet& results)
{
    // Initialise all affected members before reading new data. 
	templateId_ = 0;
	templateName_ = "";
	
    // Column count and current column name. 
    int numColumns = results.getColumnCount(); 
    DfsCString columnName; 

    // Read new data. 
    for(int i = 1; i <= numColumns; ++i)
    {
        columnName = results.getColumnName(i); 

        if(columnName.contains("od_id"))
        {
            templateId_ = results.getLongValue(i);
        }
        else if(columnName.contains("item_name"))
        {
            templateName_ = results.getStringValue(i);
        }
    }
}

inline std::ostream& ArcTemplate::print(std::ostream& aStream) const
{
    aStream << "Template id: " << templateId_ << " Name: " << templateName_ << std::endl;
    
    return aStream;
}

/**
****************************************************************************
* It is used to create an instance of DfsTValSlist 
* that is a single linked list of ArcTemplate instances
* An instance of ArcTemplateList can be accessed like an array, 
* by using the operator[]. 
* The call to the entries() function will return the number
* of ArcTemplate instances actually in the list.
* NOTE: Since instances of ArcTemplate in the DfsTValSlist instance
* were created on the heap, they must be destroyed manually.
* The clear() member function of DfsTValSlist
* takes care of that task.
@memo
It is used to create an instance of DfsTValSlist 
that is a list of ArcTemplate instances
@see
DfsTValSlist template class
*/

typedef DfsTValSlist<ArcTemplate> ArcTemplateList;

#endif /* _ArcTemplate_h_ */
