// -*-c++-*-
#ifndef _ArcTplParam_h_
#define _ArcTplParam_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcTplParam.h,v 1.4 2008/09/05 15:36:18 cguirao Exp $
 *
 * ArcTplParam.h - class definitions for class ArcTplParam, a class 
 * representing an application that read/write from/to the
 * table template_signatures of OB repository database
 *
 * who                  when         what
 * -------------------  -----------  ----------------------------------------
 * Elisabetta Angeloni  26 Jan 1998  Created
 * Neil Ferguson        02 Nov 2004  Converted to DfsStdLib/SybConn
 */

#include "DfsCString.h"
#include "DfsTValSlist.h"
#include <sybconnection.h>
#include <iostream>

/**
 * This class is used by the ArcObRep class for storing the data retrieved or
 * to be recorded in the database table template_signatures; 
 * this class contains only methods
 * for accessing the class's members, none of them access the database fields. 
 */
class ArcTplParam {
    
private:
    
    // Template parameter (table "parameters")
    
    // param ID (column "param_index")
    int paramId_;
    
    // param  name (column "item_name")
    DfsCString paramName_;
    
    // param  value (column "value")
    DfsCString paramValue_;
    
    // param  type (column "type")
    DfsCString paramType_;
    
public:
    
/**@name Constructors
 */
//@{
    /// Default constructor
    ArcTplParam();
    
//@}
    void init();
    
    //
    DfsBoolean operator== (const ArcTplParam& t) const;
    
/**@name Members access (Read mode)
 */
//@{
    
    /// Get the value of the column parameters.id
    inline int getId() {return paramId_;}
    
    /// Get the value of the column parameters.item_name
    inline const DfsCString& getName() {return paramName_;}
    
    /// Get the value of the column parameters.value
    inline const DfsCString& getValue() {return paramValue_;}
    
    /// Get the value of the column parameters.type
    inline const char getType() {
	return *(paramType_.data());
    }
    
//@}
    
/**@name Members access (Write mode)
 */
//@{
    
    /// Set the value of the column template_signatures.param_index
    // Set the value of private member paramId_
    inline void id(int id) {paramId_ = id;}
    
    /// Set the value of the column template_signatures.item_name
    // Set the value of private member paramName_
    inline void name(DfsCString name) {paramName_ = name;}
    
    /// Set the value of the column template_signatures.value
    // Set the value of private member paramValue_
    inline void value(DfsCString value) {paramValue_ = value;}
    
    /// Set the value of the column template_signatures.type
    // Set the value of private member paramType_
    inline void type(char type) {paramType_ = type;}
    
    /// @brief Set private member values based on the current row of the 
    /// given SybaseResultSet. 
    void storeSQLQueryResult(SybaseResultSet& results); 

//@}
    
/**
****************************************************************************
* This method prints out the content of the ArcTplParam class

@memo
Print out the content of the ArcTplParam class
@param
output stream (default is standard output)
*/
    
    std::ostream&  print(std::ostream& o) const;
    
}; // end of definition of class ArcTplParam

inline ArcTplParam::ArcTplParam()  { 
    init();
}

inline void ArcTplParam::init() {
    paramId_ = 0;
    paramName_ = "";
    paramValue_ = "";
    paramType_ = 'E';
}

inline DfsBoolean ArcTplParam::operator== (const ArcTplParam& t) const
{
    return paramId_ == t.paramId_;
}

// Set private member values based on the current row of the given 
// SybaseResultSet. 
inline void ArcTplParam::storeSQLQueryResult(SybaseResultSet& results)
{
    // Initialise all affected members before reading new data. 
	paramId_ = 0;
	paramValue_ = paramType_ = paramName_ = "";
	
    // Column count and current column name. 
    int numColumns = results.getColumnCount(); 
    DfsCString columnName; 

    // Read new data. 
    for(int i = 1; i <= numColumns; ++i)
    {
        columnName = results.getColumnName(i); 

        if(columnName.contains("param_index"))
        {
            paramId_ = results.getLongValue(i);
        }
        else if(columnName.contains("value"))
        {
            paramValue_ = results.getStringValue(i);
        }
        else if(columnName.contains("type"))
        {
            paramType_ = results.getStringValue(i);
        }
        else if(columnName.contains("item_name"))
        {
            paramName_ = results.getStringValue(i);
        }
    }
}

inline std::ostream& ArcTplParam::print(std::ostream& aStream) const
{
    aStream << "param id: " << paramId_ 
	    << " type: "    << paramType_ 
	    << " name: "    << paramName_ 
	    << " value: "   << paramValue_ 
	    << std::endl;
    return aStream;
}

/**
****************************************************************************
* It is used to create an instance of DfsTValSlist 
* that is a single linked list of ArcTplParam instances
* An instance of ArcTplParamList can be accessed like an array, 
* by using the operator[]. 
* The call to the entries() function will return the number
* of ArcTplParam instances actually in the list.
* NOTE: Since instances of ArcTplParam in the DfsTValSlist instance
* were created on the heap, they must be destroyed manually.
* The clear() member function of DfsTValSlist
* takes care of that task.
@memo
It is used to create an instance of DfsTValSlist 
that is a list of ArcTplParam instances
@see
DfsTValSlist template class
*/

typedef DfsTValSlist<ArcTplParam> ArcTplParamList;

#endif /* _ArcTplParam_h_ */
