//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Michael Wiedmer - ESO/DMD/DPG>
//SUBSYSTEM: <Core>
//
//$Revision: 1.1 $
//$Log: CorBase.h,v $
//Revision 1.1  2004/09/06 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/23 12:47:09  cguirao
//First check in
//
//Revision 1.3  1998/06/30 08:49:48  dataflow
//Removing $NAME
//
//Revision 1.2  1997/12/03 15:05:47  dfadmin
//update for NTT Jan 98
//
//Revision 1.9  1997/07/11 08:32:09  mwiedmer
//use new CorDebug class
//
//Revision 1.8  1997/04/11 13:31:55  mwiedmer
//use RWStrings
//
//Revision 1.1  1997/04/10 08:05:44  dfadmin
//Initial revision
//
//Revision 1.7  1996/11/25 09:34:21  mwiedmer
//added operators new and delete for persistence
//purposes
//
//Revision 1.6  1996/10/07 14:38:10  mwiedmer
//added CorBase::eps
//
//Revision 1.5  1996/09/23 09:10:48  mwiedmer
//added CorUnitTest
//
//Revision 1.4  1996/09/17 11:12:27  mwiedmer
//added tracing
//
//Revision 1.3  1996/09/10 11:22:21  mwiedmer
//update debugging and debug printing
//mechanism
//
//Revision 1.2  1996/09/03 14:41:26  mwiedmer
//added RCS Name tag
//
//Revision 1.1  1996/09/02 11:04:00  dfadmin
//First release
//
//Revision 1.3  1996/09/02 09:45:34  mwiedmer
//first release
//
//Revision 1.2  1996/09/02 08:25:29  mwiedmer
//*** empty log message ***
//
//-----------------------------------------------------------------------------

#ifndef COR_BASE_H
#define COR_BASE_H

// includes from this library
// includes from other ESO libraries
#include <cor_assert.h>
#include <CorDebug.h>
#include <CorTrace.h>
#include <CorBool.h>
#include <CorCheckValid.h>
// includes from other libraries
// system includes


// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorUnitTest;
#define corCountOf(array) (sizeof(array)/sizeof(array[0]))

/// CorBase - THE Data Flow System Base Class
class CorBase {
/**
  {\footnotesize $Id: CorBase.h,v 1.1 2004/09/06 14:35:21 cguirao Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <CorBase.h>
    
    CorBase myBase;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This {\tt}base{\tt} class is the one and only root for all data flow classes.
  It implements the assertion and debugging facilities.
*/
public:

  ///
  static const double eps;
  /** a "small" number, ie. you should not do:
  \begin{verbatim}
  if (double1 == double2) {
  }
  \end{verbatim}
  but
  \begin{verbatim}
  if (double1 - double2 < CorBase::eps) {
  }
  \end{verbatim}
  */

  ///
  CorBase();
  /** {\large Purpose} \\
  The default c'tor.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  CorBase(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorBase& aRef
    /** the reference instance */
  );


  ///
  virtual ~CorBase();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  CorBase& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorBase& aRef
    /** the reference instance */
  );

  ///
  CorBool operator==(
    /** {\large Purpose} \\
    Query whether two instances are the same.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    True or False.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorBase& aRef
    /** a reference */
  ) const;


  /// not-equal operator
  CorBool operator!=(
    /** {\large Purpose} \\
    Query whether two instances are not the same.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    True or False.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorBase& aRef
    /** a reference */
  ) const;


  /// generate the assertion text and stop the program
  static void reportAssertFailure(
    /** {\large Purpose} \\
    This function will generate an assertion report.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const char* aFilename,
    /** file name of the error */
    ///
    int aLinenumber,
    /** line number */
    /// 
    const char* aText
    /** additional explanation */
  );


  ///
  void* operator new(
    /** {\large Purpose} \\
    Provide our own operator new which also takes an address.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns the address of the newly initialized instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    size_t aSize,
    /** size of a CorBase instance */
    ///
    void* anotherPlace
    /** memory location where object should be stored */
  );


  ///
  void* operator new(
    /** {\large Purpose} \\
    The normal operator new overloaded.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    size_t aSize
    /** size of a CorBase instance */
  );


  ///
  void operator delete(
    /** {\large Purpose} \\
    This function will generate an assertion report.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    void* anInstance
    /** address of a CorBase instance */
  );


protected:
  COR_CHECKVALID_CLASS;  // this protects our instances from invalid access

private:

  friend void utest_CorBase(CorUnitTest&);  // a tester class that has
                                            // access to ALL members

};

#endif
