//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Michael Wiedmer - ESO/DMD/DPG>
//SUBSYSTEM: <Core>
//
//$Revision: 1.1 $
//$Log: CorBool.h,v $
//Revision 1.1  2004/09/06 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/23 12:47:09  cguirao
//First check in
//
//Revision 1.2  1998/06/30 08:49:48  dataflow
//Removing $NAME
//
//Revision 1.1  1997/04/10 08:05:44  dfadmin
//Initial revision
//
//Revision 1.3  1996/09/10 11:29:36  mwiedmer
//standardised
//
//Revision 1.2  1996/09/03 14:41:26  mwiedmer
//added RCS Name tag
//
//Revision 1.1  1996/09/02 11:04:01  dfadmin
//First release
//
//Revision 1.3  1996/09/02 09:45:34  mwiedmer
//first release
//
//Revision 1.2  1996/09/02 08:25:30  mwiedmer
//*** empty log message ***
//
//-----------------------------------------------------------------------------

#ifndef COR_BOOL_H 
#define COR_BOOL_H 

/// Boolean value: TRUE or FALSE
typedef int CorBool;
/**
  {\footnotesize $Id: CorBool.h,v 1.1 2004/09/06 14:35:21 cguirao Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <CorBool.h>
    
    CorBool boolean = corTRUE;
    boolean = corFALSE;
  \end{verbatim}

  {\large DESCRIPTION} \\
   This class has the following restriction: a non-zero integer is not the
   same as corTRUE.  This semantic restriction could be removed by making
   CorBool a class.  This, however, was found to be inefficient. \\
   CorBool is not an enumerated type, otherwise all conditional expressions
   must be cast to CorBool, which is tedious and looks horrible. \\
   The alternatives are to use a DEFINE or a typedef, the latter of which
   is better.

  {\large CAUTIONS} \\
  \begin{verbatim}
  CorBool boolean = -14; // this does not generate a compiler error
  void foo(CorBool);
  void foo(int);

  foo(-14); // might call foo(CorBool);
  
  \end{verbatim}
*/

const CorBool corFALSE = 0;
const CorBool corTRUE = 1;

#endif

// Set mode for emacs.
// Local Variables:
// mode:C++
// End:
