//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: System Core
//
//$Revision: 1.1 $
//$Log: CorMapPtrToPtr.h,v $
//Revision 1.1  2004/09/06 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/23 12:47:09  cguirao
//First check in
//
//Revision 1.2  1998/06/30 08:49:49  dataflow
//Removing $NAME
//
//Revision 1.1  1997/04/10 08:05:44  dfadmin
//Initial revision
//
//Revision 1.1  1996/11/25 09:34:50  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef COR_MAPPTRTOPTR_H
#define COR_MAPPTRTOPTR_H

// includes from this library
// includes from other ESO libraries
#include <CorPersistent.h>
//#include <CorPlexer.h>
// includes from other libraries
// system includes

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorUnitTest;
class CorPlexer;


/// A simple pointer to pointer map
class CorMapPtrToPtr : public CorPersistent {
  DECLARE_DYNAMIC(CorMapPtrToPtr);
  /**
  {\footnotesize $Id: CorMapPtrToPtr.h,v 1.1 2004/09/06 14:35:21 cguirao Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <CorMapPtrToPtr.h>
  
    CorMapPtrToPtr myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class represents a pointer to pointer map.

  {\large CAUTIONS} \\
  <optional> \\

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  \Ref{CorPersistent}\\
  */

public:

  /// 
  CorMapPtrToPtr(
    /** {\large Purpose} \\
    The default c'tor.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\    
    */
    ///
    int aSize = 10
    /** initial size */
  );
  

  /// 
  CorMapPtrToPtr(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorMapPtrToPtr& aRef
    /** the reference instance */
  );
    

  /// 
  CorMapPtrToPtr& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorMapPtrToPtr& aRef
    /** the reference instance */
  );
    

  ///
  virtual ~CorMapPtrToPtr();  // do you need keyword virtual here?
  /** {\large Purpose} \\
  The d'tor. If it's not in a hierarchy it probably doesn't want to be
  virtual.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  unsigned int size() const;
  /** {\large Purpose} \\
  Query the number of elements in the map.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the number of elements.
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  CorBool empty() const;
  /** {\large Purpose} \\
  Query whether this map is empty.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns corTRUE if this map is empty.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


 ///
  CorBool lookUp(
    /** {\large Purpose} \\
    Lookup the value to a given key.  

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corFALSE if the key is not in this map.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    void* aKey,
    /** My new value */
    ///
    void*& aValue
    /** Another value */
  );


  ///
  void*& operator[](
    /** {\large Purpose} \\
    Lookup and add if not there.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns the value that matches the key.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    void* aKey
    /** key to look for */
  );


  ///
  void setAt(
    /** {\large Purpose} \\
    Add a new (key, value) pair.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    void* aKey,
    /** the key */
    ///
    void* aValue
    /** the value */
  );


  ///
  CorBool removeKey(
    /** {\large Purpose} \\
    Remove an existing (key, ?) pair.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE if it could remove the pair.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    void* aKey
    /** key of the pair to remove */
  );


  ///
  void removeAll();
  /** {\large Purpose} \\
  Remove all the key value pairs.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  void* getStartPosition() const;
  /** {\large Purpose} \\
  Iterate all (key, value) pair.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns something?

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  void getNextAssoc(
    /** {\large Purpose} \\
    Iterate over key value pairs.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    void*& aNewPosition,
    /** position (output) */
    ///
    void* aKey,
    /** key (output) */
    ///
    void* aValue
    /** value (output) */
  ) const;


  ///
  unsigned int getHashTableSize() const;
  /** {\large Purpose} \\
  Query the size of the hash table.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the current (full) size of the hash table.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Arguments}\\
  */


  ///
  void initHashTable(
    /** {\large Purpose} \\
    Initialize the hash table.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned int aSize,
    /** required size of the table */
    ///
    CorBool allocNow = corTRUE
    /** do we need to allocate it now */
  );


  ///
  unsigned int hashKey(
    /** {\large Purpose} \\
    Query the hash value of a given key.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns the hash value (the slot) of the key.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    void* aKey
    /** key to check */
  ) const;


  ///
  void stream(
    /** {\large Purpose} \\
    Stream this instance through a file.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    CorStream& aFile
    /** file to stream through */
  );

  ///
  virtual CorPersistent* makeClone() const;
  /** {\large Purpose} \\
  Make a clone of this instance.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function retutns a pointer to a newly created instance that has the same
  properties as this one.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */

  
protected:

  // Map Association
  struct CorMapAssoc {
    CorMapAssoc* mNext;
    unsigned int mHashValue;  // needed for efficient iteration
    void* mKey;
    void* mValue;
  };


  ///
  CorMapAssoc* newAssoc();
  /** {\large Purpose} \\
  Create a new map association.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns a pointer to a new CorMapAssociation.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  void deleteAssoc(
    /** {\large Purpose} \\
    Delete a map association.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    CorMapAssoc* aMapAssoc
    /** the association to delete */
  );


  ///
  CorMapAssoc* getAssocAt(
    /** {\large Purpose} \\
    Query the association at a specific hash key.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns the map association.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    void* aKey,
    /** key to match */
    ///
    unsigned int& aHash
    /** hash value of the key (output) */
  ) const;


private:
  friend void utest_CorMapPtrToPtr(CorUnitTest&);  // the unit test function

  CorMapAssoc**     mHashTable;
  unsigned int      mHashTableSize;
  int               mCount;
  CorMapAssoc*      mFreeList;
  struct CorPlexer* mBlocks;
  int               mBlockSize;
};

#endif // COR_MAPPTRTOPTR_H
