//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: System Core
//
//$Revision: 1.1 $
//$Log: CorPtrArray.h,v $
//Revision 1.1  2004/09/06 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/23 12:47:09  cguirao
//First check in
//
//Revision 1.2  1998/06/30 08:49:51  dataflow
//Removing $NAME
//
//Revision 1.1  1997/04/10 08:05:44  dfadmin
//Initial revision
//
//Revision 1.1  1996/11/25 09:35:58  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef COR_PTRARRAY_H
#define COR_PTRARRAY_H

// includes from this library
// includes from other ESO libraries
#include <CorPersistent.h>
// includes from other libraries
// system includes

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorUnitTest;

/// A array of pointers
class CorPtrArray : public CorPersistent {
  DECLARE_DYNAMIC(CorPtrArray);
  /**
  {\footnotesize $Id: CorPtrArray.h,v 1.1 2004/09/06 14:35:21 cguirao Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <CorPtrArray.h>
  
    CorPtrArray myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This text serves as an example for the documentation of a class definition

  {\large CAUTIONS} \\
  <optional> \\

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  <optional> \\
  \Ref{CorBase}\\
  */

public:

  /// 
  CorPtrArray();
  /** {\large Purpose} \\
  The default c'tor.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */
    

  /// 
  CorPtrArray(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorPtrArray& aRef
    /** the reference instance */
  );
    

  /// 
  CorPtrArray& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorPtrArray& aRef
    /** the reference instance */
  );
    

  ///
  virtual ~CorPtrArray();  // do you need keyword virtual here?
  /** {\large Purpose} \\
  The d'tor. If it's not in a hierarchy it probably doesn't want to be
  virtual.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  unsigned int getUpperBound() const;
  /** {\large Purpose} \\
  Query the upper bound of this array (next free position).

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the position of the next available slot.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  unsigned int getSize() const;
  /** {\large Purpose} \\
  Query the size of the array.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the nuber of elements in the array.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  void setSize(
    /** {\large Purpose} \\
    Set the array size of this instance.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned int aNewSize,
    /** new size of the array */
    ///
    int aGrwothRate = -1
    /** rate by which the array should grow */
  );


  ///
  void freeExtra();
  /** {\large Purpose} \\
  Clear extra memory.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  void removeAll();
  /** {\large Purpose} \\
  Clean up the array.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  void* getAt(
    /** {\large Purpose} \\
    Query the element at a given position.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns a pointer to the element.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned int aPosition
    /** position of the required element */
  ) const;


  ///
  void setAt(
    /** {\large Purpose} \\
    Set the element for a given position.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned int aPosition,
    /** position */
    ///
    void* anElement
    /** element */
  );


  ///
  void*& elementAt(
    /** {\large Purpose} \\
    Query the element at a given position.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns the element at the position.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned int aPosition
    /** position of the element */
  );


  ///
  void setAtGrow(
    /** {\large Purpose} \\
    Set the element for a given position, potentially grow the array.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned int aPosition,
    /** position of the element */
    ///
    void* anElement
    /** new element */
  );


  ///
  unsigned int add(
    /** {\large Purpose} \\
    Add a new element to the array, potentially grow the array.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns the index of the new element.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    void* anElement
    /** the new element to add */
  );


  ///
  void* operator[](
    /** {\large Purpose} \\
    Query the element at a given position.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns a pointer to the element.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned int aPosition
    /** position of the elment */
  ) const;


  ///
  void*& operator[](
    /** {\large Purpose} \\
    Query the element at a given position.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns the element.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned int aPosition
    /** position of the element */
  );


  ///
  void insertAt(
    /** {\large Purpose} \\
    Insert an element at a given position, move the following ones backward
    by a specified number of positions.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned int aPosition,
    /** position of the element */
    ///
    void* anElement,
    /** element to insert */
    ///
    unsigned int aCount = 1
    /** number of positions to move the others back by */
  );


  ///
  void removeAt(
    /** {\large Purpose} \\
    Remove an element at a given position and move the following ones forward
    by a specified number of positions.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned int aPosition,
    /** position of the element */
    ///
    unsigned int aCount = 1
    /** number of positions to move the others forward by */
  );


  ///
  void insertAt(
    /** {\large Purpose} \\
    Insert a whole array into this one at a given position.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned int aPostion,
    /** position of the array */
    ///
    CorPtrArray* anArray
    /** array to insert */
  );


  ///
  virtual CorPersistent* makeClone() const;
  /** {\large Purpose} \\
  Make a clone of this instance.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function retutns a pointer to a newly created instance that has the same
  properties as this one.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */

  virtual void stream(CorStream& aFile);

protected:

private:
  friend void utest_CorPtrArray(CorUnitTest&);  // the unit test function

  void** mData;    // the actual array of data
  int mSize;       // # of elements (upperBound - 1)
  int mMaxSize;    // max allocated
  int mGrowBy;     // grow amount

};

#endif // COR_PTRARRAY_H
