//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: System Core
//
//$Revision: 1.1 $
//$Log: CorRunTimeType.h,v $
//Revision 1.1  2004/09/06 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/23 12:47:09  cguirao
//First check in
//
//Revision 1.2  1998/06/30 08:49:52  dataflow
//Removing $NAME
//
//Revision 1.1  1997/04/10 08:05:44  dfadmin
//Initial revision
//
//Revision 1.1  1996/11/25 09:36:55  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef COR_RUNTIMETYPE_H
#define COR_RUNTIMETYPE_H

// includes from this library
#include <CorBool.h>
// includes from other ESO libraries
// includes from other libraries
// system includes

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorUnitTest;
class CorPersistent;
class CorStream;
class CorRunTimeType;


#if 0
struct CorClassState
{
  CorRunTimeType* mFirstClass;
  CorClassState();
};


extern CorClassState corClassState;
#define getCorClassState() (&corClassState)
#endif


// generate static object constructor for class registration
struct CorClassInit {
  CorClassInit(CorRunTimeType* aNewClass);
};


// helper macros
#define RUNTIME_TYPE(className) (&className::class##className)

#define IMPLEMENT_RUNTIMETYPE(className, baseClassName, aVersion, aConstructor)\
	CorRunTimeType className::class##className = { \
          #className, \
	  sizeof(className), \
	  aVersion, \
	  aConstructor, \
	  RUNTIME_TYPE(baseClassName), \
	  NULL \
	}; \
	static const CorClassInit init##className(RUNTIME_TYPE(className)); \
	CorRunTimeType* className::getRunTimeType() const \
	{ \
	  return RUNTIME_TYPE(className); \
	}


#define DECLARE_DYNAMIC(className) \
        public: \
	static  CorRunTimeType  class##className; \
	virtual CorRunTimeType* getRunTimeType() const;

#define IMPLEMENT_DYNAMIC(className, baseClassName) \
	IMPLEMENT_RUNTIMETYPE(className, baseClassName, 0xFFFF, NULL)


// not serializable, but dynamically constructable
#define DECLARE_DYNCREATE(className) \
	DECLARE_DYNAMIC(className) \
	static void construct(void* aSpace);

#define IMPLEMENT_DYNCREATE(className, baseClassName) \
	void className::construct(void* aSpace) \
	{ \
	  new(aSpace) className; \
	} \
	IMPLEMENT_RUNTIMETYPE(className, baseClassName, 0xFFFF, \
			      className::construct)


/// A simple runtime typing structure
struct CorRunTimeType {
  /**
  {\footnotesize $Id: CorRunTimeType.h,v 1.1 2004/09/06 14:35:21 cguirao Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <CorRunTimeType.h>
  
    CorRunTimeType myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class provides basic runtime typing information.

  {\large CAUTIONS} \\
  <optional> \\

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  <optional> \\
  */

public:

#if 0
  /// 
  CorRunTimeType();
  /** {\large Purpose} \\
  The default c'tor.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  CorRunTimeType(
    const char*  aClassName,
    unsigned int aObjectSize,
    unsigned int aVersion,
    void (*aConstruct)(void*),
    CorRunTimeType* aBaseClass,
    CorRunTimeType* aNextClass
  )
  {
  mClassName  = aClassName;
  mObjectSize = aObjectSize;
  mVersion    = aVersion;
  mConstruct  = aConstruct;
  mBaseClass  = aBaseClass;
  mNextClass  = aNextClass;
  }

  
  /// 
  CorRunTimeType(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorRunTimeType& aRef
    /** the reference instance */
  );
    

  /// 
  CorRunTimeType& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorRunTimeType& aRef
    /** the reference instance */
  );
    

  ///
  virtual ~CorRunTimeType();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */
#endif

  ///
  static CorRunTimeType* getRunTimeTypes();
  /** {\large Purpose} \\
  Query the list of registered run time types.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns a pointer to the first registered runtime typed
  class.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  static void addRunTimeType(
    /** {\large Purpose} \\
    Add a new class to the list of egistered run time types.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    /{\large Arguments}\\
    */
    ///
    CorRunTimeType* aNewClass
    /** new class to add */
  );


  ///
  CorPersistent* createObject();
  /** {\large Purpose} \\
  Create an object.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns a pointer to a new base object.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  CorBool constructObject(
    /** {\large Purpose} \\
    Construct an object in the space supplied.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE if the object could be constructed.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    void* aSpace
    /** space to put the object */
  );


  ///
  void store(
    /** {\large Purpose} \\
    Write this instance into a file.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    CorStream& aFile
    /** file to write to */
  );


  ///
  static CorRunTimeType* load(
    /** {\large Purpose} \\
    Load an instance from a file.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns a fresh CorRunTimeType instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    CorStream& aFile,
    /** file to read from */
    ///
    unsigned int& aVersion
    /** version of the instance */
  );

  const char*  mClassName;    // name of this class
  unsigned int mObjectSize;   // size of an instance of this class
  unsigned int mVersion;      // version of this class
  void (*mConstruct)(void*);  // constructor function: NULL => abstract class
  CorRunTimeType* mBaseClass; // our immediate base class
  CorRunTimeType* mNextClass; // linked list of registered classes

  static CorRunTimeType* mFirstClass; // all registered classes
};

#endif // COR_RUNTIMETYPE_H
