//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: System Core
//
//$Revision: 1.2 $
//$Log: CorStream.h,v $
//Revision 1.2  2004/10/26 15:23:54  nferguso
//Temporary check-in for repository move.
//
//Revision 1.1  2004/09/06 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/23 12:47:09  cguirao
//First check in
//
//Revision 1.2  1998/06/30 08:49:52  dataflow
//Removing $NAME
//
//Revision 1.1  1997/04/10 08:05:44  dfadmin
//Initial revision
//
//Revision 1.3  1996/12/17 09:19:22  mwiedmer
//add a few default arguments
//
//Revision 1.2  1996/12/13 14:33:20  mwiedmer
//*** empty log message ***
//
//Revision 1.1  1996/11/25 09:37:16  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef COR_STREAM_H
#define COR_STREAM_H

// includes from this library
#include <CorBase.h>
#include <CorBool.h>
#include <CorRef.h>
// includes from other ESO libraries
#include "DfsCString.h"
// includes from other libraries
// system includes
#include <fstream>

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorUnitTest;
class CorPersistent;
class CorRunTimeType;
class CorMapPtrToPtr;
class CorPtrArray;

/// A simple stream class 
class CorStream : public CorBase {
  /**
  {\footnotesize $Id: CorStream.h,v 1.2 2004/10/26 15:23:54 nferguso Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <CorStream.h>
  
    CorStream myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class provides the stream operations for the basic types an a
  persistent object.

  {\large CAUTIONS} \\

  {\large EXAMPLES} \\

  {\large SEE ALSO} \\
  \Ref{CorBase}\\
  \Ref{CorPersistent}\\
  */

public:

  enum StreamFlow {
    READ,
    WRITE,
    READ_WRITE
  };

  /// 
  CorStream();
  /** {\large Purpose} \\
  The default c'tor.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */
    

  /// 
  CorStream(
    /** {\large Purpose} \\
    The default c'tor.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aFileName,
    /** filename to use with this stream */
    ///
    StreamFlow aMode
    /** mode to open in */
  );


  /// 
  CorStream(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorStream& aRef
    /** the reference instance */
  );
    

  /// 
  CorStream& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorStream& aRef
    /** the reference instance */
  );
    

  ///
  virtual ~CorStream();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  CorStream& operator>>(
    /** {\large Purpose} \\
    Stream in some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    char& aChar
    /** data to stream */
  );


  ///
  CorStream& operator>>(
    /** {\large Purpose} \\
    Stream in some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    int& anInt
    /** data to stream */
  );


  ///
  CorStream& operator>>(
    /** {\large Purpose} \\
    Stream in some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    short& aValue
    /** data to stream */
  );
  

  ///
  CorStream& operator>>(
    /** {\large Purpose} \\
    Stream in some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned short& aValue
    /** data to stream */
  );
  

  ///
  CorStream& operator>>(
    /** {\large Purpose} \\
    Stream in some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    double& aDouble
    /** data to stream */
  );


  ///
  CorStream& operator>>(
    /** {\large Purpose} \\
    Stream in some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    DfsCString& aString
    /** data to stream */
  );


  ///
  CorStream& operator<<(
    /** {\large Purpose} \\
    Stream out some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const char aChar
    /** data to stream */
  );


  ///
  CorStream& operator<<(
    /** {\large Purpose} \\
    Stream out some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const int anInt
    /** data to stream */
  );


  ///
  CorStream& operator<<(
    /** {\large Purpose} \\
    Stream out some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const short anInt
    /** data to stream */
  );


  ///
  CorStream& operator<<(
    /** {\large Purpose} \\
    Stream out some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const unsigned short anInt
    /** data to stream */
  );


  ///
  CorStream& operator<<(
    /** {\large Purpose} \\
    Stream out some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const double aDouble
    /** data to stream */
  );


  ///
  CorStream& operator<<(
    /** {\large Purpose} \\
    Stream out some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const DfsCString& aString
    /** data to stream */
  );


  ///
  CorStream& operator>>(
    /** {\large Purpose} \\
    Stream in some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned char& aChar
    /** data to stream */
  );


  ///
  CorStream& operator>>(
    /** {\large Purpose} \\
    Stream in some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    unsigned int& anInt
    /** data to stream */
  );


  ///
  CorStream& operator<<(
    /** {\large Purpose} \\
    Stream out some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const unsigned char aChar
    /** data to stream */
  );


  ///
  CorStream& operator<<(
    /** {\large Purpose} \\
    Stream out some data.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns an updated stream.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const unsigned int anInt
    /** data to stream */
  );


  ///
  void write(
    /** {\large Purpose} \\
    Write out a sized buffer.
  
    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    
    {\large Arguments}\\
    */
    ///
    const void* aBuffer,
    /** buffer to write out */
    ///
    unsigned int aSize
    /** size of the buffer in bytes */
  );


  ///
  unsigned int read(
    /** {\large Purpose} \\
    Read in a sized buffer.
  
    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns the number of bytes read.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    
    {\large Arguments}\\
    */
    ///
    void* aBuffer,
    /** buffer to read to */
    ///
    unsigned int aSize
    /** size of the buffer */
  );
    

  ///
  void writeText(
    /** {\large Purpose} \\
    Write out an integer in human readable form.
  
    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    
    {\large Arguments}\\
    */
    ///
    int aVal
  );


  ///
  void writeText(
    /** {\large Purpose} \\
    Write out a double in human readable form.
  
    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    
    {\large Arguments}\\
    */
    ///
    double aVal
  );


  ///
  void writeText(
    /** {\large Purpose} \\
    Write out a string.
  
    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    
    {\large Arguments}\\
    */
    ///
    const DfsCString& aVal
  );


  ///
  CorBool readText(
    /** {\large Purpose} \\
    Read a text token from this stream and return it in an integer.
  
    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    
    {\large Arguments}\\
    */
    ///
    int& aVal
  );


  ///
  CorBool readText(
    /** {\large Purpose} \\
    Read a text token from this stream and return it in a double.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE if the token could be read.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    
    {\large Arguments}\\
    */
    ///
    double& aVal
  );


  ///
  CorBool readText(
    /** {\large Purpose} \\
    Read a text token from this stream and return it in a string.
  
    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE if the token could be read.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    
    {\large Arguments}\\
    */
    ///
    DfsCString& aVal
    /** */
  );


  ///
  DfsCString getLine(
    /** {\large Purpose} \\
    Read the next line from a given stream and return it as a string.
  
    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE if the token could be read.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    
    {\large Arguments}\\
    */
    ///
    std::istream* aStream = 0
    /** stream to read from */
  );


  ///
  void close();
  /** {\large Purpose} \\
  Close this instance. Nothing can be written or read from it anymore.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  CorBool open(
    /** {\large Purpose} \\
    Open a stream in a given mode.
    
    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE if the stream could be openend in the
    requested mode.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    */
    ///
    const DfsCString& aFileName,
    /** name of the file to open */
    ///
    StreamFlow aMode
    /** mode of the stream */
  );


  ///
  CorBool open(
    /** {\large Purpose} \\
    Open a stream in a given mode.
    
    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns corTRUE if the stream could be openend in the
    requested mode.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    */
    ///
    StreamFlow aMode = CorStream::READ
    /** mode of the stream */
  );


  ///
  CorPersistent* readObject(
    /** {\large Purpose} \\
    Read a persistent object from this stream.
  
    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function returns a new instance of a persistent object or 0 on
    error.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    
    {\large Arguments}\\
    */
    ///
    const CorRunTimeType* aType
    /** type of object expected, or 0 for any */
  );


  ///
  void writeObject(
    /** {\large Purpose} \\
    Write a persistent object to this stream.
  
    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    
    {\large Arguments}\\
    */
    ///
    const CorPersistent* pOb
    /** object to be written out */
  );


  ///
  CorBool isReading() const;
  /** {\large Purpose} \\
  Query whether this instance is in read mode.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns corTRUE if this instance is in read mode.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  CorBool isWriting() const;
  /** {\large Purpose} \\
  Query whether this instance is in write mode.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns corTRUE if this instance is in write mode.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */

  friend CorStream& operator<<(
    CorStream& aStream,
    const CorPersistent* anObject
  );

  friend CorStream& operator>>(
    CorStream& aStream,
    CorPersistent*& anObject
  );

  friend CorStream& operator>>(
    CorStream& aStream,
    const CorPersistent*& anObject
  );


protected:

  ///
  void become(
    /** {\large Purpose} \\
    This function makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorStream& aRef
    /** the reference instance */
  );
    

  ///
  CorBool clear();
  /** {\large Purpose} \\
  Clear this instance.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


private:
  friend void utest_CorStream(CorUnitTest&);  // the unit test function

  DfsCString    mName;        // file name
  StreamFlow    mFlow;        // direction of dataflow
  std::fstream* mStream;      // the stream
  CorBool       mInitialized; // state of this instance
  CorRef*       mRefCounter;  // number of times this stream is used

  unsigned int mObjectVersion; 

  unsigned int mMapCount;    // number of items in map
  union {
    CorPtrArray*    reader;
    CorMapPtrToPtr* writer;
  }            mMap ;
};

#endif // COR_STREAM_H
