//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: System Core
//
//$Revision: 1.2 $
//$Log: CorTrace.h,v $
//Revision 1.2  2004/10/26 15:23:54  nferguso
//Temporary check-in for repository move.
//
//Revision 1.1  2004/09/06 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/23 12:47:09  cguirao
//First check in
//
//Revision 1.5  2002/07/03 10:02:50  dataflow
//Compatible with g++ 3.0.4. CG
//
//Revision 1.4  1999/07/29 12:30:03  dataflow
//mIndentionLevel declared int to avoid warning.CG
//
//Revision 1.3  1998/06/30 08:49:53  dataflow
//Removing $NAME
//
//Revision 1.2  1997/12/03 15:05:47  dfadmin
//update for NTT Jan 98
//
//Revision 1.5  1997/05/16 08:10:11  mwiedmer
//removed old TRACE_ENTER and TRACE_LEAVE macros
//
//Revision 1.4  1997/04/11 13:32:04  mwiedmer
//use RWStrings
//
//Revision 1.1  1997/04/10 08:05:44  dfadmin
//Initial revision
//
//Revision 1.3  1997/04/04 16:46:44  mwiedmer
//check the environment variable only on
//startup now
//
//Revision 1.2  1996/12/18 13:34:59  pballest
//define COR_TRACE() even if not debugging
//
//Revision 1.1  1996/11/25 09:38:10  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef COR_TRACE_H
#define COR_TRACE_H

// includes from this library
#include <CorBool.h>
// includes from other ESO libraries
#include "DfsCString.h"
// includes from other libraries
// system includes

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorUnitTest;

#ifdef COR_USE_TRACE

//@Man: COR_TRACE_ENTER(), COR_TRACE_LEAVE()
//@Type: MACROS
//@See:  COR_ASSERT
//@Memo: debugging functionality
/**      This macro is provided to faciliate the use of debugging functions
         in a standard and uniform way.
         In particular, it should be used to trace the entry and exit of
         functions.
*/

//@{
/// Debugging Function - entering statement
#define COR_TRACE(TEXT) CorTrace trace(TEXT)
/**
  {\footnotesize $Id: CorTrace.h,v 1.2 2004/10/26 15:23:54 nferguso Exp $}

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <cor_trace.h> 

    void foo() {
      COR_TRACE_ENTER("foo()");

      // ...

      COR_TRACE_LEAVE("foo()");
    };
  \end{verbatim}

  {\large DESCRIPTION} \\
  This macro serves as an interface to using debugging text in the application.
  It accepts some text and will print it out if an environment variable is set.
  The environment variable is defined in "cor_debug.h".

  {\large EXAMPLE}\\
  \begin{verbatim}
    #include <cor_trace.h> 

    class A {
    public:
      A(){
        COR_TRACE_ENTER("A::foo()");
        COR_TRACE_LEAVE("A::foo()");
      }
      ~A(){
        COR_TRACE_ENTER("A::foo()");
        COR_TRACE_LEAVE("A::foo()");
      }
      void foo() {
        COR_TRACE_ENTER("A::foo()");

        // ...

        COR_TRACE_LEAVE("A::foo()");
      };
    protected:
    };

    int main() {

      A a;       //  prints out --> A::A()
      a.foo();   //  prints out   --> A::foo()
                 //               <-- A::foo()
      return 0;
    }            // prints out <-- A:~A()
      
    
  \end{verbatim}

  {\large SEE ALSO}\\
  \Ref{cor_debug.h}
 
  */
//@}


#else

#define COR_TRACE(TEXT)
#define COR_TRACE_ENTER(TEXT)
#define COR_TRACE_LEAVE(TEXT)

#endif  // COR_USE_TRACE defined


/// A simple tracer class 
class CorTrace {
  /**
  {\footnotesize $Id: CorTrace.h,v 1.2 2004/10/26 15:23:54 nferguso Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <CorTrace.h>
  
    CorTrace myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class allows simple tracing of statement entry and exit.
  Usually, you would construct an instance of this class with a message to
  print out. This message will be printed out according to an indentation
  level uppon construction and deletion of the instance. The indentation
  level will be adjusted on both accounts.

  {\large CAUTIONS} \\
  <optional> \\

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  */

public:

  /// 
  CorTrace(
    /** {\large Purpose} \\
    The default c'tor.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    */
    ///
    const DfsCString& aMessage
    /** message to print out */
  );


  ///
  ~CorTrace();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  /// 
  void print(
    /** {\large Purpose} \\
    Print out a message.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}
    */
    ///
    const DfsCString& aMessage
    /** message to print out */
  ) const;


protected:
  /// 
  CorTrace(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorTrace& aRef
    /** the reference instance */
  );
    

  /// 
  CorTrace& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorTrace& aRef
    /** the reference instance */
  );
    

private:
  friend void utest_CorTrace(CorUnitTest&);  // the unit test function

  static int mIndentionLevel;   // the current indention level for tracing
  DfsCString mText;             // text to print out
  static CorBool   mPrintOut;      // do we need to print out?
};

#endif // COR_TRACE_H
