//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: System Core
//
//$Revision: 1.1 $
//$Log: CorUnitTest.h,v $
//Revision 1.1  2004/09/06 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/23 12:47:09  cguirao
//First check in
//
//Revision 1.3  2002/07/03 10:02:48  dataflow
//Compatible with g++ 3.0.4. CG
//
//Revision 1.2  1998/06/30 08:49:54  dataflow
//Removing $NAME
//
//Revision 1.1  1997/04/10 08:05:44  dfadmin
//Initial revision
//
//Revision 1.1  1996/09/23 09:11:43  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef COR_UNITTEST_H
#define COR_UNITTEST_H

// includes from this library
#include <CorBool.h>
// includes from other ESO libraries
// includes from other libraries
// system includes

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
///
#define COR_UTEST(BOOL,EXP) \
  if(BOOL) \
    CorUnitTest::getTester()->successHandler(__FILE__,__LINE__,#BOOL);\
  else \
    CorUnitTest::getTester()->failHandler(__FILE__,__LINE__,EXP)

// Macro that always fails
///
#define COR_UFAIL(EXP) \
  COR_UTEST(0,EXP)

class _IO_ostream_withassign;

/// A tester class
class CorUnitTest {
  /**
  {\footnotesize $Id: CorUnitTest.h,v 1.1 2004/09/06 14:35:21 cguirao Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <CorUnitTest.h>
  
    CorUnitTest myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This class provides unit testing utility functions. The test program should
  create an instance of this class and pass it to the unit test function.
  Each class should have a friend function called\\
  {\bf {\tt utest_<className>(CorUnitTest&)}}, eg.

  \begin{verbatim}
  class SubClass {
  private:
    friend void utest_SubClass(CorUnitTest&);
  };
  \end{verbatim}

  The checker maintains an instance count to ensure that only one CorUnitTest
  can exist at any time. This is because: \\

  \begin{itemize}
  \item The CorUnitTest handles the assertions and thus requires a static
  pointer to the one and only CorUnitTest so it's member functions can be
  invoked.
 
  \item The COR_UTEST macro needs to invoke a member function for the
  instance of the CorUnitTest.
 
  \item Finally only one unit test is allowed per invocation of a test
  harness so why could you possibly want more than one?
  \end{itemize}
 
  The CorUnitTest in the test file should be called 'cortest' as in:
 
            void TestMyClass(CorUnitTest& cortest){ .... }
 
  The CorUnitTest grabs the assertion handler and processes the assertions for
  the caller.

  {\large CAUTIONS} \\
  <optional> \\

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  \Ref{COR_ASSERT}\\
  \Ref{COR_DEBUG}\\
  */

public:

  /// 
  CorUnitTest();
  /** {\large Purpose} \\
  The default c'tor.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */
    

  /// 
  CorUnitTest(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorUnitTest& aRef
    /** the reference instance */
  );
    

  /// 
  CorUnitTest& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorUnitTest& aRef
    /** the reference instance */
  );
    

  ///
  ~CorUnitTest();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  void expectAssert(
    /** {\large Purpose} \\
    Tell this instance that aNumber of assertions are expected.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    int aNumber
    /** number of assertions to expect */
  );


  ///
  void checkAssert();
  /** {\large Purpose} \\
  Check that all expected assertions were fired.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  void print(
    /** {\large Purpose} \\
    Print out a progress message.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const char* aText
    /** message to print */
  );


  ///
  void printf(
    /** {\large Purpose} \\
    Print out a variable list in C's printf() style.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const char* aFormat,
    /** format to print out stuff */
    ///
    ...
    /** variable list */
  );


  ///
  void printBanner(
    /** {\large Purpose} \\
    Define if an outpout should also print a banner of ==='s.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    CorBool aPrintOn
    /** print equal signs? */
  );


  ///
  void testMessages(
    /** {\large Purpose} \\
    Define if the COR_UTEST messages should be on or off.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    CorBool aMessagesOn
    /** print messages? */
  );


  ///
  _IO_ostream_withassign& getOstream();
  /** {\large Purpose} \\
  Query the stream to be used for C++ << type message output.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the current output stream.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  static CorUnitTest* getTester();
  /** {\large Purpose} \\
  Query the current CorUnitTest instance.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function returns the current CorUnitTest instance or 0.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  void failHandler(
    /** {\large Purpose} \\
    The function for when the test fails.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const char* aFile,
    /** file name */
    ///
    int aLineNumber,
    /** line number */
    ///
    const char* aText
    /** explanatory text */
  );


  ///
  void successHandler(
    /** {\large Purpose} \\
    A function for when the test succeeds.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const char* aFile,
    /** file name */
    ///
    int aLineNumber,
    /** line number */
    ///
    const char* aText
    /** explanatory text */
  );


protected:


private:

  ///
  void assertHandler(
    /** {\large Purpose} \\
    Assertion handling function.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const char* aFile,
    /** file name */
    ///
    int aLineNumber,
    /** line number */
    ///
    const char* aText
    /** explanatory text */
  );


  ///
  void reportHandler(
    /** {\large Purpose} \\
    Report printer, called from handlers.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const char* aHeader,
    /** header */
    ///
    const char* aFile,
    /** file name */
    ///
    int aLineNumber,
    /** line number */
    ///
    const char* aText
    /** explanatory text */
  );


  ///
  static void assertion(
    /** {\large Purpose} \\
    A static assertion callback function.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const char* aFile,
    /** file name */
    ///
    int aLineNumber,
    /** line number */
    ///
    const char* aText
    /** explanatory text */
  );


  ///
  static void abortTest();
  /** {\large Purpose} \\
  Abort the test.
  */

  int mNumExpected;               // Expecting some asserts.
  int mNumCallsExpectAssert;      // number of times expectAssert was called
  int mNumCallsCheckAssert;       // number of times checkAssert was called
  CorBool mTestMessagesOn;        // corFALSE if messages are off
  CorBool mPrintBannerOn;         // corFALSE if banner (===) is off
  static CorUnitTest* mTheTest;   // the CorUnitTest instance
};

#endif // COR_UNITTEST_H
