// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 2003, 2004 European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System - RogueWave->dfsStdLib port
// AUTHOR:    Neil Ferguson - ESO/DMD/DFS
// SUBSYSTEM: dfsStdLib constant definitions 
//
// $Revision: 1.9 $
// $Log: DfsConstants.h,v $
// Revision 1.9  2004/08/17 15:54:22  nferguso
// Documentation updates.
//
// Revision 1.8  2004/04/27 19:42:07  nferguso
// Added a note to the DFS_NPOS documentation, stating that it is equivalent
// to RW_NPOS.
//
// Revision 1.7  2004/04/05 14:52:09  nferguso
// Changed dfsnil constant to a #define, matching RogueWave.
//
// Revision 1.6  2004/04/05 13:29:46  nferguso
// Added the dfsnil constant (equivalent to rwnil).
//
// Revision 1.5  2004/03/26 15:07:39  nferguso
// Copyright years updated.
//
// Revision 1.4  2004/03/22 18:03:21  nferguso
// Added typedef for Dfsoffset.
//
// Revision 1.3  2004/02/20 10:11:19  nferguso
// Missing comment.
// Protected section in DfsSize_T class.
//
// Revision 1.2  2003/12/19 15:03:02  nferguso
//
// String-handling classes can be built and used, although some things remain
// to be done. Most significantly:
//
// - methods working with iostreams are not fully compliant with RogueWave
// - STL string reserve() memory management is incomplete
//
// No non-string RogueWave classes have been replaced yet.
//
// Revision 1.1.1.1  2003/11/28 14:21:16  nferguso
// First Check-In
//
// 
//------------------------------------------------------------------------------

#ifndef DFS_CONSTANTS_H
#define DFS_CONSTANTS_H

// C++ STL includes
#include <string>

/// Equivalent of RWBoolean. 
typedef bool DfsBoolean; 

/// Equivalent of RWoffset. 
typedef long Dfsoffset;

/// Equivalent of rwnil. 
#define dfsnil  0

/// Null value for undefined string positions; equivalent of RW_NPOS. 
const size_t DFS_NPOS = std::string::npos;

/// Wrapper class for size_t type - equivalent of RWSize_T. 
/** Avoids unwanted implicit type conversions in overloaded method calls. */
class DfsSize_T
{
public:
    /// Constructs an instance encapsulating the given size_t value. 
    DfsSize_T(size_t size) :
        storedValue(size)
    {
        // This constructor does nothing beyond member initialisation. 
    }

    // The default destructor suffices for this class. 

    /// Provides access to the encapsulated size_t value. 
    /** @return The encapsulated size_t value. 
     */
    size_t value() const
    {
        return(storedValue);
    }

protected:

private:
    /// The encapsulated size_t value. 
    size_t storedValue;
};

#endif  // DFS_CONSTANTS_H
