/// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.1 $
// $Log: KeywordRepDAO.h,v $
// Revision 1.1  2007/12/14 16:40:22  cmoins
// first version
//
// Revision 1.2  2007/12/14 15:58:20  cmoins
// first "operation version" of the header of the fly tool
//
// Revision 1.1  2007/12/07 12:57:20  cmoins
// first version of the header on the fly correction tool
//
//
// 
//------------------------------------------------------------------------------

#ifndef KEYWORDREP_DAO_H
#define KEYWORDREP_DAO_H

#include <string>
#include <vector>
#include "KeywordRep.h"

class SybaseConnection;
class SybaseResultSet;

namespace headerOnTheFly {

  /**
   *  Implementation of a DAO class for KeywordRep objects
   */ 
  class KeywordRepDAO
  {
  public:

    /// default Constructor
    KeywordRepDAO(){}

    /**
     * Constructor
     */
    KeywordRepDAO(std::string keywordRepTable);

    /**
     * Destructor 
     * Closes the connection to the database
     */
    ~KeywordRepDAO(){}

    /**
     * finds a set of keyword data associated to a dp id and sort them 
     * according to the extension and keyword index.
     * dpId data product id whose keyword data shall be retrieved
     * keywordRepSet set of keyword data
     * rconnection  database connection handle
     * returns the number of retrieved items if successful, -1 otherwise
     */
    int findByDpId(std::string dpId,
		   KeywordRep::ResultSet& rkeywordRepSet,
		   SybaseConnection& rconnectionHandle) const;

  private :

    /// Fields used in the queries
    static const std::string DP_ID_FIELD;
    static const std::string KW_NAME_FIELD;
    static const std::string KW_VALUE_FIELD;
    static const std::string KW_TYPE_FIELD;
    static const std::string KW_COMMENT_FIELD;
    static const std::string EXT_ID_FIELD; 
    static const std::string KW_IND_FIELD;


    /// \brief gives the whole list of fields to be used by a query
    std::string getFieldList() const;

    /// \brief creates a KeywordRep instance from a sybase record
    /**
     * rsybRes set of sybase records
     * rkeywordRep instance created from the current processed record
     * returns 0 if successful, -1 otherwise
     */
    int convertResultSetToKeywordRep(SybaseResultSet& rsybRes,
				     KeywordRep& rkeywordRep) const;

    /// table where keywordRep data are stored
    std::string keywordRepTable_;

    std::vector<std::string> listOfField_;

  };
}
#endif //KEYWORDREP_DAO_H
