// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys 
//
// $Revision: 1.4 $
// $Log: MagDisk.h,v $
// Revision 1.4  2007/05/02 09:36:15  cmoins
// include of set container header file
//
// Revision 1.3  2007/04/26 12:50:41  cmoins
// doxygen comments modification
//
// Revision 1.2  2007/04/25 15:00:49  cmoins
// made compliant to doxygen
//
// Revision 1.1  2007/03/09 17:06:11  cmoins
// implementation of a magnetic disk structure
//
// 
//------------------------------------------------------------------------------

#ifndef MAG_DISK_H
#define MAG_DISK_H

#include <string>
#include <set>

using namespace std;

/// Implementation of  a magnetic disk
/** 
 *  Stores the following informations related to a disk :
 *  - name of the archive the volume belongs to,
 *  - name of the volume,
 *  - mounting flag,
 *  - mount point of the volume
 */
struct MagDisk
{
  typedef set<MagDisk> ResultSet;
  typedef ResultSet::iterator pMagDisk;

  /// Default Constructor
  MagDisk() 
  {}

  /// Constructor using the volume name
  MagDisk(string diskName):
    archive_(""),
    diskName_(diskName),
    mounted_(""),
    mountPoint_("")
  {}

  /// Constructor setting all the volume data
  MagDisk(string archive,
	  string diskName,
	  string mounted,
	  string mountPoint):
    archive_(archive),
    diskName_(diskName),
    mounted_(mounted),
    mountPoint_(mountPoint)
  {}

   /// name of the database archive
  string archive_;     
  /// volume name
  string diskName_;
  /// 'Y' if volume mounted, 'N' otherwise
  string mounted_;     
  /// mount point of the volume
  string mountPoint_;  

  /// operator < necessary for the set container
  bool operator < (const MagDisk& rmagDisk) const
  {
    return (diskName_ < rmagDisk.diskName_);
  }

  /// operator == necessary for the set container
  bool operator == (const MagDisk& rmagDisk) const
  {
    return (diskName_ == rmagDisk.diskName_);
  }
};
#endif /* MAG_DISK_H */

