// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys 
//
// $Revision: 1.5 $
// $Log: MagDiskFile.h,v $
// Revision 1.5  2007/05/15 09:55:53  cmoins
// initialization of magDisk_ data member
//
// Revision 1.4  2007/05/02 09:37:25  cmoins
// fileId_ becomes a key for the set container
//
// Revision 1.3  2007/04/26 12:50:56  cmoins
// doxygen comments modification
//
// Revision 1.2  2007/04/25 15:01:06  cmoins
// Made compliant to doxygen
//
// Revision 1.1  2007/03/09 17:06:41  cmoins
// implementation of a file on magnetic disk structure
//
// 
//------------------------------------------------------------------------------

#ifndef MAG_DISK_FILE_H
#define MAG_DISK_FILE_H

#include <set>
#include <string>
#include "MagDisk.h"

using namespace std;

/// Implementation of a file as stored on a magnetic disk
/**
 *  For a given file id, allows to retrieve the associated file 
 *  and the disk it is stored on.
 */
struct MagDiskFile
{
   /// Set of files on magnetic disk
  typedef set<MagDiskFile> ResultSet;   
  typedef ResultSet::iterator pMagDiskFile;

  /// Default constructor
  MagDiskFile()
  {}

  /// Constructor using a file id
  MagDiskFile(string fileId):
    fileId_(fileId) 
  {}

  MagDiskFile(string diskName,
	      string fname):
    diskName_(diskName),
    fname_(fname),
    magDisk_(diskName)
  {}

  /// Constructor setting all data of a file on a magnetic disk
  MagDiskFile(string fileId,
	      string diskName,
	      string fname,
	      int fsize,
	      char status,
	      int uncompFSize,
	      string compression,
	      string format,
	      string hostName):
    fileId_(fileId),
    diskName_(diskName),
    fname_(fname),
    fsize_(fsize),
    status_(status),
    uncompFSize_(uncompFSize),
    compression_(compression),
    format_(format),
    magDisk_("",diskName,"",hostName)
  {}
 
  /// File identifier 
  string fileId_;    
  /// Name of the volume where the file is stored
  string diskName_;  
  /// Name of the file on the disk
  string fname_; 
  /// Size of the file
  int fsize_; 
  /// 'C' if current, "I" for obsolete, "E" for error
  char status_;  
  /// size of the uncompressed file
  int uncompFSize_;
  /// compression type
  string compression_; 
  /// format of the file : "F" for FITS.
  string format_;  
  /// Data of the disk the file is stored on
  MagDisk magDisk_;
 

  /// operator < used for the set container 
  bool operator < (const MagDiskFile& rmagDiskFile) const
  {
    return (fileId_ < rmagDiskFile.fileId_);
  }	  
  
  /// operator == used for the set container
  bool operator == (const MagDiskFile& rmagDiskFile) const
  {
    return (fileId_ == rmagDiskFile.fileId_);
  }
};
#endif /* MAG_DISK_FILE_H */

