// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys 
//
// $Revision: 1.4 $
// $Log: MediaAction.h,v $
// Revision 1.4  2007/08/13 11:05:29  cmoins
// correction of media_unit_name_ initialization so that no segmentation fault shows up
//
// Revision 1.3  2007/07/25 07:27:32  cmoins
// doxygen comments correction
//
// Revision 1.2  2007/07/12 08:13:31  cmoins
// adding of a MediaAction container type
// VS: ----------------------------------------------------------------------
//
// Revision 1.1  2007/04/27 13:23:29  cmoins
// creation of a structure implementing an action associated to a media request
//
//------------------------------------------------------------------------------

#ifndef MEDIA_ACTIONS_H
#define MEDIA_ACTIONS_H

#include <string>
#include <vector>

using namespace std;

/// Implementation of  a media action
/** 
 * Stores information about a media request action being currently performed.
 * An insert shall be performed each time an action is beginning and a 
 * delete shall be performed each time the action has finished.
 */
struct MediaAction
{
  /// set of media actions
  typedef vector<MediaAction> ResultSet; 
  typedef ResultSet::iterator pMediaAction;
  typedef ResultSet::const_iterator pconstMediaAction;
 
  /// Default constructor
  MediaAction()
  {}

  /// Constructor setting the media request id, media unit name and 
  /// the action data member.
  MediaAction(int mediaRequestId,
	      string mediaUnitName,
	      string action):
    mediaRequestId_(mediaRequestId),
    mediaUnitName_(mediaUnitName),
    action_(action)
  {}

  /// Constructor setting all the data members of a media request action
  MediaAction(int mediaRequestId,
	      string mediaUnitName,
	      string action,
	      string time,
	      int processId):
    mediaRequestId_(mediaRequestId),
    mediaUnitName_(mediaUnitName),
    action_(action),
    time_(time),
    processId_(processId)
  {}

  /// media request identifier automatically generated
  int mediaRequestId_; 

  /// name of the media request
  string mediaUnitName_;  

  /// action being performed
  /** I(ngest), IR (Ingest Rollback), Q(ueuing), QR(Queuing rollback)
   * P(reparing), PR(preparing rollback), W(riting), WR (writing rollback),
   * C(leanup), CR (Cleanup Rollback)
   */
  string action_;

  /// Time the action was started
  string time_;

  /// process id of the task that started the action
  int processId_;

};
#endif /* MEDIA_ACTIONS_H */

