// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys 
//
// $Revision: 1.2 $
// $Log: MediaType.h,v $
// Revision 1.2  2007/07/25 07:28:00  cmoins
// doxygen comments correction
//
// Revision 1.1  2007/07/05 09:17:35  cmoins
// implementation of a media type content
//
// 
//------------------------------------------------------------------------------

#ifndef MEDIA_TYPE_H
#define MEDIA_TYPE_H

#include <string>
#include <vector>

using namespace std;

/// Implementation of  a media type
struct MediaType
{ 
  /// set of media type files
  typedef vector<MediaType> ResultSet; 
  typedef ResultSet::iterator pMediaType;
  typedef ResultSet::const_iterator pconstMediaType;

  /// Constructor using the file identifier
  MediaType(string mediaType):
    mediaType_(mediaType)
  {}

  /// type of physical media
  string mediaType_;                
  /// media capacity in kilobytes
  int capacity_;       
  /** 
   * minimum percentage of data that must be on the media
   * before it is considered full
   */
  double threshold_;
  /** 
   *  amount of data in kb that must be contained in a directory
   *  before it can be considered for splitting
   */
  int splitSize_;
  /// number of bytes in a block of data
  int blockSize_;
  /// number of extra blocks needed for a file
  int    fileOverhead_;     
  /// number of extra blocks needed for a directory
  int    directoryOverhead_;
  /// media type description
  string description_;

};
#endif /* MEDIA_TYPE_H */

