// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.3 $
// $Log: ObsProgFactory.h,v $
// Revision 1.3  2007/05/04 12:23:13  cmoins
// adding of ObservationService and StorageService template parameters
//
// Revision 1.2  2007/04/26 12:58:00  cmoins
// modification of doxygen comments
//
// Revision 1.1  2007/04/25 15:57:41  cmoins
// *** empty log message ***
//
// 
//------------------------------------------------------------------------------

#ifndef OBS_PROG_FACTORY_H
#define OBS_PROG_FACTORY_H

#include <string>
#include "ObsProg.h"
#include "GenObsProg.h"
#include "CalibProg.h"

using namespace std;

/// Observation Program  class factory.
/**
 * This class implements a factory that instantiates observation
 * program according to an identifier given by the user.
 */
template <class ObservationService,class StorageService>
class ObsProgFactory
{
public:

  /// \brief Constructor 
  ObsProgFactory(){}
 
  /// \brief Destructor
  ~ObsProgFactory(){}

  /**
   * Instantiates an observation program according to the 
   * identifier given by the user
   * \param obsProgCId   obs prog class identifier
   * \param connAlias   data repositiory connection keyword
   * \param progId      observation program identifier
   * \returns a pointer on the created instance if successful, NULL otherwise
   */
  static ObsProg<StorageService>* getObsProg(string obsProgCId,
					     string connAlias,
					     string progId);

  /// General observation program class id
  static const string GenObsProgCId; 
  /// Calibration program class id
  static const string CalibProgCId; 

};

// observation program identifiers creation
template <class ObservationService,class StorageService>
const string ObsProgFactory<ObservationService,StorageService>
::GenObsProgCId = "GEN_OBS_PROG";

template <class ObservationService,class StorageService>
const string ObsProgFactory<ObservationService,StorageService>
::CalibProgCId = "CALIB_OBS_PROG";


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Instantiates an observation program according to the identifier given by
// the user.
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
ObsProg<StorageService>*
ObsProgFactory<ObservationService,StorageService>::
getObsProg(string obsProgCId,
	   string connAlias,
	   string progId)
{ 
  ObsProg<StorageService>* pobsProg;

  if (obsProgCId == GenObsProgCId)
    {
      pobsProg = 
	new GenObsProg<ObservationService,StorageService>(progId,
							  connAlias);
    }
  else if (obsProgCId == CalibProgCId)
    {
      pobsProg = 
	new CalibProg<ObservationService,StorageService>(progId,
							 connAlias);
    }
  else
    { 
      pobsProg = NULL;   
    }
  return pobsProg;
}
#endif // OBS_PROG_FACTORY_H
