// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/DMD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.2 $
// $Log: PANL2NightObsDAOSyb.h,v $
// Revision 1.2  2007/10/25 09:05:38  cmoins
// adding of the Request_ID field
//
// Revision 1.1  2007/10/12 12:31:58  cmoins
// adding of PANL2NightObs tabke DAO class
//
//
// 
//------------------------------------------------------------------------------

#ifndef PANL2_NIGHT_OBS_DAO_SYB_H
#define PANL2_NIGHT_OBS_DAO_SYB_H

#include "PANL2NightObs.h" 

class ServiceConnectionSyb;

/// Sybase Data Access Object Class to PANL2NightObs instances
class PANL2NightObsDAOSyb
{
public:


  /// \brief default constructor
  PANL2NightObsDAOSyb() {}
  /// \brief Constructor
  /// \param panl2NightObsTable  observation night conditions table
  PANL2NightObsDAOSyb(std::string panl2NightObsTable);

  /** 
   * Retrieves a set of observation night conditions  associated to an 
   * observation block
   * \param obsId observation block Id
   * \param rPANL2NightObs observation conditions of the observation block
   * \param rconnectionHandle database connection handle
   * \return the number of retrieved objects, or -1 in case of error
   */
  int findByObsId(int obsId, 
		  PANL2NightObs::Container& rpanl2NightObs,
		  const ServiceConnectionSyb& rconnectionHandle) const;
  
private:

  /// Field Names in the night observation table
  static const std::string REQUEST_ID_FIELD;
  static const std::string OB_ID_FIELD; 
  static const std::string SKY_TRANSPARENCY_FIELD; 
  static const std::string TRANSPARENCY_FIELD; 

  /// table storing observation night conditions
  std::string panl2NightObsTable_;

};

#endif // PANL2_NIGHT_OBS_DAO_SYB_H
