// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Jens Knudstrup - ESO/DMD/DFS
// SUBSYSTEM: DFS Archive System
//
// $Name: arcsys-1_2_8 $
// $Revision: 1.13 $
// $Log: RepGenTask.h,v $
// Revision 1.13  2007/11/22 13:01:57  cmoins
// adding of friend class TestRepGenTask
//
// Revision 1.12  2007/01/11 15:35:10  jknudstr
// -
//
// Revision 1.11  2006/08/08 15:29:22  jknudstr
// -
//
//-----------------------------------------------------------------------------

#ifndef REPGENTASK_H
#define REPGENTASK_H


// Local includes
#include <Report.h>
#include <GenTask.h>


/// Class that implements the Report Generation Task.
/// This type of task should only run in one instance.
class RepGenTask: public GenTask
{
  public:

#if defined UNIT_TEST
  friend class TestRepGenTask;
#endif
    /// Constructor registering common instance of the Replication Agent Class
    /// and the output message queue to use for this task.
    RepGenTask(RepAgent&     repAgent,
	       MsgQueue&     msgOutputQueue);

    /// Destructor method.
    ~RepGenTask();

    /// ACE server loop.
    int svc(void);

    /// Method implementing the business logic of the initialization.
    void initBusiness(void);

    /// Method implementing the business logic of this server process.
    void srvBusiness(void);

    /// Generate the report from the internal information.
    void genReport(Report&  report);

    /// Submit the report to the recipients.
    void submitReport(Report&  report);

    /// Remove all entries markes as "Replicated" or "Error".
    void cleanUp(void);

    /// Restore the previous state from the DB information.
    void restore(void);

    /// Title of Report Generation Task reports.
    static const string REP_GEN_TASK_REP_TITLE;

    /// Extension of report files.
    static const string REP_GEN_TASK_REP_EXT;

  private:
    bool    intervalReporting_;  // Interval reporting (0|1).
    time_t  reportTimeCfg_;      // Report time from configuration.
    string  reportDir_;          // Report output directory.
};


#endif // REPGENTASK_H

// EOF

