// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Jens Knudstrup - ESO/DMD/DFS
// SUBSYSTEM: DFS Archive System
//
// $Name: arcsys-1_2_8 $
// $Revision: 1.4 $
// $Log: Report.h,v $
// Revision 1.4  2007/01/11 15:35:10  jknudstr
// -
//
// Revision 1.3  2006/08/08 15:29:22  jknudstr
// -
//
//-----------------------------------------------------------------------------

#ifndef REPORT_H
#define REPORT_H


// STL includes
#include <string>
#include <map>

// Local includes
#include <StringUtils.h>
#include <ArcSysEx.h>


/// Class to store the information in for one report entry.
class ReportEntry
{
  public:
    /// Constructor. 
    /**  
     * \param info Message for this report entry.
     */
    ReportEntry(const string&  info) {info_ = info;}

    /// Destructor.
    ~ReportEntry() {}

    /// Return report info.
    const string&  info(void) {return info_;}

  private:
    string  info_;   // The information for this report entry.
};


/// Class to build up reports.
class Report
{
  public:
    /// Constructor. 
    /**  
     * \param title Title for the report.
     * \param context Context in which the report is generated.
     */
    Report(const string&  title,
	   const string&  context) 
    {
	title_ = title;
	context_ = context;
    }

    /// Destructor.
    ~Report() {}

    /// Add report summary information.
    Report&  setSummary(const string& repSum);

    /// Return report summary information.
    const string& getSummary(void);

    /// Add an entry in the report.
    Report&  addReportEntry(const string&  reportEntry);

    /// Generate the report.
    Report&  generateReport(string&    report);

    /// Lay-out of report header.
    static const string REPORT_HDR_FORMAT;

    /// Lay-out of report footer.
    static const string REPORT_FOOTER_FORMAT;

  private:
    string               title_;         // Title of the report.
    string               context_;       // Context of the report.
    string               summary_;       // Summary message of report.
    vector<ReportEntry>  reportEntries_; // Entries in the report.
};

#endif // REPORT_H

// EOF
