#ifndef SYBASE_STATUS
#define SYBASE_STATUS
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// $Id: SybaseStatus.h,v 1.1 2005/08/11 17:29:46 szampier Exp $
// 
// COPYRIGHT (c) European Southern Observatory
//
// PROJECT:   VLT Data Flow System
// AUTHOR :   Stefano Zampieri - ESO/DMD
// SUBSYSTEM: DFS Back End
//
// $Revision: 1.1 $
// $Log: SybaseStatus.h,v $
// Revision 1.1  2005/08/11 17:29:46  szampier
// first version
//
//
// This class represents a DB-library error or a server message
//
//
//-----------------------------------------------------------------------------

#include <string>

using namespace std;

class SybaseStatus
{
 public:
    typedef void (*ErrorHandler) (const SybaseStatus&);
  
 private:
    // The severity of the error (datatype int).
    int dbLibSeverity;
    // The identifying number of the error (datatype int). Error numbers are defined in sybdb.h.
    int dberr;
    // The operating-system-specific error number that describes the cause of the error (datatype int).
    int oserr;
    // A printable description of dberr (datatype char *).
    string dberrstr;
    // A printable description of oserr (datatype char *).
    string oserrstr;
    
    // The current message's number (datatype DBINT).
    int msgno;
    // The current message's error state number (datatype int).
    int msgstate;
    // The current message's information class or error severity (datatype int).
    int srvSeverity;
    // The null-terminated text of the current message (datatype char *).
    string msgtext;
    // The null-terminated name of the server that generated the message (datatype char *). 
    string srvname;
    // The null-terminated name of the stored procedure that generated the message (datatype char *).
    string procname;
    // The number of the command batch or stored procedure line that generated the message (datatype int).
    int line;
    
 public:
  
    // constructor from a DB-library error (see sybase doc)
    SybaseStatus(int severity, int dberr, int oserr, char *dberrstr, char *oserrstr);
    
    // constructor from a server message (see sybase doc)
    SybaseStatus(long msgno, int msgstate, int severity, char *msgtext, char *srvname, char *procname, int line);
    
    const char *serverName() const { return srvname.data(); }
    
    unsigned short isServerError() const { return msgno != 0; }
    
    const char *errorMessage() const { return isServerError() ? msgtext.data() : dberrstr.data(); }
    
    int severity() const { return isServerError() ? srvSeverity : dbLibSeverity;}
    
    int errorCode() const { return isServerError() ? msgno : dberr; }
};


#endif
