/// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.5 $
// $Log: TempHdrHandler.h,v $
// Revision 1.5  2008/03/03 15:27:33  cmoins
// Changing of the command line tool interface
//
// Revision 1.4  2008/02/15 11:15:04  cmoins
// adding of options
//
// Revision 1.3  2008/01/09 13:51:46  cmoins
// use of dbrc file to retrieve database connection parameters
//
// Revision 1.2  2008/01/08 12:48:20  cmoins
// database parameters become configurable
//
// Revision 1.1  2008/01/07 10:43:45  cmoins
// adding of TempHdrHandler to display database content on the standard
// output
//
// 
//------------------------------------------------------------------------------


#ifndef TEMP_HDR_HANDLER_H
#define TEMP_HDR_HANDLER_H

#include <string>
#include "FrameHandler.h"

namespace headerOnTheFly {

  /**
   * Implementation of a "temporary" hdr file handler. Its main task will be 
   * to display on the standard output the database header records associated 
   * to a data product id
   */ 
  class TempHdrHandler : public FrameHandler
  {
  public:

    /// Default constructor
    TempHdrHandler(){}

    /**
     * Constructor
     * Gets the name of data product id whose database header shall be printed
     * \param dpId  data product id
     * \param dbServer database server
     * \param dbUser database user
     * \param dbPasswd user passwd
     */
    TempHdrHandler(std::string dpId,
		   std::string dbServer,
		   std::string dbUser,
		   std::string dbPasswd);

    /**
     * Destructor 
     * No action is performed
     */
    ~TempHdrHandler(){};

    /**
     * Displays the content of the database header
     * returns 0 if successful, -1 otherwise
     */
    virtual int substituteExtHeader() ;

  private :
    /// data product id to handle
    std::string dpId_;
  };
}
#endif // TEMP_HDR_HANDLER_H
