// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Jens Knudstrup - ESO/DMD/DFS
// SUBSYSTEM: DFS Archive System
//
// $Name: arcsys-1_2_8 $
// $Revision: 1.7 $
// $Log: TlFileAvailChkTask.h,v $
// Revision 1.7  2007/05/03 15:32:07  jknudstr
// Intermediate check-in while developing the SA->PA Archive Replicator.
//
// Revision 1.6  2006/08/08 15:29:22  jknudstr
// -
//
//-----------------------------------------------------------------------------

#ifndef TLFILEAVAILCHKTASK_H
#define TLFILEAVAILCHKTASK_H


// Local includes
#include <GenTask.h>


/// Class that implements the TL File Availability Check Task.
class TlFileAvailChkTask: public GenTask
{
  public:
    /// Constructor registering common instance of the Replication Agent Class
    /// and the output message queue to use for this task.
    TlFileAvailChkTask(RepAgent&     repAgent,
		       MsgQueue&     msgOutputQueue);

    /// Destructor method.
    ~TlFileAvailChkTask();

    /// ACE_Task server method.
    int svc(void);

    /// Method implementing the business logic of the initialization.
    void initBusiness(void);
    
    /// Method implementing the business logic of this server process.
    void srvBusiness(void);

    /// Restore the previous state from the DB information.
    void restore(void);

  private:  
    /// Interfacing to pro_trieve.
    ProTrieve                          proTrieve_;

    map<string, ProTrieveFileRequest>  proTrieveRetrieveReqs_;
};


#endif // TLFILEAVAILCHKTASK_H

// EOF

