//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Michael Wiedmer - ESO/DMD/DPG>
//SUBSYSTEM: <Core>
//
//$Revision: 1.1 $
//$Log: cor_print.h,v $
//Revision 1.1  2004/09/06 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/23 12:47:09  cguirao
//First check in
//
//Revision 1.3  1998/06/30 08:49:55  dataflow
//Removing $NAME
//
//Revision 1.2  1997/12/03 15:05:47  dfadmin
//update for NTT Jan 98
//
//Revision 1.5  1997/04/11 13:32:08  mwiedmer
//use RWStrings
//
//Revision 1.1  1997/04/10 08:05:44  dfadmin
//Initial revision
//
//Revision 1.4  1996/10/17 12:57:56  mwiedmer
//remove compiler warnings
//
//Revision 1.3  1996/09/23 09:16:21  mwiedmer
//modified algorithm to test the environment variable
//
//Revision 1.2  1996/09/10 11:46:04  mwiedmer
//fixed multiple declaration of variable
//
//Revision 1.1  1996/09/10 11:22:23  mwiedmer
//update debugging and debug printing
//mechanism
//
//Revision 1.0  1996/09/10 11:22:23  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef COR_PRINT_H
#define COR_PRINT_H

// includes from this library
#include <cor_debug.h>
// includes from other ESO libraries
// includes from other libraries
// system includes

#ifdef COR_USE_PRINTS

//@Man: COR_DEBUG()
//@Type: MACROS
//@See:  COR_ASSERT
//@Memo: debugging functionality
/**      This macro is provided to faciliate the use of debugging functions
         in a standard and uniform way.
*/

// - COR_L1(FUNC)    executes the function FUNC if the variable is 1

// - COR_L2(FUNC)    executes the function FUNC if the variable is 1 or 2

// - COR_L3(FUNC)    executes the function FUNC if the variable is 1, 2 or 3

//@{
/// Debugging Function - level 1
#define COR_L1(FUNC)\
{\
  const char* env = getenv(corDEBUG_ENV);\
  if (env &&\
      (atoi(env) < 2)) {\
    FUNC;\
  }\
}

/// Debugging Function - level 2
#define COR_L2(FUNC)\
{\
  const char* env = getenv(corDEBUG_ENV);\
  if (env &&\
      (atoi(env) < 3)) {\
    FUNC;\
  }\
}

/// Debugging Function - level 3
#define COR_L3(FUNC)\
{\
  const char* env = getenv(corDEBUG_ENV);\
  if (env &&\
      (atoi(env) < 4)) {\
    FUNC;\
  }\
}
/**
  {\footnotesize $Id: cor_print.h,v 1.1 2004/09/06 14:35:21 cguirao Exp $}

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <cor_print.h> 

    void foo();

    COR_L1(foo());
    COR_L2(foo());
    COR_L3(foo());
  \end{verbatim}

  {\large DESCRIPTION} \\
  This macro serves as an interface to using debugging code in the application.
  It accepts a function call as its only parameter and will execute that
  function if the user desires so (through setting an environment variable or
  some such).\\
  COR_L1(foo()); executes foo() if the environment variable is set to 1\\
  COR_L2(foo()); executes foo() if the environment variable is set to 1 or 2\\
  COR_L3(foo()); executes foo() if the environment variable is set to 1, 2 or
  3\\
  The environment variable is defined in "cor_debug.h".

  {\large EXAMPLE}\\
  \begin{verbatim}
    #include <cor_print.h> 

    class A {
    public:
      A(){}
      ~A(){}
      void foo() {
        COR_L1(check_sanity());
      }
    protected:
      void check_sanity(){
        // check the validity of this instance
      }
    };

    int main() {
      COR_L2(cout << "This is some text to help me debug.")
      
      A a;
      a.foo();   //  this will also call check_sanity() in the debug version
                 //  if the environment variable defined in cor_debug.h is
		 //  set to "1"

      return 0;
    }
      
    
  \end{verbatim}

  {\large SEE ALSO}\\
  \Ref{cor_debug.h}
 

  */
//@}


#else

#define COR_L1(FUNC)
#define COR_L2(FUNC)
#define COR_L3(FUNC)

#endif  // COR_USE_PRINTS defined


#endif


