package org.eso.ohs.core.dbb.client;

// $Id: DbbButtonPanel.java,v 1.6 2006/04/26 09:26:44 tcanavan Exp $

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.event.EventListenerList;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.sql.DbbCountStarted;
import org.eso.ohs.core.dbb.sql.DbbQueryReset;
import org.eso.ohs.core.dbb.sql.DbbQueryStarted;
import org.eso.ohs.core.dbb.sql.DbbQueryStopped;
import org.eso.ohs.core.dbb.sql.DbbSqlListener;

/**
 * The DbbButtonPanel class defines a button panel with five default
 * buttons: Query, Count, Break, Sort and Clear. Optional buttons may
 * be added in order to support specific user defined actions.
 * Default buttons can be hidden when not required.
 * <br>
 * <br>
 * @author Carlo Boarotto, ESO 20-Jul-1999
 * <p>
 */
public class DbbButtonPanel extends JPanel {

	private MouseClickListener startMouseListener_;
	private static final String rcsid =
		"$Id: DbbButtonPanel.java,v 1.6 2006/04/26 09:26:44 tcanavan Exp $";
	private static Logger stdlog_ = Logger.getLogger(DbbButtonPanel.class);

	/** Default button: start query */
	protected JButton startQueryButton_;

	/** Default button: count */
	protected JButton countQueryButton_;

	/** Default button: break */
	protected JButton stopQueryButton_;

	/** Default button: sort */
	protected JButton sortTableButton_;

	/** Default button: clear */
	protected JButton resetQueryButton_;

	/** Inner panel hosting optional buttons */
	protected JPanel optionalButtons_;

	private QueryActionListener startQueryListener_;

	/** List of optional buttons */
	private Vector<JButton> additionalButtonList_ = new Vector<JButton>();

	/** Last fired DbbQueryStarted event. */
	protected DbbQueryStarted startEvent_ = null;

	/** Last fired DbbCountStarted event. */
	protected DbbCountStarted countEvent_ = null;

	/** Last fired DbbQueryStopped event. */
	protected DbbQueryStopped stopEvent_ = null;

	/** Last fired DbbQueryReset event. */
	protected DbbQueryReset resetEvent_ = null;

	/** Last fired DbbTableSortEvent event. */
	protected DbbTableSortEvent sortTableEvent_ = null;

	/** List of listeners to be notified when firing an event.*/
	protected EventListenerList listenerList_ = new EventListenerList();

	//----------------------------------------
	// Constructors
	//----------------------------------------

	/** 
	 * Create a DbbButtonPanel object with some default buttons to
	 * start the query, to stop it, to reset all the browser widgets.  
	 */
	public DbbButtonPanel() {

		super(true);

		optionalButtons_ = new JPanel(true);
		optionalButtons_.setLayout(
			new BoxLayout(optionalButtons_, BoxLayout.X_AXIS));

		/* Create the buttons with their action listeners. */
		/* Start (Query) button */

		startQueryButton_ = new JButton("Query");
//		startQueryListener_ = new QueryActionListener();
//		startQueryButton_.addActionListener(startQueryListener_);
		startMouseListener_ = new MouseClickListener();
				startQueryButton_.addMouseListener(startMouseListener_);

		//startQueryButton_.addActionListener(queryActionListener_);

		/* Count button. */
		countQueryButton_ = new JButton("Count");

		ActionListener count_listener = new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				if (stdlog_.isDebugEnabled())
					stdlog_.debug("Count");
				fireDbbCountStartedAction();
			}
		};

		countQueryButton_.addActionListener(count_listener);

		/* Stop (Break) button */
		stopQueryButton_ = new JButton("Break");

		ActionListener stop_listener = new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				if (stdlog_.isDebugEnabled())
					stdlog_.debug("Break");
				fireDbbQueryStoppedAction();
			}
		};

		stopQueryButton_.addActionListener(stop_listener);
		stopQueryButton_.setEnabled(false);
		/* Sort table button */
		sortTableButton_ = new JButton("Sort");

		ActionListener sort_listener = new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				if (stdlog_.isDebugEnabled())
					stdlog_.debug("Sort");
				fireDbbTableSortAction();
			}
		};

		sortTableButton_.addActionListener(sort_listener);

		/* Reset (Clear) button */
		resetQueryButton_ = new JButton("Clear");

		ActionListener reset_listener = new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				if (stdlog_.isDebugEnabled())
					stdlog_.debug("Clear");
				fireDbbQueryResetAction();
			}
		};

		resetQueryButton_.addActionListener(reset_listener);

		/* Set a box layout manager and add the buttons */
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		this.add(startQueryButton_);
		this.add(countQueryButton_);
		this.add(stopQueryButton_);
		this.add(optionalButtons_);

		this.add(Box.createGlue());

		this.add(sortTableButton_);
		this.add(resetQueryButton_);
	}

	//----------------------------------------
	// Public methods
	//----------------------------------------

	/**
	 * Enable or disable the control buttons depending on the value of
	 * the boolean parameter enable. Stop button is enabled when all
	 * the buttons have been disabled and viceversa.
	 *
	 * @param enable Boolean value to enable or disable the control
	 *               buttons.
	 */
	public void setEnabled(boolean enable) {
		startQueryButton_.setEnabled(enable);
		countQueryButton_.setEnabled(enable);
		stopQueryButton_.setEnabled(!enable);
		sortTableButton_.setEnabled(enable);
		resetQueryButton_.setEnabled(enable);
		for (int i = 0; i < additionalButtonList_.size(); i++) {
			JButton button = additionalButtonList_.elementAt(i);
			button.setEnabled(enable);
		}
	}

	/**
	 * Add a button to the button panel.
	 *
	 * @param new_button New JButton to be added.
	 **/
	public void addButton(JButton new_button) {
		optionalButtons_.add(new_button);
		additionalButtonList_.addElement(new_button);
	}

	/** @return stopQueryButton_ */
	public JButton getStopButton() {
		return stopQueryButton_;
	}
	/** @return stopQueryButton_ */
	public JButton getClearButton() {
		return resetQueryButton_;
	}
	public JButton getQueryButton() {
		return startQueryButton_;
	}

	public void removeStartQueryListener() {
		if (startQueryListener_ != null)
			startQueryButton_.removeMouseListener(startMouseListener_);
	}

	/** Hide the count button. */
	public void hideCountButton() {
		countQueryButton_.setVisible(false);
	}

	/** Hide the stop button. */
	public void hideStopButton() {
		stopQueryButton_.setVisible(false);
	}

	/** Hide the sort button. */
	public void hideSortButton() {
		sortTableButton_.setVisible(false);
	}

	/** Hide the reset button. */
	public void hideResetButton() {
		resetQueryButton_.setVisible(false);
	}

	/** Hide the query button. */
	public void hideQueryButton() {
		startQueryButton_.setVisible(false);
	}

	//----------------------------------------
	// Event methods
	//----------------------------------------

	/**
	 * Add an object implementing the DbbSqlListener interface to the
	 * listener list. 
	 *
	 * @param listener Listener.
	 */
	public void addListener(DbbSqlListener listener) {
		listenerList_.add(DbbSqlListener.class, listener);
	}

	/**
	 * Remove a DbbSqlListener listener from the listener list.  
	 *
	 * @param listener Listener.
	 */
	public void removeListener(DbbSqlListener listener) {
		listenerList_.remove(DbbSqlListener.class, listener);
	}

	/**
	 * Add an object implementing the DbbSqlListener interface to the
	 * listener list. 
	 *
	 * @param listener Table listener.
	 */
	public void addTableListener(DbbTableListener listener) {
		listenerList_.add(DbbTableListener.class, listener);
	}

	/**
	 * Remove a DbbTableListener listener from the listener list.  
	 *
	 * @param listener Table listener.
	 */
	public void removeTableListener(DbbTableListener listener) {
		listenerList_.remove(DbbTableListener.class, listener);
	}

	/** 
	* Notify all listeners that have registered interest for
	* notification on a DbbQueryStarted event type. 
	*/
	public void fireDbbQueryStartedAction() {

		Object[] listeners = listenerList_.getListenerList();

		/* Process the listeners last to first, notifying those that are
		 * interested in this event 
		 */
		for (int i = listeners.length - 2; i >= 0; i -= 2) {

			if (listeners[i] == DbbSqlListener.class) {

				/* Lazily create the event. */
				if (startEvent_ == null)
					startEvent_ = new DbbQueryStarted(this);

				((DbbSqlListener) listeners[i + 1]).sqlStartAction(startEvent_);
			}
		}
	}

	/** 
	 * Notify all listeners that have registered interest for
	 * notification on a DbbCountStarted event type. 
	 */
	protected void fireDbbCountStartedAction() {

		Object[] listeners = listenerList_.getListenerList();

		/* Process the listeners last to first, notifying those that are
		 * interested in this event 
		 */
		for (int i = listeners.length - 2; i >= 0; i -= 2) {

			if (listeners[i] == DbbSqlListener.class) {

				/* Lazily create the event. */
				if (countEvent_ == null)
					countEvent_ = new DbbCountStarted(this);

				((DbbSqlListener) listeners[i + 1]).sqlCountAction(countEvent_);
			}
		}
	}

	/** 
	 * Notify all listeners that have registered interest for
	 * notification on a DbbQueryStopped event type.
	 */
	protected void fireDbbQueryStoppedAction() {

		Object[] listeners = listenerList_.getListenerList();

		/* Process the listeners last to first, notifying those that are
		 * interested in this event 
		 */
		for (int i = listeners.length - 2; i >= 0; i -= 2) {

			if (listeners[i] == DbbSqlListener.class) {

				/* Lazily create the event. */
				if (stopEvent_ == null)
					stopEvent_ = new DbbQueryStopped(this);

				((DbbSqlListener) listeners[i + 1]).sqlStopAction(stopEvent_);
			}
		}
	}

	/** 
	 * Notify all listeners that have registered interest for
	 * notification on a DbbQueryReset event type.
	 */
	protected void fireDbbQueryResetAction() {

		Object[] listeners = listenerList_.getListenerList();

		/* Process the listeners last to first, notifying those that are
		 * interested in this event 
		 */
		for (int i = listeners.length - 2; i >= 0; i -= 2) {

			if (listeners[i] == DbbSqlListener.class) {

				/* Lazily create the event. */
				if (resetEvent_ == null)
					resetEvent_ = new DbbQueryReset(this);

				((DbbSqlListener) listeners[i + 1]).sqlResetAction(resetEvent_);
			}
		}
	}

	/** 
	 * Notify all listeners that have registered interest for
	 * notification on a DbbTableSortEvent event type.
	 */
	protected void fireDbbTableSortAction() {

		Object[] listeners = listenerList_.getListenerList();

		/* Process the listeners last to first, notifying those that are
		 * interested in this event 
		 */
		for (int i = listeners.length - 2; i >= 0; i -= 2) {

			if (listeners[i] == DbbTableListener.class) {

				/* Lazily create the event. */
				if (sortTableEvent_ == null)
					sortTableEvent_ = new DbbTableSortEvent(this);

				((DbbTableListener) listeners[i + 1]).tableSortAction(
					sortTableEvent_);
			}
		}
	}

	private class QueryActionListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
//			if (stdlog_.isDebugEnabled())
//				stdlog_.debug("Query");
//			fireDbbQueryStartedAction();
		}

	}
	private class MouseClickListener implements MouseListener {

		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
		 */
		public void mouseClicked(MouseEvent arg0) {
			if (stdlog_.isDebugEnabled())
				stdlog_.debug("Query");
			fireDbbQueryStartedAction();

		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
		 */
		public void mousePressed(MouseEvent arg0) {

		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
		 */
		public void mouseReleased(MouseEvent arg0) {

		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
		 */
		public void mouseEntered(MouseEvent arg0) {

		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
		 */
		public void mouseExited(MouseEvent arg0) {
			
		}

	}

	//----------------------------------------
	// Debug methods
	//----------------------------------------

	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.setTitle("Test DbbButtonPanel");
		DbbButtonPanel test = new DbbButtonPanel();
		frame.getContentPane().add(test);

		WindowListener l = new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		};

		frame.addWindowListener(l);
		frame.pack();
		frame.setVisible(true);
	}
}
