package org.eso.ohs.core.dbb.client;

// $Id: DbbRadioList.java,v 1.5 2005/06/24 16:22:04 gdonino Exp $

import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.sql.DbbInvalidSqlOperatorType;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;

/**
 * This is an implementation of the DbbWidget abstract class, defining
 * a SelectionList widget. The selection list allows the user to
 * select more than one item using the standard mouse action for
 * multiple selection lists: CTRL-Button1 to select separated
 * items and Shift-Button1 to select a set of contiguous items.
 * 
 * @author Carlo Boarotto, ESO 19-Jul-1999
 * @version $Revision: 1.5 $
 * @author cmavros 05/10/01 Check and set JList to its
 * required length if less than the default 10.
 *  
*/
public class DbbRadioList extends DbbWidget implements ListSelectionListener {

	private static final String rcsid =
		"$Id: DbbRadioList.java,v 1.5 2005/06/24 16:22:04 gdonino Exp $";
	private static Logger stdlog_ = Logger.getLogger(DbbRadioList.class);

	private static String CLEAR_SELECTION = DbbSelectionList.CLEAR_SELECTION;
	private static int BUTTON_WIDTH = 145;
	private static int BUTTON_HIGH = 30;
	/** Internal JLIST to be popped up within a JPopupMenu. */
	protected JList jlist_ = null;

	/** Internal JPopupMenu to pop up the item list */
	protected JPopupMenu jlistButton_ = null;

	/** SQL chunks the user can define depending from the . */
	protected DbbSqlChunk[] sqlChunkList_;

	/** Display values for the items in the list */
	private Object[] displayObjects_ = null;

	/** 
	 * Boolean flag to remind if the list has been popped up or
	 * down. 
	 */
	private boolean popupFlag_ = false;

	/** Size of JList */
	private static final int LIST_SIZE = 10;

	//----------------------------------------
	// Constructors
	//----------------------------------------

	/**
	 * Create a DbbRadioList object with a label, associated chunk
	 * and displaying values to be mapped to a specific list of data
	 * values.
	 * 
	 * @param labeltext		Label text to be displayed.
	 * @param chunk		DbbSqlChunk object
	 * @param listData		List of data values
	 * @param listDisplayData	List of displaying values
	 */
	public DbbRadioList(
		String labeltext,
		DbbSqlChunk[] sqlChunkList,
		Object[] listDisplayData) {
		super(labeltext, sqlChunkList[0]);
		initializeDbbRadioList(labeltext, sqlChunkList, listDisplayData);
	}

	/**
	 * Initialize a DbbRadioList object with a label, associated chunk
	 * and displaying values to be mapped to a specific list of data
	 * values.
	 * 
	 * @param labeltext		Label text to be displayed.
	 * @param chunk		DbbSqlChunk object
	 * @param listData		List of data values
	 * @param listDisplayData	List of displaying values
	 */
	private void initializeDbbRadioList(
		String labeltext,
		DbbSqlChunk[] listData,
		Object[] listDisplayData) {

		default_ = "         ";

		widget_ = new JButton(default_);
		Dimension bDim = new Dimension(BUTTON_WIDTH, BUTTON_HIGH);
		widget_.setPreferredSize(bDim);
		widget_.setMinimumSize(bDim);

		jlist_ = new JList();
		jlist_.setPrototypeCellValue("12345");
		jlistButton_ = new JPopupMenu();

//		/* Pop up/down the selection list when clicking the button 
//		 */
//		 ((JButton) widget_).addActionListener(new ActionListener() {
//			public void actionPerformed(ActionEvent ev) {
//				if (popupFlag_) {
//					jlistButton_.setVisible(false);
//					popupFlag_ = false;
//				}
//			}
//		});

		/* Pop up/down the selection list when clicking the button
							 */

		 ((JButton) widget_).addMouseListener(new MouseListener() {

			public void mouseClicked(MouseEvent arg0) {

				if (jlistButton_.isVisible()) {
					jlistButton_.setVisible(false);
				} else {
					jlistButton_.show(widget_, 0, widget_.getSize().height);
					jlistButton_.setVisible(true);

				}

			}

			public void mousePressed(MouseEvent arg0) {

			}

			public void mouseReleased(MouseEvent arg0) {

			}

			public void mouseEntered(MouseEvent arg0) {

			}

			public void mouseExited(MouseEvent arg0) {
			}

		});

			JScrollPane jlist_scroll = new JScrollPane(jlist_);
			jlistButton_.add(jlist_scroll);

			label_.setLabelFor(widget_);

			jlist_.addListSelectionListener(this);
			jlist_.setSelectionMode(
				ListSelectionModel.SINGLE_INTERVAL_SELECTION);
			int numRow = setListDisplayData(listDisplayData);
			setListData(listData);
			// @author cmavros 05/10/01

			if (numRow < LIST_SIZE)
				jlist_.setVisibleRowCount(numRow);
			else
				jlist_.setVisibleRowCount(LIST_SIZE);
		}

		//----------------------------------------
		// Private methods
		//----------------------------------------

		/**
		 * Set the text of the button according to the list selection. In
		 * case of multiple selections, the widget displays only the first
		 * selected item followed by dots.  
		 */
		private void setButtonText() {

			String ret_str = default_;

			Object[] objects = jlist_.getSelectedValues();
			int len = objects.length;

			if (len > 0) {

				ret_str = objects[0].toString();
				if (len > 1)
					ret_str += ", ...";
			}

			((JButton) widget_).setText(ret_str);
		}

		//----------------------------------------
		// Protected methods
		//----------------------------------------

		/**
		 * Resize the popup list width according to longest word to be
		 * displayed within the item list.
		 */
		protected void setMaxListWidth() {

			String max_str = "12345";
			int max_len = 0;

			for (int i = 0; i < displayObjects_.length; i++) {
				String tmp_str = displayObjects_[i].toString();
				int len = tmp_str.length();
				if (len > max_len) {
					max_len = len;
					max_str = tmp_str;
				}
			}

			//www used for increasing the string size to be displayed
			max_str = max_str + "wwww";
			jlist_.setPrototypeCellValue(max_str);
		}

		/**
		 * @return An object reference to the DbbSqlChunk object associated
		 *          to this widget, updated with the current displayed
		 *          values.
		 *      
		 *  @exception DbbInvalidSqlOperatorType If the text/value currently
		 *                                       displayed in the widget
		 *                                       does not translate to a
		 *                                       valid operator. 
		 */
		public DbbSqlChunk getSqlChunk() throws DbbInvalidSqlOperatorType {
			int selIndex = Integer.decode(get()).intValue();
			return sqlChunkList_[selIndex - 1];
			//return jlist_.[selIndex];
		}

		/**
		 * Convert a display value into the corresponding data value.
		 * @param display_obj Display value to be converted.
		 * @return a data object corresponding to the given display
		 *         value. 
		 */
		protected DbbSqlChunk displayObjectToDataObject(Object display_obj) {

			String value = display_obj.toString();
			ListModel model = jlist_.getModel();
			int len = model.getSize();
			int idx = -1;

			for (int i = 0; i < len; i++) {
				Object item = model.getElementAt(i);
				if (value.equals(item.toString())) {
					idx = i;
					break;
				}
			}

			if (idx != -1 && idx < sqlChunkList_.length)
				return sqlChunkList_[idx];

			return null;
		}

		/**
		 * Convert a data value index into the corresponding display value.
		 * @param idx Data value index to be converted.
		 * @return a data object corresponding to the given data value.  
		 */
		protected Object DataObjectToDisplayObject(int idx) {
			ListModel model = jlist_.getModel();
			if (idx >= 0 && idx < model.getSize())
				return model.getElementAt(idx);

			return null;
		}

		//----------------------------------------
		// Public methods
		//----------------------------------------

		/**
		 * Set the internal list of display values.
		 * @param listDisplayData Array of items to be displayed
		 */
		public int setListDisplayData(Object[] listDisplayData) {
			Object[] listDisplayDataClear = listDisplayData;
			if (listDisplayData != null) {
				listDisplayDataClear = new Object[listDisplayData.length + 1];
				listDisplayDataClear[0] = CLEAR_SELECTION;
				for (int i = 0; i < listDisplayData.length; i++) {
					listDisplayDataClear[i+1] = listDisplayData[i];
				}
			}
			jlist_.setListData(listDisplayDataClear);
			displayObjects_ = listDisplayDataClear;

			// Set the list width
			setMaxListWidth();

			setButtonText();
			return listDisplayDataClear.length;
		}

		/**
		 * Set the internal list of data values.
		 * @param listDisplayData	Array of items to be associated to the
		 *				display values.
		 */
		public void setListData(DbbSqlChunk[] sqlChunkList) {
			sqlChunkList_ = sqlChunkList;
		}

		/** 
		 * Get the displayed index item.
		 * @return Currently displayed item index.
		 *
		  */
		public String get() {
			//  A M Chavan, 01-Feb-2000: rewritten to fix a bug [no SPR]
			String ret = "";
			int i = jlist_.getSelectedIndex();
			if (i >= 0) {
				ret = String.valueOf(i);
			}
			return ret;
		}

		/** Set the displayed string item.
		 * @param value value to be displayed.
		 */
		public void set(String value) {

			//stdlog_.debug ("the selection value \""+value+"\"");
			if (value != null && !value.equals("")) {
				ListModel model = jlist_.getModel();
				int len = model.getSize();
				int selIndex = Integer.decode(value).intValue();

				if (selIndex < 0 || selIndex >= len) {
					selIndex = 0;
				}
				jlist_.setSelectedIndex(selIndex);
				setButtonText();
			} else {
				jlist_.clearSelection();
			}
		}

		/**
		 * Method to check if the widget contents have been cleared.
		 *
		 * @return A boolean value: <code>true</code> if it is empty,
		 *         <code>false</code> otherwise.  
		 */
		public boolean isEmpty() {
			Object[] objects = jlist_.getSelectedValues();
			int len = objects.length;
			return (len == 0);
		}

		/**
		 * Reset the widget to its default state.
		 */
		public void reset() {
			jlist_.clearSelection();
			setButtonText();
		}

		//----------------------------------------
		// Event methods
		//----------------------------------------

		/**
		 * Listener interface implementation.
		 * @param ev	 Event triggering the action.
		 */
		public void valueChanged(ListSelectionEvent e) {

			// DbbWidgetEvent has to be released
			fireDbbWidgetAction();
			Object[] objects = jlist_.getSelectedValues();
			for (int i = 0; i < objects.length; i++) {
				if (objects[i].equals(CLEAR_SELECTION)) {
					jlist_.clearSelection();
					break;
				}
			}
			setButtonText();
		}
	}
