package org.eso.ohs.core.dbb.client;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import org.eso.ohs.core.dbb.xml.DbbXMLException;
import org.eso.ohs.core.utilities.MsgManager;


public abstract class Registry {

	protected HashMap<String, Object> registry = null;
	
	
	
	/** @return The registry of  DBB elements */
	private HashMap<String, Object> getRegistry() {
		return registry;
	}
	
	/**
	 * @return All the elements defined in the input XML document.
	 */
	public Iterator<Object> getElements() {
		Collection<Object> values = registry.values();
		Iterator<Object> iterator = values.iterator();
		return iterator;
	}
	
	/**
	 * Retrieve a DBB element by id.
	 * 
	 * @param id   Key to retrieve the object. If the input XML document
	 *             contains for instance an element like:
	 *             <pre>
	 *               &lt;DbbSqlChunk id="chunk1" column="surname_dl" ... /&gt;
	 *             </pre>
	 *             this method allows to retrieve the corresponding
	 *             <code>DbbSqlChunk</code> using key <em>chunk1</em>.
	 * 
	 * @return The object registered with the input id, if any;
	 *         <tt>null</tt> otherwise.
	 */
	 public Object getElement(String id ){
		 /* Design by Contract */
		 if ((id == null))
			 	throw new IllegalArgumentException(MsgManager.errPrecondition("id attribute is null"));
		 /* DbC */
		 //TODO Check for null element. Be aware of code that use hashmap.get() to avoid the insertion of duplicated item
		return registry.get(id);
	}
	/**
	 * 
	 * @param id The id of the object to look for.
	 * @return true if the the object is in the registry
	 */
	 public boolean containsKey(String id){
		 /* Design by Contract */
		 if ((id == null))
			 	throw new IllegalArgumentException(MsgManager.errPrecondition("id attribute is null"));
		 /* DbC */
		 
		return registry.containsKey(id);
	 }
	 /**
	  * Insert a new element in the element registry
	  * @param key A string used to retrieve the object
	  * @param o the object to insert
	  * @throws DbbXMLException
	  */
	 public void putElement( String key, Object obj ) throws DbbXMLException {
		 /* Design by Contract */
		 if ((key == null))
			 	throw new IllegalArgumentException(MsgManager.errPrecondition("key attribute is null"));
		 if ((obj == null))
			 	throw new IllegalArgumentException(MsgManager.errPrecondition("obj attribute is null"));
		/* DbC */
		 
		 if( registry.containsKey( key ) == true )
				throw new DbbXMLException(MsgManager.errmsg( "duplicate key: " + key ));
	            
		registry.put( key, obj );
	}//putElement
}
