package org.eso.ohs.core.dbb.client;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.util.Enumeration;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.SimpleInputDialog;
import org.eso.ohs.core.utilities.GTreeNode;
import org.eso.ohs.core.utilities.OHSConfig;

/**
 * The goal to this action is to save and
 * activate from menu
 * a widget configuration to the
 * DbbPanels columns, rows, sort.<p>
 * This action gets the current widget selection and
 * adds to a menu a new submenu.
 * The selection of this submenus retrieves and
 * applies the saved widget selection to the
 * DbbPanels columns, rows, sort.
 * The setting is saved in a file.
 * OHS Keywords used:readOnly, defaultMenuFolder,defaultMenuName,
 * rows,cols,sort, menuRoot, menuName
 * <br>
 * @version $Revision: 1.8 $
 * @author D. Dorigo
 * @author cmavros 24May02 Rearranged code to make methods more accessable to
 *          to the DefaultConfigAction().
 * @deprecated use ManageConfigAction class
 */
public class SaveConfigAction extends ActionSuperclass {

    public static final String cvsID_ =
        "$Id: SaveConfigAction.java,v 1.8 2006/04/26 09:26:44 tcanavan Exp $";

    /**
     * An OHSConfig default menu setting child: if "" the menu name is
     * in the main menu.
     */
    protected static String DEF_FOLDER_KEY = "defaultMenuFolder";

    /**
     * An OHSConfig default menu setting child: the menu name
     */
    protected static String DEF_NAME_KEY = "defaultMenuName";

    /**
     * An OHSConfig menu child: the configuration can't be overridden
     */
    protected static String READ_ONLY = "readOnly";

    /**
     * An OHSConfig menu child: all the row widgets configuration are under it
     */
    protected static String ROWS = "rows";

    /**
     * An OHSConfig menu child: all the column widgets configuration are under it
     */
    protected static String COLS = "cols";

    /**
     * An OHSConfig menu child: all the sort widgets configuration are under it
     */
    protected static String SORT = "sort";

    /**
     * An OHSConfig menu child: if "" the menu name is
     * in the main menu.
     */
    protected static String MENU_ITEM_FOLDER = "menuRoot";

    /**
     * An OHSConfig menu child:  the menu name
     */
    protected static String MENU_ITEM = "menuName";

    /**
     * paf node separator
     */
    protected static String NODE_SEPARATOR = ".";

    /**
     * empty menu
     */
    protected static String EMPTY_NAME = "";

    private JFrame parentFrame_;
    private DbbPanel rowPanel_;
    private DbbPanel colPanel_;
    private DbbPanel sortPanel_;
    private String menuRoot_;
    protected String menuFolderKeyValue_ = EMPTY_NAME;
    private String helperConfFile_;
    private boolean singleMenu_ = false;
    private JMenu menu_ = null;
    private MenuMap menuMap_ = null;
    private String requestDefault_;
    protected final String DEFAULT = "default";
    protected final String DEFAULT_REQUEST = "Save As Default";

    private static Logger stdlog_ = Logger.getLogger(SaveConfigAction.class);

    /**
     * @param parentFrame the frame
     * @param menu the menu in which config is added
     * @param rowPanel The DbbWiew filter panel
     * @param colPanel The DbbWiew show panel
     * @param sortPanel The DbbWiew sorting panel
     * @param menuRoot The menu path
     * @param helperConfFile the configuration file
     * @param name the action name is used as menu item name
     * @param singleMenu foldering of menu actions
     */
    public SaveConfigAction(
        JFrame parentFrame,
        JMenu menu,
        DbbPanel rowPanel,
        DbbPanel colPanel,
        DbbPanel sortPanel,
        String menuRoot,
        String helperConfFile,
        String name,
        boolean singleMenu) {

        super(parentFrame, name);
        menu_ = menu;
        parentFrame_ = parentFrame;
        rowPanel_ = rowPanel;
        colPanel_ = colPanel;
        sortPanel_ = sortPanel;
        menuRoot_ = menuRoot;
        helperConfFile_ = helperConfFile;
        singleMenu_ = singleMenu;
        menuMap_ = new MenuMap();
        requestDefault_ = name;
    }

    /**
     * The MenuMap container object
     * @return menuMap the menu item list of configuring widgets action.
     */
    public MenuMap getMenuMap() {
        return menuMap_;
    }

    /**
     * Action starting method
     * @param ev       user-initiated ActionEvent.
     */
    public void actionPerformedImpl(ActionEvent ev) {
        String menuName = null;
        if (requestDefault_.equals(DEFAULT_REQUEST)) {
            menuName = DEFAULT;
        } else {
            menuName = getMenuName();
        }
        if (menuName != null) {
            boolean addedMenu = addMenuToOHSConfig(menuName);
            if (addedMenu) {
                OHSConfig config = OHSConfig.getInstance();
                config.saveSubTreeToPafFile(helperConfFile_, menuRoot_);
            }
        }
    }

    /**
     * Request menu from user.
     * If the nothing is inputted return null.
     * @return String The configuration menu name to be added
     */
    private String getMenuName() {
        String menuName = null;
        if (!singleMenu_)
            menuName = getMenuViaDialog();
        else
            menuName = getSingleMenuViaDialog();

        // if not cancel op but menu empty
        if (menuName != null && menuName.length() == 0) {
            JOptionPane.showMessageDialog(
                parentFrame_,
                "Cannot save the setting: the menu name is empty.");
            menuName = null;
        }
        return menuName;
    }

    /**
     * Saves a configuration option under Options menu
     * into an helper file.
     * If a menu with this name is found and is not
     * in read only mode
     * (the menu has a child with name "readOnly"),
     * the configuration will be overridden
     * @param menuName the menuName to be saved
     * @return the operation went fine
     */
    private boolean addMenuToOHSConfig(String menuName) {
        boolean result = false;
        boolean okToAdd = false;
        OHSConfig stdConfig = OHSConfig.getInstance();
        Cursor currentCursor = parentFrame_.getCursor();
        parentFrame_.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        GTreeNode menu = menuAlreadyIn(menuName);
        if (menu != null) {

            // the menu you want to remove is read only
            if (menu.getChild(READ_ONLY) != null) {
                JOptionPane.showMessageDialog(
                    parentFrame_,
                    "A configuration having this  \""
                        + menuName
                        + "\" name is already stored \n"
                        + "and is in read only mode: "
                        + " cannot override the configuration",
                    "Warning",
                    JOptionPane.WARNING_MESSAGE);
            } else {
                int overwrite =
                    JOptionPane.showConfirmDialog(
                        parentFrame_,
                        "A configuration having this  \""
                            + menuName
                            + "\" name is already stored."
                            + "\nDo you want to overwrite it?.",
                        "Confirmation",
                        JOptionPane.YES_NO_OPTION);

                // if you want to replace, first remove it
                if (overwrite == JOptionPane.YES_OPTION) {
                    removeMenu(menu);
                    stdConfig.removeSubTree(menu.getPathName());
                    okToAdd = true;
                }
            }
        } else {
            okToAdd = true;
        }

        // adds the menu to the configuration
        if (okToAdd) {
            int newIndexMenu = 0;

            // finds the first id not used
            while (stdConfig.getNode(menuRoot_ + NODE_SEPARATOR + newIndexMenu)
                != null) {
                newIndexMenu++;
            };

            // save the new configuration
            String menuRoot = menuRoot_ + NODE_SEPARATOR + newIndexMenu;
            stdConfig.setValue(menuRoot + NODE_SEPARATOR + MENU_ITEM, menuName);
            if (!menuFolderKeyValue_.trim().equals(EMPTY_NAME)) {
                stdConfig.setValue(
                    menuRoot + NODE_SEPARATOR + MENU_ITEM_FOLDER,
                    menuFolderKeyValue_);
            }
            if (rowPanel_ != null)
                appendWidgetConf(menuRoot, rowPanel_, ROWS);
            if (colPanel_ != null)
                appendWidgetConf(menuRoot, colPanel_, COLS);
            if (sortPanel_ != null)
                appendWidgetConf(menuRoot, sortPanel_, SORT);

            // update menu popup window and savew configuration
            if (updateMenu(menuRoot) != null) {
                result = true;
            }
        }

        parentFrame_.setCursor(currentCursor);
        return result;
    }

    /**
     * Deletes a menu from the OHSconfig structure and from the menu
     * item list
     * @param menuNode the node to be removed
     */
    private void removeMenu(GTreeNode menuNode) {
        GTreeNode menuNameNode = menuNode.getChild(MENU_ITEM);
        String menuName = menuNameNode.getValue();

        GTreeNode menuFolderNode = menuNode.getChild(MENU_ITEM_FOLDER);

        // if menu is in folder
        if (menuFolderNode != null
            && !menuFolderNode.getValue().trim().equals(EMPTY_NAME)) {
            String menuFolder = menuFolderNode.getValue();

            // remove menu from map
            JMenu menuFolderMenu = menuMap_.getMenuFolder(menuFolder);
            JMenuItem menuItem = menuMap_.removeMenuItem(menuFolder, menuName);

            //remove menu from folder in gui
            if (menuItem != null) {
                menuFolderMenu.remove(menuItem);
            }
        } else {
            // the menu is in main menu popup window,
            // remove the item from map and from menu
            JMenuItem menuItem = menuMap_.removeMenuItem(menuName);
            if (menuItem != null) {
                menu_.remove(menuItem);
            }
        }
    }

    /**
     * adds a menu configuration list from the OHSconfig structure
     * @param menuPath the starting path: the children are menu configuration
     */
    public void addMenus(String menuPath) {
        OHSConfig config = OHSConfig.getInstance();
        GTreeNode node = config.getNode(menuPath);
        if (node != null) {
            for (Enumeration<GTreeNode> e = node.getChildren(); e.hasMoreElements();) {
                GTreeNode nextMenu = e.nextElement();
                String nextMenuPath = nextMenu.getPathName();
               //  ConfigWidgetValuesAction action = 
                updateMenu(nextMenuPath);
            }
        }
    }

    /**
     * apply this menu setting.
     *
     * @param menuDefaultKey the starting path:
     * the children are menu folder (defaultMenuFolder)  and name (defaultMenuName)
     */
    public void applyDefault(String menuDefaultKey) {
        ActionSuperclass defAction = null;
        OHSConfig config = OHSConfig.getInstance();
        String nameFolder =
            config.getValue(menuDefaultKey + NODE_SEPARATOR + DEF_FOLDER_KEY);
        String nameMenu =
            config.getValue(menuDefaultKey + NODE_SEPARATOR + DEF_NAME_KEY);

        if (!nameFolder.equals(EMPTY_NAME)) {
            defAction =
                (ActionSuperclass)menuMap_.getActionMenu(nameFolder, nameMenu);
        } else {
            defAction = (ActionSuperclass)menuMap_.getActionMenu(nameMenu);
        }
        if (defAction != null) {
            defAction.actionPerformedImpl(new ActionEvent(defAction, 0, EMPTY_NAME));
        }
    }

    /**
     * adds a menu item to the menu passed in the constructor
     * @param menuPath is the menu path to the OHSConfig that stores the
     *                  widget configurations.
     */
    private ConfigWidgetValuesAction updateMenu(String menuPath) {
        String menuName = null;
        String menuFolder = EMPTY_NAME;
        JMenuItem menuItemValue = null;
        ConfigWidgetValuesAction action = null;
        OHSConfig config = OHSConfig.getInstance();
        GTreeNode menuItem = config.getNode(menuPath);
        GTreeNode colWidgets = null;
        GTreeNode rowWidgets = null;
        GTreeNode sortWidgets = null;
        GTreeNode menuNameNode = null;
        if (menuItem != null) {
            colWidgets = menuItem.getChild(COLS);
            rowWidgets = menuItem.getChild(ROWS);
            sortWidgets = menuItem.getChild(SORT);
            menuNameNode = menuItem.getChild(MENU_ITEM);
        }
        if (menuNameNode != null) {
            menuName = menuNameNode.getValue();
            action =
                new ConfigWidgetValuesAction(
                    parentFrame_,
                    menuName,
                    colWidgets,
                    rowWidgets,
                    sortWidgets,
                    colPanel_,
                    rowPanel_,
                    sortPanel_);

            GTreeNode menuFolderNode = menuItem.getChild(MENU_ITEM_FOLDER);
            // search if folder in
            if (menuFolderNode != null) {

                menuFolder = menuFolderNode.getValue();
            }
            if (!menuFolder.trim().equals(EMPTY_NAME)) {
                JMenu menuFolderMenu = menuMap_.getMenuFolder(menuFolder);

                // the menu folder didn't exist, create it
                if (menuFolderMenu == null) {
                    stdlog_.debug("create menu folder: " + menuFolder);
                    menuFolderMenu = new JMenu(menuFolder);
                    menuMap_.addMenuFolder(menuFolder, menu_.add(menuFolderMenu));
                }

                // add menu to folder
                menuItemValue = menuFolderMenu.add(action);
                menuMap_.addMenuItem(menuFolder, menuName, menuItemValue, action);
            } else {
                // add menu to main menu
                menuItemValue = menu_.add(action);
                menuMap_.addMenuItem(menuName, menuItemValue, action);
            }

        }
        return action;
    }

    /**
     * Writes the modified buffer (after the removing
     * of the option) to the helper config file.
     *
     * @param newFileConfigBuffer buffer to be written.
     *                  If is null the file is not touch.
     * @return true if file is written successfully
     */
   /* public boolean writeConfigFile(StringBuffer newFileConfigBuffer) {
        boolean result = true;
        try {
            if (newFileConfigBuffer != null) {
                PrintWriter out = new PrintWriter(new FileWriter(helperConfFile_));
                String body = newFileConfigBuffer.toString();
                out.println(body);
                out.close();
            }
        } catch (IOException e) {
            stdlog_.error("File \"" + helperConfFile_ + "\" not written", e);
            result = false;
        }
        return result;
    }*/

    /**
     * append the widget configuration to the menuRoot
     * @param menuRoot menu path parent
     * @param dbbPanel panel to check
     * @param panelName name of the panel used in configuration
     */
    private void appendWidgetConf(
        String menuRoot,
        DbbPanel dbbPanel,
        String panelName) {
        OHSConfig stdConfig = OHSConfig.getInstance();
        DbbWidget[] rowList = dbbPanel.getAllWidgets();
        DbbWidget[] rowActiveList = dbbPanel.getActiveWidgets();
        // add widget configuration for the active widgets
        for (int i = 0; i < rowActiveList.length; i++) {
            String widgetName = rowActiveList[i].getLabel().getText();
            String widgetValue = rowActiveList[i].get().replace('\'', ' ');
            String pathMenuWidget =
                menuRoot
                    + NODE_SEPARATOR
                    + panelName
                    + NODE_SEPARATOR
                    + i
                    + NODE_SEPARATOR;
            stdConfig.setValue(pathMenuWidget + DbbPanel.NAME_KEY, widgetName);
            stdConfig.setValue(pathMenuWidget + DbbPanel.VALUE_KEY, widgetValue);
            stdConfig.setValue(pathMenuWidget + DbbPanel.ORDER_KEY, i);
        }
        // add widget configuration for the remaining widgets
        int ordIndex = rowActiveList.length;
        for (int j = 0; j < rowList.length; j++) {
            String widgetName = rowList[j].getLabel().getText();
            boolean isActive = false;
            for (int k = 0; k < rowActiveList.length; k++) {
                String widgetActiveName = rowActiveList[k].getLabel().getText();
                if (widgetActiveName.equals(widgetName)) {
                    isActive = true;
                }
            }
            if (!isActive) {
                String widgetValue = rowList[j].get();
                String pathMenuWidget =
                    menuRoot
                        + NODE_SEPARATOR
                        + panelName
                        + NODE_SEPARATOR
                        + ordIndex
                        + NODE_SEPARATOR;
                stdConfig.setValue(pathMenuWidget + DbbPanel.NAME_KEY, widgetName);
                stdConfig.setValue(pathMenuWidget + DbbPanel.VALUE_KEY, widgetValue);
                stdConfig.setValue(pathMenuWidget + DbbPanel.ORDER_KEY, ordIndex);
                ordIndex++;

            }
        }
    }

    /**
     * Checks if the menu is already stored.
     * @param menuName menu to be stored
     * @return null if menu is not stored
     */
    private GTreeNode menuAlreadyIn(String menuName) {
        OHSConfig stdConfig = OHSConfig.getInstance();
        GTreeNode menusRootNode = stdConfig.getNode(menuRoot_);
        String nextMenuName = EMPTY_NAME;
        String nextMenuNameFolder = EMPTY_NAME;
        if (menusRootNode != null) {
            for (Enumeration<GTreeNode> e = menusRootNode.getChildren(); e.hasMoreElements();) {

                GTreeNode nextMenu = e.nextElement();
                GTreeNode menuNameNode = nextMenu.getChild(MENU_ITEM);
                GTreeNode menuFolderNode = nextMenu.getChild(MENU_ITEM_FOLDER);
                if (menuNameNode != null) {
                    nextMenuName = menuNameNode.getValue();
                } else {
                    nextMenuName = EMPTY_NAME;
                }
                if (menuFolderNode != null) {
                    nextMenuNameFolder = menuFolderNode.getValue();
                } else {
                    nextMenuNameFolder = EMPTY_NAME;
                }
                if (nextMenuNameFolder.trim().equals(menuFolderKeyValue_.trim())) {
                    if (!nextMenuName.equals(EMPTY_NAME)
                        && nextMenuName.equals(menuName)) {
                        return nextMenu;
                    }
                }
            }
        }
        return null;
    }

    /**
     * Starts an menu dialog in which the client inserts the menu name
     */
    private String getSingleMenuViaDialog() {

        menuFolderKeyValue_ = EMPTY_NAME;
        JTextField newNameTF = new JTextField();
        // JTextField newRootNameTF = new JTextField();

        SimpleInputDialog dlg =
            new SimpleInputDialog(
                parentFrame_,
                "Save Settings",
                "Enter details:\n",
                new String[] { "Settings Name:" },
                new Component[] { newNameTF });
        dlg.show();

        if (dlg.getUserChoice() == JOptionPane.OK_OPTION) {
            return newNameTF.getText();
        } else {
            return null; // Cancelled or closed window?
        }
    }

    /**
     * Starts an menu dialog in which the client inserts the menu name
     */
    private String getMenuViaDialog() {
        menuFolderKeyValue_ = EMPTY_NAME;
        JTextField newNameTF = new JTextField();
        JTextField newRootNameTF = new JTextField();
        SimpleInputDialog dlg =
            new SimpleInputDialog(
                parentFrame_,
                "Save Settings",
                "Enter details:\n",
                new String[] { "Folder Name <optional>:", "Settings Name:" },
                new Component[] { newRootNameTF, newNameTF });
        dlg.show();
        if (dlg.getUserChoice() == JOptionPane.OK_OPTION) {
            menuFolderKeyValue_ = newRootNameTF.getText();
            return newNameTF.getText();
        } else {
            return null; // Cancelled or closed window?
        }
    }
}

