/* Created on Dec 9, 2003 */

package org.eso.ohs.core.dbb.htmlclient;

import java.util.Enumeration;
import java.util.Vector;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;

/**
 * TODO: javadoc
 * 
 * @author amchavan
 * @version $Revision: 1.4 $
 */

// $Id: DbbHtmlPanel.java,v 1.4 2006/04/26 09:26:44 tcanavan Exp $

public class DbbHtmlPanel {

	protected static Logger stdlog_ = Logger.getLogger(DbbHtmlPanel.class);
	/** Max number of rows in this panel. */
	public static final int MAXROWS = 10;

	private static final String rcsid =
		"$Id: DbbHtmlPanel.java,v 1.4 2006/04/26 09:26:44 tcanavan Exp $";

	/** Number of displayed columns */
	protected int cols_ = -1;

	/** Number of displayed rows */
	private int rows_ = -1;

	/** Current column: used when laying out the widgets in the panel */
	private int currentCol_ = 0;

	/** Current row: used when laying out the widgets in the panel */
	private int currentRow_ = 0;

	/** Max row: used when rendering the panel */
	protected int maxRow_ = -1;

	/** Implements the grid of widgets making up this panel */
	protected DbbHtmlWidget grid[][] = null;

	/** This panel's title */
	private String title_ = null;
	
	private String dbbPanelType_ = null;

	/**
	 * Create a panel with a specific title and a grid of
	 * <code>cols</code> columns of widgets. Row number is
	 * arbitrary, but cannot exceed the limit set by the
	 * <code>MAXROWS</code> constant.
	 *
	 * @param title		    Title label of the panel.
	 * @param cols          Number of widget columns.
	 * @see #MAXROWS
	 */
	public DbbHtmlPanel(String title, int cols) {
		this(title, MAXROWS, cols);
	}

	/**
	 * Create a panel with a specific title and a grid of 
	 * <code>rows</code> x <code>cols</code> widgets.
	 *
	 * @param title		    Title label of the panel.
	 * @param rows          Number of widget rows.  
	 * @param cols          Number of widget columns.
	 */
	public DbbHtmlPanel(String title, int rows, int cols) {

		cols_ = cols;
		rows_ = rows;
		title_ = title;
		grid = new DbbHtmlWidget[rows][cols];
	}

	/** 
	 * Add a DbbWidget to the column selection panel.  Widgets are
	 * added left-to-right; once the last column of the panel is
	 * filled, a new row of widgets is created.
	 *
	 * @param w DbbWidget object to be added.  
	 */
	public void addWidget(DbbHtmlWidget w) {

		addWidget(w, currentRow_, currentCol_);

		currentCol_++; // we added one column

		// If necessary, start a new row
		//----------------------------------------
		if (currentCol_ >= cols_) {
			currentCol_ = 0;
			currentRow_++;
		}
	}

	/** 
	 *
	 * Add a DbbHtmlWidget to the column selection panel.
	 *
	 * @param w       DbbHtmlWidget object to be added.  
	 * @param new_row If true, the DbbHtmlWidget is placed on the first
	 *                column of a ne row; otherwise on the next column 
	 *                of the current row.
	 */
	public void addWidget(DbbHtmlWidget w, boolean new_row) {

		if (new_row) {
			currentCol_ = 0;
			currentRow_++;
		}

		addWidget(w);
	}

	/** 
	 * Add a DbbHtmlWidget to the column selection panel.
	 * <BR>
	 * The widget is placed on the grid in position (row,col).
	 *
	 * @param w   DbbHtmlWidget object to be added.  
	 * @param row Row number
	 * @param col Column number: must be between 0 and cols-1,
	 *            where cols is the size parameter given
	 *            to the constructor.
	 *
	 */
	public void addWidget(DbbHtmlWidget w, int row, int col) {

		if (row >= rows_)
			throw new IllegalArgumentException(
				"Arg row (" + row + ") too large, should be < " + rows_);
		if (col >= cols_)
			throw new IllegalArgumentException(
				"Arg col (" + col + ") too large, should be < " + cols_);

		grid[row][col] = w;
		if (row > maxRow_)
			maxRow_ = row;
	}

	/** 
	 * @return Array of DbbHtmlWidget objects, corresponding to the all the
	 *         widgets.  
	 */
	public DbbHtmlWidget[] getAllWidgets() {

		Vector<DbbHtmlWidget> v = new Vector<DbbHtmlWidget>();

		for (int r = 0; r < rows_; r++)
			for (int c = 0; c < cols_; c++) {
				DbbHtmlWidget w = grid[r][c];
				if (w != null)
					v.add(w);
			}

		int len = v.size();
		DbbHtmlWidget[] ret = new DbbHtmlWidget[len];
		v.copyInto(ret);

		return ret;
	}

	/** @return This panel's title string */
	public String getTitle() {
		return title_;
	};

	/**
	 * Generate a fragment of HTML code rendering this panel. The fragment is
	 * suitable to be included in the HTML rendering of a DbbHtmlView (an HTML
	 * table).
	 * <BR>
	 * The generated fragment is not indented.
	 * 
	 * @return An HTML fragment rendering this widget.
	 */
	public String render() {
		return render(0);
	}

	/**
	 * Generate a fragment of HTML code rendering this panel. The fragment is
	 * suitable to be included in the HTML rendering of a DbbHtmlView (an HTML
	 * table).
	 * 
	 * @param indentLevel
	 *            The level of indentation for the HTML fragment. Each level is
	 *            indented by INDENTSPACES spaces.
	 * 
	 * @return An HTML fragment rendering this widget.
	 * 
	 * @see DbbHtmlWidget#INDENTSPACES
	 */
	public String render(int indentLevel) {
		String ind0 = DbbHtmlWidget.indentSpaces(indentLevel);
		String ind1 = DbbHtmlWidget.indentSpaces(indentLevel + 1);
		String ind2 = DbbHtmlWidget.indentSpaces(indentLevel + 2);

		StringBuffer sb = new StringBuffer();

		sb.append(ind0).append("<table>\n");
		if (getTitle() != null) {
			sb.append(ind1).append("<caption>").append(getTitle()).append(
				"</caption>\n");
		}

		for (int i = 0; i <= maxRow_; i++) {

			sb.append(ind1).append("<tr>\n");

			for (int j = 0; j < cols_; j++)
				if (grid[i][j] != null)
					sb.append(grid[i][j].render(indentLevel + 2));
				else
					sb.append(ind2).append("<td colspan=\"2\"><br></td>\n");
			sb.append(ind1).append("</tr>\n");
		}
		sb.append(ind0).append("</table>\n");

		return sb.toString();
	}

	/** 
	 * @return Array of SQL chunks, corresponding to the active widgets;
	 *         that is, widgets having some non-blank values.
	 * @see DbbSqlChunk 
	 */
	public DbbSqlChunk[] getActiveChunks(HttpServletRequest req) {

		Vector<DbbSqlChunk> v = new Vector<DbbSqlChunk>();

		for (int i = 0; i < rows_; i++)
			for (int j = 0; j < cols_; j++) {

				if (grid[i][j] == null)
					continue;

				DbbHtmlWidget w = grid[i][j];
				String wname = w.getName();
				Enumeration pnames = req.getParameterNames();

				stdlog_.debug(">>> wname: " + wname);

				while (pnames.hasMoreElements()) {

					String pname = (String) pnames.nextElement();
					stdlog_.debug("      pname: " + pname);
					if (!pname.equals(wname))
						continue;
					String value = req.getParameter(pname);
					stdlog_.debug("        value: '" + value + "'");
					w.set(value);
					DbbSqlChunk c = w.getSqlChunk();
					if (c.isActive())
						v.add(c);
					stdlog_.debug(">>> active: " + c);
					break;
				}
			}
		DbbSqlChunk t[] = (DbbSqlChunk[]) v.toArray(new DbbSqlChunk[0]);
		return t;
	}
	
	public void setType(String type) {
		dbbPanelType_ = type;
		
	}
	public String getType() {
		return dbbPanelType_;
		
	}
}