package org.eso.ohs.core.dbb.htmlclient;

import org.eso.ohs.core.dbb.sql.DbbSqlChunk;

/**
 * This is an implementation of the DbbHtmlWidget abstract class,
 * defining a text field.
 * <br>
 * @author amchavan, ESO 09-Dec-2003
 * @version $Revision: 1.5 $
 */

// $Id: DbbHtmlTextField.java,v 1.5 2006/03/07 10:29:04 gdonino Exp $

public class DbbHtmlTextField extends DbbHtmlWidget {

	private boolean isCaseSensitive = false;
	/** Deafult width for text widgets */
	public final static int COLUMNSIZE = 15;

	public static final String cvsID_ =
		"$Id: DbbHtmlTextField.java,v 1.5 2006/03/07 10:29:04 gdonino Exp $";

	/**
	 * Create an instance associated with a specific chunk.
	 *
	 * @param chunk DbbSqlChunk object.
	 */
	public DbbHtmlTextField(DbbSqlChunk chunk) {
		this(chunk.getLabel(), chunk, true);
	}

	/**
	 * Create an instance, with a specific label, associated with a
	 * specific chunk
	 *
	 * @param labeltext Label text to be displayed.
	 * @param chunk     DbbSqlChunk object.
	 */
	public DbbHtmlTextField(String labeltext, DbbSqlChunk chunk) {
		this(labeltext, chunk, true);
		setDefault("");
	}
	
	/**
	 * Create an instance, with a specific label, associated with a
	 * specific chunk
	 *
	 * @param labeltext Label text to be displayed.
	 * @param chunk     DbbSqlChunk object.
	 * @param caseSensitive yes if the widget is caseSensitive
	 */
	public DbbHtmlTextField(String labeltext, DbbSqlChunk chunk, boolean caseSensitive) {
		super(labeltext, chunk);
		setDefault("");
		isCaseSensitive = caseSensitive;
		setCaseSensitive(chunk, caseSensitive);
	}

	/** Implementation of the abstract method */
	public String render(int indentLevel) {
		String name = getName();
		StringBuffer sb = new StringBuffer();

		sb
			.append(indentSpaces(indentLevel))
			.append("<td align=\"right\"> ")
			.append(getLabel())
			.append(" </td>\n")
			.append(indentSpaces(indentLevel))
			.append("<td> ")
			.append("<input type=\"text\" name=\"")
			.append(name)
			.append("\">")
			.append(" </td>\n");

		return sb.toString();
	}

	/**
	 * Set the widget's value (implementation of the abstract method).
	 *
	 * @param val Can be any string.
	 */
	public void set(String val) {
		boolean active = false;
		if (val != null && val.length() > 0) {
			active = true;
		}
		DbbSqlChunk c = getSqlChunk();
		c.setActive(active);
		c.setValue(val);
		value_ = val;
	}
	
	protected void setCaseSensitive(DbbSqlChunk chunk, boolean isCaseSensitive){
		this.isCaseSensitive = isCaseSensitive;
		chunk.setCaseSensitive(isCaseSensitive);
	}
	
}
