package org.eso.ohs.core.dbb.htmlclient;

import org.eso.ohs.core.dbb.sql.DbbSqlChunk;

/**
 * This is an implementation of the DbbHtmlWidget abstract class,
 * defining three-state widget.
 * <br>
 * @author amchavan, ESO 09-Dec-2003
 * @version $Revision: 1.2 $
 */

// $Id: DbbHtmlThreeState.java,v 1.2 2006/03/07 10:29:04 gdonino Exp $

public class DbbHtmlThreeState extends DbbHtmlWidget {


	public static final String cvsID_ =
		"$Id: DbbHtmlThreeState.java,v 1.2 2006/03/07 10:29:04 gdonino Exp $";

	/**
	 * Create a DbbHtmlThreeState associated with a specific chunk.
	 *
	 * @param chunk DbbSqlChunk object.
	 */
	public DbbHtmlThreeState( DbbSqlChunk chunk ) {
		this( chunk.getLabel(), chunk );
	}

	/**
	 * Create a DbbHtmlThreeState, with a specific label, associated with a
	 * specific chunk
	 *
	 * @param labeltext Label text to be displayed.
	 * @param chunk     DbbSqlChunk object.
	 */
	public DbbHtmlThreeState(String labeltext, DbbSqlChunk chunk) {
		super( labeltext, chunk );
		setDefault( SORT_NO );
	}
	
	/** Implementation of the abstract method. The widget is
     *  rendered so that by default value "none" is selected. */
	public String render( int indentLevel ) {
        
        String     name = getName(); 
		StringBuffer sb = new StringBuffer();
		
		sb
			.append(indentSpaces( indentLevel ))
			.append("<td>")
			.append(getLabel())
			.append("</td>\n")
		
			.append(indentSpaces( indentLevel ))
			.append("<td>")
			.append("<input type=\"radio\" value=\"none\"       name=\"")
			.append( name )
            // by default, value "none" is selected.
            .append("\" CHECKED> none")
            .append("</td>\n")
            
			.append(indentSpaces( indentLevel ))
            .append("<td>")
			.append("<input type=\"radio\" value=\"ascending\"  name=\"")
			.append( name )
			.append("\"> ascending")
            .append("</td>\n")

			.append(indentSpaces( indentLevel ))
            .append("<td>")
			.append("<input type=\"radio\" value=\"descending\" name=\"")
			.append( name )
			.append("\"> descending")
            .append("</td>\n");
		
		return sb.toString();
	}

    /**
     * Set the widget's value (implementation of the abstract method).
     *
     * @param val Can be DbbHtmlWidget.SORT_ASC, DbbHtmlWidget.SORT_DESC
     *            or DbbHtmlWidget.SORT_NO.
     * 
     * @see DbbHtmlWidget#SORT_ASC
     * @see DbbHtmlWidget#SORT_DESC
     * @see DbbHtmlWidget#SORT_NO
     */
	public void set( String val ) {
		String      v = val.toLowerCase();
		DbbSqlChunk c = getSqlChunk();
        
        if( v.equals( SORT_ASC  )) {
            c.setOperator( OP_ASC );
            c.setActive( true );
        }
        else if( v.equals( SORT_DESC )) {
            c.setOperator( OP_DESC );
            c.setActive( true );
        }
        else if( v.equals( SORT_NO   )) { 
            c.setOperator( null );
            c.setActive( false );
        }
        else
            throw new RuntimeException( "Invalid value: '" + val + "'" );

        value_ = v;
        //  TODO: there is a bug in DbbSqlChunk.processText() (to be fixed)
        //        such that c.setValue() will fail.
        //        Commented out for the time being. 
        //        amchavan, 30-Dec-2003
        // c.setValue( v );
    }
}
