package org.eso.ohs.core.dbb.server;

// $Id: DbbSqlEngine.java,v 1.2 2006/01/20 10:31:20 gdonino Exp $

import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;

import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlQueryChunks;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;

/**   
 * This is the main engine of a database browser: builds SQL
 * statements, performs queries (within threads), returns result data.
 *
 * @author Carlo Boarotto, ESO 19-Jul-1999
 * <br>ChangeLog: A M Chavan, ESO 26-Jul-1999
 *
 */
public interface DbbSqlEngine {


    /**
     * Create and start a thread to get the result set and to process it
     * into an Object array.  
     *
     * @param sql	Sql statement.
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public DbbQuery startQuery(DbbSqlQueryChunks chunks)
	throws RemoteException, SQLException,
	MalformedURLException, NotBoundException;

    /**
     * Create and start a thread to get the result set and to process it
     * into an Object array.  
     *
     * @param sql	Sql statement.
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
   /* public DbbQuery startQuery(DbbSqlQueryChunks chunks, boolean isPaginated)
	throws RemoteException, SQLException,
	MalformedURLException, NotBoundException;*/
    
    /**
     * Number of records in a result set   
     *
     * @param sql	Sql statement.
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public int execSqlCount(DbbSqlQueryChunks chunks)
	throws SQLException, RemoteException,
	MalformedURLException, NotBoundException;


    /**
     * Add default colums to a particular chunk
     *
     * @param sql	Sql statement.
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public void addDefaultColumnsChunk(DbbSqlChunk chunk)
	throws SQLException, RemoteException,
	MalformedURLException, NotBoundException;


    public DbbSqlChunk[] getDefaultColumnsChunk()
	throws SQLException, RemoteException,
	MalformedURLException, NotBoundException;

    /**
     * Add a preferential join condition to be considered when building
     * automatically the query.
     *
     * @param from_tbl	DbbSqlTable indicating the primary key table.
     * @param from_col	String indicating the primary key table column.
     * @param to_tbl	DbbSqlTable indicating the foreign key table.
     * @param to_col	String indicating the foreign key table column.
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public void addJoinCondition(DbbSqlTable from_tbl,
				 String from_col,
				 DbbSqlTable to_tbl,
				 String to_col)
	throws SQLException, RemoteException,
	MalformedURLException, NotBoundException;
	
    /**
     * Specify a left outer join condition to be considered when building
     * automatically the query.
     *
     * @param from_tbl  DbbSqlTable indicating the primary key table.
     * @param from_col  String indicating the primary key table column.
     * @param to_tbl    DbbSqlTable indicating the foreign key table.
     * @param to_col    String indicating the foreign key table column.
     *
     * @exception SQLException JDBC exception occurs
     * @exception RemoteException RMI exception occurs
     * @exception MalformedURLException malformed url i.e. servername//port
     * @exception NotBoundException client could not bind to server
     */
    public void addCompositeJoinCondition(DbbSqlTable from_tbl,
                 String from_col[],
                 DbbSqlTable to_tbl,
                 String to_col[], boolean leftOuterJoin)
    throws SQLException, RemoteException,
    MalformedURLException, NotBoundException;
    
   /* public String sqlBuildPositioningQuery(DbbSqlQueryChunks chunks, boolean isCount)
    throws SQLException;*/
    
    /**
     * Build an SQL string query to be used for pagination purpose.
     * The query is based on the default chunks as well as sume
     * new chunks passed as parameters. Note that default chunks take
     * precedence (they are processed first).
     *
     * @param columns   List of chunks: columns to retrieve
     * @param rows      List of chunks: rows to retrieve
     * @param rows      List of chunks: sort columns
     * @param group     List of chunks: "group by" columns
     * @param isCount   Boolean flag to indicate if it is only a sql count statement
     * @exception SQLException if a jgbc call throws an excpetion
     */
   /* public String sqlBuildFetchingQuery(DbbSqlQueryChunks chunks, boolean isCount)
        throws SQLException;*/
    
   /* *//**
     * This method start a paginated query.
     *
     * @param positioning      String to be convertend into a prepared statement. The sql is use to find the correct page
     * @param fecthing      String to be convertend into a prepared statement. The sql is use to fetch the correct page
     * @param pageNumber      Page number to be displayed
     * @param pageSize      Size of the page
     * @exception SQLException if a jdbc call throws an exception
     *//*
    public DbbQuery execPaginatedQuery(String positioning, String fetching, int pageNumber, int pageSize ) throws SQLException;
*/
}
