package org.eso.ohs.core.gui.baseaction;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.utilities.Utilities;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.OutOfMemoryReport;

/**
 * Common superclass for all Action classes.
 * 
 * <p>
 * $Id: ActionSuperclass.java,v 1.4 2006/02/07 08:04:08 gdonino Exp $
 * 
 * <br>
 * @author Bob Kemp
 */
abstract public class ActionSuperclass extends AbstractAction {

	
	private String name_ = "";
	private JFrame baseFrame_;
    private JComponent parentComp;
	   /**
     * Field stdlog_
     */
    protected static org.apache.log4j.Logger stdlog_ = Logger
            .getLogger(ActionSuperclass.class);
	/**
	 * Constructor: saves a copy of the window in front of which
	 * any dialogue box should appear.
	 */
	public ActionSuperclass(JComponent comp, String name) {
		super(name);
        if (comp == null) {
            throw new NullPointerException("Always need a top level ancestor");
        }
		setName(name);  
        parentComp = comp;
	    baseFrame_ = (JFrame) comp.getTopLevelAncestor();        
        
	}

	public ActionSuperclass(JFrame frm, String name) {
		super(name);
        if (frm == null) {
            throw new NullPointerException("Always need a top level ancestor");
        }
		setName(name);
		baseFrame_ = frm;
	}

	protected JFrame getWdwComponent() throws NullPointerException {
        if (baseFrame_ == null) {
            baseFrame_ = (JFrame) parentComp.getTopLevelAncestor();  
        }
		return baseFrame_;
	}

	public final void actionPerformed(final ActionEvent ev) {
		final JFrame mg = getWdwComponent();

		final Runnable runner = new Runnable() {
			public void run() {
				try {
					actionPerformedImpl(ev);
				} catch (OutOfMemoryError e) {
					e.printStackTrace();
					OutOfMemoryReport.showMessage();
					throw e;
				} finally {
					Utilities.normalCursor(mg);
				}
			}
		};

		if (mg != null)
			Utilities.waitCursor(mg);
		SwingUtilities.invokeLater(runner);

		/********************************************
		// FOR TESTING OF CURSORS, USE THIS CODE INSTEAD OF JUST invokeLater()
		new Thread() {
		    public void run() {
			try {
			    sleep(5000);
			}
			catch (Exception e) {
			    // ignore
			}
			SwingUtilities.invokeLater(runner);
		    }
		}.start();
		*********************************************/

	}

	public abstract void actionPerformedImpl(ActionEvent ev);

	/**
	 * Convenience method to call announceIOError() in the P2PP class.
	 * 
	 * @param e the exception that was thrown.
	 */
	public void announceIOError(Exception e) {
		ErrorMessages.announceIOError(getWdwComponent(), e);
	}

	/**
	 * Convenience method to call announceNoObject() in the P2PP class.
	 * 
	 * @param e the exception that was thrown.
	 */
	public void announceNoObject(Exception e) {
		ErrorMessages.announceNoObject(getWdwComponent(), e);
	}


	public void setName(String name) {
		this.name_ = name;
		
	}

	public String getName() {
		return name_;
	}

}
