package org.eso.ohs.core.gui.models;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;

/**
 * A Document model to only permit the input of a list of integers.
 * 
 * <p>
 * $Id: IntListModel.java,v
 1.2 1999/10/20 08:40:21 bkemp Exp $
 */

public class IntListModel extends InputMaskModel {

 /** Maximum number of characters that this is allowed to contain. */
    private int maxLength_ = -1;

    /** The regular expression describing legal buffer values */
    private final static String legalValuesPattern_ =
	"\\s*-?\\d*(\\s+-?\\d*)*\\s*";

    /**
     * Constructor for the model.
     */
    public IntListModel() {
	super(legalValuesPattern_);
    }

  /**
     * Constructor for the model.
     * @param maxlength  Maximum number of characters that
     *                   can be typed into the view.
     */
    public IntListModel(int maxlength) {
	super(legalValuesPattern_);
	maxLength_ = maxlength;
    }

    /**
     * Inserts integers from the given string into this Document, upto
     * a fixed limit.
     * 
     * @param offset	offset in the text buffer at which to insert string.
     * @param str	string to be inserted.
     * @param attr	text attributes.
     * 
     * @exception BadLocationException	an illegal offset was specified.
     */
    public void insertString(int offset, String str, AttributeSet attr)
		throws BadLocationException {
	int len = getLength();

	if ((maxLength_ >= 0) && (len+str.length() > maxLength_)) {
			// If insertion would make text too long, just return.
	    return;
	}

	super.insertString(offset, str, attr);
    }

}

