/*
 * Created on Sep 15, 2004
 */
package org.eso.ohs.core.gui.widgets;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Calendar;
import java.util.Date;

import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.DateContainer;
/**
 * @author ddorigo
 * @version $Revision: 1.9 $
 */

// $Id: DateComboBox.java,v 1.9 2006/03/06 10:57:00 ddorigo Exp $

public class DateComboBox extends JPanel {
    /**
     * Logger for this class
     */
    private static final Logger stdlog_ = Logger.getLogger(DateComboBox.class);

    private static final int STD_LENGTH = 10;

    private static final int STD_WIDTH = 210;

    private static Integer[] dayList_ = null;

    private static final int MAX_DAYS = 31;

    private JComboBox day_;

    private JComboBox month_;

    private JComboBox year_;

    private Date date_;

    private boolean guiSet_ = false;

    private JLabel label_ = null;

    public DateComboBox(String label) {
        this(STD_WIDTH, STD_LENGTH, label);
    }

    public DateComboBox(int width, int high,String title ) {
        if (title != null) {
        label_ = new JLabel(title);
        }
         guiSet_ = false;
        Dimension standardDimension = new Dimension(width, high);
        day_ = new JComboBox();
        month_ = new JComboBox();
        year_ = new JComboBox();
        Dimension standardDayDimension = new Dimension(60, 23);
        Dimension standardMonthDimension = new Dimension(60, 23);
        Dimension standardYearDimension = new Dimension(60, 23);

     //   day_.setPreferredSize(standardDayDimension);
        setMaxDayList();
       
        month_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                assignDays();
            }

        });
        setMonthList(Convert.getMonths());
      // month_.setPreferredSize(standardMonthDimension);
       
        year_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                assignDays();
            }

        });
        setYearList(Calendar.getInstance().get(Calendar.YEAR), Calendar
                .getInstance()
                .get(Calendar.YEAR));
     //  year_.setPreferredSize(standardYearDimension);
         this.setLayout(new BoxLayout(this,
                 BoxLayout.X_AXIS));
      
        if (label_!= null) {
        this.add(label_);   
        }
        this.add(day_);
        this.add(month_);
        this.add(year_);
       // this.setPreferredSize(standardDimension);
    }

    /**
     *  
     */
    protected void assignDays() {
        if (guiSet_) {
            String month = (String) month_.getSelectedItem();
        Integer year = (Integer) year_.getSelectedItem();
        Calendar cal = Calendar.getInstance();
        cal.clear();
        if (month != null && year != null) {
            cal.set(year.intValue(), Convert.monthToCalendarMonth(month),1);
        }
        int days = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
        setDayList(days);
        }
    }

    /**
     * @param maxDays
     */
    private void setDayList(int numDays) {
   
            Integer day = (Integer) day_.getSelectedItem();
            Integer[] dayList = getDays(numDays);
            day_.removeAllItems();
            for (int i = 0; i < dayList.length; i++) {
                day_.addItem(dayList[i]);
            }
            day_.setSelectedItem(day);
      
    }

    /**
     * @param maxDays
     */
    private void setMaxDayList() {
        setDayList(MAX_DAYS);
    }

    /**
     * @param object
     */
    public void unSelect() {
        day_.setSelectedItem(null);
        month_.setSelectedItem(null);
        year_.setSelectedItem(null);

    }

    /**
     * @param maxDays
     */
    public void setMonthList(String[] monthList) {
        String month = (String) month_.getSelectedItem();
        month_.removeAllItems();
        for (int i = 0; i < monthList.length; i++) {
            month_.addItem(monthList[i]);
        }
        month_.setSelectedItem(month);
    }

    /** Return Years for Detailed View */
    private Integer[] getYearsDetailedView(int min, int max) {
        int minimum = min;
        int diff = max - min;
        Integer[] years = new Integer[diff + 1];
        for (int i = 0; i <= diff; i++) {
            years[i] = new Integer(min);
            min += 1;
        }
        return years;
    }

    /**
     * @param startYear
     * @param endYear
     */
    public void setYearList(int startYear, int endYear) {
        Integer[] yearList = getYearsDetailedView(startYear, endYear);
        Integer year = (Integer) year_.getSelectedItem();
        year_.removeAllItems();
        for (int i = 0; i < yearList.length; i++) {
            year_.addItem(yearList[i]);
        }
        year_.setSelectedItem(year);

    }

    /**
     * @return Integer[] numDays in month
     */
    public static Integer[] getMaxDays() {
        return getDays(MAX_DAYS);
    }

    public static Integer[] getDays(int max) {
        if (dayList_ == null) {
            dayList_ = new Integer[MAX_DAYS];
            for (int i = 0; i < MAX_DAYS; i++) {
                dayList_[i] = new Integer(i + 1);
            }
        }
        if (max > MAX_DAYS) {
            stdlog_.error("No more than " + MAX_DAYS
                    + " days in a month.  Requested " + max + " days");
            max = MAX_DAYS;
        }
        Integer[] days = new Integer[max];
        for (int i = 0; i < max; i++) {
            days[i] = dayList_[i];
        }
        return days;
    }

    /**
     * @param fromDateObj
     */
    public void setDate(Date dateObj) {
        guiSet_ = false;
        DateContainer dateFields = DateContainer.convertDate(dateObj);
        Calendar cal = Calendar.getInstance();
        cal.setTime(dateObj);
        int daysInMonth = cal.getActualMaximum(Calendar.DAY_OF_MONTH);
        setDayList(daysInMonth);
        date_ = dateObj;
        day_.setSelectedItem(new Integer(dateFields.getDay()));
        month_.setSelectedItem(dateFields.getMonth());
        year_.setSelectedItem(new Integer(dateFields.getYear()));
        guiSet_ = true;
    }

    /**
     * @return Integer Get current date
     */
    public Integer getDay() {
        Integer day = (Integer) day_.getSelectedItem();
        if (day == null) {
            day = new Integer(0);
        }
        return day;
    }

    /**
     * @return String Month (three letters or empty string)
     */
    public String getMonth() {
        String month = (String) month_.getSelectedItem();
        if (month == null) {
            month = "";
        }
        return month;
    }

    /**
     * @return Inteeger year 
     */
    public Integer getYear() {
        Integer year = (Integer) year_.getSelectedItem();
        if (year == null) {
            year = new Integer(0);
        }
        return year;
    }

    /**
     * @return String date MEDIUM
     */
    public String getSelectedDateString() {
        Date dateObj = getSelectedDate();
        return Convert.dateToSimpleFormatString(dateObj);

    }

    /**
     * @return long timemilliseconds
     */
    public long getSelectedDateLong() {
        Date dateObj = getSelectedDate();
        return Convert.dateToStarcatDate(dateObj);

    }

    /**
     * @return Date obj Date
     */
    public Date getSelectedDate() {
        Integer day = (Integer) day_.getSelectedItem();
        String month = (String) month_.getSelectedItem();
        Integer year = (Integer) year_.getSelectedItem();
        Calendar cal = Calendar.getInstance();
        cal.clear();
        if (day != null && month != null && year != null) {
            cal.set(
                    year.intValue(), Convert.monthToCalendarMonth(month),
                    day.intValue());
        }
        Date dateObj = cal.getTime();
        return dateObj;
    }

}