package org.eso.ohs.core.gui.widgets;

import java.util.Comparator;
import java.util.Date;
import java.util.Vector;

import javax.swing.table.TableModel;

import org.eso.ohs.core.utilities.MsgManager;

public class DbbTableComparator implements Comparator {
	TableModel model_ = null;
	boolean isAscending_ = false;
	Vector<Integer> sortingColumns_ = null;
	public DbbTableComparator(TableModel model, Vector<Integer> sortingColumns, boolean isAscending){
		//Dbc//
		if (model==null) MsgManager.errPrecondition("object model is null");
		if (sortingColumns==null) MsgManager.errPrecondition("object sortingColumns is null");
		if (sortingColumns.size() == 0) MsgManager.errPrecondition("object sortingColumns has no elements");
		//Dbc//
		model_ = model;
		isAscending_ = isAscending;
		sortingColumns_ = sortingColumns;
	}
	
	public int compare(Object a, Object b) {
	    //Dbc//
		if (a==null) MsgManager.errPrecondition("object a is null");
		if (b==null) MsgManager.errPrecondition("object b is null");
		if (!Integer.class.isInstance(a)) MsgManager.errPrecondition("object a is not an Integer value");
		if (!Integer.class.isInstance(b)) MsgManager.errPrecondition("object b is not an Integer value");
		//Dbc//
		for (int level = 0; level < sortingColumns_.size(); level++) {
			Integer column = sortingColumns_.elementAt(level);
		    int result = compareRowsByColumn(((Integer)a).intValue(), ((Integer)b).intValue(), column.intValue());
		    if (result != 0) {
				return isAscending_ ? result : -result;
		    }
		}
		return 0;
	}
	
	 public int compareRowsByColumn(int row1, int row2, int column) {
        Class type = model_.getColumnClass(column);
        TableModel data = model_;

        // Check for nulls.

        Object o1 = data.getValueAt(row1, column);
        Object o2 = data.getValueAt(row2, column); 

        // If both values are null, return 0.
        if (o1 == null && o2 == null) {
            return 0; 
        } else if (o1 == null) { // Define null less than everything. 
            return -1; 
        } else if (o2 == null) { 
            return 1; 
        }

        /*
         * We copy all returned values from the getValue call in case
         * an optimised model is reusing one object to return many
         * values.  The Number subclasses in the JDK are immutable and
         * so will not be used in this way but other subclasses of
         * Number might want to do this to save space and avoid
         * unnecessary heap allocation.
         */

        if (o1 instanceof java.lang.Number) {
            Number n1 = (Number)data.getValueAt(row1, column);
            double d1 = n1.doubleValue();
            Number n2 = (Number)data.getValueAt(row2, column);
            double d2 = n2.doubleValue();

            if (d1 < d2) {
                return -1;
            } else if (d1 > d2) {
                return 1;
            } else {
                return 0;
            }
        } else if (type == java.util.Date.class) {
            Date d1 = (Date)data.getValueAt(row1, column);
            long n1 = d1.getTime();
            Date d2 = (Date)data.getValueAt(row2, column);
            long n2 = d2.getTime();

            if (n1 < n2) {
                return -1;
            } else if (n1 > n2) {
                return 1;
            } else {
                return 0;
            }
        } else if (type == String.class) {
              Object v1 = data.getValueAt(row1, column);
              String s1 = v1.toString().toUpperCase();
              Object v2 = data.getValueAt(row2, column);
              String s2 = v2.toString().toUpperCase();
           
            int result = s1.compareTo(s2);

            if (result < 0) {
                return -1;
            } else if (result > 0) {
                return 1;
            } else {
                return 0;
            }
        } else if (type == Boolean.class) {
            Boolean bool1 = (Boolean)data.getValueAt(row1, column);
            boolean b1 = bool1.booleanValue();
            Boolean bool2 = (Boolean)data.getValueAt(row2, column);
            boolean b2 = bool2.booleanValue();

            if (b1 == b2) {
                return 0;
            } else if (b1) { // Define false < true
                return 1;
            } else {
                return -1;
            }
        } else {
            Object v1 = data.getValueAt(row1, column);
            String s1 = v1.toString().toUpperCase();
            Object v2 = data.getValueAt(row2, column);
            String s2 = v2.toString().toUpperCase();
            int result = s1.compareTo(s2);

            if (result < 0) {
                return -1;
            } else if (result > 0) {
                return 1;
            } else {
        	return 0;
            }
        }
	    }

}
