package org.eso.ohs.core.gui.widgets;

import java.awt.Color;

import javax.swing.JFrame;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

public class MultiLineCellRenderer
    extends javax.swing.JTextArea
    implements javax.swing.table.TableCellRenderer {

    boolean spaceBreak = false;

 

    /**
    * setLineWrap(false);  We control the wrapping of the JTextArea below.  I do not  like how the
    * JTextArea did its wrapping. setWrapStyleWord(false); Again, this will break the line on a
    * space (which we don't want for this application.) setEditable(false); I don't want the user
    * to be able to edit the table cells. setOpaque(true);  So we can see the background of the
    * Table
    */
    public MultiLineCellRenderer() {
        setLineWrap(false);
        setWrapStyleWord(false);
        setEditable(false);
        setOpaque(true);
        spaceBreak = false;
    }

    /**
    * getTableCellRendererComponent() is the only method that must be over-ridden.  It will
    * basically stamp an image of the object that we want to appear into the cell of the  jTable.
    * It is not actually the object but only an rendering of the object.  Any  special actions or
    * events we want to happen, must be added progromatically to the Table.
    *
    * @param table the JTable holding cell we wish to render
    * @param value the object which we wish to put into the cell
    * @param isSelected boolean to allow selected cell to appear as such
    * @param hasFocus boolean to alert as to the cell having focus
    * @param row row of the JTable in which cell resides
    * @param column column of the JTable in which the cell resides.
    *
    * @return Component the image of the value object.  Not a true object but an  image of the
    *         object.
    */
    public java.awt.Component getTableCellRendererComponent(
        javax.swing.JTable table,
        Object value,
        boolean isSelected,
        boolean hasFocus,
        int row,
        int column) {
        if (isSelected) {
            setForeground(table.getSelectionForeground());
            setBackground(table.getSelectionBackground());
        } else {
            setForeground(table.getForeground());
            setBackground(table.getBackground());
        }
        setFont(table.getFont());
        if (hasFocus) {
            setBorder(
                javax.swing.UIManager.getBorder(
                    "Table.focusCellHighlightBorder"));
            if (table.isCellEditable(row, column)) {
                setForeground(
                    javax.swing.UIManager.getColor(
                        "Table.focusCellForeground"));
                setBackground(
                    javax.swing.UIManager.getColor(
                        "Table.focusCellBackground"));
            }
        } else {
            setBorder(new javax.swing.border.EmptyBorder(1, 2, 1, 2));
        }
        setText(value.toString());

        /**
        * What makes this version different than others is that I reset the height of the row
        * based on the number of lines in the JTextArea that is now the cell.  (I add the +1 to
        * even the spacing a bit.  It accounts for the cell-border height.)
        */
        //if (getLineCount() > 1) {
            int thisrowHeight = getLineCount() * (table.getRowHeight() + 1);
            if (table.getRowHeight(row) != thisrowHeight) {
                table.setRowHeight(
                    row,
                    getLineCount() * (table.getRowHeight() + 1));
            }
        //}

        return this;
    }

    @SuppressWarnings("serial")
	public static void main(String[] args) {
        
        
        // Create the table
        // The table data
        
        JFrame f = new JFrame("Using a Column Renderer");


         final Object[][] data = { { "Hot & Spicy", Boolean.TRUE }, {
                 "123456    123456      128901\n90-01221 90-01331 50-01331", Boolean.FALSE }, {
                 "Ham", Boolean.TRUE }, {
                 "New Yorker", Boolean.FALSE }, {
                 "Vegetarian", Boolean.TRUE }
         };

        final String[] columnNames = { "Pizza", "Ordered?" };
        DefaultTableModel model = new DefaultTableModel(data, columnNames) {
            @SuppressWarnings("unchecked")
			public Class getColumnClass(int columnIndex) {
                return data[0][columnIndex].getClass();
            }
        };

        JTable tbl = new JTable(model) {
            public boolean isCellEditable(int row, int column) {
                return false;
            }
        };
        System.out.println("WE ARE HERE 11");
        tbl.setColumnSelectionAllowed(true);

        // Create the column renderer for the first column
        MultiLineCellRenderer renderer = new MultiLineCellRenderer();
        System.out.println("WE ARE HERE 22");

        // Configure the renderer as we want it to be
        renderer.setBackground(Color.yellow);
        renderer.setForeground(Color.black);
        System.out.println("WE ARE HERE 33");

        renderer.setToolTipText("Pizza Name");

        // Attach the renderer to the first column
        tbl.getColumnModel().getColumn(0).setCellRenderer(renderer);
        f.getContentPane().add(tbl);
        System.out.println("WE ARE HERE 44");

        f.pack();
        f.setVisible(true);
    }

}
