package org.eso.ohs.core.gui.widgets;

/** 
 * In a chain of data manipulators some behaviour is common. TableMap
 * provides most of this behavour and can be subclassed by filters
 * that only need to override a handful of specific methods. TableMap 
 * implements TableModel by routing all requests to its model, and
 * TableModelListener by routing all events to its listeners. Inserting 
 * a TableMap which has not been subclassed into a chain of table filters 
 * should have no effect.
 *
 * @version 1.4 12/17/97
 * @author Philip Milne */

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import org.eso.ohs.core.dbb.client.AbstractDbbTableModel;

public class TableMap extends AbstractTableModel 
                      implements TableModelListener {
    protected AbstractDbbTableModel model; 

    public TableModel getModel() {
        return model;
    }

    public void setModel(AbstractDbbTableModel model) {
    	
        this.model = model; 
        this.model.addTableModelListener(this); 
    }

    // By default, implement TableModel by forwarding all messages 
    // to the model. 

    public Object getValueAt(int aRow, int aColumn) {
        return model.getValueAt(aRow, aColumn); 
    }
        

    public Object getValueAt(int aRow, String aColumn) {
	int cols = getColumnCount();
	int colId = cols+1;
	for (int i = 0; i < cols; i++) {
	    if (aColumn.equals( model.getColumnName(i))) {
		colId = i;
		break;
	    }
	}
        return model.getValueAt(aRow, colId); 
    }
      
    public void setValueAt(Object aValue, int aRow, int aColumn) {
        model.setValueAt(aValue, aRow, aColumn); 
    }

    public int getRowCount() {
        return (model == null) ? 0 : model.getRowCount(); 
    }

    public int getColumnCount() {
        return (model == null) ? 0 : model.getColumnCount(); 
    }
        
    public String getColumnName(int aColumn) {
        return model.getColumnName(aColumn); 
    }

   
	public Class<?> getColumnClass(int aColumn) {
        return model.getColumnClass(aColumn); 
    }
        
    public boolean isCellEditable(int row, int column) { 
         return model.isCellEditable(row, column); 
    }
//
// Implementation of the TableModelListener interface, 
//
    // By default forward all events to all the listeners. 
    public void tableChanged(TableModelEvent e) {
        fireTableChanged(e);
    }
}
