package org.eso.ohs.core.gui.widgets;

import java.awt.Component;
import java.awt.event.ActionEvent;

import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

/**
 * There is not getToolTipText method in JComboBox
 * This class provides it so tooltip text cab
 * be rendered for a JComboBox.

 */

public class ToolTipJComboBox extends JComboBox {

	private String[] tooltips_;

	public ToolTipJComboBox() {
		initialise();
	}
	
	public ToolTipJComboBox(Object[] items , String [] tooltips) {
		super(items);
		tooltips_ = tooltips;
		initialise();
	}

	private void initialise() {
		setRenderer(new Renderer());
		addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	toolTipJComboBoxActionPerformed(evt);
            }

			
        });
	}

	private void toolTipJComboBoxActionPerformed(ActionEvent evt) {
		int index = getSelectedIndex();
		if ( index > -1)
			setToolTipText(getToolTips()[index]);
		
	}
	
	public String[] getToolTips() {
		return tooltips_;
	}

	public void setToolTips(String[] tips) {
		tooltips_ = tips;
	}

	private class Renderer extends BasicComboBoxRenderer {

		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			if (isSelected) {
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			} else {
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}
			if (index >= 0) {
				list.setToolTipText(tooltips_[index]);
			}
			setFont(list.getFont());
			setText((value == null) ? "" : value.toString());

			return this;
		}

	}
	
	

}
