package org.eso.ohs.core.utilities;

import java.util.Calendar;
import java.util.Date;

/**
 * This class contains Date utilities
 * 
 * @author pnunes 24.03.2006
 *
 */
public class DateUtils {

	
	/** number of milliseconds in a day as a constant for easier date computation */
	public static final double MILLISECONDS_IN_A_DAY = (double) (24 * 60 * 60 * 1000);
	
	/**
	 * Computes the number of hours between two dates
	 * 
	 * @param startTime
	 * @param endTime
	 * @return number of hours
	 */
	public static double numHours(Date startTime, Date endTime) {
		if (startTime == null) {
			throw new IllegalArgumentException("Start time is null");
		}
		
		if (endTime == null) {
			throw new IllegalArgumentException("Start time is null");
		}
		
		if ( startTime.after(endTime)) {
			throw new IllegalArgumentException("Start time after end time");
		}
		
		double hoursDiff = (endTime.getTime() - startTime.getTime()) / (1000*60*60);
		return hoursDiff;
	}
	
	
	/**
	 * Removes hours, minutes and seconds from a date
	 * 
	 * @param date
	 * @return
	 */
	public static Date getDate(Date date) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		calendar.set(Calendar.HOUR_OF_DAY, 0);
		calendar.set(Calendar.MINUTE, 0);
		calendar.set(Calendar.SECOND, 0);
		return(calendar.getTime());
	}
	
	/**
	 * Removes year, month and day from a date
	 * @param date
	 * @return
	 */
	public static Date getTime(Date date) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		int hours = calendar.get(Calendar.HOUR_OF_DAY);
		int mins = calendar.get(Calendar.MINUTE);
		int secs = calendar.get(Calendar.SECOND);
		
		calendar.setTime(new Date(0));
		calendar.set(Calendar.HOUR_OF_DAY, hours);
		calendar.set(Calendar.MINUTE, mins);
		calendar.set(Calendar.SECOND, secs);
		return(calendar.getTime());
		
	}
	
	
	public static double numDays(Date startTime, Date endTime ) {
		if (startTime == null) {
			throw new IllegalArgumentException("Start time is null");
		}
		
		if (endTime == null) {
			throw new IllegalArgumentException("Start time is null");
		}
		
		double startInMilliseconds = (double) (startTime.getTime());
		double endInMilliseconds = (double)   (endTime.getTime());
		double numDays = (endInMilliseconds - startInMilliseconds)/ MILLISECONDS_IN_A_DAY;
		
		return(numDays);
	}
	


}
