package org.eso.ohs.core.utilities;

/**
 * A GTreePath implementation that can build a tree path from a Paramfile
 * syntax path specification.
 *
 * @author Nick Kornweibel for ESO.
 */

import java.util.Vector;

public class ParamfileTreePath extends DefaultGTreePath {

    /**
     * Build a tree path based on the Paramfile syntax, where
     * the tree node names are separated by "." character.
     * The resulting path will be empty if the path String is not
     * a valid Paramfile syntax keyword name.
     */

    public ParamfileTreePath(String path) {
        String[] pathNodes;
        if (Paramfile.isValidKeyword(path)) {
            Vector<String> nodes = new Vector<String>();
            int end;
            while ((end = path.indexOf('.')) >= 0) {
                String nName = path.substring(0, end);
                if (nName.length() > 0) {
                    nodes.addElement(nName);
                }
                end++;
                if (end == path.length()) {
                    break;
                }
                path = path.substring(end);
            }
            nodes.addElement(path);
            pathNodes = new String[nodes.size()];
            for (int i = 0; i < pathNodes.length; i++) {
                pathNodes[i] = nodes.elementAt(i);
            }
        } else {
            pathNodes = new String[0];
        }
        setPath(pathNodes);
    }

    /**
     * @return true if this tree path is valid or not.
     * A valid tree path is built from a valid Paramfile keyword.
     */
    public boolean isValidPath() {
        return getPath().length > 0;
    }

    /**
     * return path as string (in Paramfile format). Returns "invalid path"
     * if the path is invalid.
     */
    public String toString() {
        StringBuffer rc = new StringBuffer();
        if (isValidPath()) {
            GTreeNode[] path = getPath();
            for (int i = 0; i < path.length; i++) {
                if (i > 0) {
                    rc.append(".");
                }
                rc.append(path[i].getName());
            }
        } else {
            rc.append("invalid path");
        }
        return rc.toString();
    }
}
