package org.eso.ohs.core.utilities;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;

/**
 * Sybase-related utilities.
 * 
 * @version $Revision: 1.1 $
 * @author amchavan, 31-Jan-2006: was SybaseErrorCodes
 */

// $Id: Sybase.java,v 1.1 2006/02/28 15:51:38 amchavan Exp $

public class Sybase {

    /*
     * Integer values for Sybase error messages. It would be nice to produce a
     * general mapping for these error codes so that more meningful error
     * messages could be printed. For example user error messages and system
     * error messages
     */
    
    /** Error code */
    public static final int INVALID_LOGIN = 4002;

    /** Error code */
    public static final int INVALID_PASSWORD = 4067;

    /**
     * Runs all the machinery of connecting to a Sybase server.
     * 
     * @param server    Server and port specification, separated by a colon
     *                  char, like <code>acdbdev.hq.eso.org:6789</code>
     * @param user      Sybase user ID 
     * @param password  Passowrd for that user ID
     * @param database  Name of the database to connect to.
     * 
     * @return A connection to the input server.
     * 
     * @throws SQLException
     */
    public static Connection getConnection( String server,
                                            String user,
                                            String password,
                                            String database ) 
        throws SQLException {
        
        final String jdbcDriver = "jdbc:sybase:Tds";
        final String driverClass = "com.sybase.jdbc2.jdbc.SybDriver";
    
        Properties props = new Properties();
        props.put( "CHARSET", "iso_1" );
        props.put( "user", user );
        props.put( "password", password );
    
        Properties sysProps = System.getProperties();
        StringBuffer drivers = new StringBuffer( driverClass );
        String oldDrivers = sysProps.getProperty( "jdbc.drivers" );
        if( oldDrivers != null )
            drivers.append( ":" ).append( oldDrivers );
        sysProps.put( "jdbc.drivers", drivers.toString() );
    
        String url = jdbcDriver + ":" + server + "/" + database;
    
        Connection jdbcConnection = DriverManager.getConnection( url, props );
        return jdbcConnection;
    }
}
