/**
 $Id: TextUtils.java,v 1.19.4.2 2007/10/23 08:08:38 pnunes Exp $
 */

package org.eso.ohs.core.utilities;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.Vector;


/**
 * Static class used to manipulate texts, especially used for SQL queries.
 * 
 * <h3>Usage</h3>
 * 
 * <pre>String sqlStatement = TextUtils.quoteSqlString(sql);</pre>
 * 
 */
public class TextUtils {

    /** New-line */
    private static final String NL = "\n";
    
    /** Empty string */
    private static final String EMPTY = "";
    
	/**
	 * Field stdlog_
	 */
	protected static org.apache.log4j.Logger stdlog_ = org.apache.log4j.Logger
			.getLogger(TextUtils.class);

	public static int DEFAULT_STRING_LENGTH = 80;

	/**
	 * Field spaces
	 */
	private static final String spaces = "                               ";

	/**
	 * Field zeroes
	 */
	private static final String zeroes = "0000000000000000000000000000000";

	

	/** Constant defining the single dollar character. */
	public static final char DOLLAR = '$';

	/** Constant defining the single dollar character. */
	public static final char BACKSLASH = '\\';
	/** Constant defining the single quote character. */
	private static char QUOTE = '\'';
	/** Constant defining the double quote character. */
	private static char DOUBLEQUOTE = '\"';

	/**
	 * Return a string with numDigits as lenght.
	 * <BR>
	 * The right side of the string contains the long "n"
	 * 
	 * @param padding
	 *            String
	 * @param numDigits
	 *            int
	 * @param n
	 *            long
	 * @return String
	 */
	public static String padded(String padding, int numDigits, long n) {
		String unpadded = new Long(n).toString();

		// string is already >= field size?
		if (numDigits <= unpadded.length()) {
			return unpadded;
		}

		return padding.substring(0, numDigits - unpadded.length()) + unpadded;
	}

	/**
	 *  Return a string with numDigits as lenght.
	 * <BR>
	 * The left side of the string contains the long "n"
	 * 
	 * @param s
	 *            String
	 * @param numchars
	 *            int
	 * @param ch
	 *            char
	 * @return String
	 */
	public static String padLeft(String s, int numchars, char ch) {
		int initStrSize = s.length();
		if (initStrSize >= numchars)
			return s; // no padding required
		if (numchars < initStrSize)
			throw new ArrayIndexOutOfBoundsException(
					"Error nmber of chanracters to to small");
		int numToInsert = numchars - s.length();
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < numToInsert; i++) {
			buf.append(ch);
		}
		return buf.toString() + s;
	}

	/**
	 * Method getSubString if the value is null returns empty String, if the
	 * value length is lower then maxLentgth, it gives back the value
	 * 
	 * @param value
	 *            String
	 * @param maxLength
	 *            int
	 * @return String
	 */
	public String getSubString(String value, int maxLength) {
		if (value == null) {
			value = "";
		}

		if (value.length() <= maxLength) {
			maxLength = value.length();
		}
		return value.substring(0, maxLength);
	}

	/**
	 * 
	 * Append to the right the string "unpadded" up to numChars characters
	 * Input string is: abcdefghilmno
	 * Unpadded string is: test
	 * numChars: 10
	 * Padded string is: abcdftest
	 * 
	 * @param padding
	 *            String
	 * @param numChars
	 *            int
	 * @param unpadded
	 *            String
	 * @return String
	 */
	public static String padded(String padding, int numChars, String unpadded) {

		// string is already >= field size?
		if (numChars <= unpadded.length()) {
			return unpadded;
		}

		return padding.substring(0, numChars - unpadded.length()) + unpadded;
	}

	/**
	 * <br>
	 * ChangeLog: A M Chavan, 05-Oct-2001: creation
	 * 
	 * Append to the right the string "unpadded" up to numChars characters
	 * Input string is: abcdefghilmno
	 * Unpadded string is: test
	 * numChars: 10
	 * Padded string is: testabcdf
	 * 
	 * 
	 * @param padding
	 *            String
	 * @param numChars
	 *            int
	 * @param unpadded
	 *            String
	 * @return String
	 */
	public static String rightpadded(String padding, int numChars,
			String unpadded) {

		// IF input string is already longer than the field size
		// THEN return input string
		if (numChars <= unpadded.length())
			return unpadded;

		return unpadded + padding.substring(0, numChars - unpadded.length());
	}

	/**
	 * Return the ISO Date and time.
	 * Eg. 2005-05-04T10:19:56
	 * 
	 * @return String
	 */
	public static String currentISODateTime() {
		SimpleDateFormat timeFormat = new SimpleDateFormat(
				Convert.ISO_DATE_FORMAT);
		return timeFormat.format(new Date());
	}

	/**
	 * Static method to "quote" single quote in a given text.
	 * 
	 * @param str
	 *            String to be processed and "quoted".
	 * 
	 * @return Processed string.
	 */
	public static String escapeDollarString(String str) {

		if (str == null) {
			return str;
		}
		return quoteString(str,DOLLAR,BACKSLASH);
	}

	/**
	 * Static method to "quote" single quote in a given text.
	 * 
	 * @param str
	 *            String to be processed and "quoted".
	 * 
	 * @return Processed string.
	 */
	public static String quoteSqlString(String str) {
        if (str == null) {
			return str;
		}       
       return quoteString(str,QUOTE,QUOTE);
	}
	
	/**
	 * Static method to "quote" single quote in a given text.
	 * 
	 * @param str
	 *            String to be processed and "quoted".
	 * 
	 * @return Processed string.
	 */
	public static String doubleQuoteString(String str) {
        if (str == null) {
			return str;
		}       
       return DOUBLEQUOTE+ quoteString(str,DOUBLEQUOTE,DOUBLEQUOTE)+DOUBLEQUOTE;
	}

	/**
	 * Static method to "quote" single quote in a given text.
	 * 
	 * @param str
	 *            String to be processed and "quoted".
	 * 
	 * @return Processed string.
	 */
	public static String quoteSqlString(String str, int maxLength) {

		if (str == null) {
			return str;
		}
		int length = maxLength;
		if (length > str.length()) {
			length = str.length();
		}
		String subStr = str.substring(0, length);
		return quoteString(subStr,QUOTE,QUOTE);
	}

	/**
	 * @param length
	 * @param subStr
	 * @return
	 */
	private static String quoteString(String subStr,char charToEscape, char escapingChar) {
		if (subStr.indexOf(charToEscape) == -1) {		
			return subStr;
		}
		
		StringBuffer out_str = new StringBuffer("");
		
		char character;

		for (int i = 0; i < subStr.length(); i++) {
			character = subStr.charAt(i);

			if (character == charToEscape) {
				out_str.append(escapingChar);
			}
			out_str.append(character);
		}

		return out_str.toString();
	}

	/**
	 * Static method to avoid null values
	 * 
	 * @param str
	 *            String to be processed.
	 * 
	 * @return Processed string.
	 */
	public static String getEmptySqlStringIfNull(String str) {

		if (str == null) {
			return "";
		}
		return str;
	}

	/**
	 * Create a string of numDigits character, pad spaces on the left and add the long n on the right.
	 * 
	 * @param numDigits
	 *            int
	 * @param n
	 *            long
	 * @return String
	 */
	public static String spacePadded(int numDigits, long n) {
		return padded(spaces, numDigits, n);
	}

	/**
	 * Create a string of numDigits character, pad zeroes on the left and add the long n on the right.
	 * 
	 * @param numDigits
	 *            int
	 * @param n
	 *            long
	 * @return String
	 */
	public static String zeroPadded(int numDigits, long n) {
		return padded(zeroes, numDigits, n);
	}

	/**
	 * <br>
	 * Create a string of numDigits character, pad zeroes on the left and add the String s on the right.
	 * 
	 * @param numDigits
	 *            int
	 * @param s
	 *            String
	 * @return String
	 */
	public static String zeroPadded(int numDigits, String s) {
		return padded(zeroes, numDigits, s);
	}

	/**
	 * Return a string newLen characters long, where on the left is the initial string s and on the right
	 * is padded with spaces.
	 * 
	 * @param s
	 *            String
	 * @param newLen
	 *            int
	 * @return String
	 */
	public static String padString(String s, int newLen) {
		if (s.length() >= newLen)
			return s;
		StringBuffer buf = new StringBuffer(s);
		int padLen = newLen - s.length();
		for (int i = 0; i < padLen; i++)
			buf.append(" ");
		return buf.toString();
	}

	/**
	 * Split a Tcl/Tk string representing an array into constituent elements and
	 * return as a Java array.
	 * 
	 * <p>
	 * For example, "a {b c} d" is the java array { "a", "b c", "d" }
	 * 
	 * <b>Note: </b> this method does not handle all possible Tcl arrays!!
	 * However, it's good enough for most purposes.
	 * 
	 * @param msg
	 *            tcl array as a string.
	 * @return a Java array of strings
	 */
	public static String[] splitTclArray(String msg) {
		Vector<String> vec = new Vector<String>();
		int index;

		for (; msg.length() > 0;) {
			if (msg.charAt(0) == '{') {
				index = msg.indexOf('}');
				vec.addElement(msg.substring(1, index));
			} else {
				index = msg.indexOf(' ');

				if (index < 0) {
					vec.addElement(msg);
					break;
				}
				vec.addElement(msg.substring(0, index));
			}

			msg = msg.substring(index + 1);
		}

		String[] array = new String[vec.size()];
		vec.copyInto(array);

		return array;
	}

	/**
	 * Escape (add \\) to backslashes, newlines and doublequotes.
	 * 
	 * @param original
	 *            String
	 * @return String
	 */
	public static String escapedString(String original) {
		final int len = original.length();
		StringBuffer buf = new StringBuffer();

		// Escape backslashes, newlines or doublequotes
		for (int k = 0; k < len; k++) {
			char ch = original.charAt(k);
			switch (ch) {
			case '\\':
			case '\"':
				buf.append("\\");
				buf.append(ch);
				break;

			case '\r':
			case '\n':
				buf.append("\\n");
				break;
				
			case '\t':
				buf.append("\\t");
				break;				

			default:
				buf.append(ch);
				break;
			}
		}

		return buf.toString();
	}

	/**
	 * Escape just the quotes from a string
	 * 
	 * @return String
	 */
	public static String escapedQuoteString(String original) {
		final int len = original.length();
		StringBuffer buf = new StringBuffer();

		// Escape backslashes, newlines or doublequotes
		for (int k = 0; k < len; k++) {
			char ch = original.charAt(k);
			switch (ch) {
			case '\\':
			case '\"':
				buf.append("\\");
				buf.append(ch);
				break;
			default:
				buf.append(ch);
				break;
			}
		}

		return buf.toString();
	}

	/**
	 * <br>
	 * ChangeLog: A M Chavan, 09-Aug-2001: fixed bug: was not dealing correctly
	 * with strings whose last char is a backslash
	 * 
	 * Replace backslashes, newlines and returns from an escaped string. 
	 * 
	 * @param original
	 *            String
	 * @return String
	 */
	public static String revertEscapedString(String original) {
		final int len = original.length();
		StringBuffer buf = new StringBuffer();

		// Escape backslashes, newlines or doublequotes
		for (int k = 0; k < len; k++) {
			char ch = original.charAt(k);
			if (ch == '\\') {
				k++;
				if (k < len) // A M Chavan, 09-Aug-2001
					ch = original.charAt(k);
				switch (ch) {
				case 'n':
					buf.append('\n');
					break;
				case 'r':
					buf.append('\r');
					break;
				case 't':
					buf.append('\t');  //escape tabs
					break;					
				default:
					buf.append(ch);
				}
			} else
				buf.append(ch);

		}
		return buf.toString();
	}

	/**
	 * Returns the path corresponding to the package containing the given class.
	 * For example, pkgToPath(org.eso.ohs.p2pp.P2PP.class) = "org/eso/ohs/p2pp"
	 * 
	 * @param cls
	 *            Some class contained in an explicit package.
	 * 
	 * @return relative path to the directory containing the class
	 */
	public static String pkgToPath(Class cls) {
		String path = cls.getName();
		int index = path.lastIndexOf('.');

		if (index < 0) {
			return ""; // Not contained in an explicit package
			// XXX Should throw IllegalArgumentException???
		}

		path = path.substring(0, index);

		while ((index = path.indexOf('.')) > 0) {
			path = path.substring(0, index) + File.separator
					+ path.substring(index + 1);
		}
		return path + File.separator;
	}

	/**
	 * Accept a file as input and return a strining with the content of the file.
	 * 
	 * @param f
	 *            File
	 * @return String
	 */
	public static String convertFileToString(File f) {
		char[] buff = new char[64000];
		StringBuffer sb = new StringBuffer();
		try {
			BufferedReader br = new BufferedReader(new FileReader(f));
			while (br.ready()) {
				int nread = br.read(buff, 0, buff.length);
				if (nread <= 0) {
					break;
				}
				sb.append(buff, 0, nread);
			}
			br.close();
		} catch (Throwable t) {
			sb.append("Failed to read file contents.");
		}
		return sb.toString();
	}

	/**
	 * Convert a string to an array of Integer.
	 * 
	 * @param s
	 *            the origin
	 * @param sep
	 * @return Integer[]
	 */
	public static Integer[] stringToIntObjArray(String s, String sep) {
		// convert a String s to an Array, the elements

		// are delimited by sep
		int[] intArray = stringToIntArray(s, sep);
		Integer[] intObjArray = new Integer[intArray.length];
		for (int i = 0; i < intArray.length; i++) {
			intObjArray[i] = new Integer(intArray[i]);
		}
		return intObjArray;
	}

	/**
	 * Method StringtoArray. convert a string to an array of strings. If trim is
	 * true it cuts the blanks or tab, an empty string returns an empty array (0
	 * lenght).
	 * 
	 * The behavior when the separator is a single blank is a bit triky. 
	 * Any string with N blanks  with trim flag false, 
	 * the result is of [N+1] elements, some empty if there are more blanks together.
	 * If the trim is true, the result is of [J] elements.
	 * J being the number of words in the string, 
	 * therefore the number of empty elements will be 0.	
	 * @param s
	 *            the origin
	 * @param sep
	 *            separator
	 * @param trim
	 *            boolean
	 * @return String[]
	 * @throws ObjectIOException 
	 */
	public static String[] stringToStringArray(String s, String sep,
			boolean trim)  {
		// convert a String s to an Array, the elements
		// are delimited by sep
        String[] elements = new String[0];
        if (sep == null || sep.length() != 1) {
            stdlog_.error("The separator can be only of one char. Instead: "+sep + ". Returning empty String"); 
            return elements;
        }
		
		if (s != null) {	 
			
            StringBuffer buf = new StringBuffer(s);
			int arraysize = 1;
			for (int i = 0; i < buf.length(); i++) {
				if (sep.indexOf(buf.charAt(i)) != -1)
					arraysize++;
			}
           
			elements = new String[arraysize];
			int y, z = 0;
			if (buf.toString().indexOf(sep) != -1) {
				while (buf.length() > 0) {
					if (buf.toString().indexOf(sep) != -1) {
						y = buf.toString().indexOf(sep);
						if (y != buf.toString().lastIndexOf(sep)) {
							elements[z] = buf.toString().substring(0, y);
							z++;
							buf.delete(0, y + 1);
						} else if (buf.toString().lastIndexOf(sep) == y) {
							elements[z] = buf.toString().substring(0,
									buf.toString().indexOf(sep));
							z++;
							buf.delete(0, buf.toString().indexOf(sep) + 1);
							elements[z] = buf.toString();
							z++;
							buf.delete(0, buf.length());
						}
					}
				}
			} else {
				elements[0] = buf.toString();
			}
			buf = null;
			if (trim) {
				elements = trimStringArray(elements);
			   if (sep.equals(" ")) {
			       Vector<String> subArray = new Vector<String>();
			        for (int i = 0; i < elements.length; i++) {
			            if (elements[i].length() > 0) {
			            subArray.addElement(elements[i]);
			            }
			        }
			        elements = (String[]) subArray.toArray(new String[subArray.size()]);
			   }
			}
		}
		return elements;
	}

    /** 
     * Break a multi-line string into an array of strings; it is assumed
     * that lines are separated by a single new-line char.
     *  
     * @return An array of strings, each element corresponding to a
     *         line of the original text.
     * 
     * @see #stringToStringArray(String, String, boolean); this method 
     *      uses a simpler algorithm
     */
    // amchavan, 09-Jun-2005: added
    public static String[] toLines( String text ) {
        Vector<String> v = new Vector<String>();
        String line = EMPTY;

        StringTokenizer st = new StringTokenizer( text, "\n", true );
        while (st.hasMoreTokens()) {
            
            String token = st.nextToken();  // get a token
            
            if( token.equals( NL )) {       // is it end-of-line?
                v.add( line );              // YES, save the line itself...
                line = EMPTY;               //      ...and reset it
                continue;
            }
            
            line = token;                   // NO, remember this line next time
        }
        
        if( line != EMPTY )                 // any leftover lines?
            v.add( line );                  // YES, save that
            
        String[] ret = (String[]) v.toArray( new String[0] );
        return ret;
    }

    /** 
     * @return The number of lines in the input text; it is assumed
     *         that lines are separated by a single new-line char.
     * 
     * @see #toLines(String)
     */
    // amchavan, 09-Jun-2005: added 
    public static int countLines( String text ) {
        // very simple minded implementation, should count newlines instead
        String[] sa = toLines( text );
        return sa.length;
    }

    /**
	 * Convert a string to an array of int
	 * 
	 * @param s
	 *            String
	 * @param sep
	 *            String
	 * @return int[]
	 */
	public static int[] stringToIntArray(String s, String sep) {
		String[] stringArray = stringToStringArray(s, sep, true);
		int[] intArray = new int[stringArray.length];

		for (int i = 0; i < stringArray.length; i++) {
			intArray[i] = Integer.parseInt(stringArray[i]);
		}
		return intArray;

	}

	/**
	 * Convert an array of float to an array of String
	 * 
	 * @param floatArray
	 *            float[]
	 * @return String[]
	 */
	public static String[] floatArrayToStringArray(float[] floatArray) {
		String[] stringArray = new String[floatArray.length];
		for (int i = 0; i < floatArray.length; i++) {
			stringArray[i] = String.valueOf(floatArray[i]);
		}
		return stringArray;

	}

	/**
	 * Trims all the elements of a String array.
	 * 
	 * @param elements
	 *            String[]
	 * @return String[]
	 */
	private static String[] trimStringArray(String[] elements) {

		for (int i = 0; i < elements.length; i++) {
			if (elements[i] != null) {
				elements[i] = elements[i].trim();
			}
		}
		// if trim, an ampty string returns an empty array
		if (elements.length == 1 && elements[0].length() == 0) {
			elements = new String[0];
		}
		return elements;
	}

	/**
	 * Converts back an array into a string
	 * In case the trim argument is ture, the elements are trimmed.
	 * 
	 * @param s
	 * @param sep
	 * @param trim
	 *            boolean
	 * @return String
	 */
	public static String arrayToString(String s[], String sep, boolean trim) {
		int  lenthArray = 0;
		StringBuffer result = new StringBuffer("");
		if (s != null) {
            lenthArray = s.length;
		}
        if (lenthArray >0) {
            String firstString = s[0];
            if (trim && firstString!= null) {          
                result.append(firstString.trim());
               } else {
                   result.append(firstString); 
               }
			for (int i = 1; i < lenthArray; i++) {
                String currentString = s[i];
				result.append(sep);
                if (trim && currentString!= null) {          
                 result.append(currentString.trim());
                } else {
                    result.append(currentString); 
                }
			}
        }
		return result.toString();
	}
	
	
	/**
	 * Converts an array of Integer into a string, giving a separator.
	 * 
	 * @param s
	 * @param sep
	 * 
	 * @return String
	 */
	public static String arrayOfIntegersToString(Integer s[], String sep) {
		int k;
		String result = "";
		if (s == null) {
			k = 0;
		} else {
			k = s.length;
		}
		if (k > 0) {
			result = s[0].toString();
			for (int i = 1; i < k; i++) {
				result += sep + s[i];
			}
		}
		return result;
	}
	
	

	/**
	 * Join two string arrays.
	 * 
	 * @param a
	 * @param b
	 * @return String[]
	 */
	public static String[] concatenateStringArrays(String[] a, String[] b) {

		int arrayLength = a.length + b.length;
		int resultIndex = 0;
		String[] resultArray = new String[arrayLength];

		for (int i = 0; i < a.length; i++) {
			resultArray[resultIndex] = a[i];
			resultIndex++;
		}
		for (int j = 0; j < b.length; j++) {
			resultArray[resultIndex] = b[j];
			resultIndex++;
		}

		return resultArray;
	}

	/**
	 * Split a string in chunks of "size" characters and fill in a string array.
	 * 
	 * @param s
	 *            String
	 * @param size
	 *            int
	 * @return String[]
	 */
	public static String[] splitString(String s, int size) {
		Vector<String> strList = new Vector<String>();
		for (int i = 0; i < s.length(); i += size) {
			String substr;
			if (i + size > s.length()) {
				substr = s.substring(i);
			} else {
				substr = s.substring(i, i + size);
			}
			strList.addElement(substr);
		}
		String[] splitLines = new String[strList.size()];
		strList.copyInto(splitLines);
		return splitLines;
	}

	/***************************************************************************
	 * public static void main(String[] args) {
	 * pkgToPath(org.eso.ohs.dfs.ObservationBlock.class);
	 * pkgToPath(org.eso.ohs.icdVcs.BOB.class);
	 * pkgToPath(org.eso.ohs.gui.widgets.LogWindow.class); }
	 * 
	 * @param in
	 *            String[]
	 * @return String[] /
	 **************************************************************************/

	/**
	 * Sort the strings and return them (in the same array as passed in).
	 * 
	 * @param the
	 *            array of Strings to be sorted.
	 * @return the sorted String array
	 */

	public static String[] sort(String[] in) {
		Vector<String> out = new Vector<String>();
		for (int i = 0; i < in.length; i++) {
			String str = in[i];
			int o = 0;
			for (; o < i; o++) {
				if (str.compareTo(out.elementAt(o)) <= 0) {
					break;
				}
			}
			if (o == i) {
				out.addElement(str);
			} else {
				out.insertElementAt(str, o);
			}
		}
		for (int i = 0; i < in.length; i++) {
			in[i] = out.elementAt(i);
		}
		return in;
	}

	/**
	 * Sort the array of Sortable objects, and return them (in the same array as
	 * passed in).
	 * 
	 * @param in
	 *            array of Sortable objects to be sorted.
	 * @return the sorted array of Sortable objects.
	 */

	public static TextUtils.Sortable[] sort(TextUtils.Sortable[] in) {
		Vector<Sortable> out = new Vector<Sortable>();
		for (int i = 0; i < in.length; i++) {
			String str = in[i].getSortKey();
			int o = 0;
			for (; o < i; o++) {
				if (str.compareTo(out.elementAt(o).getSortKey()) <= 0) {
					break;
				}
			}
			if (o == i) {
				out.addElement(in[i]);
			} else {
				out.insertElementAt(in[i], o);
			}
		}
		for (int i = 0; i < in.length; i++) {
			in[i] = out.elementAt(i);
		}
		return in;
	}

	/**
	 * Sort the array of Sortable objects, and return them (in the same array as
	 * passed in).
	 * 
	 * @param in
	 *            array of SortableNumber objects to be sorted.
	 * @return the sorted array of Sortable objects.
	 */

	public static TextUtils.SortableNumber[] sort(TextUtils.SortableNumber[] in) {
		Vector<SortableNumber> out = new Vector<SortableNumber>();
		for (int i = 0; i < in.length; i++) {
			double val = in[i].getSortKey();
			int o = 0;
			for (; o < i; o++) {
				if (val <= (out.elementAt(o).getSortKey())) {
					break;
				}
			}
			if (o == i) {
				out.addElement(in[i]);
			} else {
				out.insertElementAt(in[i], o);
			}
		}
		for (int i = 0; i < in.length; i++) {
			in[i] = out.elementAt(i);
		}
		return in;
	}

	/**
	 * Interface to be implemented in order to allow sorting vie the sort()
	 * routine above.
	 * 
	 * @author ddorigo
	 */

	public static interface Sortable {
		/**
		 * Method getSortKey
		 * 
		 * @return String
		 */
		public String getSortKey();
	}

	/**
	 * Interface to be implemented in order to allow sorting vie the sort()
	 * routine above.
	 * 
	 * @author ddorigo
	 */

	public static interface SortableNumber {
		/**
		 * Method getSortKey
		 * 
		 * @return double
		 */
		public double getSortKey();
	}

	/**
	 * Checks if the first characters of the input string are ASCII printable.
	 * 
	 * @param s
	 *            Input string.
	 * @param maxn
	 *            Check is performed only on the first maxn chars.
	 * 
	 * @return The index of the first non-ASCII printable character, if if was
	 *         found among the first characters in the input string, -1
	 *         otherwise.
	 * 
	 * @see #isASCIIPrintable( char )
	 */
	public static int isASCIIPrintable(String s, int maxn) {

		int testl = Math.min(s.length(), maxn);

		for (int i = 0; i < testl; i++)
			if (!isASCIIPrintable(s.charAt(i)))
				return (i);

		return (-1);
	}

	/**
	 * Checks if the input string only contains ASCII printable characters.
	 * 
	 * @param s
	 *            Input string.
	 * 
	 * @return The index of the first non-ASCII printable character, if it was
	 *         found in the input string, -1 otherwise.
	 * 
	 * @see #isASCIIPrintable( char )
	 */
	public static int isASCIIPrintable(String s) {
		return (isASCIIPrintable(s, s.length()));
	}

	/**
	 * Checks if the specified character is an ASCII printable character.
	 * 
	 * @param c
	 *            Input character.
	 * 
	 * @return <em>true</em> for whitespace characters (HORIZONTAL TABULATION,
	 *         NEW LINE, FORM FEED, CARRIAGE RETURN, SPACE) and for characters
	 *         whose ASCII (Unicode) value is between 0x21 ('!') and 0x7E ('~');
	 *         <em>false</em> otherwise.
	 */

	public static boolean isASCIIPrintable(char c) {

		int code = (int) c;

		if (code >= 0x20 && code < 0x7f) // SPACE and 'normal' chars
			return true;
		if (code == 0x09 || // HORIZONTAL TABULATION
				code == 0x0a || // NEW LINE
				code == 0x0c || // FORM FEED
				code == 0x0d) // CARRIAGE RETURN

			return true;
		return (false);
	}

	/**
	 * Insert newline chars into the input string s, so that no resulting line
	 * is longer than n chars -- if possible. For instance, breaking up <BR>
	 * <tt>abc abc abcd abcd abcde abcde 0123456789123 abcdef</tt><BR>
	 * with a limit of 10 chars will result in <BR>
	 * <tt>abc abc\nabcd abcd\nabcde\nabcde\n0123456789123\nabcdef</tt>
	 * 
	 * <br>
	 * ChangeLog: amchavan, 08-Jan-2002: added
	 * 
	 * @param s
	 *            String
	 * @param n
	 *            int
	 * @return String
	 */
	public static String breakString(String s, int n) {

		if (s == null)
			return s;

		int l = s.length();

		if (l <= n)
			return s;

		int p = s.lastIndexOf(" ", n - 1); // look for the last space

		if (p == -1) { // did we find it?

			// we could not break within
			// the first n chars: let's
			// break at the next blank

			p = s.indexOf(' '); // is there another blank?
			if (p == -1)
				return s; // not really, give up
		}

		return s.substring(0, p) + '\n' + breakString(s.substring(p + 1, l), n);
	}
}