package org.eso.ohs.core.utilities.hibernate;



/**
 * This is a custom class to override hibernates default 
 * mechanisms to reverse engineer code.  
 * 
 * Most of the tables on the opc70/obrep2 databases are pluralised. 
 * but we would like to generate classes which are not pluralised.  
 */

import org.hibernate.cfg.reveng.DelegatingReverseEngineeringStrategy;
import org.hibernate.cfg.reveng.ReverseEngineeringStrategy;
import org.hibernate.cfg.reveng.TableIdentifier;


public class ReverseEngineeringUssStrategy extends DelegatingReverseEngineeringStrategy
		 {
	
	/**
	 * Default constructor
	 * @param delegate
	 */
    public ReverseEngineeringUssStrategy(ReverseEngineeringStrategy delegate) {
    	super(delegate);
    }
    
    
    
    /**
     * Most of the tables on the opc70/obrep2 databases are pluralised. 
     * but we would like to generate classes which are not pluralised.       
     */
    
    public String tableToClassName(TableIdentifier tableIdentifier) {
    	String className =  super.tableToClassName(tableIdentifier);
    	int strLen = className.length();
    	if(strLen > 2 && (className.indexOf("s",strLen-1)!=-1
    	|| className.indexOf("S",strLen-1)!=-1)) {
    		className = className.substring(0,strLen-2);
    	}
    	return className;
    }
    
}
