package org.eso.ohs.dfs;

//import org.eso.ohs.gui.docview.*;
import java.util.Hashtable;
import java.util.Vector;

import org.eso.ohs.core.utilities.AssocList;

/**
 * Record in the Masks database.
 * 
 * <p>
 * $Id: Mask.java,v 1.30 2006/04/27 10:37:43 tcanavan Exp $
 */
public class Mask extends BusinessObject {


   
    private static final long serialVersionUID = 1L;
    public static final String MANUFACTURING_ORDER = "Manufacturing Order";
    public static final String INSERTION_ORDER = "Insertion Order"; 
    public static final String DELETION_ORDER = "Discarding Order";
    public static final int MANUFACTURING_ORDER_ID = 0;
    public static final int INSERTION_ORDER_ID = 1;
    public static final int  DELETION_ORDER_ID = 2;



    /**set to -99, as per Bianca Garilli's request. */

    public static final String DEFAULT_MASK_ID = "-99";


    /** set to -99, as per Bianca Garilli's request. */

    public static final int DEFAULT_MASK_SLOT_NUMBER = -99;


    public static Hashtable<String, Integer> orders = new Hashtable<String, Integer>();

    static {
       orders.put(MANUFACTURING_ORDER,new Integer(MANUFACTURING_ORDER_ID));
       orders.put(INSERTION_ORDER,new Integer(INSERTION_ORDER_ID));
       orders.put(DELETION_ORDER,new Integer(DELETION_ORDER_ID)); 
    }
        

    public static final int MAX_TS_PER_OD    = 255;

    public static final int MAX_PARAM_PER_TS = 255;


    /**
     * The ob associated with a given mask
     */

    private long OBId_;

    /**
     * The template signature containing the mask data
     */

    private int tsIndex_;

    /**
     * The parameter index which defined where the parameter 
     * is the parameter array
     */

    private int paramIndex_;


    /**
     * This is a string in the PAF file which is used for Mask Tracking
     */

    private String maskNAID_;

    /**
     * The id of a give Mask. See Maurizio Document on Mask Tracking for details
     */

    private String maskId_;

    /**
     * The slot in the cabinet where the mask should be placed
     */

    private int slotNum_;


    /**
     * The admFile as a String 
     */

    private String admFile_ ;
    /**
     * What status the mask is in. Is it manufactured,Inserted,Deleted etc..
     * 
     */

    private String status_ = NO_MASK;


    /**
     * The Mask has not been Manufactured yet. It is a virtual entity
     */

    public static String NO_MASK = "Z";



     
    /**
     * Mask Manufacturing report has been sent to BOB.
     *
     */
    public static String PENDING_MANUFACTURED = "Q";



    public static String MASK_REMANUFACTURE = "R"; 


    /**
     * Mask is Manufactured
     */

    public static String MASK_MANUFACTURED = "M";


    /**
     * Mask is pending insertion
     */

    public static String PENDING_INSERTION = "K";


    /**
     * Mask Insertion report succeddfully processed
     *
     */

    public static String MASK_INSERTED = "I";

    

    /**
     * Mask Discrding report has been sent to BOB
     */

    public static String PENDING_DISCARDED = "G";

    /**
     * Mask Discarding report has been received from BOB
     */

    public static String MASK_DISCARDED = "D";   


   /**
     * Mappings from the status field value to the displayed text
     */
    private static final AssocList<String,String> statusValues_
	= new AssocList<String,String>()
	.add("NoMask",NO_MASK)
        .add("PendingManufacturing",PENDING_MANUFACTURED)
	.add("MaskStored",MASK_MANUFACTURED)
	.add("MaskRemanufacture",MASK_REMANUFACTURE)
        .add("PendingInsertion",PENDING_INSERTION)
	.add("MaskInserted",MASK_INSERTED)    
        .add("PendingDiscarding",PENDING_DISCARDED)
        .add("MaskDiscarded",MASK_DISCARDED);

    private static final Hashtable<String,String> maskStatusNotEditable_ = new Hashtable<String,String>();
    
    static {
        maskStatusNotEditable_.put("NoMask",NO_MASK);
    }

    public static AssocList getStatusValues() {
        return statusValues_;
    }


    public static String getDisplayStatusValue(String status) {
         return statusValues_.getKey(status);
    }


    private String maskName_;


    /**
     * Default constructor for Mask
     */

    public Mask() {
    }


    /**
     * Combine the ids to produce a Mask Id for the object
     * @return Id for the mask using the combined ids of the ob,ts,param
     */

    public static long combineIds(long obId, int tsIndex, int paramIndex) {
        return obId + (((long)tsIndex) << 40) + (((long)paramIndex) << 48);
    }



    public void setMaskName(String maskName) {
        maskName_ = maskName;
    }


    public String getMaskName() {
        return maskName_;
    }



    public static String [] getEditableMaskDisplayStatuses() {
       String[] displayed = Mask.getStatusValues().keys();
       Vector<String> retDisplay = new Vector<String>();
       for(int i=0;i<displayed.length;i++) {
           if(!maskStatusNotEditable_.containsKey(displayed[i])) {
               retDisplay.addElement(displayed[i]);
           }
       }
       displayed = new String[retDisplay.size()];
       retDisplay.copyInto(displayed);
       return displayed;
     } 

   
     public static String [] getEditableMaskStatuses() {
       String[] displayed = Mask.getStatusValues().keys();
       Vector <String>retStatus = new Vector<String>();
       for(int i=0;i<displayed.length;i++) {
           if(!maskStatusNotEditable_.containsKey(displayed[i])) {
               retStatus.addElement(statusValues_.get(displayed[i]));
           }
       }
       displayed = new String[retStatus.size()];
       retStatus.copyInto(displayed);
       return displayed;
     } 

    /**
     * Extract the obid from the combined id
     * @return obid from combined id     
     */
    

    public static long extractOBId(long combinedId) {
        return combinedId & 0xFFFFFFFFFFL;
    }

    /**
     * Extract the tsid from the combined id
     * @return tsid from combined id     
     */


    public static int extractTSIndex(long combinedId) {
        return (int) (combinedId >> 40) & 0xFF;
    }


    /**
     * Extract the param id from the combined id
     * @return param id from combined id     
     */

    public static int extractParamIndex(long combinedId) {
        return (int) ((combinedId >> 48) & 0xFF);
    }


    /**
     * Return the status of the Mask
     * @return Mask Status whether it is Manufactured,Inserted etc..
     */

    public String getStatus() {
        return status_;
    }


    /**
     * Set the status of the Mask
     * @param status Mask Status String
     */

    public void setStatus(String status) {
        status_ = status;
    }


    /**
     * Set value for OBId_ property
     *
     * @param new value of the property
     */
    public void setOBId(long newValue) {
	OBId_ = newValue;
    }


    /**
     * Get value for OBId_ property
     *
     * @return current value of the property
     */
    public long getOBId() {
	return OBId_;
    }


    /**
     * Get value for tsIndex_ property
     *
     * @return the current value of the property
     */
    public void setTSIndex(int newValue) {
	tsIndex_ = newValue;
    }


    /**
     * Get value for tsIndex_ property
     *
     * @return the current value of the property
     */
    public int getTSIndex() {
	return tsIndex_;
    }


    /**
     * Set value for paramIndex_ property
     *
     * @return the current value of the property
     */
    public void setParamIndex(int newValue) {
	paramIndex_ = newValue;
    }


    /**
     * Get value for paramIndex_ property
     *
     * @return the current value of the property
     */
    public int getParamIndex() {
	return paramIndex_;
    }


    /**
     * Set new value for maskNAID_ property
     *
     * @param newValue the new value for the property
     */
    public void setMaskNAID(String newValue) {
	    maskNAID_ = newValue;
	    firePropertyChangeEvent();
    }


    /**
     * Get value for maskNAID_ property
     *
     * @return the current value of the property
     */
    public String getMaskNAID() {
	    return maskNAID_;
    }


    /**
     * Set new value for maskId_ property
     *
     * @param newValue the new value for the property
     */
    public void setMaskId(String newValue) {
	    maskId_ = newValue;
	    firePropertyChangeEvent();
    }


    /**
     * Get value for maskId_ property
     *
     * @return the current value of the property
     */
    public String getMaskId() {
	    return maskId_;
    }


    /**
     * Set new value for slotNum_ property
     *
     * @param newValue the new value for the property
     */
    public void setSlotNum(int newValue) {
	    slotNum_ = newValue;
	    firePropertyChangeEvent();
    }


    public void setADMFile(String admFile) {
        admFile_ = admFile; 
        firePropertyChangeEvent();
    }

    
    public String getADMFile() {
        return admFile_;
    }



    /**
     * Get value for slotNum_ property
     *
     * @return the current value of the property
     */
    public int getSlotNum() {
	    return slotNum_;
    }

    public Object accept(BusinessVisitor v, Object arg) {
	return null;
    }
    /**
     * @see org.eso.ohs.dfs.StorableObject#postProecess()
     */
    public void postProecess() {
    }


}


