package org.eso.ohs.dfs;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.instruments.TemplateSignatureFactory;

/**
 * 
 * 
 * <p>
 * $Id: ObservationDescription.java,v 1.43 2006/04/26 09:26:49 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * <br>ChangeLog: cmavros 15-10-01 Added a space between Observation and Description. 
 */

public class ObservationDescription extends BusinessObject {

    /** Standard log channel */
    private static Logger stdlog_ =
        Logger.getLogger(ObservationDescription.class);

    /** Version id for object serialization */
    static final long serialVersionUID = -1929856469897387244L;

    /** Comments attached to this OD by its author */
    private String userComments_;

    /** The instrument mode associated with this OD */
    private String mode_;

    /** The instrument associated with this OD */
    private String instrument_;

    /**
     * Version of the instrument package used to construct this OB:
     * this is the value of the <code>VERSION</code> keyword of the
     * ISF.
     */
    private float ipVersion_ = -1;

    /** The list of TemplateSignatures associated with this OD. */
    private Vector<TemplateSignature> templSigVec_;

    /**
     * Null constructor for ObservationDescription.
     * Initialises templSigVec_ only.
     */
    ObservationDescription() {
        templSigVec_ = new Vector<TemplateSignature>();
    }

    /**
     * Set new value for userComments_ property
     *
     * @param newValue the new value for the property
     */
    public void setUserComments(String newValue) {
        userComments_ = newValue;
        firePropertyChangeEvent();
    }

    /**
     * Get value for userComments_ property
     *
     * @return the current value of the property
     */
    public String getUserComments() {
        if (userComments_==null)
            return("");
        else
            return userComments_;
       
    }

    /**
     * Set new value for mode_ property
     *
     * @param newValue the new value for the property
     */
    public void setMode(String newValue) {
        mode_ = newValue;
        firePropertyChangeEvent();
    }

    /**
     * Get value for mode_ property
     *
     * @return the current value of the property
     */
    public String getMode() {
        if (mode_ == null)
            return("");
        else
            return mode_;
    }

    /**
     * Get value for instrument_ property
     *
     * @return the current value of the property
     */
    public String getInstrument() {
        if (instrument_==null)
            return("");
        else
            return instrument_;
    }

    /**
     * Set new value for instrument_ property
     *
     * @param newValue the new value for the property
     */
    public void setInstrument(String newValue) {
        instrument_ = newValue;
        firePropertyChangeEvent();
    }

    public void setInstrument(String newValue, float version) {
        stdlog_.debug(
            "MIPS OD: setInstrument:" + newValue + " v" + version);
        instrument_ = newValue;
        ipVersion_ = version;
        firePropertyChangeEvent();
    }

    /**
     * Set new value for ipVersion_ property
     *
     * @param newValue the new value for the property
     */
    public void setIPVersion(float v) {
        stdlog_.debug("MIPS OD: setIPVersion: v" + v);
        ipVersion_ = v;
        firePropertyChangeEvent();
    }

    /**
     * Set the ipVersion_ property to the value of the <code>VERSION</code>
     * keyword of the Instrument Summary File.
     *
     * @see #ipVersion_
     * <br>ChangeLog: amchavan, 22-Oct-2001: Creation.
     */
    /*
    public void setIPVersion() {
        String instName = getInstrument();
        Instrument inst = InstrumentList.getInstance().getInstrument( instName );
        ipVersion_ = inst.getVersionNumber();
    }
    */
    /**
     * Get value for ipVersion_ property
     *
     * @return The current value of the property
     * @see #ipVersion_                              
     */
    public float getIPVersion() {
        stdlog_.debug("MIPS  OD: getIPVersion:" + ipVersion_);
        return ipVersion_;
    }

    public void setSignatures(TemplateSignature[] templSigs) {

        templSigVec_.removeAllElements();
        for (int i = 0; i < templSigs.length; i++) {
            templSigVec_.addElement(templSigs[i]);
        }
        stdlog_.debug("Initialising TemplateSigVec_ " + templSigVec_.size());
        firePropertyChangeEvent(templateEvent_);
    }

    /**
     * Appends the specified template signature to the end of
     * the list associated with this OD.
     * 
     * @param the template signature to be appended.
     */
    public void appendSignature(TemplateSignature ts) {
        templSigVec_.addElement(ts);
        firePropertyChangeEvent(templateEvent_);
    }

    /**
     * Inserts the specified template signature into the list
     * associated with this OD.
     * 
     * @param the template signature to be inserted.
     */
    public void insertSignature(TemplateSignature ts, int index) {
        templSigVec_.insertElementAt(ts, index);
        firePropertyChangeEvent(templateEvent_);
    }

    /**
     * Deletes the specified template signature from the list
     * associated with this OD.
     * 
     * @param the template signature to be deleted.
     */
    public void deleteSignature(TemplateSignature ts) {
        templSigVec_.removeElement(ts);
        firePropertyChangeEvent(templateEvent_);
    }

    /**
     * Deletes the specified template signature from the list
     * associated with this OD.
     * 
     * @param the template signature to be deleted.
     */
    public void removeAllSignatures() {
        templSigVec_.removeAllElements();
        firePropertyChangeEvent(templateEvent_);
    }

    /**
     * Returns the current list of template signatures associated with
     * this OD.
     * 
     * @return the template signatures associated with this OD.
     */
    public TemplateSignature[] getSignatures() {
        int size = templSigVec_.size();
        TemplateSignature[] result = new TemplateSignature[size];
        templSigVec_.copyInto(result);
        return result;
    }

    /**
     * Returns the string used to refer to this type of BusinessObject
     * in messages to the user.
     *
     * @return user-oriented name
     */
    public static String getDisplayName() {
        return "ObsDesc";
    }

    /**
     * Returns the filename suffix for files containing this type of object.
     *
     * @return suffix for filename when stored in local filesystem
     */
    public static String getSuffix() {
        return "od";
    }

    /**
     * Copies the value of the specified property of this object to
     * the given BusinessObject.
     * 
     * @param propName	    property name to copy
     * @param destBO	    destination for the copy operation
     */
    public void copyProperty(String propName, BusinessObject destBO)
        throws
            NoSuchMethodException,
            IllegalAccessException,
            InvocationTargetException {
        ObservationDescription od = (ObservationDescription) destBO;

        if (propName.equals("Signatures")) {
            TemplateSignature[] tsigs = getSignatures();
            TemplateSignature[] newTsigs = new TemplateSignature[tsigs.length];

            od.templSigVec_ = new Vector<TemplateSignature>(); // Avoid sharing old vector!!

            for (int j = 0; j < tsigs.length; j++) {
                newTsigs[j] = new TemplateSignature(tsigs[j]);
            }

            od.setSignatures(newTsigs);
            return;
        }

        super.copyProperty(propName, destBO);
    }

    /**
     * Verifies this block and all attached blocks, returning the
     * concatenation of all error messages.
     * 
     * @return the (possibly empty) concatenation of all error messages.
     */
    public String verify() {
        StringBuffer sb = new StringBuffer();

        TemplateSignature[] tsigs = getSignatures();
        int numErrs = 0;

        if (tsigs.length == 0) {
            return "\n There are no Template Signatures"
                + " in the Observation Description.";
        }

        for (int j = 0; j < tsigs.length; j++) {
            String[] errs = tsigs[j].verify();

            for (int k = 0; k < errs.length; k++) {
                if (!errs[k].equals("OK")) {
                    numErrs++;
                }
            }
        }

        if (numErrs > 0) {
            sb.append("\n");
            if (numErrs == 1) {
                sb.append(" There is 1 error");
            } else {
                sb.append(" There are " + numErrs + " errors");
            }
            sb.append(" in the Observation Description.");

        }

        return sb.toString();
    }

    public String verifyBO() {
        return verify();
    }


    /**
     * MIPS Project:
     * Set the ipversion of all templates in this object. Not, the "conversion"
     * to the new template does not occur until refreshTemplates() is called.
     */
    public void convertIPVersion(float ipv) {
        for (int i = 0; i < templSigVec_.size(); i++) {
            TemplateSignature ts =
                templSigVec_.elementAt(i);
            ts.setVersion(ipv);
        }
        setIPVersion(ipv);
    }

    /**
     * MIPS Project:
     * Reload the templates (converting as required to a new IP version).
     * The firePropChangeEvent indicates if a property change event should
     * be fired as a result of the change (this will trigger the autosave).
     */
    
    public void refreshTemplates(boolean firePropChangeEvent) throws IOException {
        refreshTemplates(firePropChangeEvent, false);        
    }

    public void refreshTemplates(boolean firePropChangeEvent, boolean fullRefresh)
        throws IOException {
        for (int j = templSigVec_.size() - 1; j >= 0; j--) {
            TemplateSignature ts =
                templSigVec_.elementAt(j);
            if (!TemplateSignatureFactory.refreshTemplate(ts, fullRefresh)) {
                templSigVec_.removeElementAt(j);
            }
        }

        if (firePropChangeEvent) {
            firePropertyChangeEvent();
        }
    }

    /**
     * Read varianle ob details from local storage
     * @param in Stream to capture data from cache
     * @exception IOException IO Exception if record cannot be read from disk
     * @exception ClassNotFoundException class not found exception
     */

    private void readObject(java.io.ObjectInputStream in)
        throws IOException, ClassNotFoundException {

        in.defaultReadObject();

        /*
         *   >>>>>>>>>>> ALERT <<<<<<<<<<<<<
         *
         * THESE FEW LINES ARE A DESPERATE ACTION TO RESPOND TO ITEM 1293 FROM THE TO-DO LIST.
         * WE NEED A WAY TO COMPLETE THE VALUE FOR THE INSTRUMENT IF IT IS MISSING DUE TO A
         * CORRUPTED CACHE. 
         */
        TemplateSignature ts0 = null;

        if (!templSigVec_.isEmpty())
            ts0 = templSigVec_.elementAt(0);
        else
            stdlog_.debug(
                "CLASS ObservationDescription - readObject: No Template");

        if (instrument_ == null && ts0 != null) {
            instrument_ = ts0.getInstrumentName();
        }

        /*
         *   >>>>>>>>>> END ALERT <<<<<<<<<<
         */
        // check if the instrument package exists to load the templates.
        if (InstrumentList.getInstance().getInstrument(getInstrument(), getIPVersion()) == null) {
            throw new IOException("Object could not be loaded as IP "+getInstrument()+" v"+getIPVersion()+" was not found.");
        }
        refreshTemplates(false);
    }

    /**
     * Returns the list of field for this object.
     * Used by toString().
     * 
     * @return the fields of this object as a string.
     */
    public String paramString() {
        return super.paramString()
            + ",instrument="
            + instrument_
            + ", version="
            + ipVersion_
            + ",mode="
            + mode_;
    }

    /**
     * Generic hook for "visitor" Design Pattern.
     * See the BusinessVisitor interface for more details.
     * 
     * @param visitor	the class whose visit() method we wish to invoke.
     * 
     * @see BusinessVisitor
     */
    public Object accept(BusinessVisitor visitor, Object arg) {
        return visitor.visit(this, arg);
    }
    /**
     * @see org.eso.ohs.dfs.StorableObject#postProecess()
     */
    public void postProecess() {
    }

    // to be put into BusinessObject later;
    
    
    
    /**
     * Compares two <code>ObservationDescription</code> objects
     * 
     * @param other
     * @return boolean
     */
    public boolean equals(ObservationDescription other) {
        Vector<TemplateSignature> templates1 , templates2;
        
        if (this.instrument_.compareTo(other.getInstrument()) != 0 )
            return(false);
        
        if (this.userComments_.compareTo(other.getUserComments()) != 0 )
            return(false);
        
        templates1 = templSigVec_;
        templates2 = other.templSigVec_;
        
        // compare the two vectors of templates, order matters        
        
        // different number of templates
        if ( templates1.size() != templates2.size())
            return(false);
                
        // equal number, have to be equal side-to-side
        Iterator<TemplateSignature> iter2 = templates2.iterator(); 
        
        for (Iterator<TemplateSignature> iter = templates1.iterator(); iter.hasNext();) {
            TemplateSignature ts1 = iter.next();
            TemplateSignature ts2 = iter2.next();
            
            if (! ts1.equals(ts2))
                return(false);
            
        }
        
        return(true);
        
    }

}
