package org.eso.ohs.dfs;


/**
 * An adaptor class for the BusinessVisitor interface, analogous to
 * MouseListener and MouseAdaptor in java.awt.event.
 * Simplifies the common case whereby most of the visit() methods
 * actually do the same thing.  In the adaptor, all visit() methods
 * call commonVisitor() and only exceptional cases need to be overridden.
 * 
 * <p>
 * <em>NOTE:</em> when you add a method for a new type of BusinessObject,
 * be sure to check all subclasses behave correctly!  Perhaps some of
 * them require the method to be overridden and not call commonVisitor().
 * 
 * <p>
 * $Id: VisitorAdaptor.java,v 1.5 2006/02/16 10:21:21 gdonino Exp $
 * <br>
 * @author Bob Kemp
 */
public class VisitorAdaptor implements BusinessVisitor {

    /** Visitor method for a CalibrationBlock object */
    public Object visit(CalibrationBlock bo, Object arg) {
	return commonVisitor(bo, arg);
    }

    /** Visitor method for an ObservationBlock object */
    public Object visit(ObservationBlock bo, Object arg) {
	return commonVisitor(bo, arg);
    }

    /** Visitor method for an ObservationDescription object */
    public Object visit(ObservationDescription bo, Object arg) {
	return commonVisitor(bo, arg);
    }

    /** Visitor method for an ObservingRun object */
    public Object visit(ObservingRun bo, Object arg) {
	return commonVisitor(bo, arg);
    }

    /** Visitor method for an Target object */
    public Object visit(Target bo, Object arg) {
	return commonVisitor(bo, arg);
    }

    public Object visit(ConstraintSet cs, Object arg) {
	return commonVisitor(cs, arg);
    }

    public Object visit(Queue q, Object arg) {
	return commonVisitor(q, arg);
    }

    protected Object commonVisitor(BusinessObject bo, Object arg) {
	return bo;
    }

}


