package org.eso.ohs.dfs;


import org.eso.ohs.dfs.Seeing;
import org.eso.ohs.dfs.SkyTransparency;


/**
 * Class WeatherConstraint
 *
 * contains the weather constraint 
 * to be used to transfer data from the view to the model in Orang
 * TODO Javadoc
 *
 * @version $Revision: 1.5 $
 * @author ddorigo, Nov 7, 2005
 */

// $Id: WeatherConstraint.java,v 1.5 2006/06/08 15:03:24 ddorigo Exp $
public class WeatherConstraint {

    private Seeing minSeeing;
    private Seeing maxSeeing;
    private WindInfo wind;
    private SkyTransparency sky;
    private boolean seeingEnable;
    private boolean skyEnable;
    private boolean windEnable;

    /**
     * the orang weather constraints
     * @param seeing current seeing
     * @param sky current sky
     * @param WindAngle current strehl ratio  (applicable to naco)
     * @param seeingEnable if the seeing data should be taken into account
     * @param skyEnable if the sky data should be taken into account
     * @param strehlRatioEnable  if the ratio data should be taken into account
     */
    public WeatherConstraint(Seeing minSeeing,Seeing maxSeeing, SkyTransparency sky, WindInfo wind,
            boolean seeingEnable,
            boolean skyEnable, boolean windEnable) {
        setMinSeeing(minSeeing);
        setMaxSeeing(maxSeeing);
            setWind(wind);
     setSky(sky);
     setSeeingEnable(seeingEnable);
     setSkyEnable(skyEnable);
     setWindEnable(windEnable);
    } 
    
    /**
     * the orang weather constraints
     * @param seeing current seeing
     * @param sky current sky
     * @param strehlRatio current strehl ratio  (applicable to naco)
     */
    public WeatherConstraint(Seeing minSeeing,Seeing maxSeeing, SkyTransparency sky, WindInfo wind) {
     this(minSeeing,maxSeeing,sky,wind,true,true,false);
    }

    /**
     * @return Seeing
     */
    public Seeing getMinSeeing() {
        return minSeeing;
    }

    /**
     * @param seeing void
     */
    public void setMinSeeing(Seeing seeing) {
        this.minSeeing = seeing;
    }
    
    /**
     * @return Seeing
     */
    public Seeing getMaxSeeing() {
        return maxSeeing;
    }

    /**
     * @param seeing void
     */
    public void setMaxSeeing(Seeing seeing) {
        this.maxSeeing = seeing;
    }

    /**
     * @return boolean
     */
    public boolean isSeeingEnable() {
        return seeingEnable;
    }

    /**
     * @param seeingEnable void
     */
    public void setSeeingEnable(boolean seeingEnable) {
        this.seeingEnable = seeingEnable;
    }

    /**
     * @return Seeing
     */
    public WindInfo getWind() {
        return wind;
    }

    /**
     * @param seeing void
     */
    public void setWind(WindInfo wind) {
        this.wind = wind;
    }

    /**
     * @return boolean
     */
    public boolean isWindEnable() {
        return windEnable;
    }

    /**
     * @param seeingEnable void
     */
    public void setWindEnable(boolean windEnable) {
        this.windEnable = windEnable;
    }

    /**
     * @return SkyTransparency
     */
    public SkyTransparency getSky() {
        return sky;
    }

    /**
     * @param sky void
     */
    public void setSky(SkyTransparency sky) {
        this.sky = sky;
    }

    /**
     * @return boolean
     */
    public boolean isSkyEnable() {
        return skyEnable;
    }

    /**
     * @param skyEnable void
     */
    public void setSkyEnable(boolean skyEnable) {
        this.skyEnable = skyEnable;
    }

  
}
