package org.eso.ohs.dfs.docviewdatatrans;



import org.eso.ohs.core.utilities.Convert;

/**
 * A transducer to convert set/get of String values into set/get
 * on an int property.  The target property's name is given to the
 * constructor.
 *
 * <p>
 * For example, users will call it's
 * setValue/getValue methods.  Those calls will be translated into
 * calls to setX/getX on the object given to the constructor.
 *
 * @version $Id: RaAdaptor.java,v 1.3 2006/01/11 16:24:24 amchavan Exp $
 *
 * @author Bob Kemp
 */
public class RaAdaptor extends DegreeAdaptor {

    /**
     * Construct a transducer.
     */
    public RaAdaptor() {
	super();
    }

    /**
     * Construct a transducer to convert set/get of String values into
     * setX/getX on an int property of <code>object</code>.
     */
    public RaAdaptor(Object obj, String fieldName) {
	super(obj, fieldName);
    }

    /**
     * Converts data from a String to a meta-level equivalent of the
     * actual property's type, i.e. to a java.lang.Integer
     * representing an int.  Used with reflection to set
     * the new property value.
     * 
     * <p>
     * Right Ascension values are stored in milli-arcseconds of degrees,
     * even though they are displayed to the user as hours:minutes:seconds.
     * 
     * @param value the new value of the property.
     */
    public Object toMetaLevelObject(String value) {
	int milliArcSecondsOfHours = convertToMilliArcSeconds(value);

	return new Integer( milliArcSecondsOfHours);
    }

    /**
     * Converts data to a String from the meta-level equivalent of the
     * actual property's type, i.e. from a java.lang.Long
     * to a long int.  Used with reflection to get
     * the current property value.
     *
     * <p>
     * Right Ascension values are stored in milli-arcseconds of degrees,
     * even though they are displayed to the user as hours:minutes:seconds.
     * 
     * @param value	value of the property.
     */
    public String fromMetaLevelObject(Object obj) {
	if (obj == null) {
	    return "00:00:00.0000";
	}

	int milliArcSecondsOfDegrees = ((Integer)obj).intValue();

	return convertToHMS(milliArcSecondsOfDegrees);
    }

    /**
     * @see DegreeAdaptor#convertToMilliArcSeconds(String)
     * @see Convert#HHMMSSToMillisec(String)
     */
    protected int convertToMilliArcSeconds( String value ) {
	return( Convert.HHMMSSToMillisec( value ));
    }

    /**
     * @see DegreeAdaptor#convertToHMS(int)
     * @see Convert#millisecToHHMMSS(int)
     *
     * <br>ChangeLog: A M Chavan, 04-Jul-2001 -- fixed 1448
     */
    protected String convertToHMS( int value ) {
	return( Convert.millisecToHHMMSS( value ));
    }
}

