package org.eso.ohs.dfs.docviewdatatrans;

import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.eso.ohs.core.docview.datatrans.ObjAdaptor;
import org.eso.ohs.core.utilities.Convert;

/**
 * A transducer to convert set/get of String values into set/get on a long
 * integer property. The target property's name is given to the constructor.
 * 
 * <p>
 * For example, users will call it's setValue/getValue methods. Those calls will
 * be translated into calls to setX/getX on the object given to the constructor.
 * 
 * TimeInterval = int ":" int TimeIntervalArray = TimeInterval ";" TimeInterval
 * 
 * <p>
 * $Id: TimeDurationAdaptor.java,v 1.3 2005/10/11 14:58:33 ddorigo Exp $
 */

public class TimeDurationAdaptor extends ObjAdaptor {

    private static final String ZERO_TIME = "0T00:00:00";

    /**
     * Construct a transducer.
     */
    public TimeDurationAdaptor() {
        super(long.class);
    }

    /**
     * Construct a transducer to convert set/get of String values into setX/getX
     * on an int property of <code>object</code>.
     */
    public TimeDurationAdaptor(Object obj, String fieldName) {
        super(long.class, obj, fieldName);
    }

    /**
     * Converts data from a String to a meta-level equivalent of the actual
     * property's type, i.e. to a java.lang.Long representing a long int. Used
     * with reflection to set the new property value.
     * 
     * @param value
     *            the new value of the property.
     */
    public Object toMetaLevelObject(String value) {
        Long returnValue = new Long(0);
        if (value != null) {
            try {
                SimpleDateFormat dFormatter = new SimpleDateFormat(
                        Convert.DURATION_FORMAT);
                Date tmpDate = dFormatter.parse(value, new ParsePosition(0));
                Date originDate = dFormatter.parse(ZERO_TIME,
                        new ParsePosition(0));
                if (tmpDate != null) {
                    returnValue = new Long((tmpDate.getTime() - originDate
                            .getTime())
                            / Convert.MILLISECS_PER_SEC);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return returnValue;
    }

    /**
     * Converts data to a String from the meta-level equivalent of the actual
     * property's type, i.e. from a java.lang.Long to a long int. Used with
     * reflection to get the current property value.
     * 
     * @param value
     *            value of the property.
     */
    public String fromMetaLevelObject(Object obj) {
        if (obj == null) {
            return ZERO_TIME;
        }
        long duration = ((Long) obj).longValue();
        Date currDate = new Date(duration * Convert.MILLISECS_PER_SEC);
        SimpleDateFormat fmt = new SimpleDateFormat(Convert.DURATION_FORMAT);
        fmt.format(new Date(duration * Convert.MILLISECS_PER_SEC));

        SimpleDateFormat fmtDays = new SimpleDateFormat(Convert.DURATION_DAYS);
        int days = Integer.parseInt(fmtDays.format(currDate)) - 1;

        SimpleDateFormat fmtHours = new SimpleDateFormat(Convert.DURATION_HOURS);
        String hours = fmtHours.format(currDate);
        StringBuffer date = new StringBuffer().append(days).append('T').append(
                hours);
        return date.toString();

    }

}
