package org.eso.ohs.instruments;

import java.util.Hashtable;


/**
 * These are constants that the TemplateSignature Metd data class 
 * must be aware of when parsing .tsf file
 * <p>
 *
 * $Id: TemplateSignatureConstants.java,v 1.12.4.1 2007/07/20 11:56:57 pnunes Exp $
 *
 * @author Tim Canavan 
 */

public abstract class TemplateSignatureConstants {


	/*
	 * Template types
	 */
    public static final String TEST_TYPE = "test";
	public static final String CALIB_TYPE = "calib";
	public static final String SCIENCE_TYPE = "science";
	public static final String ACQUISITION_TYPE = "acquisition";


	/**
     *  Keywords that have to be in the list of identifiers in the .tsf file beginning with TLP
     */

    public static final String[] mandatoryTemplateKeywords = {"INSTRUM","MODE","VERSION","REFSUP",
                                                               "PRESEQ","TYPE","EXECTIME"};


  /**
     *  Keywords that have to be in the list of identifiers in the .tsf file beginning with TLP
     */

    public static final String[] mandatoryTemplateValues = {"INSTRUM","TYPE","PAF.NAME"};

    /**
     * Valid identifiers must start with one of the following prefixes.
     */

    public static final Hashtable<String,Boolean> firstKeywords = new Hashtable<String,Boolean>();

    static {
        firstKeywords.put("PAF",Boolean.FALSE);
        firstKeywords.put("TPL",Boolean.FALSE);
        firstKeywords.put("SEQ",Boolean.TRUE);
        firstKeywords.put("OCS",Boolean.TRUE);
        firstKeywords.put("DET",Boolean.TRUE);       
        firstKeywords.put("INS",Boolean.TRUE);
        firstKeywords.put("DPR",Boolean.TRUE);
        firstKeywords.put("ADA",Boolean.TRUE);
        firstKeywords.put("TEL",Boolean.TRUE);
        firstKeywords.put("AOS",Boolean.TRUE);
        /**
         * The following were added on 30 April 2002 as a result of todo item 1747/P2PP
         */
        firstKeywords.put("COU",Boolean.TRUE);
        firstKeywords.put("DEL",Boolean.TRUE); 
        firstKeywords.put("ISS",Boolean.TRUE); 
        /**
         * The following was omitted from the 30 April ICD VCS doc. Added for 2.7.1.
         */ 
        firstKeywords.put("LGS",Boolean.TRUE);
    }         
    
   
    /**
     * Every template has to have a TYPE, RANGE, DEFAULT identifer in
     * the .tsf file 
     */
    public static final String[] manadatoryParamKeywords = {
	"TYPE", "RANGE", "DEFAULT"
    };

    /**
     * List of valid paramater types that are allowed to be defined by
     * the applicatiom
     */
    public static final String[] validParamTypes = {
	"filename", "keyword", "keywordlist", "numlist", "coord",
	"intlist", "intrect", "number", "integer",
	"string", "pixel", "boolean", "file", "paramfile"
    };


    /**   
     * Template types that are allowed by the application.     
     */

    public static final String[] allowedTemplateTypes  = {ACQUISITION_TYPE,SCIENCE_TYPE,CALIB_TYPE,TEST_TYPE};

    /**
     * All parameters must begin with a keyword,name pairing that
     * has the keyword of TPL.PARAM
     */
   

    public static final String paramKeyword = "TPL.PARAM";

    /**   
     * Identifer for resources keyword.
     */
    public static final String resourceKeyword = "TPL.RESOURCES";

    /**
     * Exec time keyword
     */
    public static final String execTimeKeyword = "TPL.EXECTIME";

    /**
     * Mode keyword
     */
    public static final String modeKeyword = "TPL.MODE";

    /**
     * Template Type keyword.
     */
    public static final String typeKeyword = "TPL.TYPE";

    /**
     * Instrument name keyword
     */
    public static final String instrumentKeyword = "TPL.INSTRUM";

    /**
     * Template version keyword
     */
    public static final String versionKeyword = "TPL.VERSION";



}
