package org.eso.ohs.persistence;

import org.eso.ohs.core.docview.datatrans.LongAdaptor;
import org.eso.ohs.dfs.StorableObject;

/**
 * A transducer to link to a long property representing a database Id.
 * Database ids are only unique within a table however for P2PP it
 * is useful to make them unique across tables, ie for all object types.
 * We do this using the methods in persistence.Config: tableToUniqueId()
 * and uniqueToTableId() to convert between the real and internal form.
 * This adaptor displays an internal form database id as the real,
 * external form.
 *
 * <p>
 * $Id: DbaseIdAdaptor.java,v 1.2 2005/04/15 19:02:30 tcanavan Exp $
 */

public class DbaseIdAdaptor extends LongAdaptor {

    public void setValue(Object obj) {
	throw new IllegalArgumentException(
	    "DbaseIdAdaptor: write access not allowed"
	);
    }

    public void setReadWrite(boolean newValue) {
	if (newValue) {
	    throw new IllegalArgumentException(
		"DbaseIdAdaptor: write access not allowed"
	    );
	}
    }

    /**
     * Converts data from a String to a meta-level equivalent of the
     * actual property's type -- in this case a null operation.  Used
     * with reflection to set the new property value.
     * 
     * @param value the new value of the property.
     */
    public Object toMetaLevelObject(String value) {
	return value;
    }


    /**
     * Retrieves the nominated property of the Object.
     * 
     * @return		value stored in the Object,
     * 			represented as a Java meta-level object,
     * 			eg Long instead of a long, etc.
     */
    public Object getRawValue() {
	Long idLong = (Long)super.getRawValue();
	Config cfg = Config.getCfg();

	return new Long(cfg.uniqueToTableId(idLong.longValue()));
    }

    /**
     * Converts data to a String from the meta-level equivalent of the
     * actual property's type, i.e. from a java.lang.Long
     * to a long int.  Used with reflection to get
     * the current property value.
     *
     * @param value	value of the property.
     */
    public String FROMmETAlEVELoBJECT(Object obj) {
	if (obj == null) {
	    return "";
	}
	else {
	    return ((StorableObject)obj).getName();
	}
    }

}

