package org.eso.ohs.persistence.dao;

import java.util.List;

import org.eso.ohs.dfs.account.Account;
import org.eso.ohs.dfs.account.Profile;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.HibernateCallback;
import org.springframework.orm.hibernate3.HibernateTemplate;



/**
 * This is the implentation class for the AccountDAO
 * interface.
 * @author tcanvan
 *
 */


public class AccountDAOHibernate implements AccountDAO {
    
    private HibernateTemplate hibernateTemplate; 

    public AccountDAOHibernate() {
    }
    
    public Account find(Integer id) throws DataAccessException {
       return (Account)hibernateTemplate.load(Account.class,id); 
    }
    
    
    /* (non-Javadoc)
     * ${see_to_overridden}
     */
    public void create(Account account) throws DataAccessException {
        hibernateTemplate.saveOrUpdate(account);
    }
    
    
    /* (non-Javadoc)
     * ${see_to_overridden}
     */
    public void updateAccount(Account account) throws DataAccessException {
    	hibernateTemplate.update(account);
    }

    /**
     * @return Returns the hibernateTemplate.
     */
    public HibernateTemplate getHibernateTemplate() throws DataAccessException {
        return hibernateTemplate;
    }

    /**
     * @param hibernateTemplate The hibernateTemplate to set.
     */
    public void setHibernateTemplate(HibernateTemplate hibernateTemplate) {
        this.hibernateTemplate = hibernateTemplate;
    }

    /* (non-Javadoc)
     * ${see_to_overridden}
     */
    
    public Account getFullAccountDatails(String userName) throws IllegalArgumentException,DataAccessException {
        String [] tokens = new String[1];
        tokens[0] = userName;  
        List l = hibernateTemplate.find("from Account Account where Account.accountName = ?",tokens);
        if(l.size() > 1) {
            throw new IllegalArgumentException("Duplicate username in system");            
        }
        Account user = (Account)l.get(0);
        return user;
    }
    
    /* (non-Javadoc)
     * ${see_to_overridden}
     */
    
    public Account find(String userName) throws DataAccessException {    	   
        String [] tokens = new String[1];
        tokens[0] = userName;  
        List l = hibernateTemplate.find("from Account Account where Account.accountName = ?",tokens);
        if(l.size() > 1) {
            throw new IllegalArgumentException("Duplicate username in system");            
        }
        Account user = (Account)l.get(0);
        return user;
    }
    
	
	
	public Account findProfiles(final String name) throws DataAccessException {
		   HibernateCallback callback = new HibernateCallback(){

		    public Object doInHibernate(Session session) throws HibernateException {
				Account account =null;
				String [] tokens = new String[1];
		        tokens[0] = name;  
				String query = "from Account Account where Account.username=?";
			    
                Query queryObject = session.createQuery(query);
				

				queryObject.setParameter(0, name);
				account = (Account) queryObject.list().get(0);
			    account.getProfile().size();
			    //account.grtProfile
				return account;
				}			
			};
			
			return (Account) getHibernateTemplate().execute(callback);
		}


    /* (non-Javadoc)
     * ${see_to_overridden}
     */
    
    public void delete(Account account) throws DataAccessException {
        hibernateTemplate.delete(account);
    }
    
    /* (non-Javadoc)
     * ${see_to_overridden}
     */
 
    public void changePassword(Account account) throws DataAccessException {
    	hibernateTemplate.update(account);
    }
    
    /* (non-Javadoc)
     * ${see_to_overridden}
     */

	public boolean accountExists(String userName)  throws DataAccessException {
        String [] tokens = new String[1];
        tokens[0] = userName; 
        List l = hibernateTemplate.find("from Account Account where Account.accountName = ?",tokens);
        if(l.size() > 0 ) {
        	return true;
        }
        return false;
	}

	/* (non-Javadoc)
	 * ${see_to_overridden}
	 */	
	
	public Profile findDefaultProfile(final String userName) throws DataAccessException {
		Profile accountProfile =null;
		Object [] tokens = new Object[2];
        tokens[0] = Boolean.TRUE;  
        tokens[1] = userName;
		
		String query = "select A,AP,T from Account A " + 
		" join A.profile AP " +
		" join AP.title T " +
		" where AP.defaultProfile=?" +		
		" and A.username=?";	    
		List l = hibernateTemplate.find(query,tokens);
		
        Object [] objectArray = (Object [])l.get(0);
        accountProfile = (Profile)objectArray[1];
        return accountProfile;
        
	
	}
	
	/* (non-Javadoc)
	 * ${see_to_overridden}
	 */
	
	public void updateProfile(Profile accProfile) throws DataAccessException {
		hibernateTemplate.update(accProfile);
		
	}
	
	/* (non-Javadoc)
	 * ${see_to_overridden}
	 */

	public Account getAccountRoles(final String userName) throws DataAccessException {
		   HibernateCallback callback = new HibernateCallback(){

			    public Object doInHibernate(Session session) throws HibernateException {
					Account account =null;
					String query = "from Account Account where Account.accountName=?";
					Query queryObject = session.createQuery(query);
					queryObject.setParameter(0,userName);
					account = (Account) queryObject.list().get(0);
					account.getRoleInfoMany().size();
					return account;
					}			
				};				
				return (Account) getHibernateTemplate().execute(callback);
	
	}

	
}
