package org.eso.ohs.persistence.dao;

import java.util.List;

import org.eso.ohs.dfs.phase1.GtoConflicts;
import org.springframework.dao.DataAccessException;

/**
 * @author fsogni
 * This interface is the main interface to the proposal_gtoConflicts table in the opc70 database
 * This table represents the db interface for the ProposalGtoConflicts business object.  
 */


public interface GtoConflictsDAO {
	
	/**
	 * Find a ProposalGtoConflicts observation based upon
	 * the predefined attributes. Returns these attributes and the database id.
	 * @param proposalGtoConflict
	 * @return
	 * @throws DataAccessException
	 */
     
	public List find(GtoConflicts gtoConflict) throws DataAccessException;
	
	
	
	/**
	 * Retrieve a specific object (it has to contain a valid id)
	 * @param proposalGtoConflict
	 * @return
	 * @throws DataAccessException
	 */
     
	public List findById(GtoConflicts gtoConflict) throws DataAccessException;
	
	
	/**
	 * Find a ProposalGtoConflicts based on the summary id
	 * Returns an observation object
	 * @param gtoObservation
	 * @return
	 * @throws DataAccessException
	 */
     
	public List findByProgrammeId(Long programmeId) throws DataAccessException;
	
	
	
	/**
	 * Create a gto observation
	 * @param gtoObservation
	 * @throws DataAccessException
	 */
	
    public void create(GtoConflicts gtoConflict) throws DataAccessException;
    
    /**
     * Update a gto observation. This expects the database
     * id to be initialised. 
     * @param gtoObservation
     * @throws DataAccessException
     */
    
    public void update(GtoConflicts gtoConflict) throws DataAccessException;
    
    /**
     * Delete a gto observation from the database. Only possible 
     * if it does not refer to a gto conflict (entry in gto_conflicts) 
     * @param gtoObservation
     * @throws DataAccessException
     */
    
    public void delete(GtoConflicts gtoConflict) throws DataAccessException;
	
}
