/**
 * 
 */
package org.eso.ohs.persistence.dao;

import java.util.List;

import org.apache.log4j.Logger;
import org.eso.ohs.dfs.phase1.GtoManagers;
import org.eso.ohs.dfs.phase1.InstrumentConfig;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.HibernateTemplate;

/**
 * @author fsogni
 *
 */
public class GtoManagersDAOHibernate implements GtoManagersDAO {

	
	private HibernateTemplate hibernateTemplate;


	/**
	 * @return Returns the hibernateTemplate.
	 */
	public HibernateTemplate getHibernateTemplate() {
		return hibernateTemplate;
	}

	/**
	 * @param hibernateTemplate The hibernateTemplate to set.
	 */
	public void setHibernateTemplate(HibernateTemplate hibernateTemplate) {
		this.hibernateTemplate = hibernateTemplate;
	}
	

	/**
	 * Field stdlog
	 */
	private static Logger stdlog = Logger.getLogger(GtoManagersDAOHibernate.class);
	
	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.dao.GtoManagersDAO#find(org.eso.ohs.dfs.phase1.GtoManagers)
	 */
	public GtoManagers find(GtoManagers gtoManager) throws DataAccessException {
		
		Object [] paramArray = new Object[1];
		paramArray[0] = gtoManager.getId();		
		List l = hibernateTemplate.findByNamedQuery("findGtoManagersById",paramArray);
		if(l.size() <=0) {
			throw new IllegalArgumentException("Not found");
		}
		GtoManagers gtoManagerFromDb = (GtoManagers)l.get(0);	
		return gtoManagerFromDb;
	}
	
	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.dao.GtoManagersDAO#find(org.eso.ohs.dfs.phase1.GtoManagers)
	 */
	public GtoManagers findByUserId(String userId, String cycleId) throws DataAccessException {
		GtoManagers result = null;
		Object [] paramArray = new Object[2];
		paramArray[0] = new Integer(userId);
		paramArray[1] = new Integer(cycleId);
		List l = hibernateTemplate.findByNamedQuery("findGtoManagersByUserId",paramArray);
		if(l.size() > 0) {
			result = (GtoManagers)l.get(0);	
			 if (l.size() > 1)
				 stdlog.error("the Table gto_managers has duplicate entries for user_id = " + paramArray[0] + " and cycle_id = " + paramArray[1]);
		}
			
		return result;
	}
	
	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.dao.GtoManagersDAO#find(org.eso.ohs.dfs.phase1.GtoManagers)
	 */
	public List getAll() throws DataAccessException {
		
		List result = null;

		List l = hibernateTemplate.findByNamedQuery("getAllGtoManagers");
		if(l.size() > 0) {
			result = l;	
		} else {
			stdlog.error("gto_managers table has no entries");
		}
			
		return result;
	}
	
	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.dao.GtoManagersDAO#find(org.eso.ohs.dfs.phase1.GtoManagers)
	 */
	public GtoManagers findByProperties(GtoManagers gtoManager) throws DataAccessException {
		GtoManagers result = null;
		
		Object [] paramArray = new Object[2];
		paramArray[0] = gtoManager.getUserId();
		paramArray[1] = gtoManager.getCycleId();
		List l = hibernateTemplate.findByNamedQuery("findGtoManagersByProperties",paramArray);
		if(l.size() > 0) {
			 result = (GtoManagers)l.get(0);	
			 if (l.size() > 1)
				 stdlog.error("the Table gto_managers has duplicate entries for user_id = " + paramArray[0] + " and cycle_id = " + paramArray[1]);
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.dao.GtoManagersDAO#find(org.eso.ohs.dfs.phase1.InstrumentConfig)
	 */
	public List find(InstrumentConfig insConfig) throws DataAccessException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.dao.GtoManagersDAO#create(org.eso.ohs.dfs.phase1.GtoManagers)
	 */
	public void create(GtoManagers gtoManager) throws DataAccessException {

		hibernateTemplate.save(gtoManager);

	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.dao.GtoManagersDAO#update(org.eso.ohs.dfs.phase1.GtoManagers)
	 */
	public void update(GtoManagers gtoManager) throws DataAccessException {
		
		hibernateTemplate.update(gtoManager);

	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.dao.GtoManagersDAO#delete(org.eso.ohs.dfs.phase1.GtoManagers)
	 */
	public void delete(GtoManagers gtoManager) throws DataAccessException {
		
		hibernateTemplate.delete(gtoManager);

	}

	public boolean gtoManagerExists(String userName, String cycleId) throws DataAccessException {
		
	        try { findByUserId(userName, cycleId);
	        	  return true;
	        } catch (IllegalArgumentException e){
	        	return false;
	        	
	        	
	        }
	}

}
