package org.eso.ohs.persistence.dao;

import java.util.List;

import org.eso.ohs.dfs.phase1.GtoManagers;
import org.eso.ohs.dfs.phase1.GtoObservations;
import org.eso.ohs.dfs.phase1.InstrumentConfig;
import org.springframework.dao.DataAccessException;

/**
 * @author fsogni
 * This interface is the main interface to the gto_observations table in the opc70 database
 * This table represents the db interface for the gto_observations business object.  
 */


public interface GtoObservationsDAO {
	
	/**
	 * Find a gto observation based upon
	 * the predefined attributes. Returns these attributes and the database id.
	 * @param gtoObservation
	 * @return
	 * @throws DataAccessException
	 */
     
	public List find(GtoObservations gtoObservation) throws DataAccessException;
	
	
	
	/**
	 * Retrieve a specific object (it has to contain a valid id)
	 * @param gtoObservation
	 * @return
	 * @throws DataAccessException
	 */
     
	public GtoObservations findById(GtoObservations gtoObservation) throws DataAccessException;
	
	
	/**
	 * Retrieve a specific object from the id
	 * @param gtoObservation
	 * @return
	 * @throws DataAccessException
	 */
     
	public GtoObservations findById(String gtoId) throws DataAccessException;
	
	
	
	/**
	 * Find a gto observation based upon a given instrument configuration. 
	 * Returns an observation object
	 * @param gtoObservation
	 * @return
	 * @throws DataAccessException
	 */
     
	public List find(InstrumentConfig insConfig) throws DataAccessException;
	
	
	
	/**
	 * Find a gto observation based upon a given gto manager 
	 * Returns a list of observations
	 * @param gtoObservation
	 * @return
	 * @throws DataAccessException
	 */
     
	public List findByGtoManager(GtoManagers gtoManager) throws DataAccessException;
	
	
	/**
	 * Find a gto observation based upon a given list of cycle ids
	 * Returns a list of observations
	 * @param gtoObservation
	 * @return
	 * @throws DataAccessException
	 */
     
	public List<GtoObservations> findByCycleIds(List<Integer> cycleIds) throws DataAccessException;
	
	
	
	
	/**
	 * Create a gto observation
	 * @param gtoObservation
	 * @throws DataAccessException
	 */
	
    public void create(GtoObservations gtoObservation) throws DataAccessException;
    
    /**
     * Update a gto observation. This expects the database
     * id to be initialised. 
     * @param gtoObservation
     * @throws DataAccessException
     */
    
    public void update(GtoObservations gtoObservation) throws DataAccessException;
    
    /**
     * Delete a gto observation from the database. Only possible 
     * if it does not refer to a gto conflict (entry in gto_conflicts) 
     * @param gtoObservation
     * @throws DataAccessException
     */
    
    public void delete(GtoObservations gtoObservation) throws DataAccessException;
	
}
