/**
 * 
 */
package org.eso.ohs.persistence.dao;

import java.util.List;

import org.eso.ohs.dfs.phase1.GtoManagers;
import org.eso.ohs.dfs.phase1.GtoObservations;
import org.eso.ohs.dfs.phase1.InstrumentConfig;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.HibernateTemplate;

/**
 * @author fsogni
 *
 */
public class GtoObservationsDAOHibernate implements GtoObservationsDAO {

	
	private HibernateTemplate hibernateTemplate;


	/**
	 * @return Returns the hibernateTemplate.
	 */
	public HibernateTemplate getHibernateTemplate() {
		return hibernateTemplate;
	}

	/**
	 * @param hibernateTemplate The hibernateTemplate to set.
	 */
	public void setHibernateTemplate(HibernateTemplate hibernateTemplate) {
		this.hibernateTemplate = hibernateTemplate;
	}

	
	public List find(GtoObservations gtoObservation) throws DataAccessException {
		
		Object [] paramArray = new Object[12];
		paramArray[0] = gtoObservation.getGtoManagers().getId();
		paramArray[1] = gtoObservation.getCycle().getId();
		paramArray[2] = gtoObservation.getPiSurname();
		paramArray[3] = gtoObservation.getTargetName();
		paramArray[4] = gtoObservation.getRa();
		paramArray[5] = gtoObservation.getDec();
		paramArray[6] = gtoObservation.getInstrumentConfig().getId();
		paramArray[7] = gtoObservation.getParameter();
		paramArray[8] = gtoObservation.getExposureTime();
		paramArray[9] = gtoObservation.getMovingFlag();
		paramArray[10] = gtoObservation.getGtolpFlag();
		paramArray[11] = gtoObservation.getComments();
		List l = hibernateTemplate.findByNamedQuery("findGtoObservationsByProperties",paramArray);
		if(l.size() <=0) {
			throw new IllegalArgumentException("Not found");
		}
		return l;
		
	}
	
	
	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.dao.GtoObservationsDAO#find(org.eso.ohs.dfs.phase1.GtoObservations)
	 */
	public GtoObservations findById(GtoObservations gtoObservation)
			throws DataAccessException {
		
		Object [] paramArray = new Object[1];
		paramArray[0] = gtoObservation.getId(); 
		List l = hibernateTemplate.findByNamedQuery("queryGtoObservationsById",paramArray);
		if(l.size() <=0) {
			throw new IllegalArgumentException("Not found");
		}
		GtoObservations gtoObservations = (GtoObservations)l.get(0);	
		return gtoObservations;
		
		
	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.dao.GtoObservationsDAO#find(org.eso.ohs.dfs.phase1.GtoObservations)
	 */
	public GtoObservations findById(String gtoId)
			throws DataAccessException {
		
		Integer id = new Integer(gtoId);
		Object [] paramArray = new Object[1];
		paramArray[0] = id; 
		List l = hibernateTemplate.findByNamedQuery("queryGtoObservationsById",paramArray);
		if(l.size() <=0) {
			throw new IllegalArgumentException("Not found");
		}
		GtoObservations gtoObservations = (GtoObservations)l.get(0);	
		return gtoObservations;
		
		
	}
	
	
	
public List find(InstrumentConfig insConfig) throws DataAccessException {
		
		Object [] paramArray = new Object[1];
		paramArray[0] = insConfig.getId(); 
		List l = hibernateTemplate.findByNamedQuery("findGtoObservationsByInstrumentConfig",paramArray);
		if(l.size() <=0) {
			throw new IllegalArgumentException("Not found");
		}
			
		return l;
	}
	
public List findByGtoManager(GtoManagers gtoManager) throws DataAccessException {
	
	   Object[] paramArray = new Object[1];
	   paramArray[0] = gtoManager.getId();
	   List gtoObservationList = hibernateTemplate.findByNamedQuery("findGtoObservationsByGtoManager",paramArray);
		
		
	return gtoObservationList;
}


public List findByCycleIds(List<Integer> cycleIds) throws DataAccessException {
	
	Object [] array = new Object[1];
	array[0] = cycleIds;
	List gtoObservationList = hibernateTemplate.findByNamedQueryAndNamedParam("findGtoObservationsByCycleId","cycleIds",cycleIds);
	if(gtoObservationList.size() <=0) {
		throw new IllegalArgumentException("Not found");
	}
	
return gtoObservationList;
	
}


	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.dao.GtoObservationsDAO#create(org.eso.ohs.dfs.phase1.GtoObservations)
	 */
	public void create(GtoObservations gtoObservation)
			throws DataAccessException {
		
		
		hibernateTemplate.saveOrUpdate(gtoObservation);
	

	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.dao.GtoObservationsDAO#update(org.eso.ohs.dfs.phase1.GtoObservations)
	 */
	public void update(GtoObservations gtoObservation)
			throws DataAccessException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.dao.GtoObservationsDAO#delete(org.eso.ohs.dfs.phase1.GtoObservations)
	 */
	public void delete(GtoObservations gtoObservation)
			throws DataAccessException {
		
		hibernateTemplate.delete(gtoObservation);

	}

	

	

	

	

}
