package org.eso.ohs.persistence.dao;

import java.util.List;

import org.eso.ohs.dfs.phase1.InstrumentConfig;
import org.springframework.dao.DataAccessException;

/**
 * @author fsogni
 * This interface is the main interface to the instrument_config table in the opc70 database
 * This table represents the db interface for the instrument_config business object.  
 */


public interface InstrumentConfigDAO {
	
	/**
	 * Find an instrument config based upon
	 * cyle_id,instrument,mode. Returns these attributes and the database id.
	 * @param instConfig
	 * @return
	 * @throws DataAccessException
	 */
     
	public InstrumentConfig find(InstrumentConfig instConfig) throws DataAccessException;
	
	/**
	 * Find an instrument config based upon
	 * cyle_id,instrument. Returns these attributes and the database id.
	 * @param instConfig
	 * @return
	 * @throws DataAccessException
	 */
     
	public List findByCycleIdInstrument(Integer cycleId, String instrument) throws DataAccessException;
	
	/**
	 * Create an instrument configuration
	 * @param instConfig
	 * @throws DataAccessException
	 */
	
    public void create(InstrumentConfig instConfig) throws DataAccessException;
    
    /**
     * Update an instrument configuration. This expects the database
     * id to be initialised. 
     * @param instConfig
     * @throws DataAccessException
     */
    
    public void update(InstrumentConfig instConfig) throws DataAccessException;
    
    /**
     * Delete an instrunent configuration from the databas. Only possible 
     * if it does not refer to ant gto_observations. 
     * @param instConfig
     * @throws DataAccessException
     */
    
    public void delete(InstrumentConfig instConfig) throws DataAccessException;
	
}
