package org.eso.ohs.persistence.dbase;

import java.sql.Statement;

import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;

/**
 * I/O class for object types that are not stored in the database.
 * Every method just generates an exception.
 *
 * <p>
 * $Id: DbaseHandlerUnknown.java,v 1.1 2005/03/30 09:56:12 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * 
 * @see org.eso.ohs.persistence.dbase.phase2.DbaseMgr
 * @see org.eso.ohs.persistence.ObjectManager
 */

public class DbaseHandlerUnknown extends DbaseIOHandler {

    /**
     * Constructor for Unknown object I/O class.
     */
    public DbaseHandlerUnknown() {
    }

    /**
     * Managed class
     * Object
     * @return Class
     */

    public Class getManagedClass() {
	return Object.class;
    }


    /**
     * Throws an ObjectNotFoundException because this kind of
     * object is never stored.
     * 
     * <p><b>NOTE:</b> FolderFactory relies on this method throwing
     * ObjectNotFoundException when an attempt is made to read
     * a folder summary object.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be retrieved.
     * @return	    array in the format required by reassembleReferences()
     * 
     * @exception ObjectNotFoundException
     *		    the requested object was not found in the database
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public UnpackedStorableObject read(Statement stmt, long id)
			throws ObjectNotFoundException {
	throw new ObjectNotFoundException(
			    "Object not supported on this medium"
			);
    }


    /**
     * Throws an IllegalArgumentException because this kind of
     * object is never stored.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj) {
	throw new IllegalArgumentException(
			    "Object not supported on this medium"
			);
    }


    /**
     * Throws an IllegalArgumentException because this kind of
     * object is never stored.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject obj) {
	throw new IllegalArgumentException(
			    "Object not supported on this medium"
			);
    }


    /**
     * Throws an IllegalArgumentException because this kind of
     * object is never stored.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     */
    public void delete(Statement stmt, long id) {
	throw new IllegalArgumentException(
			    "Object not supported on this medium"
			);
    }


}

