/* $Id: DbaseHandlerProposalSummaryDBFields.java,v 1.6 2006/04/26 09:36:38 ddorigo Exp $ */

package org.eso.ohs.persistence.dbase.phase1;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.BusinessVisitor;
import org.eso.ohs.dfs.ProposalSummaryDBFields;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;

/**
 * Sql handler to retrieve, to get and to update EsoUser objects
 * on the database.  
 * <br>
 *
 * @see ProposalSummaryDBFields
 */
public class DbaseHandlerProposalSummaryDBFields extends DbaseIOHandler {

	private static final String PROPOSAL_RESULT = "proposal_result";
    private static Logger stdlog_ =
		Logger.getLogger(DbaseHandlerProposalSummaryDBFields.class);

	/**
	  * Managed class
	  * @return EsoUser class
	  */
	public Class getManagedClass() {
		return ProposalSummaryDBFields.class;
	}

	/** A reference to the singleton DbaseHandlerProposalSummaryDBFields instance. */
	private static DbaseHandlerProposalSummaryDBFields singleton_ = null;

	 
	public static final String  ID = "id";
	
	public static final String LATEX_SOURCE = "latex_source"; // 
	public static final String SUMMARY = "summary"; // 
	public static final String CYCLE_LABEL = "cycle_label"; // 
	public static final String SUBMITTER_EMAIL = "submitter_email"; // 
	public static final String PI_EMAIL = "pi_email"; // 
	public static final String STATUS = "status"; // 
	public static final String DATE_REC = "date_received"; // 
	public static final String PROP_CODE = "proposal_code"; // 
	public static final String PROP_ID = "prog_id"; // 
    public static final String PROPOSAL_TYPE = "proposal_type";

	/** Select  statement to retireve proposal summary entry. */
	protected final static String prog_sql =
		"SELECT DISTINCT "
			+ Phase1SelectStmt.proposalSummaryTable.get()
			+ "."
			+ ID
			+ ","
			+ Phase1SelectStmt.proposalSummaryTable.get()
			+ "."
			+ DATE_REC
			+ ","
			+ Phase1SelectStmt.proposalSummaryTable.get()
			+ "."
			+ SUBMITTER_EMAIL
			+ ","
			+ Phase1SelectStmt.proposalSummaryTable.get()
			+ "."
			+ PI_EMAIL
			+ ","
			+ Phase1SelectStmt.proposalSummaryTable.get()
			+ "."
			+ PROP_ID
			+ ","
			+ Phase1SelectStmt.proposalSummaryTable.get()
			+ "."
			+ PROP_CODE
			+ ","	
			+ Phase1SelectStmt.proposalSummaryTable.get()
			+ "."
			+ PROPOSAL_RESULT
			+ ","	
			+ Phase1SelectStmt.proposalSummaryTable.get()
			+ "."
			+ STATUS
			+ ","
			+ Phase1SelectStmt.proposalSummaryTable.get()
			+ "."
			+ PROPOSAL_TYPE
			+ ","
			+ Phase1SelectStmt.proposalSummaryTable.get()
			+ "."
			+ CYCLE_LABEL
			+ "\n FROM "
			+ Phase1SelectStmt.proposalSummaryTable.get()
			+ " ";
	private static final String PANEL = "panel"; 

	

	/** 
	 */
	public DbaseHandlerProposalSummaryDBFields() {
		super();
	}

	/**
	  * Read  user data
	  * @param sql statement
	  * @param Object id to be read
	  * @exception SQLException
	  *		    an error occurred during the dbase operation
	  * @exception ObjectNotFoundException
	  *		    the requested object was not found in the database   
	  * @exception ObjectIOException IO Eror occured while reading from the database
	  */

	public UnpackedStorableObject read(Statement stmt, long objId)
		throws ObjectIOException, ObjectNotFoundException, SQLException {

		stdlog_.debug("read method: id " + objId);
		ProposalSummaryDBFields prog = (ProposalSummaryDBFields) newInstance();

		UnpackedStorableObject uso = new UnpackedStorableObject(prog);
		String sql =
			prog_sql
				+ " WHERE "
				+ Phase1SelectStmt.proposalSummaryTable.get()
				+ "."
				+ ID
				+ " = "
				+ objId;
		stdlog_.debug("sql statement: " + sql);

		stdlog_.debug(sql);
		ResultSet rs = stmt.executeQuery(sql);

		if (!rs.next()) { // Only one with this id !!
			throw new ObjectNotFoundException("Object not found in database");
		}

		prog.setId(Config.getCfg().tableToUniqueId(objId, ProposalSummaryDBFields.class));
		setProgFields(prog, rs);
		
		rs.close();
		return new UnpackedStorableObject(prog);
	}

	/**
	  * Read  data
	  * @param sql statement
	  * @param whereClause where condition for retreiving the object
	  * @exception SQLException
	  *		    an error occurred during the dbase operation
	  * @exception ObjectNotFoundException
	  *		    the requested object was not found in the database   
	  * @exception ObjectIOException IO Eror occured while reading from the database
	  */

	public UnpackedStorableObject[] read(
		Statement stmt,
		String whereClause,
		String otherTables)
		throws ObjectIOException, ObjectNotFoundException, SQLException {

		if (otherTables == null) {
			otherTables = "";
		}
		UnpackedStorableObject[] prog_array = null;
		ProposalSummaryDBFields prog = null;
		String sql =
			prog_sql
				+ otherTables
				+ "\n WHERE "
				+ whereClause;

		stdlog_.debug(sql);
		ResultSet rs = stmt.executeQuery(sql);

			Vector<ProposalSummaryDBFields> prog_vector = new Vector<ProposalSummaryDBFields>();

			while (rs.next()) {

				prog = (ProposalSummaryDBFields) newInstance();
				// New element
				prog_vector.addElement(prog);

				// Set only when attaching the object as a new
				// element
				prog.setId(
					Config.getCfg().tableToUniqueId(
						rs.getInt(ID),
						ProposalSummaryDBFields.class));
				stdlog_.debug("prog retrieved:" + rs.getInt(ID));

				setProgFields(prog, rs);
			}
			rs.close();

			prog_array = new UnpackedStorableObject[prog_vector.size()];
			for (int i = 0; i < prog_vector.size(); i++) {
				prog = prog_vector.elementAt(i);
				prog_array[i] = new UnpackedStorableObject(prog);
			}
		

		return prog_array;
	}

	/**
	 * Set the prog fields from the result set.
	 * Only for avoiding duplicated code
	 * @throws SQLException
	 *
	 */
	protected void setProgFields(ProposalSummaryDBFields prog, ResultSet rs) throws SQLException
		 {
          prog.setMessageId(rs.getInt(ID));
        
		prog.setProposalResult(rs.getString(PROPOSAL_RESULT));
			prog.setCycleLabel(rs.getString(CYCLE_LABEL));
	    prog.setDateReceived(rs.getLong(DATE_REC));
	    prog.setPiEmail(rs.getString(PI_EMAIL));
	    prog.setEmail(rs.getString(SUBMITTER_EMAIL));
	    prog.setStatus(rs.getInt(STATUS));
	    prog.setProposalId(rs.getLong(PROP_ID));
	    prog.setProposalCode(rs.getString(PROP_CODE));
		prog.setProposalType(rs.getInt(PROPOSAL_TYPE));

	}

	
	
	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj   an array: the first element is the BO to be stored.
	 * 
	 * @exception ObjectIOException	operation is not permitted exception
	 * 
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void write(Statement stmt, long id, UnpackedStorableObject obj)
		throws ObjectIOException {

		throw new ObjectIOException("Operation not supported for this object");

	}

	/**
	* 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj    the BO to be stored.
	 * 
	 * @exception ObjectIOException	operation is not permitted exception
	 * 
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void update(Statement stmt, long id, UnpackedStorableObject objSpec)
		throws ObjectIOException, SQLException {
		stdlog_.debug("update object");
		StorableObject obj = objSpec.getObject();
		update(stmt, obj);
		stdlog_.debug("update object done");

	}

	/**
	
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj   an array: the first element is the BO to be stored.
	 * 
	 * @exception ObjectIOException	operation is not permitted exception
	 * 
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void update(Statement stmt, StorableObject obj)
		throws ObjectIOException, SQLException {

		ProposalSummaryDBFields op = (ProposalSummaryDBFields) obj;
		StringBuffer flags = new StringBuffer();
		String sql =
			"UPDATE "
				+ Phase1SelectStmt.proposalSummaryTable.get()
				+ " "
				+ "SET "
				+ CYCLE_LABEL
				+ " = '"
				+ op.getCycleLabel()
				+ "', "
				+ SUBMITTER_EMAIL
				+ " = '"
				+ TextUtils.quoteSqlString(op.getEmail())
				+ "', "
				+ PANEL
				+ " = '"
				+ TextUtils.quoteSqlString(op.getPanel())
				+ "', "
				+ PI_EMAIL
				+ " = '"
				+ TextUtils.quoteSqlString(op.getPiEmail())
				+ "', "
				+ STATUS
				+ " = "
				+ op.getStatus()
				+ ", "
				+ PROPOSAL_TYPE
				+ " = "
				+ op.getProposalType()
				+ ", "
				+ DATE_REC
				+ " = "
				+ op.getDateReceived()
				+ ", "
				+ PROP_ID
				+ " = "
				+ op.getProposalId()
				+ ", "
				+ PROP_CODE
				+ " = '"
				+ TextUtils.quoteSqlString(op.getProposalCode())
				+ "', "
				+ PROPOSAL_RESULT
				+ " = '"
				+ TextUtils.quoteSqlString(op.getProposalResult())
				+ "'\n WHERE "
				+ ID
				+ " = "
				+ Config.getCfg().uniqueToTableId(op.getId())
				+ "\n\n";
		stdlog_.debug(sql);
		stmt.executeUpdate(sql);

	}

	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * 
	 * @exception ObjectIOException	operation is not permitted exception
	 */
	public void delete(Statement stmt, long id) throws ObjectIOException {
		throw new ObjectIOException("Operation not supported for this object");
	}

	public Object accept(BusinessVisitor v, Object arg) {
		return null;
	}

	/**
	   * Return the one and only instance of the DbaseHandlerProposalSummaryDBFields.
	   * 
	   * @return Singleton DbaseHandlerProposalSummaryDBFields instance.
	   */
	public static DbaseHandlerProposalSummaryDBFields getDbaseHandlerProposalSummaryDBFields() {
		if (singleton_ == null) {
			singleton_ = new DbaseHandlerProposalSummaryDBFields();
		}
		return singleton_;
	}
}
