package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseHandlerBaseTG;

/**
 * I/O class for TGs stored in the database.
 *
 * <p>
 * $Id: DbaseHandlerTG.java,v 1.1.8.3 2007/10/11 08:27:31 pnunes Exp $
 * <br>
 * @author Bob Kemp
 * 
 * @see org.eso.ohs.persistence.dbase.phase2.DbaseMgr
 * @see org.eso.ohs.persistence.ObjectManager
 */

public class DbaseHandlerTG extends DbaseHandlerBaseTG {

   /** Standard log channel */ 
   private static Logger stdlog_ =
       Logger.getLogger( DbaseHandlerTG.class );


     private static String obrepDbName = Config.getCfg().getDbName(
					       ObservationBlock.getSuffix());

 
    /**
     * Constructor for CS I/O class.
     */
    public DbaseHandlerTG() {

    }


    /**
     * Manged class
     * @return Target class
     */

    public Class getManagedClass() {
	return Target.class;
    }

    /**
     * Reads the object from the database, partially reassembling it.
     * References to other business objects are stored in other fields
     * of the array
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be retrieved.
     * @return	    array in the format required by reassembleReferences()
     * 
     * @exception SQLException
     *		    an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *		    the requested object was not found in the database
     * @exception IOException
     *              if an I/O error occurred while reading the object
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public UnpackedStorableObject read(Statement stmt, long id)
			throws ObjectNotFoundException,
				IOException, SQLException {
	ResultSet rs = null;
	Target tg = (Target)newInstance();
	UnpackedStorableObject uso = new UnpackedStorableObject(tg);

	String sql = "SELECT * FROM " + obrepDbName + "..target_packages"
			+ " WHERE tp_id = " + id;

	if ( ! Config.getCfg().isInOperationsMode() ) {
	    sql = sql + " AND " + obrepDbName + "..target_packages.user_id=" + userId_;
	}
 stdlog_.debug("sql = " + sql);

	rs = stmt.executeQuery(sql);

	if ( ! rs.next()) { // Only one with this id !!
	    throw new ObjectNotFoundException("Object not found in database");
	}

	tg.setOwnerId(rs.getInt("user_id"));

	tg.setId(Config.getCfg().tableToUniqueId(rs.getLong("tp_id"), Target.class));
	tg.setName(rs.getString("item_name"));
	    // XXX COMPATIBILITY KLUDGE because P2PP v1.x (Tcl/Tk) got it wrong
	tg.setRA(rs.getInt("ra")/DEGREES_PER_HOUR);
	tg.setDeclination(rs.getInt("dec"));
	tg.setEquinox(rs.getString("equinox").trim());
	tg.setPropRADouble(rs.getDouble("propRA"));
	tg.setProperDeclinationDouble(rs.getDouble("propDec"));
	tg.setMag(rs.getInt("mag"));
	tg.setUserComments(rs.getString("comments").trim());
	tg.setColor(rs.getFloat("color"));
	tg.setDiffRADouble(rs.getDouble("diffra"));
	tg.setDiffDecDouble(rs.getDouble("diffdec"));
	tg.setEpoch(new Float(rs.getString("epoch")).floatValue());
	tg.setObjectClass(rs.getString("obj_class").trim());
	tg.setCloneParentId(rs.getLong("local_storage_id"));

	rs.close();

	return uso;
    }


    /**
     * Writes the object to the database, for the first time.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
			throws SQLException {
	Target tg = (Target)obj.getObject();

	StringBuffer sb = new StringBuffer();

	sb.append(id);
	sb.append(",");
	sb.append(tg.getOwnerId());
	sb.append(",");
	sb.append(SqlUtils.sqlEscapedString(tg.getName()));
	sb.append(",");
	    // XXX COMPATIBILITY KLUDGE because P2PP v1.x (Tcl/Tk) got it wrong
	sb.append(tg.getRA()*DEGREES_PER_HOUR);
	sb.append(",");
	sb.append(tg.getDeclination());
	sb.append(",");
	sb.append(SqlUtils.sqlEscapedString(tg.getEquinox()));
	sb.append(",");
	sb.append(tg.getPropRADouble());
	sb.append(",");
	sb.append(tg.getProperDeclinationDouble());
	sb.append(",");
	sb.append(tg.getMag());
	sb.append(",");
	sb.append(SqlUtils.sqlEscapedString(tg.getUserComments()));
	sb.append(",");
	sb.append(tg.getColor());
	sb.append(",");
	sb.append(tg.getDiffRADouble());
	sb.append(",");
	sb.append(tg.getDiffDecDouble());
	sb.append(",");
	sb.append(tg.getEpoch());
	sb.append(",");
	sb.append(SqlUtils.sqlEscapedString(tg.getObjectClass()));
	sb.append(",");
	sb.append("'");
	sb.append(tg.getCloneParentId());
	sb.append("'");

	String sqlText = "INSERT INTO target_packages"
			+ " (tp_id, user_id, item_name, ra, dec, equinox,"
			+ "  propRA, propDec, mag, comments, color,"
			+ "  diffra, diffdec, epoch, obj_class,"
			+ "  local_storage_id)"
			+ " VALUES (" + sb + ")";

	stdlog_.debug(sqlText);
	stmt.executeUpdate(sqlText);
    }
    
    
    
    /**
       * Writes the object to the database, for the first time.
       * 
       * @param stmt  JDBC Statement for contacting the dbase
       * @param id    per-table id of the object to be written.
       * @param obj   an array: the first element is the BO to be stored.
       * @exception SQLException  an error occurred during the dbase operation
       * 
       * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
       * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
       */
      public void write(Connection conn, long id, UnpackedStorableObject obj)
              throws SQLException {
           Statement stmt = conn.createStatement();
           write(stmt,id,obj);       
      }



    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject obj)
			throws SQLException {
	Target tg = (Target)obj.getObject();

	StringBuffer sb = new StringBuffer();

	sb.append("tp_id = ");
	sb.append(id);
        if(!Config.getCfg().isInOperationsMode()) {
	    sb.append(", user_id = ");
	    sb.append(userId_);
        }
	sb.append(", item_name = ");
	sb.append(SqlUtils.sqlEscapedString(tg.getName()));

	sb.append(", ra = ");
	    // XXX COMPATIBILITY KLUDGE because P2PP v1.x (Tcl/Tk) got it wrong
	sb.append(tg.getRA()*DEGREES_PER_HOUR);
	sb.append(", dec = ");
	sb.append(tg.getDeclination());
	sb.append(", equinox = ");
	sb.append(SqlUtils.sqlEscapedString(tg.getEquinox()));
	sb.append(", propRA = ");
	sb.append(tg.getPropRADouble());
	sb.append(", propDec = ");
	sb.append(tg.getProperDeclinationDouble());
	sb.append(", mag = ");
	sb.append(tg.getMag());
	sb.append(", comments = ");
	sb.append(SqlUtils.sqlEscapedString(tg.getUserComments()));
	sb.append(", color = ");
	sb.append(tg.getColor());
	sb.append(", diffra = ");
	sb.append(tg.getDiffRADouble());
	sb.append(", diffdec = ");
	sb.append(tg.getDiffDecDouble());
	sb.append(", epoch = ");
	sb.append(tg.getEpoch());
	sb.append(", obj_class = ");
	sb.append(SqlUtils.sqlEscapedString(tg.getObjectClass()));
	sb.append(", local_storage_id = ");
	sb.append("'");
	sb.append(tg.getCloneParentId());
	sb.append("'");
        String sqlText;
        if(!Config.getCfg().isInOperationsMode()) {
       
	    sqlText = "UPDATE target_packages"
			    + " SET " + sb
			    + " WHERE tp_id = " + id
			    + " AND user_id = " + userId_;
        }
        else {
            sqlText = "UPDATE target_packages"
			    + " SET " + sb
			    + " WHERE tp_id = " + id;			   
        } 


	stdlog_.debug(sqlText);
	stmt.executeUpdate(sqlText);
    }


    /**
     * Deletes an object from the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @exception SQLException  an error occurred during the dbase operation
     */
    public void delete(Statement stmt, long id)
			throws SQLException {
	String sqlText = "DELETE FROM target_packages"
			    + " WHERE tp_id = " + id
			    + " AND user_id = " + userId_;

	stdlog_.debug(sqlText);
	stmt.executeUpdate(sqlText);
    }

}


