package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;

/**
 * I/O class for TemplateSignatures stored in the database.
 * 
 *
 * <p>
 * $Id: TimeIntervalDBIO.java,v 1.4 2006/04/26 09:36:38 ddorigo Exp $
 * <br>
 * @author Bob Kemp
 */ 

public class TimeIntervalDBIO {
	
	/** Standard log channel */ 
	private static Logger stdlog_ =
		Logger.getLogger( TimeIntervalDBIO.class );
	
	
	private static String obrepDbName = Config.getCfg().getDbName(
			ObservationBlock.getSuffix());
	
	/**
	 * Read time intervals
	 * @param stmt Sql statement
	 * @param boId Read time intervals with a given id
	 * @return array of TimeIntervals
	 * @exception SQLException
	 *		    an error occurred during the dbase operation   
	 * @exception IOException
	 *              if an I/O error occurred while reading the object
	 * @exception ObjectIOException IO Eror occured while reading from the database
	 */
	
	
	public static TimeInterval[] read(Statement stmt,long boId,String type)
	throws ObjectIOException,
	IOException, SQLException {       
		Vector<TimeInterval> vec = new Vector<TimeInterval>();
		ResultSet rs = null;
		
		String sqlText = "SELECT "
			+ obrepDbName + "..time_intervals.start_interval AS tiStartInterval,"
			+ obrepDbName + "..time_intervals.end_interval AS tiEndInterval,"
			+ obrepDbName + "..time_intervals.preference as tiPreference"
			
			+ " FROM"
			+ " " + obrepDbName + "..time_intervals"	   
			
			+ " WHERE"
			+ " " + obrepDbName + "..time_intervals.ti_id = " + boId
			+ " AND " + obrepDbName + "..time_intervals.ti_type = " + "'" + type + "'";
		
		
		stdlog_.debug(sqlText);
		rs = stmt.executeQuery(sqlText);
		while (rs.next()) {
			TimeInterval ti = new TimeInterval();
			ti.setStartTime(rs.getInt("tiStartInterval"));
			ti.setEndTime(rs.getInt("tiEndInterval"));
			ti.setPreference(rs.getInt("tiPreference"));         
			vec.addElement(ti);
		}
		
		rs.close();   
		TimeInterval [] tiList = new TimeInterval[vec.size()];
		vec.copyInto(tiList);
		return tiList;
	}
	
	
		
	public TimeInterval [] read(Statement stmt, String whereClause,
			String otherTables)
	throws ObjectIOException, ObjectNotFoundException, SQLException {
		
		if (otherTables == null) {
			otherTables = "";
		}
		
		TimeInterval [] tiArray = null;
		TimeInterval ti = null;
		
		String sql = buildSqlQuery(whereClause, otherTables);
		stdlog_.debug(sql);
		
		ResultSet rs = stmt.executeQuery(sql);
		
		try {
			Vector<TimeInterval> tiVector = new Vector<TimeInterval>();
			
			while (rs.next()) {
				
				ti = new TimeInterval();
								
				setIntervalFields(ti, rs);			
				
				//	 New element
				tiVector.addElement(ti);
			}
			rs.close();
			
			tiArray = new TimeInterval[tiVector.size()];
			for (int i = 0; i < tiVector.size(); i++) {
				ti = tiVector.elementAt(i);
				tiArray[i] = new TimeInterval(ti);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return tiArray;
	}
	
	
	/**
	 * @param whereClause
	 * @param otherTables
	 * @return
	 */
	public static String buildSqlQuery(String whereClause, String otherTables) {
		String sql = getSelectSql() + otherTables + " WHERE " + getJoinClause() +  whereClause;
		return sql;
	}
	

    /**
     * @return Returns the run_sql.
     */
    private static String getSelectSql() {
    	String sql = 
    		" SELECT " + 
			Phase2SelectStmt.timeIntervalsTable.get() + ".ti_id,  " + 
			Phase2SelectStmt.timeIntervalsTable.get() + ".start_interval, "+ 
			Phase2SelectStmt.timeIntervalsTable.get() + ".end_interval, "+
			Phase2SelectStmt.timeIntervalsTable.get() + ".preference, "+
			Phase2SelectStmt.timeIntervalsTable.get() + ".ti_type \n "+			
			" FROM " + 			
			Phase2SelectStmt.timeIntervalsTable.get() + ", " +
			Phase2SelectStmt.obsBlocksTable.get();
    	return sql;
    }
    
    private static String getJoinClause() {
    	String res = Phase2SelectStmt.obsBlocksTable.get() + ".ti_id  = " +
    	Phase2SelectStmt.timeIntervalsTable.get() +".ti_id " + " and "; 
    	
    	return(res);
    	
    }
	
	
	
    /**
     * Set the prog fields from the result set. Only for avoiding duplicated
     * code
     *  
     */
    protected void setIntervalFields(TimeInterval ti, ResultSet rs) throws SQLException {
    	
    	ti.setId(rs.getLong("ti_id"));
    	ti.setStartTime(rs.getLong("start_interval"));
    	ti.setEndTime(rs.getLong("end_interval"));
    	ti.setPreference(rs.getInt("preference"));
    	ti.setType(rs.getString("ti_type"));
           	        
    }
	
	/**
	 * Write time intervals
	 * @param stmt Sql statement
	 * @param boId Wrtie time intervals with a given id
	 * @exception SQLException
	 *		    an error occurred during the dbase operation        
	 */
	
	
	public static void write(Statement stmt, long boId,
			TimeInterval[] tiList,String type)
	throws SQLException {
		StringBuffer sb = new StringBuffer();
		for (int i=0; i < tiList.length; i++) {	  
			sb.setLength(0);
			sb.append(boId);
			sb.append(",");
			sb.append(i);
			sb.append(",");
			sb.append(tiList[i].getStartTime());
			sb.append(",");
			sb.append(tiList[i].getEndTime());
			sb.append(",");
			sb.append(tiList[i].getPreference());	   
			sb.append(",");
			sb.append(SqlUtils.sqlEscapedString(type));
			String sqlText = "INSERT INTO time_intervals"
				+ " (ti_id, ti_index,start_interval,end_interval,preference,ti_type)"
				+ " VALUES (" + sb + ")";
			stdlog_.debug(sqlText);
			stmt.executeUpdate(sqlText);
		}
	}
	
	/**
	 * Updates time intervals by deleting and writing
	 * @param stmt Sql statement
	 * @param boId Update time intravls with a given id
	 * @exception SQLException
	 *		    an error occurred during the dbase operation
	 */
	
	
	public static void update(Statement stmt, long boId,
			TimeInterval[] tiList,String type)
	throws SQLException {
		delete(stmt, boId,type);
		write(stmt, boId, tiList,type);
	}
	
	
	/**
	 * Read time intervals
	 * @param stmt Sql statement
	 * @param boId Delete time intervals that have this id
	 * @exception SQLException
	 *		    an error occurred during the dbase operation
	 */
	
	
	public static void delete(Statement stmt, long id,String type)
	throws SQLException {
		String sqlText = "DELETE FROM time_intervals"
			+ " WHERE ti_id = " + id
			+ " AND ti_type = " + "'" + type + "'";
		
		stdlog_.debug(sqlText);
		stmt.executeUpdate(sqlText);	
	}
	
}


