package org.eso.ohs.persistence.service;

import org.eso.ohs.dfs.account.Account;
import org.eso.ohs.dfs.account.Profile;
import org.springframework.dao.DataAccessException;

public interface AccountService {

	/**
	 * Return an account class with no associated objects based 
	 * on its internal database id
	 * @param id
	 * @return
	 * @throws DataAccessException
	 */
	
	public Account getAccount(final Integer id) throws DataAccessException;

	
	/**
	 * Return an account based upon username
	 * @param userName
	 * @return
	 * @throws DataAccessException
	 */
	
	public Account getAccount(String userName) throws DataAccessException;
	
	
	/**
	 * Return an account object with its full list of profiles.
	 * @param userName
	 * @return
	 * @throws DataAccessException
	 */

	public Account getUserAndProfiles(String userName)
			throws DataAccessException;
	
	/**
	 * Return the Profile Object with a reference to the account object.
	 * 
	 * @param userName
	 * @return
	 * @throws DataAccessException
	 */

	public Profile findDefaultProfile(String userName)
			throws DataAccessException;
	
	/**
	 * Updaate the profile information. 
	 * @param accProfile
	 * @throws DataAccessException
	 */

	public void updateProfile(Profile accProfile)
			throws DataAccessException;
	
	/**
	 * Create an account based on a fully instanciated 
	 * object
	 * @param account
	 * @throws DataAccessException
	 */

	public void create(Account account) throws DataAccessException;
	
	/**
	 * Update an account given a specific business object
	 * @param account
	 * @throws DataAccessException
	 */

	public void updateAccount(Account account) throws DataAccessException;
	
	/**
	 * Change the password for a given account
	 * @param account
	 * @throws DataAccessException
	 */

	public void changePassword(Account account) throws DataAccessException;

	/**
	 * Delete an account
	 * @param account
	 * @throws DataAccessException
	 */
	
	public void delete(Account account) throws DataAccessException;

	/**
	 * Check if account already exists.
	 * @param userName
	 * @return
	 * @throws DataAccessException
	 */
	
	public boolean accountExists(String userName) throws DataAccessException;

	
	
	
	/**
	 * Get the roles associated with a given user.
	 * @param userName
	 * @return
	 * @throws DataAccessException
	 */

	public Account getAccountRoles(final String userName)
			throws DataAccessException;
}