package org.eso.ohs.phase2.apps.ipacman;

import java.sql.SQLException;

import org.eso.ohs.core.dbb.server.DbbSession;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.server.DbbSqlEngineImpl;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.Config;

/**
 * This class provides only static methods to build an IPACMAN database browser
 *
 * $Id: BrowserConfig.java,v 1.6 2007/10/11 11:41:27 pnunes Exp $
 * @version $Revision 1.0 $
 * @author Ron J Lobo
 */
public class BrowserConfig {

    public static DbbSqlTable ip;
    public static DbbSqlTable ipp;

    private static String obrepDbName_ =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());

    // private static String opcDbName_ =
        // Config.getCfg().getDbName(ObservingRun.getSuffix());


    //----------------------------------------
    // Public Methods
    //----------------------------------------

    /**
     * Build the sql engine and return its reference to main application
     *
     * @return refence to the SqlEngine to build query
     * @exception SQLException JDBC exception on opening a session and creating
     * the engine
     */
    public static DbbSqlEngine getEngine()
        throws SQLException {

        DbbSession session = new DbbSession(
            Config.getCfg().getUrl(ObservationBlock.getSuffix()),
            obrepDbName_,
            Config.getCfg().getDbUserName(),
            Config.getCfg().getDbPassword());

        /* Create the server engine without building the key hash table
         * from the database metadata. We assume that the information
         * provided by this end application is sufficient to build the sql
         * join clauses.
         */
        DbbSqlEngineImpl engine = new DbbSqlEngineImpl(session, false);
        generateTables();
        buildDefaultJoins(engine);
        return engine;
    }


    /**
     * Build the end application table objects.
     */
    public static void generateTables() {
        ip = new DbbSqlTable(obrepDbName_, "dbo", "instrument_packages");
        ipp = new DbbSqlTable(obrepDbName_, "dbo", "ip_periods");
    }


    //----------------------------------------
    // Private Methods
    //----------------------------------------

    /**
     * Set up the end application information necessary to build the sql
     * join clauses within the sql engine
     */
    private static void buildDefaultJoins(DbbSqlEngineImpl engine) {
        // instrument_packages - ip_periods
        String[] cols = { "version", "inst_name" };
        engine.addCompositeJoinCondition(ip, cols, ipp, cols, true);
    }
}

