package org.eso.ohs.phase2.apps.ipacman;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.sql.SQLException;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.persistence.dbase.phase2.AffectedObsRun;
import org.eso.ohs.persistence.dbase.phase2.InstrumentPackageDBIO;

/**
 * JDialog for checking which Observing Runs are associated with
 * an Instrument Package, or were affected by
 * insertion/deletion/editing of an Instrument Package
 * <p>
 * @version $Revision: 1.17 $
 * @author Ron J Lobo
 */
public class CheckObsRuns extends JDialog {

    public static final String cvsID_ =
        "$Id: CheckObsRuns.java,v 1.17 2006/04/28 08:37:30 tcanavan Exp $";

    private static Logger stdlog_ = Logger.getLogger(CheckObsRuns.class);

    private DbbView view_;
    private Vector<?> obsRuns_;
    private String instr_;
    private float version_;
    private float[] newVersion_;
    private int[] periods_;
    private MyJTable myTable_;

    // constants
    private final float valUnAss_ = -1;
    private final float valNoChg_ = -2;
    private final int periodColumn_ = 4;
    private final int lastColumn_ = 7;
    private final String clear_ = " ";
    private final String unassigned_ = "unassigned";
    private final String selRows_ = "Set Selected Rows";
    private final String allRows_ = "Set All Rows";
    private final Dimension screen_ = Toolkit.getDefaultToolkit().getScreenSize();


    /**
     * Create the dialog
     *
     * @param instr     instrument name
     * @param version   IP version
     * @param periods   associated periods
     * @param view      parent view
     */
    public CheckObsRuns(String instr, float version, int[] periods, DbbView view)
    throws SQLException {

        // stdlog_.debug("CheckObsRuns: instr=" + instr + ", version=" + version);
        view_ = view;
        instr_ = instr;
        version_ = version;
        periods_ = periods;

        obsRuns_ = InstrumentPackageDBIO.getObsRuns(instr_, version_, periods_);
        if (obsRuns_.size() == 0) {
            String mesg = "No observing runs were found for " +
                "\nInstrument: " + instr_;
            if (version_ >= 0) {
                mesg += "\nVersion: " + versionAsString(version_);
            } else if (version == InstrumentPackageDBIO.unassignedVersion) {
                mesg += "\nVersion: unassigned";
            }
            if (periods_ != null) {
                mesg += "\nPeriod(s): " + periods_[0];
                for (int i=1; i < periods_.length; i++) {
                    mesg += ", " + periods_[i];
                }
            }
            JOptionPane.showMessageDialog(view_, mesg,
                "Observing Runs", JOptionPane.INFORMATION_MESSAGE);
            return;
        }

        newVersion_ = new float[obsRuns_.size()];
        for (int i=0; i < newVersion_.length; i++) {
            newVersion_[i] = valNoChg_;
        }

        createMenuBar();
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(createScrollTable(), BorderLayout.CENTER);
        getContentPane().add(createButtonPanel(), BorderLayout.SOUTH);

        pack();
        Dimension win = getPreferredSize();
        setLocation((screen_.width-win.width)/3, (screen_.height-win.height)/3);
        setSize(win);
        setTitle("Observing Runs");
        setVisible(true);
    }


    private void createMenuBar() {
        JMenuBar mb = new JMenuBar();
        setJMenuBar(mb);

        JMenu menu = new JMenu("File");
        mb.add(menu);
        JMenuItem mnItem = menu.add(new CloseAction("Close"));
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_W, InputEvent.CTRL_MASK, false));

        menu = new JMenu("Edit");
        mb.add(menu);

        JMenu subMenu = new JMenu(selRows_);
        menu.add(subMenu);
        subMenu.addMenuListener(new ObsRunsML());

        subMenu = new JMenu(allRows_);
        menu.add(subMenu);
        subMenu.addMenuListener(new ObsRunsML());
    }


    /**
     * Setup the JTable in a JScrollPane
     */
    private JScrollPane createScrollTable() throws SQLException {

        IPMtableModel tableModel_ = new IPMtableModel();
        myTable_ = new MyJTable(tableModel_);
        myTable_.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

        // create the cell editors
        Hashtable<String, DefaultCellEditor> ceHT = new Hashtable<String, DefaultCellEditor>();
        for (int row=0; row < tableModel_.getRowCount(); row++) {
            String p = (String)myTable_.getModel().getValueAt(row, periodColumn_);
            int period = Integer.parseInt(p);
            // stdlog_.debug("row=" + row + ", period=" + period);

            // retrieve or create a cell editor for this period
            DefaultCellEditor dce = ceHT.get(""+period);
            if (dce == null) {
                stdlog_.debug("creating cell editor for period " + period);
                Vector<?> versions = new Vector<Object>();
                if (period == IPACMAN.EngObsRunsPeriod ||
                    period >= IPACMAN.firstMipsPeriod) {
                    versions = InstrumentPackageDBIO.getIPversions(instr_, period);
                }
                JComboBox comboBox = new JComboBox();
                comboBox.addItem(clear_);
                comboBox.addItem(unassigned_);
                for (int i=0; i < versions.size(); i++) {
                    Float f = (Float)versions.elementAt(i);
                    comboBox.addItem(versionAsString(f.floatValue()));
                }
                dce = new DefaultCellEditor(comboBox);
                ceHT.put(""+period, dce);
            }
            myTable_.setCellEditor(dce, row, lastColumn_);
        }

        // Set up tool tips for the editable newVersion column
        TableColumn nvColumn = myTable_.getColumnModel().getColumn(lastColumn_);
        DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
        renderer.setToolTipText("click to edit");
        nvColumn.setCellRenderer(renderer);

        // define constants for dimensions
        final int extraWidth = 10;
        final int extraHeight = 21;
        final int maxHt = screen_.height * 4 / 5;
        final int minHt = 100;

        // set column widths to header text width
        for (int col=0; col <= lastColumn_; col++) {
            TableColumn column = myTable_.getColumnModel().getColumn(col);
            Component comp = myTable_.getTableHeader().getDefaultRenderer().
                getTableCellRendererComponent(
                    myTable_, column.getHeaderValue(), false, false, -1, col);
            int extra = (col == lastColumn_ ? extraWidth*2 : extraWidth);
            int headerWidth = comp.getPreferredSize().width + extra;
            column.setMinWidth(headerWidth);
            column.setPreferredWidth(headerWidth);
        }

        // put the table in a JScrollPane and set its dimensions
        Dimension tblSize = myTable_.getPreferredSize();
        int jspWidth = tblSize.width + extraWidth/3;
        int jspHeight = tblSize.height + extraHeight;
        if (jspHeight > maxHt) {
            jspHeight = maxHt;
            jspWidth = tblSize.width + extraWidth*2;
        } else if (jspHeight < minHt) {
            jspHeight = minHt;
        }
        JScrollPane scrollpane = new JScrollPane(myTable_);
        scrollpane.setPreferredSize(new Dimension(jspWidth, jspHeight));
        return scrollpane;
    }


    /**
     * convert a version number float to a string ensuring that it has 2dps
     */
    private String versionAsString(float f) {
        if (f == valNoChg_) {
            return "";
        } else if (f == valUnAss_) {
            return unassigned_;
        } else {
            String s = ""+f;
            if (s.charAt(s.length()-2) == '.') {
                s+= "0";
            }
            return s;
        }
    }


    /**
     * Setup the button panel
     */
    private JPanel createButtonPanel() {

        JButton updateButton = new JButton();
        updateButton.setText("Update");
        updateButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                update();
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            }
        });

        JButton cancelButton = new JButton();
        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                dispose();
            }
        });

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(updateButton);
        buttonPanel.add(cancelButton);
        return buttonPanel;
    }


    /**
     * Verifies and Inserts into database
     */
    private void update() {

        // find how many rows were changed
        int numChanges = 0;
        for (int i=0; i < obsRuns_.size(); i++) {
            AffectedObsRun obRun = (AffectedObsRun)obsRuns_.elementAt(i);
            if ((newVersion_[i] >= 0 && newVersion_[i] != obRun.ipVersion_) ||
                (newVersion_[i] == valUnAss_ && obRun.ipVersion_ >= 0)) {
                numChanges++;
                // stdlog_.debug("ObsRun " + i + ": " +
                    // obRun.instrument_ + " " + obRun.period_ +
                    // ": SR.id=" + obRun.id_ + " v1=" +
                    // obRun.ipVersion_ + " v2=" + newVersion_[i]);
            }
        }

        // stdlog_.debug("Update " + numChanges + " Obs Run(s)");
        if (numChanges == 0) {
            JOptionPane.showMessageDialog(this,
                "No changes were made. A database update is unnecessary.",
                "Update Cancelled", JOptionPane.WARNING_MESSAGE);
            return;
        }

        // make arrays of sched_rep IDs and new IP versions
        // and prepare a report of the updates
        float[] ipVersions = new float[numChanges];
        int[] schedRepIDs = new int[numChanges];
        StringBuffer report = new StringBuffer(1000);
        report.append("\nThe following Observing Runs were updated:\n");

        // add all changed items to the report
        for (int i=0, j=0; i < obsRuns_.size(); i++) {
            AffectedObsRun obRun = (AffectedObsRun)obsRuns_.elementAt(i);
            if ((newVersion_[i] >= 0 && newVersion_[i] != obRun.ipVersion_) ||
                (newVersion_[i] == valUnAss_ && obRun.ipVersion_ >= 0)) {
                ipVersions[j] = newVersion_[i];
                schedRepIDs[j] = obRun.id_;
                j++;

                // report related
                String pi = obRun.piInitial_ + " " + obRun.piName_;
                String runState = obRun.runState_ == null ?
                    "unknown" : obRun.runState_;
                String oldIP = versionAsString(obRun.ipVersion_);
                String newIP = versionAsString(newVersion_[i]);
                report.append(
                    "\nPrincipal Investigator   " + pi +
                    "\nObs Run Name             " + obRun.obsRunName_.trim() +
                    "\nSched Rep ID             " + obRun.id_ +
                    "\nRun State                " + runState +
                    "\nEmail (OP)               " + obRun.emailOP_ +
                    "\nEmail (EU)               " + obRun.emailEU_ +
                    "\nPeriod                   " + obRun.period_ +
                    "\nInstrument               " + obRun.instrument_ +
                    "\nOld IP Version           " + oldIP +
                    "\nNew IP Version           " + newIP +
                    "\n");
            }
        }

        // database update
        try {
            InstrumentPackageDBIO.updateObsRunsIpVersions(schedRepIDs, ipVersions);
        } catch (SQLException e) {
            stdlog_.error("error updating Observing Run IP versions ", e);
            errorMsg("Update Failed", DbWin.DBerror);
            return;
        }

        // success message
        String s = numChanges > 1 ? "s" : "";
        JOptionPane.showMessageDialog(this,
            numChanges + " Observing Run" + s + " updated successfully ",
            "Update Successful", JOptionPane.INFORMATION_MESSAGE);
        dispose();

        // display a text report of the updates
        TextDisplayWidget txtDisplay = new TextDisplayWidget("Updated Observing Runs");
        txtDisplay.clearTextArea();
        txtDisplay.setTextArea(report.toString());
        int wid=450, ht=600;
        txtDisplay.setSize(new Dimension(wid, ht));
        setLocation((screen_.width-wid)/3, (screen_.height-ht)/3);
        txtDisplay.setVisible(true);
    }


    /**
     * display an error message for the user
     * @param t     title of message dialog
     * @param msg   message
     */
    private void errorMsg(String t, String msg) {
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        JOptionPane.showMessageDialog(this, msg, t, JOptionPane.ERROR_MESSAGE);
        stdlog_.error(msg);
    }



    /**
     * table model class for the JTable
     */
    class IPMtableModel extends AbstractTableModel {


        /** constructor */
        public IPMtableModel() throws SQLException {
            // stdlog_.debug("IPMtableModel:ctor");
        }


        /**
         * returns column name for a given column
         *
         * @param col   column number
         * @return column name
         */
        public String getColumnName(int col) {
            switch (col) {
                case 0: return "Principal Investigator";
                case 1: return "Sched Rep ID";
                case 2: return "Obs Run Name";
                case 3: return "Run State";
                case 4: return "Period";
                case 5: return "Instrument";
                case 6: return "Current Version";
                case 7: return "New Version";
                default: return "help!";
            }
        }


        /**
         * @return number of columns in table
         */
        public int getColumnCount() {
            return lastColumn_ + 1;
        }


        /**
         * @return number of rows in table
         */
        public int getRowCount() {
            return (obsRuns_ != null) ? obsRuns_.size() : 0;
        }


        /**
         * indicates whether or not a cell is editable
         * @param row   cell's row number
         * @param col   cell's column number
         * @return number of rows in table
         */
        public boolean isCellEditable(int row, int col) {
            return col == lastColumn_ ? true : false;
        }


        /**
         * returns value in a given cell
         * @param row   cell's row number
         * @param col   cell's column number
         * @return Object in given cell
         */
        public Object getValueAt(int row, int col) {
            // stdlog_.debug("IPMtableModel:getValueAt " + row + "," + col);
            AffectedObsRun obRun = (AffectedObsRun)obsRuns_.elementAt(row);
            switch (col) {
                case 0: return obRun.piInitial_ + " " + obRun.piName_;
                case 1: return obRun.id_ + "";
                case 2: return obRun.obsRunName_;
                case 3: return obRun.runState_;
                case 4: return obRun.period_ + "";
                case 5: return obRun.instrument_;
                case 6: return versionAsString(obRun.ipVersion_);
                case 7: return versionAsString(newVersion_[row]);
                default: return "help!";
            }
        }


        /**
         * sets value in a given cell
         * @param val   new value to set
         * @param row   cell's row number
         * @param col   cell's column number
         */
        public void setValueAt(Object val, int row, int col) {
            // stdlog_.debug("setValueAt " + row + "," + col + " [" + val + "]");
            if (col == lastColumn_) {
                String valStr = val + "";
                if (valStr.trim().equals("")) {
                    newVersion_[row] = valNoChg_;
                } else if (valStr.trim().equals(unassigned_)) {
                    newVersion_[row] = valUnAss_;
                } else {
                    newVersion_[row] = Float.parseFloat(valStr);
                }
            } else {
                stdlog_.error("setValueAt() bad column! " + col);
            }
        }
    }



    /**
     * extend JTable to enable multiple cell editors per column
     */
    class MyJTable extends JTable {

        // hashtable of table cell editors
        private Hashtable<String, TableCellEditor> tceHT_;


        /**
         * constructor
         * @param tm    table model
         */
        public MyJTable(TableModel tm) {
            super(tm);
            tceHT_ = new Hashtable<String, TableCellEditor>();
            // stdlog_.debug("MyJTable:ctor");
        }


        /**
         * returns cell editor for given cell
         * @param row   cell's row number
         * @param col   cell's column number
         * @return cell editor at given cell
         */
        public TableCellEditor getCellEditor(int row, int col) {
            // stdlog_.debug("MyJTable:getCellEditor row=" + row + ", col=" + col);
            if (col == lastColumn_) {
                return tceHT_.get(""+row);
            } else {
                stdlog_.error("getting cell editor at wrong column: " + col);
                return null;
            }
        }


        /**
         * sets a cell editor for a cell
         * @param tce   table cell editor to use at given cell
         * @param row   cell's row number
         * @param col   cell's column number
         */
        public void setCellEditor(TableCellEditor tce, int row, int col) {
            // stdlog_.debug("MyJTable:setCellEditor row=" + row);
            if (col == lastColumn_) {
                tceHT_.put(""+row, tce);
            } else {
                stdlog_.error("setting cell editor at wrong column: " + col);
            }
        }
    }


    /**
     * MenuListener class to dynamically produce a submenu of choices
     */
    class ObsRunsML implements MenuListener {

        public void menuSelected(MenuEvent e) {

            JMenu menu = (JMenu)e.getSource();
            // stdlog_.debug("menuSelected: " + menu.getText());
            boolean changeAll = menu.getText() == allRows_;

            // get the selected rows (or all rows)
            int[] rows;
            if (changeAll) {
                int numRows = myTable_.getRowCount();
                rows = new int[numRows];
                for (int i=0; i < numRows; i++) {
                    rows[i] = i;
                }
            } else {
                rows = myTable_.getSelectedRows();
                if (rows.length == 0) {
                    return;
                }
            }

            // get the cell editors of the selected rows
            Vector<TableCellEditor> ceVec = new Vector<TableCellEditor>();
            for (int i=0; i < rows.length; i++) {
                TableCellEditor ce = myTable_.getCellEditor(rows[i], lastColumn_);
                if (!ceVec.contains(ce)) {
                    ceVec.addElement(ce);
                }
            }

            // find common selections in cell editors
            Vector<String> selections0 = getSelections((DefaultCellEditor)ceVec.elementAt(0));
            for (int i=1; i < ceVec.size(); i++) {
                Vector<String> selections = getSelections((DefaultCellEditor)ceVec.elementAt(i));
                for (int j = selections0.size() - 1; j >= 0; j--) {
                    String s = selections0.elementAt(j);
                    // stdlog_.debug("j=" + j + ": " + s);
                    if (!selections.contains(s)) {
                        selections0.removeElementAt(j);
                        // stdlog_.debug("removed " + s);
                    }
                }
            }

            menu.removeAll();
            for (int i=0; i < selections0.size(); i++) {
                String s = selections0.elementAt(i);
                menu.add(new NewVersionAction(s, rows));
                // stdlog_.debug("add menu item: " + s);
            }
        }


        private Vector<String> getSelections(DefaultCellEditor dce) {
            Vector<String> selections = new Vector<String>();
            JComboBox cb = (JComboBox)dce.getComponent();
            for (int i=0; i < cb.getItemCount(); i++) {
                String s = (String)cb.getItemAt(i);
                // stdlog_.debug("i=" + i + ": " + s);
                selections.addElement(s);
            }
            return selections;
        }


        /* methods required for the interface*/
        public void menuDeselected(MenuEvent e) {}
        public void menuCanceled(MenuEvent e) {}
    }


    /**
     * action class to handle editing multiple NewVersion rows
     */
    class NewVersionAction extends AbstractAction {

        String value_;
        int[] rows_;
        final static String clearStr = "<clear>";

        public NewVersionAction(String newVal, int[] rows) {
            super(newVal == clear_ ? clearStr : newVal);
            value_ = newVal;
            rows_ = rows;
        }

        public void actionPerformed(ActionEvent e) {
            IPMtableModel ipmTM = (IPMtableModel)myTable_.getModel();
            for (int i=0; i < rows_.length; i++) {
                ipmTM.setValueAt(value_, rows_[i], lastColumn_);
                ipmTM.fireTableCellUpdated(rows_[i], lastColumn_);
            }
        }
    }


    /**
     * simple action class to close the ObsRun dialog
     */
    class CloseAction extends AbstractAction {
        public CloseAction(String s) {
            super(s);
        }
        public void actionPerformed(ActionEvent e) {
            dispose();
        }
    }
}

