package org.eso.ohs.phase2.apps.ipacman;

import gnu.regexp.RE;
import gnu.regexp.REException;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbArrowButton;
import org.eso.ohs.core.dbb.client.DbbCheckBox;
import org.eso.ohs.core.dbb.client.DbbISODateTime;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbSelectionList;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.client.DbbWidget;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbInvalidSqlOperatorType;
import org.eso.ohs.core.dbb.sql.DbbQueryStarted;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.Period;
import org.eso.ohs.core.utilities.SystemStatusEvent;
import org.eso.ohs.core.utilities.SystemStatusListener;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.dbase.phase2.InstrumentPackageDBIO;
import org.eso.ohs.phase2.apps.ipacman.actions.DeleteIPAction;
import org.eso.ohs.phase2.apps.ipacman.actions.DiffIPAction;
import org.eso.ohs.phase2.apps.ipacman.actions.EditIPAction;
import org.eso.ohs.phase2.apps.ipacman.actions.ExtractIPAction;
import org.eso.ohs.phase2.apps.ipacman.actions.InsertIPAction;
import org.eso.ohs.phase2.apps.ipacman.actions.MissingIPsAction;
import org.eso.ohs.phase2.apps.ipacman.actions.QuitAction;
import org.eso.ohs.phase2.apps.ipacman.actions.VerifyIPAction;
import org.eso.ohs.phase2.apps.ipacman.actions.VerifyTsfAction;
import org.eso.ohs.phase2.apps.ipacman.actions.ViewObsRunsIPAction;

/**
 * Main window of the application
 * <p>
 * @version $Revision: 1.18 $
 * @author Ron J Lobo
 */
public class DbWin extends JFrame {

    public static final String cvsID_ =
        "$Id: DbWin.java,v 1.18 2007/01/12 16:57:32 pnunes Exp $";

    private static Logger stdlog_ = Logger.getLogger(DbWin.class);

    private static DbWin dbWdw_;

    public static File dir_ = new File(System.getProperty("user.dir"));

    public static final String illegalFilename_ =
        "Acceptable filename format is:\n" +
        "[Instrument]-[Version.Revision]-[Date].zip\n" +
        "e.g. SINFONI-3.14-20031231.zip\n" +
        "Version must be between 0 and 9999\n" +
        "Revision must be 2 digits\n" +
        "Date must be 8 digits: YYYYMMDD";

    public static final String informSA = "Please inform your system administrator.";
    public static final String DBerror = "A Database Error has occured.\n" + informSA;
    public static final String click4ObsRuns = "Click <OK> to check for affected Observing Runs";

    private DbbView view_;
    private DbbSqlEngine engine_;
    private DbbSelectionList instList_;

    private DbbSqlTable ipTbl;
    private DbbSqlTable ippTbl;

    private final int SELECT_COLUMNS = 4;
    private final int SELECT_CRITERIA = 2;


    /**
      * Constructor
      *
      * @exception ObjectIOException Object I/O Exception
      */
    public DbWin() throws IPacManException {

        // stdlog_.debug("IPacMan.DbWin");

        try {
            IPACMAN.addSystemStatusListener(new SystemStatusListener() {
                public void systemStatusChange(SystemStatusEvent ev) {
                    if (ev.getType() == SystemStatusEvent.RELOGIN) {
                        if (dbWdw_ != null) {
                            dbWdw_.setVisible(false);
                            dbWdw_ = null;
                        }
                    }
                }
            });

            engine_ = BrowserConfig.getEngine();
            view_ = new DbbView(engine_);

            ipTbl = BrowserConfig.ip;   // instrument_packages Table
            ippTbl = BrowserConfig.ipp; // ip_periods Table

            createFileMenu();
            createReportMenu();

            buildColumns();
            buildRows();
            buildSort();

        } catch (ObjectIOException e) {
            throw new IPacManException(e);
        } catch (RemoteException e) {
            throw new IPacManException(e);
        } catch (MalformedURLException e) {
            throw new IPacManException(e);
        } catch (NotBoundException e) {
            throw new IPacManException(e);
        } catch (SQLException e) {
            throw new IPacManException(e);
        }

        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent ev) {
                if (Config.getCfg().isInOperationsMode()) {
                    IPACMAN.askAndQuit();
                }
            }
        });

        getContentPane().add(view_);
        setTitle("IPacMan v" + OHSApplication.getApplication().getVersion() +
            "  Server=" + AppConfig.getAppConfig().getDbServer());

        // pack();
        Dimension p = getPreferredSize();
        Dimension s = Toolkit.getDefaultToolkit().getScreenSize();
        setSize(p);
        setLocation((s.width-p.width)/3, (s.height-p.height)/3);
        setVisible(true);
    }


    /**
      * Build DB browser view: Select panel
      *
      * @exception SQLException JDBC exception occurs
      * @exception RemoteException RMI exception occurs
      * @exception MalformedURLException malformed url i.e. servername//port
      * @exception NotBoundException client could not bind to server
      */
    private void buildColumns() throws NotBoundException,
    MalformedURLException, SQLException, RemoteException {

        DbbPanel panel = new DbbPanel("Selected Columns", SELECT_COLUMNS);

        String label_name = "Instrument";
        String column_name = "inst_name";
        DbbSqlChunk c = new DbbSqlChunk(ipTbl, column_name, label_name);
        engine_.addDefaultColumnsChunk(c);

        label_name = "Version";
        column_name = "version";
        c = new DbbSqlChunk(ipTbl, column_name, label_name);
        engine_.addDefaultColumnsChunk(c);

        label_name = "Period";
        column_name = "period";
        c = new DbbSqlChunk(ippTbl, column_name, label_name, Integer.class);
        DbbCheckBox dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        label_name = "IP Date";
        column_name = "added_date";
        c = new DbbSqlChunk(ipTbl, column_name, label_name, DbbISODateTime.class);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        label_name = "Added by";
        column_name = "added_by";
        c = new DbbSqlChunk(ipTbl, column_name, label_name, String.class);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        label_name = "Comments";
        column_name = "comment";
        c = new DbbSqlChunk(ipTbl, column_name, label_name, String.class);
        dbbCBox = new DbbCheckBox(label_name, c);
        panel.addWidget(dbbCBox);

        view_.setColumnsPanel(panel, "North");
    }


    /**
     * Build DB browser view: rows selection panel
     */
    private void buildRows() {

        try {
            DbbSqlOperator op_in = new DbbSqlOperator("in");

            String column = "inst_name";
            String label = "Instrument";
            DbbSqlChunk sqlChunk =
                new DbbSqlChunk(ipTbl, column, label, op_in, String.class);
            String[] instruments = InstrumentPackageDBIO.getInstruments();
            instList_ = new DbbSelectionList(label, sqlChunk, instruments);
/*
            // Adding period as a selection criteria doesn't work correctly
            // due to the outer join condition
            column = "period";
            label = "Period";
            sqlChunk = new DbbSqlChunk(ippTbl, column, label, op_in, Integer.class);
            Integer[] periodsA = getPeriods();
            DbbSelectionList periodList =
                new DbbSelectionList(label, sqlChunk, periodsA);
            panel.addWidget(periodList);
*/
            DbbPanel panel = new DbbPanel("Selection Criteria", SELECT_CRITERIA);
            panel.addWidget(instList_);
            view_.setWherePanel(panel, "North", true);

        } catch (SQLException e) {
            stdlog_.error("buildRows(): " + e);
        } catch (DbbInvalidSqlOperatorType e) {
            stdlog_.error("buildRows(): " + e);
        }
    }


    protected Integer[] getPeriods() {
        int last = Period.getCurrentPeriod() + Period.ADDITIONAL;
        int numItems = last - Period.STARTPERIOD + 1;
        Integer[] periodArr = new Integer[numItems];
        for (int x=0, i=Period.STARTPERIOD; i <= last; i++) {
            periodArr[x++] = new Integer(i);
        }
        return periodArr;
    }


    /**
     * Build DB browser view: sort panel
     */
    private void buildSort() {

        DbbPanel panel = new DbbPanel("Sort By", 1);

        String label_name = "Instrument";
        String column_name = "inst_name";
        DbbSqlChunk c = new DbbSqlChunk(ipTbl, column_name, label_name);
        DbbWidget dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        label_name = "Version";
        column_name = "version";
        c = new DbbSqlChunk(ipTbl, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        label_name = "Period";
        column_name = "period";
        c = new DbbSqlChunk(ippTbl, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        label_name = "IP Date";
        column_name = "added_date";
        c = new DbbSqlChunk(ipTbl, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        label_name = "Added by";
        column_name = "added_by";
        c = new DbbSqlChunk(ipTbl, column_name, label_name);
        dbbWidget = new DbbArrowButton(label_name, c);
        panel.addWidget(dbbWidget);

        view_.setSortPanel(panel, "West");
    }


    /**
     * Create and populate the "File" menu in the database browser view
     * @exception ObjectIOException object IO Exception
     */
    public void createFileMenu() throws ObjectIOException {

        JMenuBar mb = new JMenuBar();
        setJMenuBar(mb);
        JMenu menu = new JMenu("File");
        mb.add(menu);

        JMenuItem mnItem;
        mnItem = menu.add(new InsertIPAction(this, view_));
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_I, InputEvent.CTRL_MASK, false));

        mnItem = menu.add(new EditIPAction(this, view_));
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_E, InputEvent.CTRL_MASK, false));

        mnItem = menu.add(new ExtractIPAction(view_));
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_X, InputEvent.CTRL_MASK, false));

        mnItem = menu.add(new DeleteIPAction(this, view_));
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_D, InputEvent.CTRL_MASK, false));

        menu.addSeparator();

        mnItem = menu.add(new QuitAction(this));
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_Q, InputEvent.CTRL_MASK, false));
    }


    /**
     * Create and populate the "File" menu in the database browser view
     *
     * @exception ObjectIOException object IO Exception
     */
    public void createReportMenu() throws ObjectIOException {

        JMenuBar mb = getJMenuBar();
        JMenu menu = new JMenu("Reports");
        mb.add(menu);

        JMenuItem mnItem;
        mnItem = menu.add(new ViewObsRunsIPAction(this, view_));
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_O, InputEvent.CTRL_MASK, false));

        mnItem = menu.add(new VerifyIPAction(view_));
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_V, InputEvent.CTRL_MASK, false));

        mnItem = menu.add(new VerifyTsfAction(this, view_));
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_T, InputEvent.CTRL_MASK, false));

        mnItem = menu.add(new DiffIPAction(this, view_));
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_F, InputEvent.CTRL_MASK, false));

        Vector<String> ignoredInstrs = getIgnoredInstruments();
        mnItem = menu.add(new MissingIPsAction(this, view_, ignoredInstrs));
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_M, InputEvent.CTRL_MASK, false));
    }


    /**
     * @return Vector of String names of instruments that are to be ignored in the
     * missing IPs report. This list is taken from the configuration data (site.cf)
     * An empty Vector is returned if no ignored instruments are found.
     */
    public Vector<String> getIgnoredInstruments() {
        Vector<String> rc = new Vector<String>();
        String list = AppConfig.getAppConfig().getValue(OHSKeys.IPacManIgnoreIPList);
        if (list != null && list.length() > 0) {
            String delim = " ,;";
            StringTokenizer st = new StringTokenizer(list, delim);
            while (st.hasMoreTokens()) {
                rc.addElement(st.nextToken());
            }
        }
        return rc;
    }


    /**
     * Rereads the table, sets focus on the JTable and also (if required)
     * resets the list of instruments displayed.
     * Called after add/edit/delete IP
     */
    public void resetDbbView(boolean resetInstrList) {
        org.eso.ohs.core.gui.utilities.Utilities.waitCursor(this);
        view_.sqlStartAction(new DbbQueryStarted(this));
        view_.getTable().getTable().requestFocusInWindow();
        if (resetInstrList) {
            try {
                String[] instruments = InstrumentPackageDBIO.getInstruments();
                /* int i = */ instList_.setListDisplayData(instruments);
                // stdlog_.debug("resetDbbView() InstrsInList=" + i);
            } catch (SQLException e) {
                stdlog_.error("resetDbbView() Error: " + e);
            }
        }
    }


    /**
     * ensure that the query is executed only after the frame appears
     */
    public void addNotify() {
        super.addNotify();
        // resetDbbView(true);
        view_.sqlStartAction(new DbbQueryStarted(this));
    }


    /**
     * gets a reference to the DbWin
     */
    public static DbWin getDbWin() throws IPacManException {
        if (dbWdw_ == null) {
            synchronized (DbWin.class) {
                if (dbWdw_ == null) {
                    dbWdw_ = new DbWin();
                }
            }
        }
        return dbWdw_;
    }


    /**
     * uses a regular expression to check an IP filename is valid
     */
    public static boolean checkIPfileName(String filename) {
        try {
            RE regExp = new RE("^[a-zA-Z][.a-zA-Z0-9]{0,99}-(0|[1-9][0-9]{0,3})\\.[0-9]{2}-[0-9]{8}\\.zip$");
            return regExp.isMatch(filename);
        } catch (REException e) {
            stdlog_.error("RegExp error: " + e.getMessage());
            return false;
        }
    }
}

