package org.eso.ohs.phase2.apps.ipacman;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.ProgressMonitor;
import javax.swing.border.Border;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.PafData;
import org.eso.ohs.core.utilities.PafFile;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.ParamMetaData;
import org.eso.ohs.instruments.TemplateSignatureMetaData;
import org.eso.ohs.instruments.ZipUtils;
import org.eso.ohs.instruments.ZipUtilsException;

/**
 * JDialog for comparing two IPs
 * <p>
 * $Id: DiffIPDialog.java,v 1.13 2006/04/28 08:37:30 tcanavan Exp $
 * @version $Revision: 1.13 $
 * @author Ron J Lobo
 */
public class DiffIPDialog extends JDialog {

    private static Logger stdlog_ = Logger.getLogger(DiffIPDialog.class);
    private static final String sep = File.separator;

    private DbWin dbwin_;
    private DbbView view_;

    private JTree diffTree_;
    private DiffObj topObj_;
    private DiffTreeModel dtm_;

    private String[] files_;
    private boolean[] db_;

    private ZipUtils[] zu_;
    private ZipFile[] zf_;

    private Instrument[] instrument_;
    private String[] colour_;
    private boolean colourOn_;

    private Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    private ProgressMonitor progressMonitor;
    private javax.swing.Timer timer;



    /**
     * Create the dialog
     */
    public DiffIPDialog(DbWin dbwin, DbbView view) {
        super(dbwin, "IP Differences", false);
        dbwin_ = dbwin;
        view_ = view;

        zu_ = new ZipUtils[2];
        zf_ = new ZipFile[2];
        instrument_ = new Instrument[2];

        colour_ = new String[2];
        colour_[0] = "blue";
        colour_[1] = "green";
        colourOn_ = true;
    }


    /**
     * public interface to do an IP diff
     *
     * @param files     two IP zip files
     * @param db        booleans indicate IPs are from database or filesystem
     */
    public void doDiff(String[] files, boolean[] db) throws ZipUtilsException {

        db_ = db;
        files_ = files;
        stdlog_.debug("doDiff: db1=" + db_[0] + ", file1=" + files_[0]);
        stdlog_.debug("        db2=" + db_[1] + ", file2=" + files_[1]);

        //Create a timer.
        timer = new javax.swing.Timer(1000, new TimerListener());
        progressMonitor = new ProgressMonitor(this,
            "Comparing IPs", "please wait ...", 0, 1000); //getLengthOfTask()
        progressMonitor.setProgress(0);
        progressMonitor.setMillisToDecideToPopup(2);
        progressMonitor.setMillisToPopup(3);

        dtm_ = new DiffTreeModel();
        topObj_ = new DiffObj("IP Differences");
        findDiffs();
        if (topObj_.kids_.size() == 0) {
            JOptionPane.showMessageDialog(view_,
                "There are no differences between the Instrument Packages",
                "Identical Instrument Packages", JOptionPane.INFORMATION_MESSAGE);
            return;
        }
        diffTree_ = new JTree(dtm_);
        diffTree_.setSelectionRow(0);
        JScrollPane jSP = new JScrollPane(diffTree_);

        getContentPane().add(createTopPanel(), BorderLayout.NORTH);
        getContentPane().add(jSP, BorderLayout.CENTER);
        getContentPane().add(createButtonPanel(), BorderLayout.SOUTH);

        Dimension win = getPreferredSize();
        setLocation((screen.width-win.width)/3, (screen.height-win.height)/3);
        setTitle("Instrument Package Differences");
        pack();
        setVisible(true);
    }


    /**
     * The actionPerformed method in this class
     * is called each time the Timer "goes off".
     */
    class TimerListener implements ActionListener {

        // int p = 1;
        // String n = "a";

        public void actionPerformed(ActionEvent ae) {
            // stdlog_.debug("TimerListener.actionPerformed()");
            // if (progressMonitor.isCanceled() /*|| task.done() */ ) {
                // stdlog_.debug("TimerListener 1");
                // progressMonitor.close();
                // // task.stop();
                // Toolkit.getDefaultToolkit().beep();
                // timer.stop();
                // // if (task.done()) {
                    // // taskOutput.append("Task completed." + newline);
                // // }
                // // startButton.setEnabled(true);
                // stdlog_.debug("task completed!");
            // } else {
                // stdlog_.debug("TimerListener 2");
                // p++;
                // progressMonitor.setNote(task.getMessage());
                // progressMonitor.setProgress(task.getCurrent());
                // progressMonitor.setNote(n + p);
                // progressMonitor.setProgress(3);
                // taskOutput.append(task.getMessage() + newline);
                // taskOutput.setCaretPosition(
                    // taskOutput.getDocument().getLength());
            // }
        }
    }


    /**
     * creates the top panel showing the IP names
     */
    private JPanel createTopPanel() {

        JPanel topPanel = new JPanel();
        topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.Y_AXIS));
        Border padding = BorderFactory.createEmptyBorder(5,5,5,5);
        topPanel.setBorder(padding);

        for (int i = 0; i < 2; i++) {
            String lbl = "<html><b><font color=" + colour_[i] + ">" + getIPtitle(i);
            topPanel.add(new JLabel(lbl));
        }

        return topPanel;
    }


    /**
     * returns a title string containing the IP name
     */
    private String getIPtitle(int i) {
        String title = "IP" + (i+1) + ": ";
        if (db_[i]) {
            int pos0 = files_[i].lastIndexOf(sep);
            int pos1 = files_[i].indexOf("-", pos0);
            int pos2 = files_[i].lastIndexOf("-");
            String instr = files_[i].substring(pos0 + 1, pos1);
            String ver = files_[i].substring(pos1 + 1, pos2);
            title += "Database Entry - " + instr + " v" + ver;
        } else {
            title += files_[i];
        }
        return title;
    }


    /**
     * creates the button panel
     */
    private JPanel createButtonPanel() {

        JButton expandButton = new JButton();
        expandButton.setText("Expand Selection");
        expandButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                colourOn_ = false;
                alterBranch(diffTree_.getSelectionPaths(), true);
                colourOn_ = true;
            }
        });

        JButton contractButton = new JButton();
        contractButton.setText("Contract Selection");
        contractButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                colourOn_ = false;
                alterBranch(diffTree_.getSelectionPaths(), false);
                colourOn_ = true;
            }
        });

        JButton textButton = new JButton();
        textButton.setText("View As Text");
        textButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                showTreeAsText();
            }
        });

        JButton quitButton = new JButton();
        quitButton.setText("Quit");
        quitButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                dispose();
            }
        });

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(expandButton);
        buttonPanel.add(contractButton);
        buttonPanel.add(textButton);
        buttonPanel.add(quitButton);
        return buttonPanel;
    }


    /**
     * expands or contracts a branch and all of its sub-branches recursively
     */
    private void alterBranch(TreePath[] paths, boolean expand) {
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        if (paths != null) {
            for (int i = 0; i < paths.length; i++) {
                // stdlog_.debug("alter: " + paths[i]);
                alterBranch(paths[i], expand);
            }
        }
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }


    /**
     * expands or contracts a branch and all of its sub-branches recursively
     */
    private void alterBranch(TreePath tp, boolean expand) {

        int row = diffTree_.getRowForPath(tp);
        // stdlog_.debug("alterBranch: " + tp + " row: " + row);

        diffTree_.collapsePath(tp);
        int numRowsA = diffTree_.getRowCount();
        diffTree_.expandPath(tp);
        int numRowsB = diffTree_.getRowCount();

        int numNewRows = numRowsB - numRowsA;
        int lastNewRow = row + numNewRows;
        // stdlog_.debug("num rows: " + numRowsA + ", " + numRowsB);
        // stdlog_.debug("numNewRows=" + numNewRows);

        for (int i = lastNewRow; i > row; i--) {
            alterBranch(diffTree_.getPathForRow(i), expand);
        }

        if (!expand) {
            diffTree_.collapsePath(tp);
        }
    }


    /**
     * starts finding differences between IPs
     */
    private void findDiffs() throws ZipUtilsException {

        stdlog_.debug("findDiffs");
        openZips();
        compareDefaultISFs();

        Vector<String> v0 = new Vector<String>();
        Vector<String> v1 = new Vector<String>();

        getZipEntries(v0, v1);
        findTSFdiffs(v0, v1);

        zu_[0].removeTmpDirs();
        zu_[1].removeTmpDirs();
    }


    /**
     * opens the two zip files, checking for errors
     */
    private void openZips() throws ZipUtilsException {

        stdlog_.debug("openZips");
        String ISFfile = null;

        for (int i = 0; i < 2; i++) {
            try {
                zu_[i] = new ZipUtils(files_[i]);
                zf_[i] = new ZipFile(files_[i]);
                Enumeration<? extends ZipEntry> enu = zf_[i].entries();

                // find 'default.isf' file
                ZipEntry defaultISF = zu_[i].basicChecks(enu);
                String filename = defaultISF.getName();
                stdlog_.debug(" defaultISF " + i + ": " + filename);

                // make temp directory
                zu_[i].makeTmpDir(filename);

                ISFfile = zu_[i].writeZipEntryToFile(zf_[i], defaultISF);
                String ISFdir = ISFfile.substring(0, ISFfile.lastIndexOf(sep));
                instrument_[i] = new Instrument(ISFdir, ISFfile);
                zu_[i].deleteFile(ISFfile);

            } catch (ZipUtilsException e) {
                err("Error in IP" + (i+1) + " (" + files_[i] + ")\n" + e.getMessage());
            } catch (IOException e) {
                if (ISFfile != null)
                    zu_[i].deleteFile(ISFfile);
                err("Error in IP" + (i+1) + " (" + files_[i] + ")\n" + e.getMessage());
            }
        }
    }


    /**
     * finds diffs between the default.isf files
     */
    private void compareDefaultISFs() {

        stdlog_.debug("compareDefaultISFs");

        PafData pd[][] = new PafData[2][];
        for (int i = 0; i < 2; i++) {
            String inst = instrument_[i].getInstrumentName();
            stdlog_.debug("instrument" + i + "=" + inst);
            PafFile pf = instrument_[i].getPafFile();
            pd[i] = pf.getIdentifierList();
            stdlog_.debug(" pd" + i + "=" + pd[i].length);
        }

        // compare the two ISF files
        boolean aliasFound = false;
        Vector<PafData> v0 = new Vector<PafData>();
        Vector<PafData> v1 = new Vector<PafData>();
        Vector<PafData> vB = new Vector<PafData>();
        for (int i1 = 0; i1 < pd[1].length; i1++) {
            v1.addElement(pd[1][i1]);
        }
        for (int i0 = 0; i0 < pd[0].length; i0++) {
            String id0 = pd[0][i0].getIdentifier();
            boolean match = false;

            for (int i1 = 0; i1 < pd[1].length; i1++) {
                String id1 = pd[1][i1].getIdentifier();
                if (id0.equals(id1)) {
                    vB.addElement(pd[0][i0]);
                    vB.addElement(pd[1][i1]);
                    if (!v1.removeElement(pd[1][i1])) {
                        aliasFound = true;
                    }
                    match = true;
                    break;
                }
            }
            if (!match) {
                v0.addElement(pd[0][i0]);
            }
        }
        stdlog_.debug(" ISF v0=" + v0.size());
        stdlog_.debug(" ISF v1=" + v1.size());
        stdlog_.debug(" ISF vB=" + vB.size());

        if (aliasFound) {
            errorMsg("Alias in default.isf",
                "Alias(es) found in IPs - results may be incorrect");
        }

        updateTreeModelWithDefaultIsfDiffs(v0, v1, vB);
    }


    /**
     * update the tree model with param differences for these TSFs
     */
    private void updateTreeModelWithDefaultIsfDiffs(
            Vector<PafData> v0, Vector<PafData> v1, Vector<PafData> vB) {

        DiffObj dObjIP1 = new DiffObj(colour_[0], "Parameters only in IP1");
        for (int i = 0; i < v0.size(); i++) {
            PafData pd = v0.elementAt(i);
            DiffObj dObjB = new DiffObj(colour_[0], pd.getIdentifier());
            dObjIP1.addLeaf(dObjB);
        }

        DiffObj dObjIP2 = new DiffObj(colour_[1], "Parameters only in IP2");
        for (int i = 0; i < v1.size(); i++) {
            PafData pd = v1.elementAt(i);
            DiffObj dObjB = new DiffObj(colour_[1], pd.getIdentifier());
            dObjIP2.addLeaf(dObjB);
        }

        DiffObj dObjIPs = new DiffObj("Parameters in both IPs");
        for (int i = 0; i < vB.size(); i+=2) {
            PafData pd0 = vB.elementAt(i);
            PafData pd1 = vB.elementAt(i+1);
            diffIsfParams(pd0, pd1, dObjIPs);
        }

        DiffObj dObjISF = new DiffObj("default.isf");
        dObjISF.addBranch(dObjIP1);
        dObjISF.addBranch(dObjIP2);
        dObjISF.addBranch(dObjIPs);

        topObj_.addBranch(dObjISF);
    }


    /**
     * finds diffs between the parameters in the default.isf files
     */
    private void diffIsfParams(PafData pd0, PafData pd1, DiffObj dObjA) {
        stdlog_.debug("diffIsfParams: " + pd0.getIdentifier());
        String param = pd0.getIdentifier();
        param = param.substring(param.lastIndexOf(sep) + 1);
        addParam(dObjA, param, pd0.getValue(), pd1.getValue());
    }


    /**
     * copies ZipEntries into 2 vectors for convenience
     */
    private void getZipEntries(Vector<String> v0, Vector<String> v1) {

        Enumeration<? extends ZipEntry> enu1 = zf_[0].entries();
        while (enu1.hasMoreElements()) {
            ZipEntry ze = enu1.nextElement();
            v0.addElement(ze.getName());
        }
        stdlog_.debug("v0.size=" + v0.size());

        Enumeration<? extends ZipEntry> enu2 = zf_[1].entries();
        while (enu2.hasMoreElements()) {
            ZipEntry ze = enu2.nextElement();
            v1.addElement(ze.getName());
        }
        stdlog_.debug("v1.size=" + v1.size());
    }


    /**
     * loops through the TSF files looking for differences
     */
    private void findTSFdiffs(Vector<String> v0, Vector<String> v1) throws ZipUtilsException {

        stdlog_.debug("findTSFdiffs");
        Vector<String> inIP1 = new Vector<String>();
        Vector<String> inIP2 = new Vector<String>();
        Vector<String> inBoth = new Vector<String>();

        while (v0.size() > 0) {
            stdlog_.debug("   v0:" + v0.size());
            String tsf1 = v0.elementAt(0);
            if (tsf1.endsWith(".tsf")) {
                String tsf1a = tsf1.substring(tsf1.indexOf(sep));
                boolean match = false;
                for (int i1 = 0; i1 < v1.size(); i1++) {
                    stdlog_.debug("   i1:" + i1 + "/" + v1.size());
                    String tsf2 = v1.elementAt(i1);
                    if (tsf2.endsWith(".tsf")) {
                        String tsf2a = tsf2.substring(tsf2.indexOf(sep));
                        if (tsf1a.equals(tsf2a)) {
                            stdlog_.debug(" found match: " + tsf1a);
                            inBoth.addElement(tsf1);
                            inBoth.addElement(tsf2);
                            v0.removeElementAt(0);
                            v1.removeElementAt(i1);
                            match = true;
                            break;
                        }
                    } else {
                        stdlog_.debug(" v1: ignoring " + tsf2);
                        v1.removeElementAt(i1);
                        i1--;
                    }
                }
                if (!match) {
                    stdlog_.debug(" add to inIP1: " + tsf1);
                    inIP1.addElement(tsf1);
                    v0.removeElementAt(0);
                }
            } else {
                stdlog_.debug(" v0: ignoring " + tsf1);
                v0.removeElementAt(0);
            }
        }
        for (int i1 = 0; i1 < v1.size(); i1++) {
            String tsf2 = v1.elementAt(i1);
            if (tsf2.endsWith(".tsf")) {
                inIP2.addElement(tsf2);
            } else {
                stdlog_.debug(" v1: ignoring " + tsf2);
            }
        }

        DiffObj IP1tsfs = new DiffObj(colour_[0], "TSFs only in IP1");
        stdlog_.debug("TSFs only in IP1: " + inIP1.size());
        for (int i = 0; i < inIP1.size(); i++) {
            String s = inIP1.elementAt(i);
            stdlog_.debug("   " + s);
            s = s.substring(s.lastIndexOf(sep) + 1);
            IP1tsfs.addLeaf(new DiffObj(colour_[0], s));
        }

        DiffObj IP2tsfs = new DiffObj(colour_[1], "TSFs only in IP2");
        stdlog_.debug("TSFs only in IP2: " + inIP2.size());
        for (int i = 0; i < inIP2.size(); i++) {
            String s = inIP2.elementAt(i);
            stdlog_.debug("   " + s);
            s = s.substring(s.lastIndexOf(sep) + 1);
            IP2tsfs.addLeaf(new DiffObj(colour_[1], s));
        }

        DiffObj inBothTsfs = new DiffObj("TSFs with parameter differences");
        DiffObj errTSFs = new DiffObj("TSFs with errors");
        stdlog_.debug("TSFs in both IPs = " + inBoth.size());
        for (int i = 0; i < inBoth.size(); i+=2) {
            String tsf1 = inBoth.elementAt(i);
            String tsf2 = inBoth.elementAt(i+1);
            DiffObj dObj = new DiffObj(tsf1.substring(tsf1.lastIndexOf(sep)+1));
            findParamDiffs(tsf1, tsf2, dObj, errTSFs);
            inBothTsfs.addBranch(dObj);
        }

        topObj_.addBranch(IP1tsfs);
        topObj_.addBranch(IP2tsfs);
        topObj_.addBranch(inBothTsfs);
        topObj_.addBranch(errTSFs);
    }


    /**
     * searches through a common TSF file looking for parameter differences
     */
    private void findParamDiffs(
        String tsf1, String tsf2, DiffObj dObj, DiffObj errTSFs)
    throws ZipUtilsException {

        stdlog_.debug("findParamDiffs " + tsf1 + ", " + tsf2);

        // get the TSFs from the zip files
        Enumeration<? extends ZipEntry> enu1 = zf_[0].entries();
        ZipEntry ze0 = null;
        while (enu1.hasMoreElements()) {
            ze0 = enu1.nextElement();
            if (ze0.getName().equals(tsf1)) {
                break;
            }
        }

        Enumeration<? extends ZipEntry> enu2 = zf_[1].entries();
        ZipEntry ze1 = null;
        while (enu2.hasMoreElements()) {
            ze1 = enu2.nextElement();
            if (ze1.getName().equals(tsf2)) {
                break;
            }
        }

        // write the TSFs as files
        String TSFfile0 = zu_[0].writeZipEntryToFile(zf_[0], ze0);
        String TSFfile1 = zu_[1].writeZipEntryToFile(zf_[1], ze1);
        stdlog_.debug(" comparing: " + TSFfile0 + " + " + TSFfile1);

        try {
            TemplateSignatureMetaData tsmd0 =
                new TemplateSignatureMetaData(TSFfile0, instrument_[0]);
            TemplateSignatureMetaData tsmd1 =
                new TemplateSignatureMetaData(TSFfile1, instrument_[1]);
            zu_[0].deleteFile(TSFfile0);
            zu_[1].deleteFile(TSFfile1);

            // check for errors in both TSF files
            boolean templateErrorsFound = false;
            if (tsmd0.getCriticalStatus() || !tsmd0.isValidTemplateName()) {
                stdlog_.debug("Error in TSF: " + tsf1);
                DiffObj dObjErr = new DiffObj(tsf1);
                errTSFs.addLeaf(dObjErr);
                templateErrorsFound = true;
            }
            if (tsmd1.getCriticalStatus() || !tsmd1.isValidTemplateName()) {
                stdlog_.debug("Error in TSF: " + tsf2);
                DiffObj dObjErr = new DiffObj(tsf2);
                errTSFs.addLeaf(dObjErr);
                templateErrorsFound = true;
            }
            if (templateErrorsFound) {
                return;
            }

            // now test their parameters
            ParamMetaData pmd0[] = tsmd0.getParamList();
            ParamMetaData pmd1[] = tsmd1.getParamList();
            stdlog_.debug(" pmd0: " + pmd0.length);
            stdlog_.debug(" pmd1: " + pmd1.length);

            Vector<ParamMetaData> v0 = new Vector<ParamMetaData>();
            Vector<ParamMetaData> v1 = new Vector<ParamMetaData>();
            Vector<ParamMetaData> vB = new Vector<ParamMetaData>();
            for (int i1 = 0; i1 < pmd1.length; i1++) {
                v1.addElement(pmd1[i1]);
            }
            for (int i0 = 0; i0 < pmd0.length; i0++) {
                String p1 = pmd0[i0].getParameterName();
                boolean match = false;
                for (int i1 = 0; i1 < pmd1.length; i1++) {
                    if (p1.equals(pmd1[i1].getParameterName())) {
                        vB.addElement(pmd0[i0]);
                        vB.addElement(pmd1[i1]);
                        v1.removeElement(pmd1[i1]);
                        match = true;
                        break;
                    }
                }
                if (!match) {
                    v0.addElement(pmd0[i0]);
                }
            }
            stdlog_.debug(" v0: " + v0.size());
            stdlog_.debug(" v1: " + v1.size());
            stdlog_.debug(" vB: " + vB.size());

            // update the tree model with param differences for these TSFs
            if (v0.size() > 0) {
                DiffObj dObjA = new DiffObj(colour_[0], "Parameters only in IP1");
                for (int i = 0; i < v0.size(); i++) {
                    ParamMetaData pmd = v0.elementAt(i);
                    DiffObj dObjB = new DiffObj(colour_[0], pmd.getParameterName());
                    dObjA.addLeaf(dObjB);
                }
                dObj.addBranch(dObjA);
            }
            if (v1.size() > 0) {
                DiffObj dObjA = new DiffObj(colour_[1], "Parameters only in IP2");
                for (int i = 0; i < v1.size(); i++) {
                    ParamMetaData pmd = v1.elementAt(i);
                    DiffObj dObjB = new DiffObj(colour_[1], pmd.getParameterName());
                    dObjA.addLeaf(dObjB);
                }
                dObj.addBranch(dObjA);
            }
            if (vB.size() > 0) {
                DiffObj dObjA = new DiffObj("Parameters in both IPs");
                for (int i = 0; i < vB.size(); i+=2) {
                    ParamMetaData pmdA = vB.elementAt(i);
                    ParamMetaData pmdB = vB.elementAt(i+1);
                    diffTsfParams(pmdA, pmdB, dObjA);
                }
                dObj.addBranch(dObjA);
            }

        } catch (IOException e) {
            zu_[0].deleteFile(TSFfile0);
            zu_[1].deleteFile(TSFfile1);
            err("File IO error\n" + e.getMessage());
        }
    }


    /**
     * compares the 2 versions of a parameter found in 2 TSF files
     */
    private void diffTsfParams(ParamMetaData pmd0, ParamMetaData pmd1, DiffObj dObjA) {
        stdlog_.debug("diffTsfParams: " + pmd0.getParameterName());
        DiffObj dObjB = new DiffObj(pmd0.getParameterName());
        addParam(dObjB, "Type", pmd0.getType(), pmd1.getType());
        addParam(dObjB, "Range", pmd0.getRange(), pmd1.getRange());
        addParam(dObjB, "Default Value", pmd0.getDefaultValue(), pmd1.getDefaultValue());
        addParam(dObjB, "Label", pmd0.getLabel(), pmd1.getLabel());
        addParam(dObjB, "Mini Help", pmd0.getMiniHelp(), pmd1.getMiniHelp());
        addParam(dObjB, "Value", pmd0.getValue(), pmd1.getValue());
        dObjA.addBranch(dObjB);
    }


    /**
     * Adds a leaf to the tree with values from both IPs
     */
    private void addParam(DiffObj dObj, String name, String val0, String val1) {
        val0 = (val0 == null ? "" : val0);
        val1 = (val1 == null ? "" : val1);
        if (!val0.equals(val1)) {
            DiffObj dObjA = new DiffObj(name);
            DiffObj dObj0 = new DiffObj(colour_[0], val0);
            DiffObj dObj1 = new DiffObj(colour_[1], val1);
            dObjA.addLeaf(dObj0);
            dObjA.addLeaf(dObj1);
            dObj.addBranch(dObjA);
        }
    }


    /**
     * Displays the tree in a TextDisplayWidget
     */
    private void showTreeAsText() {

        // String report = "IP1: " + files_[0] + "\nIP2: " + files_[1] + "\n\n";
        String report = getIPtitle(0) + "\n" + getIPtitle(1) + "\n\n";
        report += showBranch("", (DiffObj)dtm_.getRoot());

        TextDisplayWidget txtDisplay = new TextDisplayWidget("IP Differences");
        txtDisplay.clearTextArea();
        txtDisplay.setTextArea(report);
        txtDisplay.setSize(new Dimension(450, 600));
        txtDisplay.setLocation((screen.width-450)/3, (screen.height-600)/3);
        txtDisplay.setVisible(true);
    }


    /**
     * recursive method to make text display of the tree
     */
    private String showBranch(String pre, DiffObj dObj) {
        String report = pre + dObj.text_ + "\n";
        if (!dtm_.isLeaf(dObj)) {
            pre += "    ";
            for (int i = 0; i < dtm_.getChildCount(dObj); i++) {
                DiffObj child = (DiffObj)dtm_.getChild(dObj, i);
                report += showBranch(pre, child);
            }
        }
        return report;
    }


    /**
     * display an error message for the user
     * @param t     title of message dialog
     * @param msg   message
     */
    private void errorMsg(String t, String msg) {
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        JOptionPane.showMessageDialog(this, msg, t, JOptionPane.ERROR_MESSAGE);
        stdlog_.error(msg);
    }


    /**
     * Close the frame and dispose of its components
     */
    protected void closeDialog() {
        setVisible(false);
        dispose();
    }


    /**
     * called when an error in the IP is found
     * either clean up files/directories and quit by throwing ZipUtilsException
     * or add to the report and allow verification to continue
     *
     * @param message   error message
     */
    private void err(String message) throws ZipUtilsException {
        stdlog_.error(message);
        if (zu_[0] != null) zu_[0].removeTmpDirs();
        if (zu_[1] != null) zu_[1].removeTmpDirs();
        throw new ZipUtilsException(message);
    }



    /**
     * class for the JTree model (OMC architecture)
     */
    class DiffTreeModel implements TreeModel {

        // DiffTreeModel() {}


        /* interface methods*/

        // Returns the root of the tree.
        public Object getRoot() {
            stdlog_.debug("DTM:getRoot()");
            return topObj_;
        }

        // Returns the child of parent at index index in the parent's child array
        public Object getChild(Object parent, int index) {
            return (((DiffObj)parent).kids_.elementAt(index));
        }

        // Returns the number of children of parent.
        public int getChildCount(Object parent) {
            return ((DiffObj)parent).kids_.size();
        }

        // Returns the index of child in parent.
        public int getIndexOfChild(Object parent, Object child) {
            stdlog_.debug("DTM:getIndexOfChild()");
            DiffObj p = (DiffObj) parent;
            for (int i = 0; i < p.kids_.size(); i++) {
                DiffObj c1 = (p.kids_.elementAt(i));
                if (c1 == child) {
                    return i;
                }
            }
            return -1;
        }

        // Returns true if node is a leaf.
        public boolean isLeaf(Object node) {
            return (((DiffObj)node).kids_.size() == 0);
        }

        // Adds a listener for the TreeModelEvent posted after the tree changes
        public void addTreeModelListener(TreeModelListener l) {
            stdlog_.debug("DTM:addTreeModelListener()");
        }

        // Removes a listener previously added with addTreeModelListener.
        public void removeTreeModelListener(TreeModelListener l) {
            stdlog_.debug("DTM:removeTreeModelListener()");
        }

        // Messaged when the user has altered the value for the item identified by path to newValue.
        public void valueForPathChanged(TreePath path, Object newValue) {
            stdlog_.debug("DTM:valueForPathChanged()");
        }
    }


    class DiffObj {

        public String text_;
        public String html_;
        public Vector<DiffObj> kids_ = new Vector<DiffObj>();

        /** ctor */
        DiffObj(String text) {
            html_ = "<font color=black bgcolor=white>";
            text_ = text;
        }

        /** ctor */
        DiffObj(String colour, String text) {
            html_ = "<font color=" + colour + " bgcolor=white>";
            text_ = text;
        }

        /** return html for JLabel */
        public String toString() {
            // stdlog_.debug("DiffObj.toString: col=" + colourOn_);
            return colourOn_ ? "<html>" + html_ + text_ : text_;
        }

        /** add a leaf to the tree */
        public void addLeaf(DiffObj child) {
            kids_.addElement(child);
        }

        /** add a branch to the tree (if it isn't empty) */
        public void addBranch(DiffObj child) {
            if (child.kids_.size() > 0) {
                kids_.addElement(child);
            }
        }
    }
}

