package org.eso.ohs.phase2.apps.ipacman;

// import org.apache.log4j.*;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.persistence.OHSPersistence;

/**
 * This subclass of the ObjectManager handles all application-specific
 * issues relating to storage of objects.
 * <p>
 * For correct operation, there should be at most one instance of this
 * object.  Thus it is a singleton and can only be instantiated using
 * the static Persistence.getInstance() method.  Each time this
 * method is called it will return the <em>same</em> instance of the
 * ObjectManager.
 * <p>
 * @version $Revision: 1.8 $
 * @author Bob Kemp
 */
public class Persistence extends OHSPersistence {

    // private static Logger stdlog_ = Logger.getLogger(Persistence.class);

    public final String cvsID_ =
        "$Id: Persistence.java,v 1.8 2004/04/08 12:09:52 tcanavan Exp $";

    /** current user id */
    protected int userId_ = -1;

    private static Persistence instance_;


    /**
     * Return the one and only instance of the ObjectManager
     *
     * @return Singleton ObjectManager instance
     */
    public static Persistence getInstance() {
        return instance_;
    }


    /**
     * Install a new instance of the Persistence
     *
     * @param instance  new Persistence instance
     */
    public static void setInstance(Persistence instance) {
        instance_ = instance;
        setObjectManager(instance);
    }


    public static void startup() {
        Persistence.setInstance(new Persistence());
    }


    /**
     * Shutdown the current instance of ObjectManager and AutoSaver
     */
    public void shutdown() {
        ConnectionPool.shutdown();
        super.shutdown();
    }


    /**
     * Re-creation hook for subclasses
     *
     * @see #startup
     * @see #shutdown
     */
    public void createNewInstance() {
        startup();
    }
}

