package org.eso.ohs.phase2.apps.ipacman.actions;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.Date;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.instruments.InstrumentPackage;
import org.eso.ohs.persistence.dbase.phase2.InstrumentPackageDBIO;
import org.eso.ohs.phase2.apps.ipacman.CheckObsRuns;
import org.eso.ohs.phase2.apps.ipacman.DbWin;
import org.eso.ohs.phase2.apps.ipacman.IPacManException;

/**
 * Action class to edit an IP in the database
 * <p>
 * $Id: EditIPAction.java,v 1.17 2006/04/28 08:43:04 tcanavan Exp $
 * @version $Revision: 1.17 $
 * @author Ron J Lobo
 */
public class EditIPAction extends InsertEditIP {

    private InstrumentPackage origIP_;
    private Vector<Integer> origPeriods_;

    private String instr_;
    private float versionF_;


    /**
     * constructor
     */
    public EditIPAction(DbWin dbWin, DbbView view) {
        super(dbWin, view, "Edit IP");
        // stdlog_.debug("EditIPAction ctor");
        okButtonText_ = "Save Changes";
        createComponents();
    }


    /**
     * Event handler for the InsertIPAction action
     *
     * @param ev User-initiated ActionEvent to Insert an IP
     */
    public void actionPerformedImpl(ActionEvent ae) {
        // stdlog_.debug("EditIPAction.actionPerformedImpl()");
        int rows[] = view_.getSelectedRows();
        if (rows.length != 1) {
            JOptionPane.showMessageDialog(view_.getTopLevelAncestor(),
                "Please select one IP to perform this operation",
                "Selection Error", JOptionPane.ERROR_MESSAGE);
            return;
        }

        setInitialValues(rows);
        dlg_.setVisible(true);
        addedBy_.requestFocusInWindow();
    }


    private void setInitialValues(int rows[]) {

        instr_ = view_.getValueAt(rows[0], 0).toString();
        String version = view_.getValueAt(rows[0], 1).toString();
        versionF_ = Float.valueOf(version).floatValue();

        try {
            origIP_ = InstrumentPackageDBIO.getIP(instr_, versionF_);
            instrument_.setText(instr_);

            iPVersion_.setText(version);
            date_.setText(getDateAsString(origIP_.getDateAdded()));
            addedBy_.setText(origIP_.getAddedBy());
            userComments_.setText(origIP_.getComment());

            origPeriods_ = InstrumentPackageDBIO.getPeriods(instr_, versionF_);
            setSelectedPeriods(origPeriods_);

        } catch (SQLException e) {
            stdlog_.error("SQL error: " + e.getMessage());
            errorMsg("Database Error", DbWin.DBerror);

        } catch (IOException e) {
            stdlog_.error("IO error: " + e.getMessage());
            errorMsg("Database Error", DbWin.DBerror);
        }
    }


    /**
     * Updates entries in the database if changes were made to
     * added_by, comments or ip_periods
     */
    protected void okButtonClick() {

        String addedBy = addedBy_.getText().trim();
        if (addedBy.length() > 30) {
            errorMsg("Entry Error", "'Added By' must be less than 30 characters long");
            return;
        }

        String comments = userComments_.getText().trim();
        if (comments.length() > 230) {
            errorMsg("Entry Error", "Comments must be less than 230 characters long");
            return;
        }

        // get period selection
        int[] newPeriods = {};
        try {
            newPeriods = getSelectedPeriods();
        } catch (IPacManException e) {
            stdlog_.error("error getting selected periods: " + e.getMessage());
            return;
        }

        // check if period selection has changed
        Vector<Integer> periodChanges = new Vector<Integer>(origPeriods_);
        // stdlog_.debug("orig selection: " + periodChanges.size());
        for (int i=0; i < newPeriods.length; i++) {
            boolean match = false;
            // stdlog_.debug("new " + i + ": " + newPeriods[i]);
            for (int j=0; j < periodChanges.size(); j++) {
                int p = ((Integer)periodChanges.elementAt(j)).intValue();
                // stdlog_.debug("old " + j + ": " + p);
                if (newPeriods[i] == p) {
                    match = true;
                    // stdlog_.debug("lose: " + p);
                    periodChanges.removeElementAt(j);
                    break;
                }
            }
            if (match == false) {
                // stdlog_.debug("gain: " + newPeriods[i]);
                periodChanges.addElement(new Integer(newPeriods[i]));
            }
        }

        // show the period selection
        // stdlog_.debug("periodChanges: " + periodChanges.size());
        for (int i=0; i < periodChanges.size(); i++) {
            // stdlog_.debug(i + ": " + periodChanges.elementAt(i));
        }

        // check for changes
        if (addedBy.equals(origIP_.getAddedBy()) &&
                comments.equals(origIP_.getComment()) &&
                periodChanges.size() == 0 ) {
            JOptionPane.showMessageDialog(dlg_,
                "No changes were made. A database update is unnecessary.",
                "Update Cancelled", JOptionPane.WARNING_MESSAGE);
            return;
        }

        // update DB
        try {
            String mesg = "The database update was successful\n";
            if (periodChanges.size() > 0) {
                mesg += DbWin.click4ObsRuns;
            } else {
                // no need for updateIP() to update periods
                newPeriods = null;
            }

            InstrumentPackageDBIO.updateIP(instr_, versionF_, addedBy,
                comments, newPeriods);
            JOptionPane.showMessageDialog(dlg_, mesg, "Update Successful",
                JOptionPane.INFORMATION_MESSAGE);
            closeInsertEditIPDialog();
            dbWin_.resetDbbView(false);

            if (periodChanges.size() > 0) {
                int[] periodChangesArr = new int[periodChanges.size()];
                for (int i=0; i < periodChanges.size(); i++) {
                    periodChangesArr[i] = ((Integer)periodChanges.elementAt(i)).intValue();
                }
                new CheckObsRuns(instr_, -1, periodChangesArr, view_);
            }
        } catch (SQLException e) {
            errorMsg("Database Error", DbWin.DBerror);
        }
    }


    public static String getDateAsString(long l) {

        Date d = new Date(l * 1000);
        Calendar cal = Calendar.getInstance();
        cal.setTime(d);

        int yr = cal.get(Calendar.YEAR);
        int da = cal.get(Calendar.DAY_OF_MONTH);
        int mo = cal.get(Calendar.MONTH);
        mo++;

        String s1 = (mo < 10) ? "-0" : "-";
        String s2 = (da < 10) ? "-0" : "-";
        return yr + s1 + mo + s2 + da;
    }
}

