package org.eso.ohs.phase2.apps.ipacman.actions;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.border.EtchedBorder;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.utilities.Period;
import org.eso.ohs.phase2.apps.ipacman.DbWin;
import org.eso.ohs.phase2.apps.ipacman.IPACMAN;
import org.eso.ohs.phase2.apps.ipacman.IPacManException;

/**
 * Parent class for Inserting and Editing IPs
 * <p>
 * $Id: InsertEditIP.java,v 1.18 2005/05/06 14:11:37 tcanavan Exp $
 * @version $Revision: 1.18 $
 * @author Ron J Lobo
 */
public abstract class InsertEditIP extends ActionSuperclass {

    protected static Logger stdlog_ = Logger.getLogger(InsertEditIP.class);

    protected DbbView view_;
    protected DbWin dbWin_;

    private String dlgType_;
    protected JDialog dlg_;
    protected String okButtonText_;

    protected int firstPeriod_;
    protected int lastPeriod_;

    // gui components
    protected JTextField instrument_;
    protected JTextField iPVersion_;
    protected JTextField date_;
    protected JTextField addedBy_;
    protected JTextField userComments_;
    protected JList periodWidget_;
    protected JButton okButton_;
    protected JButton cancelButton_;

    private final int numSpecialPeriods = 2;    // "None" and EngObsRunsPeriod


    /**
     * Ctor
     */
    public InsertEditIP(DbWin dbWin, DbbView view, String dlgType) {
        super((JComponent)view, dlgType);

        stdlog_.debug("InsertEditIP ctor: " + dlgType);
        dbWin_ = dbWin;
        view_ = view;
        dlgType_ = dlgType;

        firstPeriod_ = Period.STARTPERIOD < IPACMAN.firstMipsPeriod ?
            IPACMAN.firstMipsPeriod : Period.STARTPERIOD;
        lastPeriod_ = Period.getCurrentPeriod() + Period.ADDITIONAL;
    }


    public void createComponents() {

        // top 3 boxes in panel1
        JPanel panel1 = new JPanel();
        panel1.setLayout(new BoxLayout(panel1, BoxLayout.Y_AXIS));
        JPanel fcPanel = createFileChooserPanel();
        if (fcPanel != null) {
            panel1.add(fcPanel);
        }
        panel1.add(createIPPanel());
        panel1.add(createInfoPanel());

        // everything in panel0
        JPanel panel0 = new JPanel();
        panel0.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        panel0.setLayout(new BorderLayout());
        panel0.add(panel1, BorderLayout.NORTH);
        panel0.add(createPeriodPanel(), BorderLayout.CENTER);
        panel0.add(createButtonPanel(), BorderLayout.SOUTH);

        // create main dialog
        dlg_ = new JDialog(dbWin_, false);
        dlg_.getContentPane().add(panel0);

        panel0.registerKeyboardAction(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                cancelButton_.doClick(200);
            }},
            KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
            JComponent.WHEN_IN_FOCUSED_WINDOW
        );

        Dimension win = dlg_.getPreferredSize();
        Dimension scrn = Toolkit.getDefaultToolkit().getScreenSize();
        dlg_.setSize(win);
        dlg_.setLocation((scrn.width-win.width)/3, (scrn.height-win.height)/3);
        dlg_.setTitle(dlgType_ + " Instrument Package");
        dlg_.getRootPane().setDefaultButton(okButton_);
    }


    /**
     * redefined in child class if necessary
     */
    protected JPanel createFileChooserPanel() {
        return null;
    }


    /**
     * Setup the input panel for the instrument, version and date
     */
    protected JPanel createIPPanel() {

        // Instrument
        JLabel instrLabel = new JLabel(" Instrument ");

        instrument_ = new JTextField();
        instrument_.setEditable(false);
        instrument_.setFocusable(false);
        instrument_.setBackground(Color.lightGray);

        // Version number
        JLabel versionLabel = new JLabel(" IP Version ");

        iPVersion_ = new JTextField();
        iPVersion_.setEditable(false);
        iPVersion_.setFocusable(false);
        iPVersion_.setBackground(Color.lightGray);

        // Date added
        JLabel dateLabel = new JLabel(" Date");

        date_ = new JTextField();
        date_.setEditable(false);
        date_.setFocusable(false);
        date_.setBackground(Color.lightGray);
        date_.setText("                         ");  // hack to widen this dlg

        // create panel
        JPanel ipPanel = new JPanel();
        ipPanel.setLayout(new GridLayout(0, 2, 0, 5));
        ipPanel.setBorder(BorderFactory.createTitledBorder(
            new EtchedBorder(), "IP Details"));
        ipPanel.add(instrLabel);
        ipPanel.add(instrument_);
        ipPanel.add(versionLabel);
        ipPanel.add(iPVersion_);
        ipPanel.add(dateLabel);
        ipPanel.add(date_);
        return ipPanel;
    }


    /**
     * Setup the info panel (Added-By and Comments)
     */
    protected JPanel createInfoPanel() {

        // labels
        JLabel addedLabel = new JLabel(" Added by ");
        JLabel commentsLabel = new JLabel(" Comments ");

        // textfields
        addedBy_ = new JTextField();
        userComments_ = new JTextField();

        JPanel infoPanel = new JPanel();
        infoPanel.setLayout(new GridLayout(0, 2, 0, 5));
        infoPanel.setBorder(BorderFactory.createTitledBorder(
            new EtchedBorder(), "Info"));
        infoPanel.add(addedLabel);
        infoPanel.add(addedBy_);
        infoPanel.add(commentsLabel);
        infoPanel.add(userComments_);

        return infoPanel;
    }


    /**
     * Setup the input panel for the period
     */
    protected JPanel createPeriodPanel() {

        // selection widget
        int numItems = lastPeriod_ - firstPeriod_ + 1 + numSpecialPeriods;
        String[] periodArr = new String[numItems];

        // add the 'special' periods here
        periodArr[0] = "None";
        periodArr[1] = "" + IPACMAN.EngObsRunsPeriod;

        // add all the other periods here
        for (int j=numSpecialPeriods, i=firstPeriod_; i<=lastPeriod_; i++) {
            periodArr[j++] = "" + i;
        }
        periodWidget_ = new JList(periodArr);
        JScrollPane scrollPane = new JScrollPane(periodWidget_);

        // panel
        JPanel periodPanel = new JPanel();
        periodPanel.setLayout(new BorderLayout());
        periodPanel.setBorder(BorderFactory.createTitledBorder(
            new EtchedBorder(), "Associated Period"));
        JLabel addedLabel = new JLabel(" Period   ");
        periodPanel.add(addedLabel, BorderLayout.WEST);
        periodPanel.add(scrollPane, BorderLayout.CENTER);
        return periodPanel;
    }


    /**
     * Setup the button panel
     */
    protected JPanel createButtonPanel() {

        okButton_ = new JButton();
        okButton_.setText(okButtonText_);
        okButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                dlg_.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                okButtonClick();
                dlg_.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            }
        });

        cancelButton_ = new JButton();
        cancelButton_.setText("Cancel");
        cancelButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                closeInsertEditIPDialog();
            }
        });

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(okButton_);
        buttonPanel.add(cancelButton_);
        return buttonPanel;
    }


    /**
     * behaviour implemented in child class
     */
    protected abstract void okButtonClick();


    /**
     * read the user's selection of periods
     */
    protected int[] getSelectedPeriods() throws IPacManException {

        Object[] newPeriodsA = periodWidget_.getSelectedValues();
        stdlog_.debug("#new = " + newPeriodsA.length);
        int[] periods = new int[newPeriodsA.length];

        String err = "Please select either 'None' or\none or more Associated Periods";
        if (newPeriodsA.length == 0) {
            errorMsg("Selection Error", err);
            throw new IPacManException("No periods selected");
        }

        for (int i = 0; i < newPeriodsA.length; i++) {
            String s = (String)newPeriodsA[i];
            if (s.equals("None")) {
                if (newPeriodsA.length == 1) {
                    return new int[0];
                } else {
                    errorMsg("Selection Error", err);
                    throw new IPacManException("Illegal Selection");
                }
            } else {
                stdlog_.debug(" add: " + s);
                periods[i] = Integer.parseInt(s);
            }
        }
        return periods;
    }


    /**
     * set the period selection
     */
    protected void setSelectedPeriods(Vector periodsV) {
        int numPeriods = periodsV.size();
        // stdlog_.debug("selecting " + numPeriods + " period from list");
        if (numPeriods == 0) {
            periodWidget_.setSelectedIndex(0); // 'None'
        } else {
            int[] origPeriodsArr = new int[numPeriods];
            for (int i = 0; i < numPeriods; i++) {
                int period = ((Integer)periodsV.elementAt(i)).intValue();
                // stdlog_.debug("period=" + period);
                if (period == IPACMAN.EngObsRunsPeriod) {
                    origPeriodsArr[i] = 1;
                } else if (firstPeriod_ <= period && period <= lastPeriod_) {
                    origPeriodsArr[i] = period - firstPeriod_ + numSpecialPeriods;
                } else {
                    stdlog_.error("can't select period " + period);
                }
            }
            periodWidget_.setSelectedIndices(origPeriodsArr);
        }
    }


    /**
     * remove the dialog
     */
    protected void closeInsertEditIPDialog() {
        dlg_.setVisible(false);
    }


    /**
     * display an error message for the user
     * @param t     title of message dialog
     * @param msg   message
     */
    protected void errorMsg(String t, String msg) {
        dlg_.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        JOptionPane.showMessageDialog(dlg_, msg, t, JOptionPane.ERROR_MESSAGE);
        stdlog_.error(msg);
    }
}

