package org.eso.ohs.phase2.apps.ipacman.actions;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;

import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.gui.widgets.ExtensionFileFilter;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.core.utilities.Period;
import org.eso.ohs.instruments.ZipUtils;
import org.eso.ohs.instruments.ZipUtilsException;
import org.eso.ohs.persistence.dbase.phase2.InstrumentPackageDBIO;
import org.eso.ohs.phase2.apps.ipacman.CheckObsRuns;
import org.eso.ohs.phase2.apps.ipacman.DbWin;
import org.eso.ohs.phase2.apps.ipacman.IPacManException;

/**
 * Action class to insert an IP into the database
 * <p>
 * @version $Revision: 1.26 $
 * @author Ron J Lobo
 */
public class InsertIPAction extends InsertEditIP {

    public static final String cvsID_ =
        "$Id: InsertIPAction.java,v 1.26 2006/04/28 08:43:04 tcanavan Exp $";

    private File file_;
    private Date dateIP_;

    private JButton loadButton_;
    private JTextField ipFileName_;


    /**
     * constructor
     */
    public InsertIPAction(DbWin dbWin, DbbView view) {
        super(dbWin, view, "Insert IP");
        stdlog_.debug("InsertIPAction ctor");
        okButtonText_ = "Verify and Insert";
        createComponents();
    }


    /**
     * Event handler for the InsertIPAction action
     *
     * @param ev User-initiated ActionEvent to Insert an IP
     */
    public void actionPerformedImpl(ActionEvent ae) {
        stdlog_.debug("InsertIPAction.actionPerformedImpl()");
        setDefaults();
        dlg_.setVisible(true);
        loadButton_.requestFocusInWindow();
    }


    /**
     * Setup the input panel for the instrument package file
     */
    protected JPanel createFileChooserPanel() {

        loadButton_ = new JButton();
        loadButton_.setText("Load IP");
        loadButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                loadIP();
            }
        });

        ipFileName_ = new JTextField();
        ipFileName_.setEditable(false);
        ipFileName_.setFocusable(false);
        ipFileName_.setBackground(Color.lightGray);

        GridBagLayout gbLayout = new GridBagLayout();
        JPanel fcPanel = new JPanel(gbLayout);
        Border b = BorderFactory.createTitledBorder(
            new EtchedBorder(), "Instrument Package File");
        fcPanel.setBorder(b);

        GridBagConstraints gbCons = new GridBagConstraints();
        gbCons.gridwidth = GridBagConstraints.RELATIVE;
        gbCons.weighty = 0;
        gbLayout.setConstraints(loadButton_, gbCons);
        fcPanel.add(loadButton_);

        gbCons.weightx = 1;
        gbCons.gridwidth = GridBagConstraints.REMAINDER;
        gbCons.fill = GridBagConstraints.HORIZONTAL;
        gbLayout.setConstraints(ipFileName_, gbCons);
        fcPanel.add(ipFileName_);

        return fcPanel;
    }


    /**
     * Verifies and Inserts into database
     */
    protected void okButtonClick() {

        // read all user input
        String instr = instrument_.getText();
        float version = Float.parseFloat(iPVersion_.getText());
        int addDate = (int)(dateIP_.getTime() / 1000);

        String addedBy = addedBy_.getText().trim();
        if (addedBy.length() > 30)
            addedBy = addedBy.substring(0, 30);
        stdlog_.debug("version=" + version + ", addDate=" + addDate);

        String comments = userComments_.getText().trim();
        if (comments.length() > 230) {
            comments = comments.substring(0, 230);
        }

        int[] periods;
        try {
            periods = getSelectedPeriods();
        } catch (IPacManException e) {
            stdlog_.error("error verfying: " + e.getMessage());
            return;
        }

        if (!checkIPok(instr, version)) {
            return;
        }

        String mesg = "";
        try {
            // insert IP in database
            InstrumentPackageDBIO.insertIP(
                instr, version, addedBy, addDate, comments, file_, periods);
        } catch (SQLException e) {
            errorMsg("Insert Failed", DbWin.DBerror);
            return;
        } catch (FileNotFoundException e) {
            errorMsg("Insert Failed", "Error accessing Zip file");
            return;
        }

        // success message
        mesg = "Instrument Package inserted into database successfully. \n" + mesg;
        if (periods != null && periods.length > 0) {
            mesg += DbWin.click4ObsRuns;
        }
        JOptionPane.showMessageDialog(dlg_, mesg,
            "Insert Successful", JOptionPane.INFORMATION_MESSAGE);

        closeInsertEditIPDialog();
        dbWin_.resetDbbView(true);

        try {
            if (periods != null && periods.length > 0) {
                new CheckObsRuns(instr, -1, periods, view_);
            }
        } catch (java.sql.SQLException e) {
            errorMsg("Database Error", DbWin.DBerror);
        }
    }


    private boolean checkIPok(String instr, float version) {

        try {
            // check the IP isn't already in the DB
            if (InstrumentPackageDBIO.checkDuplicateIP(instr, version)) {
                errorMsg("Insert Failed", "This Instrument Package version\n" +
                    "already exists in the database");
                return false;
            }

            // check that the zip file is valid
            ZipUtils zu = new ZipUtils(ipFileName_.getText());
            int warnings = zu.verifyIP(false);
            String result = "Click 'OK' to Insert IP into the database";
            if (warnings == 0) {
                result = "No errors or warnings found\n" + result;
                int opt = JOptionPane.showConfirmDialog(dlg_, result,
                    "Zip File Verfied", JOptionPane.OK_CANCEL_OPTION);
                if (opt == JOptionPane.CANCEL_OPTION) {
                    return false;
                }
            } else {
                result = "Warnings found in " + warnings + " TSF files\n" +
                    "Click 'View Report' for more details\n" + result;
                Object[] options = {"OK", "View Report", "Cancel"};
                int opt = JOptionPane.showOptionDialog(dlg_,
                    result, "Zip File Verfied", JOptionPane.YES_NO_CANCEL_OPTION,
                    JOptionPane.WARNING_MESSAGE, null, options, options[0]);
                switch (opt) {
                case JOptionPane.YES_OPTION:
                    break;
                case JOptionPane.NO_OPTION: // view report option
                    VerifyIPAction.showReport(zu.getReport());
                case JOptionPane.CANCEL_OPTION:
                default:
                    return false;
                }
            }

        } catch (SQLException e) {
            stdlog_.error("error checking duplicate IP ", e);
            errorMsg("Insert Failed", DbWin.DBerror);
            return false;

        } catch (ZipUtilsException e) {
            String text = "Errors found in Instrument Package\n" +
                "Click 'View Report' for more details";
            Object[] options = {"View Report", "Cancel"};
            int opt = JOptionPane.showOptionDialog(dlg_, text,
                "Errors in IP", JOptionPane.OK_CANCEL_OPTION,
                JOptionPane.ERROR_MESSAGE, null, options, options[0]);
            if (opt == JOptionPane.OK_OPTION) {
                VerifyIPAction.generateReport(ipFileName_.getText());
            }
            return false;
        }

        return true;
    }


    /**
     * allow user to select an IP file
     */
    private void loadIP() {

        SwingFileChooser fileChooser = new SwingFileChooser();
        fileChooser.setDialogTitle("Select Instrument Package:");
        fileChooser.setFileFilter(new ExtensionFileFilter("zip", "Zip Files"));
        fileChooser.setCurrentDirectory(DbWin.dir_);

        int response = fileChooser.showOpenDialog(dlg_);
        File selectedFile = fileChooser.getSelectedFile();
        DbWin.dir_ = fileChooser.getCurrentDirectory();

        if (response != JFileChooser.APPROVE_OPTION || selectedFile == null) {
            return;
        }

        String pathName = FileIOUtils.getCanonicalPath(selectedFile);
        stdlog_.debug("got file: " + pathName);

        if (checkIPfileName(selectedFile.getName())) {
            ipFileName_.setText(pathName);
            okButton_.setEnabled(true);
            file_ = selectedFile;
        } else {
            errorMsg("Illegal Filename", DbWin.illegalFilename_);
        }
    }


    /**
     * set default values or clear input fields
     * (abstract method in parent class)
     */
    protected void setDefaults() {

        ipFileName_.setText("");
        instrument_.setText("");
        iPVersion_.setText("");
        date_.setText("");
        userComments_.setText("");

        String userName = java.lang.System.getProperty("user.name");
        stdlog_.debug("userName=" + userName);
        addedBy_.setText(userName);

        // default period selection = current_period + 1
        Vector<Integer> periodV = new Vector<Integer>();
        periodV.addElement(new Integer(Period.getCurrentPeriod()+1));
        setSelectedPeriods(periodV);

        okButton_.setEnabled(false);
    }


    /**
     * check the format of the filename is correct
     */
    private boolean checkIPfileName(String filename) {

        if (!DbWin.checkIPfileName(filename)) {
            return false;
        }

        // break up filename into the 3 parts
        filename = filename.substring(0, filename.length()-4); // remove ext
        StringTokenizer st = new StringTokenizer(filename, "-");
        if (st.countTokens() != 3)
            return false;

        String instr = st.nextToken().toUpperCase();
        String version = st.nextToken();
        String date = st.nextToken();
        stdlog_.debug("instr="+instr + ", ver="+version + ", date="+date);

        // check version
        float f = Float.parseFloat(version);
        if (f < 0 || f >= 10000) {
            return false;
        }

        // check date
        int year = Integer.parseInt(date.substring(0,4));
        int month = Integer.parseInt(date.substring(4,6));
        int day = Integer.parseInt(date.substring(6,8));
        stdlog_.debug("year=" + year + ", month=" + month + ", day=" + day);
        Calendar c = Calendar.getInstance();
        c.setLenient(false);
        try {
            c.set(year, month-1, day, 0, 0, 0);
            dateIP_ = c.getTime();
        } catch (IllegalArgumentException e) {
            stdlog_.error("Date error in filename: " + e.getMessage());
            return false;
        }
        stdlog_.debug("date=" + dateIP_.toString());

        // set text fields
        instrument_.setText(instr);
        iPVersion_.setText(version);
        date_.setText(year+"-"+date.substring(4,6)+"-"+date.substring(6,8));

        if (f == 0) {
            setSelectedPeriods(new Vector());
        }

        // filename is acceptable
        return true;
    }
}

