package org.eso.ohs.phase2.apps.ipacman.actions;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ExtensionFileFilter;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.instruments.ZipUtils;
import org.eso.ohs.instruments.ZipUtilsException;
import org.eso.ohs.persistence.dbase.phase2.InstrumentPackageDBIO;
import org.eso.ohs.phase2.apps.ipacman.DbWin;

/**
 * Action class to verify IP package and produce a report
 * <p>
 * $Id: VerifyIPAction.java,v 1.15 2005/03/30 09:56:13 tcanavan Exp $
 * @version $Revision: 1.15 $
 * @author Ron J Lobo
 */
public class VerifyIPAction extends ActionSuperclass {

    private static Logger stdlog_ = Logger.getLogger(VerifyIPAction.class);

    private DbbView view_;


    /**
     * Constructs an ISF Report action
     */
    public VerifyIPAction(DbbView view) {
        super((JComponent)view, "Verify IP");
        view_ = view;
    }


    /**
     * Event handler for this action
     *
     * @param ev User-initiated ActionEvent to add an IP
     */
    public void actionPerformedImpl(ActionEvent ae) {

        boolean IPfromDB = false;
        String pathName;
        view_.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

        int[] rows = view_.getSelectedRows();
        stdlog_.debug("rows selected = " + rows.length);

        switch (rows.length) {
        case 0:
            pathName = getUserSelection();
            break;
        case 1:
            IPfromDB = true;
            pathName = extractDBentry(rows);
            break;
        default:
            errorMsg("Selection Error", "Too many rows selected\n" +
                "Select 1 row to verify an IP in the database\n" +
                "Select 0 rows to verify an IP in the filesystem");
            return;
        }

        if (!pathName.equals("")) {

            generateReport(pathName);

            // delete the extracted tmp file & tmp dir
            if (IPfromDB) {
                stdlog_.debug("deleting: " + pathName);
                (new File(pathName)).delete();

                String dir = pathName.substring(0, pathName.lastIndexOf("/"));
                stdlog_.debug("deleting: " + dir);
                (new File(dir)).delete();
            }
        }

        view_.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }


    /**
     * extracts the selected IP from the database for verification
     */
    private String extractDBentry(int[] rows) {

        String instr   = view_.getValueAt(rows[0], 0).toString();
        String version = view_.getValueAt(rows[0], 1).toString();
        float ver = Float.valueOf(version).floatValue();
        String path = ZipUtils.makeNewTmpDir().toString() + File.separator;

        try {
            return path + InstrumentPackageDBIO.getIP(instr, ver, path, false);
        } catch (java.sql.SQLException e) {
            errorMsg("Database Error",
                "A error occured while reading the IP from the database\n" +
                DbWin.informSA);
        } catch (IOException e) {
            errorMsg("File IO Error",
                "A error occured while writing the IP\n" + DbWin.informSA);
        }

        // error occured - clean up & return
        stdlog_.debug("deleting: " + path);
        (new File(path)).delete();
        return "";
    }


    /**
     * provides a file chooser for the user to select an IP zip file
     */
    private String getUserSelection() {
        JFileChooser fileChooser = new SwingFileChooser();
        fileChooser.setDialogTitle("Select Instrument Package File");
        fileChooser.setCurrentDirectory(DbWin.dir_);
        fileChooser.setFileFilter(new ExtensionFileFilter("zip", "Zip Files"));

        int response = fileChooser.showOpenDialog(view_);
        File zipFile = fileChooser.getSelectedFile();
        DbWin.dir_ = fileChooser.getCurrentDirectory();

        if (response != JFileChooser.APPROVE_OPTION || zipFile == null) {
            return "";
        }

        if (!DbWin.checkIPfileName(zipFile.getName())) {
            errorMsg("Illegal Filename", DbWin.illegalFilename_);
            return "";
        }

        String pathName = FileIOUtils.getCanonicalPath(zipFile);
        stdlog_.debug("got file: " + pathName);
        return pathName;
    }


    /**
     * Make and then display an IP verification report
     */
    public static void generateReport(String pathName) {
        ZipUtils zu = new ZipUtils(pathName);
        String report = "";
        try {
            /* int warnings = */ zu.verifyIP(true);
        } catch (ZipUtilsException e) {
            stdlog_.error("Error generating report!" + e.getMessage());
            report = "\nError during IP verification: " + e.getMessage() +
                "\nReport may be incomplete\n\n";
        }
        report += zu.getReport();
        showReport(report);
    }


    /**
     * Display an IP verification report
     */
    public static void showReport(String report) {
        TextDisplayWidget txtDisplay = new TextDisplayWidget("IP Report");
        txtDisplay.clearTextArea();
        txtDisplay.setTextArea(report);
        txtDisplay.setSize(new Dimension(600, 600));
        txtDisplay.setVisible(true);
    }


    /**
     * display an error message for the user
     *
     * @param title title of message dialog
     * @param mesg  message to display
     */
    private void errorMsg(String title, String mesg) {
        view_.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        JOptionPane.showMessageDialog(view_, mesg, title, JOptionPane.ERROR_MESSAGE);
        stdlog_.error(mesg);
    }
}

