package org.eso.ohs.phase2.apps.ipacman.actions;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ExtensionFileFilter;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.TemplateSignatureMetaData;
import org.eso.ohs.phase2.apps.ipacman.DbWin;

/**
 * Action class to verify a TSF file in the filesystem
 * <p>
 * $Id: VerifyTsfAction.java,v 1.15 2005/02/11 10:31:50 tcanavan Exp $
 * @version $Revision: 1.15 $
 * @author Ron J Lobo
 */
public class VerifyTsfAction extends ActionSuperclass {

    private static Logger stdlog_ = Logger.getLogger(VerifyTsfAction.class);

    private DbbView view_;
    private DbWin dbWin_;

    private JDialog tsfDlg_;
    private JPanel ipPanel_;
    private JButton okButton_;
    private JButton cancelButton_;

    private boolean isfSelected_;
    private boolean tsfSelected_;

    private JTextField isfTF_ = new JTextField();
    private JTextField tsfTF_ = new JTextField();


    /**
     * Constructs a TSF Verify action
     */
    public VerifyTsfAction(DbWin dbWin, DbbView view) {
        super((JComponent)view, "Verify TSF");
        dbWin_ = dbWin;
        view_ = view;
    }


    /**
     * Event handler for this action
     *
     * @param ev User-initiated ActionEvent to add an IP
     */
    public void actionPerformedImpl(ActionEvent ae) {

        tsfDlg_ = new JDialog(dbWin_, "Verify TSF file", true);
        isfSelected_ = tsfSelected_ = false;

        // create the components
        ipPanel_ = new JPanel();
        ipPanel_.setLayout(new BoxLayout(ipPanel_, BoxLayout.Y_AXIS));
        ipPanel_.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

        ipPanel_.add(new JLabel("Please select an Instrument file and a Template Signature File"));
        ipPanel_.add(createSelectIsfPanel());
        ipPanel_.add(createSelectTsfPanel());

        tsfDlg_.getContentPane().add(ipPanel_, BorderLayout.CENTER);
        tsfDlg_.getContentPane().add(createButtonPanel(), BorderLayout.SOUTH);

        ipPanel_.registerKeyboardAction(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                cancelButton_.doClick(200);
            }},
            KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
            JComponent.WHEN_IN_FOCUSED_WINDOW
        );

        Dimension win = tsfDlg_.getPreferredSize();
        Dimension scrn = Toolkit.getDefaultToolkit().getScreenSize();
        tsfDlg_.setLocation((scrn.width-win.width)/3, (scrn.height-win.height)/3);
        tsfDlg_.getRootPane().setDefaultButton(okButton_);
        tsfDlg_.pack();
        tsfDlg_.setVisible(true);
    }


    private JPanel createSelectIsfPanel() {

        JButton selectBtn = new JButton();
        selectBtn.setText(" Select .isf file ");
        selectBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                selectFile("isf");
            }
        });

        isfTF_ = new JTextField();
        isfTF_.setColumns(60);
        isfTF_.setEditable(false);
        isfTF_.setFocusable(false);
        isfTF_.setBackground(Color.lightGray);

        JPanel p = new JPanel();
        p.add(selectBtn);
        p.add(isfTF_);
        return p;
    }


    private JPanel createSelectTsfPanel() {

        JButton selectBtn = new JButton();
        selectBtn.setText(" Select .tsf file ");
        selectBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                selectFile("tsf");
            }
        });

        tsfTF_ = new JTextField();
        tsfTF_.setColumns(60);
        tsfTF_.setEditable(false);
        tsfTF_.setFocusable(false);
        tsfTF_.setBackground(Color.lightGray);

        JPanel p = new JPanel();
        p.add(selectBtn);
        p.add(tsfTF_);
        return p;
    }


    private JPanel createButtonPanel() {

        okButton_ = new JButton();
        okButton_.setText("Ok");
        okButton_.setEnabled(false);
        okButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                okBtnClick();
            }
        });

        cancelButton_ = new JButton();
        cancelButton_.setText("Cancel");
        cancelButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                stdlog_.debug("cancelBtnClick");
                // setVisible(false);
                tsfDlg_.dispose();
            }
        });

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(okButton_);
        buttonPanel.add(cancelButton_);
        return buttonPanel;
    }


    public void selectFile(String ft) {

        stdlog_.debug("selectFile ft=" + ft);
        SwingFileChooser fileChooser = new SwingFileChooser();
        fileChooser.setDialogTitle("Select " + ft + " File");
        fileChooser.setCurrentDirectory(DbWin.dir_);
        fileChooser.setFileFilter(new ExtensionFileFilter(ft, ft + " Files"));

        int response = fileChooser.showOpenDialog(view_);
        File aFile = fileChooser.getSelectedFile();
        DbWin.dir_ = fileChooser.getCurrentDirectory();

        if (response != SwingFileChooser.APPROVE_OPTION || aFile == null) {
            return;
        } else {
            String pathName = FileIOUtils.getCanonicalPath(aFile);
            stdlog_.debug("got file: " + pathName);

            if (ft.equals("isf")) {
                isfSelected_ = true;
                isfTF_.setText(pathName);
            } else {
                tsfSelected_ = true;
                tsfTF_.setText(pathName);
            }
        }

        okButton_.setEnabled(isfSelected_ && tsfSelected_);
    }


    public void okBtnClick() {

        stdlog_.debug("okBtnClick");
        tsfDlg_.dispose();
        view_.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

        String tsfFile = tsfTF_.getText();
        String isfFile = isfTF_.getText();
        String isfDir = isfFile.substring(0, isfFile.lastIndexOf(File.separator));
        stdlog_.debug("verify tsf: " + tsfFile);

        try {
            Instrument instr = new Instrument(isfDir, isfFile);
            stdlog_.debug("instrument=" + instr.getInstrumentName());

            TemplateSignatureMetaData tsmd =
                new TemplateSignatureMetaData(tsfFile, instr);

            String report = "\nTSF Report\n\n" +
                "ISF file: " + isfFile + "\n" +
                "TSF file: " + tsfFile + "\n\n" +
                instr.toEngString() + "\n\n" +
                tsmd.toEngString();

            TextDisplayWidget displayResults_ = new TextDisplayWidget("TSF Report");
            displayResults_.clearTextArea();
            displayResults_.setTextArea(report);
            displayResults_.setSize(new Dimension(600, 600));
            displayResults_.setVisible(true);

        } catch (IOException e) {
            stdlog_.error("File IO error: " + e.getMessage());
            String msg = "Error occured while verfying tsf file\n" + e.getMessage();
            errorMsg("error", msg);
        }

        view_.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }


    /**
     * display an error message for the user
     *
     * @param title title of message dialog
     * @param mesg  message to display
     */
    private void errorMsg(String title, String mesg) {
        stdlog_.error(mesg);
        JOptionPane.showMessageDialog(view_, mesg, title, JOptionPane.ERROR_MESSAGE);
    }
}

