package org.eso.ohs.phase2.apps.p2ppAppServer;

import java.sql.SQLException;

import org.eso.ohs.core.dbb.server.DbbSession;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.server.DbbSqlEngineImpl;
import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.persistence.Config;

/**
 * This class provides only static methods to build up a P2PP database
 * browser.
 * 
 */
public class BrowserConfig {

  // Phase I Tables
  public static DbbSqlTable op;
  public static DbbSqlTable or;
  public static DbbSqlTable accounts;
  public static DbbSqlTable profiles;
  
  public static DbbSqlTable pr;
  public static DbbSqlTable inst;

  // Phase II Tables
  public static DbbSqlTable ob;
  public static DbbSqlTable od;
  public static DbbSqlTable ts;
  public static DbbSqlTable tp;
  public static DbbSqlTable pa;
  public static DbbSqlTable ti;
  public static DbbSqlTable cs;
  public static DbbSqlTable sr;
  public static DbbSqlTable pm;
  public static DbbSqlTable finds_view;
  public static DbbSqlTable fcs;
  public static DbbSqlTable standard_time_intervals_view;
  public static DbbSqlTable sidereal_time_intervals_view;
  public static DbbSqlTable baselines;
  public static DbbSqlTable ephemeris_file;




  private static String obrepDbName_ = Config.getCfg().getDbName(
                                     ObservationBlock.getSuffix());


 private static  String opcDbName_ = Config.getCfg().getDbName(
                                     ObservingRun.getSuffix());

  //----------------------------------------
  // Public Methods
  //----------------------------------------

  /**
   * Build the sql engine and return its reference to main
   * application.
   * @return refence to the SqlEngine to build query
   * @exception SQLException JDBC exception on opening a session and creating the engine 
   */
  public static DbbSqlEngine getEngine()
    throws SQLException {
     

      DbbSession session = new DbbSession(Config.getCfg().getUrl(ObservationBlock.getSuffix()),
					  obrepDbName_,
					  Config.getCfg().getDbUserName(),
					  Config.getCfg().getDbPassword());


      /* Create the server engine without building the key hash table
       * from the database metadata. We assume that the information
       * provided by this end application is sufficient to build the sql
       * join clauses.  
       * C.Boarotto 16-Jan-2000
       */
      DbbSqlEngineImpl engine = new DbbSqlEngineImpl(session,
						     false);

//       System.out.println("After construction of Engine");
      generateTables();     
      buildDefaultJoins(engine);

      return engine;
  }


  /**
   * Build the end application table objects.
   */
  public static void generateTables() {
    op = new DbbSqlTable(opcDbName_, "dbo", "obs_programmes");
    or = new DbbSqlTable(opcDbName_, "dbo", "obs_runs");
    accounts = new DbbSqlTable(opcDbName_, "dbo", "accounts");
    profiles = new DbbSqlTable(opcDbName_, "dbo", "profiles");
    pr = new DbbSqlTable(opcDbName_, "dbo", "proposed");
    sr = new DbbSqlTable(opcDbName_,"dbo","sched_rep");
    inst = new DbbSqlTable(opcDbName_,"dbo","opc_instruments");


    ob = new DbbSqlTable(obrepDbName_, "dbo", "obs_blocks");
    od = new DbbSqlTable(obrepDbName_, "dbo", "obs_descriptions");
    ts = new DbbSqlTable(obrepDbName_, "dbo", "template_signatures");
    tp = new DbbSqlTable(obrepDbName_, "dbo", "target_packages");
    pa = new DbbSqlTable(obrepDbName_, "dbo", "parameters");
    ti = new DbbSqlTable(obrepDbName_, "dbo", "time_intervals");
    cs = new DbbSqlTable(obrepDbName_, "dbo", "constraint_sets");
    pm = new DbbSqlTable(obrepDbName_, "dbo", "parameters");
    finds_view = new DbbSqlTable(obrepDbName_, "dbo", "finds_view");
    ephemeris_file = new DbbSqlTable(obrepDbName_, "dbo", "ephemeris_file");
    fcs = new DbbSqlTable(obrepDbName_, "dbo", "finding_charts");
    standard_time_intervals_view = new DbbSqlTable(obrepDbName_, "dbo", "standard_time_intervals");
    sidereal_time_intervals_view = new DbbSqlTable(obrepDbName_, "dbo", "sidereal_time_intervals");
    baselines = new DbbSqlTable(obrepDbName_, "dbo", "baselines");
  }


  //----------------------------------------
  // Private Methods
  //----------------------------------------

  /**
   * Set up the end application information necessary to build the sql
   * join clauses within the sql engine.
   */
  private static void buildDefaultJoins(DbbSqlEngineImpl engine) {

    // eso_users - obs_programmes (through proposed)   
    //engine.addJoinCondition(op, "id", pr, "programme_id", op, eu);
    //engine.addJoinCondition(eu, "id", pr, "user_id", op, eu);

    engine.addJoinCondition(ob,"run_id",sr,"run_id",ob,sr);

    // obs_blocks - phase I tables
    //     engine.addJoinCondition(ob, "run_id", or, "id", ob, or);
    engine.addJoinCondition(ob, "user_id", accounts, "account_id", ob, accounts);    
    engine.addJoinCondition(accounts, "account_id", profiles, "account_id", ob, profiles);

    // obs_blocks - obs_descriptions
    engine.addJoinCondition(ob, "ob_id", od, "od_id", ob, od);
    
    engine.addJoinCondition(ob, "ob_id",standard_time_intervals_view, "ti_id", ob, ti);
    
    //engine.addJoinCondition(ob, "ob_id",sidereal_time_intervals_view, "ti_id", ob,sidereal_time_intervals_view);


    String [] fromCols = {"ob_id"};
    String [] toCols = {"ti_id"};
    engine.addCompositeJoinCondition(ob,fromCols,sidereal_time_intervals_view,toCols,true);


    String [] ephCols = {"ob_id"};
    
    engine.addCompositeJoinCondition(ob,ephCols,ephemeris_file,ephCols,true);

    
    
    // obs_blocks - target_packages
    engine.addJoinCondition(ob, "tp_id", tp, "tp_id", ob, tp);

    // obs_blocks - constraint_sets
    engine.addJoinCondition(ob, "cs_id", cs, "cs_id", ob, cs);
    
    engine.addJoinCondition(cs, "baseline_id",baselines, "baseline_id",cs,baselines);


    // obs_blocks - time_intervals
    // XXX ??? engine.addJoinCondition(ob, "id", ti, "ob_id", ob, ti);

    // obs_descriptions - template_signatures
    engine.addJoinCondition(ts, "od_id", od, "od_id", ts, od);

    // obs_descriptions - parameters
    engine.addJoinCondition(pa, "od_id", od, "od_id", pa, od);
    
    //engine.addJoinCondition(finds, "fc_id",fcs,"fc_id",finds,fcs);

    //engine.addJoinCondition(finds, "ob_id",ob,"ob_id",finds,ob);
    String [] cols = {"ob_id"};
    engine.addCompositeJoinCondition(ob,cols,finds_view,cols,true);

  }
}


