package org.eso.ohs.scripting;

/**
 * This interface defines the Constraint Set for an ObservationBlock.
 *
 * IMPORTANT: Changes to this interface must result in a new version string 
 * in the ScriptingBox class.
 *
 * @author Nick Kornweibel
 */

public interface ConstraintSetAPI {

    /**
     * Get the value of the Airmass as a string
     * @return the air mass
     */

    public String getAirmass();

    /**
     * Get value for Fractional Lunar Illumination as string
     * @return the Fractional Lunar Illumination
     */

    public String getFractionalLunarIllumination();

    /**
     * Get value for Sky Transparency
     * @return the Sky Transparency
     */

    public String getSkyTransparency();

    /**
     * Get value for Moon Angular Distance
     * @return the Moon Angular Distance
     */

    public int getMoonAngularDistance();

    /**
     * Get value for Strehl Ratio
     * @return the strehl ratio
     */

    public float getStrehlRatio();


    /**
     * Get value for Seeing as a string
     * @return the seeing
     */

    public String getSeeing();

    /**
     * Get value for Tel Temp
     * @return the Tel Temp
     */

    public int getTelTemp();

    /**
     * Get value for Water Vapour
     * @return the water vapou
     */

    public String getWaterVapour();

    /**
     * Get value for Description property
     * @return the description
     */

    public String getDescription();

    /**
     * Get the name of the Constraint set
     * @return the name of the constraint set
     */

    public String getName();
    
    /**
     * Get the baseline of the Constraint set
     * @return the name of the constraint set
     */

    public String getBaseline();


    /**
     * Get the Id of the Constraint set
     * @return the id of the Constraint set
     */

    public String getId();
}
