package org.eso.ohs.scripting;

/**
 * Defines the Target data interface within an OB. 
 * There is no means to set data within the Target object.
 *
 * IMPORTANT: Changes to this interface must result in a new version string 
 * in the ScriptingBox class.
 *
 * @author Nick Kornweibel
 */

public interface TargetAPI {

    /**
     * Get the name of the target for this OB
     * @return string containing the name of the target
     */

    public String getName();

    /**
     * Get the Id of this target
     * @return string id for this target
     */

    public String getId();

    /**
     * Get the string form of the RA value for this OB
     * @return string containing the RA e.g. "05:45:36.5"
     */

    public String getRightAscension();

    /**
     * Get the string form of the DEC value for this OB
     * @return string containing the DEC e.g. "-78:11:09.0"
     */

    public String getDeclination();

    /**
     * Get the Equinox value for this target
     * @return String containing the equinox for this target e.g. "J2000"
     */

    public String getEquinox();

    /**
     * Get the Epoch value for this target
     * @return string value for the epoch for this target e.g. "2000.0"
     */

    public String getEpoch();
    
    /**
     * Get the Proper RA value for this target
     * @return string value for the proper RA for this target
     */
    public String getPropRA();
    
    /**
     * Get the Proper Declination value for this target
     * @return string value for Proper Declination  for this target 
     */
    public String getPropDec();
    
    /**
     * Get the DiffRA value for this target
     * @return string value for the diff RA for this target
     */
    public String getDiffRA();
    
    /**
     * Get the DiffDec value for this target
     * @return string value for DiffDec  for this target 
     */
    public String getDiffDec();
}
